#ifndef random_systems_nonic_symmetric_system_hpp
#define random_systems_nonic_symmetric_system_hpp

#include <utility>
#include <iterator>
#include <math.h>

namespace flopoco
{
namespace random
{
namespace systems
 {

template<class T,class TACC=T>
class SymmetricNonicSystem{
private:
  T ss[91];
  T tmom[11];
  template<class TT> TT square(const TT &x) const { return x*x; }
public:
  typedef T real_t;

  int arity() const { return 5; };

  SymmetricNonicSystem(  const T *_ss ,   const T *_tmom )
  {
    std::copy(_ss, _ss+91, ss);
    std::copy(_tmom, _tmom+11, tmom);
  }

  void DefaultGuess(T *params) const{
    params[0]=1; // c1
    params[1]=0; // c3
    params[2]=0; // c5
    params[3]=0; // c7
    params[4]=0; // c9
  }

T metric(const T *parameters) const {
  T c1p1 = parameters[0];
  T c3p1 = parameters[1];
  T c5p1 = parameters[2];
  T c7p1 = parameters[3];
  T c9p1 = parameters[4];
  T c1p2 = c1p1*c1p1;
  T c3p2 = c3p1*c3p1;
  T c5p2 = c5p1*c5p1;
  T c7p2 = c7p1*c7p1;
  T c9p2 = c9p1*c9p1;
  TACC root_0 = convert<TACC>(2*c1p1*c3p1*ss[4]);
  root_0 += 2*c1p1*c5p1*ss[6];
  root_0 += 2*c1p1*c7p1*ss[8];
  root_0 += 2*c1p1*c9p1*ss[10];
  root_0 += 2*c3p1*c5p1*ss[8];
  root_0 += 2*c3p1*c7p1*ss[10];
  root_0 += 2*c3p1*c9p1*ss[12];
  root_0 += 2*c5p1*c7p1*ss[12];
  root_0 += 2*c5p1*c9p1*ss[14];
  root_0 += 2*c7p1*c9p1*ss[16];
  root_0 += c1p2*ss[2];
  root_0 += c3p2*ss[6];
  root_0 += c5p2*ss[10];
  root_0 += c7p2*ss[14];
  root_0 += c9p2*ss[18];
  root_0 += -tmom[2];
  T c1p3 = c1p2*c1p1;
  T c1p4 = c1p3*c1p1;
  T c3p3 = c3p2*c3p1;
  T c3p4 = c3p3*c3p1;
  T c5p3 = c5p2*c5p1;
  T c5p4 = c5p3*c5p1;
  T c7p3 = c7p2*c7p1;
  T c7p4 = c7p3*c7p1;
  T c9p3 = c9p2*c9p1;
  T c9p4 = c9p3*c9p1;
  TACC root_1 = convert<TACC>(24*c1p1*c3p1*c5p1*c7p1*ss[16]);
  root_1 += 24*c1p1*c3p1*c5p1*c9p1*ss[18];
  root_1 += 24*c1p1*c3p1*c7p1*c9p1*ss[20];
  root_1 += 24*c1p1*c5p1*c7p1*c9p1*ss[22];
  root_1 += 24*c3p1*c5p1*c7p1*c9p1*ss[24];
  root_1 += 12*c1p1*c3p1*c5p2*ss[14];
  root_1 += 12*c1p1*c3p1*c7p2*ss[18];
  root_1 += 12*c1p1*c3p1*c9p2*ss[22];
  root_1 += 12*c1p1*c3p2*c5p1*ss[12];
  root_1 += 12*c1p1*c3p2*c7p1*ss[14];
  root_1 += 12*c1p1*c3p2*c9p1*ss[16];
  root_1 += 12*c1p1*c5p1*c7p2*ss[20];
  root_1 += 12*c1p1*c5p1*c9p2*ss[24];
  root_1 += 12*c1p1*c5p2*c7p1*ss[18];
  root_1 += 12*c1p1*c5p2*c9p1*ss[20];
  root_1 += 12*c1p1*c7p1*c9p2*ss[26];
  root_1 += 12*c1p1*c7p2*c9p1*ss[24];
  root_1 += 12*c1p2*c3p1*c5p1*ss[10];
  root_1 += 12*c1p2*c3p1*c7p1*ss[12];
  root_1 += 12*c1p2*c3p1*c9p1*ss[14];
  root_1 += 12*c1p2*c5p1*c7p1*ss[14];
  root_1 += 12*c1p2*c5p1*c9p1*ss[16];
  root_1 += 12*c1p2*c7p1*c9p1*ss[18];
  root_1 += 12*c3p1*c5p1*c7p2*ss[22];
  root_1 += 12*c3p1*c5p1*c9p2*ss[26];
  root_1 += 12*c3p1*c5p2*c7p1*ss[20];
  root_1 += 12*c3p1*c5p2*c9p1*ss[22];
  root_1 += 12*c3p1*c7p1*c9p2*ss[28];
  root_1 += 12*c3p1*c7p2*c9p1*ss[26];
  root_1 += 12*c3p2*c5p1*c7p1*ss[18];
  root_1 += 12*c3p2*c5p1*c9p1*ss[20];
  root_1 += 12*c3p2*c7p1*c9p1*ss[22];
  root_1 += 12*c5p1*c7p1*c9p2*ss[30];
  root_1 += 12*c5p1*c7p2*c9p1*ss[28];
  root_1 += 12*c5p2*c7p1*c9p1*ss[26];
  root_1 += 4*c1p1*c3p3*ss[10];
  root_1 += 4*c1p1*c5p3*ss[16];
  root_1 += 4*c1p1*c7p3*ss[22];
  root_1 += 4*c1p1*c9p3*ss[28];
  root_1 += 6*c1p2*c3p2*ss[8];
  root_1 += 6*c1p2*c5p2*ss[12];
  root_1 += 6*c1p2*c7p2*ss[16];
  root_1 += 6*c1p2*c9p2*ss[20];
  root_1 += 4*c1p3*c3p1*ss[6];
  root_1 += 4*c1p3*c5p1*ss[8];
  root_1 += 4*c1p3*c7p1*ss[10];
  root_1 += 4*c1p3*c9p1*ss[12];
  root_1 += 4*c3p1*c5p3*ss[18];
  root_1 += 4*c3p1*c7p3*ss[24];
  root_1 += 4*c3p1*c9p3*ss[30];
  root_1 += 6*c3p2*c5p2*ss[16];
  root_1 += 6*c3p2*c7p2*ss[20];
  root_1 += 6*c3p2*c9p2*ss[24];
  root_1 += 4*c3p3*c5p1*ss[14];
  root_1 += 4*c3p3*c7p1*ss[16];
  root_1 += 4*c3p3*c9p1*ss[18];
  root_1 += 4*c5p1*c7p3*ss[26];
  root_1 += 4*c5p1*c9p3*ss[32];
  root_1 += 6*c5p2*c7p2*ss[24];
  root_1 += 6*c5p2*c9p2*ss[28];
  root_1 += 4*c5p3*c7p1*ss[22];
  root_1 += 4*c5p3*c9p1*ss[24];
  root_1 += 4*c7p1*c9p3*ss[34];
  root_1 += 6*c7p2*c9p2*ss[32];
  root_1 += 4*c7p3*c9p1*ss[30];
  root_1 += c1p4*ss[4];
  root_1 += c3p4*ss[12];
  root_1 += c5p4*ss[20];
  root_1 += c7p4*ss[28];
  root_1 += c9p4*ss[36];
  root_1 += -tmom[4];
  T c1p5 = c1p4*c1p1;
  T c1p6 = c1p5*c1p1;
  T c3p5 = c3p4*c3p1;
  T c3p6 = c3p5*c3p1;
  T c5p5 = c5p4*c5p1;
  T c5p6 = c5p5*c5p1;
  T c7p5 = c7p4*c7p1;
  T c7p6 = c7p5*c7p1;
  T c9p5 = c9p4*c9p1;
  T c9p6 = c9p5*c9p1;
  TACC root_2 = convert<TACC>(360*c1p1*c3p1*c5p1*c7p1*c9p2*ss[34]);
  root_2 += 360*c1p1*c3p1*c5p1*c7p2*c9p1*ss[32];
  root_2 += 360*c1p1*c3p1*c5p2*c7p1*c9p1*ss[30];
  root_2 += 360*c1p1*c3p2*c5p1*c7p1*c9p1*ss[28];
  root_2 += 360*c1p2*c3p1*c5p1*c7p1*c9p1*ss[26];
  root_2 += 120*c1p1*c3p1*c5p1*c7p3*ss[30];
  root_2 += 120*c1p1*c3p1*c5p1*c9p3*ss[36];
  root_2 += 180*c1p1*c3p1*c5p2*c7p2*ss[28];
  root_2 += 180*c1p1*c3p1*c5p2*c9p2*ss[32];
  root_2 += 120*c1p1*c3p1*c5p3*c7p1*ss[26];
  root_2 += 120*c1p1*c3p1*c5p3*c9p1*ss[28];
  root_2 += 120*c1p1*c3p1*c7p1*c9p3*ss[38];
  root_2 += 180*c1p1*c3p1*c7p2*c9p2*ss[36];
  root_2 += 120*c1p1*c3p1*c7p3*c9p1*ss[34];
  root_2 += 180*c1p1*c3p2*c5p1*c7p2*ss[26];
  root_2 += 180*c1p1*c3p2*c5p1*c9p2*ss[30];
  root_2 += 180*c1p1*c3p2*c5p2*c7p1*ss[24];
  root_2 += 180*c1p1*c3p2*c5p2*c9p1*ss[26];
  root_2 += 180*c1p1*c3p2*c7p1*c9p2*ss[32];
  root_2 += 180*c1p1*c3p2*c7p2*c9p1*ss[30];
  root_2 += 120*c1p1*c3p3*c5p1*c7p1*ss[22];
  root_2 += 120*c1p1*c3p3*c5p1*c9p1*ss[24];
  root_2 += 120*c1p1*c3p3*c7p1*c9p1*ss[26];
  root_2 += 120*c1p1*c5p1*c7p1*c9p3*ss[40];
  root_2 += 180*c1p1*c5p1*c7p2*c9p2*ss[38];
  root_2 += 120*c1p1*c5p1*c7p3*c9p1*ss[36];
  root_2 += 180*c1p1*c5p2*c7p1*c9p2*ss[36];
  root_2 += 180*c1p1*c5p2*c7p2*c9p1*ss[34];
  root_2 += 120*c1p1*c5p3*c7p1*c9p1*ss[32];
  root_2 += 180*c1p2*c3p1*c5p1*c7p2*ss[24];
  root_2 += 180*c1p2*c3p1*c5p1*c9p2*ss[28];
  root_2 += 180*c1p2*c3p1*c5p2*c7p1*ss[22];
  root_2 += 180*c1p2*c3p1*c5p2*c9p1*ss[24];
  root_2 += 180*c1p2*c3p1*c7p1*c9p2*ss[30];
  root_2 += 180*c1p2*c3p1*c7p2*c9p1*ss[28];
  root_2 += 180*c1p2*c3p2*c5p1*c7p1*ss[20];
  root_2 += 180*c1p2*c3p2*c5p1*c9p1*ss[22];
  root_2 += 180*c1p2*c3p2*c7p1*c9p1*ss[24];
  root_2 += 180*c1p2*c5p1*c7p1*c9p2*ss[32];
  root_2 += 180*c1p2*c5p1*c7p2*c9p1*ss[30];
  root_2 += 180*c1p2*c5p2*c7p1*c9p1*ss[28];
  root_2 += 120*c1p3*c3p1*c5p1*c7p1*ss[18];
  root_2 += 120*c1p3*c3p1*c5p1*c9p1*ss[20];
  root_2 += 120*c1p3*c3p1*c7p1*c9p1*ss[22];
  root_2 += 120*c1p3*c5p1*c7p1*c9p1*ss[24];
  root_2 += 120*c3p1*c5p1*c7p1*c9p3*ss[42];
  root_2 += 180*c3p1*c5p1*c7p2*c9p2*ss[40];
  root_2 += 120*c3p1*c5p1*c7p3*c9p1*ss[38];
  root_2 += 180*c3p1*c5p2*c7p1*c9p2*ss[38];
  root_2 += 180*c3p1*c5p2*c7p2*c9p1*ss[36];
  root_2 += 120*c3p1*c5p3*c7p1*c9p1*ss[34];
  root_2 += 180*c3p2*c5p1*c7p1*c9p2*ss[36];
  root_2 += 180*c3p2*c5p1*c7p2*c9p1*ss[34];
  root_2 += 180*c3p2*c5p2*c7p1*c9p1*ss[32];
  root_2 += 120*c3p3*c5p1*c7p1*c9p1*ss[30];
  root_2 += 30*c1p1*c3p1*c5p4*ss[24];
  root_2 += 30*c1p1*c3p1*c7p4*ss[32];
  root_2 += 30*c1p1*c3p1*c9p4*ss[40];
  root_2 += 60*c1p1*c3p2*c5p3*ss[22];
  root_2 += 60*c1p1*c3p2*c7p3*ss[28];
  root_2 += 60*c1p1*c3p2*c9p3*ss[34];
  root_2 += 60*c1p1*c3p3*c5p2*ss[20];
  root_2 += 60*c1p1*c3p3*c7p2*ss[24];
  root_2 += 60*c1p1*c3p3*c9p2*ss[28];
  root_2 += 30*c1p1*c3p4*c5p1*ss[18];
  root_2 += 30*c1p1*c3p4*c7p1*ss[20];
  root_2 += 30*c1p1*c3p4*c9p1*ss[22];
  root_2 += 30*c1p1*c5p1*c7p4*ss[34];
  root_2 += 30*c1p1*c5p1*c9p4*ss[42];
  root_2 += 60*c1p1*c5p2*c7p3*ss[32];
  root_2 += 60*c1p1*c5p2*c9p3*ss[38];
  root_2 += 60*c1p1*c5p3*c7p2*ss[30];
  root_2 += 60*c1p1*c5p3*c9p2*ss[34];
  root_2 += 30*c1p1*c5p4*c7p1*ss[28];
  root_2 += 30*c1p1*c5p4*c9p1*ss[30];
  root_2 += 30*c1p1*c7p1*c9p4*ss[44];
  root_2 += 60*c1p1*c7p2*c9p3*ss[42];
  root_2 += 60*c1p1*c7p3*c9p2*ss[40];
  root_2 += 30*c1p1*c7p4*c9p1*ss[38];
  root_2 += 60*c1p2*c3p1*c5p3*ss[20];
  root_2 += 60*c1p2*c3p1*c7p3*ss[26];
  root_2 += 60*c1p2*c3p1*c9p3*ss[32];
  root_2 += 90*c1p2*c3p2*c5p2*ss[18];
  root_2 += 90*c1p2*c3p2*c7p2*ss[22];
  root_2 += 90*c1p2*c3p2*c9p2*ss[26];
  root_2 += 60*c1p2*c3p3*c5p1*ss[16];
  root_2 += 60*c1p2*c3p3*c7p1*ss[18];
  root_2 += 60*c1p2*c3p3*c9p1*ss[20];
  root_2 += 60*c1p2*c5p1*c7p3*ss[28];
  root_2 += 60*c1p2*c5p1*c9p3*ss[34];
  root_2 += 90*c1p2*c5p2*c7p2*ss[26];
  root_2 += 90*c1p2*c5p2*c9p2*ss[30];
  root_2 += 60*c1p2*c5p3*c7p1*ss[24];
  root_2 += 60*c1p2*c5p3*c9p1*ss[26];
  root_2 += 60*c1p2*c7p1*c9p3*ss[36];
  root_2 += 90*c1p2*c7p2*c9p2*ss[34];
  root_2 += 60*c1p2*c7p3*c9p1*ss[32];
  root_2 += 60*c1p3*c3p1*c5p2*ss[16];
  root_2 += 60*c1p3*c3p1*c7p2*ss[20];
  root_2 += 60*c1p3*c3p1*c9p2*ss[24];
  root_2 += 60*c1p3*c3p2*c5p1*ss[14];
  root_2 += 60*c1p3*c3p2*c7p1*ss[16];
  root_2 += 60*c1p3*c3p2*c9p1*ss[18];
  root_2 += 60*c1p3*c5p1*c7p2*ss[22];
  root_2 += 60*c1p3*c5p1*c9p2*ss[26];
  root_2 += 60*c1p3*c5p2*c7p1*ss[20];
  root_2 += 60*c1p3*c5p2*c9p1*ss[22];
  root_2 += 60*c1p3*c7p1*c9p2*ss[28];
  root_2 += 60*c1p3*c7p2*c9p1*ss[26];
  root_2 += 30*c1p4*c3p1*c5p1*ss[12];
  root_2 += 30*c1p4*c3p1*c7p1*ss[14];
  root_2 += 30*c1p4*c3p1*c9p1*ss[16];
  root_2 += 30*c1p4*c5p1*c7p1*ss[16];
  root_2 += 30*c1p4*c5p1*c9p1*ss[18];
  root_2 += 30*c1p4*c7p1*c9p1*ss[20];
  root_2 += 30*c3p1*c5p1*c7p4*ss[36];
  root_2 += 30*c3p1*c5p1*c9p4*ss[44];
  root_2 += 60*c3p1*c5p2*c7p3*ss[34];
  root_2 += 60*c3p1*c5p2*c9p3*ss[40];
  root_2 += 60*c3p1*c5p3*c7p2*ss[32];
  root_2 += 60*c3p1*c5p3*c9p2*ss[36];
  root_2 += 30*c3p1*c5p4*c7p1*ss[30];
  root_2 += 30*c3p1*c5p4*c9p1*ss[32];
  root_2 += 30*c3p1*c7p1*c9p4*ss[46];
  root_2 += 60*c3p1*c7p2*c9p3*ss[44];
  root_2 += 60*c3p1*c7p3*c9p2*ss[42];
  root_2 += 30*c3p1*c7p4*c9p1*ss[40];
  root_2 += 60*c3p2*c5p1*c7p3*ss[32];
  root_2 += 60*c3p2*c5p1*c9p3*ss[38];
  root_2 += 90*c3p2*c5p2*c7p2*ss[30];
  root_2 += 90*c3p2*c5p2*c9p2*ss[34];
  root_2 += 60*c3p2*c5p3*c7p1*ss[28];
  root_2 += 60*c3p2*c5p3*c9p1*ss[30];
  root_2 += 60*c3p2*c7p1*c9p3*ss[40];
  root_2 += 90*c3p2*c7p2*c9p2*ss[38];
  root_2 += 60*c3p2*c7p3*c9p1*ss[36];
  root_2 += 60*c3p3*c5p1*c7p2*ss[28];
  root_2 += 60*c3p3*c5p1*c9p2*ss[32];
  root_2 += 60*c3p3*c5p2*c7p1*ss[26];
  root_2 += 60*c3p3*c5p2*c9p1*ss[28];
  root_2 += 60*c3p3*c7p1*c9p2*ss[34];
  root_2 += 60*c3p3*c7p2*c9p1*ss[32];
  root_2 += 30*c3p4*c5p1*c7p1*ss[24];
  root_2 += 30*c3p4*c5p1*c9p1*ss[26];
  root_2 += 30*c3p4*c7p1*c9p1*ss[28];
  root_2 += 30*c5p1*c7p1*c9p4*ss[48];
  root_2 += 60*c5p1*c7p2*c9p3*ss[46];
  root_2 += 60*c5p1*c7p3*c9p2*ss[44];
  root_2 += 30*c5p1*c7p4*c9p1*ss[42];
  root_2 += 60*c5p2*c7p1*c9p3*ss[44];
  root_2 += 90*c5p2*c7p2*c9p2*ss[42];
  root_2 += 60*c5p2*c7p3*c9p1*ss[40];
  root_2 += 60*c5p3*c7p1*c9p2*ss[40];
  root_2 += 60*c5p3*c7p2*c9p1*ss[38];
  root_2 += 30*c5p4*c7p1*c9p1*ss[36];
  root_2 += 6*c1p1*c3p5*ss[16];
  root_2 += 6*c1p1*c5p5*ss[26];
  root_2 += 6*c1p1*c7p5*ss[36];
  root_2 += 6*c1p1*c9p5*ss[46];
  root_2 += 15*c1p2*c3p4*ss[14];
  root_2 += 15*c1p2*c5p4*ss[22];
  root_2 += 15*c1p2*c7p4*ss[30];
  root_2 += 15*c1p2*c9p4*ss[38];
  root_2 += 20*c1p3*c3p3*ss[12];
  root_2 += 20*c1p3*c5p3*ss[18];
  root_2 += 20*c1p3*c7p3*ss[24];
  root_2 += 20*c1p3*c9p3*ss[30];
  root_2 += 15*c1p4*c3p2*ss[10];
  root_2 += 15*c1p4*c5p2*ss[14];
  root_2 += 15*c1p4*c7p2*ss[18];
  root_2 += 15*c1p4*c9p2*ss[22];
  root_2 += 6*c1p5*c3p1*ss[8];
  root_2 += 6*c1p5*c5p1*ss[10];
  root_2 += 6*c1p5*c7p1*ss[12];
  root_2 += 6*c1p5*c9p1*ss[14];
  root_2 += 6*c3p1*c5p5*ss[28];
  root_2 += 6*c3p1*c7p5*ss[38];
  root_2 += 6*c3p1*c9p5*ss[48];
  root_2 += 15*c3p2*c5p4*ss[26];
  root_2 += 15*c3p2*c7p4*ss[34];
  root_2 += 15*c3p2*c9p4*ss[42];
  root_2 += 20*c3p3*c5p3*ss[24];
  root_2 += 20*c3p3*c7p3*ss[30];
  root_2 += 20*c3p3*c9p3*ss[36];
  root_2 += 15*c3p4*c5p2*ss[22];
  root_2 += 15*c3p4*c7p2*ss[26];
  root_2 += 15*c3p4*c9p2*ss[30];
  root_2 += 6*c3p5*c5p1*ss[20];
  root_2 += 6*c3p5*c7p1*ss[22];
  root_2 += 6*c3p5*c9p1*ss[24];
  root_2 += 6*c5p1*c7p5*ss[40];
  root_2 += 6*c5p1*c9p5*ss[50];
  root_2 += 15*c5p2*c7p4*ss[38];
  root_2 += 15*c5p2*c9p4*ss[46];
  root_2 += 20*c5p3*c7p3*ss[36];
  root_2 += 20*c5p3*c9p3*ss[42];
  root_2 += 15*c5p4*c7p2*ss[34];
  root_2 += 15*c5p4*c9p2*ss[38];
  root_2 += 6*c5p5*c7p1*ss[32];
  root_2 += 6*c5p5*c9p1*ss[34];
  root_2 += 6*c7p1*c9p5*ss[52];
  root_2 += 15*c7p2*c9p4*ss[50];
  root_2 += 20*c7p3*c9p3*ss[48];
  root_2 += 15*c7p4*c9p2*ss[46];
  root_2 += 6*c7p5*c9p1*ss[44];
  root_2 += c1p6*ss[6];
  root_2 += c3p6*ss[18];
  root_2 += c5p6*ss[30];
  root_2 += c7p6*ss[42];
  root_2 += c9p6*ss[54];
  root_2 += -tmom[6];
  T c1p7 = c1p6*c1p1;
  T c1p8 = c1p7*c1p1;
  T c3p7 = c3p6*c3p1;
  T c3p8 = c3p7*c3p1;
  T c5p7 = c5p6*c5p1;
  T c5p8 = c5p7*c5p1;
  T c7p7 = c7p6*c7p1;
  T c7p8 = c7p7*c7p1;
  T c9p7 = c9p6*c9p1;
  T c9p8 = c9p7*c9p1;
  TACC root_3 = convert<TACC>(1680*c1p1*c3p1*c5p1*c7p1*c9p4*ss[52]);
  root_3 += 3360*c1p1*c3p1*c5p1*c7p2*c9p3*ss[50];
  root_3 += 3360*c1p1*c3p1*c5p1*c7p3*c9p2*ss[48];
  root_3 += 1680*c1p1*c3p1*c5p1*c7p4*c9p1*ss[46];
  root_3 += 3360*c1p1*c3p1*c5p2*c7p1*c9p3*ss[48];
  root_3 += 5040*c1p1*c3p1*c5p2*c7p2*c9p2*ss[46];
  root_3 += 3360*c1p1*c3p1*c5p2*c7p3*c9p1*ss[44];
  root_3 += 3360*c1p1*c3p1*c5p3*c7p1*c9p2*ss[44];
  root_3 += 3360*c1p1*c3p1*c5p3*c7p2*c9p1*ss[42];
  root_3 += 1680*c1p1*c3p1*c5p4*c7p1*c9p1*ss[40];
  root_3 += 3360*c1p1*c3p2*c5p1*c7p1*c9p3*ss[46];
  root_3 += 5040*c1p1*c3p2*c5p1*c7p2*c9p2*ss[44];
  root_3 += 3360*c1p1*c3p2*c5p1*c7p3*c9p1*ss[42];
  root_3 += 5040*c1p1*c3p2*c5p2*c7p1*c9p2*ss[42];
  root_3 += 5040*c1p1*c3p2*c5p2*c7p2*c9p1*ss[40];
  root_3 += 3360*c1p1*c3p2*c5p3*c7p1*c9p1*ss[38];
  root_3 += 3360*c1p1*c3p3*c5p1*c7p1*c9p2*ss[40];
  root_3 += 3360*c1p1*c3p3*c5p1*c7p2*c9p1*ss[38];
  root_3 += 3360*c1p1*c3p3*c5p2*c7p1*c9p1*ss[36];
  root_3 += 1680*c1p1*c3p4*c5p1*c7p1*c9p1*ss[34];
  root_3 += 3360*c1p2*c3p1*c5p1*c7p1*c9p3*ss[44];
  root_3 += 5040*c1p2*c3p1*c5p1*c7p2*c9p2*ss[42];
  root_3 += 3360*c1p2*c3p1*c5p1*c7p3*c9p1*ss[40];
  root_3 += 5040*c1p2*c3p1*c5p2*c7p1*c9p2*ss[40];
  root_3 += 5040*c1p2*c3p1*c5p2*c7p2*c9p1*ss[38];
  root_3 += 3360*c1p2*c3p1*c5p3*c7p1*c9p1*ss[36];
  root_3 += 5040*c1p2*c3p2*c5p1*c7p1*c9p2*ss[38];
  root_3 += 5040*c1p2*c3p2*c5p1*c7p2*c9p1*ss[36];
  root_3 += 5040*c1p2*c3p2*c5p2*c7p1*c9p1*ss[34];
  root_3 += 3360*c1p2*c3p3*c5p1*c7p1*c9p1*ss[32];
  root_3 += 3360*c1p3*c3p1*c5p1*c7p1*c9p2*ss[36];
  root_3 += 3360*c1p3*c3p1*c5p1*c7p2*c9p1*ss[34];
  root_3 += 3360*c1p3*c3p1*c5p2*c7p1*c9p1*ss[32];
  root_3 += 3360*c1p3*c3p2*c5p1*c7p1*c9p1*ss[30];
  root_3 += 1680*c1p4*c3p1*c5p1*c7p1*c9p1*ss[28];
  root_3 += 336*c1p1*c3p1*c5p1*c7p5*ss[44];
  root_3 += 336*c1p1*c3p1*c5p1*c9p5*ss[54];
  root_3 += 840*c1p1*c3p1*c5p2*c7p4*ss[42];
  root_3 += 840*c1p1*c3p1*c5p2*c9p4*ss[50];
  root_3 += 1120*c1p1*c3p1*c5p3*c7p3*ss[40];
  root_3 += 1120*c1p1*c3p1*c5p3*c9p3*ss[46];
  root_3 += 840*c1p1*c3p1*c5p4*c7p2*ss[38];
  root_3 += 840*c1p1*c3p1*c5p4*c9p2*ss[42];
  root_3 += 336*c1p1*c3p1*c5p5*c7p1*ss[36];
  root_3 += 336*c1p1*c3p1*c5p5*c9p1*ss[38];
  root_3 += 336*c1p1*c3p1*c7p1*c9p5*ss[56];
  root_3 += 840*c1p1*c3p1*c7p2*c9p4*ss[54];
  root_3 += 1120*c1p1*c3p1*c7p3*c9p3*ss[52];
  root_3 += 840*c1p1*c3p1*c7p4*c9p2*ss[50];
  root_3 += 336*c1p1*c3p1*c7p5*c9p1*ss[48];
  root_3 += 840*c1p1*c3p2*c5p1*c7p4*ss[40];
  root_3 += 840*c1p1*c3p2*c5p1*c9p4*ss[48];
  root_3 += 1680*c1p1*c3p2*c5p2*c7p3*ss[38];
  root_3 += 1680*c1p1*c3p2*c5p2*c9p3*ss[44];
  root_3 += 1680*c1p1*c3p2*c5p3*c7p2*ss[36];
  root_3 += 1680*c1p1*c3p2*c5p3*c9p2*ss[40];
  root_3 += 840*c1p1*c3p2*c5p4*c7p1*ss[34];
  root_3 += 840*c1p1*c3p2*c5p4*c9p1*ss[36];
  root_3 += 840*c1p1*c3p2*c7p1*c9p4*ss[50];
  root_3 += 1680*c1p1*c3p2*c7p2*c9p3*ss[48];
  root_3 += 1680*c1p1*c3p2*c7p3*c9p2*ss[46];
  root_3 += 840*c1p1*c3p2*c7p4*c9p1*ss[44];
  root_3 += 1120*c1p1*c3p3*c5p1*c7p3*ss[36];
  root_3 += 1120*c1p1*c3p3*c5p1*c9p3*ss[42];
  root_3 += 1680*c1p1*c3p3*c5p2*c7p2*ss[34];
  root_3 += 1680*c1p1*c3p3*c5p2*c9p2*ss[38];
  root_3 += 1120*c1p1*c3p3*c5p3*c7p1*ss[32];
  root_3 += 1120*c1p1*c3p3*c5p3*c9p1*ss[34];
  root_3 += 1120*c1p1*c3p3*c7p1*c9p3*ss[44];
  root_3 += 1680*c1p1*c3p3*c7p2*c9p2*ss[42];
  root_3 += 1120*c1p1*c3p3*c7p3*c9p1*ss[40];
  root_3 += 840*c1p1*c3p4*c5p1*c7p2*ss[32];
  root_3 += 840*c1p1*c3p4*c5p1*c9p2*ss[36];
  root_3 += 840*c1p1*c3p4*c5p2*c7p1*ss[30];
  root_3 += 840*c1p1*c3p4*c5p2*c9p1*ss[32];
  root_3 += 840*c1p1*c3p4*c7p1*c9p2*ss[38];
  root_3 += 840*c1p1*c3p4*c7p2*c9p1*ss[36];
  root_3 += 336*c1p1*c3p5*c5p1*c7p1*ss[28];
  root_3 += 336*c1p1*c3p5*c5p1*c9p1*ss[30];
  root_3 += 336*c1p1*c3p5*c7p1*c9p1*ss[32];
  root_3 += 336*c1p1*c5p1*c7p1*c9p5*ss[58];
  root_3 += 840*c1p1*c5p1*c7p2*c9p4*ss[56];
  root_3 += 1120*c1p1*c5p1*c7p3*c9p3*ss[54];
  root_3 += 840*c1p1*c5p1*c7p4*c9p2*ss[52];
  root_3 += 336*c1p1*c5p1*c7p5*c9p1*ss[50];
  root_3 += 840*c1p1*c5p2*c7p1*c9p4*ss[54];
  root_3 += 1680*c1p1*c5p2*c7p2*c9p3*ss[52];
  root_3 += 1680*c1p1*c5p2*c7p3*c9p2*ss[50];
  root_3 += 840*c1p1*c5p2*c7p4*c9p1*ss[48];
  root_3 += 1120*c1p1*c5p3*c7p1*c9p3*ss[50];
  root_3 += 1680*c1p1*c5p3*c7p2*c9p2*ss[48];
  root_3 += 1120*c1p1*c5p3*c7p3*c9p1*ss[46];
  root_3 += 840*c1p1*c5p4*c7p1*c9p2*ss[46];
  root_3 += 840*c1p1*c5p4*c7p2*c9p1*ss[44];
  root_3 += 336*c1p1*c5p5*c7p1*c9p1*ss[42];
  root_3 += 840*c1p2*c3p1*c5p1*c7p4*ss[38];
  root_3 += 840*c1p2*c3p1*c5p1*c9p4*ss[46];
  root_3 += 1680*c1p2*c3p1*c5p2*c7p3*ss[36];
  root_3 += 1680*c1p2*c3p1*c5p2*c9p3*ss[42];
  root_3 += 1680*c1p2*c3p1*c5p3*c7p2*ss[34];
  root_3 += 1680*c1p2*c3p1*c5p3*c9p2*ss[38];
  root_3 += 840*c1p2*c3p1*c5p4*c7p1*ss[32];
  root_3 += 840*c1p2*c3p1*c5p4*c9p1*ss[34];
  root_3 += 840*c1p2*c3p1*c7p1*c9p4*ss[48];
  root_3 += 1680*c1p2*c3p1*c7p2*c9p3*ss[46];
  root_3 += 1680*c1p2*c3p1*c7p3*c9p2*ss[44];
  root_3 += 840*c1p2*c3p1*c7p4*c9p1*ss[42];
  root_3 += 1680*c1p2*c3p2*c5p1*c7p3*ss[34];
  root_3 += 1680*c1p2*c3p2*c5p1*c9p3*ss[40];
  root_3 += 2520*c1p2*c3p2*c5p2*c7p2*ss[32];
  root_3 += 2520*c1p2*c3p2*c5p2*c9p2*ss[36];
  root_3 += 1680*c1p2*c3p2*c5p3*c7p1*ss[30];
  root_3 += 1680*c1p2*c3p2*c5p3*c9p1*ss[32];
  root_3 += 1680*c1p2*c3p2*c7p1*c9p3*ss[42];
  root_3 += 2520*c1p2*c3p2*c7p2*c9p2*ss[40];
  root_3 += 1680*c1p2*c3p2*c7p3*c9p1*ss[38];
  root_3 += 1680*c1p2*c3p3*c5p1*c7p2*ss[30];
  root_3 += 1680*c1p2*c3p3*c5p1*c9p2*ss[34];
  root_3 += 1680*c1p2*c3p3*c5p2*c7p1*ss[28];
  root_3 += 1680*c1p2*c3p3*c5p2*c9p1*ss[30];
  root_3 += 1680*c1p2*c3p3*c7p1*c9p2*ss[36];
  root_3 += 1680*c1p2*c3p3*c7p2*c9p1*ss[34];
  root_3 += 840*c1p2*c3p4*c5p1*c7p1*ss[26];
  root_3 += 840*c1p2*c3p4*c5p1*c9p1*ss[28];
  root_3 += 840*c1p2*c3p4*c7p1*c9p1*ss[30];
  root_3 += 840*c1p2*c5p1*c7p1*c9p4*ss[50];
  root_3 += 1680*c1p2*c5p1*c7p2*c9p3*ss[48];
  root_3 += 1680*c1p2*c5p1*c7p3*c9p2*ss[46];
  root_3 += 840*c1p2*c5p1*c7p4*c9p1*ss[44];
  root_3 += 1680*c1p2*c5p2*c7p1*c9p3*ss[46];
  root_3 += 2520*c1p2*c5p2*c7p2*c9p2*ss[44];
  root_3 += 1680*c1p2*c5p2*c7p3*c9p1*ss[42];
  root_3 += 1680*c1p2*c5p3*c7p1*c9p2*ss[42];
  root_3 += 1680*c1p2*c5p3*c7p2*c9p1*ss[40];
  root_3 += 840*c1p2*c5p4*c7p1*c9p1*ss[38];
  root_3 += 1120*c1p3*c3p1*c5p1*c7p3*ss[32];
  root_3 += 1120*c1p3*c3p1*c5p1*c9p3*ss[38];
  root_3 += 1680*c1p3*c3p1*c5p2*c7p2*ss[30];
  root_3 += 1680*c1p3*c3p1*c5p2*c9p2*ss[34];
  root_3 += 1120*c1p3*c3p1*c5p3*c7p1*ss[28];
  root_3 += 1120*c1p3*c3p1*c5p3*c9p1*ss[30];
  root_3 += 1120*c1p3*c3p1*c7p1*c9p3*ss[40];
  root_3 += 1680*c1p3*c3p1*c7p2*c9p2*ss[38];
  root_3 += 1120*c1p3*c3p1*c7p3*c9p1*ss[36];
  root_3 += 1680*c1p3*c3p2*c5p1*c7p2*ss[28];
  root_3 += 1680*c1p3*c3p2*c5p1*c9p2*ss[32];
  root_3 += 1680*c1p3*c3p2*c5p2*c7p1*ss[26];
  root_3 += 1680*c1p3*c3p2*c5p2*c9p1*ss[28];
  root_3 += 1680*c1p3*c3p2*c7p1*c9p2*ss[34];
  root_3 += 1680*c1p3*c3p2*c7p2*c9p1*ss[32];
  root_3 += 1120*c1p3*c3p3*c5p1*c7p1*ss[24];
  root_3 += 1120*c1p3*c3p3*c5p1*c9p1*ss[26];
  root_3 += 1120*c1p3*c3p3*c7p1*c9p1*ss[28];
  root_3 += 1120*c1p3*c5p1*c7p1*c9p3*ss[42];
  root_3 += 1680*c1p3*c5p1*c7p2*c9p2*ss[40];
  root_3 += 1120*c1p3*c5p1*c7p3*c9p1*ss[38];
  root_3 += 1680*c1p3*c5p2*c7p1*c9p2*ss[38];
  root_3 += 1680*c1p3*c5p2*c7p2*c9p1*ss[36];
  root_3 += 1120*c1p3*c5p3*c7p1*c9p1*ss[34];
  root_3 += 840*c1p4*c3p1*c5p1*c7p2*ss[26];
  root_3 += 840*c1p4*c3p1*c5p1*c9p2*ss[30];
  root_3 += 840*c1p4*c3p1*c5p2*c7p1*ss[24];
  root_3 += 840*c1p4*c3p1*c5p2*c9p1*ss[26];
  root_3 += 840*c1p4*c3p1*c7p1*c9p2*ss[32];
  root_3 += 840*c1p4*c3p1*c7p2*c9p1*ss[30];
  root_3 += 840*c1p4*c3p2*c5p1*c7p1*ss[22];
  root_3 += 840*c1p4*c3p2*c5p1*c9p1*ss[24];
  root_3 += 840*c1p4*c3p2*c7p1*c9p1*ss[26];
  root_3 += 840*c1p4*c5p1*c7p1*c9p2*ss[34];
  root_3 += 840*c1p4*c5p1*c7p2*c9p1*ss[32];
  root_3 += 840*c1p4*c5p2*c7p1*c9p1*ss[30];
  root_3 += 336*c1p5*c3p1*c5p1*c7p1*ss[20];
  root_3 += 336*c1p5*c3p1*c5p1*c9p1*ss[22];
  root_3 += 336*c1p5*c3p1*c7p1*c9p1*ss[24];
  root_3 += 336*c1p5*c5p1*c7p1*c9p1*ss[26];
  root_3 += 336*c3p1*c5p1*c7p1*c9p5*ss[60];
  root_3 += 840*c3p1*c5p1*c7p2*c9p4*ss[58];
  root_3 += 1120*c3p1*c5p1*c7p3*c9p3*ss[56];
  root_3 += 840*c3p1*c5p1*c7p4*c9p2*ss[54];
  root_3 += 336*c3p1*c5p1*c7p5*c9p1*ss[52];
  root_3 += 840*c3p1*c5p2*c7p1*c9p4*ss[56];
  root_3 += 1680*c3p1*c5p2*c7p2*c9p3*ss[54];
  root_3 += 1680*c3p1*c5p2*c7p3*c9p2*ss[52];
  root_3 += 840*c3p1*c5p2*c7p4*c9p1*ss[50];
  root_3 += 1120*c3p1*c5p3*c7p1*c9p3*ss[52];
  root_3 += 1680*c3p1*c5p3*c7p2*c9p2*ss[50];
  root_3 += 1120*c3p1*c5p3*c7p3*c9p1*ss[48];
  root_3 += 840*c3p1*c5p4*c7p1*c9p2*ss[48];
  root_3 += 840*c3p1*c5p4*c7p2*c9p1*ss[46];
  root_3 += 336*c3p1*c5p5*c7p1*c9p1*ss[44];
  root_3 += 840*c3p2*c5p1*c7p1*c9p4*ss[54];
  root_3 += 1680*c3p2*c5p1*c7p2*c9p3*ss[52];
  root_3 += 1680*c3p2*c5p1*c7p3*c9p2*ss[50];
  root_3 += 840*c3p2*c5p1*c7p4*c9p1*ss[48];
  root_3 += 1680*c3p2*c5p2*c7p1*c9p3*ss[50];
  root_3 += 2520*c3p2*c5p2*c7p2*c9p2*ss[48];
  root_3 += 1680*c3p2*c5p2*c7p3*c9p1*ss[46];
  root_3 += 1680*c3p2*c5p3*c7p1*c9p2*ss[46];
  root_3 += 1680*c3p2*c5p3*c7p2*c9p1*ss[44];
  root_3 += 840*c3p2*c5p4*c7p1*c9p1*ss[42];
  root_3 += 1120*c3p3*c5p1*c7p1*c9p3*ss[48];
  root_3 += 1680*c3p3*c5p1*c7p2*c9p2*ss[46];
  root_3 += 1120*c3p3*c5p1*c7p3*c9p1*ss[44];
  root_3 += 1680*c3p3*c5p2*c7p1*c9p2*ss[44];
  root_3 += 1680*c3p3*c5p2*c7p2*c9p1*ss[42];
  root_3 += 1120*c3p3*c5p3*c7p1*c9p1*ss[40];
  root_3 += 840*c3p4*c5p1*c7p1*c9p2*ss[42];
  root_3 += 840*c3p4*c5p1*c7p2*c9p1*ss[40];
  root_3 += 840*c3p4*c5p2*c7p1*c9p1*ss[38];
  root_3 += 336*c3p5*c5p1*c7p1*c9p1*ss[36];
  root_3 += 56*c1p1*c3p1*c5p6*ss[34];
  root_3 += 56*c1p1*c3p1*c7p6*ss[46];
  root_3 += 56*c1p1*c3p1*c9p6*ss[58];
  root_3 += 168*c1p1*c3p2*c5p5*ss[32];
  root_3 += 168*c1p1*c3p2*c7p5*ss[42];
  root_3 += 168*c1p1*c3p2*c9p5*ss[52];
  root_3 += 280*c1p1*c3p3*c5p4*ss[30];
  root_3 += 280*c1p1*c3p3*c7p4*ss[38];
  root_3 += 280*c1p1*c3p3*c9p4*ss[46];
  root_3 += 280*c1p1*c3p4*c5p3*ss[28];
  root_3 += 280*c1p1*c3p4*c7p3*ss[34];
  root_3 += 280*c1p1*c3p4*c9p3*ss[40];
  root_3 += 168*c1p1*c3p5*c5p2*ss[26];
  root_3 += 168*c1p1*c3p5*c7p2*ss[30];
  root_3 += 168*c1p1*c3p5*c9p2*ss[34];
  root_3 += 56*c1p1*c3p6*c5p1*ss[24];
  root_3 += 56*c1p1*c3p6*c7p1*ss[26];
  root_3 += 56*c1p1*c3p6*c9p1*ss[28];
  root_3 += 56*c1p1*c5p1*c7p6*ss[48];
  root_3 += 56*c1p1*c5p1*c9p6*ss[60];
  root_3 += 168*c1p1*c5p2*c7p5*ss[46];
  root_3 += 168*c1p1*c5p2*c9p5*ss[56];
  root_3 += 280*c1p1*c5p3*c7p4*ss[44];
  root_3 += 280*c1p1*c5p3*c9p4*ss[52];
  root_3 += 280*c1p1*c5p4*c7p3*ss[42];
  root_3 += 280*c1p1*c5p4*c9p3*ss[48];
  root_3 += 168*c1p1*c5p5*c7p2*ss[40];
  root_3 += 168*c1p1*c5p5*c9p2*ss[44];
  root_3 += 56*c1p1*c5p6*c7p1*ss[38];
  root_3 += 56*c1p1*c5p6*c9p1*ss[40];
  root_3 += 56*c1p1*c7p1*c9p6*ss[62];
  root_3 += 168*c1p1*c7p2*c9p5*ss[60];
  root_3 += 280*c1p1*c7p3*c9p4*ss[58];
  root_3 += 280*c1p1*c7p4*c9p3*ss[56];
  root_3 += 168*c1p1*c7p5*c9p2*ss[54];
  root_3 += 56*c1p1*c7p6*c9p1*ss[52];
  root_3 += 168*c1p2*c3p1*c5p5*ss[30];
  root_3 += 168*c1p2*c3p1*c7p5*ss[40];
  root_3 += 168*c1p2*c3p1*c9p5*ss[50];
  root_3 += 420*c1p2*c3p2*c5p4*ss[28];
  root_3 += 420*c1p2*c3p2*c7p4*ss[36];
  root_3 += 420*c1p2*c3p2*c9p4*ss[44];
  root_3 += 560*c1p2*c3p3*c5p3*ss[26];
  root_3 += 560*c1p2*c3p3*c7p3*ss[32];
  root_3 += 560*c1p2*c3p3*c9p3*ss[38];
  root_3 += 420*c1p2*c3p4*c5p2*ss[24];
  root_3 += 420*c1p2*c3p4*c7p2*ss[28];
  root_3 += 420*c1p2*c3p4*c9p2*ss[32];
  root_3 += 168*c1p2*c3p5*c5p1*ss[22];
  root_3 += 168*c1p2*c3p5*c7p1*ss[24];
  root_3 += 168*c1p2*c3p5*c9p1*ss[26];
  root_3 += 168*c1p2*c5p1*c7p5*ss[42];
  root_3 += 168*c1p2*c5p1*c9p5*ss[52];
  root_3 += 420*c1p2*c5p2*c7p4*ss[40];
  root_3 += 420*c1p2*c5p2*c9p4*ss[48];
  root_3 += 560*c1p2*c5p3*c7p3*ss[38];
  root_3 += 560*c1p2*c5p3*c9p3*ss[44];
  root_3 += 420*c1p2*c5p4*c7p2*ss[36];
  root_3 += 420*c1p2*c5p4*c9p2*ss[40];
  root_3 += 168*c1p2*c5p5*c7p1*ss[34];
  root_3 += 168*c1p2*c5p5*c9p1*ss[36];
  root_3 += 168*c1p2*c7p1*c9p5*ss[54];
  root_3 += 420*c1p2*c7p2*c9p4*ss[52];
  root_3 += 560*c1p2*c7p3*c9p3*ss[50];
  root_3 += 420*c1p2*c7p4*c9p2*ss[48];
  root_3 += 168*c1p2*c7p5*c9p1*ss[46];
  root_3 += 280*c1p3*c3p1*c5p4*ss[26];
  root_3 += 280*c1p3*c3p1*c7p4*ss[34];
  root_3 += 280*c1p3*c3p1*c9p4*ss[42];
  root_3 += 560*c1p3*c3p2*c5p3*ss[24];
  root_3 += 560*c1p3*c3p2*c7p3*ss[30];
  root_3 += 560*c1p3*c3p2*c9p3*ss[36];
  root_3 += 560*c1p3*c3p3*c5p2*ss[22];
  root_3 += 560*c1p3*c3p3*c7p2*ss[26];
  root_3 += 560*c1p3*c3p3*c9p2*ss[30];
  root_3 += 280*c1p3*c3p4*c5p1*ss[20];
  root_3 += 280*c1p3*c3p4*c7p1*ss[22];
  root_3 += 280*c1p3*c3p4*c9p1*ss[24];
  root_3 += 280*c1p3*c5p1*c7p4*ss[36];
  root_3 += 280*c1p3*c5p1*c9p4*ss[44];
  root_3 += 560*c1p3*c5p2*c7p3*ss[34];
  root_3 += 560*c1p3*c5p2*c9p3*ss[40];
  root_3 += 560*c1p3*c5p3*c7p2*ss[32];
  root_3 += 560*c1p3*c5p3*c9p2*ss[36];
  root_3 += 280*c1p3*c5p4*c7p1*ss[30];
  root_3 += 280*c1p3*c5p4*c9p1*ss[32];
  root_3 += 280*c1p3*c7p1*c9p4*ss[46];
  root_3 += 560*c1p3*c7p2*c9p3*ss[44];
  root_3 += 560*c1p3*c7p3*c9p2*ss[42];
  root_3 += 280*c1p3*c7p4*c9p1*ss[40];
  root_3 += 280*c1p4*c3p1*c5p3*ss[22];
  root_3 += 280*c1p4*c3p1*c7p3*ss[28];
  root_3 += 280*c1p4*c3p1*c9p3*ss[34];
  root_3 += 420*c1p4*c3p2*c5p2*ss[20];
  root_3 += 420*c1p4*c3p2*c7p2*ss[24];
  root_3 += 420*c1p4*c3p2*c9p2*ss[28];
  root_3 += 280*c1p4*c3p3*c5p1*ss[18];
  root_3 += 280*c1p4*c3p3*c7p1*ss[20];
  root_3 += 280*c1p4*c3p3*c9p1*ss[22];
  root_3 += 280*c1p4*c5p1*c7p3*ss[30];
  root_3 += 280*c1p4*c5p1*c9p3*ss[36];
  root_3 += 420*c1p4*c5p2*c7p2*ss[28];
  root_3 += 420*c1p4*c5p2*c9p2*ss[32];
  root_3 += 280*c1p4*c5p3*c7p1*ss[26];
  root_3 += 280*c1p4*c5p3*c9p1*ss[28];
  root_3 += 280*c1p4*c7p1*c9p3*ss[38];
  root_3 += 420*c1p4*c7p2*c9p2*ss[36];
  root_3 += 280*c1p4*c7p3*c9p1*ss[34];
  root_3 += 168*c1p5*c3p1*c5p2*ss[18];
  root_3 += 168*c1p5*c3p1*c7p2*ss[22];
  root_3 += 168*c1p5*c3p1*c9p2*ss[26];
  root_3 += 168*c1p5*c3p2*c5p1*ss[16];
  root_3 += 168*c1p5*c3p2*c7p1*ss[18];
  root_3 += 168*c1p5*c3p2*c9p1*ss[20];
  root_3 += 168*c1p5*c5p1*c7p2*ss[24];
  root_3 += 168*c1p5*c5p1*c9p2*ss[28];
  root_3 += 168*c1p5*c5p2*c7p1*ss[22];
  root_3 += 168*c1p5*c5p2*c9p1*ss[24];
  root_3 += 168*c1p5*c7p1*c9p2*ss[30];
  root_3 += 168*c1p5*c7p2*c9p1*ss[28];
  root_3 += 56*c1p6*c3p1*c5p1*ss[14];
  root_3 += 56*c1p6*c3p1*c7p1*ss[16];
  root_3 += 56*c1p6*c3p1*c9p1*ss[18];
  root_3 += 56*c1p6*c5p1*c7p1*ss[18];
  root_3 += 56*c1p6*c5p1*c9p1*ss[20];
  root_3 += 56*c1p6*c7p1*c9p1*ss[22];
  root_3 += 56*c3p1*c5p1*c7p6*ss[50];
  root_3 += 56*c3p1*c5p1*c9p6*ss[62];
  root_3 += 168*c3p1*c5p2*c7p5*ss[48];
  root_3 += 168*c3p1*c5p2*c9p5*ss[58];
  root_3 += 280*c3p1*c5p3*c7p4*ss[46];
  root_3 += 280*c3p1*c5p3*c9p4*ss[54];
  root_3 += 280*c3p1*c5p4*c7p3*ss[44];
  root_3 += 280*c3p1*c5p4*c9p3*ss[50];
  root_3 += 168*c3p1*c5p5*c7p2*ss[42];
  root_3 += 168*c3p1*c5p5*c9p2*ss[46];
  root_3 += 56*c3p1*c5p6*c7p1*ss[40];
  root_3 += 56*c3p1*c5p6*c9p1*ss[42];
  root_3 += 56*c3p1*c7p1*c9p6*ss[64];
  root_3 += 168*c3p1*c7p2*c9p5*ss[62];
  root_3 += 280*c3p1*c7p3*c9p4*ss[60];
  root_3 += 280*c3p1*c7p4*c9p3*ss[58];
  root_3 += 168*c3p1*c7p5*c9p2*ss[56];
  root_3 += 56*c3p1*c7p6*c9p1*ss[54];
  root_3 += 168*c3p2*c5p1*c7p5*ss[46];
  root_3 += 168*c3p2*c5p1*c9p5*ss[56];
  root_3 += 420*c3p2*c5p2*c7p4*ss[44];
  root_3 += 420*c3p2*c5p2*c9p4*ss[52];
  root_3 += 560*c3p2*c5p3*c7p3*ss[42];
  root_3 += 560*c3p2*c5p3*c9p3*ss[48];
  root_3 += 420*c3p2*c5p4*c7p2*ss[40];
  root_3 += 420*c3p2*c5p4*c9p2*ss[44];
  root_3 += 168*c3p2*c5p5*c7p1*ss[38];
  root_3 += 168*c3p2*c5p5*c9p1*ss[40];
  root_3 += 168*c3p2*c7p1*c9p5*ss[58];
  root_3 += 420*c3p2*c7p2*c9p4*ss[56];
  root_3 += 560*c3p2*c7p3*c9p3*ss[54];
  root_3 += 420*c3p2*c7p4*c9p2*ss[52];
  root_3 += 168*c3p2*c7p5*c9p1*ss[50];
  root_3 += 280*c3p3*c5p1*c7p4*ss[42];
  root_3 += 280*c3p3*c5p1*c9p4*ss[50];
  root_3 += 560*c3p3*c5p2*c7p3*ss[40];
  root_3 += 560*c3p3*c5p2*c9p3*ss[46];
  root_3 += 560*c3p3*c5p3*c7p2*ss[38];
  root_3 += 560*c3p3*c5p3*c9p2*ss[42];
  root_3 += 280*c3p3*c5p4*c7p1*ss[36];
  root_3 += 280*c3p3*c5p4*c9p1*ss[38];
  root_3 += 280*c3p3*c7p1*c9p4*ss[52];
  root_3 += 560*c3p3*c7p2*c9p3*ss[50];
  root_3 += 560*c3p3*c7p3*c9p2*ss[48];
  root_3 += 280*c3p3*c7p4*c9p1*ss[46];
  root_3 += 280*c3p4*c5p1*c7p3*ss[38];
  root_3 += 280*c3p4*c5p1*c9p3*ss[44];
  root_3 += 420*c3p4*c5p2*c7p2*ss[36];
  root_3 += 420*c3p4*c5p2*c9p2*ss[40];
  root_3 += 280*c3p4*c5p3*c7p1*ss[34];
  root_3 += 280*c3p4*c5p3*c9p1*ss[36];
  root_3 += 280*c3p4*c7p1*c9p3*ss[46];
  root_3 += 420*c3p4*c7p2*c9p2*ss[44];
  root_3 += 280*c3p4*c7p3*c9p1*ss[42];
  root_3 += 168*c3p5*c5p1*c7p2*ss[34];
  root_3 += 168*c3p5*c5p1*c9p2*ss[38];
  root_3 += 168*c3p5*c5p2*c7p1*ss[32];
  root_3 += 168*c3p5*c5p2*c9p1*ss[34];
  root_3 += 168*c3p5*c7p1*c9p2*ss[40];
  root_3 += 168*c3p5*c7p2*c9p1*ss[38];
  root_3 += 56*c3p6*c5p1*c7p1*ss[30];
  root_3 += 56*c3p6*c5p1*c9p1*ss[32];
  root_3 += 56*c3p6*c7p1*c9p1*ss[34];
  root_3 += 56*c5p1*c7p1*c9p6*ss[66];
  root_3 += 168*c5p1*c7p2*c9p5*ss[64];
  root_3 += 280*c5p1*c7p3*c9p4*ss[62];
  root_3 += 280*c5p1*c7p4*c9p3*ss[60];
  root_3 += 168*c5p1*c7p5*c9p2*ss[58];
  root_3 += 56*c5p1*c7p6*c9p1*ss[56];
  root_3 += 168*c5p2*c7p1*c9p5*ss[62];
  root_3 += 420*c5p2*c7p2*c9p4*ss[60];
  root_3 += 560*c5p2*c7p3*c9p3*ss[58];
  root_3 += 420*c5p2*c7p4*c9p2*ss[56];
  root_3 += 168*c5p2*c7p5*c9p1*ss[54];
  root_3 += 280*c5p3*c7p1*c9p4*ss[58];
  root_3 += 560*c5p3*c7p2*c9p3*ss[56];
  root_3 += 560*c5p3*c7p3*c9p2*ss[54];
  root_3 += 280*c5p3*c7p4*c9p1*ss[52];
  root_3 += 280*c5p4*c7p1*c9p3*ss[54];
  root_3 += 420*c5p4*c7p2*c9p2*ss[52];
  root_3 += 280*c5p4*c7p3*c9p1*ss[50];
  root_3 += 168*c5p5*c7p1*c9p2*ss[50];
  root_3 += 168*c5p5*c7p2*c9p1*ss[48];
  root_3 += 56*c5p6*c7p1*c9p1*ss[46];
  root_3 += 8*c1p1*c3p7*ss[22];
  root_3 += 8*c1p1*c5p7*ss[36];
  root_3 += 8*c1p1*c7p7*ss[50];
  root_3 += 8*c1p1*c9p7*ss[64];
  root_3 += 28*c1p2*c3p6*ss[20];
  root_3 += 28*c1p2*c5p6*ss[32];
  root_3 += 28*c1p2*c7p6*ss[44];
  root_3 += 28*c1p2*c9p6*ss[56];
  root_3 += 56*c1p3*c3p5*ss[18];
  root_3 += 56*c1p3*c5p5*ss[28];
  root_3 += 56*c1p3*c7p5*ss[38];
  root_3 += 56*c1p3*c9p5*ss[48];
  root_3 += 70*c1p4*c3p4*ss[16];
  root_3 += 70*c1p4*c5p4*ss[24];
  root_3 += 70*c1p4*c7p4*ss[32];
  root_3 += 70*c1p4*c9p4*ss[40];
  root_3 += 56*c1p5*c3p3*ss[14];
  root_3 += 56*c1p5*c5p3*ss[20];
  root_3 += 56*c1p5*c7p3*ss[26];
  root_3 += 56*c1p5*c9p3*ss[32];
  root_3 += 28*c1p6*c3p2*ss[12];
  root_3 += 28*c1p6*c5p2*ss[16];
  root_3 += 28*c1p6*c7p2*ss[20];
  root_3 += 28*c1p6*c9p2*ss[24];
  root_3 += 8*c1p7*c3p1*ss[10];
  root_3 += 8*c1p7*c5p1*ss[12];
  root_3 += 8*c1p7*c7p1*ss[14];
  root_3 += 8*c1p7*c9p1*ss[16];
  root_3 += 8*c3p1*c5p7*ss[38];
  root_3 += 8*c3p1*c7p7*ss[52];
  root_3 += 8*c3p1*c9p7*ss[66];
  root_3 += 28*c3p2*c5p6*ss[36];
  root_3 += 28*c3p2*c7p6*ss[48];
  root_3 += 28*c3p2*c9p6*ss[60];
  root_3 += 56*c3p3*c5p5*ss[34];
  root_3 += 56*c3p3*c7p5*ss[44];
  root_3 += 56*c3p3*c9p5*ss[54];
  root_3 += 70*c3p4*c5p4*ss[32];
  root_3 += 70*c3p4*c7p4*ss[40];
  root_3 += 70*c3p4*c9p4*ss[48];
  root_3 += 56*c3p5*c5p3*ss[30];
  root_3 += 56*c3p5*c7p3*ss[36];
  root_3 += 56*c3p5*c9p3*ss[42];
  root_3 += 28*c3p6*c5p2*ss[28];
  root_3 += 28*c3p6*c7p2*ss[32];
  root_3 += 28*c3p6*c9p2*ss[36];
  root_3 += 8*c3p7*c5p1*ss[26];
  root_3 += 8*c3p7*c7p1*ss[28];
  root_3 += 8*c3p7*c9p1*ss[30];
  root_3 += 8*c5p1*c7p7*ss[54];
  root_3 += 8*c5p1*c9p7*ss[68];
  root_3 += 28*c5p2*c7p6*ss[52];
  root_3 += 28*c5p2*c9p6*ss[64];
  root_3 += 56*c5p3*c7p5*ss[50];
  root_3 += 56*c5p3*c9p5*ss[60];
  root_3 += 70*c5p4*c7p4*ss[48];
  root_3 += 70*c5p4*c9p4*ss[56];
  root_3 += 56*c5p5*c7p3*ss[46];
  root_3 += 56*c5p5*c9p3*ss[52];
  root_3 += 28*c5p6*c7p2*ss[44];
  root_3 += 28*c5p6*c9p2*ss[48];
  root_3 += 8*c5p7*c7p1*ss[42];
  root_3 += 8*c5p7*c9p1*ss[44];
  root_3 += 8*c7p1*c9p7*ss[70];
  root_3 += 28*c7p2*c9p6*ss[68];
  root_3 += 56*c7p3*c9p5*ss[66];
  root_3 += 70*c7p4*c9p4*ss[64];
  root_3 += 56*c7p5*c9p3*ss[62];
  root_3 += 28*c7p6*c9p2*ss[60];
  root_3 += 8*c7p7*c9p1*ss[58];
  root_3 += c1p8*ss[8];
  root_3 += c3p8*ss[24];
  root_3 += c5p8*ss[40];
  root_3 += c7p8*ss[56];
  root_3 += c9p8*ss[72];
  root_3 += -tmom[8];
  T c1p9 = c1p8*c1p1;
  T c1p10 = c1p9*c1p1;
  T c3p9 = c3p8*c3p1;
  T c3p10 = c3p9*c3p1;
  T c5p9 = c5p8*c5p1;
  T c5p10 = c5p9*c5p1;
  T c7p9 = c7p8*c7p1;
  T c7p10 = c7p9*c7p1;
  T c9p9 = c9p8*c9p1;
  T c9p10 = c9p9*c9p1;
  TACC root_4 = convert<TACC>(5040*c1p1*c3p1*c5p1*c7p1*c9p6*ss[70]);
  root_4 += 15120*c1p1*c3p1*c5p1*c7p2*c9p5*ss[68];
  root_4 += 25200*c1p1*c3p1*c5p1*c7p3*c9p4*ss[66];
  root_4 += 25200*c1p1*c3p1*c5p1*c7p4*c9p3*ss[64];
  root_4 += 15120*c1p1*c3p1*c5p1*c7p5*c9p2*ss[62];
  root_4 += 5040*c1p1*c3p1*c5p1*c7p6*c9p1*ss[60];
  root_4 += 15120*c1p1*c3p1*c5p2*c7p1*c9p5*ss[66];
  root_4 += 37800*c1p1*c3p1*c5p2*c7p2*c9p4*ss[64];
  root_4 += 50400*c1p1*c3p1*c5p2*c7p3*c9p3*ss[62];
  root_4 += 37800*c1p1*c3p1*c5p2*c7p4*c9p2*ss[60];
  root_4 += 15120*c1p1*c3p1*c5p2*c7p5*c9p1*ss[58];
  root_4 += 25200*c1p1*c3p1*c5p3*c7p1*c9p4*ss[62];
  root_4 += 50400*c1p1*c3p1*c5p3*c7p2*c9p3*ss[60];
  root_4 += 50400*c1p1*c3p1*c5p3*c7p3*c9p2*ss[58];
  root_4 += 25200*c1p1*c3p1*c5p3*c7p4*c9p1*ss[56];
  root_4 += 25200*c1p1*c3p1*c5p4*c7p1*c9p3*ss[58];
  root_4 += 37800*c1p1*c3p1*c5p4*c7p2*c9p2*ss[56];
  root_4 += 25200*c1p1*c3p1*c5p4*c7p3*c9p1*ss[54];
  root_4 += 15120*c1p1*c3p1*c5p5*c7p1*c9p2*ss[54];
  root_4 += 15120*c1p1*c3p1*c5p5*c7p2*c9p1*ss[52];
  root_4 += 5040*c1p1*c3p1*c5p6*c7p1*c9p1*ss[50];
  root_4 += 15120*c1p1*c3p2*c5p1*c7p1*c9p5*ss[64];
  root_4 += 37800*c1p1*c3p2*c5p1*c7p2*c9p4*ss[62];
  root_4 += 50400*c1p1*c3p2*c5p1*c7p3*c9p3*ss[60];
  root_4 += 37800*c1p1*c3p2*c5p1*c7p4*c9p2*ss[58];
  root_4 += 15120*c1p1*c3p2*c5p1*c7p5*c9p1*ss[56];
  root_4 += 37800*c1p1*c3p2*c5p2*c7p1*c9p4*ss[60];
  root_4 += 75600*c1p1*c3p2*c5p2*c7p2*c9p3*ss[58];
  root_4 += 75600*c1p1*c3p2*c5p2*c7p3*c9p2*ss[56];
  root_4 += 37800*c1p1*c3p2*c5p2*c7p4*c9p1*ss[54];
  root_4 += 50400*c1p1*c3p2*c5p3*c7p1*c9p3*ss[56];
  root_4 += 75600*c1p1*c3p2*c5p3*c7p2*c9p2*ss[54];
  root_4 += 50400*c1p1*c3p2*c5p3*c7p3*c9p1*ss[52];
  root_4 += 37800*c1p1*c3p2*c5p4*c7p1*c9p2*ss[52];
  root_4 += 37800*c1p1*c3p2*c5p4*c7p2*c9p1*ss[50];
  root_4 += 15120*c1p1*c3p2*c5p5*c7p1*c9p1*ss[48];
  root_4 += 25200*c1p1*c3p3*c5p1*c7p1*c9p4*ss[58];
  root_4 += 50400*c1p1*c3p3*c5p1*c7p2*c9p3*ss[56];
  root_4 += 50400*c1p1*c3p3*c5p1*c7p3*c9p2*ss[54];
  root_4 += 25200*c1p1*c3p3*c5p1*c7p4*c9p1*ss[52];
  root_4 += 50400*c1p1*c3p3*c5p2*c7p1*c9p3*ss[54];
  root_4 += 75600*c1p1*c3p3*c5p2*c7p2*c9p2*ss[52];
  root_4 += 50400*c1p1*c3p3*c5p2*c7p3*c9p1*ss[50];
  root_4 += 50400*c1p1*c3p3*c5p3*c7p1*c9p2*ss[50];
  root_4 += 50400*c1p1*c3p3*c5p3*c7p2*c9p1*ss[48];
  root_4 += 25200*c1p1*c3p3*c5p4*c7p1*c9p1*ss[46];
  root_4 += 25200*c1p1*c3p4*c5p1*c7p1*c9p3*ss[52];
  root_4 += 37800*c1p1*c3p4*c5p1*c7p2*c9p2*ss[50];
  root_4 += 25200*c1p1*c3p4*c5p1*c7p3*c9p1*ss[48];
  root_4 += 37800*c1p1*c3p4*c5p2*c7p1*c9p2*ss[48];
  root_4 += 37800*c1p1*c3p4*c5p2*c7p2*c9p1*ss[46];
  root_4 += 25200*c1p1*c3p4*c5p3*c7p1*c9p1*ss[44];
  root_4 += 15120*c1p1*c3p5*c5p1*c7p1*c9p2*ss[46];
  root_4 += 15120*c1p1*c3p5*c5p1*c7p2*c9p1*ss[44];
  root_4 += 15120*c1p1*c3p5*c5p2*c7p1*c9p1*ss[42];
  root_4 += 5040*c1p1*c3p6*c5p1*c7p1*c9p1*ss[40];
  root_4 += 15120*c1p2*c3p1*c5p1*c7p1*c9p5*ss[62];
  root_4 += 37800*c1p2*c3p1*c5p1*c7p2*c9p4*ss[60];
  root_4 += 50400*c1p2*c3p1*c5p1*c7p3*c9p3*ss[58];
  root_4 += 37800*c1p2*c3p1*c5p1*c7p4*c9p2*ss[56];
  root_4 += 15120*c1p2*c3p1*c5p1*c7p5*c9p1*ss[54];
  root_4 += 37800*c1p2*c3p1*c5p2*c7p1*c9p4*ss[58];
  root_4 += 75600*c1p2*c3p1*c5p2*c7p2*c9p3*ss[56];
  root_4 += 75600*c1p2*c3p1*c5p2*c7p3*c9p2*ss[54];
  root_4 += 37800*c1p2*c3p1*c5p2*c7p4*c9p1*ss[52];
  root_4 += 50400*c1p2*c3p1*c5p3*c7p1*c9p3*ss[54];
  root_4 += 75600*c1p2*c3p1*c5p3*c7p2*c9p2*ss[52];
  root_4 += 50400*c1p2*c3p1*c5p3*c7p3*c9p1*ss[50];
  root_4 += 37800*c1p2*c3p1*c5p4*c7p1*c9p2*ss[50];
  root_4 += 37800*c1p2*c3p1*c5p4*c7p2*c9p1*ss[48];
  root_4 += 15120*c1p2*c3p1*c5p5*c7p1*c9p1*ss[46];
  root_4 += 37800*c1p2*c3p2*c5p1*c7p1*c9p4*ss[56];
  root_4 += 75600*c1p2*c3p2*c5p1*c7p2*c9p3*ss[54];
  root_4 += 75600*c1p2*c3p2*c5p1*c7p3*c9p2*ss[52];
  root_4 += 37800*c1p2*c3p2*c5p1*c7p4*c9p1*ss[50];
  root_4 += 75600*c1p2*c3p2*c5p2*c7p1*c9p3*ss[52];
  root_4 += 113400*c1p2*c3p2*c5p2*c7p2*c9p2*ss[50];
  root_4 += 75600*c1p2*c3p2*c5p2*c7p3*c9p1*ss[48];
  root_4 += 75600*c1p2*c3p2*c5p3*c7p1*c9p2*ss[48];
  root_4 += 75600*c1p2*c3p2*c5p3*c7p2*c9p1*ss[46];
  root_4 += 37800*c1p2*c3p2*c5p4*c7p1*c9p1*ss[44];
  root_4 += 50400*c1p2*c3p3*c5p1*c7p1*c9p3*ss[50];
  root_4 += 75600*c1p2*c3p3*c5p1*c7p2*c9p2*ss[48];
  root_4 += 50400*c1p2*c3p3*c5p1*c7p3*c9p1*ss[46];
  root_4 += 75600*c1p2*c3p3*c5p2*c7p1*c9p2*ss[46];
  root_4 += 75600*c1p2*c3p3*c5p2*c7p2*c9p1*ss[44];
  root_4 += 50400*c1p2*c3p3*c5p3*c7p1*c9p1*ss[42];
  root_4 += 37800*c1p2*c3p4*c5p1*c7p1*c9p2*ss[44];
  root_4 += 37800*c1p2*c3p4*c5p1*c7p2*c9p1*ss[42];
  root_4 += 37800*c1p2*c3p4*c5p2*c7p1*c9p1*ss[40];
  root_4 += 15120*c1p2*c3p5*c5p1*c7p1*c9p1*ss[38];
  root_4 += 25200*c1p3*c3p1*c5p1*c7p1*c9p4*ss[54];
  root_4 += 50400*c1p3*c3p1*c5p1*c7p2*c9p3*ss[52];
  root_4 += 50400*c1p3*c3p1*c5p1*c7p3*c9p2*ss[50];
  root_4 += 25200*c1p3*c3p1*c5p1*c7p4*c9p1*ss[48];
  root_4 += 50400*c1p3*c3p1*c5p2*c7p1*c9p3*ss[50];
  root_4 += 75600*c1p3*c3p1*c5p2*c7p2*c9p2*ss[48];
  root_4 += 50400*c1p3*c3p1*c5p2*c7p3*c9p1*ss[46];
  root_4 += 50400*c1p3*c3p1*c5p3*c7p1*c9p2*ss[46];
  root_4 += 50400*c1p3*c3p1*c5p3*c7p2*c9p1*ss[44];
  root_4 += 25200*c1p3*c3p1*c5p4*c7p1*c9p1*ss[42];
  root_4 += 50400*c1p3*c3p2*c5p1*c7p1*c9p3*ss[48];
  root_4 += 75600*c1p3*c3p2*c5p1*c7p2*c9p2*ss[46];
  root_4 += 50400*c1p3*c3p2*c5p1*c7p3*c9p1*ss[44];
  root_4 += 75600*c1p3*c3p2*c5p2*c7p1*c9p2*ss[44];
  root_4 += 75600*c1p3*c3p2*c5p2*c7p2*c9p1*ss[42];
  root_4 += 50400*c1p3*c3p2*c5p3*c7p1*c9p1*ss[40];
  root_4 += 50400*c1p3*c3p3*c5p1*c7p1*c9p2*ss[42];
  root_4 += 50400*c1p3*c3p3*c5p1*c7p2*c9p1*ss[40];
  root_4 += 50400*c1p3*c3p3*c5p2*c7p1*c9p1*ss[38];
  root_4 += 25200*c1p3*c3p4*c5p1*c7p1*c9p1*ss[36];
  root_4 += 25200*c1p4*c3p1*c5p1*c7p1*c9p3*ss[46];
  root_4 += 37800*c1p4*c3p1*c5p1*c7p2*c9p2*ss[44];
  root_4 += 25200*c1p4*c3p1*c5p1*c7p3*c9p1*ss[42];
  root_4 += 37800*c1p4*c3p1*c5p2*c7p1*c9p2*ss[42];
  root_4 += 37800*c1p4*c3p1*c5p2*c7p2*c9p1*ss[40];
  root_4 += 25200*c1p4*c3p1*c5p3*c7p1*c9p1*ss[38];
  root_4 += 37800*c1p4*c3p2*c5p1*c7p1*c9p2*ss[40];
  root_4 += 37800*c1p4*c3p2*c5p1*c7p2*c9p1*ss[38];
  root_4 += 37800*c1p4*c3p2*c5p2*c7p1*c9p1*ss[36];
  root_4 += 25200*c1p4*c3p3*c5p1*c7p1*c9p1*ss[34];
  root_4 += 15120*c1p5*c3p1*c5p1*c7p1*c9p2*ss[38];
  root_4 += 15120*c1p5*c3p1*c5p1*c7p2*c9p1*ss[36];
  root_4 += 15120*c1p5*c3p1*c5p2*c7p1*c9p1*ss[34];
  root_4 += 15120*c1p5*c3p2*c5p1*c7p1*c9p1*ss[32];
  root_4 += 5040*c1p6*c3p1*c5p1*c7p1*c9p1*ss[30];
  root_4 += 720*c1p1*c3p1*c5p1*c7p7*ss[58];
  root_4 += 720*c1p1*c3p1*c5p1*c9p7*ss[72];
  root_4 += 2520*c1p1*c3p1*c5p2*c7p6*ss[56];
  root_4 += 2520*c1p1*c3p1*c5p2*c9p6*ss[68];
  root_4 += 5040*c1p1*c3p1*c5p3*c7p5*ss[54];
  root_4 += 5040*c1p1*c3p1*c5p3*c9p5*ss[64];
  root_4 += 6300*c1p1*c3p1*c5p4*c7p4*ss[52];
  root_4 += 6300*c1p1*c3p1*c5p4*c9p4*ss[60];
  root_4 += 5040*c1p1*c3p1*c5p5*c7p3*ss[50];
  root_4 += 5040*c1p1*c3p1*c5p5*c9p3*ss[56];
  root_4 += 2520*c1p1*c3p1*c5p6*c7p2*ss[48];
  root_4 += 2520*c1p1*c3p1*c5p6*c9p2*ss[52];
  root_4 += 720*c1p1*c3p1*c5p7*c7p1*ss[46];
  root_4 += 720*c1p1*c3p1*c5p7*c9p1*ss[48];
  root_4 += 720*c1p1*c3p1*c7p1*c9p7*ss[74];
  root_4 += 2520*c1p1*c3p1*c7p2*c9p6*ss[72];
  root_4 += 5040*c1p1*c3p1*c7p3*c9p5*ss[70];
  root_4 += 6300*c1p1*c3p1*c7p4*c9p4*ss[68];
  root_4 += 5040*c1p1*c3p1*c7p5*c9p3*ss[66];
  root_4 += 2520*c1p1*c3p1*c7p6*c9p2*ss[64];
  root_4 += 720*c1p1*c3p1*c7p7*c9p1*ss[62];
  root_4 += 2520*c1p1*c3p2*c5p1*c7p6*ss[54];
  root_4 += 2520*c1p1*c3p2*c5p1*c9p6*ss[66];
  root_4 += 7560*c1p1*c3p2*c5p2*c7p5*ss[52];
  root_4 += 7560*c1p1*c3p2*c5p2*c9p5*ss[62];
  root_4 += 12600*c1p1*c3p2*c5p3*c7p4*ss[50];
  root_4 += 12600*c1p1*c3p2*c5p3*c9p4*ss[58];
  root_4 += 12600*c1p1*c3p2*c5p4*c7p3*ss[48];
  root_4 += 12600*c1p1*c3p2*c5p4*c9p3*ss[54];
  root_4 += 7560*c1p1*c3p2*c5p5*c7p2*ss[46];
  root_4 += 7560*c1p1*c3p2*c5p5*c9p2*ss[50];
  root_4 += 2520*c1p1*c3p2*c5p6*c7p1*ss[44];
  root_4 += 2520*c1p1*c3p2*c5p6*c9p1*ss[46];
  root_4 += 2520*c1p1*c3p2*c7p1*c9p6*ss[68];
  root_4 += 7560*c1p1*c3p2*c7p2*c9p5*ss[66];
  root_4 += 12600*c1p1*c3p2*c7p3*c9p4*ss[64];
  root_4 += 12600*c1p1*c3p2*c7p4*c9p3*ss[62];
  root_4 += 7560*c1p1*c3p2*c7p5*c9p2*ss[60];
  root_4 += 2520*c1p1*c3p2*c7p6*c9p1*ss[58];
  root_4 += 5040*c1p1*c3p3*c5p1*c7p5*ss[50];
  root_4 += 5040*c1p1*c3p3*c5p1*c9p5*ss[60];
  root_4 += 12600*c1p1*c3p3*c5p2*c7p4*ss[48];
  root_4 += 12600*c1p1*c3p3*c5p2*c9p4*ss[56];
  root_4 += 16800*c1p1*c3p3*c5p3*c7p3*ss[46];
  root_4 += 16800*c1p1*c3p3*c5p3*c9p3*ss[52];
  root_4 += 12600*c1p1*c3p3*c5p4*c7p2*ss[44];
  root_4 += 12600*c1p1*c3p3*c5p4*c9p2*ss[48];
  root_4 += 5040*c1p1*c3p3*c5p5*c7p1*ss[42];
  root_4 += 5040*c1p1*c3p3*c5p5*c9p1*ss[44];
  root_4 += 5040*c1p1*c3p3*c7p1*c9p5*ss[62];
  root_4 += 12600*c1p1*c3p3*c7p2*c9p4*ss[60];
  root_4 += 16800*c1p1*c3p3*c7p3*c9p3*ss[58];
  root_4 += 12600*c1p1*c3p3*c7p4*c9p2*ss[56];
  root_4 += 5040*c1p1*c3p3*c7p5*c9p1*ss[54];
  root_4 += 6300*c1p1*c3p4*c5p1*c7p4*ss[46];
  root_4 += 6300*c1p1*c3p4*c5p1*c9p4*ss[54];
  root_4 += 12600*c1p1*c3p4*c5p2*c7p3*ss[44];
  root_4 += 12600*c1p1*c3p4*c5p2*c9p3*ss[50];
  root_4 += 12600*c1p1*c3p4*c5p3*c7p2*ss[42];
  root_4 += 12600*c1p1*c3p4*c5p3*c9p2*ss[46];
  root_4 += 6300*c1p1*c3p4*c5p4*c7p1*ss[40];
  root_4 += 6300*c1p1*c3p4*c5p4*c9p1*ss[42];
  root_4 += 6300*c1p1*c3p4*c7p1*c9p4*ss[56];
  root_4 += 12600*c1p1*c3p4*c7p2*c9p3*ss[54];
  root_4 += 12600*c1p1*c3p4*c7p3*c9p2*ss[52];
  root_4 += 6300*c1p1*c3p4*c7p4*c9p1*ss[50];
  root_4 += 5040*c1p1*c3p5*c5p1*c7p3*ss[42];
  root_4 += 5040*c1p1*c3p5*c5p1*c9p3*ss[48];
  root_4 += 7560*c1p1*c3p5*c5p2*c7p2*ss[40];
  root_4 += 7560*c1p1*c3p5*c5p2*c9p2*ss[44];
  root_4 += 5040*c1p1*c3p5*c5p3*c7p1*ss[38];
  root_4 += 5040*c1p1*c3p5*c5p3*c9p1*ss[40];
  root_4 += 5040*c1p1*c3p5*c7p1*c9p3*ss[50];
  root_4 += 7560*c1p1*c3p5*c7p2*c9p2*ss[48];
  root_4 += 5040*c1p1*c3p5*c7p3*c9p1*ss[46];
  root_4 += 2520*c1p1*c3p6*c5p1*c7p2*ss[38];
  root_4 += 2520*c1p1*c3p6*c5p1*c9p2*ss[42];
  root_4 += 2520*c1p1*c3p6*c5p2*c7p1*ss[36];
  root_4 += 2520*c1p1*c3p6*c5p2*c9p1*ss[38];
  root_4 += 2520*c1p1*c3p6*c7p1*c9p2*ss[44];
  root_4 += 2520*c1p1*c3p6*c7p2*c9p1*ss[42];
  root_4 += 720*c1p1*c3p7*c5p1*c7p1*ss[34];
  root_4 += 720*c1p1*c3p7*c5p1*c9p1*ss[36];
  root_4 += 720*c1p1*c3p7*c7p1*c9p1*ss[38];
  root_4 += 720*c1p1*c5p1*c7p1*c9p7*ss[76];
  root_4 += 2520*c1p1*c5p1*c7p2*c9p6*ss[74];
  root_4 += 5040*c1p1*c5p1*c7p3*c9p5*ss[72];
  root_4 += 6300*c1p1*c5p1*c7p4*c9p4*ss[70];
  root_4 += 5040*c1p1*c5p1*c7p5*c9p3*ss[68];
  root_4 += 2520*c1p1*c5p1*c7p6*c9p2*ss[66];
  root_4 += 720*c1p1*c5p1*c7p7*c9p1*ss[64];
  root_4 += 2520*c1p1*c5p2*c7p1*c9p6*ss[72];
  root_4 += 7560*c1p1*c5p2*c7p2*c9p5*ss[70];
  root_4 += 12600*c1p1*c5p2*c7p3*c9p4*ss[68];
  root_4 += 12600*c1p1*c5p2*c7p4*c9p3*ss[66];
  root_4 += 7560*c1p1*c5p2*c7p5*c9p2*ss[64];
  root_4 += 2520*c1p1*c5p2*c7p6*c9p1*ss[62];
  root_4 += 5040*c1p1*c5p3*c7p1*c9p5*ss[68];
  root_4 += 12600*c1p1*c5p3*c7p2*c9p4*ss[66];
  root_4 += 16800*c1p1*c5p3*c7p3*c9p3*ss[64];
  root_4 += 12600*c1p1*c5p3*c7p4*c9p2*ss[62];
  root_4 += 5040*c1p1*c5p3*c7p5*c9p1*ss[60];
  root_4 += 6300*c1p1*c5p4*c7p1*c9p4*ss[64];
  root_4 += 12600*c1p1*c5p4*c7p2*c9p3*ss[62];
  root_4 += 12600*c1p1*c5p4*c7p3*c9p2*ss[60];
  root_4 += 6300*c1p1*c5p4*c7p4*c9p1*ss[58];
  root_4 += 5040*c1p1*c5p5*c7p1*c9p3*ss[60];
  root_4 += 7560*c1p1*c5p5*c7p2*c9p2*ss[58];
  root_4 += 5040*c1p1*c5p5*c7p3*c9p1*ss[56];
  root_4 += 2520*c1p1*c5p6*c7p1*c9p2*ss[56];
  root_4 += 2520*c1p1*c5p6*c7p2*c9p1*ss[54];
  root_4 += 720*c1p1*c5p7*c7p1*c9p1*ss[52];
  root_4 += 2520*c1p2*c3p1*c5p1*c7p6*ss[52];
  root_4 += 2520*c1p2*c3p1*c5p1*c9p6*ss[64];
  root_4 += 7560*c1p2*c3p1*c5p2*c7p5*ss[50];
  root_4 += 7560*c1p2*c3p1*c5p2*c9p5*ss[60];
  root_4 += 12600*c1p2*c3p1*c5p3*c7p4*ss[48];
  root_4 += 12600*c1p2*c3p1*c5p3*c9p4*ss[56];
  root_4 += 12600*c1p2*c3p1*c5p4*c7p3*ss[46];
  root_4 += 12600*c1p2*c3p1*c5p4*c9p3*ss[52];
  root_4 += 7560*c1p2*c3p1*c5p5*c7p2*ss[44];
  root_4 += 7560*c1p2*c3p1*c5p5*c9p2*ss[48];
  root_4 += 2520*c1p2*c3p1*c5p6*c7p1*ss[42];
  root_4 += 2520*c1p2*c3p1*c5p6*c9p1*ss[44];
  root_4 += 2520*c1p2*c3p1*c7p1*c9p6*ss[66];
  root_4 += 7560*c1p2*c3p1*c7p2*c9p5*ss[64];
  root_4 += 12600*c1p2*c3p1*c7p3*c9p4*ss[62];
  root_4 += 12600*c1p2*c3p1*c7p4*c9p3*ss[60];
  root_4 += 7560*c1p2*c3p1*c7p5*c9p2*ss[58];
  root_4 += 2520*c1p2*c3p1*c7p6*c9p1*ss[56];
  root_4 += 7560*c1p2*c3p2*c5p1*c7p5*ss[48];
  root_4 += 7560*c1p2*c3p2*c5p1*c9p5*ss[58];
  root_4 += 18900*c1p2*c3p2*c5p2*c7p4*ss[46];
  root_4 += 18900*c1p2*c3p2*c5p2*c9p4*ss[54];
  root_4 += 25200*c1p2*c3p2*c5p3*c7p3*ss[44];
  root_4 += 25200*c1p2*c3p2*c5p3*c9p3*ss[50];
  root_4 += 18900*c1p2*c3p2*c5p4*c7p2*ss[42];
  root_4 += 18900*c1p2*c3p2*c5p4*c9p2*ss[46];
  root_4 += 7560*c1p2*c3p2*c5p5*c7p1*ss[40];
  root_4 += 7560*c1p2*c3p2*c5p5*c9p1*ss[42];
  root_4 += 7560*c1p2*c3p2*c7p1*c9p5*ss[60];
  root_4 += 18900*c1p2*c3p2*c7p2*c9p4*ss[58];
  root_4 += 25200*c1p2*c3p2*c7p3*c9p3*ss[56];
  root_4 += 18900*c1p2*c3p2*c7p4*c9p2*ss[54];
  root_4 += 7560*c1p2*c3p2*c7p5*c9p1*ss[52];
  root_4 += 12600*c1p2*c3p3*c5p1*c7p4*ss[44];
  root_4 += 12600*c1p2*c3p3*c5p1*c9p4*ss[52];
  root_4 += 25200*c1p2*c3p3*c5p2*c7p3*ss[42];
  root_4 += 25200*c1p2*c3p3*c5p2*c9p3*ss[48];
  root_4 += 25200*c1p2*c3p3*c5p3*c7p2*ss[40];
  root_4 += 25200*c1p2*c3p3*c5p3*c9p2*ss[44];
  root_4 += 12600*c1p2*c3p3*c5p4*c7p1*ss[38];
  root_4 += 12600*c1p2*c3p3*c5p4*c9p1*ss[40];
  root_4 += 12600*c1p2*c3p3*c7p1*c9p4*ss[54];
  root_4 += 25200*c1p2*c3p3*c7p2*c9p3*ss[52];
  root_4 += 25200*c1p2*c3p3*c7p3*c9p2*ss[50];
  root_4 += 12600*c1p2*c3p3*c7p4*c9p1*ss[48];
  root_4 += 12600*c1p2*c3p4*c5p1*c7p3*ss[40];
  root_4 += 12600*c1p2*c3p4*c5p1*c9p3*ss[46];
  root_4 += 18900*c1p2*c3p4*c5p2*c7p2*ss[38];
  root_4 += 18900*c1p2*c3p4*c5p2*c9p2*ss[42];
  root_4 += 12600*c1p2*c3p4*c5p3*c7p1*ss[36];
  root_4 += 12600*c1p2*c3p4*c5p3*c9p1*ss[38];
  root_4 += 12600*c1p2*c3p4*c7p1*c9p3*ss[48];
  root_4 += 18900*c1p2*c3p4*c7p2*c9p2*ss[46];
  root_4 += 12600*c1p2*c3p4*c7p3*c9p1*ss[44];
  root_4 += 7560*c1p2*c3p5*c5p1*c7p2*ss[36];
  root_4 += 7560*c1p2*c3p5*c5p1*c9p2*ss[40];
  root_4 += 7560*c1p2*c3p5*c5p2*c7p1*ss[34];
  root_4 += 7560*c1p2*c3p5*c5p2*c9p1*ss[36];
  root_4 += 7560*c1p2*c3p5*c7p1*c9p2*ss[42];
  root_4 += 7560*c1p2*c3p5*c7p2*c9p1*ss[40];
  root_4 += 2520*c1p2*c3p6*c5p1*c7p1*ss[32];
  root_4 += 2520*c1p2*c3p6*c5p1*c9p1*ss[34];
  root_4 += 2520*c1p2*c3p6*c7p1*c9p1*ss[36];
  root_4 += 2520*c1p2*c5p1*c7p1*c9p6*ss[68];
  root_4 += 7560*c1p2*c5p1*c7p2*c9p5*ss[66];
  root_4 += 12600*c1p2*c5p1*c7p3*c9p4*ss[64];
  root_4 += 12600*c1p2*c5p1*c7p4*c9p3*ss[62];
  root_4 += 7560*c1p2*c5p1*c7p5*c9p2*ss[60];
  root_4 += 2520*c1p2*c5p1*c7p6*c9p1*ss[58];
  root_4 += 7560*c1p2*c5p2*c7p1*c9p5*ss[64];
  root_4 += 18900*c1p2*c5p2*c7p2*c9p4*ss[62];
  root_4 += 25200*c1p2*c5p2*c7p3*c9p3*ss[60];
  root_4 += 18900*c1p2*c5p2*c7p4*c9p2*ss[58];
  root_4 += 7560*c1p2*c5p2*c7p5*c9p1*ss[56];
  root_4 += 12600*c1p2*c5p3*c7p1*c9p4*ss[60];
  root_4 += 25200*c1p2*c5p3*c7p2*c9p3*ss[58];
  root_4 += 25200*c1p2*c5p3*c7p3*c9p2*ss[56];
  root_4 += 12600*c1p2*c5p3*c7p4*c9p1*ss[54];
  root_4 += 12600*c1p2*c5p4*c7p1*c9p3*ss[56];
  root_4 += 18900*c1p2*c5p4*c7p2*c9p2*ss[54];
  root_4 += 12600*c1p2*c5p4*c7p3*c9p1*ss[52];
  root_4 += 7560*c1p2*c5p5*c7p1*c9p2*ss[52];
  root_4 += 7560*c1p2*c5p5*c7p2*c9p1*ss[50];
  root_4 += 2520*c1p2*c5p6*c7p1*c9p1*ss[48];
  root_4 += 5040*c1p3*c3p1*c5p1*c7p5*ss[46];
  root_4 += 5040*c1p3*c3p1*c5p1*c9p5*ss[56];
  root_4 += 12600*c1p3*c3p1*c5p2*c7p4*ss[44];
  root_4 += 12600*c1p3*c3p1*c5p2*c9p4*ss[52];
  root_4 += 16800*c1p3*c3p1*c5p3*c7p3*ss[42];
  root_4 += 16800*c1p3*c3p1*c5p3*c9p3*ss[48];
  root_4 += 12600*c1p3*c3p1*c5p4*c7p2*ss[40];
  root_4 += 12600*c1p3*c3p1*c5p4*c9p2*ss[44];
  root_4 += 5040*c1p3*c3p1*c5p5*c7p1*ss[38];
  root_4 += 5040*c1p3*c3p1*c5p5*c9p1*ss[40];
  root_4 += 5040*c1p3*c3p1*c7p1*c9p5*ss[58];
  root_4 += 12600*c1p3*c3p1*c7p2*c9p4*ss[56];
  root_4 += 16800*c1p3*c3p1*c7p3*c9p3*ss[54];
  root_4 += 12600*c1p3*c3p1*c7p4*c9p2*ss[52];
  root_4 += 5040*c1p3*c3p1*c7p5*c9p1*ss[50];
  root_4 += 12600*c1p3*c3p2*c5p1*c7p4*ss[42];
  root_4 += 12600*c1p3*c3p2*c5p1*c9p4*ss[50];
  root_4 += 25200*c1p3*c3p2*c5p2*c7p3*ss[40];
  root_4 += 25200*c1p3*c3p2*c5p2*c9p3*ss[46];
  root_4 += 25200*c1p3*c3p2*c5p3*c7p2*ss[38];
  root_4 += 25200*c1p3*c3p2*c5p3*c9p2*ss[42];
  root_4 += 12600*c1p3*c3p2*c5p4*c7p1*ss[36];
  root_4 += 12600*c1p3*c3p2*c5p4*c9p1*ss[38];
  root_4 += 12600*c1p3*c3p2*c7p1*c9p4*ss[52];
  root_4 += 25200*c1p3*c3p2*c7p2*c9p3*ss[50];
  root_4 += 25200*c1p3*c3p2*c7p3*c9p2*ss[48];
  root_4 += 12600*c1p3*c3p2*c7p4*c9p1*ss[46];
  root_4 += 16800*c1p3*c3p3*c5p1*c7p3*ss[38];
  root_4 += 16800*c1p3*c3p3*c5p1*c9p3*ss[44];
  root_4 += 25200*c1p3*c3p3*c5p2*c7p2*ss[36];
  root_4 += 25200*c1p3*c3p3*c5p2*c9p2*ss[40];
  root_4 += 16800*c1p3*c3p3*c5p3*c7p1*ss[34];
  root_4 += 16800*c1p3*c3p3*c5p3*c9p1*ss[36];
  root_4 += 16800*c1p3*c3p3*c7p1*c9p3*ss[46];
  root_4 += 25200*c1p3*c3p3*c7p2*c9p2*ss[44];
  root_4 += 16800*c1p3*c3p3*c7p3*c9p1*ss[42];
  root_4 += 12600*c1p3*c3p4*c5p1*c7p2*ss[34];
  root_4 += 12600*c1p3*c3p4*c5p1*c9p2*ss[38];
  root_4 += 12600*c1p3*c3p4*c5p2*c7p1*ss[32];
  root_4 += 12600*c1p3*c3p4*c5p2*c9p1*ss[34];
  root_4 += 12600*c1p3*c3p4*c7p1*c9p2*ss[40];
  root_4 += 12600*c1p3*c3p4*c7p2*c9p1*ss[38];
  root_4 += 5040*c1p3*c3p5*c5p1*c7p1*ss[30];
  root_4 += 5040*c1p3*c3p5*c5p1*c9p1*ss[32];
  root_4 += 5040*c1p3*c3p5*c7p1*c9p1*ss[34];
  root_4 += 5040*c1p3*c5p1*c7p1*c9p5*ss[60];
  root_4 += 12600*c1p3*c5p1*c7p2*c9p4*ss[58];
  root_4 += 16800*c1p3*c5p1*c7p3*c9p3*ss[56];
  root_4 += 12600*c1p3*c5p1*c7p4*c9p2*ss[54];
  root_4 += 5040*c1p3*c5p1*c7p5*c9p1*ss[52];
  root_4 += 12600*c1p3*c5p2*c7p1*c9p4*ss[56];
  root_4 += 25200*c1p3*c5p2*c7p2*c9p3*ss[54];
  root_4 += 25200*c1p3*c5p2*c7p3*c9p2*ss[52];
  root_4 += 12600*c1p3*c5p2*c7p4*c9p1*ss[50];
  root_4 += 16800*c1p3*c5p3*c7p1*c9p3*ss[52];
  root_4 += 25200*c1p3*c5p3*c7p2*c9p2*ss[50];
  root_4 += 16800*c1p3*c5p3*c7p3*c9p1*ss[48];
  root_4 += 12600*c1p3*c5p4*c7p1*c9p2*ss[48];
  root_4 += 12600*c1p3*c5p4*c7p2*c9p1*ss[46];
  root_4 += 5040*c1p3*c5p5*c7p1*c9p1*ss[44];
  root_4 += 6300*c1p4*c3p1*c5p1*c7p4*ss[40];
  root_4 += 6300*c1p4*c3p1*c5p1*c9p4*ss[48];
  root_4 += 12600*c1p4*c3p1*c5p2*c7p3*ss[38];
  root_4 += 12600*c1p4*c3p1*c5p2*c9p3*ss[44];
  root_4 += 12600*c1p4*c3p1*c5p3*c7p2*ss[36];
  root_4 += 12600*c1p4*c3p1*c5p3*c9p2*ss[40];
  root_4 += 6300*c1p4*c3p1*c5p4*c7p1*ss[34];
  root_4 += 6300*c1p4*c3p1*c5p4*c9p1*ss[36];
  root_4 += 6300*c1p4*c3p1*c7p1*c9p4*ss[50];
  root_4 += 12600*c1p4*c3p1*c7p2*c9p3*ss[48];
  root_4 += 12600*c1p4*c3p1*c7p3*c9p2*ss[46];
  root_4 += 6300*c1p4*c3p1*c7p4*c9p1*ss[44];
  root_4 += 12600*c1p4*c3p2*c5p1*c7p3*ss[36];
  root_4 += 12600*c1p4*c3p2*c5p1*c9p3*ss[42];
  root_4 += 18900*c1p4*c3p2*c5p2*c7p2*ss[34];
  root_4 += 18900*c1p4*c3p2*c5p2*c9p2*ss[38];
  root_4 += 12600*c1p4*c3p2*c5p3*c7p1*ss[32];
  root_4 += 12600*c1p4*c3p2*c5p3*c9p1*ss[34];
  root_4 += 12600*c1p4*c3p2*c7p1*c9p3*ss[44];
  root_4 += 18900*c1p4*c3p2*c7p2*c9p2*ss[42];
  root_4 += 12600*c1p4*c3p2*c7p3*c9p1*ss[40];
  root_4 += 12600*c1p4*c3p3*c5p1*c7p2*ss[32];
  root_4 += 12600*c1p4*c3p3*c5p1*c9p2*ss[36];
  root_4 += 12600*c1p4*c3p3*c5p2*c7p1*ss[30];
  root_4 += 12600*c1p4*c3p3*c5p2*c9p1*ss[32];
  root_4 += 12600*c1p4*c3p3*c7p1*c9p2*ss[38];
  root_4 += 12600*c1p4*c3p3*c7p2*c9p1*ss[36];
  root_4 += 6300*c1p4*c3p4*c5p1*c7p1*ss[28];
  root_4 += 6300*c1p4*c3p4*c5p1*c9p1*ss[30];
  root_4 += 6300*c1p4*c3p4*c7p1*c9p1*ss[32];
  root_4 += 6300*c1p4*c5p1*c7p1*c9p4*ss[52];
  root_4 += 12600*c1p4*c5p1*c7p2*c9p3*ss[50];
  root_4 += 12600*c1p4*c5p1*c7p3*c9p2*ss[48];
  root_4 += 6300*c1p4*c5p1*c7p4*c9p1*ss[46];
  root_4 += 12600*c1p4*c5p2*c7p1*c9p3*ss[48];
  root_4 += 18900*c1p4*c5p2*c7p2*c9p2*ss[46];
  root_4 += 12600*c1p4*c5p2*c7p3*c9p1*ss[44];
  root_4 += 12600*c1p4*c5p3*c7p1*c9p2*ss[44];
  root_4 += 12600*c1p4*c5p3*c7p2*c9p1*ss[42];
  root_4 += 6300*c1p4*c5p4*c7p1*c9p1*ss[40];
  root_4 += 5040*c1p5*c3p1*c5p1*c7p3*ss[34];
  root_4 += 5040*c1p5*c3p1*c5p1*c9p3*ss[40];
  root_4 += 7560*c1p5*c3p1*c5p2*c7p2*ss[32];
  root_4 += 7560*c1p5*c3p1*c5p2*c9p2*ss[36];
  root_4 += 5040*c1p5*c3p1*c5p3*c7p1*ss[30];
  root_4 += 5040*c1p5*c3p1*c5p3*c9p1*ss[32];
  root_4 += 5040*c1p5*c3p1*c7p1*c9p3*ss[42];
  root_4 += 7560*c1p5*c3p1*c7p2*c9p2*ss[40];
  root_4 += 5040*c1p5*c3p1*c7p3*c9p1*ss[38];
  root_4 += 7560*c1p5*c3p2*c5p1*c7p2*ss[30];
  root_4 += 7560*c1p5*c3p2*c5p1*c9p2*ss[34];
  root_4 += 7560*c1p5*c3p2*c5p2*c7p1*ss[28];
  root_4 += 7560*c1p5*c3p2*c5p2*c9p1*ss[30];
  root_4 += 7560*c1p5*c3p2*c7p1*c9p2*ss[36];
  root_4 += 7560*c1p5*c3p2*c7p2*c9p1*ss[34];
  root_4 += 5040*c1p5*c3p3*c5p1*c7p1*ss[26];
  root_4 += 5040*c1p5*c3p3*c5p1*c9p1*ss[28];
  root_4 += 5040*c1p5*c3p3*c7p1*c9p1*ss[30];
  root_4 += 5040*c1p5*c5p1*c7p1*c9p3*ss[44];
  root_4 += 7560*c1p5*c5p1*c7p2*c9p2*ss[42];
  root_4 += 5040*c1p5*c5p1*c7p3*c9p1*ss[40];
  root_4 += 7560*c1p5*c5p2*c7p1*c9p2*ss[40];
  root_4 += 7560*c1p5*c5p2*c7p2*c9p1*ss[38];
  root_4 += 5040*c1p5*c5p3*c7p1*c9p1*ss[36];
  root_4 += 2520*c1p6*c3p1*c5p1*c7p2*ss[28];
  root_4 += 2520*c1p6*c3p1*c5p1*c9p2*ss[32];
  root_4 += 2520*c1p6*c3p1*c5p2*c7p1*ss[26];
  root_4 += 2520*c1p6*c3p1*c5p2*c9p1*ss[28];
  root_4 += 2520*c1p6*c3p1*c7p1*c9p2*ss[34];
  root_4 += 2520*c1p6*c3p1*c7p2*c9p1*ss[32];
  root_4 += 2520*c1p6*c3p2*c5p1*c7p1*ss[24];
  root_4 += 2520*c1p6*c3p2*c5p1*c9p1*ss[26];
  root_4 += 2520*c1p6*c3p2*c7p1*c9p1*ss[28];
  root_4 += 2520*c1p6*c5p1*c7p1*c9p2*ss[36];
  root_4 += 2520*c1p6*c5p1*c7p2*c9p1*ss[34];
  root_4 += 2520*c1p6*c5p2*c7p1*c9p1*ss[32];
  root_4 += 720*c1p7*c3p1*c5p1*c7p1*ss[22];
  root_4 += 720*c1p7*c3p1*c5p1*c9p1*ss[24];
  root_4 += 720*c1p7*c3p1*c7p1*c9p1*ss[26];
  root_4 += 720*c1p7*c5p1*c7p1*c9p1*ss[28];
  root_4 += 720*c3p1*c5p1*c7p1*c9p7*ss[78];
  root_4 += 2520*c3p1*c5p1*c7p2*c9p6*ss[76];
  root_4 += 5040*c3p1*c5p1*c7p3*c9p5*ss[74];
  root_4 += 6300*c3p1*c5p1*c7p4*c9p4*ss[72];
  root_4 += 5040*c3p1*c5p1*c7p5*c9p3*ss[70];
  root_4 += 2520*c3p1*c5p1*c7p6*c9p2*ss[68];
  root_4 += 720*c3p1*c5p1*c7p7*c9p1*ss[66];
  root_4 += 2520*c3p1*c5p2*c7p1*c9p6*ss[74];
  root_4 += 7560*c3p1*c5p2*c7p2*c9p5*ss[72];
  root_4 += 12600*c3p1*c5p2*c7p3*c9p4*ss[70];
  root_4 += 12600*c3p1*c5p2*c7p4*c9p3*ss[68];
  root_4 += 7560*c3p1*c5p2*c7p5*c9p2*ss[66];
  root_4 += 2520*c3p1*c5p2*c7p6*c9p1*ss[64];
  root_4 += 5040*c3p1*c5p3*c7p1*c9p5*ss[70];
  root_4 += 12600*c3p1*c5p3*c7p2*c9p4*ss[68];
  root_4 += 16800*c3p1*c5p3*c7p3*c9p3*ss[66];
  root_4 += 12600*c3p1*c5p3*c7p4*c9p2*ss[64];
  root_4 += 5040*c3p1*c5p3*c7p5*c9p1*ss[62];
  root_4 += 6300*c3p1*c5p4*c7p1*c9p4*ss[66];
  root_4 += 12600*c3p1*c5p4*c7p2*c9p3*ss[64];
  root_4 += 12600*c3p1*c5p4*c7p3*c9p2*ss[62];
  root_4 += 6300*c3p1*c5p4*c7p4*c9p1*ss[60];
  root_4 += 5040*c3p1*c5p5*c7p1*c9p3*ss[62];
  root_4 += 7560*c3p1*c5p5*c7p2*c9p2*ss[60];
  root_4 += 5040*c3p1*c5p5*c7p3*c9p1*ss[58];
  root_4 += 2520*c3p1*c5p6*c7p1*c9p2*ss[58];
  root_4 += 2520*c3p1*c5p6*c7p2*c9p1*ss[56];
  root_4 += 720*c3p1*c5p7*c7p1*c9p1*ss[54];
  root_4 += 2520*c3p2*c5p1*c7p1*c9p6*ss[72];
  root_4 += 7560*c3p2*c5p1*c7p2*c9p5*ss[70];
  root_4 += 12600*c3p2*c5p1*c7p3*c9p4*ss[68];
  root_4 += 12600*c3p2*c5p1*c7p4*c9p3*ss[66];
  root_4 += 7560*c3p2*c5p1*c7p5*c9p2*ss[64];
  root_4 += 2520*c3p2*c5p1*c7p6*c9p1*ss[62];
  root_4 += 7560*c3p2*c5p2*c7p1*c9p5*ss[68];
  root_4 += 18900*c3p2*c5p2*c7p2*c9p4*ss[66];
  root_4 += 25200*c3p2*c5p2*c7p3*c9p3*ss[64];
  root_4 += 18900*c3p2*c5p2*c7p4*c9p2*ss[62];
  root_4 += 7560*c3p2*c5p2*c7p5*c9p1*ss[60];
  root_4 += 12600*c3p2*c5p3*c7p1*c9p4*ss[64];
  root_4 += 25200*c3p2*c5p3*c7p2*c9p3*ss[62];
  root_4 += 25200*c3p2*c5p3*c7p3*c9p2*ss[60];
  root_4 += 12600*c3p2*c5p3*c7p4*c9p1*ss[58];
  root_4 += 12600*c3p2*c5p4*c7p1*c9p3*ss[60];
  root_4 += 18900*c3p2*c5p4*c7p2*c9p2*ss[58];
  root_4 += 12600*c3p2*c5p4*c7p3*c9p1*ss[56];
  root_4 += 7560*c3p2*c5p5*c7p1*c9p2*ss[56];
  root_4 += 7560*c3p2*c5p5*c7p2*c9p1*ss[54];
  root_4 += 2520*c3p2*c5p6*c7p1*c9p1*ss[52];
  root_4 += 5040*c3p3*c5p1*c7p1*c9p5*ss[66];
  root_4 += 12600*c3p3*c5p1*c7p2*c9p4*ss[64];
  root_4 += 16800*c3p3*c5p1*c7p3*c9p3*ss[62];
  root_4 += 12600*c3p3*c5p1*c7p4*c9p2*ss[60];
  root_4 += 5040*c3p3*c5p1*c7p5*c9p1*ss[58];
  root_4 += 12600*c3p3*c5p2*c7p1*c9p4*ss[62];
  root_4 += 25200*c3p3*c5p2*c7p2*c9p3*ss[60];
  root_4 += 25200*c3p3*c5p2*c7p3*c9p2*ss[58];
  root_4 += 12600*c3p3*c5p2*c7p4*c9p1*ss[56];
  root_4 += 16800*c3p3*c5p3*c7p1*c9p3*ss[58];
  root_4 += 25200*c3p3*c5p3*c7p2*c9p2*ss[56];
  root_4 += 16800*c3p3*c5p3*c7p3*c9p1*ss[54];
  root_4 += 12600*c3p3*c5p4*c7p1*c9p2*ss[54];
  root_4 += 12600*c3p3*c5p4*c7p2*c9p1*ss[52];
  root_4 += 5040*c3p3*c5p5*c7p1*c9p1*ss[50];
  root_4 += 6300*c3p4*c5p1*c7p1*c9p4*ss[60];
  root_4 += 12600*c3p4*c5p1*c7p2*c9p3*ss[58];
  root_4 += 12600*c3p4*c5p1*c7p3*c9p2*ss[56];
  root_4 += 6300*c3p4*c5p1*c7p4*c9p1*ss[54];
  root_4 += 12600*c3p4*c5p2*c7p1*c9p3*ss[56];
  root_4 += 18900*c3p4*c5p2*c7p2*c9p2*ss[54];
  root_4 += 12600*c3p4*c5p2*c7p3*c9p1*ss[52];
  root_4 += 12600*c3p4*c5p3*c7p1*c9p2*ss[52];
  root_4 += 12600*c3p4*c5p3*c7p2*c9p1*ss[50];
  root_4 += 6300*c3p4*c5p4*c7p1*c9p1*ss[48];
  root_4 += 5040*c3p5*c5p1*c7p1*c9p3*ss[54];
  root_4 += 7560*c3p5*c5p1*c7p2*c9p2*ss[52];
  root_4 += 5040*c3p5*c5p1*c7p3*c9p1*ss[50];
  root_4 += 7560*c3p5*c5p2*c7p1*c9p2*ss[50];
  root_4 += 7560*c3p5*c5p2*c7p2*c9p1*ss[48];
  root_4 += 5040*c3p5*c5p3*c7p1*c9p1*ss[46];
  root_4 += 2520*c3p6*c5p1*c7p1*c9p2*ss[48];
  root_4 += 2520*c3p6*c5p1*c7p2*c9p1*ss[46];
  root_4 += 2520*c3p6*c5p2*c7p1*c9p1*ss[44];
  root_4 += 720*c3p7*c5p1*c7p1*c9p1*ss[42];
  root_4 += 90*c1p1*c3p1*c5p8*ss[44];
  root_4 += 90*c1p1*c3p1*c7p8*ss[60];
  root_4 += 90*c1p1*c3p1*c9p8*ss[76];
  root_4 += 360*c1p1*c3p2*c5p7*ss[42];
  root_4 += 360*c1p1*c3p2*c7p7*ss[56];
  root_4 += 360*c1p1*c3p2*c9p7*ss[70];
  root_4 += 840*c1p1*c3p3*c5p6*ss[40];
  root_4 += 840*c1p1*c3p3*c7p6*ss[52];
  root_4 += 840*c1p1*c3p3*c9p6*ss[64];
  root_4 += 1260*c1p1*c3p4*c5p5*ss[38];
  root_4 += 1260*c1p1*c3p4*c7p5*ss[48];
  root_4 += 1260*c1p1*c3p4*c9p5*ss[58];
  root_4 += 1260*c1p1*c3p5*c5p4*ss[36];
  root_4 += 1260*c1p1*c3p5*c7p4*ss[44];
  root_4 += 1260*c1p1*c3p5*c9p4*ss[52];
  root_4 += 840*c1p1*c3p6*c5p3*ss[34];
  root_4 += 840*c1p1*c3p6*c7p3*ss[40];
  root_4 += 840*c1p1*c3p6*c9p3*ss[46];
  root_4 += 360*c1p1*c3p7*c5p2*ss[32];
  root_4 += 360*c1p1*c3p7*c7p2*ss[36];
  root_4 += 360*c1p1*c3p7*c9p2*ss[40];
  root_4 += 90*c1p1*c3p8*c5p1*ss[30];
  root_4 += 90*c1p1*c3p8*c7p1*ss[32];
  root_4 += 90*c1p1*c3p8*c9p1*ss[34];
  root_4 += 90*c1p1*c5p1*c7p8*ss[62];
  root_4 += 90*c1p1*c5p1*c9p8*ss[78];
  root_4 += 360*c1p1*c5p2*c7p7*ss[60];
  root_4 += 360*c1p1*c5p2*c9p7*ss[74];
  root_4 += 840*c1p1*c5p3*c7p6*ss[58];
  root_4 += 840*c1p1*c5p3*c9p6*ss[70];
  root_4 += 1260*c1p1*c5p4*c7p5*ss[56];
  root_4 += 1260*c1p1*c5p4*c9p5*ss[66];
  root_4 += 1260*c1p1*c5p5*c7p4*ss[54];
  root_4 += 1260*c1p1*c5p5*c9p4*ss[62];
  root_4 += 840*c1p1*c5p6*c7p3*ss[52];
  root_4 += 840*c1p1*c5p6*c9p3*ss[58];
  root_4 += 360*c1p1*c5p7*c7p2*ss[50];
  root_4 += 360*c1p1*c5p7*c9p2*ss[54];
  root_4 += 90*c1p1*c5p8*c7p1*ss[48];
  root_4 += 90*c1p1*c5p8*c9p1*ss[50];
  root_4 += 90*c1p1*c7p1*c9p8*ss[80];
  root_4 += 360*c1p1*c7p2*c9p7*ss[78];
  root_4 += 840*c1p1*c7p3*c9p6*ss[76];
  root_4 += 1260*c1p1*c7p4*c9p5*ss[74];
  root_4 += 1260*c1p1*c7p5*c9p4*ss[72];
  root_4 += 840*c1p1*c7p6*c9p3*ss[70];
  root_4 += 360*c1p1*c7p7*c9p2*ss[68];
  root_4 += 90*c1p1*c7p8*c9p1*ss[66];
  root_4 += 360*c1p2*c3p1*c5p7*ss[40];
  root_4 += 360*c1p2*c3p1*c7p7*ss[54];
  root_4 += 360*c1p2*c3p1*c9p7*ss[68];
  root_4 += 1260*c1p2*c3p2*c5p6*ss[38];
  root_4 += 1260*c1p2*c3p2*c7p6*ss[50];
  root_4 += 1260*c1p2*c3p2*c9p6*ss[62];
  root_4 += 2520*c1p2*c3p3*c5p5*ss[36];
  root_4 += 2520*c1p2*c3p3*c7p5*ss[46];
  root_4 += 2520*c1p2*c3p3*c9p5*ss[56];
  root_4 += 3150*c1p2*c3p4*c5p4*ss[34];
  root_4 += 3150*c1p2*c3p4*c7p4*ss[42];
  root_4 += 3150*c1p2*c3p4*c9p4*ss[50];
  root_4 += 2520*c1p2*c3p5*c5p3*ss[32];
  root_4 += 2520*c1p2*c3p5*c7p3*ss[38];
  root_4 += 2520*c1p2*c3p5*c9p3*ss[44];
  root_4 += 1260*c1p2*c3p6*c5p2*ss[30];
  root_4 += 1260*c1p2*c3p6*c7p2*ss[34];
  root_4 += 1260*c1p2*c3p6*c9p2*ss[38];
  root_4 += 360*c1p2*c3p7*c5p1*ss[28];
  root_4 += 360*c1p2*c3p7*c7p1*ss[30];
  root_4 += 360*c1p2*c3p7*c9p1*ss[32];
  root_4 += 360*c1p2*c5p1*c7p7*ss[56];
  root_4 += 360*c1p2*c5p1*c9p7*ss[70];
  root_4 += 1260*c1p2*c5p2*c7p6*ss[54];
  root_4 += 1260*c1p2*c5p2*c9p6*ss[66];
  root_4 += 2520*c1p2*c5p3*c7p5*ss[52];
  root_4 += 2520*c1p2*c5p3*c9p5*ss[62];
  root_4 += 3150*c1p2*c5p4*c7p4*ss[50];
  root_4 += 3150*c1p2*c5p4*c9p4*ss[58];
  root_4 += 2520*c1p2*c5p5*c7p3*ss[48];
  root_4 += 2520*c1p2*c5p5*c9p3*ss[54];
  root_4 += 1260*c1p2*c5p6*c7p2*ss[46];
  root_4 += 1260*c1p2*c5p6*c9p2*ss[50];
  root_4 += 360*c1p2*c5p7*c7p1*ss[44];
  root_4 += 360*c1p2*c5p7*c9p1*ss[46];
  root_4 += 360*c1p2*c7p1*c9p7*ss[72];
  root_4 += 1260*c1p2*c7p2*c9p6*ss[70];
  root_4 += 2520*c1p2*c7p3*c9p5*ss[68];
  root_4 += 3150*c1p2*c7p4*c9p4*ss[66];
  root_4 += 2520*c1p2*c7p5*c9p3*ss[64];
  root_4 += 1260*c1p2*c7p6*c9p2*ss[62];
  root_4 += 360*c1p2*c7p7*c9p1*ss[60];
  root_4 += 840*c1p3*c3p1*c5p6*ss[36];
  root_4 += 840*c1p3*c3p1*c7p6*ss[48];
  root_4 += 840*c1p3*c3p1*c9p6*ss[60];
  root_4 += 2520*c1p3*c3p2*c5p5*ss[34];
  root_4 += 2520*c1p3*c3p2*c7p5*ss[44];
  root_4 += 2520*c1p3*c3p2*c9p5*ss[54];
  root_4 += 4200*c1p3*c3p3*c5p4*ss[32];
  root_4 += 4200*c1p3*c3p3*c7p4*ss[40];
  root_4 += 4200*c1p3*c3p3*c9p4*ss[48];
  root_4 += 4200*c1p3*c3p4*c5p3*ss[30];
  root_4 += 4200*c1p3*c3p4*c7p3*ss[36];
  root_4 += 4200*c1p3*c3p4*c9p3*ss[42];
  root_4 += 2520*c1p3*c3p5*c5p2*ss[28];
  root_4 += 2520*c1p3*c3p5*c7p2*ss[32];
  root_4 += 2520*c1p3*c3p5*c9p2*ss[36];
  root_4 += 840*c1p3*c3p6*c5p1*ss[26];
  root_4 += 840*c1p3*c3p6*c7p1*ss[28];
  root_4 += 840*c1p3*c3p6*c9p1*ss[30];
  root_4 += 840*c1p3*c5p1*c7p6*ss[50];
  root_4 += 840*c1p3*c5p1*c9p6*ss[62];
  root_4 += 2520*c1p3*c5p2*c7p5*ss[48];
  root_4 += 2520*c1p3*c5p2*c9p5*ss[58];
  root_4 += 4200*c1p3*c5p3*c7p4*ss[46];
  root_4 += 4200*c1p3*c5p3*c9p4*ss[54];
  root_4 += 4200*c1p3*c5p4*c7p3*ss[44];
  root_4 += 4200*c1p3*c5p4*c9p3*ss[50];
  root_4 += 2520*c1p3*c5p5*c7p2*ss[42];
  root_4 += 2520*c1p3*c5p5*c9p2*ss[46];
  root_4 += 840*c1p3*c5p6*c7p1*ss[40];
  root_4 += 840*c1p3*c5p6*c9p1*ss[42];
  root_4 += 840*c1p3*c7p1*c9p6*ss[64];
  root_4 += 2520*c1p3*c7p2*c9p5*ss[62];
  root_4 += 4200*c1p3*c7p3*c9p4*ss[60];
  root_4 += 4200*c1p3*c7p4*c9p3*ss[58];
  root_4 += 2520*c1p3*c7p5*c9p2*ss[56];
  root_4 += 840*c1p3*c7p6*c9p1*ss[54];
  root_4 += 1260*c1p4*c3p1*c5p5*ss[32];
  root_4 += 1260*c1p4*c3p1*c7p5*ss[42];
  root_4 += 1260*c1p4*c3p1*c9p5*ss[52];
  root_4 += 3150*c1p4*c3p2*c5p4*ss[30];
  root_4 += 3150*c1p4*c3p2*c7p4*ss[38];
  root_4 += 3150*c1p4*c3p2*c9p4*ss[46];
  root_4 += 4200*c1p4*c3p3*c5p3*ss[28];
  root_4 += 4200*c1p4*c3p3*c7p3*ss[34];
  root_4 += 4200*c1p4*c3p3*c9p3*ss[40];
  root_4 += 3150*c1p4*c3p4*c5p2*ss[26];
  root_4 += 3150*c1p4*c3p4*c7p2*ss[30];
  root_4 += 3150*c1p4*c3p4*c9p2*ss[34];
  root_4 += 1260*c1p4*c3p5*c5p1*ss[24];
  root_4 += 1260*c1p4*c3p5*c7p1*ss[26];
  root_4 += 1260*c1p4*c3p5*c9p1*ss[28];
  root_4 += 1260*c1p4*c5p1*c7p5*ss[44];
  root_4 += 1260*c1p4*c5p1*c9p5*ss[54];
  root_4 += 3150*c1p4*c5p2*c7p4*ss[42];
  root_4 += 3150*c1p4*c5p2*c9p4*ss[50];
  root_4 += 4200*c1p4*c5p3*c7p3*ss[40];
  root_4 += 4200*c1p4*c5p3*c9p3*ss[46];
  root_4 += 3150*c1p4*c5p4*c7p2*ss[38];
  root_4 += 3150*c1p4*c5p4*c9p2*ss[42];
  root_4 += 1260*c1p4*c5p5*c7p1*ss[36];
  root_4 += 1260*c1p4*c5p5*c9p1*ss[38];
  root_4 += 1260*c1p4*c7p1*c9p5*ss[56];
  root_4 += 3150*c1p4*c7p2*c9p4*ss[54];
  root_4 += 4200*c1p4*c7p3*c9p3*ss[52];
  root_4 += 3150*c1p4*c7p4*c9p2*ss[50];
  root_4 += 1260*c1p4*c7p5*c9p1*ss[48];
  root_4 += 1260*c1p5*c3p1*c5p4*ss[28];
  root_4 += 1260*c1p5*c3p1*c7p4*ss[36];
  root_4 += 1260*c1p5*c3p1*c9p4*ss[44];
  root_4 += 2520*c1p5*c3p2*c5p3*ss[26];
  root_4 += 2520*c1p5*c3p2*c7p3*ss[32];
  root_4 += 2520*c1p5*c3p2*c9p3*ss[38];
  root_4 += 2520*c1p5*c3p3*c5p2*ss[24];
  root_4 += 2520*c1p5*c3p3*c7p2*ss[28];
  root_4 += 2520*c1p5*c3p3*c9p2*ss[32];
  root_4 += 1260*c1p5*c3p4*c5p1*ss[22];
  root_4 += 1260*c1p5*c3p4*c7p1*ss[24];
  root_4 += 1260*c1p5*c3p4*c9p1*ss[26];
  root_4 += 1260*c1p5*c5p1*c7p4*ss[38];
  root_4 += 1260*c1p5*c5p1*c9p4*ss[46];
  root_4 += 2520*c1p5*c5p2*c7p3*ss[36];
  root_4 += 2520*c1p5*c5p2*c9p3*ss[42];
  root_4 += 2520*c1p5*c5p3*c7p2*ss[34];
  root_4 += 2520*c1p5*c5p3*c9p2*ss[38];
  root_4 += 1260*c1p5*c5p4*c7p1*ss[32];
  root_4 += 1260*c1p5*c5p4*c9p1*ss[34];
  root_4 += 1260*c1p5*c7p1*c9p4*ss[48];
  root_4 += 2520*c1p5*c7p2*c9p3*ss[46];
  root_4 += 2520*c1p5*c7p3*c9p2*ss[44];
  root_4 += 1260*c1p5*c7p4*c9p1*ss[42];
  root_4 += 840*c1p6*c3p1*c5p3*ss[24];
  root_4 += 840*c1p6*c3p1*c7p3*ss[30];
  root_4 += 840*c1p6*c3p1*c9p3*ss[36];
  root_4 += 1260*c1p6*c3p2*c5p2*ss[22];
  root_4 += 1260*c1p6*c3p2*c7p2*ss[26];
  root_4 += 1260*c1p6*c3p2*c9p2*ss[30];
  root_4 += 840*c1p6*c3p3*c5p1*ss[20];
  root_4 += 840*c1p6*c3p3*c7p1*ss[22];
  root_4 += 840*c1p6*c3p3*c9p1*ss[24];
  root_4 += 840*c1p6*c5p1*c7p3*ss[32];
  root_4 += 840*c1p6*c5p1*c9p3*ss[38];
  root_4 += 1260*c1p6*c5p2*c7p2*ss[30];
  root_4 += 1260*c1p6*c5p2*c9p2*ss[34];
  root_4 += 840*c1p6*c5p3*c7p1*ss[28];
  root_4 += 840*c1p6*c5p3*c9p1*ss[30];
  root_4 += 840*c1p6*c7p1*c9p3*ss[40];
  root_4 += 1260*c1p6*c7p2*c9p2*ss[38];
  root_4 += 840*c1p6*c7p3*c9p1*ss[36];
  root_4 += 360*c1p7*c3p1*c5p2*ss[20];
  root_4 += 360*c1p7*c3p1*c7p2*ss[24];
  root_4 += 360*c1p7*c3p1*c9p2*ss[28];
  root_4 += 360*c1p7*c3p2*c5p1*ss[18];
  root_4 += 360*c1p7*c3p2*c7p1*ss[20];
  root_4 += 360*c1p7*c3p2*c9p1*ss[22];
  root_4 += 360*c1p7*c5p1*c7p2*ss[26];
  root_4 += 360*c1p7*c5p1*c9p2*ss[30];
  root_4 += 360*c1p7*c5p2*c7p1*ss[24];
  root_4 += 360*c1p7*c5p2*c9p1*ss[26];
  root_4 += 360*c1p7*c7p1*c9p2*ss[32];
  root_4 += 360*c1p7*c7p2*c9p1*ss[30];
  root_4 += 90*c1p8*c3p1*c5p1*ss[16];
  root_4 += 90*c1p8*c3p1*c7p1*ss[18];
  root_4 += 90*c1p8*c3p1*c9p1*ss[20];
  root_4 += 90*c1p8*c5p1*c7p1*ss[20];
  root_4 += 90*c1p8*c5p1*c9p1*ss[22];
  root_4 += 90*c1p8*c7p1*c9p1*ss[24];
  root_4 += 90*c3p1*c5p1*c7p8*ss[64];
  root_4 += 90*c3p1*c5p1*c9p8*ss[80];
  root_4 += 360*c3p1*c5p2*c7p7*ss[62];
  root_4 += 360*c3p1*c5p2*c9p7*ss[76];
  root_4 += 840*c3p1*c5p3*c7p6*ss[60];
  root_4 += 840*c3p1*c5p3*c9p6*ss[72];
  root_4 += 1260*c3p1*c5p4*c7p5*ss[58];
  root_4 += 1260*c3p1*c5p4*c9p5*ss[68];
  root_4 += 1260*c3p1*c5p5*c7p4*ss[56];
  root_4 += 1260*c3p1*c5p5*c9p4*ss[64];
  root_4 += 840*c3p1*c5p6*c7p3*ss[54];
  root_4 += 840*c3p1*c5p6*c9p3*ss[60];
  root_4 += 360*c3p1*c5p7*c7p2*ss[52];
  root_4 += 360*c3p1*c5p7*c9p2*ss[56];
  root_4 += 90*c3p1*c5p8*c7p1*ss[50];
  root_4 += 90*c3p1*c5p8*c9p1*ss[52];
  root_4 += 90*c3p1*c7p1*c9p8*ss[82];
  root_4 += 360*c3p1*c7p2*c9p7*ss[80];
  root_4 += 840*c3p1*c7p3*c9p6*ss[78];
  root_4 += 1260*c3p1*c7p4*c9p5*ss[76];
  root_4 += 1260*c3p1*c7p5*c9p4*ss[74];
  root_4 += 840*c3p1*c7p6*c9p3*ss[72];
  root_4 += 360*c3p1*c7p7*c9p2*ss[70];
  root_4 += 90*c3p1*c7p8*c9p1*ss[68];
  root_4 += 360*c3p2*c5p1*c7p7*ss[60];
  root_4 += 360*c3p2*c5p1*c9p7*ss[74];
  root_4 += 1260*c3p2*c5p2*c7p6*ss[58];
  root_4 += 1260*c3p2*c5p2*c9p6*ss[70];
  root_4 += 2520*c3p2*c5p3*c7p5*ss[56];
  root_4 += 2520*c3p2*c5p3*c9p5*ss[66];
  root_4 += 3150*c3p2*c5p4*c7p4*ss[54];
  root_4 += 3150*c3p2*c5p4*c9p4*ss[62];
  root_4 += 2520*c3p2*c5p5*c7p3*ss[52];
  root_4 += 2520*c3p2*c5p5*c9p3*ss[58];
  root_4 += 1260*c3p2*c5p6*c7p2*ss[50];
  root_4 += 1260*c3p2*c5p6*c9p2*ss[54];
  root_4 += 360*c3p2*c5p7*c7p1*ss[48];
  root_4 += 360*c3p2*c5p7*c9p1*ss[50];
  root_4 += 360*c3p2*c7p1*c9p7*ss[76];
  root_4 += 1260*c3p2*c7p2*c9p6*ss[74];
  root_4 += 2520*c3p2*c7p3*c9p5*ss[72];
  root_4 += 3150*c3p2*c7p4*c9p4*ss[70];
  root_4 += 2520*c3p2*c7p5*c9p3*ss[68];
  root_4 += 1260*c3p2*c7p6*c9p2*ss[66];
  root_4 += 360*c3p2*c7p7*c9p1*ss[64];
  root_4 += 840*c3p3*c5p1*c7p6*ss[56];
  root_4 += 840*c3p3*c5p1*c9p6*ss[68];
  root_4 += 2520*c3p3*c5p2*c7p5*ss[54];
  root_4 += 2520*c3p3*c5p2*c9p5*ss[64];
  root_4 += 4200*c3p3*c5p3*c7p4*ss[52];
  root_4 += 4200*c3p3*c5p3*c9p4*ss[60];
  root_4 += 4200*c3p3*c5p4*c7p3*ss[50];
  root_4 += 4200*c3p3*c5p4*c9p3*ss[56];
  root_4 += 2520*c3p3*c5p5*c7p2*ss[48];
  root_4 += 2520*c3p3*c5p5*c9p2*ss[52];
  root_4 += 840*c3p3*c5p6*c7p1*ss[46];
  root_4 += 840*c3p3*c5p6*c9p1*ss[48];
  root_4 += 840*c3p3*c7p1*c9p6*ss[70];
  root_4 += 2520*c3p3*c7p2*c9p5*ss[68];
  root_4 += 4200*c3p3*c7p3*c9p4*ss[66];
  root_4 += 4200*c3p3*c7p4*c9p3*ss[64];
  root_4 += 2520*c3p3*c7p5*c9p2*ss[62];
  root_4 += 840*c3p3*c7p6*c9p1*ss[60];
  root_4 += 1260*c3p4*c5p1*c7p5*ss[52];
  root_4 += 1260*c3p4*c5p1*c9p5*ss[62];
  root_4 += 3150*c3p4*c5p2*c7p4*ss[50];
  root_4 += 3150*c3p4*c5p2*c9p4*ss[58];
  root_4 += 4200*c3p4*c5p3*c7p3*ss[48];
  root_4 += 4200*c3p4*c5p3*c9p3*ss[54];
  root_4 += 3150*c3p4*c5p4*c7p2*ss[46];
  root_4 += 3150*c3p4*c5p4*c9p2*ss[50];
  root_4 += 1260*c3p4*c5p5*c7p1*ss[44];
  root_4 += 1260*c3p4*c5p5*c9p1*ss[46];
  root_4 += 1260*c3p4*c7p1*c9p5*ss[64];
  root_4 += 3150*c3p4*c7p2*c9p4*ss[62];
  root_4 += 4200*c3p4*c7p3*c9p3*ss[60];
  root_4 += 3150*c3p4*c7p4*c9p2*ss[58];
  root_4 += 1260*c3p4*c7p5*c9p1*ss[56];
  root_4 += 1260*c3p5*c5p1*c7p4*ss[48];
  root_4 += 1260*c3p5*c5p1*c9p4*ss[56];
  root_4 += 2520*c3p5*c5p2*c7p3*ss[46];
  root_4 += 2520*c3p5*c5p2*c9p3*ss[52];
  root_4 += 2520*c3p5*c5p3*c7p2*ss[44];
  root_4 += 2520*c3p5*c5p3*c9p2*ss[48];
  root_4 += 1260*c3p5*c5p4*c7p1*ss[42];
  root_4 += 1260*c3p5*c5p4*c9p1*ss[44];
  root_4 += 1260*c3p5*c7p1*c9p4*ss[58];
  root_4 += 2520*c3p5*c7p2*c9p3*ss[56];
  root_4 += 2520*c3p5*c7p3*c9p2*ss[54];
  root_4 += 1260*c3p5*c7p4*c9p1*ss[52];
  root_4 += 840*c3p6*c5p1*c7p3*ss[44];
  root_4 += 840*c3p6*c5p1*c9p3*ss[50];
  root_4 += 1260*c3p6*c5p2*c7p2*ss[42];
  root_4 += 1260*c3p6*c5p2*c9p2*ss[46];
  root_4 += 840*c3p6*c5p3*c7p1*ss[40];
  root_4 += 840*c3p6*c5p3*c9p1*ss[42];
  root_4 += 840*c3p6*c7p1*c9p3*ss[52];
  root_4 += 1260*c3p6*c7p2*c9p2*ss[50];
  root_4 += 840*c3p6*c7p3*c9p1*ss[48];
  root_4 += 360*c3p7*c5p1*c7p2*ss[40];
  root_4 += 360*c3p7*c5p1*c9p2*ss[44];
  root_4 += 360*c3p7*c5p2*c7p1*ss[38];
  root_4 += 360*c3p7*c5p2*c9p1*ss[40];
  root_4 += 360*c3p7*c7p1*c9p2*ss[46];
  root_4 += 360*c3p7*c7p2*c9p1*ss[44];
  root_4 += 90*c3p8*c5p1*c7p1*ss[36];
  root_4 += 90*c3p8*c5p1*c9p1*ss[38];
  root_4 += 90*c3p8*c7p1*c9p1*ss[40];
  root_4 += 90*c5p1*c7p1*c9p8*ss[84];
  root_4 += 360*c5p1*c7p2*c9p7*ss[82];
  root_4 += 840*c5p1*c7p3*c9p6*ss[80];
  root_4 += 1260*c5p1*c7p4*c9p5*ss[78];
  root_4 += 1260*c5p1*c7p5*c9p4*ss[76];
  root_4 += 840*c5p1*c7p6*c9p3*ss[74];
  root_4 += 360*c5p1*c7p7*c9p2*ss[72];
  root_4 += 90*c5p1*c7p8*c9p1*ss[70];
  root_4 += 360*c5p2*c7p1*c9p7*ss[80];
  root_4 += 1260*c5p2*c7p2*c9p6*ss[78];
  root_4 += 2520*c5p2*c7p3*c9p5*ss[76];
  root_4 += 3150*c5p2*c7p4*c9p4*ss[74];
  root_4 += 2520*c5p2*c7p5*c9p3*ss[72];
  root_4 += 1260*c5p2*c7p6*c9p2*ss[70];
  root_4 += 360*c5p2*c7p7*c9p1*ss[68];
  root_4 += 840*c5p3*c7p1*c9p6*ss[76];
  root_4 += 2520*c5p3*c7p2*c9p5*ss[74];
  root_4 += 4200*c5p3*c7p3*c9p4*ss[72];
  root_4 += 4200*c5p3*c7p4*c9p3*ss[70];
  root_4 += 2520*c5p3*c7p5*c9p2*ss[68];
  root_4 += 840*c5p3*c7p6*c9p1*ss[66];
  root_4 += 1260*c5p4*c7p1*c9p5*ss[72];
  root_4 += 3150*c5p4*c7p2*c9p4*ss[70];
  root_4 += 4200*c5p4*c7p3*c9p3*ss[68];
  root_4 += 3150*c5p4*c7p4*c9p2*ss[66];
  root_4 += 1260*c5p4*c7p5*c9p1*ss[64];
  root_4 += 1260*c5p5*c7p1*c9p4*ss[68];
  root_4 += 2520*c5p5*c7p2*c9p3*ss[66];
  root_4 += 2520*c5p5*c7p3*c9p2*ss[64];
  root_4 += 1260*c5p5*c7p4*c9p1*ss[62];
  root_4 += 840*c5p6*c7p1*c9p3*ss[64];
  root_4 += 1260*c5p6*c7p2*c9p2*ss[62];
  root_4 += 840*c5p6*c7p3*c9p1*ss[60];
  root_4 += 360*c5p7*c7p1*c9p2*ss[60];
  root_4 += 360*c5p7*c7p2*c9p1*ss[58];
  root_4 += 90*c5p8*c7p1*c9p1*ss[56];
  root_4 += 10*c1p1*c3p9*ss[28];
  root_4 += 10*c1p1*c5p9*ss[46];
  root_4 += 10*c1p1*c7p9*ss[64];
  root_4 += 10*c1p1*c9p9*ss[82];
  root_4 += 45*c1p2*c3p8*ss[26];
  root_4 += 45*c1p2*c5p8*ss[42];
  root_4 += 45*c1p2*c7p8*ss[58];
  root_4 += 45*c1p2*c9p8*ss[74];
  root_4 += 120*c1p3*c3p7*ss[24];
  root_4 += 120*c1p3*c5p7*ss[38];
  root_4 += 120*c1p3*c7p7*ss[52];
  root_4 += 120*c1p3*c9p7*ss[66];
  root_4 += 210*c1p4*c3p6*ss[22];
  root_4 += 210*c1p4*c5p6*ss[34];
  root_4 += 210*c1p4*c7p6*ss[46];
  root_4 += 210*c1p4*c9p6*ss[58];
  root_4 += 252*c1p5*c3p5*ss[20];
  root_4 += 252*c1p5*c5p5*ss[30];
  root_4 += 252*c1p5*c7p5*ss[40];
  root_4 += 252*c1p5*c9p5*ss[50];
  root_4 += 210*c1p6*c3p4*ss[18];
  root_4 += 210*c1p6*c5p4*ss[26];
  root_4 += 210*c1p6*c7p4*ss[34];
  root_4 += 210*c1p6*c9p4*ss[42];
  root_4 += 120*c1p7*c3p3*ss[16];
  root_4 += 120*c1p7*c5p3*ss[22];
  root_4 += 120*c1p7*c7p3*ss[28];
  root_4 += 120*c1p7*c9p3*ss[34];
  root_4 += 45*c1p8*c3p2*ss[14];
  root_4 += 45*c1p8*c5p2*ss[18];
  root_4 += 45*c1p8*c7p2*ss[22];
  root_4 += 45*c1p8*c9p2*ss[26];
  root_4 += 10*c1p9*c3p1*ss[12];
  root_4 += 10*c1p9*c5p1*ss[14];
  root_4 += 10*c1p9*c7p1*ss[16];
  root_4 += 10*c1p9*c9p1*ss[18];
  root_4 += 10*c3p1*c5p9*ss[48];
  root_4 += 10*c3p1*c7p9*ss[66];
  root_4 += 10*c3p1*c9p9*ss[84];
  root_4 += 45*c3p2*c5p8*ss[46];
  root_4 += 45*c3p2*c7p8*ss[62];
  root_4 += 45*c3p2*c9p8*ss[78];
  root_4 += 120*c3p3*c5p7*ss[44];
  root_4 += 120*c3p3*c7p7*ss[58];
  root_4 += 120*c3p3*c9p7*ss[72];
  root_4 += 210*c3p4*c5p6*ss[42];
  root_4 += 210*c3p4*c7p6*ss[54];
  root_4 += 210*c3p4*c9p6*ss[66];
  root_4 += 252*c3p5*c5p5*ss[40];
  root_4 += 252*c3p5*c7p5*ss[50];
  root_4 += 252*c3p5*c9p5*ss[60];
  root_4 += 210*c3p6*c5p4*ss[38];
  root_4 += 210*c3p6*c7p4*ss[46];
  root_4 += 210*c3p6*c9p4*ss[54];
  root_4 += 120*c3p7*c5p3*ss[36];
  root_4 += 120*c3p7*c7p3*ss[42];
  root_4 += 120*c3p7*c9p3*ss[48];
  root_4 += 45*c3p8*c5p2*ss[34];
  root_4 += 45*c3p8*c7p2*ss[38];
  root_4 += 45*c3p8*c9p2*ss[42];
  root_4 += 10*c3p9*c5p1*ss[32];
  root_4 += 10*c3p9*c7p1*ss[34];
  root_4 += 10*c3p9*c9p1*ss[36];
  root_4 += 10*c5p1*c7p9*ss[68];
  root_4 += 10*c5p1*c9p9*ss[86];
  root_4 += 45*c5p2*c7p8*ss[66];
  root_4 += 45*c5p2*c9p8*ss[82];
  root_4 += 120*c5p3*c7p7*ss[64];
  root_4 += 120*c5p3*c9p7*ss[78];
  root_4 += 210*c5p4*c7p6*ss[62];
  root_4 += 210*c5p4*c9p6*ss[74];
  root_4 += 252*c5p5*c7p5*ss[60];
  root_4 += 252*c5p5*c9p5*ss[70];
  root_4 += 210*c5p6*c7p4*ss[58];
  root_4 += 210*c5p6*c9p4*ss[66];
  root_4 += 120*c5p7*c7p3*ss[56];
  root_4 += 120*c5p7*c9p3*ss[62];
  root_4 += 45*c5p8*c7p2*ss[54];
  root_4 += 45*c5p8*c9p2*ss[58];
  root_4 += 10*c5p9*c7p1*ss[52];
  root_4 += 10*c5p9*c9p1*ss[54];
  root_4 += 10*c7p1*c9p9*ss[88];
  root_4 += 45*c7p2*c9p8*ss[86];
  root_4 += 120*c7p3*c9p7*ss[84];
  root_4 += 210*c7p4*c9p6*ss[82];
  root_4 += 252*c7p5*c9p5*ss[80];
  root_4 += 210*c7p6*c9p4*ss[78];
  root_4 += 120*c7p7*c9p3*ss[76];
  root_4 += 45*c7p8*c9p2*ss[74];
  root_4 += 10*c7p9*c9p1*ss[72];
  root_4 += c1p10*ss[10];
  root_4 += c3p10*ss[30];
  root_4 += c5p10*ss[50];
  root_4 += c7p10*ss[70];
  root_4 += c9p10*ss[90];
  root_4 += -tmom[10];
  return convert<T>(sqrt(root_0*root_0+root_1*root_1+root_2*root_2+root_3*root_3+root_4*root_4));
}
T metric_and_gradient(const T *parameters, T *gradient) const {
  T c1p1 = parameters[0];
  T c3p1 = parameters[1];
  T c5p1 = parameters[2];
  T c7p1 = parameters[3];
  T c9p1 = parameters[4];
  T c1p2 = c1p1*c1p1;
  T c3p2 = c3p1*c3p1;
  T c5p2 = c5p1*c5p1;
  T c7p2 = c7p1*c7p1;
  T c9p2 = c9p1*c9p1;
  TACC sys_part_0 = convert<TACC>(2*c1p1*c3p1*ss[4]);
  sys_part_0 += 2*c1p1*c5p1*ss[6];
  sys_part_0 += 2*c1p1*c7p1*ss[8];
  sys_part_0 += 2*c1p1*c9p1*ss[10];
  sys_part_0 += 2*c3p1*c5p1*ss[8];
  sys_part_0 += 2*c3p1*c7p1*ss[10];
  sys_part_0 += 2*c3p1*c9p1*ss[12];
  sys_part_0 += 2*c5p1*c7p1*ss[12];
  sys_part_0 += 2*c5p1*c9p1*ss[14];
  sys_part_0 += 2*c7p1*c9p1*ss[16];
  sys_part_0 += c1p2*ss[2];
  sys_part_0 += c3p2*ss[6];
  sys_part_0 += c5p2*ss[10];
  sys_part_0 += c7p2*ss[14];
  sys_part_0 += c9p2*ss[18];
  sys_part_0 += -tmom[2];
  T c1p3 = c1p2*c1p1;
  T c1p4 = c1p3*c1p1;
  T c3p3 = c3p2*c3p1;
  T c3p4 = c3p3*c3p1;
  T c5p3 = c5p2*c5p1;
  T c5p4 = c5p3*c5p1;
  T c7p3 = c7p2*c7p1;
  T c7p4 = c7p3*c7p1;
  T c9p3 = c9p2*c9p1;
  T c9p4 = c9p3*c9p1;
  TACC sys_part_1 = convert<TACC>(24*c1p1*c3p1*c5p1*c7p1*ss[16]);
  sys_part_1 += 24*c1p1*c3p1*c5p1*c9p1*ss[18];
  sys_part_1 += 24*c1p1*c3p1*c7p1*c9p1*ss[20];
  sys_part_1 += 24*c1p1*c5p1*c7p1*c9p1*ss[22];
  sys_part_1 += 24*c3p1*c5p1*c7p1*c9p1*ss[24];
  sys_part_1 += 12*c1p1*c3p1*c5p2*ss[14];
  sys_part_1 += 12*c1p1*c3p1*c7p2*ss[18];
  sys_part_1 += 12*c1p1*c3p1*c9p2*ss[22];
  sys_part_1 += 12*c1p1*c3p2*c5p1*ss[12];
  sys_part_1 += 12*c1p1*c3p2*c7p1*ss[14];
  sys_part_1 += 12*c1p1*c3p2*c9p1*ss[16];
  sys_part_1 += 12*c1p1*c5p1*c7p2*ss[20];
  sys_part_1 += 12*c1p1*c5p1*c9p2*ss[24];
  sys_part_1 += 12*c1p1*c5p2*c7p1*ss[18];
  sys_part_1 += 12*c1p1*c5p2*c9p1*ss[20];
  sys_part_1 += 12*c1p1*c7p1*c9p2*ss[26];
  sys_part_1 += 12*c1p1*c7p2*c9p1*ss[24];
  sys_part_1 += 12*c1p2*c3p1*c5p1*ss[10];
  sys_part_1 += 12*c1p2*c3p1*c7p1*ss[12];
  sys_part_1 += 12*c1p2*c3p1*c9p1*ss[14];
  sys_part_1 += 12*c1p2*c5p1*c7p1*ss[14];
  sys_part_1 += 12*c1p2*c5p1*c9p1*ss[16];
  sys_part_1 += 12*c1p2*c7p1*c9p1*ss[18];
  sys_part_1 += 12*c3p1*c5p1*c7p2*ss[22];
  sys_part_1 += 12*c3p1*c5p1*c9p2*ss[26];
  sys_part_1 += 12*c3p1*c5p2*c7p1*ss[20];
  sys_part_1 += 12*c3p1*c5p2*c9p1*ss[22];
  sys_part_1 += 12*c3p1*c7p1*c9p2*ss[28];
  sys_part_1 += 12*c3p1*c7p2*c9p1*ss[26];
  sys_part_1 += 12*c3p2*c5p1*c7p1*ss[18];
  sys_part_1 += 12*c3p2*c5p1*c9p1*ss[20];
  sys_part_1 += 12*c3p2*c7p1*c9p1*ss[22];
  sys_part_1 += 12*c5p1*c7p1*c9p2*ss[30];
  sys_part_1 += 12*c5p1*c7p2*c9p1*ss[28];
  sys_part_1 += 12*c5p2*c7p1*c9p1*ss[26];
  sys_part_1 += 4*c1p1*c3p3*ss[10];
  sys_part_1 += 4*c1p1*c5p3*ss[16];
  sys_part_1 += 4*c1p1*c7p3*ss[22];
  sys_part_1 += 4*c1p1*c9p3*ss[28];
  sys_part_1 += 6*c1p2*c3p2*ss[8];
  sys_part_1 += 6*c1p2*c5p2*ss[12];
  sys_part_1 += 6*c1p2*c7p2*ss[16];
  sys_part_1 += 6*c1p2*c9p2*ss[20];
  sys_part_1 += 4*c1p3*c3p1*ss[6];
  sys_part_1 += 4*c1p3*c5p1*ss[8];
  sys_part_1 += 4*c1p3*c7p1*ss[10];
  sys_part_1 += 4*c1p3*c9p1*ss[12];
  sys_part_1 += 4*c3p1*c5p3*ss[18];
  sys_part_1 += 4*c3p1*c7p3*ss[24];
  sys_part_1 += 4*c3p1*c9p3*ss[30];
  sys_part_1 += 6*c3p2*c5p2*ss[16];
  sys_part_1 += 6*c3p2*c7p2*ss[20];
  sys_part_1 += 6*c3p2*c9p2*ss[24];
  sys_part_1 += 4*c3p3*c5p1*ss[14];
  sys_part_1 += 4*c3p3*c7p1*ss[16];
  sys_part_1 += 4*c3p3*c9p1*ss[18];
  sys_part_1 += 4*c5p1*c7p3*ss[26];
  sys_part_1 += 4*c5p1*c9p3*ss[32];
  sys_part_1 += 6*c5p2*c7p2*ss[24];
  sys_part_1 += 6*c5p2*c9p2*ss[28];
  sys_part_1 += 4*c5p3*c7p1*ss[22];
  sys_part_1 += 4*c5p3*c9p1*ss[24];
  sys_part_1 += 4*c7p1*c9p3*ss[34];
  sys_part_1 += 6*c7p2*c9p2*ss[32];
  sys_part_1 += 4*c7p3*c9p1*ss[30];
  sys_part_1 += c1p4*ss[4];
  sys_part_1 += c3p4*ss[12];
  sys_part_1 += c5p4*ss[20];
  sys_part_1 += c7p4*ss[28];
  sys_part_1 += c9p4*ss[36];
  sys_part_1 += -tmom[4];
  T c1p5 = c1p4*c1p1;
  T c1p6 = c1p5*c1p1;
  T c3p5 = c3p4*c3p1;
  T c3p6 = c3p5*c3p1;
  T c5p5 = c5p4*c5p1;
  T c5p6 = c5p5*c5p1;
  T c7p5 = c7p4*c7p1;
  T c7p6 = c7p5*c7p1;
  T c9p5 = c9p4*c9p1;
  T c9p6 = c9p5*c9p1;
  TACC sys_part_2 = convert<TACC>(360*c1p1*c3p1*c5p1*c7p1*c9p2*ss[34]);
  sys_part_2 += 360*c1p1*c3p1*c5p1*c7p2*c9p1*ss[32];
  sys_part_2 += 360*c1p1*c3p1*c5p2*c7p1*c9p1*ss[30];
  sys_part_2 += 360*c1p1*c3p2*c5p1*c7p1*c9p1*ss[28];
  sys_part_2 += 360*c1p2*c3p1*c5p1*c7p1*c9p1*ss[26];
  sys_part_2 += 120*c1p1*c3p1*c5p1*c7p3*ss[30];
  sys_part_2 += 120*c1p1*c3p1*c5p1*c9p3*ss[36];
  sys_part_2 += 180*c1p1*c3p1*c5p2*c7p2*ss[28];
  sys_part_2 += 180*c1p1*c3p1*c5p2*c9p2*ss[32];
  sys_part_2 += 120*c1p1*c3p1*c5p3*c7p1*ss[26];
  sys_part_2 += 120*c1p1*c3p1*c5p3*c9p1*ss[28];
  sys_part_2 += 120*c1p1*c3p1*c7p1*c9p3*ss[38];
  sys_part_2 += 180*c1p1*c3p1*c7p2*c9p2*ss[36];
  sys_part_2 += 120*c1p1*c3p1*c7p3*c9p1*ss[34];
  sys_part_2 += 180*c1p1*c3p2*c5p1*c7p2*ss[26];
  sys_part_2 += 180*c1p1*c3p2*c5p1*c9p2*ss[30];
  sys_part_2 += 180*c1p1*c3p2*c5p2*c7p1*ss[24];
  sys_part_2 += 180*c1p1*c3p2*c5p2*c9p1*ss[26];
  sys_part_2 += 180*c1p1*c3p2*c7p1*c9p2*ss[32];
  sys_part_2 += 180*c1p1*c3p2*c7p2*c9p1*ss[30];
  sys_part_2 += 120*c1p1*c3p3*c5p1*c7p1*ss[22];
  sys_part_2 += 120*c1p1*c3p3*c5p1*c9p1*ss[24];
  sys_part_2 += 120*c1p1*c3p3*c7p1*c9p1*ss[26];
  sys_part_2 += 120*c1p1*c5p1*c7p1*c9p3*ss[40];
  sys_part_2 += 180*c1p1*c5p1*c7p2*c9p2*ss[38];
  sys_part_2 += 120*c1p1*c5p1*c7p3*c9p1*ss[36];
  sys_part_2 += 180*c1p1*c5p2*c7p1*c9p2*ss[36];
  sys_part_2 += 180*c1p1*c5p2*c7p2*c9p1*ss[34];
  sys_part_2 += 120*c1p1*c5p3*c7p1*c9p1*ss[32];
  sys_part_2 += 180*c1p2*c3p1*c5p1*c7p2*ss[24];
  sys_part_2 += 180*c1p2*c3p1*c5p1*c9p2*ss[28];
  sys_part_2 += 180*c1p2*c3p1*c5p2*c7p1*ss[22];
  sys_part_2 += 180*c1p2*c3p1*c5p2*c9p1*ss[24];
  sys_part_2 += 180*c1p2*c3p1*c7p1*c9p2*ss[30];
  sys_part_2 += 180*c1p2*c3p1*c7p2*c9p1*ss[28];
  sys_part_2 += 180*c1p2*c3p2*c5p1*c7p1*ss[20];
  sys_part_2 += 180*c1p2*c3p2*c5p1*c9p1*ss[22];
  sys_part_2 += 180*c1p2*c3p2*c7p1*c9p1*ss[24];
  sys_part_2 += 180*c1p2*c5p1*c7p1*c9p2*ss[32];
  sys_part_2 += 180*c1p2*c5p1*c7p2*c9p1*ss[30];
  sys_part_2 += 180*c1p2*c5p2*c7p1*c9p1*ss[28];
  sys_part_2 += 120*c1p3*c3p1*c5p1*c7p1*ss[18];
  sys_part_2 += 120*c1p3*c3p1*c5p1*c9p1*ss[20];
  sys_part_2 += 120*c1p3*c3p1*c7p1*c9p1*ss[22];
  sys_part_2 += 120*c1p3*c5p1*c7p1*c9p1*ss[24];
  sys_part_2 += 120*c3p1*c5p1*c7p1*c9p3*ss[42];
  sys_part_2 += 180*c3p1*c5p1*c7p2*c9p2*ss[40];
  sys_part_2 += 120*c3p1*c5p1*c7p3*c9p1*ss[38];
  sys_part_2 += 180*c3p1*c5p2*c7p1*c9p2*ss[38];
  sys_part_2 += 180*c3p1*c5p2*c7p2*c9p1*ss[36];
  sys_part_2 += 120*c3p1*c5p3*c7p1*c9p1*ss[34];
  sys_part_2 += 180*c3p2*c5p1*c7p1*c9p2*ss[36];
  sys_part_2 += 180*c3p2*c5p1*c7p2*c9p1*ss[34];
  sys_part_2 += 180*c3p2*c5p2*c7p1*c9p1*ss[32];
  sys_part_2 += 120*c3p3*c5p1*c7p1*c9p1*ss[30];
  sys_part_2 += 30*c1p1*c3p1*c5p4*ss[24];
  sys_part_2 += 30*c1p1*c3p1*c7p4*ss[32];
  sys_part_2 += 30*c1p1*c3p1*c9p4*ss[40];
  sys_part_2 += 60*c1p1*c3p2*c5p3*ss[22];
  sys_part_2 += 60*c1p1*c3p2*c7p3*ss[28];
  sys_part_2 += 60*c1p1*c3p2*c9p3*ss[34];
  sys_part_2 += 60*c1p1*c3p3*c5p2*ss[20];
  sys_part_2 += 60*c1p1*c3p3*c7p2*ss[24];
  sys_part_2 += 60*c1p1*c3p3*c9p2*ss[28];
  sys_part_2 += 30*c1p1*c3p4*c5p1*ss[18];
  sys_part_2 += 30*c1p1*c3p4*c7p1*ss[20];
  sys_part_2 += 30*c1p1*c3p4*c9p1*ss[22];
  sys_part_2 += 30*c1p1*c5p1*c7p4*ss[34];
  sys_part_2 += 30*c1p1*c5p1*c9p4*ss[42];
  sys_part_2 += 60*c1p1*c5p2*c7p3*ss[32];
  sys_part_2 += 60*c1p1*c5p2*c9p3*ss[38];
  sys_part_2 += 60*c1p1*c5p3*c7p2*ss[30];
  sys_part_2 += 60*c1p1*c5p3*c9p2*ss[34];
  sys_part_2 += 30*c1p1*c5p4*c7p1*ss[28];
  sys_part_2 += 30*c1p1*c5p4*c9p1*ss[30];
  sys_part_2 += 30*c1p1*c7p1*c9p4*ss[44];
  sys_part_2 += 60*c1p1*c7p2*c9p3*ss[42];
  sys_part_2 += 60*c1p1*c7p3*c9p2*ss[40];
  sys_part_2 += 30*c1p1*c7p4*c9p1*ss[38];
  sys_part_2 += 60*c1p2*c3p1*c5p3*ss[20];
  sys_part_2 += 60*c1p2*c3p1*c7p3*ss[26];
  sys_part_2 += 60*c1p2*c3p1*c9p3*ss[32];
  sys_part_2 += 90*c1p2*c3p2*c5p2*ss[18];
  sys_part_2 += 90*c1p2*c3p2*c7p2*ss[22];
  sys_part_2 += 90*c1p2*c3p2*c9p2*ss[26];
  sys_part_2 += 60*c1p2*c3p3*c5p1*ss[16];
  sys_part_2 += 60*c1p2*c3p3*c7p1*ss[18];
  sys_part_2 += 60*c1p2*c3p3*c9p1*ss[20];
  sys_part_2 += 60*c1p2*c5p1*c7p3*ss[28];
  sys_part_2 += 60*c1p2*c5p1*c9p3*ss[34];
  sys_part_2 += 90*c1p2*c5p2*c7p2*ss[26];
  sys_part_2 += 90*c1p2*c5p2*c9p2*ss[30];
  sys_part_2 += 60*c1p2*c5p3*c7p1*ss[24];
  sys_part_2 += 60*c1p2*c5p3*c9p1*ss[26];
  sys_part_2 += 60*c1p2*c7p1*c9p3*ss[36];
  sys_part_2 += 90*c1p2*c7p2*c9p2*ss[34];
  sys_part_2 += 60*c1p2*c7p3*c9p1*ss[32];
  sys_part_2 += 60*c1p3*c3p1*c5p2*ss[16];
  sys_part_2 += 60*c1p3*c3p1*c7p2*ss[20];
  sys_part_2 += 60*c1p3*c3p1*c9p2*ss[24];
  sys_part_2 += 60*c1p3*c3p2*c5p1*ss[14];
  sys_part_2 += 60*c1p3*c3p2*c7p1*ss[16];
  sys_part_2 += 60*c1p3*c3p2*c9p1*ss[18];
  sys_part_2 += 60*c1p3*c5p1*c7p2*ss[22];
  sys_part_2 += 60*c1p3*c5p1*c9p2*ss[26];
  sys_part_2 += 60*c1p3*c5p2*c7p1*ss[20];
  sys_part_2 += 60*c1p3*c5p2*c9p1*ss[22];
  sys_part_2 += 60*c1p3*c7p1*c9p2*ss[28];
  sys_part_2 += 60*c1p3*c7p2*c9p1*ss[26];
  sys_part_2 += 30*c1p4*c3p1*c5p1*ss[12];
  sys_part_2 += 30*c1p4*c3p1*c7p1*ss[14];
  sys_part_2 += 30*c1p4*c3p1*c9p1*ss[16];
  sys_part_2 += 30*c1p4*c5p1*c7p1*ss[16];
  sys_part_2 += 30*c1p4*c5p1*c9p1*ss[18];
  sys_part_2 += 30*c1p4*c7p1*c9p1*ss[20];
  sys_part_2 += 30*c3p1*c5p1*c7p4*ss[36];
  sys_part_2 += 30*c3p1*c5p1*c9p4*ss[44];
  sys_part_2 += 60*c3p1*c5p2*c7p3*ss[34];
  sys_part_2 += 60*c3p1*c5p2*c9p3*ss[40];
  sys_part_2 += 60*c3p1*c5p3*c7p2*ss[32];
  sys_part_2 += 60*c3p1*c5p3*c9p2*ss[36];
  sys_part_2 += 30*c3p1*c5p4*c7p1*ss[30];
  sys_part_2 += 30*c3p1*c5p4*c9p1*ss[32];
  sys_part_2 += 30*c3p1*c7p1*c9p4*ss[46];
  sys_part_2 += 60*c3p1*c7p2*c9p3*ss[44];
  sys_part_2 += 60*c3p1*c7p3*c9p2*ss[42];
  sys_part_2 += 30*c3p1*c7p4*c9p1*ss[40];
  sys_part_2 += 60*c3p2*c5p1*c7p3*ss[32];
  sys_part_2 += 60*c3p2*c5p1*c9p3*ss[38];
  sys_part_2 += 90*c3p2*c5p2*c7p2*ss[30];
  sys_part_2 += 90*c3p2*c5p2*c9p2*ss[34];
  sys_part_2 += 60*c3p2*c5p3*c7p1*ss[28];
  sys_part_2 += 60*c3p2*c5p3*c9p1*ss[30];
  sys_part_2 += 60*c3p2*c7p1*c9p3*ss[40];
  sys_part_2 += 90*c3p2*c7p2*c9p2*ss[38];
  sys_part_2 += 60*c3p2*c7p3*c9p1*ss[36];
  sys_part_2 += 60*c3p3*c5p1*c7p2*ss[28];
  sys_part_2 += 60*c3p3*c5p1*c9p2*ss[32];
  sys_part_2 += 60*c3p3*c5p2*c7p1*ss[26];
  sys_part_2 += 60*c3p3*c5p2*c9p1*ss[28];
  sys_part_2 += 60*c3p3*c7p1*c9p2*ss[34];
  sys_part_2 += 60*c3p3*c7p2*c9p1*ss[32];
  sys_part_2 += 30*c3p4*c5p1*c7p1*ss[24];
  sys_part_2 += 30*c3p4*c5p1*c9p1*ss[26];
  sys_part_2 += 30*c3p4*c7p1*c9p1*ss[28];
  sys_part_2 += 30*c5p1*c7p1*c9p4*ss[48];
  sys_part_2 += 60*c5p1*c7p2*c9p3*ss[46];
  sys_part_2 += 60*c5p1*c7p3*c9p2*ss[44];
  sys_part_2 += 30*c5p1*c7p4*c9p1*ss[42];
  sys_part_2 += 60*c5p2*c7p1*c9p3*ss[44];
  sys_part_2 += 90*c5p2*c7p2*c9p2*ss[42];
  sys_part_2 += 60*c5p2*c7p3*c9p1*ss[40];
  sys_part_2 += 60*c5p3*c7p1*c9p2*ss[40];
  sys_part_2 += 60*c5p3*c7p2*c9p1*ss[38];
  sys_part_2 += 30*c5p4*c7p1*c9p1*ss[36];
  sys_part_2 += 6*c1p1*c3p5*ss[16];
  sys_part_2 += 6*c1p1*c5p5*ss[26];
  sys_part_2 += 6*c1p1*c7p5*ss[36];
  sys_part_2 += 6*c1p1*c9p5*ss[46];
  sys_part_2 += 15*c1p2*c3p4*ss[14];
  sys_part_2 += 15*c1p2*c5p4*ss[22];
  sys_part_2 += 15*c1p2*c7p4*ss[30];
  sys_part_2 += 15*c1p2*c9p4*ss[38];
  sys_part_2 += 20*c1p3*c3p3*ss[12];
  sys_part_2 += 20*c1p3*c5p3*ss[18];
  sys_part_2 += 20*c1p3*c7p3*ss[24];
  sys_part_2 += 20*c1p3*c9p3*ss[30];
  sys_part_2 += 15*c1p4*c3p2*ss[10];
  sys_part_2 += 15*c1p4*c5p2*ss[14];
  sys_part_2 += 15*c1p4*c7p2*ss[18];
  sys_part_2 += 15*c1p4*c9p2*ss[22];
  sys_part_2 += 6*c1p5*c3p1*ss[8];
  sys_part_2 += 6*c1p5*c5p1*ss[10];
  sys_part_2 += 6*c1p5*c7p1*ss[12];
  sys_part_2 += 6*c1p5*c9p1*ss[14];
  sys_part_2 += 6*c3p1*c5p5*ss[28];
  sys_part_2 += 6*c3p1*c7p5*ss[38];
  sys_part_2 += 6*c3p1*c9p5*ss[48];
  sys_part_2 += 15*c3p2*c5p4*ss[26];
  sys_part_2 += 15*c3p2*c7p4*ss[34];
  sys_part_2 += 15*c3p2*c9p4*ss[42];
  sys_part_2 += 20*c3p3*c5p3*ss[24];
  sys_part_2 += 20*c3p3*c7p3*ss[30];
  sys_part_2 += 20*c3p3*c9p3*ss[36];
  sys_part_2 += 15*c3p4*c5p2*ss[22];
  sys_part_2 += 15*c3p4*c7p2*ss[26];
  sys_part_2 += 15*c3p4*c9p2*ss[30];
  sys_part_2 += 6*c3p5*c5p1*ss[20];
  sys_part_2 += 6*c3p5*c7p1*ss[22];
  sys_part_2 += 6*c3p5*c9p1*ss[24];
  sys_part_2 += 6*c5p1*c7p5*ss[40];
  sys_part_2 += 6*c5p1*c9p5*ss[50];
  sys_part_2 += 15*c5p2*c7p4*ss[38];
  sys_part_2 += 15*c5p2*c9p4*ss[46];
  sys_part_2 += 20*c5p3*c7p3*ss[36];
  sys_part_2 += 20*c5p3*c9p3*ss[42];
  sys_part_2 += 15*c5p4*c7p2*ss[34];
  sys_part_2 += 15*c5p4*c9p2*ss[38];
  sys_part_2 += 6*c5p5*c7p1*ss[32];
  sys_part_2 += 6*c5p5*c9p1*ss[34];
  sys_part_2 += 6*c7p1*c9p5*ss[52];
  sys_part_2 += 15*c7p2*c9p4*ss[50];
  sys_part_2 += 20*c7p3*c9p3*ss[48];
  sys_part_2 += 15*c7p4*c9p2*ss[46];
  sys_part_2 += 6*c7p5*c9p1*ss[44];
  sys_part_2 += c1p6*ss[6];
  sys_part_2 += c3p6*ss[18];
  sys_part_2 += c5p6*ss[30];
  sys_part_2 += c7p6*ss[42];
  sys_part_2 += c9p6*ss[54];
  sys_part_2 += -tmom[6];
  T c1p7 = c1p6*c1p1;
  T c1p8 = c1p7*c1p1;
  T c3p7 = c3p6*c3p1;
  T c3p8 = c3p7*c3p1;
  T c5p7 = c5p6*c5p1;
  T c5p8 = c5p7*c5p1;
  T c7p7 = c7p6*c7p1;
  T c7p8 = c7p7*c7p1;
  T c9p7 = c9p6*c9p1;
  T c9p8 = c9p7*c9p1;
  TACC sys_part_3 = convert<TACC>(1680*c1p1*c3p1*c5p1*c7p1*c9p4*ss[52]);
  sys_part_3 += 3360*c1p1*c3p1*c5p1*c7p2*c9p3*ss[50];
  sys_part_3 += 3360*c1p1*c3p1*c5p1*c7p3*c9p2*ss[48];
  sys_part_3 += 1680*c1p1*c3p1*c5p1*c7p4*c9p1*ss[46];
  sys_part_3 += 3360*c1p1*c3p1*c5p2*c7p1*c9p3*ss[48];
  sys_part_3 += 5040*c1p1*c3p1*c5p2*c7p2*c9p2*ss[46];
  sys_part_3 += 3360*c1p1*c3p1*c5p2*c7p3*c9p1*ss[44];
  sys_part_3 += 3360*c1p1*c3p1*c5p3*c7p1*c9p2*ss[44];
  sys_part_3 += 3360*c1p1*c3p1*c5p3*c7p2*c9p1*ss[42];
  sys_part_3 += 1680*c1p1*c3p1*c5p4*c7p1*c9p1*ss[40];
  sys_part_3 += 3360*c1p1*c3p2*c5p1*c7p1*c9p3*ss[46];
  sys_part_3 += 5040*c1p1*c3p2*c5p1*c7p2*c9p2*ss[44];
  sys_part_3 += 3360*c1p1*c3p2*c5p1*c7p3*c9p1*ss[42];
  sys_part_3 += 5040*c1p1*c3p2*c5p2*c7p1*c9p2*ss[42];
  sys_part_3 += 5040*c1p1*c3p2*c5p2*c7p2*c9p1*ss[40];
  sys_part_3 += 3360*c1p1*c3p2*c5p3*c7p1*c9p1*ss[38];
  sys_part_3 += 3360*c1p1*c3p3*c5p1*c7p1*c9p2*ss[40];
  sys_part_3 += 3360*c1p1*c3p3*c5p1*c7p2*c9p1*ss[38];
  sys_part_3 += 3360*c1p1*c3p3*c5p2*c7p1*c9p1*ss[36];
  sys_part_3 += 1680*c1p1*c3p4*c5p1*c7p1*c9p1*ss[34];
  sys_part_3 += 3360*c1p2*c3p1*c5p1*c7p1*c9p3*ss[44];
  sys_part_3 += 5040*c1p2*c3p1*c5p1*c7p2*c9p2*ss[42];
  sys_part_3 += 3360*c1p2*c3p1*c5p1*c7p3*c9p1*ss[40];
  sys_part_3 += 5040*c1p2*c3p1*c5p2*c7p1*c9p2*ss[40];
  sys_part_3 += 5040*c1p2*c3p1*c5p2*c7p2*c9p1*ss[38];
  sys_part_3 += 3360*c1p2*c3p1*c5p3*c7p1*c9p1*ss[36];
  sys_part_3 += 5040*c1p2*c3p2*c5p1*c7p1*c9p2*ss[38];
  sys_part_3 += 5040*c1p2*c3p2*c5p1*c7p2*c9p1*ss[36];
  sys_part_3 += 5040*c1p2*c3p2*c5p2*c7p1*c9p1*ss[34];
  sys_part_3 += 3360*c1p2*c3p3*c5p1*c7p1*c9p1*ss[32];
  sys_part_3 += 3360*c1p3*c3p1*c5p1*c7p1*c9p2*ss[36];
  sys_part_3 += 3360*c1p3*c3p1*c5p1*c7p2*c9p1*ss[34];
  sys_part_3 += 3360*c1p3*c3p1*c5p2*c7p1*c9p1*ss[32];
  sys_part_3 += 3360*c1p3*c3p2*c5p1*c7p1*c9p1*ss[30];
  sys_part_3 += 1680*c1p4*c3p1*c5p1*c7p1*c9p1*ss[28];
  sys_part_3 += 336*c1p1*c3p1*c5p1*c7p5*ss[44];
  sys_part_3 += 336*c1p1*c3p1*c5p1*c9p5*ss[54];
  sys_part_3 += 840*c1p1*c3p1*c5p2*c7p4*ss[42];
  sys_part_3 += 840*c1p1*c3p1*c5p2*c9p4*ss[50];
  sys_part_3 += 1120*c1p1*c3p1*c5p3*c7p3*ss[40];
  sys_part_3 += 1120*c1p1*c3p1*c5p3*c9p3*ss[46];
  sys_part_3 += 840*c1p1*c3p1*c5p4*c7p2*ss[38];
  sys_part_3 += 840*c1p1*c3p1*c5p4*c9p2*ss[42];
  sys_part_3 += 336*c1p1*c3p1*c5p5*c7p1*ss[36];
  sys_part_3 += 336*c1p1*c3p1*c5p5*c9p1*ss[38];
  sys_part_3 += 336*c1p1*c3p1*c7p1*c9p5*ss[56];
  sys_part_3 += 840*c1p1*c3p1*c7p2*c9p4*ss[54];
  sys_part_3 += 1120*c1p1*c3p1*c7p3*c9p3*ss[52];
  sys_part_3 += 840*c1p1*c3p1*c7p4*c9p2*ss[50];
  sys_part_3 += 336*c1p1*c3p1*c7p5*c9p1*ss[48];
  sys_part_3 += 840*c1p1*c3p2*c5p1*c7p4*ss[40];
  sys_part_3 += 840*c1p1*c3p2*c5p1*c9p4*ss[48];
  sys_part_3 += 1680*c1p1*c3p2*c5p2*c7p3*ss[38];
  sys_part_3 += 1680*c1p1*c3p2*c5p2*c9p3*ss[44];
  sys_part_3 += 1680*c1p1*c3p2*c5p3*c7p2*ss[36];
  sys_part_3 += 1680*c1p1*c3p2*c5p3*c9p2*ss[40];
  sys_part_3 += 840*c1p1*c3p2*c5p4*c7p1*ss[34];
  sys_part_3 += 840*c1p1*c3p2*c5p4*c9p1*ss[36];
  sys_part_3 += 840*c1p1*c3p2*c7p1*c9p4*ss[50];
  sys_part_3 += 1680*c1p1*c3p2*c7p2*c9p3*ss[48];
  sys_part_3 += 1680*c1p1*c3p2*c7p3*c9p2*ss[46];
  sys_part_3 += 840*c1p1*c3p2*c7p4*c9p1*ss[44];
  sys_part_3 += 1120*c1p1*c3p3*c5p1*c7p3*ss[36];
  sys_part_3 += 1120*c1p1*c3p3*c5p1*c9p3*ss[42];
  sys_part_3 += 1680*c1p1*c3p3*c5p2*c7p2*ss[34];
  sys_part_3 += 1680*c1p1*c3p3*c5p2*c9p2*ss[38];
  sys_part_3 += 1120*c1p1*c3p3*c5p3*c7p1*ss[32];
  sys_part_3 += 1120*c1p1*c3p3*c5p3*c9p1*ss[34];
  sys_part_3 += 1120*c1p1*c3p3*c7p1*c9p3*ss[44];
  sys_part_3 += 1680*c1p1*c3p3*c7p2*c9p2*ss[42];
  sys_part_3 += 1120*c1p1*c3p3*c7p3*c9p1*ss[40];
  sys_part_3 += 840*c1p1*c3p4*c5p1*c7p2*ss[32];
  sys_part_3 += 840*c1p1*c3p4*c5p1*c9p2*ss[36];
  sys_part_3 += 840*c1p1*c3p4*c5p2*c7p1*ss[30];
  sys_part_3 += 840*c1p1*c3p4*c5p2*c9p1*ss[32];
  sys_part_3 += 840*c1p1*c3p4*c7p1*c9p2*ss[38];
  sys_part_3 += 840*c1p1*c3p4*c7p2*c9p1*ss[36];
  sys_part_3 += 336*c1p1*c3p5*c5p1*c7p1*ss[28];
  sys_part_3 += 336*c1p1*c3p5*c5p1*c9p1*ss[30];
  sys_part_3 += 336*c1p1*c3p5*c7p1*c9p1*ss[32];
  sys_part_3 += 336*c1p1*c5p1*c7p1*c9p5*ss[58];
  sys_part_3 += 840*c1p1*c5p1*c7p2*c9p4*ss[56];
  sys_part_3 += 1120*c1p1*c5p1*c7p3*c9p3*ss[54];
  sys_part_3 += 840*c1p1*c5p1*c7p4*c9p2*ss[52];
  sys_part_3 += 336*c1p1*c5p1*c7p5*c9p1*ss[50];
  sys_part_3 += 840*c1p1*c5p2*c7p1*c9p4*ss[54];
  sys_part_3 += 1680*c1p1*c5p2*c7p2*c9p3*ss[52];
  sys_part_3 += 1680*c1p1*c5p2*c7p3*c9p2*ss[50];
  sys_part_3 += 840*c1p1*c5p2*c7p4*c9p1*ss[48];
  sys_part_3 += 1120*c1p1*c5p3*c7p1*c9p3*ss[50];
  sys_part_3 += 1680*c1p1*c5p3*c7p2*c9p2*ss[48];
  sys_part_3 += 1120*c1p1*c5p3*c7p3*c9p1*ss[46];
  sys_part_3 += 840*c1p1*c5p4*c7p1*c9p2*ss[46];
  sys_part_3 += 840*c1p1*c5p4*c7p2*c9p1*ss[44];
  sys_part_3 += 336*c1p1*c5p5*c7p1*c9p1*ss[42];
  sys_part_3 += 840*c1p2*c3p1*c5p1*c7p4*ss[38];
  sys_part_3 += 840*c1p2*c3p1*c5p1*c9p4*ss[46];
  sys_part_3 += 1680*c1p2*c3p1*c5p2*c7p3*ss[36];
  sys_part_3 += 1680*c1p2*c3p1*c5p2*c9p3*ss[42];
  sys_part_3 += 1680*c1p2*c3p1*c5p3*c7p2*ss[34];
  sys_part_3 += 1680*c1p2*c3p1*c5p3*c9p2*ss[38];
  sys_part_3 += 840*c1p2*c3p1*c5p4*c7p1*ss[32];
  sys_part_3 += 840*c1p2*c3p1*c5p4*c9p1*ss[34];
  sys_part_3 += 840*c1p2*c3p1*c7p1*c9p4*ss[48];
  sys_part_3 += 1680*c1p2*c3p1*c7p2*c9p3*ss[46];
  sys_part_3 += 1680*c1p2*c3p1*c7p3*c9p2*ss[44];
  sys_part_3 += 840*c1p2*c3p1*c7p4*c9p1*ss[42];
  sys_part_3 += 1680*c1p2*c3p2*c5p1*c7p3*ss[34];
  sys_part_3 += 1680*c1p2*c3p2*c5p1*c9p3*ss[40];
  sys_part_3 += 2520*c1p2*c3p2*c5p2*c7p2*ss[32];
  sys_part_3 += 2520*c1p2*c3p2*c5p2*c9p2*ss[36];
  sys_part_3 += 1680*c1p2*c3p2*c5p3*c7p1*ss[30];
  sys_part_3 += 1680*c1p2*c3p2*c5p3*c9p1*ss[32];
  sys_part_3 += 1680*c1p2*c3p2*c7p1*c9p3*ss[42];
  sys_part_3 += 2520*c1p2*c3p2*c7p2*c9p2*ss[40];
  sys_part_3 += 1680*c1p2*c3p2*c7p3*c9p1*ss[38];
  sys_part_3 += 1680*c1p2*c3p3*c5p1*c7p2*ss[30];
  sys_part_3 += 1680*c1p2*c3p3*c5p1*c9p2*ss[34];
  sys_part_3 += 1680*c1p2*c3p3*c5p2*c7p1*ss[28];
  sys_part_3 += 1680*c1p2*c3p3*c5p2*c9p1*ss[30];
  sys_part_3 += 1680*c1p2*c3p3*c7p1*c9p2*ss[36];
  sys_part_3 += 1680*c1p2*c3p3*c7p2*c9p1*ss[34];
  sys_part_3 += 840*c1p2*c3p4*c5p1*c7p1*ss[26];
  sys_part_3 += 840*c1p2*c3p4*c5p1*c9p1*ss[28];
  sys_part_3 += 840*c1p2*c3p4*c7p1*c9p1*ss[30];
  sys_part_3 += 840*c1p2*c5p1*c7p1*c9p4*ss[50];
  sys_part_3 += 1680*c1p2*c5p1*c7p2*c9p3*ss[48];
  sys_part_3 += 1680*c1p2*c5p1*c7p3*c9p2*ss[46];
  sys_part_3 += 840*c1p2*c5p1*c7p4*c9p1*ss[44];
  sys_part_3 += 1680*c1p2*c5p2*c7p1*c9p3*ss[46];
  sys_part_3 += 2520*c1p2*c5p2*c7p2*c9p2*ss[44];
  sys_part_3 += 1680*c1p2*c5p2*c7p3*c9p1*ss[42];
  sys_part_3 += 1680*c1p2*c5p3*c7p1*c9p2*ss[42];
  sys_part_3 += 1680*c1p2*c5p3*c7p2*c9p1*ss[40];
  sys_part_3 += 840*c1p2*c5p4*c7p1*c9p1*ss[38];
  sys_part_3 += 1120*c1p3*c3p1*c5p1*c7p3*ss[32];
  sys_part_3 += 1120*c1p3*c3p1*c5p1*c9p3*ss[38];
  sys_part_3 += 1680*c1p3*c3p1*c5p2*c7p2*ss[30];
  sys_part_3 += 1680*c1p3*c3p1*c5p2*c9p2*ss[34];
  sys_part_3 += 1120*c1p3*c3p1*c5p3*c7p1*ss[28];
  sys_part_3 += 1120*c1p3*c3p1*c5p3*c9p1*ss[30];
  sys_part_3 += 1120*c1p3*c3p1*c7p1*c9p3*ss[40];
  sys_part_3 += 1680*c1p3*c3p1*c7p2*c9p2*ss[38];
  sys_part_3 += 1120*c1p3*c3p1*c7p3*c9p1*ss[36];
  sys_part_3 += 1680*c1p3*c3p2*c5p1*c7p2*ss[28];
  sys_part_3 += 1680*c1p3*c3p2*c5p1*c9p2*ss[32];
  sys_part_3 += 1680*c1p3*c3p2*c5p2*c7p1*ss[26];
  sys_part_3 += 1680*c1p3*c3p2*c5p2*c9p1*ss[28];
  sys_part_3 += 1680*c1p3*c3p2*c7p1*c9p2*ss[34];
  sys_part_3 += 1680*c1p3*c3p2*c7p2*c9p1*ss[32];
  sys_part_3 += 1120*c1p3*c3p3*c5p1*c7p1*ss[24];
  sys_part_3 += 1120*c1p3*c3p3*c5p1*c9p1*ss[26];
  sys_part_3 += 1120*c1p3*c3p3*c7p1*c9p1*ss[28];
  sys_part_3 += 1120*c1p3*c5p1*c7p1*c9p3*ss[42];
  sys_part_3 += 1680*c1p3*c5p1*c7p2*c9p2*ss[40];
  sys_part_3 += 1120*c1p3*c5p1*c7p3*c9p1*ss[38];
  sys_part_3 += 1680*c1p3*c5p2*c7p1*c9p2*ss[38];
  sys_part_3 += 1680*c1p3*c5p2*c7p2*c9p1*ss[36];
  sys_part_3 += 1120*c1p3*c5p3*c7p1*c9p1*ss[34];
  sys_part_3 += 840*c1p4*c3p1*c5p1*c7p2*ss[26];
  sys_part_3 += 840*c1p4*c3p1*c5p1*c9p2*ss[30];
  sys_part_3 += 840*c1p4*c3p1*c5p2*c7p1*ss[24];
  sys_part_3 += 840*c1p4*c3p1*c5p2*c9p1*ss[26];
  sys_part_3 += 840*c1p4*c3p1*c7p1*c9p2*ss[32];
  sys_part_3 += 840*c1p4*c3p1*c7p2*c9p1*ss[30];
  sys_part_3 += 840*c1p4*c3p2*c5p1*c7p1*ss[22];
  sys_part_3 += 840*c1p4*c3p2*c5p1*c9p1*ss[24];
  sys_part_3 += 840*c1p4*c3p2*c7p1*c9p1*ss[26];
  sys_part_3 += 840*c1p4*c5p1*c7p1*c9p2*ss[34];
  sys_part_3 += 840*c1p4*c5p1*c7p2*c9p1*ss[32];
  sys_part_3 += 840*c1p4*c5p2*c7p1*c9p1*ss[30];
  sys_part_3 += 336*c1p5*c3p1*c5p1*c7p1*ss[20];
  sys_part_3 += 336*c1p5*c3p1*c5p1*c9p1*ss[22];
  sys_part_3 += 336*c1p5*c3p1*c7p1*c9p1*ss[24];
  sys_part_3 += 336*c1p5*c5p1*c7p1*c9p1*ss[26];
  sys_part_3 += 336*c3p1*c5p1*c7p1*c9p5*ss[60];
  sys_part_3 += 840*c3p1*c5p1*c7p2*c9p4*ss[58];
  sys_part_3 += 1120*c3p1*c5p1*c7p3*c9p3*ss[56];
  sys_part_3 += 840*c3p1*c5p1*c7p4*c9p2*ss[54];
  sys_part_3 += 336*c3p1*c5p1*c7p5*c9p1*ss[52];
  sys_part_3 += 840*c3p1*c5p2*c7p1*c9p4*ss[56];
  sys_part_3 += 1680*c3p1*c5p2*c7p2*c9p3*ss[54];
  sys_part_3 += 1680*c3p1*c5p2*c7p3*c9p2*ss[52];
  sys_part_3 += 840*c3p1*c5p2*c7p4*c9p1*ss[50];
  sys_part_3 += 1120*c3p1*c5p3*c7p1*c9p3*ss[52];
  sys_part_3 += 1680*c3p1*c5p3*c7p2*c9p2*ss[50];
  sys_part_3 += 1120*c3p1*c5p3*c7p3*c9p1*ss[48];
  sys_part_3 += 840*c3p1*c5p4*c7p1*c9p2*ss[48];
  sys_part_3 += 840*c3p1*c5p4*c7p2*c9p1*ss[46];
  sys_part_3 += 336*c3p1*c5p5*c7p1*c9p1*ss[44];
  sys_part_3 += 840*c3p2*c5p1*c7p1*c9p4*ss[54];
  sys_part_3 += 1680*c3p2*c5p1*c7p2*c9p3*ss[52];
  sys_part_3 += 1680*c3p2*c5p1*c7p3*c9p2*ss[50];
  sys_part_3 += 840*c3p2*c5p1*c7p4*c9p1*ss[48];
  sys_part_3 += 1680*c3p2*c5p2*c7p1*c9p3*ss[50];
  sys_part_3 += 2520*c3p2*c5p2*c7p2*c9p2*ss[48];
  sys_part_3 += 1680*c3p2*c5p2*c7p3*c9p1*ss[46];
  sys_part_3 += 1680*c3p2*c5p3*c7p1*c9p2*ss[46];
  sys_part_3 += 1680*c3p2*c5p3*c7p2*c9p1*ss[44];
  sys_part_3 += 840*c3p2*c5p4*c7p1*c9p1*ss[42];
  sys_part_3 += 1120*c3p3*c5p1*c7p1*c9p3*ss[48];
  sys_part_3 += 1680*c3p3*c5p1*c7p2*c9p2*ss[46];
  sys_part_3 += 1120*c3p3*c5p1*c7p3*c9p1*ss[44];
  sys_part_3 += 1680*c3p3*c5p2*c7p1*c9p2*ss[44];
  sys_part_3 += 1680*c3p3*c5p2*c7p2*c9p1*ss[42];
  sys_part_3 += 1120*c3p3*c5p3*c7p1*c9p1*ss[40];
  sys_part_3 += 840*c3p4*c5p1*c7p1*c9p2*ss[42];
  sys_part_3 += 840*c3p4*c5p1*c7p2*c9p1*ss[40];
  sys_part_3 += 840*c3p4*c5p2*c7p1*c9p1*ss[38];
  sys_part_3 += 336*c3p5*c5p1*c7p1*c9p1*ss[36];
  sys_part_3 += 56*c1p1*c3p1*c5p6*ss[34];
  sys_part_3 += 56*c1p1*c3p1*c7p6*ss[46];
  sys_part_3 += 56*c1p1*c3p1*c9p6*ss[58];
  sys_part_3 += 168*c1p1*c3p2*c5p5*ss[32];
  sys_part_3 += 168*c1p1*c3p2*c7p5*ss[42];
  sys_part_3 += 168*c1p1*c3p2*c9p5*ss[52];
  sys_part_3 += 280*c1p1*c3p3*c5p4*ss[30];
  sys_part_3 += 280*c1p1*c3p3*c7p4*ss[38];
  sys_part_3 += 280*c1p1*c3p3*c9p4*ss[46];
  sys_part_3 += 280*c1p1*c3p4*c5p3*ss[28];
  sys_part_3 += 280*c1p1*c3p4*c7p3*ss[34];
  sys_part_3 += 280*c1p1*c3p4*c9p3*ss[40];
  sys_part_3 += 168*c1p1*c3p5*c5p2*ss[26];
  sys_part_3 += 168*c1p1*c3p5*c7p2*ss[30];
  sys_part_3 += 168*c1p1*c3p5*c9p2*ss[34];
  sys_part_3 += 56*c1p1*c3p6*c5p1*ss[24];
  sys_part_3 += 56*c1p1*c3p6*c7p1*ss[26];
  sys_part_3 += 56*c1p1*c3p6*c9p1*ss[28];
  sys_part_3 += 56*c1p1*c5p1*c7p6*ss[48];
  sys_part_3 += 56*c1p1*c5p1*c9p6*ss[60];
  sys_part_3 += 168*c1p1*c5p2*c7p5*ss[46];
  sys_part_3 += 168*c1p1*c5p2*c9p5*ss[56];
  sys_part_3 += 280*c1p1*c5p3*c7p4*ss[44];
  sys_part_3 += 280*c1p1*c5p3*c9p4*ss[52];
  sys_part_3 += 280*c1p1*c5p4*c7p3*ss[42];
  sys_part_3 += 280*c1p1*c5p4*c9p3*ss[48];
  sys_part_3 += 168*c1p1*c5p5*c7p2*ss[40];
  sys_part_3 += 168*c1p1*c5p5*c9p2*ss[44];
  sys_part_3 += 56*c1p1*c5p6*c7p1*ss[38];
  sys_part_3 += 56*c1p1*c5p6*c9p1*ss[40];
  sys_part_3 += 56*c1p1*c7p1*c9p6*ss[62];
  sys_part_3 += 168*c1p1*c7p2*c9p5*ss[60];
  sys_part_3 += 280*c1p1*c7p3*c9p4*ss[58];
  sys_part_3 += 280*c1p1*c7p4*c9p3*ss[56];
  sys_part_3 += 168*c1p1*c7p5*c9p2*ss[54];
  sys_part_3 += 56*c1p1*c7p6*c9p1*ss[52];
  sys_part_3 += 168*c1p2*c3p1*c5p5*ss[30];
  sys_part_3 += 168*c1p2*c3p1*c7p5*ss[40];
  sys_part_3 += 168*c1p2*c3p1*c9p5*ss[50];
  sys_part_3 += 420*c1p2*c3p2*c5p4*ss[28];
  sys_part_3 += 420*c1p2*c3p2*c7p4*ss[36];
  sys_part_3 += 420*c1p2*c3p2*c9p4*ss[44];
  sys_part_3 += 560*c1p2*c3p3*c5p3*ss[26];
  sys_part_3 += 560*c1p2*c3p3*c7p3*ss[32];
  sys_part_3 += 560*c1p2*c3p3*c9p3*ss[38];
  sys_part_3 += 420*c1p2*c3p4*c5p2*ss[24];
  sys_part_3 += 420*c1p2*c3p4*c7p2*ss[28];
  sys_part_3 += 420*c1p2*c3p4*c9p2*ss[32];
  sys_part_3 += 168*c1p2*c3p5*c5p1*ss[22];
  sys_part_3 += 168*c1p2*c3p5*c7p1*ss[24];
  sys_part_3 += 168*c1p2*c3p5*c9p1*ss[26];
  sys_part_3 += 168*c1p2*c5p1*c7p5*ss[42];
  sys_part_3 += 168*c1p2*c5p1*c9p5*ss[52];
  sys_part_3 += 420*c1p2*c5p2*c7p4*ss[40];
  sys_part_3 += 420*c1p2*c5p2*c9p4*ss[48];
  sys_part_3 += 560*c1p2*c5p3*c7p3*ss[38];
  sys_part_3 += 560*c1p2*c5p3*c9p3*ss[44];
  sys_part_3 += 420*c1p2*c5p4*c7p2*ss[36];
  sys_part_3 += 420*c1p2*c5p4*c9p2*ss[40];
  sys_part_3 += 168*c1p2*c5p5*c7p1*ss[34];
  sys_part_3 += 168*c1p2*c5p5*c9p1*ss[36];
  sys_part_3 += 168*c1p2*c7p1*c9p5*ss[54];
  sys_part_3 += 420*c1p2*c7p2*c9p4*ss[52];
  sys_part_3 += 560*c1p2*c7p3*c9p3*ss[50];
  sys_part_3 += 420*c1p2*c7p4*c9p2*ss[48];
  sys_part_3 += 168*c1p2*c7p5*c9p1*ss[46];
  sys_part_3 += 280*c1p3*c3p1*c5p4*ss[26];
  sys_part_3 += 280*c1p3*c3p1*c7p4*ss[34];
  sys_part_3 += 280*c1p3*c3p1*c9p4*ss[42];
  sys_part_3 += 560*c1p3*c3p2*c5p3*ss[24];
  sys_part_3 += 560*c1p3*c3p2*c7p3*ss[30];
  sys_part_3 += 560*c1p3*c3p2*c9p3*ss[36];
  sys_part_3 += 560*c1p3*c3p3*c5p2*ss[22];
  sys_part_3 += 560*c1p3*c3p3*c7p2*ss[26];
  sys_part_3 += 560*c1p3*c3p3*c9p2*ss[30];
  sys_part_3 += 280*c1p3*c3p4*c5p1*ss[20];
  sys_part_3 += 280*c1p3*c3p4*c7p1*ss[22];
  sys_part_3 += 280*c1p3*c3p4*c9p1*ss[24];
  sys_part_3 += 280*c1p3*c5p1*c7p4*ss[36];
  sys_part_3 += 280*c1p3*c5p1*c9p4*ss[44];
  sys_part_3 += 560*c1p3*c5p2*c7p3*ss[34];
  sys_part_3 += 560*c1p3*c5p2*c9p3*ss[40];
  sys_part_3 += 560*c1p3*c5p3*c7p2*ss[32];
  sys_part_3 += 560*c1p3*c5p3*c9p2*ss[36];
  sys_part_3 += 280*c1p3*c5p4*c7p1*ss[30];
  sys_part_3 += 280*c1p3*c5p4*c9p1*ss[32];
  sys_part_3 += 280*c1p3*c7p1*c9p4*ss[46];
  sys_part_3 += 560*c1p3*c7p2*c9p3*ss[44];
  sys_part_3 += 560*c1p3*c7p3*c9p2*ss[42];
  sys_part_3 += 280*c1p3*c7p4*c9p1*ss[40];
  sys_part_3 += 280*c1p4*c3p1*c5p3*ss[22];
  sys_part_3 += 280*c1p4*c3p1*c7p3*ss[28];
  sys_part_3 += 280*c1p4*c3p1*c9p3*ss[34];
  sys_part_3 += 420*c1p4*c3p2*c5p2*ss[20];
  sys_part_3 += 420*c1p4*c3p2*c7p2*ss[24];
  sys_part_3 += 420*c1p4*c3p2*c9p2*ss[28];
  sys_part_3 += 280*c1p4*c3p3*c5p1*ss[18];
  sys_part_3 += 280*c1p4*c3p3*c7p1*ss[20];
  sys_part_3 += 280*c1p4*c3p3*c9p1*ss[22];
  sys_part_3 += 280*c1p4*c5p1*c7p3*ss[30];
  sys_part_3 += 280*c1p4*c5p1*c9p3*ss[36];
  sys_part_3 += 420*c1p4*c5p2*c7p2*ss[28];
  sys_part_3 += 420*c1p4*c5p2*c9p2*ss[32];
  sys_part_3 += 280*c1p4*c5p3*c7p1*ss[26];
  sys_part_3 += 280*c1p4*c5p3*c9p1*ss[28];
  sys_part_3 += 280*c1p4*c7p1*c9p3*ss[38];
  sys_part_3 += 420*c1p4*c7p2*c9p2*ss[36];
  sys_part_3 += 280*c1p4*c7p3*c9p1*ss[34];
  sys_part_3 += 168*c1p5*c3p1*c5p2*ss[18];
  sys_part_3 += 168*c1p5*c3p1*c7p2*ss[22];
  sys_part_3 += 168*c1p5*c3p1*c9p2*ss[26];
  sys_part_3 += 168*c1p5*c3p2*c5p1*ss[16];
  sys_part_3 += 168*c1p5*c3p2*c7p1*ss[18];
  sys_part_3 += 168*c1p5*c3p2*c9p1*ss[20];
  sys_part_3 += 168*c1p5*c5p1*c7p2*ss[24];
  sys_part_3 += 168*c1p5*c5p1*c9p2*ss[28];
  sys_part_3 += 168*c1p5*c5p2*c7p1*ss[22];
  sys_part_3 += 168*c1p5*c5p2*c9p1*ss[24];
  sys_part_3 += 168*c1p5*c7p1*c9p2*ss[30];
  sys_part_3 += 168*c1p5*c7p2*c9p1*ss[28];
  sys_part_3 += 56*c1p6*c3p1*c5p1*ss[14];
  sys_part_3 += 56*c1p6*c3p1*c7p1*ss[16];
  sys_part_3 += 56*c1p6*c3p1*c9p1*ss[18];
  sys_part_3 += 56*c1p6*c5p1*c7p1*ss[18];
  sys_part_3 += 56*c1p6*c5p1*c9p1*ss[20];
  sys_part_3 += 56*c1p6*c7p1*c9p1*ss[22];
  sys_part_3 += 56*c3p1*c5p1*c7p6*ss[50];
  sys_part_3 += 56*c3p1*c5p1*c9p6*ss[62];
  sys_part_3 += 168*c3p1*c5p2*c7p5*ss[48];
  sys_part_3 += 168*c3p1*c5p2*c9p5*ss[58];
  sys_part_3 += 280*c3p1*c5p3*c7p4*ss[46];
  sys_part_3 += 280*c3p1*c5p3*c9p4*ss[54];
  sys_part_3 += 280*c3p1*c5p4*c7p3*ss[44];
  sys_part_3 += 280*c3p1*c5p4*c9p3*ss[50];
  sys_part_3 += 168*c3p1*c5p5*c7p2*ss[42];
  sys_part_3 += 168*c3p1*c5p5*c9p2*ss[46];
  sys_part_3 += 56*c3p1*c5p6*c7p1*ss[40];
  sys_part_3 += 56*c3p1*c5p6*c9p1*ss[42];
  sys_part_3 += 56*c3p1*c7p1*c9p6*ss[64];
  sys_part_3 += 168*c3p1*c7p2*c9p5*ss[62];
  sys_part_3 += 280*c3p1*c7p3*c9p4*ss[60];
  sys_part_3 += 280*c3p1*c7p4*c9p3*ss[58];
  sys_part_3 += 168*c3p1*c7p5*c9p2*ss[56];
  sys_part_3 += 56*c3p1*c7p6*c9p1*ss[54];
  sys_part_3 += 168*c3p2*c5p1*c7p5*ss[46];
  sys_part_3 += 168*c3p2*c5p1*c9p5*ss[56];
  sys_part_3 += 420*c3p2*c5p2*c7p4*ss[44];
  sys_part_3 += 420*c3p2*c5p2*c9p4*ss[52];
  sys_part_3 += 560*c3p2*c5p3*c7p3*ss[42];
  sys_part_3 += 560*c3p2*c5p3*c9p3*ss[48];
  sys_part_3 += 420*c3p2*c5p4*c7p2*ss[40];
  sys_part_3 += 420*c3p2*c5p4*c9p2*ss[44];
  sys_part_3 += 168*c3p2*c5p5*c7p1*ss[38];
  sys_part_3 += 168*c3p2*c5p5*c9p1*ss[40];
  sys_part_3 += 168*c3p2*c7p1*c9p5*ss[58];
  sys_part_3 += 420*c3p2*c7p2*c9p4*ss[56];
  sys_part_3 += 560*c3p2*c7p3*c9p3*ss[54];
  sys_part_3 += 420*c3p2*c7p4*c9p2*ss[52];
  sys_part_3 += 168*c3p2*c7p5*c9p1*ss[50];
  sys_part_3 += 280*c3p3*c5p1*c7p4*ss[42];
  sys_part_3 += 280*c3p3*c5p1*c9p4*ss[50];
  sys_part_3 += 560*c3p3*c5p2*c7p3*ss[40];
  sys_part_3 += 560*c3p3*c5p2*c9p3*ss[46];
  sys_part_3 += 560*c3p3*c5p3*c7p2*ss[38];
  sys_part_3 += 560*c3p3*c5p3*c9p2*ss[42];
  sys_part_3 += 280*c3p3*c5p4*c7p1*ss[36];
  sys_part_3 += 280*c3p3*c5p4*c9p1*ss[38];
  sys_part_3 += 280*c3p3*c7p1*c9p4*ss[52];
  sys_part_3 += 560*c3p3*c7p2*c9p3*ss[50];
  sys_part_3 += 560*c3p3*c7p3*c9p2*ss[48];
  sys_part_3 += 280*c3p3*c7p4*c9p1*ss[46];
  sys_part_3 += 280*c3p4*c5p1*c7p3*ss[38];
  sys_part_3 += 280*c3p4*c5p1*c9p3*ss[44];
  sys_part_3 += 420*c3p4*c5p2*c7p2*ss[36];
  sys_part_3 += 420*c3p4*c5p2*c9p2*ss[40];
  sys_part_3 += 280*c3p4*c5p3*c7p1*ss[34];
  sys_part_3 += 280*c3p4*c5p3*c9p1*ss[36];
  sys_part_3 += 280*c3p4*c7p1*c9p3*ss[46];
  sys_part_3 += 420*c3p4*c7p2*c9p2*ss[44];
  sys_part_3 += 280*c3p4*c7p3*c9p1*ss[42];
  sys_part_3 += 168*c3p5*c5p1*c7p2*ss[34];
  sys_part_3 += 168*c3p5*c5p1*c9p2*ss[38];
  sys_part_3 += 168*c3p5*c5p2*c7p1*ss[32];
  sys_part_3 += 168*c3p5*c5p2*c9p1*ss[34];
  sys_part_3 += 168*c3p5*c7p1*c9p2*ss[40];
  sys_part_3 += 168*c3p5*c7p2*c9p1*ss[38];
  sys_part_3 += 56*c3p6*c5p1*c7p1*ss[30];
  sys_part_3 += 56*c3p6*c5p1*c9p1*ss[32];
  sys_part_3 += 56*c3p6*c7p1*c9p1*ss[34];
  sys_part_3 += 56*c5p1*c7p1*c9p6*ss[66];
  sys_part_3 += 168*c5p1*c7p2*c9p5*ss[64];
  sys_part_3 += 280*c5p1*c7p3*c9p4*ss[62];
  sys_part_3 += 280*c5p1*c7p4*c9p3*ss[60];
  sys_part_3 += 168*c5p1*c7p5*c9p2*ss[58];
  sys_part_3 += 56*c5p1*c7p6*c9p1*ss[56];
  sys_part_3 += 168*c5p2*c7p1*c9p5*ss[62];
  sys_part_3 += 420*c5p2*c7p2*c9p4*ss[60];
  sys_part_3 += 560*c5p2*c7p3*c9p3*ss[58];
  sys_part_3 += 420*c5p2*c7p4*c9p2*ss[56];
  sys_part_3 += 168*c5p2*c7p5*c9p1*ss[54];
  sys_part_3 += 280*c5p3*c7p1*c9p4*ss[58];
  sys_part_3 += 560*c5p3*c7p2*c9p3*ss[56];
  sys_part_3 += 560*c5p3*c7p3*c9p2*ss[54];
  sys_part_3 += 280*c5p3*c7p4*c9p1*ss[52];
  sys_part_3 += 280*c5p4*c7p1*c9p3*ss[54];
  sys_part_3 += 420*c5p4*c7p2*c9p2*ss[52];
  sys_part_3 += 280*c5p4*c7p3*c9p1*ss[50];
  sys_part_3 += 168*c5p5*c7p1*c9p2*ss[50];
  sys_part_3 += 168*c5p5*c7p2*c9p1*ss[48];
  sys_part_3 += 56*c5p6*c7p1*c9p1*ss[46];
  sys_part_3 += 8*c1p1*c3p7*ss[22];
  sys_part_3 += 8*c1p1*c5p7*ss[36];
  sys_part_3 += 8*c1p1*c7p7*ss[50];
  sys_part_3 += 8*c1p1*c9p7*ss[64];
  sys_part_3 += 28*c1p2*c3p6*ss[20];
  sys_part_3 += 28*c1p2*c5p6*ss[32];
  sys_part_3 += 28*c1p2*c7p6*ss[44];
  sys_part_3 += 28*c1p2*c9p6*ss[56];
  sys_part_3 += 56*c1p3*c3p5*ss[18];
  sys_part_3 += 56*c1p3*c5p5*ss[28];
  sys_part_3 += 56*c1p3*c7p5*ss[38];
  sys_part_3 += 56*c1p3*c9p5*ss[48];
  sys_part_3 += 70*c1p4*c3p4*ss[16];
  sys_part_3 += 70*c1p4*c5p4*ss[24];
  sys_part_3 += 70*c1p4*c7p4*ss[32];
  sys_part_3 += 70*c1p4*c9p4*ss[40];
  sys_part_3 += 56*c1p5*c3p3*ss[14];
  sys_part_3 += 56*c1p5*c5p3*ss[20];
  sys_part_3 += 56*c1p5*c7p3*ss[26];
  sys_part_3 += 56*c1p5*c9p3*ss[32];
  sys_part_3 += 28*c1p6*c3p2*ss[12];
  sys_part_3 += 28*c1p6*c5p2*ss[16];
  sys_part_3 += 28*c1p6*c7p2*ss[20];
  sys_part_3 += 28*c1p6*c9p2*ss[24];
  sys_part_3 += 8*c1p7*c3p1*ss[10];
  sys_part_3 += 8*c1p7*c5p1*ss[12];
  sys_part_3 += 8*c1p7*c7p1*ss[14];
  sys_part_3 += 8*c1p7*c9p1*ss[16];
  sys_part_3 += 8*c3p1*c5p7*ss[38];
  sys_part_3 += 8*c3p1*c7p7*ss[52];
  sys_part_3 += 8*c3p1*c9p7*ss[66];
  sys_part_3 += 28*c3p2*c5p6*ss[36];
  sys_part_3 += 28*c3p2*c7p6*ss[48];
  sys_part_3 += 28*c3p2*c9p6*ss[60];
  sys_part_3 += 56*c3p3*c5p5*ss[34];
  sys_part_3 += 56*c3p3*c7p5*ss[44];
  sys_part_3 += 56*c3p3*c9p5*ss[54];
  sys_part_3 += 70*c3p4*c5p4*ss[32];
  sys_part_3 += 70*c3p4*c7p4*ss[40];
  sys_part_3 += 70*c3p4*c9p4*ss[48];
  sys_part_3 += 56*c3p5*c5p3*ss[30];
  sys_part_3 += 56*c3p5*c7p3*ss[36];
  sys_part_3 += 56*c3p5*c9p3*ss[42];
  sys_part_3 += 28*c3p6*c5p2*ss[28];
  sys_part_3 += 28*c3p6*c7p2*ss[32];
  sys_part_3 += 28*c3p6*c9p2*ss[36];
  sys_part_3 += 8*c3p7*c5p1*ss[26];
  sys_part_3 += 8*c3p7*c7p1*ss[28];
  sys_part_3 += 8*c3p7*c9p1*ss[30];
  sys_part_3 += 8*c5p1*c7p7*ss[54];
  sys_part_3 += 8*c5p1*c9p7*ss[68];
  sys_part_3 += 28*c5p2*c7p6*ss[52];
  sys_part_3 += 28*c5p2*c9p6*ss[64];
  sys_part_3 += 56*c5p3*c7p5*ss[50];
  sys_part_3 += 56*c5p3*c9p5*ss[60];
  sys_part_3 += 70*c5p4*c7p4*ss[48];
  sys_part_3 += 70*c5p4*c9p4*ss[56];
  sys_part_3 += 56*c5p5*c7p3*ss[46];
  sys_part_3 += 56*c5p5*c9p3*ss[52];
  sys_part_3 += 28*c5p6*c7p2*ss[44];
  sys_part_3 += 28*c5p6*c9p2*ss[48];
  sys_part_3 += 8*c5p7*c7p1*ss[42];
  sys_part_3 += 8*c5p7*c9p1*ss[44];
  sys_part_3 += 8*c7p1*c9p7*ss[70];
  sys_part_3 += 28*c7p2*c9p6*ss[68];
  sys_part_3 += 56*c7p3*c9p5*ss[66];
  sys_part_3 += 70*c7p4*c9p4*ss[64];
  sys_part_3 += 56*c7p5*c9p3*ss[62];
  sys_part_3 += 28*c7p6*c9p2*ss[60];
  sys_part_3 += 8*c7p7*c9p1*ss[58];
  sys_part_3 += c1p8*ss[8];
  sys_part_3 += c3p8*ss[24];
  sys_part_3 += c5p8*ss[40];
  sys_part_3 += c7p8*ss[56];
  sys_part_3 += c9p8*ss[72];
  sys_part_3 += -tmom[8];
  T c1p9 = c1p8*c1p1;
  T c1p10 = c1p9*c1p1;
  T c3p9 = c3p8*c3p1;
  T c3p10 = c3p9*c3p1;
  T c5p9 = c5p8*c5p1;
  T c5p10 = c5p9*c5p1;
  T c7p9 = c7p8*c7p1;
  T c7p10 = c7p9*c7p1;
  T c9p9 = c9p8*c9p1;
  T c9p10 = c9p9*c9p1;
  TACC sys_part_4 = convert<TACC>(5040*c1p1*c3p1*c5p1*c7p1*c9p6*ss[70]);
  sys_part_4 += 15120*c1p1*c3p1*c5p1*c7p2*c9p5*ss[68];
  sys_part_4 += 25200*c1p1*c3p1*c5p1*c7p3*c9p4*ss[66];
  sys_part_4 += 25200*c1p1*c3p1*c5p1*c7p4*c9p3*ss[64];
  sys_part_4 += 15120*c1p1*c3p1*c5p1*c7p5*c9p2*ss[62];
  sys_part_4 += 5040*c1p1*c3p1*c5p1*c7p6*c9p1*ss[60];
  sys_part_4 += 15120*c1p1*c3p1*c5p2*c7p1*c9p5*ss[66];
  sys_part_4 += 37800*c1p1*c3p1*c5p2*c7p2*c9p4*ss[64];
  sys_part_4 += 50400*c1p1*c3p1*c5p2*c7p3*c9p3*ss[62];
  sys_part_4 += 37800*c1p1*c3p1*c5p2*c7p4*c9p2*ss[60];
  sys_part_4 += 15120*c1p1*c3p1*c5p2*c7p5*c9p1*ss[58];
  sys_part_4 += 25200*c1p1*c3p1*c5p3*c7p1*c9p4*ss[62];
  sys_part_4 += 50400*c1p1*c3p1*c5p3*c7p2*c9p3*ss[60];
  sys_part_4 += 50400*c1p1*c3p1*c5p3*c7p3*c9p2*ss[58];
  sys_part_4 += 25200*c1p1*c3p1*c5p3*c7p4*c9p1*ss[56];
  sys_part_4 += 25200*c1p1*c3p1*c5p4*c7p1*c9p3*ss[58];
  sys_part_4 += 37800*c1p1*c3p1*c5p4*c7p2*c9p2*ss[56];
  sys_part_4 += 25200*c1p1*c3p1*c5p4*c7p3*c9p1*ss[54];
  sys_part_4 += 15120*c1p1*c3p1*c5p5*c7p1*c9p2*ss[54];
  sys_part_4 += 15120*c1p1*c3p1*c5p5*c7p2*c9p1*ss[52];
  sys_part_4 += 5040*c1p1*c3p1*c5p6*c7p1*c9p1*ss[50];
  sys_part_4 += 15120*c1p1*c3p2*c5p1*c7p1*c9p5*ss[64];
  sys_part_4 += 37800*c1p1*c3p2*c5p1*c7p2*c9p4*ss[62];
  sys_part_4 += 50400*c1p1*c3p2*c5p1*c7p3*c9p3*ss[60];
  sys_part_4 += 37800*c1p1*c3p2*c5p1*c7p4*c9p2*ss[58];
  sys_part_4 += 15120*c1p1*c3p2*c5p1*c7p5*c9p1*ss[56];
  sys_part_4 += 37800*c1p1*c3p2*c5p2*c7p1*c9p4*ss[60];
  sys_part_4 += 75600*c1p1*c3p2*c5p2*c7p2*c9p3*ss[58];
  sys_part_4 += 75600*c1p1*c3p2*c5p2*c7p3*c9p2*ss[56];
  sys_part_4 += 37800*c1p1*c3p2*c5p2*c7p4*c9p1*ss[54];
  sys_part_4 += 50400*c1p1*c3p2*c5p3*c7p1*c9p3*ss[56];
  sys_part_4 += 75600*c1p1*c3p2*c5p3*c7p2*c9p2*ss[54];
  sys_part_4 += 50400*c1p1*c3p2*c5p3*c7p3*c9p1*ss[52];
  sys_part_4 += 37800*c1p1*c3p2*c5p4*c7p1*c9p2*ss[52];
  sys_part_4 += 37800*c1p1*c3p2*c5p4*c7p2*c9p1*ss[50];
  sys_part_4 += 15120*c1p1*c3p2*c5p5*c7p1*c9p1*ss[48];
  sys_part_4 += 25200*c1p1*c3p3*c5p1*c7p1*c9p4*ss[58];
  sys_part_4 += 50400*c1p1*c3p3*c5p1*c7p2*c9p3*ss[56];
  sys_part_4 += 50400*c1p1*c3p3*c5p1*c7p3*c9p2*ss[54];
  sys_part_4 += 25200*c1p1*c3p3*c5p1*c7p4*c9p1*ss[52];
  sys_part_4 += 50400*c1p1*c3p3*c5p2*c7p1*c9p3*ss[54];
  sys_part_4 += 75600*c1p1*c3p3*c5p2*c7p2*c9p2*ss[52];
  sys_part_4 += 50400*c1p1*c3p3*c5p2*c7p3*c9p1*ss[50];
  sys_part_4 += 50400*c1p1*c3p3*c5p3*c7p1*c9p2*ss[50];
  sys_part_4 += 50400*c1p1*c3p3*c5p3*c7p2*c9p1*ss[48];
  sys_part_4 += 25200*c1p1*c3p3*c5p4*c7p1*c9p1*ss[46];
  sys_part_4 += 25200*c1p1*c3p4*c5p1*c7p1*c9p3*ss[52];
  sys_part_4 += 37800*c1p1*c3p4*c5p1*c7p2*c9p2*ss[50];
  sys_part_4 += 25200*c1p1*c3p4*c5p1*c7p3*c9p1*ss[48];
  sys_part_4 += 37800*c1p1*c3p4*c5p2*c7p1*c9p2*ss[48];
  sys_part_4 += 37800*c1p1*c3p4*c5p2*c7p2*c9p1*ss[46];
  sys_part_4 += 25200*c1p1*c3p4*c5p3*c7p1*c9p1*ss[44];
  sys_part_4 += 15120*c1p1*c3p5*c5p1*c7p1*c9p2*ss[46];
  sys_part_4 += 15120*c1p1*c3p5*c5p1*c7p2*c9p1*ss[44];
  sys_part_4 += 15120*c1p1*c3p5*c5p2*c7p1*c9p1*ss[42];
  sys_part_4 += 5040*c1p1*c3p6*c5p1*c7p1*c9p1*ss[40];
  sys_part_4 += 15120*c1p2*c3p1*c5p1*c7p1*c9p5*ss[62];
  sys_part_4 += 37800*c1p2*c3p1*c5p1*c7p2*c9p4*ss[60];
  sys_part_4 += 50400*c1p2*c3p1*c5p1*c7p3*c9p3*ss[58];
  sys_part_4 += 37800*c1p2*c3p1*c5p1*c7p4*c9p2*ss[56];
  sys_part_4 += 15120*c1p2*c3p1*c5p1*c7p5*c9p1*ss[54];
  sys_part_4 += 37800*c1p2*c3p1*c5p2*c7p1*c9p4*ss[58];
  sys_part_4 += 75600*c1p2*c3p1*c5p2*c7p2*c9p3*ss[56];
  sys_part_4 += 75600*c1p2*c3p1*c5p2*c7p3*c9p2*ss[54];
  sys_part_4 += 37800*c1p2*c3p1*c5p2*c7p4*c9p1*ss[52];
  sys_part_4 += 50400*c1p2*c3p1*c5p3*c7p1*c9p3*ss[54];
  sys_part_4 += 75600*c1p2*c3p1*c5p3*c7p2*c9p2*ss[52];
  sys_part_4 += 50400*c1p2*c3p1*c5p3*c7p3*c9p1*ss[50];
  sys_part_4 += 37800*c1p2*c3p1*c5p4*c7p1*c9p2*ss[50];
  sys_part_4 += 37800*c1p2*c3p1*c5p4*c7p2*c9p1*ss[48];
  sys_part_4 += 15120*c1p2*c3p1*c5p5*c7p1*c9p1*ss[46];
  sys_part_4 += 37800*c1p2*c3p2*c5p1*c7p1*c9p4*ss[56];
  sys_part_4 += 75600*c1p2*c3p2*c5p1*c7p2*c9p3*ss[54];
  sys_part_4 += 75600*c1p2*c3p2*c5p1*c7p3*c9p2*ss[52];
  sys_part_4 += 37800*c1p2*c3p2*c5p1*c7p4*c9p1*ss[50];
  sys_part_4 += 75600*c1p2*c3p2*c5p2*c7p1*c9p3*ss[52];
  sys_part_4 += 113400*c1p2*c3p2*c5p2*c7p2*c9p2*ss[50];
  sys_part_4 += 75600*c1p2*c3p2*c5p2*c7p3*c9p1*ss[48];
  sys_part_4 += 75600*c1p2*c3p2*c5p3*c7p1*c9p2*ss[48];
  sys_part_4 += 75600*c1p2*c3p2*c5p3*c7p2*c9p1*ss[46];
  sys_part_4 += 37800*c1p2*c3p2*c5p4*c7p1*c9p1*ss[44];
  sys_part_4 += 50400*c1p2*c3p3*c5p1*c7p1*c9p3*ss[50];
  sys_part_4 += 75600*c1p2*c3p3*c5p1*c7p2*c9p2*ss[48];
  sys_part_4 += 50400*c1p2*c3p3*c5p1*c7p3*c9p1*ss[46];
  sys_part_4 += 75600*c1p2*c3p3*c5p2*c7p1*c9p2*ss[46];
  sys_part_4 += 75600*c1p2*c3p3*c5p2*c7p2*c9p1*ss[44];
  sys_part_4 += 50400*c1p2*c3p3*c5p3*c7p1*c9p1*ss[42];
  sys_part_4 += 37800*c1p2*c3p4*c5p1*c7p1*c9p2*ss[44];
  sys_part_4 += 37800*c1p2*c3p4*c5p1*c7p2*c9p1*ss[42];
  sys_part_4 += 37800*c1p2*c3p4*c5p2*c7p1*c9p1*ss[40];
  sys_part_4 += 15120*c1p2*c3p5*c5p1*c7p1*c9p1*ss[38];
  sys_part_4 += 25200*c1p3*c3p1*c5p1*c7p1*c9p4*ss[54];
  sys_part_4 += 50400*c1p3*c3p1*c5p1*c7p2*c9p3*ss[52];
  sys_part_4 += 50400*c1p3*c3p1*c5p1*c7p3*c9p2*ss[50];
  sys_part_4 += 25200*c1p3*c3p1*c5p1*c7p4*c9p1*ss[48];
  sys_part_4 += 50400*c1p3*c3p1*c5p2*c7p1*c9p3*ss[50];
  sys_part_4 += 75600*c1p3*c3p1*c5p2*c7p2*c9p2*ss[48];
  sys_part_4 += 50400*c1p3*c3p1*c5p2*c7p3*c9p1*ss[46];
  sys_part_4 += 50400*c1p3*c3p1*c5p3*c7p1*c9p2*ss[46];
  sys_part_4 += 50400*c1p3*c3p1*c5p3*c7p2*c9p1*ss[44];
  sys_part_4 += 25200*c1p3*c3p1*c5p4*c7p1*c9p1*ss[42];
  sys_part_4 += 50400*c1p3*c3p2*c5p1*c7p1*c9p3*ss[48];
  sys_part_4 += 75600*c1p3*c3p2*c5p1*c7p2*c9p2*ss[46];
  sys_part_4 += 50400*c1p3*c3p2*c5p1*c7p3*c9p1*ss[44];
  sys_part_4 += 75600*c1p3*c3p2*c5p2*c7p1*c9p2*ss[44];
  sys_part_4 += 75600*c1p3*c3p2*c5p2*c7p2*c9p1*ss[42];
  sys_part_4 += 50400*c1p3*c3p2*c5p3*c7p1*c9p1*ss[40];
  sys_part_4 += 50400*c1p3*c3p3*c5p1*c7p1*c9p2*ss[42];
  sys_part_4 += 50400*c1p3*c3p3*c5p1*c7p2*c9p1*ss[40];
  sys_part_4 += 50400*c1p3*c3p3*c5p2*c7p1*c9p1*ss[38];
  sys_part_4 += 25200*c1p3*c3p4*c5p1*c7p1*c9p1*ss[36];
  sys_part_4 += 25200*c1p4*c3p1*c5p1*c7p1*c9p3*ss[46];
  sys_part_4 += 37800*c1p4*c3p1*c5p1*c7p2*c9p2*ss[44];
  sys_part_4 += 25200*c1p4*c3p1*c5p1*c7p3*c9p1*ss[42];
  sys_part_4 += 37800*c1p4*c3p1*c5p2*c7p1*c9p2*ss[42];
  sys_part_4 += 37800*c1p4*c3p1*c5p2*c7p2*c9p1*ss[40];
  sys_part_4 += 25200*c1p4*c3p1*c5p3*c7p1*c9p1*ss[38];
  sys_part_4 += 37800*c1p4*c3p2*c5p1*c7p1*c9p2*ss[40];
  sys_part_4 += 37800*c1p4*c3p2*c5p1*c7p2*c9p1*ss[38];
  sys_part_4 += 37800*c1p4*c3p2*c5p2*c7p1*c9p1*ss[36];
  sys_part_4 += 25200*c1p4*c3p3*c5p1*c7p1*c9p1*ss[34];
  sys_part_4 += 15120*c1p5*c3p1*c5p1*c7p1*c9p2*ss[38];
  sys_part_4 += 15120*c1p5*c3p1*c5p1*c7p2*c9p1*ss[36];
  sys_part_4 += 15120*c1p5*c3p1*c5p2*c7p1*c9p1*ss[34];
  sys_part_4 += 15120*c1p5*c3p2*c5p1*c7p1*c9p1*ss[32];
  sys_part_4 += 5040*c1p6*c3p1*c5p1*c7p1*c9p1*ss[30];
  sys_part_4 += 720*c1p1*c3p1*c5p1*c7p7*ss[58];
  sys_part_4 += 720*c1p1*c3p1*c5p1*c9p7*ss[72];
  sys_part_4 += 2520*c1p1*c3p1*c5p2*c7p6*ss[56];
  sys_part_4 += 2520*c1p1*c3p1*c5p2*c9p6*ss[68];
  sys_part_4 += 5040*c1p1*c3p1*c5p3*c7p5*ss[54];
  sys_part_4 += 5040*c1p1*c3p1*c5p3*c9p5*ss[64];
  sys_part_4 += 6300*c1p1*c3p1*c5p4*c7p4*ss[52];
  sys_part_4 += 6300*c1p1*c3p1*c5p4*c9p4*ss[60];
  sys_part_4 += 5040*c1p1*c3p1*c5p5*c7p3*ss[50];
  sys_part_4 += 5040*c1p1*c3p1*c5p5*c9p3*ss[56];
  sys_part_4 += 2520*c1p1*c3p1*c5p6*c7p2*ss[48];
  sys_part_4 += 2520*c1p1*c3p1*c5p6*c9p2*ss[52];
  sys_part_4 += 720*c1p1*c3p1*c5p7*c7p1*ss[46];
  sys_part_4 += 720*c1p1*c3p1*c5p7*c9p1*ss[48];
  sys_part_4 += 720*c1p1*c3p1*c7p1*c9p7*ss[74];
  sys_part_4 += 2520*c1p1*c3p1*c7p2*c9p6*ss[72];
  sys_part_4 += 5040*c1p1*c3p1*c7p3*c9p5*ss[70];
  sys_part_4 += 6300*c1p1*c3p1*c7p4*c9p4*ss[68];
  sys_part_4 += 5040*c1p1*c3p1*c7p5*c9p3*ss[66];
  sys_part_4 += 2520*c1p1*c3p1*c7p6*c9p2*ss[64];
  sys_part_4 += 720*c1p1*c3p1*c7p7*c9p1*ss[62];
  sys_part_4 += 2520*c1p1*c3p2*c5p1*c7p6*ss[54];
  sys_part_4 += 2520*c1p1*c3p2*c5p1*c9p6*ss[66];
  sys_part_4 += 7560*c1p1*c3p2*c5p2*c7p5*ss[52];
  sys_part_4 += 7560*c1p1*c3p2*c5p2*c9p5*ss[62];
  sys_part_4 += 12600*c1p1*c3p2*c5p3*c7p4*ss[50];
  sys_part_4 += 12600*c1p1*c3p2*c5p3*c9p4*ss[58];
  sys_part_4 += 12600*c1p1*c3p2*c5p4*c7p3*ss[48];
  sys_part_4 += 12600*c1p1*c3p2*c5p4*c9p3*ss[54];
  sys_part_4 += 7560*c1p1*c3p2*c5p5*c7p2*ss[46];
  sys_part_4 += 7560*c1p1*c3p2*c5p5*c9p2*ss[50];
  sys_part_4 += 2520*c1p1*c3p2*c5p6*c7p1*ss[44];
  sys_part_4 += 2520*c1p1*c3p2*c5p6*c9p1*ss[46];
  sys_part_4 += 2520*c1p1*c3p2*c7p1*c9p6*ss[68];
  sys_part_4 += 7560*c1p1*c3p2*c7p2*c9p5*ss[66];
  sys_part_4 += 12600*c1p1*c3p2*c7p3*c9p4*ss[64];
  sys_part_4 += 12600*c1p1*c3p2*c7p4*c9p3*ss[62];
  sys_part_4 += 7560*c1p1*c3p2*c7p5*c9p2*ss[60];
  sys_part_4 += 2520*c1p1*c3p2*c7p6*c9p1*ss[58];
  sys_part_4 += 5040*c1p1*c3p3*c5p1*c7p5*ss[50];
  sys_part_4 += 5040*c1p1*c3p3*c5p1*c9p5*ss[60];
  sys_part_4 += 12600*c1p1*c3p3*c5p2*c7p4*ss[48];
  sys_part_4 += 12600*c1p1*c3p3*c5p2*c9p4*ss[56];
  sys_part_4 += 16800*c1p1*c3p3*c5p3*c7p3*ss[46];
  sys_part_4 += 16800*c1p1*c3p3*c5p3*c9p3*ss[52];
  sys_part_4 += 12600*c1p1*c3p3*c5p4*c7p2*ss[44];
  sys_part_4 += 12600*c1p1*c3p3*c5p4*c9p2*ss[48];
  sys_part_4 += 5040*c1p1*c3p3*c5p5*c7p1*ss[42];
  sys_part_4 += 5040*c1p1*c3p3*c5p5*c9p1*ss[44];
  sys_part_4 += 5040*c1p1*c3p3*c7p1*c9p5*ss[62];
  sys_part_4 += 12600*c1p1*c3p3*c7p2*c9p4*ss[60];
  sys_part_4 += 16800*c1p1*c3p3*c7p3*c9p3*ss[58];
  sys_part_4 += 12600*c1p1*c3p3*c7p4*c9p2*ss[56];
  sys_part_4 += 5040*c1p1*c3p3*c7p5*c9p1*ss[54];
  sys_part_4 += 6300*c1p1*c3p4*c5p1*c7p4*ss[46];
  sys_part_4 += 6300*c1p1*c3p4*c5p1*c9p4*ss[54];
  sys_part_4 += 12600*c1p1*c3p4*c5p2*c7p3*ss[44];
  sys_part_4 += 12600*c1p1*c3p4*c5p2*c9p3*ss[50];
  sys_part_4 += 12600*c1p1*c3p4*c5p3*c7p2*ss[42];
  sys_part_4 += 12600*c1p1*c3p4*c5p3*c9p2*ss[46];
  sys_part_4 += 6300*c1p1*c3p4*c5p4*c7p1*ss[40];
  sys_part_4 += 6300*c1p1*c3p4*c5p4*c9p1*ss[42];
  sys_part_4 += 6300*c1p1*c3p4*c7p1*c9p4*ss[56];
  sys_part_4 += 12600*c1p1*c3p4*c7p2*c9p3*ss[54];
  sys_part_4 += 12600*c1p1*c3p4*c7p3*c9p2*ss[52];
  sys_part_4 += 6300*c1p1*c3p4*c7p4*c9p1*ss[50];
  sys_part_4 += 5040*c1p1*c3p5*c5p1*c7p3*ss[42];
  sys_part_4 += 5040*c1p1*c3p5*c5p1*c9p3*ss[48];
  sys_part_4 += 7560*c1p1*c3p5*c5p2*c7p2*ss[40];
  sys_part_4 += 7560*c1p1*c3p5*c5p2*c9p2*ss[44];
  sys_part_4 += 5040*c1p1*c3p5*c5p3*c7p1*ss[38];
  sys_part_4 += 5040*c1p1*c3p5*c5p3*c9p1*ss[40];
  sys_part_4 += 5040*c1p1*c3p5*c7p1*c9p3*ss[50];
  sys_part_4 += 7560*c1p1*c3p5*c7p2*c9p2*ss[48];
  sys_part_4 += 5040*c1p1*c3p5*c7p3*c9p1*ss[46];
  sys_part_4 += 2520*c1p1*c3p6*c5p1*c7p2*ss[38];
  sys_part_4 += 2520*c1p1*c3p6*c5p1*c9p2*ss[42];
  sys_part_4 += 2520*c1p1*c3p6*c5p2*c7p1*ss[36];
  sys_part_4 += 2520*c1p1*c3p6*c5p2*c9p1*ss[38];
  sys_part_4 += 2520*c1p1*c3p6*c7p1*c9p2*ss[44];
  sys_part_4 += 2520*c1p1*c3p6*c7p2*c9p1*ss[42];
  sys_part_4 += 720*c1p1*c3p7*c5p1*c7p1*ss[34];
  sys_part_4 += 720*c1p1*c3p7*c5p1*c9p1*ss[36];
  sys_part_4 += 720*c1p1*c3p7*c7p1*c9p1*ss[38];
  sys_part_4 += 720*c1p1*c5p1*c7p1*c9p7*ss[76];
  sys_part_4 += 2520*c1p1*c5p1*c7p2*c9p6*ss[74];
  sys_part_4 += 5040*c1p1*c5p1*c7p3*c9p5*ss[72];
  sys_part_4 += 6300*c1p1*c5p1*c7p4*c9p4*ss[70];
  sys_part_4 += 5040*c1p1*c5p1*c7p5*c9p3*ss[68];
  sys_part_4 += 2520*c1p1*c5p1*c7p6*c9p2*ss[66];
  sys_part_4 += 720*c1p1*c5p1*c7p7*c9p1*ss[64];
  sys_part_4 += 2520*c1p1*c5p2*c7p1*c9p6*ss[72];
  sys_part_4 += 7560*c1p1*c5p2*c7p2*c9p5*ss[70];
  sys_part_4 += 12600*c1p1*c5p2*c7p3*c9p4*ss[68];
  sys_part_4 += 12600*c1p1*c5p2*c7p4*c9p3*ss[66];
  sys_part_4 += 7560*c1p1*c5p2*c7p5*c9p2*ss[64];
  sys_part_4 += 2520*c1p1*c5p2*c7p6*c9p1*ss[62];
  sys_part_4 += 5040*c1p1*c5p3*c7p1*c9p5*ss[68];
  sys_part_4 += 12600*c1p1*c5p3*c7p2*c9p4*ss[66];
  sys_part_4 += 16800*c1p1*c5p3*c7p3*c9p3*ss[64];
  sys_part_4 += 12600*c1p1*c5p3*c7p4*c9p2*ss[62];
  sys_part_4 += 5040*c1p1*c5p3*c7p5*c9p1*ss[60];
  sys_part_4 += 6300*c1p1*c5p4*c7p1*c9p4*ss[64];
  sys_part_4 += 12600*c1p1*c5p4*c7p2*c9p3*ss[62];
  sys_part_4 += 12600*c1p1*c5p4*c7p3*c9p2*ss[60];
  sys_part_4 += 6300*c1p1*c5p4*c7p4*c9p1*ss[58];
  sys_part_4 += 5040*c1p1*c5p5*c7p1*c9p3*ss[60];
  sys_part_4 += 7560*c1p1*c5p5*c7p2*c9p2*ss[58];
  sys_part_4 += 5040*c1p1*c5p5*c7p3*c9p1*ss[56];
  sys_part_4 += 2520*c1p1*c5p6*c7p1*c9p2*ss[56];
  sys_part_4 += 2520*c1p1*c5p6*c7p2*c9p1*ss[54];
  sys_part_4 += 720*c1p1*c5p7*c7p1*c9p1*ss[52];
  sys_part_4 += 2520*c1p2*c3p1*c5p1*c7p6*ss[52];
  sys_part_4 += 2520*c1p2*c3p1*c5p1*c9p6*ss[64];
  sys_part_4 += 7560*c1p2*c3p1*c5p2*c7p5*ss[50];
  sys_part_4 += 7560*c1p2*c3p1*c5p2*c9p5*ss[60];
  sys_part_4 += 12600*c1p2*c3p1*c5p3*c7p4*ss[48];
  sys_part_4 += 12600*c1p2*c3p1*c5p3*c9p4*ss[56];
  sys_part_4 += 12600*c1p2*c3p1*c5p4*c7p3*ss[46];
  sys_part_4 += 12600*c1p2*c3p1*c5p4*c9p3*ss[52];
  sys_part_4 += 7560*c1p2*c3p1*c5p5*c7p2*ss[44];
  sys_part_4 += 7560*c1p2*c3p1*c5p5*c9p2*ss[48];
  sys_part_4 += 2520*c1p2*c3p1*c5p6*c7p1*ss[42];
  sys_part_4 += 2520*c1p2*c3p1*c5p6*c9p1*ss[44];
  sys_part_4 += 2520*c1p2*c3p1*c7p1*c9p6*ss[66];
  sys_part_4 += 7560*c1p2*c3p1*c7p2*c9p5*ss[64];
  sys_part_4 += 12600*c1p2*c3p1*c7p3*c9p4*ss[62];
  sys_part_4 += 12600*c1p2*c3p1*c7p4*c9p3*ss[60];
  sys_part_4 += 7560*c1p2*c3p1*c7p5*c9p2*ss[58];
  sys_part_4 += 2520*c1p2*c3p1*c7p6*c9p1*ss[56];
  sys_part_4 += 7560*c1p2*c3p2*c5p1*c7p5*ss[48];
  sys_part_4 += 7560*c1p2*c3p2*c5p1*c9p5*ss[58];
  sys_part_4 += 18900*c1p2*c3p2*c5p2*c7p4*ss[46];
  sys_part_4 += 18900*c1p2*c3p2*c5p2*c9p4*ss[54];
  sys_part_4 += 25200*c1p2*c3p2*c5p3*c7p3*ss[44];
  sys_part_4 += 25200*c1p2*c3p2*c5p3*c9p3*ss[50];
  sys_part_4 += 18900*c1p2*c3p2*c5p4*c7p2*ss[42];
  sys_part_4 += 18900*c1p2*c3p2*c5p4*c9p2*ss[46];
  sys_part_4 += 7560*c1p2*c3p2*c5p5*c7p1*ss[40];
  sys_part_4 += 7560*c1p2*c3p2*c5p5*c9p1*ss[42];
  sys_part_4 += 7560*c1p2*c3p2*c7p1*c9p5*ss[60];
  sys_part_4 += 18900*c1p2*c3p2*c7p2*c9p4*ss[58];
  sys_part_4 += 25200*c1p2*c3p2*c7p3*c9p3*ss[56];
  sys_part_4 += 18900*c1p2*c3p2*c7p4*c9p2*ss[54];
  sys_part_4 += 7560*c1p2*c3p2*c7p5*c9p1*ss[52];
  sys_part_4 += 12600*c1p2*c3p3*c5p1*c7p4*ss[44];
  sys_part_4 += 12600*c1p2*c3p3*c5p1*c9p4*ss[52];
  sys_part_4 += 25200*c1p2*c3p3*c5p2*c7p3*ss[42];
  sys_part_4 += 25200*c1p2*c3p3*c5p2*c9p3*ss[48];
  sys_part_4 += 25200*c1p2*c3p3*c5p3*c7p2*ss[40];
  sys_part_4 += 25200*c1p2*c3p3*c5p3*c9p2*ss[44];
  sys_part_4 += 12600*c1p2*c3p3*c5p4*c7p1*ss[38];
  sys_part_4 += 12600*c1p2*c3p3*c5p4*c9p1*ss[40];
  sys_part_4 += 12600*c1p2*c3p3*c7p1*c9p4*ss[54];
  sys_part_4 += 25200*c1p2*c3p3*c7p2*c9p3*ss[52];
  sys_part_4 += 25200*c1p2*c3p3*c7p3*c9p2*ss[50];
  sys_part_4 += 12600*c1p2*c3p3*c7p4*c9p1*ss[48];
  sys_part_4 += 12600*c1p2*c3p4*c5p1*c7p3*ss[40];
  sys_part_4 += 12600*c1p2*c3p4*c5p1*c9p3*ss[46];
  sys_part_4 += 18900*c1p2*c3p4*c5p2*c7p2*ss[38];
  sys_part_4 += 18900*c1p2*c3p4*c5p2*c9p2*ss[42];
  sys_part_4 += 12600*c1p2*c3p4*c5p3*c7p1*ss[36];
  sys_part_4 += 12600*c1p2*c3p4*c5p3*c9p1*ss[38];
  sys_part_4 += 12600*c1p2*c3p4*c7p1*c9p3*ss[48];
  sys_part_4 += 18900*c1p2*c3p4*c7p2*c9p2*ss[46];
  sys_part_4 += 12600*c1p2*c3p4*c7p3*c9p1*ss[44];
  sys_part_4 += 7560*c1p2*c3p5*c5p1*c7p2*ss[36];
  sys_part_4 += 7560*c1p2*c3p5*c5p1*c9p2*ss[40];
  sys_part_4 += 7560*c1p2*c3p5*c5p2*c7p1*ss[34];
  sys_part_4 += 7560*c1p2*c3p5*c5p2*c9p1*ss[36];
  sys_part_4 += 7560*c1p2*c3p5*c7p1*c9p2*ss[42];
  sys_part_4 += 7560*c1p2*c3p5*c7p2*c9p1*ss[40];
  sys_part_4 += 2520*c1p2*c3p6*c5p1*c7p1*ss[32];
  sys_part_4 += 2520*c1p2*c3p6*c5p1*c9p1*ss[34];
  sys_part_4 += 2520*c1p2*c3p6*c7p1*c9p1*ss[36];
  sys_part_4 += 2520*c1p2*c5p1*c7p1*c9p6*ss[68];
  sys_part_4 += 7560*c1p2*c5p1*c7p2*c9p5*ss[66];
  sys_part_4 += 12600*c1p2*c5p1*c7p3*c9p4*ss[64];
  sys_part_4 += 12600*c1p2*c5p1*c7p4*c9p3*ss[62];
  sys_part_4 += 7560*c1p2*c5p1*c7p5*c9p2*ss[60];
  sys_part_4 += 2520*c1p2*c5p1*c7p6*c9p1*ss[58];
  sys_part_4 += 7560*c1p2*c5p2*c7p1*c9p5*ss[64];
  sys_part_4 += 18900*c1p2*c5p2*c7p2*c9p4*ss[62];
  sys_part_4 += 25200*c1p2*c5p2*c7p3*c9p3*ss[60];
  sys_part_4 += 18900*c1p2*c5p2*c7p4*c9p2*ss[58];
  sys_part_4 += 7560*c1p2*c5p2*c7p5*c9p1*ss[56];
  sys_part_4 += 12600*c1p2*c5p3*c7p1*c9p4*ss[60];
  sys_part_4 += 25200*c1p2*c5p3*c7p2*c9p3*ss[58];
  sys_part_4 += 25200*c1p2*c5p3*c7p3*c9p2*ss[56];
  sys_part_4 += 12600*c1p2*c5p3*c7p4*c9p1*ss[54];
  sys_part_4 += 12600*c1p2*c5p4*c7p1*c9p3*ss[56];
  sys_part_4 += 18900*c1p2*c5p4*c7p2*c9p2*ss[54];
  sys_part_4 += 12600*c1p2*c5p4*c7p3*c9p1*ss[52];
  sys_part_4 += 7560*c1p2*c5p5*c7p1*c9p2*ss[52];
  sys_part_4 += 7560*c1p2*c5p5*c7p2*c9p1*ss[50];
  sys_part_4 += 2520*c1p2*c5p6*c7p1*c9p1*ss[48];
  sys_part_4 += 5040*c1p3*c3p1*c5p1*c7p5*ss[46];
  sys_part_4 += 5040*c1p3*c3p1*c5p1*c9p5*ss[56];
  sys_part_4 += 12600*c1p3*c3p1*c5p2*c7p4*ss[44];
  sys_part_4 += 12600*c1p3*c3p1*c5p2*c9p4*ss[52];
  sys_part_4 += 16800*c1p3*c3p1*c5p3*c7p3*ss[42];
  sys_part_4 += 16800*c1p3*c3p1*c5p3*c9p3*ss[48];
  sys_part_4 += 12600*c1p3*c3p1*c5p4*c7p2*ss[40];
  sys_part_4 += 12600*c1p3*c3p1*c5p4*c9p2*ss[44];
  sys_part_4 += 5040*c1p3*c3p1*c5p5*c7p1*ss[38];
  sys_part_4 += 5040*c1p3*c3p1*c5p5*c9p1*ss[40];
  sys_part_4 += 5040*c1p3*c3p1*c7p1*c9p5*ss[58];
  sys_part_4 += 12600*c1p3*c3p1*c7p2*c9p4*ss[56];
  sys_part_4 += 16800*c1p3*c3p1*c7p3*c9p3*ss[54];
  sys_part_4 += 12600*c1p3*c3p1*c7p4*c9p2*ss[52];
  sys_part_4 += 5040*c1p3*c3p1*c7p5*c9p1*ss[50];
  sys_part_4 += 12600*c1p3*c3p2*c5p1*c7p4*ss[42];
  sys_part_4 += 12600*c1p3*c3p2*c5p1*c9p4*ss[50];
  sys_part_4 += 25200*c1p3*c3p2*c5p2*c7p3*ss[40];
  sys_part_4 += 25200*c1p3*c3p2*c5p2*c9p3*ss[46];
  sys_part_4 += 25200*c1p3*c3p2*c5p3*c7p2*ss[38];
  sys_part_4 += 25200*c1p3*c3p2*c5p3*c9p2*ss[42];
  sys_part_4 += 12600*c1p3*c3p2*c5p4*c7p1*ss[36];
  sys_part_4 += 12600*c1p3*c3p2*c5p4*c9p1*ss[38];
  sys_part_4 += 12600*c1p3*c3p2*c7p1*c9p4*ss[52];
  sys_part_4 += 25200*c1p3*c3p2*c7p2*c9p3*ss[50];
  sys_part_4 += 25200*c1p3*c3p2*c7p3*c9p2*ss[48];
  sys_part_4 += 12600*c1p3*c3p2*c7p4*c9p1*ss[46];
  sys_part_4 += 16800*c1p3*c3p3*c5p1*c7p3*ss[38];
  sys_part_4 += 16800*c1p3*c3p3*c5p1*c9p3*ss[44];
  sys_part_4 += 25200*c1p3*c3p3*c5p2*c7p2*ss[36];
  sys_part_4 += 25200*c1p3*c3p3*c5p2*c9p2*ss[40];
  sys_part_4 += 16800*c1p3*c3p3*c5p3*c7p1*ss[34];
  sys_part_4 += 16800*c1p3*c3p3*c5p3*c9p1*ss[36];
  sys_part_4 += 16800*c1p3*c3p3*c7p1*c9p3*ss[46];
  sys_part_4 += 25200*c1p3*c3p3*c7p2*c9p2*ss[44];
  sys_part_4 += 16800*c1p3*c3p3*c7p3*c9p1*ss[42];
  sys_part_4 += 12600*c1p3*c3p4*c5p1*c7p2*ss[34];
  sys_part_4 += 12600*c1p3*c3p4*c5p1*c9p2*ss[38];
  sys_part_4 += 12600*c1p3*c3p4*c5p2*c7p1*ss[32];
  sys_part_4 += 12600*c1p3*c3p4*c5p2*c9p1*ss[34];
  sys_part_4 += 12600*c1p3*c3p4*c7p1*c9p2*ss[40];
  sys_part_4 += 12600*c1p3*c3p4*c7p2*c9p1*ss[38];
  sys_part_4 += 5040*c1p3*c3p5*c5p1*c7p1*ss[30];
  sys_part_4 += 5040*c1p3*c3p5*c5p1*c9p1*ss[32];
  sys_part_4 += 5040*c1p3*c3p5*c7p1*c9p1*ss[34];
  sys_part_4 += 5040*c1p3*c5p1*c7p1*c9p5*ss[60];
  sys_part_4 += 12600*c1p3*c5p1*c7p2*c9p4*ss[58];
  sys_part_4 += 16800*c1p3*c5p1*c7p3*c9p3*ss[56];
  sys_part_4 += 12600*c1p3*c5p1*c7p4*c9p2*ss[54];
  sys_part_4 += 5040*c1p3*c5p1*c7p5*c9p1*ss[52];
  sys_part_4 += 12600*c1p3*c5p2*c7p1*c9p4*ss[56];
  sys_part_4 += 25200*c1p3*c5p2*c7p2*c9p3*ss[54];
  sys_part_4 += 25200*c1p3*c5p2*c7p3*c9p2*ss[52];
  sys_part_4 += 12600*c1p3*c5p2*c7p4*c9p1*ss[50];
  sys_part_4 += 16800*c1p3*c5p3*c7p1*c9p3*ss[52];
  sys_part_4 += 25200*c1p3*c5p3*c7p2*c9p2*ss[50];
  sys_part_4 += 16800*c1p3*c5p3*c7p3*c9p1*ss[48];
  sys_part_4 += 12600*c1p3*c5p4*c7p1*c9p2*ss[48];
  sys_part_4 += 12600*c1p3*c5p4*c7p2*c9p1*ss[46];
  sys_part_4 += 5040*c1p3*c5p5*c7p1*c9p1*ss[44];
  sys_part_4 += 6300*c1p4*c3p1*c5p1*c7p4*ss[40];
  sys_part_4 += 6300*c1p4*c3p1*c5p1*c9p4*ss[48];
  sys_part_4 += 12600*c1p4*c3p1*c5p2*c7p3*ss[38];
  sys_part_4 += 12600*c1p4*c3p1*c5p2*c9p3*ss[44];
  sys_part_4 += 12600*c1p4*c3p1*c5p3*c7p2*ss[36];
  sys_part_4 += 12600*c1p4*c3p1*c5p3*c9p2*ss[40];
  sys_part_4 += 6300*c1p4*c3p1*c5p4*c7p1*ss[34];
  sys_part_4 += 6300*c1p4*c3p1*c5p4*c9p1*ss[36];
  sys_part_4 += 6300*c1p4*c3p1*c7p1*c9p4*ss[50];
  sys_part_4 += 12600*c1p4*c3p1*c7p2*c9p3*ss[48];
  sys_part_4 += 12600*c1p4*c3p1*c7p3*c9p2*ss[46];
  sys_part_4 += 6300*c1p4*c3p1*c7p4*c9p1*ss[44];
  sys_part_4 += 12600*c1p4*c3p2*c5p1*c7p3*ss[36];
  sys_part_4 += 12600*c1p4*c3p2*c5p1*c9p3*ss[42];
  sys_part_4 += 18900*c1p4*c3p2*c5p2*c7p2*ss[34];
  sys_part_4 += 18900*c1p4*c3p2*c5p2*c9p2*ss[38];
  sys_part_4 += 12600*c1p4*c3p2*c5p3*c7p1*ss[32];
  sys_part_4 += 12600*c1p4*c3p2*c5p3*c9p1*ss[34];
  sys_part_4 += 12600*c1p4*c3p2*c7p1*c9p3*ss[44];
  sys_part_4 += 18900*c1p4*c3p2*c7p2*c9p2*ss[42];
  sys_part_4 += 12600*c1p4*c3p2*c7p3*c9p1*ss[40];
  sys_part_4 += 12600*c1p4*c3p3*c5p1*c7p2*ss[32];
  sys_part_4 += 12600*c1p4*c3p3*c5p1*c9p2*ss[36];
  sys_part_4 += 12600*c1p4*c3p3*c5p2*c7p1*ss[30];
  sys_part_4 += 12600*c1p4*c3p3*c5p2*c9p1*ss[32];
  sys_part_4 += 12600*c1p4*c3p3*c7p1*c9p2*ss[38];
  sys_part_4 += 12600*c1p4*c3p3*c7p2*c9p1*ss[36];
  sys_part_4 += 6300*c1p4*c3p4*c5p1*c7p1*ss[28];
  sys_part_4 += 6300*c1p4*c3p4*c5p1*c9p1*ss[30];
  sys_part_4 += 6300*c1p4*c3p4*c7p1*c9p1*ss[32];
  sys_part_4 += 6300*c1p4*c5p1*c7p1*c9p4*ss[52];
  sys_part_4 += 12600*c1p4*c5p1*c7p2*c9p3*ss[50];
  sys_part_4 += 12600*c1p4*c5p1*c7p3*c9p2*ss[48];
  sys_part_4 += 6300*c1p4*c5p1*c7p4*c9p1*ss[46];
  sys_part_4 += 12600*c1p4*c5p2*c7p1*c9p3*ss[48];
  sys_part_4 += 18900*c1p4*c5p2*c7p2*c9p2*ss[46];
  sys_part_4 += 12600*c1p4*c5p2*c7p3*c9p1*ss[44];
  sys_part_4 += 12600*c1p4*c5p3*c7p1*c9p2*ss[44];
  sys_part_4 += 12600*c1p4*c5p3*c7p2*c9p1*ss[42];
  sys_part_4 += 6300*c1p4*c5p4*c7p1*c9p1*ss[40];
  sys_part_4 += 5040*c1p5*c3p1*c5p1*c7p3*ss[34];
  sys_part_4 += 5040*c1p5*c3p1*c5p1*c9p3*ss[40];
  sys_part_4 += 7560*c1p5*c3p1*c5p2*c7p2*ss[32];
  sys_part_4 += 7560*c1p5*c3p1*c5p2*c9p2*ss[36];
  sys_part_4 += 5040*c1p5*c3p1*c5p3*c7p1*ss[30];
  sys_part_4 += 5040*c1p5*c3p1*c5p3*c9p1*ss[32];
  sys_part_4 += 5040*c1p5*c3p1*c7p1*c9p3*ss[42];
  sys_part_4 += 7560*c1p5*c3p1*c7p2*c9p2*ss[40];
  sys_part_4 += 5040*c1p5*c3p1*c7p3*c9p1*ss[38];
  sys_part_4 += 7560*c1p5*c3p2*c5p1*c7p2*ss[30];
  sys_part_4 += 7560*c1p5*c3p2*c5p1*c9p2*ss[34];
  sys_part_4 += 7560*c1p5*c3p2*c5p2*c7p1*ss[28];
  sys_part_4 += 7560*c1p5*c3p2*c5p2*c9p1*ss[30];
  sys_part_4 += 7560*c1p5*c3p2*c7p1*c9p2*ss[36];
  sys_part_4 += 7560*c1p5*c3p2*c7p2*c9p1*ss[34];
  sys_part_4 += 5040*c1p5*c3p3*c5p1*c7p1*ss[26];
  sys_part_4 += 5040*c1p5*c3p3*c5p1*c9p1*ss[28];
  sys_part_4 += 5040*c1p5*c3p3*c7p1*c9p1*ss[30];
  sys_part_4 += 5040*c1p5*c5p1*c7p1*c9p3*ss[44];
  sys_part_4 += 7560*c1p5*c5p1*c7p2*c9p2*ss[42];
  sys_part_4 += 5040*c1p5*c5p1*c7p3*c9p1*ss[40];
  sys_part_4 += 7560*c1p5*c5p2*c7p1*c9p2*ss[40];
  sys_part_4 += 7560*c1p5*c5p2*c7p2*c9p1*ss[38];
  sys_part_4 += 5040*c1p5*c5p3*c7p1*c9p1*ss[36];
  sys_part_4 += 2520*c1p6*c3p1*c5p1*c7p2*ss[28];
  sys_part_4 += 2520*c1p6*c3p1*c5p1*c9p2*ss[32];
  sys_part_4 += 2520*c1p6*c3p1*c5p2*c7p1*ss[26];
  sys_part_4 += 2520*c1p6*c3p1*c5p2*c9p1*ss[28];
  sys_part_4 += 2520*c1p6*c3p1*c7p1*c9p2*ss[34];
  sys_part_4 += 2520*c1p6*c3p1*c7p2*c9p1*ss[32];
  sys_part_4 += 2520*c1p6*c3p2*c5p1*c7p1*ss[24];
  sys_part_4 += 2520*c1p6*c3p2*c5p1*c9p1*ss[26];
  sys_part_4 += 2520*c1p6*c3p2*c7p1*c9p1*ss[28];
  sys_part_4 += 2520*c1p6*c5p1*c7p1*c9p2*ss[36];
  sys_part_4 += 2520*c1p6*c5p1*c7p2*c9p1*ss[34];
  sys_part_4 += 2520*c1p6*c5p2*c7p1*c9p1*ss[32];
  sys_part_4 += 720*c1p7*c3p1*c5p1*c7p1*ss[22];
  sys_part_4 += 720*c1p7*c3p1*c5p1*c9p1*ss[24];
  sys_part_4 += 720*c1p7*c3p1*c7p1*c9p1*ss[26];
  sys_part_4 += 720*c1p7*c5p1*c7p1*c9p1*ss[28];
  sys_part_4 += 720*c3p1*c5p1*c7p1*c9p7*ss[78];
  sys_part_4 += 2520*c3p1*c5p1*c7p2*c9p6*ss[76];
  sys_part_4 += 5040*c3p1*c5p1*c7p3*c9p5*ss[74];
  sys_part_4 += 6300*c3p1*c5p1*c7p4*c9p4*ss[72];
  sys_part_4 += 5040*c3p1*c5p1*c7p5*c9p3*ss[70];
  sys_part_4 += 2520*c3p1*c5p1*c7p6*c9p2*ss[68];
  sys_part_4 += 720*c3p1*c5p1*c7p7*c9p1*ss[66];
  sys_part_4 += 2520*c3p1*c5p2*c7p1*c9p6*ss[74];
  sys_part_4 += 7560*c3p1*c5p2*c7p2*c9p5*ss[72];
  sys_part_4 += 12600*c3p1*c5p2*c7p3*c9p4*ss[70];
  sys_part_4 += 12600*c3p1*c5p2*c7p4*c9p3*ss[68];
  sys_part_4 += 7560*c3p1*c5p2*c7p5*c9p2*ss[66];
  sys_part_4 += 2520*c3p1*c5p2*c7p6*c9p1*ss[64];
  sys_part_4 += 5040*c3p1*c5p3*c7p1*c9p5*ss[70];
  sys_part_4 += 12600*c3p1*c5p3*c7p2*c9p4*ss[68];
  sys_part_4 += 16800*c3p1*c5p3*c7p3*c9p3*ss[66];
  sys_part_4 += 12600*c3p1*c5p3*c7p4*c9p2*ss[64];
  sys_part_4 += 5040*c3p1*c5p3*c7p5*c9p1*ss[62];
  sys_part_4 += 6300*c3p1*c5p4*c7p1*c9p4*ss[66];
  sys_part_4 += 12600*c3p1*c5p4*c7p2*c9p3*ss[64];
  sys_part_4 += 12600*c3p1*c5p4*c7p3*c9p2*ss[62];
  sys_part_4 += 6300*c3p1*c5p4*c7p4*c9p1*ss[60];
  sys_part_4 += 5040*c3p1*c5p5*c7p1*c9p3*ss[62];
  sys_part_4 += 7560*c3p1*c5p5*c7p2*c9p2*ss[60];
  sys_part_4 += 5040*c3p1*c5p5*c7p3*c9p1*ss[58];
  sys_part_4 += 2520*c3p1*c5p6*c7p1*c9p2*ss[58];
  sys_part_4 += 2520*c3p1*c5p6*c7p2*c9p1*ss[56];
  sys_part_4 += 720*c3p1*c5p7*c7p1*c9p1*ss[54];
  sys_part_4 += 2520*c3p2*c5p1*c7p1*c9p6*ss[72];
  sys_part_4 += 7560*c3p2*c5p1*c7p2*c9p5*ss[70];
  sys_part_4 += 12600*c3p2*c5p1*c7p3*c9p4*ss[68];
  sys_part_4 += 12600*c3p2*c5p1*c7p4*c9p3*ss[66];
  sys_part_4 += 7560*c3p2*c5p1*c7p5*c9p2*ss[64];
  sys_part_4 += 2520*c3p2*c5p1*c7p6*c9p1*ss[62];
  sys_part_4 += 7560*c3p2*c5p2*c7p1*c9p5*ss[68];
  sys_part_4 += 18900*c3p2*c5p2*c7p2*c9p4*ss[66];
  sys_part_4 += 25200*c3p2*c5p2*c7p3*c9p3*ss[64];
  sys_part_4 += 18900*c3p2*c5p2*c7p4*c9p2*ss[62];
  sys_part_4 += 7560*c3p2*c5p2*c7p5*c9p1*ss[60];
  sys_part_4 += 12600*c3p2*c5p3*c7p1*c9p4*ss[64];
  sys_part_4 += 25200*c3p2*c5p3*c7p2*c9p3*ss[62];
  sys_part_4 += 25200*c3p2*c5p3*c7p3*c9p2*ss[60];
  sys_part_4 += 12600*c3p2*c5p3*c7p4*c9p1*ss[58];
  sys_part_4 += 12600*c3p2*c5p4*c7p1*c9p3*ss[60];
  sys_part_4 += 18900*c3p2*c5p4*c7p2*c9p2*ss[58];
  sys_part_4 += 12600*c3p2*c5p4*c7p3*c9p1*ss[56];
  sys_part_4 += 7560*c3p2*c5p5*c7p1*c9p2*ss[56];
  sys_part_4 += 7560*c3p2*c5p5*c7p2*c9p1*ss[54];
  sys_part_4 += 2520*c3p2*c5p6*c7p1*c9p1*ss[52];
  sys_part_4 += 5040*c3p3*c5p1*c7p1*c9p5*ss[66];
  sys_part_4 += 12600*c3p3*c5p1*c7p2*c9p4*ss[64];
  sys_part_4 += 16800*c3p3*c5p1*c7p3*c9p3*ss[62];
  sys_part_4 += 12600*c3p3*c5p1*c7p4*c9p2*ss[60];
  sys_part_4 += 5040*c3p3*c5p1*c7p5*c9p1*ss[58];
  sys_part_4 += 12600*c3p3*c5p2*c7p1*c9p4*ss[62];
  sys_part_4 += 25200*c3p3*c5p2*c7p2*c9p3*ss[60];
  sys_part_4 += 25200*c3p3*c5p2*c7p3*c9p2*ss[58];
  sys_part_4 += 12600*c3p3*c5p2*c7p4*c9p1*ss[56];
  sys_part_4 += 16800*c3p3*c5p3*c7p1*c9p3*ss[58];
  sys_part_4 += 25200*c3p3*c5p3*c7p2*c9p2*ss[56];
  sys_part_4 += 16800*c3p3*c5p3*c7p3*c9p1*ss[54];
  sys_part_4 += 12600*c3p3*c5p4*c7p1*c9p2*ss[54];
  sys_part_4 += 12600*c3p3*c5p4*c7p2*c9p1*ss[52];
  sys_part_4 += 5040*c3p3*c5p5*c7p1*c9p1*ss[50];
  sys_part_4 += 6300*c3p4*c5p1*c7p1*c9p4*ss[60];
  sys_part_4 += 12600*c3p4*c5p1*c7p2*c9p3*ss[58];
  sys_part_4 += 12600*c3p4*c5p1*c7p3*c9p2*ss[56];
  sys_part_4 += 6300*c3p4*c5p1*c7p4*c9p1*ss[54];
  sys_part_4 += 12600*c3p4*c5p2*c7p1*c9p3*ss[56];
  sys_part_4 += 18900*c3p4*c5p2*c7p2*c9p2*ss[54];
  sys_part_4 += 12600*c3p4*c5p2*c7p3*c9p1*ss[52];
  sys_part_4 += 12600*c3p4*c5p3*c7p1*c9p2*ss[52];
  sys_part_4 += 12600*c3p4*c5p3*c7p2*c9p1*ss[50];
  sys_part_4 += 6300*c3p4*c5p4*c7p1*c9p1*ss[48];
  sys_part_4 += 5040*c3p5*c5p1*c7p1*c9p3*ss[54];
  sys_part_4 += 7560*c3p5*c5p1*c7p2*c9p2*ss[52];
  sys_part_4 += 5040*c3p5*c5p1*c7p3*c9p1*ss[50];
  sys_part_4 += 7560*c3p5*c5p2*c7p1*c9p2*ss[50];
  sys_part_4 += 7560*c3p5*c5p2*c7p2*c9p1*ss[48];
  sys_part_4 += 5040*c3p5*c5p3*c7p1*c9p1*ss[46];
  sys_part_4 += 2520*c3p6*c5p1*c7p1*c9p2*ss[48];
  sys_part_4 += 2520*c3p6*c5p1*c7p2*c9p1*ss[46];
  sys_part_4 += 2520*c3p6*c5p2*c7p1*c9p1*ss[44];
  sys_part_4 += 720*c3p7*c5p1*c7p1*c9p1*ss[42];
  sys_part_4 += 90*c1p1*c3p1*c5p8*ss[44];
  sys_part_4 += 90*c1p1*c3p1*c7p8*ss[60];
  sys_part_4 += 90*c1p1*c3p1*c9p8*ss[76];
  sys_part_4 += 360*c1p1*c3p2*c5p7*ss[42];
  sys_part_4 += 360*c1p1*c3p2*c7p7*ss[56];
  sys_part_4 += 360*c1p1*c3p2*c9p7*ss[70];
  sys_part_4 += 840*c1p1*c3p3*c5p6*ss[40];
  sys_part_4 += 840*c1p1*c3p3*c7p6*ss[52];
  sys_part_4 += 840*c1p1*c3p3*c9p6*ss[64];
  sys_part_4 += 1260*c1p1*c3p4*c5p5*ss[38];
  sys_part_4 += 1260*c1p1*c3p4*c7p5*ss[48];
  sys_part_4 += 1260*c1p1*c3p4*c9p5*ss[58];
  sys_part_4 += 1260*c1p1*c3p5*c5p4*ss[36];
  sys_part_4 += 1260*c1p1*c3p5*c7p4*ss[44];
  sys_part_4 += 1260*c1p1*c3p5*c9p4*ss[52];
  sys_part_4 += 840*c1p1*c3p6*c5p3*ss[34];
  sys_part_4 += 840*c1p1*c3p6*c7p3*ss[40];
  sys_part_4 += 840*c1p1*c3p6*c9p3*ss[46];
  sys_part_4 += 360*c1p1*c3p7*c5p2*ss[32];
  sys_part_4 += 360*c1p1*c3p7*c7p2*ss[36];
  sys_part_4 += 360*c1p1*c3p7*c9p2*ss[40];
  sys_part_4 += 90*c1p1*c3p8*c5p1*ss[30];
  sys_part_4 += 90*c1p1*c3p8*c7p1*ss[32];
  sys_part_4 += 90*c1p1*c3p8*c9p1*ss[34];
  sys_part_4 += 90*c1p1*c5p1*c7p8*ss[62];
  sys_part_4 += 90*c1p1*c5p1*c9p8*ss[78];
  sys_part_4 += 360*c1p1*c5p2*c7p7*ss[60];
  sys_part_4 += 360*c1p1*c5p2*c9p7*ss[74];
  sys_part_4 += 840*c1p1*c5p3*c7p6*ss[58];
  sys_part_4 += 840*c1p1*c5p3*c9p6*ss[70];
  sys_part_4 += 1260*c1p1*c5p4*c7p5*ss[56];
  sys_part_4 += 1260*c1p1*c5p4*c9p5*ss[66];
  sys_part_4 += 1260*c1p1*c5p5*c7p4*ss[54];
  sys_part_4 += 1260*c1p1*c5p5*c9p4*ss[62];
  sys_part_4 += 840*c1p1*c5p6*c7p3*ss[52];
  sys_part_4 += 840*c1p1*c5p6*c9p3*ss[58];
  sys_part_4 += 360*c1p1*c5p7*c7p2*ss[50];
  sys_part_4 += 360*c1p1*c5p7*c9p2*ss[54];
  sys_part_4 += 90*c1p1*c5p8*c7p1*ss[48];
  sys_part_4 += 90*c1p1*c5p8*c9p1*ss[50];
  sys_part_4 += 90*c1p1*c7p1*c9p8*ss[80];
  sys_part_4 += 360*c1p1*c7p2*c9p7*ss[78];
  sys_part_4 += 840*c1p1*c7p3*c9p6*ss[76];
  sys_part_4 += 1260*c1p1*c7p4*c9p5*ss[74];
  sys_part_4 += 1260*c1p1*c7p5*c9p4*ss[72];
  sys_part_4 += 840*c1p1*c7p6*c9p3*ss[70];
  sys_part_4 += 360*c1p1*c7p7*c9p2*ss[68];
  sys_part_4 += 90*c1p1*c7p8*c9p1*ss[66];
  sys_part_4 += 360*c1p2*c3p1*c5p7*ss[40];
  sys_part_4 += 360*c1p2*c3p1*c7p7*ss[54];
  sys_part_4 += 360*c1p2*c3p1*c9p7*ss[68];
  sys_part_4 += 1260*c1p2*c3p2*c5p6*ss[38];
  sys_part_4 += 1260*c1p2*c3p2*c7p6*ss[50];
  sys_part_4 += 1260*c1p2*c3p2*c9p6*ss[62];
  sys_part_4 += 2520*c1p2*c3p3*c5p5*ss[36];
  sys_part_4 += 2520*c1p2*c3p3*c7p5*ss[46];
  sys_part_4 += 2520*c1p2*c3p3*c9p5*ss[56];
  sys_part_4 += 3150*c1p2*c3p4*c5p4*ss[34];
  sys_part_4 += 3150*c1p2*c3p4*c7p4*ss[42];
  sys_part_4 += 3150*c1p2*c3p4*c9p4*ss[50];
  sys_part_4 += 2520*c1p2*c3p5*c5p3*ss[32];
  sys_part_4 += 2520*c1p2*c3p5*c7p3*ss[38];
  sys_part_4 += 2520*c1p2*c3p5*c9p3*ss[44];
  sys_part_4 += 1260*c1p2*c3p6*c5p2*ss[30];
  sys_part_4 += 1260*c1p2*c3p6*c7p2*ss[34];
  sys_part_4 += 1260*c1p2*c3p6*c9p2*ss[38];
  sys_part_4 += 360*c1p2*c3p7*c5p1*ss[28];
  sys_part_4 += 360*c1p2*c3p7*c7p1*ss[30];
  sys_part_4 += 360*c1p2*c3p7*c9p1*ss[32];
  sys_part_4 += 360*c1p2*c5p1*c7p7*ss[56];
  sys_part_4 += 360*c1p2*c5p1*c9p7*ss[70];
  sys_part_4 += 1260*c1p2*c5p2*c7p6*ss[54];
  sys_part_4 += 1260*c1p2*c5p2*c9p6*ss[66];
  sys_part_4 += 2520*c1p2*c5p3*c7p5*ss[52];
  sys_part_4 += 2520*c1p2*c5p3*c9p5*ss[62];
  sys_part_4 += 3150*c1p2*c5p4*c7p4*ss[50];
  sys_part_4 += 3150*c1p2*c5p4*c9p4*ss[58];
  sys_part_4 += 2520*c1p2*c5p5*c7p3*ss[48];
  sys_part_4 += 2520*c1p2*c5p5*c9p3*ss[54];
  sys_part_4 += 1260*c1p2*c5p6*c7p2*ss[46];
  sys_part_4 += 1260*c1p2*c5p6*c9p2*ss[50];
  sys_part_4 += 360*c1p2*c5p7*c7p1*ss[44];
  sys_part_4 += 360*c1p2*c5p7*c9p1*ss[46];
  sys_part_4 += 360*c1p2*c7p1*c9p7*ss[72];
  sys_part_4 += 1260*c1p2*c7p2*c9p6*ss[70];
  sys_part_4 += 2520*c1p2*c7p3*c9p5*ss[68];
  sys_part_4 += 3150*c1p2*c7p4*c9p4*ss[66];
  sys_part_4 += 2520*c1p2*c7p5*c9p3*ss[64];
  sys_part_4 += 1260*c1p2*c7p6*c9p2*ss[62];
  sys_part_4 += 360*c1p2*c7p7*c9p1*ss[60];
  sys_part_4 += 840*c1p3*c3p1*c5p6*ss[36];
  sys_part_4 += 840*c1p3*c3p1*c7p6*ss[48];
  sys_part_4 += 840*c1p3*c3p1*c9p6*ss[60];
  sys_part_4 += 2520*c1p3*c3p2*c5p5*ss[34];
  sys_part_4 += 2520*c1p3*c3p2*c7p5*ss[44];
  sys_part_4 += 2520*c1p3*c3p2*c9p5*ss[54];
  sys_part_4 += 4200*c1p3*c3p3*c5p4*ss[32];
  sys_part_4 += 4200*c1p3*c3p3*c7p4*ss[40];
  sys_part_4 += 4200*c1p3*c3p3*c9p4*ss[48];
  sys_part_4 += 4200*c1p3*c3p4*c5p3*ss[30];
  sys_part_4 += 4200*c1p3*c3p4*c7p3*ss[36];
  sys_part_4 += 4200*c1p3*c3p4*c9p3*ss[42];
  sys_part_4 += 2520*c1p3*c3p5*c5p2*ss[28];
  sys_part_4 += 2520*c1p3*c3p5*c7p2*ss[32];
  sys_part_4 += 2520*c1p3*c3p5*c9p2*ss[36];
  sys_part_4 += 840*c1p3*c3p6*c5p1*ss[26];
  sys_part_4 += 840*c1p3*c3p6*c7p1*ss[28];
  sys_part_4 += 840*c1p3*c3p6*c9p1*ss[30];
  sys_part_4 += 840*c1p3*c5p1*c7p6*ss[50];
  sys_part_4 += 840*c1p3*c5p1*c9p6*ss[62];
  sys_part_4 += 2520*c1p3*c5p2*c7p5*ss[48];
  sys_part_4 += 2520*c1p3*c5p2*c9p5*ss[58];
  sys_part_4 += 4200*c1p3*c5p3*c7p4*ss[46];
  sys_part_4 += 4200*c1p3*c5p3*c9p4*ss[54];
  sys_part_4 += 4200*c1p3*c5p4*c7p3*ss[44];
  sys_part_4 += 4200*c1p3*c5p4*c9p3*ss[50];
  sys_part_4 += 2520*c1p3*c5p5*c7p2*ss[42];
  sys_part_4 += 2520*c1p3*c5p5*c9p2*ss[46];
  sys_part_4 += 840*c1p3*c5p6*c7p1*ss[40];
  sys_part_4 += 840*c1p3*c5p6*c9p1*ss[42];
  sys_part_4 += 840*c1p3*c7p1*c9p6*ss[64];
  sys_part_4 += 2520*c1p3*c7p2*c9p5*ss[62];
  sys_part_4 += 4200*c1p3*c7p3*c9p4*ss[60];
  sys_part_4 += 4200*c1p3*c7p4*c9p3*ss[58];
  sys_part_4 += 2520*c1p3*c7p5*c9p2*ss[56];
  sys_part_4 += 840*c1p3*c7p6*c9p1*ss[54];
  sys_part_4 += 1260*c1p4*c3p1*c5p5*ss[32];
  sys_part_4 += 1260*c1p4*c3p1*c7p5*ss[42];
  sys_part_4 += 1260*c1p4*c3p1*c9p5*ss[52];
  sys_part_4 += 3150*c1p4*c3p2*c5p4*ss[30];
  sys_part_4 += 3150*c1p4*c3p2*c7p4*ss[38];
  sys_part_4 += 3150*c1p4*c3p2*c9p4*ss[46];
  sys_part_4 += 4200*c1p4*c3p3*c5p3*ss[28];
  sys_part_4 += 4200*c1p4*c3p3*c7p3*ss[34];
  sys_part_4 += 4200*c1p4*c3p3*c9p3*ss[40];
  sys_part_4 += 3150*c1p4*c3p4*c5p2*ss[26];
  sys_part_4 += 3150*c1p4*c3p4*c7p2*ss[30];
  sys_part_4 += 3150*c1p4*c3p4*c9p2*ss[34];
  sys_part_4 += 1260*c1p4*c3p5*c5p1*ss[24];
  sys_part_4 += 1260*c1p4*c3p5*c7p1*ss[26];
  sys_part_4 += 1260*c1p4*c3p5*c9p1*ss[28];
  sys_part_4 += 1260*c1p4*c5p1*c7p5*ss[44];
  sys_part_4 += 1260*c1p4*c5p1*c9p5*ss[54];
  sys_part_4 += 3150*c1p4*c5p2*c7p4*ss[42];
  sys_part_4 += 3150*c1p4*c5p2*c9p4*ss[50];
  sys_part_4 += 4200*c1p4*c5p3*c7p3*ss[40];
  sys_part_4 += 4200*c1p4*c5p3*c9p3*ss[46];
  sys_part_4 += 3150*c1p4*c5p4*c7p2*ss[38];
  sys_part_4 += 3150*c1p4*c5p4*c9p2*ss[42];
  sys_part_4 += 1260*c1p4*c5p5*c7p1*ss[36];
  sys_part_4 += 1260*c1p4*c5p5*c9p1*ss[38];
  sys_part_4 += 1260*c1p4*c7p1*c9p5*ss[56];
  sys_part_4 += 3150*c1p4*c7p2*c9p4*ss[54];
  sys_part_4 += 4200*c1p4*c7p3*c9p3*ss[52];
  sys_part_4 += 3150*c1p4*c7p4*c9p2*ss[50];
  sys_part_4 += 1260*c1p4*c7p5*c9p1*ss[48];
  sys_part_4 += 1260*c1p5*c3p1*c5p4*ss[28];
  sys_part_4 += 1260*c1p5*c3p1*c7p4*ss[36];
  sys_part_4 += 1260*c1p5*c3p1*c9p4*ss[44];
  sys_part_4 += 2520*c1p5*c3p2*c5p3*ss[26];
  sys_part_4 += 2520*c1p5*c3p2*c7p3*ss[32];
  sys_part_4 += 2520*c1p5*c3p2*c9p3*ss[38];
  sys_part_4 += 2520*c1p5*c3p3*c5p2*ss[24];
  sys_part_4 += 2520*c1p5*c3p3*c7p2*ss[28];
  sys_part_4 += 2520*c1p5*c3p3*c9p2*ss[32];
  sys_part_4 += 1260*c1p5*c3p4*c5p1*ss[22];
  sys_part_4 += 1260*c1p5*c3p4*c7p1*ss[24];
  sys_part_4 += 1260*c1p5*c3p4*c9p1*ss[26];
  sys_part_4 += 1260*c1p5*c5p1*c7p4*ss[38];
  sys_part_4 += 1260*c1p5*c5p1*c9p4*ss[46];
  sys_part_4 += 2520*c1p5*c5p2*c7p3*ss[36];
  sys_part_4 += 2520*c1p5*c5p2*c9p3*ss[42];
  sys_part_4 += 2520*c1p5*c5p3*c7p2*ss[34];
  sys_part_4 += 2520*c1p5*c5p3*c9p2*ss[38];
  sys_part_4 += 1260*c1p5*c5p4*c7p1*ss[32];
  sys_part_4 += 1260*c1p5*c5p4*c9p1*ss[34];
  sys_part_4 += 1260*c1p5*c7p1*c9p4*ss[48];
  sys_part_4 += 2520*c1p5*c7p2*c9p3*ss[46];
  sys_part_4 += 2520*c1p5*c7p3*c9p2*ss[44];
  sys_part_4 += 1260*c1p5*c7p4*c9p1*ss[42];
  sys_part_4 += 840*c1p6*c3p1*c5p3*ss[24];
  sys_part_4 += 840*c1p6*c3p1*c7p3*ss[30];
  sys_part_4 += 840*c1p6*c3p1*c9p3*ss[36];
  sys_part_4 += 1260*c1p6*c3p2*c5p2*ss[22];
  sys_part_4 += 1260*c1p6*c3p2*c7p2*ss[26];
  sys_part_4 += 1260*c1p6*c3p2*c9p2*ss[30];
  sys_part_4 += 840*c1p6*c3p3*c5p1*ss[20];
  sys_part_4 += 840*c1p6*c3p3*c7p1*ss[22];
  sys_part_4 += 840*c1p6*c3p3*c9p1*ss[24];
  sys_part_4 += 840*c1p6*c5p1*c7p3*ss[32];
  sys_part_4 += 840*c1p6*c5p1*c9p3*ss[38];
  sys_part_4 += 1260*c1p6*c5p2*c7p2*ss[30];
  sys_part_4 += 1260*c1p6*c5p2*c9p2*ss[34];
  sys_part_4 += 840*c1p6*c5p3*c7p1*ss[28];
  sys_part_4 += 840*c1p6*c5p3*c9p1*ss[30];
  sys_part_4 += 840*c1p6*c7p1*c9p3*ss[40];
  sys_part_4 += 1260*c1p6*c7p2*c9p2*ss[38];
  sys_part_4 += 840*c1p6*c7p3*c9p1*ss[36];
  sys_part_4 += 360*c1p7*c3p1*c5p2*ss[20];
  sys_part_4 += 360*c1p7*c3p1*c7p2*ss[24];
  sys_part_4 += 360*c1p7*c3p1*c9p2*ss[28];
  sys_part_4 += 360*c1p7*c3p2*c5p1*ss[18];
  sys_part_4 += 360*c1p7*c3p2*c7p1*ss[20];
  sys_part_4 += 360*c1p7*c3p2*c9p1*ss[22];
  sys_part_4 += 360*c1p7*c5p1*c7p2*ss[26];
  sys_part_4 += 360*c1p7*c5p1*c9p2*ss[30];
  sys_part_4 += 360*c1p7*c5p2*c7p1*ss[24];
  sys_part_4 += 360*c1p7*c5p2*c9p1*ss[26];
  sys_part_4 += 360*c1p7*c7p1*c9p2*ss[32];
  sys_part_4 += 360*c1p7*c7p2*c9p1*ss[30];
  sys_part_4 += 90*c1p8*c3p1*c5p1*ss[16];
  sys_part_4 += 90*c1p8*c3p1*c7p1*ss[18];
  sys_part_4 += 90*c1p8*c3p1*c9p1*ss[20];
  sys_part_4 += 90*c1p8*c5p1*c7p1*ss[20];
  sys_part_4 += 90*c1p8*c5p1*c9p1*ss[22];
  sys_part_4 += 90*c1p8*c7p1*c9p1*ss[24];
  sys_part_4 += 90*c3p1*c5p1*c7p8*ss[64];
  sys_part_4 += 90*c3p1*c5p1*c9p8*ss[80];
  sys_part_4 += 360*c3p1*c5p2*c7p7*ss[62];
  sys_part_4 += 360*c3p1*c5p2*c9p7*ss[76];
  sys_part_4 += 840*c3p1*c5p3*c7p6*ss[60];
  sys_part_4 += 840*c3p1*c5p3*c9p6*ss[72];
  sys_part_4 += 1260*c3p1*c5p4*c7p5*ss[58];
  sys_part_4 += 1260*c3p1*c5p4*c9p5*ss[68];
  sys_part_4 += 1260*c3p1*c5p5*c7p4*ss[56];
  sys_part_4 += 1260*c3p1*c5p5*c9p4*ss[64];
  sys_part_4 += 840*c3p1*c5p6*c7p3*ss[54];
  sys_part_4 += 840*c3p1*c5p6*c9p3*ss[60];
  sys_part_4 += 360*c3p1*c5p7*c7p2*ss[52];
  sys_part_4 += 360*c3p1*c5p7*c9p2*ss[56];
  sys_part_4 += 90*c3p1*c5p8*c7p1*ss[50];
  sys_part_4 += 90*c3p1*c5p8*c9p1*ss[52];
  sys_part_4 += 90*c3p1*c7p1*c9p8*ss[82];
  sys_part_4 += 360*c3p1*c7p2*c9p7*ss[80];
  sys_part_4 += 840*c3p1*c7p3*c9p6*ss[78];
  sys_part_4 += 1260*c3p1*c7p4*c9p5*ss[76];
  sys_part_4 += 1260*c3p1*c7p5*c9p4*ss[74];
  sys_part_4 += 840*c3p1*c7p6*c9p3*ss[72];
  sys_part_4 += 360*c3p1*c7p7*c9p2*ss[70];
  sys_part_4 += 90*c3p1*c7p8*c9p1*ss[68];
  sys_part_4 += 360*c3p2*c5p1*c7p7*ss[60];
  sys_part_4 += 360*c3p2*c5p1*c9p7*ss[74];
  sys_part_4 += 1260*c3p2*c5p2*c7p6*ss[58];
  sys_part_4 += 1260*c3p2*c5p2*c9p6*ss[70];
  sys_part_4 += 2520*c3p2*c5p3*c7p5*ss[56];
  sys_part_4 += 2520*c3p2*c5p3*c9p5*ss[66];
  sys_part_4 += 3150*c3p2*c5p4*c7p4*ss[54];
  sys_part_4 += 3150*c3p2*c5p4*c9p4*ss[62];
  sys_part_4 += 2520*c3p2*c5p5*c7p3*ss[52];
  sys_part_4 += 2520*c3p2*c5p5*c9p3*ss[58];
  sys_part_4 += 1260*c3p2*c5p6*c7p2*ss[50];
  sys_part_4 += 1260*c3p2*c5p6*c9p2*ss[54];
  sys_part_4 += 360*c3p2*c5p7*c7p1*ss[48];
  sys_part_4 += 360*c3p2*c5p7*c9p1*ss[50];
  sys_part_4 += 360*c3p2*c7p1*c9p7*ss[76];
  sys_part_4 += 1260*c3p2*c7p2*c9p6*ss[74];
  sys_part_4 += 2520*c3p2*c7p3*c9p5*ss[72];
  sys_part_4 += 3150*c3p2*c7p4*c9p4*ss[70];
  sys_part_4 += 2520*c3p2*c7p5*c9p3*ss[68];
  sys_part_4 += 1260*c3p2*c7p6*c9p2*ss[66];
  sys_part_4 += 360*c3p2*c7p7*c9p1*ss[64];
  sys_part_4 += 840*c3p3*c5p1*c7p6*ss[56];
  sys_part_4 += 840*c3p3*c5p1*c9p6*ss[68];
  sys_part_4 += 2520*c3p3*c5p2*c7p5*ss[54];
  sys_part_4 += 2520*c3p3*c5p2*c9p5*ss[64];
  sys_part_4 += 4200*c3p3*c5p3*c7p4*ss[52];
  sys_part_4 += 4200*c3p3*c5p3*c9p4*ss[60];
  sys_part_4 += 4200*c3p3*c5p4*c7p3*ss[50];
  sys_part_4 += 4200*c3p3*c5p4*c9p3*ss[56];
  sys_part_4 += 2520*c3p3*c5p5*c7p2*ss[48];
  sys_part_4 += 2520*c3p3*c5p5*c9p2*ss[52];
  sys_part_4 += 840*c3p3*c5p6*c7p1*ss[46];
  sys_part_4 += 840*c3p3*c5p6*c9p1*ss[48];
  sys_part_4 += 840*c3p3*c7p1*c9p6*ss[70];
  sys_part_4 += 2520*c3p3*c7p2*c9p5*ss[68];
  sys_part_4 += 4200*c3p3*c7p3*c9p4*ss[66];
  sys_part_4 += 4200*c3p3*c7p4*c9p3*ss[64];
  sys_part_4 += 2520*c3p3*c7p5*c9p2*ss[62];
  sys_part_4 += 840*c3p3*c7p6*c9p1*ss[60];
  sys_part_4 += 1260*c3p4*c5p1*c7p5*ss[52];
  sys_part_4 += 1260*c3p4*c5p1*c9p5*ss[62];
  sys_part_4 += 3150*c3p4*c5p2*c7p4*ss[50];
  sys_part_4 += 3150*c3p4*c5p2*c9p4*ss[58];
  sys_part_4 += 4200*c3p4*c5p3*c7p3*ss[48];
  sys_part_4 += 4200*c3p4*c5p3*c9p3*ss[54];
  sys_part_4 += 3150*c3p4*c5p4*c7p2*ss[46];
  sys_part_4 += 3150*c3p4*c5p4*c9p2*ss[50];
  sys_part_4 += 1260*c3p4*c5p5*c7p1*ss[44];
  sys_part_4 += 1260*c3p4*c5p5*c9p1*ss[46];
  sys_part_4 += 1260*c3p4*c7p1*c9p5*ss[64];
  sys_part_4 += 3150*c3p4*c7p2*c9p4*ss[62];
  sys_part_4 += 4200*c3p4*c7p3*c9p3*ss[60];
  sys_part_4 += 3150*c3p4*c7p4*c9p2*ss[58];
  sys_part_4 += 1260*c3p4*c7p5*c9p1*ss[56];
  sys_part_4 += 1260*c3p5*c5p1*c7p4*ss[48];
  sys_part_4 += 1260*c3p5*c5p1*c9p4*ss[56];
  sys_part_4 += 2520*c3p5*c5p2*c7p3*ss[46];
  sys_part_4 += 2520*c3p5*c5p2*c9p3*ss[52];
  sys_part_4 += 2520*c3p5*c5p3*c7p2*ss[44];
  sys_part_4 += 2520*c3p5*c5p3*c9p2*ss[48];
  sys_part_4 += 1260*c3p5*c5p4*c7p1*ss[42];
  sys_part_4 += 1260*c3p5*c5p4*c9p1*ss[44];
  sys_part_4 += 1260*c3p5*c7p1*c9p4*ss[58];
  sys_part_4 += 2520*c3p5*c7p2*c9p3*ss[56];
  sys_part_4 += 2520*c3p5*c7p3*c9p2*ss[54];
  sys_part_4 += 1260*c3p5*c7p4*c9p1*ss[52];
  sys_part_4 += 840*c3p6*c5p1*c7p3*ss[44];
  sys_part_4 += 840*c3p6*c5p1*c9p3*ss[50];
  sys_part_4 += 1260*c3p6*c5p2*c7p2*ss[42];
  sys_part_4 += 1260*c3p6*c5p2*c9p2*ss[46];
  sys_part_4 += 840*c3p6*c5p3*c7p1*ss[40];
  sys_part_4 += 840*c3p6*c5p3*c9p1*ss[42];
  sys_part_4 += 840*c3p6*c7p1*c9p3*ss[52];
  sys_part_4 += 1260*c3p6*c7p2*c9p2*ss[50];
  sys_part_4 += 840*c3p6*c7p3*c9p1*ss[48];
  sys_part_4 += 360*c3p7*c5p1*c7p2*ss[40];
  sys_part_4 += 360*c3p7*c5p1*c9p2*ss[44];
  sys_part_4 += 360*c3p7*c5p2*c7p1*ss[38];
  sys_part_4 += 360*c3p7*c5p2*c9p1*ss[40];
  sys_part_4 += 360*c3p7*c7p1*c9p2*ss[46];
  sys_part_4 += 360*c3p7*c7p2*c9p1*ss[44];
  sys_part_4 += 90*c3p8*c5p1*c7p1*ss[36];
  sys_part_4 += 90*c3p8*c5p1*c9p1*ss[38];
  sys_part_4 += 90*c3p8*c7p1*c9p1*ss[40];
  sys_part_4 += 90*c5p1*c7p1*c9p8*ss[84];
  sys_part_4 += 360*c5p1*c7p2*c9p7*ss[82];
  sys_part_4 += 840*c5p1*c7p3*c9p6*ss[80];
  sys_part_4 += 1260*c5p1*c7p4*c9p5*ss[78];
  sys_part_4 += 1260*c5p1*c7p5*c9p4*ss[76];
  sys_part_4 += 840*c5p1*c7p6*c9p3*ss[74];
  sys_part_4 += 360*c5p1*c7p7*c9p2*ss[72];
  sys_part_4 += 90*c5p1*c7p8*c9p1*ss[70];
  sys_part_4 += 360*c5p2*c7p1*c9p7*ss[80];
  sys_part_4 += 1260*c5p2*c7p2*c9p6*ss[78];
  sys_part_4 += 2520*c5p2*c7p3*c9p5*ss[76];
  sys_part_4 += 3150*c5p2*c7p4*c9p4*ss[74];
  sys_part_4 += 2520*c5p2*c7p5*c9p3*ss[72];
  sys_part_4 += 1260*c5p2*c7p6*c9p2*ss[70];
  sys_part_4 += 360*c5p2*c7p7*c9p1*ss[68];
  sys_part_4 += 840*c5p3*c7p1*c9p6*ss[76];
  sys_part_4 += 2520*c5p3*c7p2*c9p5*ss[74];
  sys_part_4 += 4200*c5p3*c7p3*c9p4*ss[72];
  sys_part_4 += 4200*c5p3*c7p4*c9p3*ss[70];
  sys_part_4 += 2520*c5p3*c7p5*c9p2*ss[68];
  sys_part_4 += 840*c5p3*c7p6*c9p1*ss[66];
  sys_part_4 += 1260*c5p4*c7p1*c9p5*ss[72];
  sys_part_4 += 3150*c5p4*c7p2*c9p4*ss[70];
  sys_part_4 += 4200*c5p4*c7p3*c9p3*ss[68];
  sys_part_4 += 3150*c5p4*c7p4*c9p2*ss[66];
  sys_part_4 += 1260*c5p4*c7p5*c9p1*ss[64];
  sys_part_4 += 1260*c5p5*c7p1*c9p4*ss[68];
  sys_part_4 += 2520*c5p5*c7p2*c9p3*ss[66];
  sys_part_4 += 2520*c5p5*c7p3*c9p2*ss[64];
  sys_part_4 += 1260*c5p5*c7p4*c9p1*ss[62];
  sys_part_4 += 840*c5p6*c7p1*c9p3*ss[64];
  sys_part_4 += 1260*c5p6*c7p2*c9p2*ss[62];
  sys_part_4 += 840*c5p6*c7p3*c9p1*ss[60];
  sys_part_4 += 360*c5p7*c7p1*c9p2*ss[60];
  sys_part_4 += 360*c5p7*c7p2*c9p1*ss[58];
  sys_part_4 += 90*c5p8*c7p1*c9p1*ss[56];
  sys_part_4 += 10*c1p1*c3p9*ss[28];
  sys_part_4 += 10*c1p1*c5p9*ss[46];
  sys_part_4 += 10*c1p1*c7p9*ss[64];
  sys_part_4 += 10*c1p1*c9p9*ss[82];
  sys_part_4 += 45*c1p2*c3p8*ss[26];
  sys_part_4 += 45*c1p2*c5p8*ss[42];
  sys_part_4 += 45*c1p2*c7p8*ss[58];
  sys_part_4 += 45*c1p2*c9p8*ss[74];
  sys_part_4 += 120*c1p3*c3p7*ss[24];
  sys_part_4 += 120*c1p3*c5p7*ss[38];
  sys_part_4 += 120*c1p3*c7p7*ss[52];
  sys_part_4 += 120*c1p3*c9p7*ss[66];
  sys_part_4 += 210*c1p4*c3p6*ss[22];
  sys_part_4 += 210*c1p4*c5p6*ss[34];
  sys_part_4 += 210*c1p4*c7p6*ss[46];
  sys_part_4 += 210*c1p4*c9p6*ss[58];
  sys_part_4 += 252*c1p5*c3p5*ss[20];
  sys_part_4 += 252*c1p5*c5p5*ss[30];
  sys_part_4 += 252*c1p5*c7p5*ss[40];
  sys_part_4 += 252*c1p5*c9p5*ss[50];
  sys_part_4 += 210*c1p6*c3p4*ss[18];
  sys_part_4 += 210*c1p6*c5p4*ss[26];
  sys_part_4 += 210*c1p6*c7p4*ss[34];
  sys_part_4 += 210*c1p6*c9p4*ss[42];
  sys_part_4 += 120*c1p7*c3p3*ss[16];
  sys_part_4 += 120*c1p7*c5p3*ss[22];
  sys_part_4 += 120*c1p7*c7p3*ss[28];
  sys_part_4 += 120*c1p7*c9p3*ss[34];
  sys_part_4 += 45*c1p8*c3p2*ss[14];
  sys_part_4 += 45*c1p8*c5p2*ss[18];
  sys_part_4 += 45*c1p8*c7p2*ss[22];
  sys_part_4 += 45*c1p8*c9p2*ss[26];
  sys_part_4 += 10*c1p9*c3p1*ss[12];
  sys_part_4 += 10*c1p9*c5p1*ss[14];
  sys_part_4 += 10*c1p9*c7p1*ss[16];
  sys_part_4 += 10*c1p9*c9p1*ss[18];
  sys_part_4 += 10*c3p1*c5p9*ss[48];
  sys_part_4 += 10*c3p1*c7p9*ss[66];
  sys_part_4 += 10*c3p1*c9p9*ss[84];
  sys_part_4 += 45*c3p2*c5p8*ss[46];
  sys_part_4 += 45*c3p2*c7p8*ss[62];
  sys_part_4 += 45*c3p2*c9p8*ss[78];
  sys_part_4 += 120*c3p3*c5p7*ss[44];
  sys_part_4 += 120*c3p3*c7p7*ss[58];
  sys_part_4 += 120*c3p3*c9p7*ss[72];
  sys_part_4 += 210*c3p4*c5p6*ss[42];
  sys_part_4 += 210*c3p4*c7p6*ss[54];
  sys_part_4 += 210*c3p4*c9p6*ss[66];
  sys_part_4 += 252*c3p5*c5p5*ss[40];
  sys_part_4 += 252*c3p5*c7p5*ss[50];
  sys_part_4 += 252*c3p5*c9p5*ss[60];
  sys_part_4 += 210*c3p6*c5p4*ss[38];
  sys_part_4 += 210*c3p6*c7p4*ss[46];
  sys_part_4 += 210*c3p6*c9p4*ss[54];
  sys_part_4 += 120*c3p7*c5p3*ss[36];
  sys_part_4 += 120*c3p7*c7p3*ss[42];
  sys_part_4 += 120*c3p7*c9p3*ss[48];
  sys_part_4 += 45*c3p8*c5p2*ss[34];
  sys_part_4 += 45*c3p8*c7p2*ss[38];
  sys_part_4 += 45*c3p8*c9p2*ss[42];
  sys_part_4 += 10*c3p9*c5p1*ss[32];
  sys_part_4 += 10*c3p9*c7p1*ss[34];
  sys_part_4 += 10*c3p9*c9p1*ss[36];
  sys_part_4 += 10*c5p1*c7p9*ss[68];
  sys_part_4 += 10*c5p1*c9p9*ss[86];
  sys_part_4 += 45*c5p2*c7p8*ss[66];
  sys_part_4 += 45*c5p2*c9p8*ss[82];
  sys_part_4 += 120*c5p3*c7p7*ss[64];
  sys_part_4 += 120*c5p3*c9p7*ss[78];
  sys_part_4 += 210*c5p4*c7p6*ss[62];
  sys_part_4 += 210*c5p4*c9p6*ss[74];
  sys_part_4 += 252*c5p5*c7p5*ss[60];
  sys_part_4 += 252*c5p5*c9p5*ss[70];
  sys_part_4 += 210*c5p6*c7p4*ss[58];
  sys_part_4 += 210*c5p6*c9p4*ss[66];
  sys_part_4 += 120*c5p7*c7p3*ss[56];
  sys_part_4 += 120*c5p7*c9p3*ss[62];
  sys_part_4 += 45*c5p8*c7p2*ss[54];
  sys_part_4 += 45*c5p8*c9p2*ss[58];
  sys_part_4 += 10*c5p9*c7p1*ss[52];
  sys_part_4 += 10*c5p9*c9p1*ss[54];
  sys_part_4 += 10*c7p1*c9p9*ss[88];
  sys_part_4 += 45*c7p2*c9p8*ss[86];
  sys_part_4 += 120*c7p3*c9p7*ss[84];
  sys_part_4 += 210*c7p4*c9p6*ss[82];
  sys_part_4 += 252*c7p5*c9p5*ss[80];
  sys_part_4 += 210*c7p6*c9p4*ss[78];
  sys_part_4 += 120*c7p7*c9p3*ss[76];
  sys_part_4 += 45*c7p8*c9p2*ss[74];
  sys_part_4 += 10*c7p9*c9p1*ss[72];
  sys_part_4 += c1p10*ss[10];
  sys_part_4 += c3p10*ss[30];
  sys_part_4 += c5p10*ss[50];
  sys_part_4 += c7p10*ss[70];
  sys_part_4 += c9p10*ss[90];
  sys_part_4 += -tmom[10];
  TACC grad_0_num = convert<TACC>(2*(c1p1*ss[2] + c3p1*ss[4] + c5p1*ss[6] + c7p1*ss[8] + c9p1*ss[10])*sys_part_0);
  grad_0_num += 4*(6*c1p1*c3p1*c5p1*ss[10] + 6*c1p1*c3p1*c7p1*ss[12] + 6*c1p1*c3p1*c9p1*ss[14] + 6*c1p1*c5p1*c7p1*ss[14] + 6*c1p1*c5p1*c9p1*ss[16] + 6*c1p1*c7p1*c9p1*ss[18] + 6*c3p1*c5p1*c7p1*ss[16] + 6*c3p1*c5p1*c9p1*ss[18] + 6*c3p1*c7p1*c9p1*ss[20] + 6*c5p1*c7p1*c9p1*ss[22] + 3*c1p1*c3p2*ss[8] + 3*c1p1*c5p2*ss[12] + 3*c1p1*c7p2*ss[16] + 3*c1p1*c9p2*ss[20] + 3*c1p2*c3p1*ss[6] + 3*c1p2*c5p1*ss[8] + 3*c1p2*c7p1*ss[10] + 3*c1p2*c9p1*ss[12] + 3*c3p1*c5p2*ss[14] + 3*c3p1*c7p2*ss[18] + 3*c3p1*c9p2*ss[22] + 3*c3p2*c5p1*ss[12] + 3*c3p2*c7p1*ss[14] + 3*c3p2*c9p1*ss[16] + 3*c5p1*c7p2*ss[20] + 3*c5p1*c9p2*ss[24] + 3*c5p2*c7p1*ss[18] + 3*c5p2*c9p1*ss[20] + 3*c7p1*c9p2*ss[26] + 3*c7p2*c9p1*ss[24] + c1p3*ss[4] + c3p3*ss[10] + c5p3*ss[16] + c7p3*ss[22] + c9p3*ss[28])*sys_part_1;
  grad_0_num += 6*(120*c1p1*c3p1*c5p1*c7p1*c9p1*ss[26] + 60*c1p1*c3p1*c5p1*c7p2*ss[24] + 60*c1p1*c3p1*c5p1*c9p2*ss[28] + 60*c1p1*c3p1*c5p2*c7p1*ss[22] + 60*c1p1*c3p1*c5p2*c9p1*ss[24] + 60*c1p1*c3p1*c7p1*c9p2*ss[30] + 60*c1p1*c3p1*c7p2*c9p1*ss[28] + 60*c1p1*c3p2*c5p1*c7p1*ss[20] + 60*c1p1*c3p2*c5p1*c9p1*ss[22] + 60*c1p1*c3p2*c7p1*c9p1*ss[24] + 60*c1p1*c5p1*c7p1*c9p2*ss[32] + 60*c1p1*c5p1*c7p2*c9p1*ss[30] + 60*c1p1*c5p2*c7p1*c9p1*ss[28] + 60*c1p2*c3p1*c5p1*c7p1*ss[18] + 60*c1p2*c3p1*c5p1*c9p1*ss[20] + 60*c1p2*c3p1*c7p1*c9p1*ss[22] + 60*c1p2*c5p1*c7p1*c9p1*ss[24] + 60*c3p1*c5p1*c7p1*c9p2*ss[34] + 60*c3p1*c5p1*c7p2*c9p1*ss[32] + 60*c3p1*c5p2*c7p1*c9p1*ss[30] + 60*c3p2*c5p1*c7p1*c9p1*ss[28] + 20*c1p1*c3p1*c5p3*ss[20] + 20*c1p1*c3p1*c7p3*ss[26] + 20*c1p1*c3p1*c9p3*ss[32] + 30*c1p1*c3p2*c5p2*ss[18] + 30*c1p1*c3p2*c7p2*ss[22] + 30*c1p1*c3p2*c9p2*ss[26] + 20*c1p1*c3p3*c5p1*ss[16] + 20*c1p1*c3p3*c7p1*ss[18] + 20*c1p1*c3p3*c9p1*ss[20] + 20*c1p1*c5p1*c7p3*ss[28] + 20*c1p1*c5p1*c9p3*ss[34] + 30*c1p1*c5p2*c7p2*ss[26] + 30*c1p1*c5p2*c9p2*ss[30] + 20*c1p1*c5p3*c7p1*ss[24] + 20*c1p1*c5p3*c9p1*ss[26] + 20*c1p1*c7p1*c9p3*ss[36] + 30*c1p1*c7p2*c9p2*ss[34] + 20*c1p1*c7p3*c9p1*ss[32] + 30*c1p2*c3p1*c5p2*ss[16] + 30*c1p2*c3p1*c7p2*ss[20] + 30*c1p2*c3p1*c9p2*ss[24] + 30*c1p2*c3p2*c5p1*ss[14] + 30*c1p2*c3p2*c7p1*ss[16] + 30*c1p2*c3p2*c9p1*ss[18] + 30*c1p2*c5p1*c7p2*ss[22] + 30*c1p2*c5p1*c9p2*ss[26] + 30*c1p2*c5p2*c7p1*ss[20] + 30*c1p2*c5p2*c9p1*ss[22] + 30*c1p2*c7p1*c9p2*ss[28] + 30*c1p2*c7p2*c9p1*ss[26] + 20*c1p3*c3p1*c5p1*ss[12] + 20*c1p3*c3p1*c7p1*ss[14] + 20*c1p3*c3p1*c9p1*ss[16] + 20*c1p3*c5p1*c7p1*ss[16] + 20*c1p3*c5p1*c9p1*ss[18] + 20*c1p3*c7p1*c9p1*ss[20] + 20*c3p1*c5p1*c7p3*ss[30] + 20*c3p1*c5p1*c9p3*ss[36] + 30*c3p1*c5p2*c7p2*ss[28] + 30*c3p1*c5p2*c9p2*ss[32] + 20*c3p1*c5p3*c7p1*ss[26] + 20*c3p1*c5p3*c9p1*ss[28] + 20*c3p1*c7p1*c9p3*ss[38] + 30*c3p1*c7p2*c9p2*ss[36] + 20*c3p1*c7p3*c9p1*ss[34] + 30*c3p2*c5p1*c7p2*ss[26] + 30*c3p2*c5p1*c9p2*ss[30] + 30*c3p2*c5p2*c7p1*ss[24] + 30*c3p2*c5p2*c9p1*ss[26] + 30*c3p2*c7p1*c9p2*ss[32] + 30*c3p2*c7p2*c9p1*ss[30] + 20*c3p3*c5p1*c7p1*ss[22] + 20*c3p3*c5p1*c9p1*ss[24] + 20*c3p3*c7p1*c9p1*ss[26] + 20*c5p1*c7p1*c9p3*ss[40] + 30*c5p1*c7p2*c9p2*ss[38] + 20*c5p1*c7p3*c9p1*ss[36] + 30*c5p2*c7p1*c9p2*ss[36] + 30*c5p2*c7p2*c9p1*ss[34] + 20*c5p3*c7p1*c9p1*ss[32] + 5*c1p1*c3p4*ss[14] + 5*c1p1*c5p4*ss[22] + 5*c1p1*c7p4*ss[30] + 5*c1p1*c9p4*ss[38] + 10*c1p2*c3p3*ss[12] + 10*c1p2*c5p3*ss[18] + 10*c1p2*c7p3*ss[24] + 10*c1p2*c9p3*ss[30] + 10*c1p3*c3p2*ss[10] + 10*c1p3*c5p2*ss[14] + 10*c1p3*c7p2*ss[18] + 10*c1p3*c9p2*ss[22] + 5*c1p4*c3p1*ss[8] + 5*c1p4*c5p1*ss[10] + 5*c1p4*c7p1*ss[12] + 5*c1p4*c9p1*ss[14] + 5*c3p1*c5p4*ss[24] + 5*c3p1*c7p4*ss[32] + 5*c3p1*c9p4*ss[40] + 10*c3p2*c5p3*ss[22] + 10*c3p2*c7p3*ss[28] + 10*c3p2*c9p3*ss[34] + 10*c3p3*c5p2*ss[20] + 10*c3p3*c7p2*ss[24] + 10*c3p3*c9p2*ss[28] + 5*c3p4*c5p1*ss[18] + 5*c3p4*c7p1*ss[20] + 5*c3p4*c9p1*ss[22] + 5*c5p1*c7p4*ss[34] + 5*c5p1*c9p4*ss[42] + 10*c5p2*c7p3*ss[32] + 10*c5p2*c9p3*ss[38] + 10*c5p3*c7p2*ss[30] + 10*c5p3*c9p2*ss[34] + 5*c5p4*c7p1*ss[28] + 5*c5p4*c9p1*ss[30] + 5*c7p1*c9p4*ss[44] + 10*c7p2*c9p3*ss[42] + 10*c7p3*c9p2*ss[40] + 5*c7p4*c9p1*ss[38] + c1p5*ss[6] + c3p5*ss[16] + c5p5*ss[26] + c7p5*ss[36] + c9p5*ss[46])*sys_part_2;
  grad_0_num += 8*(840*c1p1*c3p1*c5p1*c7p1*c9p3*ss[44] + 1260*c1p1*c3p1*c5p1*c7p2*c9p2*ss[42] + 840*c1p1*c3p1*c5p1*c7p3*c9p1*ss[40] + 1260*c1p1*c3p1*c5p2*c7p1*c9p2*ss[40] + 1260*c1p1*c3p1*c5p2*c7p2*c9p1*ss[38] + 840*c1p1*c3p1*c5p3*c7p1*c9p1*ss[36] + 1260*c1p1*c3p2*c5p1*c7p1*c9p2*ss[38] + 1260*c1p1*c3p2*c5p1*c7p2*c9p1*ss[36] + 1260*c1p1*c3p2*c5p2*c7p1*c9p1*ss[34] + 840*c1p1*c3p3*c5p1*c7p1*c9p1*ss[32] + 1260*c1p2*c3p1*c5p1*c7p1*c9p2*ss[36] + 1260*c1p2*c3p1*c5p1*c7p2*c9p1*ss[34] + 1260*c1p2*c3p1*c5p2*c7p1*c9p1*ss[32] + 1260*c1p2*c3p2*c5p1*c7p1*c9p1*ss[30] + 840*c1p3*c3p1*c5p1*c7p1*c9p1*ss[28] + 210*c1p1*c3p1*c5p1*c7p4*ss[38] + 210*c1p1*c3p1*c5p1*c9p4*ss[46] + 420*c1p1*c3p1*c5p2*c7p3*ss[36] + 420*c1p1*c3p1*c5p2*c9p3*ss[42] + 420*c1p1*c3p1*c5p3*c7p2*ss[34] + 420*c1p1*c3p1*c5p3*c9p2*ss[38] + 210*c1p1*c3p1*c5p4*c7p1*ss[32] + 210*c1p1*c3p1*c5p4*c9p1*ss[34] + 210*c1p1*c3p1*c7p1*c9p4*ss[48] + 420*c1p1*c3p1*c7p2*c9p3*ss[46] + 420*c1p1*c3p1*c7p3*c9p2*ss[44] + 210*c1p1*c3p1*c7p4*c9p1*ss[42] + 420*c1p1*c3p2*c5p1*c7p3*ss[34] + 420*c1p1*c3p2*c5p1*c9p3*ss[40] + 630*c1p1*c3p2*c5p2*c7p2*ss[32] + 630*c1p1*c3p2*c5p2*c9p2*ss[36] + 420*c1p1*c3p2*c5p3*c7p1*ss[30] + 420*c1p1*c3p2*c5p3*c9p1*ss[32] + 420*c1p1*c3p2*c7p1*c9p3*ss[42] + 630*c1p1*c3p2*c7p2*c9p2*ss[40] + 420*c1p1*c3p2*c7p3*c9p1*ss[38] + 420*c1p1*c3p3*c5p1*c7p2*ss[30] + 420*c1p1*c3p3*c5p1*c9p2*ss[34] + 420*c1p1*c3p3*c5p2*c7p1*ss[28] + 420*c1p1*c3p3*c5p2*c9p1*ss[30] + 420*c1p1*c3p3*c7p1*c9p2*ss[36] + 420*c1p1*c3p3*c7p2*c9p1*ss[34] + 210*c1p1*c3p4*c5p1*c7p1*ss[26] + 210*c1p1*c3p4*c5p1*c9p1*ss[28] + 210*c1p1*c3p4*c7p1*c9p1*ss[30] + 210*c1p1*c5p1*c7p1*c9p4*ss[50] + 420*c1p1*c5p1*c7p2*c9p3*ss[48] + 420*c1p1*c5p1*c7p3*c9p2*ss[46] + 210*c1p1*c5p1*c7p4*c9p1*ss[44] + 420*c1p1*c5p2*c7p1*c9p3*ss[46] + 630*c1p1*c5p2*c7p2*c9p2*ss[44] + 420*c1p1*c5p2*c7p3*c9p1*ss[42] + 420*c1p1*c5p3*c7p1*c9p2*ss[42] + 420*c1p1*c5p3*c7p2*c9p1*ss[40] + 210*c1p1*c5p4*c7p1*c9p1*ss[38] + 420*c1p2*c3p1*c5p1*c7p3*ss[32] + 420*c1p2*c3p1*c5p1*c9p3*ss[38] + 630*c1p2*c3p1*c5p2*c7p2*ss[30] + 630*c1p2*c3p1*c5p2*c9p2*ss[34] + 420*c1p2*c3p1*c5p3*c7p1*ss[28] + 420*c1p2*c3p1*c5p3*c9p1*ss[30] + 420*c1p2*c3p1*c7p1*c9p3*ss[40] + 630*c1p2*c3p1*c7p2*c9p2*ss[38] + 420*c1p2*c3p1*c7p3*c9p1*ss[36] + 630*c1p2*c3p2*c5p1*c7p2*ss[28] + 630*c1p2*c3p2*c5p1*c9p2*ss[32] + 630*c1p2*c3p2*c5p2*c7p1*ss[26] + 630*c1p2*c3p2*c5p2*c9p1*ss[28] + 630*c1p2*c3p2*c7p1*c9p2*ss[34] + 630*c1p2*c3p2*c7p2*c9p1*ss[32] + 420*c1p2*c3p3*c5p1*c7p1*ss[24] + 420*c1p2*c3p3*c5p1*c9p1*ss[26] + 420*c1p2*c3p3*c7p1*c9p1*ss[28] + 420*c1p2*c5p1*c7p1*c9p3*ss[42] + 630*c1p2*c5p1*c7p2*c9p2*ss[40] + 420*c1p2*c5p1*c7p3*c9p1*ss[38] + 630*c1p2*c5p2*c7p1*c9p2*ss[38] + 630*c1p2*c5p2*c7p2*c9p1*ss[36] + 420*c1p2*c5p3*c7p1*c9p1*ss[34] + 420*c1p3*c3p1*c5p1*c7p2*ss[26] + 420*c1p3*c3p1*c5p1*c9p2*ss[30] + 420*c1p3*c3p1*c5p2*c7p1*ss[24] + 420*c1p3*c3p1*c5p2*c9p1*ss[26] + 420*c1p3*c3p1*c7p1*c9p2*ss[32] + 420*c1p3*c3p1*c7p2*c9p1*ss[30] + 420*c1p3*c3p2*c5p1*c7p1*ss[22] + 420*c1p3*c3p2*c5p1*c9p1*ss[24] + 420*c1p3*c3p2*c7p1*c9p1*ss[26] + 420*c1p3*c5p1*c7p1*c9p2*ss[34] + 420*c1p3*c5p1*c7p2*c9p1*ss[32] + 420*c1p3*c5p2*c7p1*c9p1*ss[30] + 210*c1p4*c3p1*c5p1*c7p1*ss[20] + 210*c1p4*c3p1*c5p1*c9p1*ss[22] + 210*c1p4*c3p1*c7p1*c9p1*ss[24] + 210*c1p4*c5p1*c7p1*c9p1*ss[26] + 210*c3p1*c5p1*c7p1*c9p4*ss[52] + 420*c3p1*c5p1*c7p2*c9p3*ss[50] + 420*c3p1*c5p1*c7p3*c9p2*ss[48] + 210*c3p1*c5p1*c7p4*c9p1*ss[46] + 420*c3p1*c5p2*c7p1*c9p3*ss[48] + 630*c3p1*c5p2*c7p2*c9p2*ss[46] + 420*c3p1*c5p2*c7p3*c9p1*ss[44] + 420*c3p1*c5p3*c7p1*c9p2*ss[44] + 420*c3p1*c5p3*c7p2*c9p1*ss[42] + 210*c3p1*c5p4*c7p1*c9p1*ss[40] + 420*c3p2*c5p1*c7p1*c9p3*ss[46] + 630*c3p2*c5p1*c7p2*c9p2*ss[44] + 420*c3p2*c5p1*c7p3*c9p1*ss[42] + 630*c3p2*c5p2*c7p1*c9p2*ss[42] + 630*c3p2*c5p2*c7p2*c9p1*ss[40] + 420*c3p2*c5p3*c7p1*c9p1*ss[38] + 420*c3p3*c5p1*c7p1*c9p2*ss[40] + 420*c3p3*c5p1*c7p2*c9p1*ss[38] + 420*c3p3*c5p2*c7p1*c9p1*ss[36] + 210*c3p4*c5p1*c7p1*c9p1*ss[34] + 42*c1p1*c3p1*c5p5*ss[30] + 42*c1p1*c3p1*c7p5*ss[40] + 42*c1p1*c3p1*c9p5*ss[50] + 105*c1p1*c3p2*c5p4*ss[28] + 105*c1p1*c3p2*c7p4*ss[36] + 105*c1p1*c3p2*c9p4*ss[44] + 140*c1p1*c3p3*c5p3*ss[26] + 140*c1p1*c3p3*c7p3*ss[32] + 140*c1p1*c3p3*c9p3*ss[38] + 105*c1p1*c3p4*c5p2*ss[24] + 105*c1p1*c3p4*c7p2*ss[28] + 105*c1p1*c3p4*c9p2*ss[32] + 42*c1p1*c3p5*c5p1*ss[22] + 42*c1p1*c3p5*c7p1*ss[24] + 42*c1p1*c3p5*c9p1*ss[26] + 42*c1p1*c5p1*c7p5*ss[42] + 42*c1p1*c5p1*c9p5*ss[52] + 105*c1p1*c5p2*c7p4*ss[40] + 105*c1p1*c5p2*c9p4*ss[48] + 140*c1p1*c5p3*c7p3*ss[38] + 140*c1p1*c5p3*c9p3*ss[44] + 105*c1p1*c5p4*c7p2*ss[36] + 105*c1p1*c5p4*c9p2*ss[40] + 42*c1p1*c5p5*c7p1*ss[34] + 42*c1p1*c5p5*c9p1*ss[36] + 42*c1p1*c7p1*c9p5*ss[54] + 105*c1p1*c7p2*c9p4*ss[52] + 140*c1p1*c7p3*c9p3*ss[50] + 105*c1p1*c7p4*c9p2*ss[48] + 42*c1p1*c7p5*c9p1*ss[46] + 105*c1p2*c3p1*c5p4*ss[26] + 105*c1p2*c3p1*c7p4*ss[34] + 105*c1p2*c3p1*c9p4*ss[42] + 210*c1p2*c3p2*c5p3*ss[24] + 210*c1p2*c3p2*c7p3*ss[30] + 210*c1p2*c3p2*c9p3*ss[36] + 210*c1p2*c3p3*c5p2*ss[22] + 210*c1p2*c3p3*c7p2*ss[26] + 210*c1p2*c3p3*c9p2*ss[30] + 105*c1p2*c3p4*c5p1*ss[20] + 105*c1p2*c3p4*c7p1*ss[22] + 105*c1p2*c3p4*c9p1*ss[24] + 105*c1p2*c5p1*c7p4*ss[36] + 105*c1p2*c5p1*c9p4*ss[44] + 210*c1p2*c5p2*c7p3*ss[34] + 210*c1p2*c5p2*c9p3*ss[40] + 210*c1p2*c5p3*c7p2*ss[32] + 210*c1p2*c5p3*c9p2*ss[36] + 105*c1p2*c5p4*c7p1*ss[30] + 105*c1p2*c5p4*c9p1*ss[32] + 105*c1p2*c7p1*c9p4*ss[46] + 210*c1p2*c7p2*c9p3*ss[44] + 210*c1p2*c7p3*c9p2*ss[42] + 105*c1p2*c7p4*c9p1*ss[40] + 140*c1p3*c3p1*c5p3*ss[22] + 140*c1p3*c3p1*c7p3*ss[28] + 140*c1p3*c3p1*c9p3*ss[34] + 210*c1p3*c3p2*c5p2*ss[20] + 210*c1p3*c3p2*c7p2*ss[24] + 210*c1p3*c3p2*c9p2*ss[28] + 140*c1p3*c3p3*c5p1*ss[18] + 140*c1p3*c3p3*c7p1*ss[20] + 140*c1p3*c3p3*c9p1*ss[22] + 140*c1p3*c5p1*c7p3*ss[30] + 140*c1p3*c5p1*c9p3*ss[36] + 210*c1p3*c5p2*c7p2*ss[28] + 210*c1p3*c5p2*c9p2*ss[32] + 140*c1p3*c5p3*c7p1*ss[26] + 140*c1p3*c5p3*c9p1*ss[28] + 140*c1p3*c7p1*c9p3*ss[38] + 210*c1p3*c7p2*c9p2*ss[36] + 140*c1p3*c7p3*c9p1*ss[34] + 105*c1p4*c3p1*c5p2*ss[18] + 105*c1p4*c3p1*c7p2*ss[22] + 105*c1p4*c3p1*c9p2*ss[26] + 105*c1p4*c3p2*c5p1*ss[16] + 105*c1p4*c3p2*c7p1*ss[18] + 105*c1p4*c3p2*c9p1*ss[20] + 105*c1p4*c5p1*c7p2*ss[24] + 105*c1p4*c5p1*c9p2*ss[28] + 105*c1p4*c5p2*c7p1*ss[22] + 105*c1p4*c5p2*c9p1*ss[24] + 105*c1p4*c7p1*c9p2*ss[30] + 105*c1p4*c7p2*c9p1*ss[28] + 42*c1p5*c3p1*c5p1*ss[14] + 42*c1p5*c3p1*c7p1*ss[16] + 42*c1p5*c3p1*c9p1*ss[18] + 42*c1p5*c5p1*c7p1*ss[18] + 42*c1p5*c5p1*c9p1*ss[20] + 42*c1p5*c7p1*c9p1*ss[22] + 42*c3p1*c5p1*c7p5*ss[44] + 42*c3p1*c5p1*c9p5*ss[54] + 105*c3p1*c5p2*c7p4*ss[42] + 105*c3p1*c5p2*c9p4*ss[50] + 140*c3p1*c5p3*c7p3*ss[40] + 140*c3p1*c5p3*c9p3*ss[46] + 105*c3p1*c5p4*c7p2*ss[38] + 105*c3p1*c5p4*c9p2*ss[42] + 42*c3p1*c5p5*c7p1*ss[36] + 42*c3p1*c5p5*c9p1*ss[38] + 42*c3p1*c7p1*c9p5*ss[56] + 105*c3p1*c7p2*c9p4*ss[54] + 140*c3p1*c7p3*c9p3*ss[52] + 105*c3p1*c7p4*c9p2*ss[50] + 42*c3p1*c7p5*c9p1*ss[48] + 105*c3p2*c5p1*c7p4*ss[40] + 105*c3p2*c5p1*c9p4*ss[48] + 210*c3p2*c5p2*c7p3*ss[38] + 210*c3p2*c5p2*c9p3*ss[44] + 210*c3p2*c5p3*c7p2*ss[36] + 210*c3p2*c5p3*c9p2*ss[40] + 105*c3p2*c5p4*c7p1*ss[34] + 105*c3p2*c5p4*c9p1*ss[36] + 105*c3p2*c7p1*c9p4*ss[50] + 210*c3p2*c7p2*c9p3*ss[48] + 210*c3p2*c7p3*c9p2*ss[46] + 105*c3p2*c7p4*c9p1*ss[44] + 140*c3p3*c5p1*c7p3*ss[36] + 140*c3p3*c5p1*c9p3*ss[42] + 210*c3p3*c5p2*c7p2*ss[34] + 210*c3p3*c5p2*c9p2*ss[38] + 140*c3p3*c5p3*c7p1*ss[32] + 140*c3p3*c5p3*c9p1*ss[34] + 140*c3p3*c7p1*c9p3*ss[44] + 210*c3p3*c7p2*c9p2*ss[42] + 140*c3p3*c7p3*c9p1*ss[40] + 105*c3p4*c5p1*c7p2*ss[32] + 105*c3p4*c5p1*c9p2*ss[36] + 105*c3p4*c5p2*c7p1*ss[30] + 105*c3p4*c5p2*c9p1*ss[32] + 105*c3p4*c7p1*c9p2*ss[38] + 105*c3p4*c7p2*c9p1*ss[36] + 42*c3p5*c5p1*c7p1*ss[28] + 42*c3p5*c5p1*c9p1*ss[30] + 42*c3p5*c7p1*c9p1*ss[32] + 42*c5p1*c7p1*c9p5*ss[58] + 105*c5p1*c7p2*c9p4*ss[56] + 140*c5p1*c7p3*c9p3*ss[54] + 105*c5p1*c7p4*c9p2*ss[52] + 42*c5p1*c7p5*c9p1*ss[50] + 105*c5p2*c7p1*c9p4*ss[54] + 210*c5p2*c7p2*c9p3*ss[52] + 210*c5p2*c7p3*c9p2*ss[50] + 105*c5p2*c7p4*c9p1*ss[48] + 140*c5p3*c7p1*c9p3*ss[50] + 210*c5p3*c7p2*c9p2*ss[48] + 140*c5p3*c7p3*c9p1*ss[46] + 105*c5p4*c7p1*c9p2*ss[46] + 105*c5p4*c7p2*c9p1*ss[44] + 42*c5p5*c7p1*c9p1*ss[42] + 7*c1p1*c3p6*ss[20] + 7*c1p1*c5p6*ss[32] + 7*c1p1*c7p6*ss[44] + 7*c1p1*c9p6*ss[56] + 21*c1p2*c3p5*ss[18] + 21*c1p2*c5p5*ss[28] + 21*c1p2*c7p5*ss[38] + 21*c1p2*c9p5*ss[48] + 35*c1p3*c3p4*ss[16] + 35*c1p3*c5p4*ss[24] + 35*c1p3*c7p4*ss[32] + 35*c1p3*c9p4*ss[40] + 35*c1p4*c3p3*ss[14] + 35*c1p4*c5p3*ss[20] + 35*c1p4*c7p3*ss[26] + 35*c1p4*c9p3*ss[32] + 21*c1p5*c3p2*ss[12] + 21*c1p5*c5p2*ss[16] + 21*c1p5*c7p2*ss[20] + 21*c1p5*c9p2*ss[24] + 7*c1p6*c3p1*ss[10] + 7*c1p6*c5p1*ss[12] + 7*c1p6*c7p1*ss[14] + 7*c1p6*c9p1*ss[16] + 7*c3p1*c5p6*ss[34] + 7*c3p1*c7p6*ss[46] + 7*c3p1*c9p6*ss[58] + 21*c3p2*c5p5*ss[32] + 21*c3p2*c7p5*ss[42] + 21*c3p2*c9p5*ss[52] + 35*c3p3*c5p4*ss[30] + 35*c3p3*c7p4*ss[38] + 35*c3p3*c9p4*ss[46] + 35*c3p4*c5p3*ss[28] + 35*c3p4*c7p3*ss[34] + 35*c3p4*c9p3*ss[40] + 21*c3p5*c5p2*ss[26] + 21*c3p5*c7p2*ss[30] + 21*c3p5*c9p2*ss[34] + 7*c3p6*c5p1*ss[24] + 7*c3p6*c7p1*ss[26] + 7*c3p6*c9p1*ss[28] + 7*c5p1*c7p6*ss[48] + 7*c5p1*c9p6*ss[60] + 21*c5p2*c7p5*ss[46] + 21*c5p2*c9p5*ss[56] + 35*c5p3*c7p4*ss[44] + 35*c5p3*c9p4*ss[52] + 35*c5p4*c7p3*ss[42] + 35*c5p4*c9p3*ss[48] + 21*c5p5*c7p2*ss[40] + 21*c5p5*c9p2*ss[44] + 7*c5p6*c7p1*ss[38] + 7*c5p6*c9p1*ss[40] + 7*c7p1*c9p6*ss[62] + 21*c7p2*c9p5*ss[60] + 35*c7p3*c9p4*ss[58] + 35*c7p4*c9p3*ss[56] + 21*c7p5*c9p2*ss[54] + 7*c7p6*c9p1*ss[52] + c1p7*ss[8] + c3p7*ss[22] + c5p7*ss[36] + c7p7*ss[50] + c9p7*ss[64])*sys_part_3;
  grad_0_num += 10*(3024*c1p1*c3p1*c5p1*c7p1*c9p5*ss[62] + 7560*c1p1*c3p1*c5p1*c7p2*c9p4*ss[60] + 10080*c1p1*c3p1*c5p1*c7p3*c9p3*ss[58] + 7560*c1p1*c3p1*c5p1*c7p4*c9p2*ss[56] + 3024*c1p1*c3p1*c5p1*c7p5*c9p1*ss[54] + 7560*c1p1*c3p1*c5p2*c7p1*c9p4*ss[58] + 15120*c1p1*c3p1*c5p2*c7p2*c9p3*ss[56] + 15120*c1p1*c3p1*c5p2*c7p3*c9p2*ss[54] + 7560*c1p1*c3p1*c5p2*c7p4*c9p1*ss[52] + 10080*c1p1*c3p1*c5p3*c7p1*c9p3*ss[54] + 15120*c1p1*c3p1*c5p3*c7p2*c9p2*ss[52] + 10080*c1p1*c3p1*c5p3*c7p3*c9p1*ss[50] + 7560*c1p1*c3p1*c5p4*c7p1*c9p2*ss[50] + 7560*c1p1*c3p1*c5p4*c7p2*c9p1*ss[48] + 3024*c1p1*c3p1*c5p5*c7p1*c9p1*ss[46] + 7560*c1p1*c3p2*c5p1*c7p1*c9p4*ss[56] + 15120*c1p1*c3p2*c5p1*c7p2*c9p3*ss[54] + 15120*c1p1*c3p2*c5p1*c7p3*c9p2*ss[52] + 7560*c1p1*c3p2*c5p1*c7p4*c9p1*ss[50] + 15120*c1p1*c3p2*c5p2*c7p1*c9p3*ss[52] + 22680*c1p1*c3p2*c5p2*c7p2*c9p2*ss[50] + 15120*c1p1*c3p2*c5p2*c7p3*c9p1*ss[48] + 15120*c1p1*c3p2*c5p3*c7p1*c9p2*ss[48] + 15120*c1p1*c3p2*c5p3*c7p2*c9p1*ss[46] + 7560*c1p1*c3p2*c5p4*c7p1*c9p1*ss[44] + 10080*c1p1*c3p3*c5p1*c7p1*c9p3*ss[50] + 15120*c1p1*c3p3*c5p1*c7p2*c9p2*ss[48] + 10080*c1p1*c3p3*c5p1*c7p3*c9p1*ss[46] + 15120*c1p1*c3p3*c5p2*c7p1*c9p2*ss[46] + 15120*c1p1*c3p3*c5p2*c7p2*c9p1*ss[44] + 10080*c1p1*c3p3*c5p3*c7p1*c9p1*ss[42] + 7560*c1p1*c3p4*c5p1*c7p1*c9p2*ss[44] + 7560*c1p1*c3p4*c5p1*c7p2*c9p1*ss[42] + 7560*c1p1*c3p4*c5p2*c7p1*c9p1*ss[40] + 3024*c1p1*c3p5*c5p1*c7p1*c9p1*ss[38] + 7560*c1p2*c3p1*c5p1*c7p1*c9p4*ss[54] + 15120*c1p2*c3p1*c5p1*c7p2*c9p3*ss[52] + 15120*c1p2*c3p1*c5p1*c7p3*c9p2*ss[50] + 7560*c1p2*c3p1*c5p1*c7p4*c9p1*ss[48] + 15120*c1p2*c3p1*c5p2*c7p1*c9p3*ss[50] + 22680*c1p2*c3p1*c5p2*c7p2*c9p2*ss[48] + 15120*c1p2*c3p1*c5p2*c7p3*c9p1*ss[46] + 15120*c1p2*c3p1*c5p3*c7p1*c9p2*ss[46] + 15120*c1p2*c3p1*c5p3*c7p2*c9p1*ss[44] + 7560*c1p2*c3p1*c5p4*c7p1*c9p1*ss[42] + 15120*c1p2*c3p2*c5p1*c7p1*c9p3*ss[48] + 22680*c1p2*c3p2*c5p1*c7p2*c9p2*ss[46] + 15120*c1p2*c3p2*c5p1*c7p3*c9p1*ss[44] + 22680*c1p2*c3p2*c5p2*c7p1*c9p2*ss[44] + 22680*c1p2*c3p2*c5p2*c7p2*c9p1*ss[42] + 15120*c1p2*c3p2*c5p3*c7p1*c9p1*ss[40] + 15120*c1p2*c3p3*c5p1*c7p1*c9p2*ss[42] + 15120*c1p2*c3p3*c5p1*c7p2*c9p1*ss[40] + 15120*c1p2*c3p3*c5p2*c7p1*c9p1*ss[38] + 7560*c1p2*c3p4*c5p1*c7p1*c9p1*ss[36] + 10080*c1p3*c3p1*c5p1*c7p1*c9p3*ss[46] + 15120*c1p3*c3p1*c5p1*c7p2*c9p2*ss[44] + 10080*c1p3*c3p1*c5p1*c7p3*c9p1*ss[42] + 15120*c1p3*c3p1*c5p2*c7p1*c9p2*ss[42] + 15120*c1p3*c3p1*c5p2*c7p2*c9p1*ss[40] + 10080*c1p3*c3p1*c5p3*c7p1*c9p1*ss[38] + 15120*c1p3*c3p2*c5p1*c7p1*c9p2*ss[40] + 15120*c1p3*c3p2*c5p1*c7p2*c9p1*ss[38] + 15120*c1p3*c3p2*c5p2*c7p1*c9p1*ss[36] + 10080*c1p3*c3p3*c5p1*c7p1*c9p1*ss[34] + 7560*c1p4*c3p1*c5p1*c7p1*c9p2*ss[38] + 7560*c1p4*c3p1*c5p1*c7p2*c9p1*ss[36] + 7560*c1p4*c3p1*c5p2*c7p1*c9p1*ss[34] + 7560*c1p4*c3p2*c5p1*c7p1*c9p1*ss[32] + 3024*c1p5*c3p1*c5p1*c7p1*c9p1*ss[30] + 504*c1p1*c3p1*c5p1*c7p6*ss[52] + 504*c1p1*c3p1*c5p1*c9p6*ss[64] + 1512*c1p1*c3p1*c5p2*c7p5*ss[50] + 1512*c1p1*c3p1*c5p2*c9p5*ss[60] + 2520*c1p1*c3p1*c5p3*c7p4*ss[48] + 2520*c1p1*c3p1*c5p3*c9p4*ss[56] + 2520*c1p1*c3p1*c5p4*c7p3*ss[46] + 2520*c1p1*c3p1*c5p4*c9p3*ss[52] + 1512*c1p1*c3p1*c5p5*c7p2*ss[44] + 1512*c1p1*c3p1*c5p5*c9p2*ss[48] + 504*c1p1*c3p1*c5p6*c7p1*ss[42] + 504*c1p1*c3p1*c5p6*c9p1*ss[44] + 504*c1p1*c3p1*c7p1*c9p6*ss[66] + 1512*c1p1*c3p1*c7p2*c9p5*ss[64] + 2520*c1p1*c3p1*c7p3*c9p4*ss[62] + 2520*c1p1*c3p1*c7p4*c9p3*ss[60] + 1512*c1p1*c3p1*c7p5*c9p2*ss[58] + 504*c1p1*c3p1*c7p6*c9p1*ss[56] + 1512*c1p1*c3p2*c5p1*c7p5*ss[48] + 1512*c1p1*c3p2*c5p1*c9p5*ss[58] + 3780*c1p1*c3p2*c5p2*c7p4*ss[46] + 3780*c1p1*c3p2*c5p2*c9p4*ss[54] + 5040*c1p1*c3p2*c5p3*c7p3*ss[44] + 5040*c1p1*c3p2*c5p3*c9p3*ss[50] + 3780*c1p1*c3p2*c5p4*c7p2*ss[42] + 3780*c1p1*c3p2*c5p4*c9p2*ss[46] + 1512*c1p1*c3p2*c5p5*c7p1*ss[40] + 1512*c1p1*c3p2*c5p5*c9p1*ss[42] + 1512*c1p1*c3p2*c7p1*c9p5*ss[60] + 3780*c1p1*c3p2*c7p2*c9p4*ss[58] + 5040*c1p1*c3p2*c7p3*c9p3*ss[56] + 3780*c1p1*c3p2*c7p4*c9p2*ss[54] + 1512*c1p1*c3p2*c7p5*c9p1*ss[52] + 2520*c1p1*c3p3*c5p1*c7p4*ss[44] + 2520*c1p1*c3p3*c5p1*c9p4*ss[52] + 5040*c1p1*c3p3*c5p2*c7p3*ss[42] + 5040*c1p1*c3p3*c5p2*c9p3*ss[48] + 5040*c1p1*c3p3*c5p3*c7p2*ss[40] + 5040*c1p1*c3p3*c5p3*c9p2*ss[44] + 2520*c1p1*c3p3*c5p4*c7p1*ss[38] + 2520*c1p1*c3p3*c5p4*c9p1*ss[40] + 2520*c1p1*c3p3*c7p1*c9p4*ss[54] + 5040*c1p1*c3p3*c7p2*c9p3*ss[52] + 5040*c1p1*c3p3*c7p3*c9p2*ss[50] + 2520*c1p1*c3p3*c7p4*c9p1*ss[48] + 2520*c1p1*c3p4*c5p1*c7p3*ss[40] + 2520*c1p1*c3p4*c5p1*c9p3*ss[46] + 3780*c1p1*c3p4*c5p2*c7p2*ss[38] + 3780*c1p1*c3p4*c5p2*c9p2*ss[42] + 2520*c1p1*c3p4*c5p3*c7p1*ss[36] + 2520*c1p1*c3p4*c5p3*c9p1*ss[38] + 2520*c1p1*c3p4*c7p1*c9p3*ss[48] + 3780*c1p1*c3p4*c7p2*c9p2*ss[46] + 2520*c1p1*c3p4*c7p3*c9p1*ss[44] + 1512*c1p1*c3p5*c5p1*c7p2*ss[36] + 1512*c1p1*c3p5*c5p1*c9p2*ss[40] + 1512*c1p1*c3p5*c5p2*c7p1*ss[34] + 1512*c1p1*c3p5*c5p2*c9p1*ss[36] + 1512*c1p1*c3p5*c7p1*c9p2*ss[42] + 1512*c1p1*c3p5*c7p2*c9p1*ss[40] + 504*c1p1*c3p6*c5p1*c7p1*ss[32] + 504*c1p1*c3p6*c5p1*c9p1*ss[34] + 504*c1p1*c3p6*c7p1*c9p1*ss[36] + 504*c1p1*c5p1*c7p1*c9p6*ss[68] + 1512*c1p1*c5p1*c7p2*c9p5*ss[66] + 2520*c1p1*c5p1*c7p3*c9p4*ss[64] + 2520*c1p1*c5p1*c7p4*c9p3*ss[62] + 1512*c1p1*c5p1*c7p5*c9p2*ss[60] + 504*c1p1*c5p1*c7p6*c9p1*ss[58] + 1512*c1p1*c5p2*c7p1*c9p5*ss[64] + 3780*c1p1*c5p2*c7p2*c9p4*ss[62] + 5040*c1p1*c5p2*c7p3*c9p3*ss[60] + 3780*c1p1*c5p2*c7p4*c9p2*ss[58] + 1512*c1p1*c5p2*c7p5*c9p1*ss[56] + 2520*c1p1*c5p3*c7p1*c9p4*ss[60] + 5040*c1p1*c5p3*c7p2*c9p3*ss[58] + 5040*c1p1*c5p3*c7p3*c9p2*ss[56] + 2520*c1p1*c5p3*c7p4*c9p1*ss[54] + 2520*c1p1*c5p4*c7p1*c9p3*ss[56] + 3780*c1p1*c5p4*c7p2*c9p2*ss[54] + 2520*c1p1*c5p4*c7p3*c9p1*ss[52] + 1512*c1p1*c5p5*c7p1*c9p2*ss[52] + 1512*c1p1*c5p5*c7p2*c9p1*ss[50] + 504*c1p1*c5p6*c7p1*c9p1*ss[48] + 1512*c1p2*c3p1*c5p1*c7p5*ss[46] + 1512*c1p2*c3p1*c5p1*c9p5*ss[56] + 3780*c1p2*c3p1*c5p2*c7p4*ss[44] + 3780*c1p2*c3p1*c5p2*c9p4*ss[52] + 5040*c1p2*c3p1*c5p3*c7p3*ss[42] + 5040*c1p2*c3p1*c5p3*c9p3*ss[48] + 3780*c1p2*c3p1*c5p4*c7p2*ss[40] + 3780*c1p2*c3p1*c5p4*c9p2*ss[44] + 1512*c1p2*c3p1*c5p5*c7p1*ss[38] + 1512*c1p2*c3p1*c5p5*c9p1*ss[40] + 1512*c1p2*c3p1*c7p1*c9p5*ss[58] + 3780*c1p2*c3p1*c7p2*c9p4*ss[56] + 5040*c1p2*c3p1*c7p3*c9p3*ss[54] + 3780*c1p2*c3p1*c7p4*c9p2*ss[52] + 1512*c1p2*c3p1*c7p5*c9p1*ss[50] + 3780*c1p2*c3p2*c5p1*c7p4*ss[42] + 3780*c1p2*c3p2*c5p1*c9p4*ss[50] + 7560*c1p2*c3p2*c5p2*c7p3*ss[40] + 7560*c1p2*c3p2*c5p2*c9p3*ss[46] + 7560*c1p2*c3p2*c5p3*c7p2*ss[38] + 7560*c1p2*c3p2*c5p3*c9p2*ss[42] + 3780*c1p2*c3p2*c5p4*c7p1*ss[36] + 3780*c1p2*c3p2*c5p4*c9p1*ss[38] + 3780*c1p2*c3p2*c7p1*c9p4*ss[52] + 7560*c1p2*c3p2*c7p2*c9p3*ss[50] + 7560*c1p2*c3p2*c7p3*c9p2*ss[48] + 3780*c1p2*c3p2*c7p4*c9p1*ss[46] + 5040*c1p2*c3p3*c5p1*c7p3*ss[38] + 5040*c1p2*c3p3*c5p1*c9p3*ss[44] + 7560*c1p2*c3p3*c5p2*c7p2*ss[36] + 7560*c1p2*c3p3*c5p2*c9p2*ss[40] + 5040*c1p2*c3p3*c5p3*c7p1*ss[34] + 5040*c1p2*c3p3*c5p3*c9p1*ss[36] + 5040*c1p2*c3p3*c7p1*c9p3*ss[46] + 7560*c1p2*c3p3*c7p2*c9p2*ss[44] + 5040*c1p2*c3p3*c7p3*c9p1*ss[42] + 3780*c1p2*c3p4*c5p1*c7p2*ss[34] + 3780*c1p2*c3p4*c5p1*c9p2*ss[38] + 3780*c1p2*c3p4*c5p2*c7p1*ss[32] + 3780*c1p2*c3p4*c5p2*c9p1*ss[34] + 3780*c1p2*c3p4*c7p1*c9p2*ss[40] + 3780*c1p2*c3p4*c7p2*c9p1*ss[38] + 1512*c1p2*c3p5*c5p1*c7p1*ss[30] + 1512*c1p2*c3p5*c5p1*c9p1*ss[32] + 1512*c1p2*c3p5*c7p1*c9p1*ss[34] + 1512*c1p2*c5p1*c7p1*c9p5*ss[60] + 3780*c1p2*c5p1*c7p2*c9p4*ss[58] + 5040*c1p2*c5p1*c7p3*c9p3*ss[56] + 3780*c1p2*c5p1*c7p4*c9p2*ss[54] + 1512*c1p2*c5p1*c7p5*c9p1*ss[52] + 3780*c1p2*c5p2*c7p1*c9p4*ss[56] + 7560*c1p2*c5p2*c7p2*c9p3*ss[54] + 7560*c1p2*c5p2*c7p3*c9p2*ss[52] + 3780*c1p2*c5p2*c7p4*c9p1*ss[50] + 5040*c1p2*c5p3*c7p1*c9p3*ss[52] + 7560*c1p2*c5p3*c7p2*c9p2*ss[50] + 5040*c1p2*c5p3*c7p3*c9p1*ss[48] + 3780*c1p2*c5p4*c7p1*c9p2*ss[48] + 3780*c1p2*c5p4*c7p2*c9p1*ss[46] + 1512*c1p2*c5p5*c7p1*c9p1*ss[44] + 2520*c1p3*c3p1*c5p1*c7p4*ss[40] + 2520*c1p3*c3p1*c5p1*c9p4*ss[48] + 5040*c1p3*c3p1*c5p2*c7p3*ss[38] + 5040*c1p3*c3p1*c5p2*c9p3*ss[44] + 5040*c1p3*c3p1*c5p3*c7p2*ss[36] + 5040*c1p3*c3p1*c5p3*c9p2*ss[40] + 2520*c1p3*c3p1*c5p4*c7p1*ss[34] + 2520*c1p3*c3p1*c5p4*c9p1*ss[36] + 2520*c1p3*c3p1*c7p1*c9p4*ss[50] + 5040*c1p3*c3p1*c7p2*c9p3*ss[48] + 5040*c1p3*c3p1*c7p3*c9p2*ss[46] + 2520*c1p3*c3p1*c7p4*c9p1*ss[44] + 5040*c1p3*c3p2*c5p1*c7p3*ss[36] + 5040*c1p3*c3p2*c5p1*c9p3*ss[42] + 7560*c1p3*c3p2*c5p2*c7p2*ss[34] + 7560*c1p3*c3p2*c5p2*c9p2*ss[38] + 5040*c1p3*c3p2*c5p3*c7p1*ss[32] + 5040*c1p3*c3p2*c5p3*c9p1*ss[34] + 5040*c1p3*c3p2*c7p1*c9p3*ss[44] + 7560*c1p3*c3p2*c7p2*c9p2*ss[42] + 5040*c1p3*c3p2*c7p3*c9p1*ss[40] + 5040*c1p3*c3p3*c5p1*c7p2*ss[32] + 5040*c1p3*c3p3*c5p1*c9p2*ss[36] + 5040*c1p3*c3p3*c5p2*c7p1*ss[30] + 5040*c1p3*c3p3*c5p2*c9p1*ss[32] + 5040*c1p3*c3p3*c7p1*c9p2*ss[38] + 5040*c1p3*c3p3*c7p2*c9p1*ss[36] + 2520*c1p3*c3p4*c5p1*c7p1*ss[28] + 2520*c1p3*c3p4*c5p1*c9p1*ss[30] + 2520*c1p3*c3p4*c7p1*c9p1*ss[32] + 2520*c1p3*c5p1*c7p1*c9p4*ss[52] + 5040*c1p3*c5p1*c7p2*c9p3*ss[50] + 5040*c1p3*c5p1*c7p3*c9p2*ss[48] + 2520*c1p3*c5p1*c7p4*c9p1*ss[46] + 5040*c1p3*c5p2*c7p1*c9p3*ss[48] + 7560*c1p3*c5p2*c7p2*c9p2*ss[46] + 5040*c1p3*c5p2*c7p3*c9p1*ss[44] + 5040*c1p3*c5p3*c7p1*c9p2*ss[44] + 5040*c1p3*c5p3*c7p2*c9p1*ss[42] + 2520*c1p3*c5p4*c7p1*c9p1*ss[40] + 2520*c1p4*c3p1*c5p1*c7p3*ss[34] + 2520*c1p4*c3p1*c5p1*c9p3*ss[40] + 3780*c1p4*c3p1*c5p2*c7p2*ss[32] + 3780*c1p4*c3p1*c5p2*c9p2*ss[36] + 2520*c1p4*c3p1*c5p3*c7p1*ss[30] + 2520*c1p4*c3p1*c5p3*c9p1*ss[32] + 2520*c1p4*c3p1*c7p1*c9p3*ss[42] + 3780*c1p4*c3p1*c7p2*c9p2*ss[40] + 2520*c1p4*c3p1*c7p3*c9p1*ss[38] + 3780*c1p4*c3p2*c5p1*c7p2*ss[30] + 3780*c1p4*c3p2*c5p1*c9p2*ss[34] + 3780*c1p4*c3p2*c5p2*c7p1*ss[28] + 3780*c1p4*c3p2*c5p2*c9p1*ss[30] + 3780*c1p4*c3p2*c7p1*c9p2*ss[36] + 3780*c1p4*c3p2*c7p2*c9p1*ss[34] + 2520*c1p4*c3p3*c5p1*c7p1*ss[26] + 2520*c1p4*c3p3*c5p1*c9p1*ss[28] + 2520*c1p4*c3p3*c7p1*c9p1*ss[30] + 2520*c1p4*c5p1*c7p1*c9p3*ss[44] + 3780*c1p4*c5p1*c7p2*c9p2*ss[42] + 2520*c1p4*c5p1*c7p3*c9p1*ss[40] + 3780*c1p4*c5p2*c7p1*c9p2*ss[40] + 3780*c1p4*c5p2*c7p2*c9p1*ss[38] + 2520*c1p4*c5p3*c7p1*c9p1*ss[36] + 1512*c1p5*c3p1*c5p1*c7p2*ss[28] + 1512*c1p5*c3p1*c5p1*c9p2*ss[32] + 1512*c1p5*c3p1*c5p2*c7p1*ss[26] + 1512*c1p5*c3p1*c5p2*c9p1*ss[28] + 1512*c1p5*c3p1*c7p1*c9p2*ss[34] + 1512*c1p5*c3p1*c7p2*c9p1*ss[32] + 1512*c1p5*c3p2*c5p1*c7p1*ss[24] + 1512*c1p5*c3p2*c5p1*c9p1*ss[26] + 1512*c1p5*c3p2*c7p1*c9p1*ss[28] + 1512*c1p5*c5p1*c7p1*c9p2*ss[36] + 1512*c1p5*c5p1*c7p2*c9p1*ss[34] + 1512*c1p5*c5p2*c7p1*c9p1*ss[32] + 504*c1p6*c3p1*c5p1*c7p1*ss[22] + 504*c1p6*c3p1*c5p1*c9p1*ss[24] + 504*c1p6*c3p1*c7p1*c9p1*ss[26] + 504*c1p6*c5p1*c7p1*c9p1*ss[28] + 504*c3p1*c5p1*c7p1*c9p6*ss[70] + 1512*c3p1*c5p1*c7p2*c9p5*ss[68] + 2520*c3p1*c5p1*c7p3*c9p4*ss[66] + 2520*c3p1*c5p1*c7p4*c9p3*ss[64] + 1512*c3p1*c5p1*c7p5*c9p2*ss[62] + 504*c3p1*c5p1*c7p6*c9p1*ss[60] + 1512*c3p1*c5p2*c7p1*c9p5*ss[66] + 3780*c3p1*c5p2*c7p2*c9p4*ss[64] + 5040*c3p1*c5p2*c7p3*c9p3*ss[62] + 3780*c3p1*c5p2*c7p4*c9p2*ss[60] + 1512*c3p1*c5p2*c7p5*c9p1*ss[58] + 2520*c3p1*c5p3*c7p1*c9p4*ss[62] + 5040*c3p1*c5p3*c7p2*c9p3*ss[60] + 5040*c3p1*c5p3*c7p3*c9p2*ss[58] + 2520*c3p1*c5p3*c7p4*c9p1*ss[56] + 2520*c3p1*c5p4*c7p1*c9p3*ss[58] + 3780*c3p1*c5p4*c7p2*c9p2*ss[56] + 2520*c3p1*c5p4*c7p3*c9p1*ss[54] + 1512*c3p1*c5p5*c7p1*c9p2*ss[54] + 1512*c3p1*c5p5*c7p2*c9p1*ss[52] + 504*c3p1*c5p6*c7p1*c9p1*ss[50] + 1512*c3p2*c5p1*c7p1*c9p5*ss[64] + 3780*c3p2*c5p1*c7p2*c9p4*ss[62] + 5040*c3p2*c5p1*c7p3*c9p3*ss[60] + 3780*c3p2*c5p1*c7p4*c9p2*ss[58] + 1512*c3p2*c5p1*c7p5*c9p1*ss[56] + 3780*c3p2*c5p2*c7p1*c9p4*ss[60] + 7560*c3p2*c5p2*c7p2*c9p3*ss[58] + 7560*c3p2*c5p2*c7p3*c9p2*ss[56] + 3780*c3p2*c5p2*c7p4*c9p1*ss[54] + 5040*c3p2*c5p3*c7p1*c9p3*ss[56] + 7560*c3p2*c5p3*c7p2*c9p2*ss[54] + 5040*c3p2*c5p3*c7p3*c9p1*ss[52] + 3780*c3p2*c5p4*c7p1*c9p2*ss[52] + 3780*c3p2*c5p4*c7p2*c9p1*ss[50] + 1512*c3p2*c5p5*c7p1*c9p1*ss[48] + 2520*c3p3*c5p1*c7p1*c9p4*ss[58] + 5040*c3p3*c5p1*c7p2*c9p3*ss[56] + 5040*c3p3*c5p1*c7p3*c9p2*ss[54] + 2520*c3p3*c5p1*c7p4*c9p1*ss[52] + 5040*c3p3*c5p2*c7p1*c9p3*ss[54] + 7560*c3p3*c5p2*c7p2*c9p2*ss[52] + 5040*c3p3*c5p2*c7p3*c9p1*ss[50] + 5040*c3p3*c5p3*c7p1*c9p2*ss[50] + 5040*c3p3*c5p3*c7p2*c9p1*ss[48] + 2520*c3p3*c5p4*c7p1*c9p1*ss[46] + 2520*c3p4*c5p1*c7p1*c9p3*ss[52] + 3780*c3p4*c5p1*c7p2*c9p2*ss[50] + 2520*c3p4*c5p1*c7p3*c9p1*ss[48] + 3780*c3p4*c5p2*c7p1*c9p2*ss[48] + 3780*c3p4*c5p2*c7p2*c9p1*ss[46] + 2520*c3p4*c5p3*c7p1*c9p1*ss[44] + 1512*c3p5*c5p1*c7p1*c9p2*ss[46] + 1512*c3p5*c5p1*c7p2*c9p1*ss[44] + 1512*c3p5*c5p2*c7p1*c9p1*ss[42] + 504*c3p6*c5p1*c7p1*c9p1*ss[40] + 72*c1p1*c3p1*c5p7*ss[40] + 72*c1p1*c3p1*c7p7*ss[54] + 72*c1p1*c3p1*c9p7*ss[68] + 252*c1p1*c3p2*c5p6*ss[38] + 252*c1p1*c3p2*c7p6*ss[50] + 252*c1p1*c3p2*c9p6*ss[62] + 504*c1p1*c3p3*c5p5*ss[36] + 504*c1p1*c3p3*c7p5*ss[46] + 504*c1p1*c3p3*c9p5*ss[56] + 630*c1p1*c3p4*c5p4*ss[34] + 630*c1p1*c3p4*c7p4*ss[42] + 630*c1p1*c3p4*c9p4*ss[50] + 504*c1p1*c3p5*c5p3*ss[32] + 504*c1p1*c3p5*c7p3*ss[38] + 504*c1p1*c3p5*c9p3*ss[44] + 252*c1p1*c3p6*c5p2*ss[30] + 252*c1p1*c3p6*c7p2*ss[34] + 252*c1p1*c3p6*c9p2*ss[38] + 72*c1p1*c3p7*c5p1*ss[28] + 72*c1p1*c3p7*c7p1*ss[30] + 72*c1p1*c3p7*c9p1*ss[32] + 72*c1p1*c5p1*c7p7*ss[56] + 72*c1p1*c5p1*c9p7*ss[70] + 252*c1p1*c5p2*c7p6*ss[54] + 252*c1p1*c5p2*c9p6*ss[66] + 504*c1p1*c5p3*c7p5*ss[52] + 504*c1p1*c5p3*c9p5*ss[62] + 630*c1p1*c5p4*c7p4*ss[50] + 630*c1p1*c5p4*c9p4*ss[58] + 504*c1p1*c5p5*c7p3*ss[48] + 504*c1p1*c5p5*c9p3*ss[54] + 252*c1p1*c5p6*c7p2*ss[46] + 252*c1p1*c5p6*c9p2*ss[50] + 72*c1p1*c5p7*c7p1*ss[44] + 72*c1p1*c5p7*c9p1*ss[46] + 72*c1p1*c7p1*c9p7*ss[72] + 252*c1p1*c7p2*c9p6*ss[70] + 504*c1p1*c7p3*c9p5*ss[68] + 630*c1p1*c7p4*c9p4*ss[66] + 504*c1p1*c7p5*c9p3*ss[64] + 252*c1p1*c7p6*c9p2*ss[62] + 72*c1p1*c7p7*c9p1*ss[60] + 252*c1p2*c3p1*c5p6*ss[36] + 252*c1p2*c3p1*c7p6*ss[48] + 252*c1p2*c3p1*c9p6*ss[60] + 756*c1p2*c3p2*c5p5*ss[34] + 756*c1p2*c3p2*c7p5*ss[44] + 756*c1p2*c3p2*c9p5*ss[54] + 1260*c1p2*c3p3*c5p4*ss[32] + 1260*c1p2*c3p3*c7p4*ss[40] + 1260*c1p2*c3p3*c9p4*ss[48] + 1260*c1p2*c3p4*c5p3*ss[30] + 1260*c1p2*c3p4*c7p3*ss[36] + 1260*c1p2*c3p4*c9p3*ss[42] + 756*c1p2*c3p5*c5p2*ss[28] + 756*c1p2*c3p5*c7p2*ss[32] + 756*c1p2*c3p5*c9p2*ss[36] + 252*c1p2*c3p6*c5p1*ss[26] + 252*c1p2*c3p6*c7p1*ss[28] + 252*c1p2*c3p6*c9p1*ss[30] + 252*c1p2*c5p1*c7p6*ss[50] + 252*c1p2*c5p1*c9p6*ss[62] + 756*c1p2*c5p2*c7p5*ss[48] + 756*c1p2*c5p2*c9p5*ss[58] + 1260*c1p2*c5p3*c7p4*ss[46] + 1260*c1p2*c5p3*c9p4*ss[54] + 1260*c1p2*c5p4*c7p3*ss[44] + 1260*c1p2*c5p4*c9p3*ss[50] + 756*c1p2*c5p5*c7p2*ss[42] + 756*c1p2*c5p5*c9p2*ss[46] + 252*c1p2*c5p6*c7p1*ss[40] + 252*c1p2*c5p6*c9p1*ss[42] + 252*c1p2*c7p1*c9p6*ss[64] + 756*c1p2*c7p2*c9p5*ss[62] + 1260*c1p2*c7p3*c9p4*ss[60] + 1260*c1p2*c7p4*c9p3*ss[58] + 756*c1p2*c7p5*c9p2*ss[56] + 252*c1p2*c7p6*c9p1*ss[54] + 504*c1p3*c3p1*c5p5*ss[32] + 504*c1p3*c3p1*c7p5*ss[42] + 504*c1p3*c3p1*c9p5*ss[52] + 1260*c1p3*c3p2*c5p4*ss[30] + 1260*c1p3*c3p2*c7p4*ss[38] + 1260*c1p3*c3p2*c9p4*ss[46] + 1680*c1p3*c3p3*c5p3*ss[28] + 1680*c1p3*c3p3*c7p3*ss[34] + 1680*c1p3*c3p3*c9p3*ss[40] + 1260*c1p3*c3p4*c5p2*ss[26] + 1260*c1p3*c3p4*c7p2*ss[30] + 1260*c1p3*c3p4*c9p2*ss[34] + 504*c1p3*c3p5*c5p1*ss[24] + 504*c1p3*c3p5*c7p1*ss[26] + 504*c1p3*c3p5*c9p1*ss[28] + 504*c1p3*c5p1*c7p5*ss[44] + 504*c1p3*c5p1*c9p5*ss[54] + 1260*c1p3*c5p2*c7p4*ss[42] + 1260*c1p3*c5p2*c9p4*ss[50] + 1680*c1p3*c5p3*c7p3*ss[40] + 1680*c1p3*c5p3*c9p3*ss[46] + 1260*c1p3*c5p4*c7p2*ss[38] + 1260*c1p3*c5p4*c9p2*ss[42] + 504*c1p3*c5p5*c7p1*ss[36] + 504*c1p3*c5p5*c9p1*ss[38] + 504*c1p3*c7p1*c9p5*ss[56] + 1260*c1p3*c7p2*c9p4*ss[54] + 1680*c1p3*c7p3*c9p3*ss[52] + 1260*c1p3*c7p4*c9p2*ss[50] + 504*c1p3*c7p5*c9p1*ss[48] + 630*c1p4*c3p1*c5p4*ss[28] + 630*c1p4*c3p1*c7p4*ss[36] + 630*c1p4*c3p1*c9p4*ss[44] + 1260*c1p4*c3p2*c5p3*ss[26] + 1260*c1p4*c3p2*c7p3*ss[32] + 1260*c1p4*c3p2*c9p3*ss[38] + 1260*c1p4*c3p3*c5p2*ss[24] + 1260*c1p4*c3p3*c7p2*ss[28] + 1260*c1p4*c3p3*c9p2*ss[32] + 630*c1p4*c3p4*c5p1*ss[22] + 630*c1p4*c3p4*c7p1*ss[24] + 630*c1p4*c3p4*c9p1*ss[26] + 630*c1p4*c5p1*c7p4*ss[38] + 630*c1p4*c5p1*c9p4*ss[46] + 1260*c1p4*c5p2*c7p3*ss[36] + 1260*c1p4*c5p2*c9p3*ss[42] + 1260*c1p4*c5p3*c7p2*ss[34] + 1260*c1p4*c5p3*c9p2*ss[38] + 630*c1p4*c5p4*c7p1*ss[32] + 630*c1p4*c5p4*c9p1*ss[34] + 630*c1p4*c7p1*c9p4*ss[48] + 1260*c1p4*c7p2*c9p3*ss[46] + 1260*c1p4*c7p3*c9p2*ss[44] + 630*c1p4*c7p4*c9p1*ss[42] + 504*c1p5*c3p1*c5p3*ss[24] + 504*c1p5*c3p1*c7p3*ss[30] + 504*c1p5*c3p1*c9p3*ss[36] + 756*c1p5*c3p2*c5p2*ss[22] + 756*c1p5*c3p2*c7p2*ss[26] + 756*c1p5*c3p2*c9p2*ss[30] + 504*c1p5*c3p3*c5p1*ss[20] + 504*c1p5*c3p3*c7p1*ss[22] + 504*c1p5*c3p3*c9p1*ss[24] + 504*c1p5*c5p1*c7p3*ss[32] + 504*c1p5*c5p1*c9p3*ss[38] + 756*c1p5*c5p2*c7p2*ss[30] + 756*c1p5*c5p2*c9p2*ss[34] + 504*c1p5*c5p3*c7p1*ss[28] + 504*c1p5*c5p3*c9p1*ss[30] + 504*c1p5*c7p1*c9p3*ss[40] + 756*c1p5*c7p2*c9p2*ss[38] + 504*c1p5*c7p3*c9p1*ss[36] + 252*c1p6*c3p1*c5p2*ss[20] + 252*c1p6*c3p1*c7p2*ss[24] + 252*c1p6*c3p1*c9p2*ss[28] + 252*c1p6*c3p2*c5p1*ss[18] + 252*c1p6*c3p2*c7p1*ss[20] + 252*c1p6*c3p2*c9p1*ss[22] + 252*c1p6*c5p1*c7p2*ss[26] + 252*c1p6*c5p1*c9p2*ss[30] + 252*c1p6*c5p2*c7p1*ss[24] + 252*c1p6*c5p2*c9p1*ss[26] + 252*c1p6*c7p1*c9p2*ss[32] + 252*c1p6*c7p2*c9p1*ss[30] + 72*c1p7*c3p1*c5p1*ss[16] + 72*c1p7*c3p1*c7p1*ss[18] + 72*c1p7*c3p1*c9p1*ss[20] + 72*c1p7*c5p1*c7p1*ss[20] + 72*c1p7*c5p1*c9p1*ss[22] + 72*c1p7*c7p1*c9p1*ss[24] + 72*c3p1*c5p1*c7p7*ss[58] + 72*c3p1*c5p1*c9p7*ss[72] + 252*c3p1*c5p2*c7p6*ss[56] + 252*c3p1*c5p2*c9p6*ss[68] + 504*c3p1*c5p3*c7p5*ss[54] + 504*c3p1*c5p3*c9p5*ss[64] + 630*c3p1*c5p4*c7p4*ss[52] + 630*c3p1*c5p4*c9p4*ss[60] + 504*c3p1*c5p5*c7p3*ss[50] + 504*c3p1*c5p5*c9p3*ss[56] + 252*c3p1*c5p6*c7p2*ss[48] + 252*c3p1*c5p6*c9p2*ss[52] + 72*c3p1*c5p7*c7p1*ss[46] + 72*c3p1*c5p7*c9p1*ss[48] + 72*c3p1*c7p1*c9p7*ss[74] + 252*c3p1*c7p2*c9p6*ss[72] + 504*c3p1*c7p3*c9p5*ss[70] + 630*c3p1*c7p4*c9p4*ss[68] + 504*c3p1*c7p5*c9p3*ss[66] + 252*c3p1*c7p6*c9p2*ss[64] + 72*c3p1*c7p7*c9p1*ss[62] + 252*c3p2*c5p1*c7p6*ss[54] + 252*c3p2*c5p1*c9p6*ss[66] + 756*c3p2*c5p2*c7p5*ss[52] + 756*c3p2*c5p2*c9p5*ss[62] + 1260*c3p2*c5p3*c7p4*ss[50] + 1260*c3p2*c5p3*c9p4*ss[58] + 1260*c3p2*c5p4*c7p3*ss[48] + 1260*c3p2*c5p4*c9p3*ss[54] + 756*c3p2*c5p5*c7p2*ss[46] + 756*c3p2*c5p5*c9p2*ss[50] + 252*c3p2*c5p6*c7p1*ss[44] + 252*c3p2*c5p6*c9p1*ss[46] + 252*c3p2*c7p1*c9p6*ss[68] + 756*c3p2*c7p2*c9p5*ss[66] + 1260*c3p2*c7p3*c9p4*ss[64] + 1260*c3p2*c7p4*c9p3*ss[62] + 756*c3p2*c7p5*c9p2*ss[60] + 252*c3p2*c7p6*c9p1*ss[58] + 504*c3p3*c5p1*c7p5*ss[50] + 504*c3p3*c5p1*c9p5*ss[60] + 1260*c3p3*c5p2*c7p4*ss[48] + 1260*c3p3*c5p2*c9p4*ss[56] + 1680*c3p3*c5p3*c7p3*ss[46] + 1680*c3p3*c5p3*c9p3*ss[52] + 1260*c3p3*c5p4*c7p2*ss[44] + 1260*c3p3*c5p4*c9p2*ss[48] + 504*c3p3*c5p5*c7p1*ss[42] + 504*c3p3*c5p5*c9p1*ss[44] + 504*c3p3*c7p1*c9p5*ss[62] + 1260*c3p3*c7p2*c9p4*ss[60] + 1680*c3p3*c7p3*c9p3*ss[58] + 1260*c3p3*c7p4*c9p2*ss[56] + 504*c3p3*c7p5*c9p1*ss[54] + 630*c3p4*c5p1*c7p4*ss[46] + 630*c3p4*c5p1*c9p4*ss[54] + 1260*c3p4*c5p2*c7p3*ss[44] + 1260*c3p4*c5p2*c9p3*ss[50] + 1260*c3p4*c5p3*c7p2*ss[42] + 1260*c3p4*c5p3*c9p2*ss[46] + 630*c3p4*c5p4*c7p1*ss[40] + 630*c3p4*c5p4*c9p1*ss[42] + 630*c3p4*c7p1*c9p4*ss[56] + 1260*c3p4*c7p2*c9p3*ss[54] + 1260*c3p4*c7p3*c9p2*ss[52] + 630*c3p4*c7p4*c9p1*ss[50] + 504*c3p5*c5p1*c7p3*ss[42] + 504*c3p5*c5p1*c9p3*ss[48] + 756*c3p5*c5p2*c7p2*ss[40] + 756*c3p5*c5p2*c9p2*ss[44] + 504*c3p5*c5p3*c7p1*ss[38] + 504*c3p5*c5p3*c9p1*ss[40] + 504*c3p5*c7p1*c9p3*ss[50] + 756*c3p5*c7p2*c9p2*ss[48] + 504*c3p5*c7p3*c9p1*ss[46] + 252*c3p6*c5p1*c7p2*ss[38] + 252*c3p6*c5p1*c9p2*ss[42] + 252*c3p6*c5p2*c7p1*ss[36] + 252*c3p6*c5p2*c9p1*ss[38] + 252*c3p6*c7p1*c9p2*ss[44] + 252*c3p6*c7p2*c9p1*ss[42] + 72*c3p7*c5p1*c7p1*ss[34] + 72*c3p7*c5p1*c9p1*ss[36] + 72*c3p7*c7p1*c9p1*ss[38] + 72*c5p1*c7p1*c9p7*ss[76] + 252*c5p1*c7p2*c9p6*ss[74] + 504*c5p1*c7p3*c9p5*ss[72] + 630*c5p1*c7p4*c9p4*ss[70] + 504*c5p1*c7p5*c9p3*ss[68] + 252*c5p1*c7p6*c9p2*ss[66] + 72*c5p1*c7p7*c9p1*ss[64] + 252*c5p2*c7p1*c9p6*ss[72] + 756*c5p2*c7p2*c9p5*ss[70] + 1260*c5p2*c7p3*c9p4*ss[68] + 1260*c5p2*c7p4*c9p3*ss[66] + 756*c5p2*c7p5*c9p2*ss[64] + 252*c5p2*c7p6*c9p1*ss[62] + 504*c5p3*c7p1*c9p5*ss[68] + 1260*c5p3*c7p2*c9p4*ss[66] + 1680*c5p3*c7p3*c9p3*ss[64] + 1260*c5p3*c7p4*c9p2*ss[62] + 504*c5p3*c7p5*c9p1*ss[60] + 630*c5p4*c7p1*c9p4*ss[64] + 1260*c5p4*c7p2*c9p3*ss[62] + 1260*c5p4*c7p3*c9p2*ss[60] + 630*c5p4*c7p4*c9p1*ss[58] + 504*c5p5*c7p1*c9p3*ss[60] + 756*c5p5*c7p2*c9p2*ss[58] + 504*c5p5*c7p3*c9p1*ss[56] + 252*c5p6*c7p1*c9p2*ss[56] + 252*c5p6*c7p2*c9p1*ss[54] + 72*c5p7*c7p1*c9p1*ss[52] + 9*c1p1*c3p8*ss[26] + 9*c1p1*c5p8*ss[42] + 9*c1p1*c7p8*ss[58] + 9*c1p1*c9p8*ss[74] + 36*c1p2*c3p7*ss[24] + 36*c1p2*c5p7*ss[38] + 36*c1p2*c7p7*ss[52] + 36*c1p2*c9p7*ss[66] + 84*c1p3*c3p6*ss[22] + 84*c1p3*c5p6*ss[34] + 84*c1p3*c7p6*ss[46] + 84*c1p3*c9p6*ss[58] + 126*c1p4*c3p5*ss[20] + 126*c1p4*c5p5*ss[30] + 126*c1p4*c7p5*ss[40] + 126*c1p4*c9p5*ss[50] + 126*c1p5*c3p4*ss[18] + 126*c1p5*c5p4*ss[26] + 126*c1p5*c7p4*ss[34] + 126*c1p5*c9p4*ss[42] + 84*c1p6*c3p3*ss[16] + 84*c1p6*c5p3*ss[22] + 84*c1p6*c7p3*ss[28] + 84*c1p6*c9p3*ss[34] + 36*c1p7*c3p2*ss[14] + 36*c1p7*c5p2*ss[18] + 36*c1p7*c7p2*ss[22] + 36*c1p7*c9p2*ss[26] + 9*c1p8*c3p1*ss[12] + 9*c1p8*c5p1*ss[14] + 9*c1p8*c7p1*ss[16] + 9*c1p8*c9p1*ss[18] + 9*c3p1*c5p8*ss[44] + 9*c3p1*c7p8*ss[60] + 9*c3p1*c9p8*ss[76] + 36*c3p2*c5p7*ss[42] + 36*c3p2*c7p7*ss[56] + 36*c3p2*c9p7*ss[70] + 84*c3p3*c5p6*ss[40] + 84*c3p3*c7p6*ss[52] + 84*c3p3*c9p6*ss[64] + 126*c3p4*c5p5*ss[38] + 126*c3p4*c7p5*ss[48] + 126*c3p4*c9p5*ss[58] + 126*c3p5*c5p4*ss[36] + 126*c3p5*c7p4*ss[44] + 126*c3p5*c9p4*ss[52] + 84*c3p6*c5p3*ss[34] + 84*c3p6*c7p3*ss[40] + 84*c3p6*c9p3*ss[46] + 36*c3p7*c5p2*ss[32] + 36*c3p7*c7p2*ss[36] + 36*c3p7*c9p2*ss[40] + 9*c3p8*c5p1*ss[30] + 9*c3p8*c7p1*ss[32] + 9*c3p8*c9p1*ss[34] + 9*c5p1*c7p8*ss[62] + 9*c5p1*c9p8*ss[78] + 36*c5p2*c7p7*ss[60] + 36*c5p2*c9p7*ss[74] + 84*c5p3*c7p6*ss[58] + 84*c5p3*c9p6*ss[70] + 126*c5p4*c7p5*ss[56] + 126*c5p4*c9p5*ss[66] + 126*c5p5*c7p4*ss[54] + 126*c5p5*c9p4*ss[62] + 84*c5p6*c7p3*ss[52] + 84*c5p6*c9p3*ss[58] + 36*c5p7*c7p2*ss[50] + 36*c5p7*c9p2*ss[54] + 9*c5p8*c7p1*ss[48] + 9*c5p8*c9p1*ss[50] + 9*c7p1*c9p8*ss[80] + 36*c7p2*c9p7*ss[78] + 84*c7p3*c9p6*ss[76] + 126*c7p4*c9p5*ss[74] + 126*c7p5*c9p4*ss[72] + 84*c7p6*c9p3*ss[70] + 36*c7p7*c9p2*ss[68] + 9*c7p8*c9p1*ss[66] + c1p9*ss[10] + c3p9*ss[28] + c5p9*ss[46] + c7p9*ss[64] + c9p9*ss[82])*sys_part_4;
  TACC grad_0_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4)));
  TACC grad_1_num = convert<TACC>(2*(c1p1*ss[4] + c3p1*ss[6] + c5p1*ss[8] + c7p1*ss[10] + c9p1*ss[12])*sys_part_0);
  grad_1_num += 4*(6*c1p1*c3p1*c5p1*ss[12] + 6*c1p1*c3p1*c7p1*ss[14] + 6*c1p1*c3p1*c9p1*ss[16] + 6*c1p1*c5p1*c7p1*ss[16] + 6*c1p1*c5p1*c9p1*ss[18] + 6*c1p1*c7p1*c9p1*ss[20] + 6*c3p1*c5p1*c7p1*ss[18] + 6*c3p1*c5p1*c9p1*ss[20] + 6*c3p1*c7p1*c9p1*ss[22] + 6*c5p1*c7p1*c9p1*ss[24] + 3*c1p1*c3p2*ss[10] + 3*c1p1*c5p2*ss[14] + 3*c1p1*c7p2*ss[18] + 3*c1p1*c9p2*ss[22] + 3*c1p2*c3p1*ss[8] + 3*c1p2*c5p1*ss[10] + 3*c1p2*c7p1*ss[12] + 3*c1p2*c9p1*ss[14] + 3*c3p1*c5p2*ss[16] + 3*c3p1*c7p2*ss[20] + 3*c3p1*c9p2*ss[24] + 3*c3p2*c5p1*ss[14] + 3*c3p2*c7p1*ss[16] + 3*c3p2*c9p1*ss[18] + 3*c5p1*c7p2*ss[22] + 3*c5p1*c9p2*ss[26] + 3*c5p2*c7p1*ss[20] + 3*c5p2*c9p1*ss[22] + 3*c7p1*c9p2*ss[28] + 3*c7p2*c9p1*ss[26] + c1p3*ss[6] + c3p3*ss[12] + c5p3*ss[18] + c7p3*ss[24] + c9p3*ss[30])*sys_part_1;
  grad_1_num += 6*(120*c1p1*c3p1*c5p1*c7p1*c9p1*ss[28] + 60*c1p1*c3p1*c5p1*c7p2*ss[26] + 60*c1p1*c3p1*c5p1*c9p2*ss[30] + 60*c1p1*c3p1*c5p2*c7p1*ss[24] + 60*c1p1*c3p1*c5p2*c9p1*ss[26] + 60*c1p1*c3p1*c7p1*c9p2*ss[32] + 60*c1p1*c3p1*c7p2*c9p1*ss[30] + 60*c1p1*c3p2*c5p1*c7p1*ss[22] + 60*c1p1*c3p2*c5p1*c9p1*ss[24] + 60*c1p1*c3p2*c7p1*c9p1*ss[26] + 60*c1p1*c5p1*c7p1*c9p2*ss[34] + 60*c1p1*c5p1*c7p2*c9p1*ss[32] + 60*c1p1*c5p2*c7p1*c9p1*ss[30] + 60*c1p2*c3p1*c5p1*c7p1*ss[20] + 60*c1p2*c3p1*c5p1*c9p1*ss[22] + 60*c1p2*c3p1*c7p1*c9p1*ss[24] + 60*c1p2*c5p1*c7p1*c9p1*ss[26] + 60*c3p1*c5p1*c7p1*c9p2*ss[36] + 60*c3p1*c5p1*c7p2*c9p1*ss[34] + 60*c3p1*c5p2*c7p1*c9p1*ss[32] + 60*c3p2*c5p1*c7p1*c9p1*ss[30] + 20*c1p1*c3p1*c5p3*ss[22] + 20*c1p1*c3p1*c7p3*ss[28] + 20*c1p1*c3p1*c9p3*ss[34] + 30*c1p1*c3p2*c5p2*ss[20] + 30*c1p1*c3p2*c7p2*ss[24] + 30*c1p1*c3p2*c9p2*ss[28] + 20*c1p1*c3p3*c5p1*ss[18] + 20*c1p1*c3p3*c7p1*ss[20] + 20*c1p1*c3p3*c9p1*ss[22] + 20*c1p1*c5p1*c7p3*ss[30] + 20*c1p1*c5p1*c9p3*ss[36] + 30*c1p1*c5p2*c7p2*ss[28] + 30*c1p1*c5p2*c9p2*ss[32] + 20*c1p1*c5p3*c7p1*ss[26] + 20*c1p1*c5p3*c9p1*ss[28] + 20*c1p1*c7p1*c9p3*ss[38] + 30*c1p1*c7p2*c9p2*ss[36] + 20*c1p1*c7p3*c9p1*ss[34] + 30*c1p2*c3p1*c5p2*ss[18] + 30*c1p2*c3p1*c7p2*ss[22] + 30*c1p2*c3p1*c9p2*ss[26] + 30*c1p2*c3p2*c5p1*ss[16] + 30*c1p2*c3p2*c7p1*ss[18] + 30*c1p2*c3p2*c9p1*ss[20] + 30*c1p2*c5p1*c7p2*ss[24] + 30*c1p2*c5p1*c9p2*ss[28] + 30*c1p2*c5p2*c7p1*ss[22] + 30*c1p2*c5p2*c9p1*ss[24] + 30*c1p2*c7p1*c9p2*ss[30] + 30*c1p2*c7p2*c9p1*ss[28] + 20*c1p3*c3p1*c5p1*ss[14] + 20*c1p3*c3p1*c7p1*ss[16] + 20*c1p3*c3p1*c9p1*ss[18] + 20*c1p3*c5p1*c7p1*ss[18] + 20*c1p3*c5p1*c9p1*ss[20] + 20*c1p3*c7p1*c9p1*ss[22] + 20*c3p1*c5p1*c7p3*ss[32] + 20*c3p1*c5p1*c9p3*ss[38] + 30*c3p1*c5p2*c7p2*ss[30] + 30*c3p1*c5p2*c9p2*ss[34] + 20*c3p1*c5p3*c7p1*ss[28] + 20*c3p1*c5p3*c9p1*ss[30] + 20*c3p1*c7p1*c9p3*ss[40] + 30*c3p1*c7p2*c9p2*ss[38] + 20*c3p1*c7p3*c9p1*ss[36] + 30*c3p2*c5p1*c7p2*ss[28] + 30*c3p2*c5p1*c9p2*ss[32] + 30*c3p2*c5p2*c7p1*ss[26] + 30*c3p2*c5p2*c9p1*ss[28] + 30*c3p2*c7p1*c9p2*ss[34] + 30*c3p2*c7p2*c9p1*ss[32] + 20*c3p3*c5p1*c7p1*ss[24] + 20*c3p3*c5p1*c9p1*ss[26] + 20*c3p3*c7p1*c9p1*ss[28] + 20*c5p1*c7p1*c9p3*ss[42] + 30*c5p1*c7p2*c9p2*ss[40] + 20*c5p1*c7p3*c9p1*ss[38] + 30*c5p2*c7p1*c9p2*ss[38] + 30*c5p2*c7p2*c9p1*ss[36] + 20*c5p3*c7p1*c9p1*ss[34] + 5*c1p1*c3p4*ss[16] + 5*c1p1*c5p4*ss[24] + 5*c1p1*c7p4*ss[32] + 5*c1p1*c9p4*ss[40] + 10*c1p2*c3p3*ss[14] + 10*c1p2*c5p3*ss[20] + 10*c1p2*c7p3*ss[26] + 10*c1p2*c9p3*ss[32] + 10*c1p3*c3p2*ss[12] + 10*c1p3*c5p2*ss[16] + 10*c1p3*c7p2*ss[20] + 10*c1p3*c9p2*ss[24] + 5*c1p4*c3p1*ss[10] + 5*c1p4*c5p1*ss[12] + 5*c1p4*c7p1*ss[14] + 5*c1p4*c9p1*ss[16] + 5*c3p1*c5p4*ss[26] + 5*c3p1*c7p4*ss[34] + 5*c3p1*c9p4*ss[42] + 10*c3p2*c5p3*ss[24] + 10*c3p2*c7p3*ss[30] + 10*c3p2*c9p3*ss[36] + 10*c3p3*c5p2*ss[22] + 10*c3p3*c7p2*ss[26] + 10*c3p3*c9p2*ss[30] + 5*c3p4*c5p1*ss[20] + 5*c3p4*c7p1*ss[22] + 5*c3p4*c9p1*ss[24] + 5*c5p1*c7p4*ss[36] + 5*c5p1*c9p4*ss[44] + 10*c5p2*c7p3*ss[34] + 10*c5p2*c9p3*ss[40] + 10*c5p3*c7p2*ss[32] + 10*c5p3*c9p2*ss[36] + 5*c5p4*c7p1*ss[30] + 5*c5p4*c9p1*ss[32] + 5*c7p1*c9p4*ss[46] + 10*c7p2*c9p3*ss[44] + 10*c7p3*c9p2*ss[42] + 5*c7p4*c9p1*ss[40] + c1p5*ss[8] + c3p5*ss[18] + c5p5*ss[28] + c7p5*ss[38] + c9p5*ss[48])*sys_part_2;
  grad_1_num += 8*(840*c1p1*c3p1*c5p1*c7p1*c9p3*ss[46] + 1260*c1p1*c3p1*c5p1*c7p2*c9p2*ss[44] + 840*c1p1*c3p1*c5p1*c7p3*c9p1*ss[42] + 1260*c1p1*c3p1*c5p2*c7p1*c9p2*ss[42] + 1260*c1p1*c3p1*c5p2*c7p2*c9p1*ss[40] + 840*c1p1*c3p1*c5p3*c7p1*c9p1*ss[38] + 1260*c1p1*c3p2*c5p1*c7p1*c9p2*ss[40] + 1260*c1p1*c3p2*c5p1*c7p2*c9p1*ss[38] + 1260*c1p1*c3p2*c5p2*c7p1*c9p1*ss[36] + 840*c1p1*c3p3*c5p1*c7p1*c9p1*ss[34] + 1260*c1p2*c3p1*c5p1*c7p1*c9p2*ss[38] + 1260*c1p2*c3p1*c5p1*c7p2*c9p1*ss[36] + 1260*c1p2*c3p1*c5p2*c7p1*c9p1*ss[34] + 1260*c1p2*c3p2*c5p1*c7p1*c9p1*ss[32] + 840*c1p3*c3p1*c5p1*c7p1*c9p1*ss[30] + 210*c1p1*c3p1*c5p1*c7p4*ss[40] + 210*c1p1*c3p1*c5p1*c9p4*ss[48] + 420*c1p1*c3p1*c5p2*c7p3*ss[38] + 420*c1p1*c3p1*c5p2*c9p3*ss[44] + 420*c1p1*c3p1*c5p3*c7p2*ss[36] + 420*c1p1*c3p1*c5p3*c9p2*ss[40] + 210*c1p1*c3p1*c5p4*c7p1*ss[34] + 210*c1p1*c3p1*c5p4*c9p1*ss[36] + 210*c1p1*c3p1*c7p1*c9p4*ss[50] + 420*c1p1*c3p1*c7p2*c9p3*ss[48] + 420*c1p1*c3p1*c7p3*c9p2*ss[46] + 210*c1p1*c3p1*c7p4*c9p1*ss[44] + 420*c1p1*c3p2*c5p1*c7p3*ss[36] + 420*c1p1*c3p2*c5p1*c9p3*ss[42] + 630*c1p1*c3p2*c5p2*c7p2*ss[34] + 630*c1p1*c3p2*c5p2*c9p2*ss[38] + 420*c1p1*c3p2*c5p3*c7p1*ss[32] + 420*c1p1*c3p2*c5p3*c9p1*ss[34] + 420*c1p1*c3p2*c7p1*c9p3*ss[44] + 630*c1p1*c3p2*c7p2*c9p2*ss[42] + 420*c1p1*c3p2*c7p3*c9p1*ss[40] + 420*c1p1*c3p3*c5p1*c7p2*ss[32] + 420*c1p1*c3p3*c5p1*c9p2*ss[36] + 420*c1p1*c3p3*c5p2*c7p1*ss[30] + 420*c1p1*c3p3*c5p2*c9p1*ss[32] + 420*c1p1*c3p3*c7p1*c9p2*ss[38] + 420*c1p1*c3p3*c7p2*c9p1*ss[36] + 210*c1p1*c3p4*c5p1*c7p1*ss[28] + 210*c1p1*c3p4*c5p1*c9p1*ss[30] + 210*c1p1*c3p4*c7p1*c9p1*ss[32] + 210*c1p1*c5p1*c7p1*c9p4*ss[52] + 420*c1p1*c5p1*c7p2*c9p3*ss[50] + 420*c1p1*c5p1*c7p3*c9p2*ss[48] + 210*c1p1*c5p1*c7p4*c9p1*ss[46] + 420*c1p1*c5p2*c7p1*c9p3*ss[48] + 630*c1p1*c5p2*c7p2*c9p2*ss[46] + 420*c1p1*c5p2*c7p3*c9p1*ss[44] + 420*c1p1*c5p3*c7p1*c9p2*ss[44] + 420*c1p1*c5p3*c7p2*c9p1*ss[42] + 210*c1p1*c5p4*c7p1*c9p1*ss[40] + 420*c1p2*c3p1*c5p1*c7p3*ss[34] + 420*c1p2*c3p1*c5p1*c9p3*ss[40] + 630*c1p2*c3p1*c5p2*c7p2*ss[32] + 630*c1p2*c3p1*c5p2*c9p2*ss[36] + 420*c1p2*c3p1*c5p3*c7p1*ss[30] + 420*c1p2*c3p1*c5p3*c9p1*ss[32] + 420*c1p2*c3p1*c7p1*c9p3*ss[42] + 630*c1p2*c3p1*c7p2*c9p2*ss[40] + 420*c1p2*c3p1*c7p3*c9p1*ss[38] + 630*c1p2*c3p2*c5p1*c7p2*ss[30] + 630*c1p2*c3p2*c5p1*c9p2*ss[34] + 630*c1p2*c3p2*c5p2*c7p1*ss[28] + 630*c1p2*c3p2*c5p2*c9p1*ss[30] + 630*c1p2*c3p2*c7p1*c9p2*ss[36] + 630*c1p2*c3p2*c7p2*c9p1*ss[34] + 420*c1p2*c3p3*c5p1*c7p1*ss[26] + 420*c1p2*c3p3*c5p1*c9p1*ss[28] + 420*c1p2*c3p3*c7p1*c9p1*ss[30] + 420*c1p2*c5p1*c7p1*c9p3*ss[44] + 630*c1p2*c5p1*c7p2*c9p2*ss[42] + 420*c1p2*c5p1*c7p3*c9p1*ss[40] + 630*c1p2*c5p2*c7p1*c9p2*ss[40] + 630*c1p2*c5p2*c7p2*c9p1*ss[38] + 420*c1p2*c5p3*c7p1*c9p1*ss[36] + 420*c1p3*c3p1*c5p1*c7p2*ss[28] + 420*c1p3*c3p1*c5p1*c9p2*ss[32] + 420*c1p3*c3p1*c5p2*c7p1*ss[26] + 420*c1p3*c3p1*c5p2*c9p1*ss[28] + 420*c1p3*c3p1*c7p1*c9p2*ss[34] + 420*c1p3*c3p1*c7p2*c9p1*ss[32] + 420*c1p3*c3p2*c5p1*c7p1*ss[24] + 420*c1p3*c3p2*c5p1*c9p1*ss[26] + 420*c1p3*c3p2*c7p1*c9p1*ss[28] + 420*c1p3*c5p1*c7p1*c9p2*ss[36] + 420*c1p3*c5p1*c7p2*c9p1*ss[34] + 420*c1p3*c5p2*c7p1*c9p1*ss[32] + 210*c1p4*c3p1*c5p1*c7p1*ss[22] + 210*c1p4*c3p1*c5p1*c9p1*ss[24] + 210*c1p4*c3p1*c7p1*c9p1*ss[26] + 210*c1p4*c5p1*c7p1*c9p1*ss[28] + 210*c3p1*c5p1*c7p1*c9p4*ss[54] + 420*c3p1*c5p1*c7p2*c9p3*ss[52] + 420*c3p1*c5p1*c7p3*c9p2*ss[50] + 210*c3p1*c5p1*c7p4*c9p1*ss[48] + 420*c3p1*c5p2*c7p1*c9p3*ss[50] + 630*c3p1*c5p2*c7p2*c9p2*ss[48] + 420*c3p1*c5p2*c7p3*c9p1*ss[46] + 420*c3p1*c5p3*c7p1*c9p2*ss[46] + 420*c3p1*c5p3*c7p2*c9p1*ss[44] + 210*c3p1*c5p4*c7p1*c9p1*ss[42] + 420*c3p2*c5p1*c7p1*c9p3*ss[48] + 630*c3p2*c5p1*c7p2*c9p2*ss[46] + 420*c3p2*c5p1*c7p3*c9p1*ss[44] + 630*c3p2*c5p2*c7p1*c9p2*ss[44] + 630*c3p2*c5p2*c7p2*c9p1*ss[42] + 420*c3p2*c5p3*c7p1*c9p1*ss[40] + 420*c3p3*c5p1*c7p1*c9p2*ss[42] + 420*c3p3*c5p1*c7p2*c9p1*ss[40] + 420*c3p3*c5p2*c7p1*c9p1*ss[38] + 210*c3p4*c5p1*c7p1*c9p1*ss[36] + 42*c1p1*c3p1*c5p5*ss[32] + 42*c1p1*c3p1*c7p5*ss[42] + 42*c1p1*c3p1*c9p5*ss[52] + 105*c1p1*c3p2*c5p4*ss[30] + 105*c1p1*c3p2*c7p4*ss[38] + 105*c1p1*c3p2*c9p4*ss[46] + 140*c1p1*c3p3*c5p3*ss[28] + 140*c1p1*c3p3*c7p3*ss[34] + 140*c1p1*c3p3*c9p3*ss[40] + 105*c1p1*c3p4*c5p2*ss[26] + 105*c1p1*c3p4*c7p2*ss[30] + 105*c1p1*c3p4*c9p2*ss[34] + 42*c1p1*c3p5*c5p1*ss[24] + 42*c1p1*c3p5*c7p1*ss[26] + 42*c1p1*c3p5*c9p1*ss[28] + 42*c1p1*c5p1*c7p5*ss[44] + 42*c1p1*c5p1*c9p5*ss[54] + 105*c1p1*c5p2*c7p4*ss[42] + 105*c1p1*c5p2*c9p4*ss[50] + 140*c1p1*c5p3*c7p3*ss[40] + 140*c1p1*c5p3*c9p3*ss[46] + 105*c1p1*c5p4*c7p2*ss[38] + 105*c1p1*c5p4*c9p2*ss[42] + 42*c1p1*c5p5*c7p1*ss[36] + 42*c1p1*c5p5*c9p1*ss[38] + 42*c1p1*c7p1*c9p5*ss[56] + 105*c1p1*c7p2*c9p4*ss[54] + 140*c1p1*c7p3*c9p3*ss[52] + 105*c1p1*c7p4*c9p2*ss[50] + 42*c1p1*c7p5*c9p1*ss[48] + 105*c1p2*c3p1*c5p4*ss[28] + 105*c1p2*c3p1*c7p4*ss[36] + 105*c1p2*c3p1*c9p4*ss[44] + 210*c1p2*c3p2*c5p3*ss[26] + 210*c1p2*c3p2*c7p3*ss[32] + 210*c1p2*c3p2*c9p3*ss[38] + 210*c1p2*c3p3*c5p2*ss[24] + 210*c1p2*c3p3*c7p2*ss[28] + 210*c1p2*c3p3*c9p2*ss[32] + 105*c1p2*c3p4*c5p1*ss[22] + 105*c1p2*c3p4*c7p1*ss[24] + 105*c1p2*c3p4*c9p1*ss[26] + 105*c1p2*c5p1*c7p4*ss[38] + 105*c1p2*c5p1*c9p4*ss[46] + 210*c1p2*c5p2*c7p3*ss[36] + 210*c1p2*c5p2*c9p3*ss[42] + 210*c1p2*c5p3*c7p2*ss[34] + 210*c1p2*c5p3*c9p2*ss[38] + 105*c1p2*c5p4*c7p1*ss[32] + 105*c1p2*c5p4*c9p1*ss[34] + 105*c1p2*c7p1*c9p4*ss[48] + 210*c1p2*c7p2*c9p3*ss[46] + 210*c1p2*c7p3*c9p2*ss[44] + 105*c1p2*c7p4*c9p1*ss[42] + 140*c1p3*c3p1*c5p3*ss[24] + 140*c1p3*c3p1*c7p3*ss[30] + 140*c1p3*c3p1*c9p3*ss[36] + 210*c1p3*c3p2*c5p2*ss[22] + 210*c1p3*c3p2*c7p2*ss[26] + 210*c1p3*c3p2*c9p2*ss[30] + 140*c1p3*c3p3*c5p1*ss[20] + 140*c1p3*c3p3*c7p1*ss[22] + 140*c1p3*c3p3*c9p1*ss[24] + 140*c1p3*c5p1*c7p3*ss[32] + 140*c1p3*c5p1*c9p3*ss[38] + 210*c1p3*c5p2*c7p2*ss[30] + 210*c1p3*c5p2*c9p2*ss[34] + 140*c1p3*c5p3*c7p1*ss[28] + 140*c1p3*c5p3*c9p1*ss[30] + 140*c1p3*c7p1*c9p3*ss[40] + 210*c1p3*c7p2*c9p2*ss[38] + 140*c1p3*c7p3*c9p1*ss[36] + 105*c1p4*c3p1*c5p2*ss[20] + 105*c1p4*c3p1*c7p2*ss[24] + 105*c1p4*c3p1*c9p2*ss[28] + 105*c1p4*c3p2*c5p1*ss[18] + 105*c1p4*c3p2*c7p1*ss[20] + 105*c1p4*c3p2*c9p1*ss[22] + 105*c1p4*c5p1*c7p2*ss[26] + 105*c1p4*c5p1*c9p2*ss[30] + 105*c1p4*c5p2*c7p1*ss[24] + 105*c1p4*c5p2*c9p1*ss[26] + 105*c1p4*c7p1*c9p2*ss[32] + 105*c1p4*c7p2*c9p1*ss[30] + 42*c1p5*c3p1*c5p1*ss[16] + 42*c1p5*c3p1*c7p1*ss[18] + 42*c1p5*c3p1*c9p1*ss[20] + 42*c1p5*c5p1*c7p1*ss[20] + 42*c1p5*c5p1*c9p1*ss[22] + 42*c1p5*c7p1*c9p1*ss[24] + 42*c3p1*c5p1*c7p5*ss[46] + 42*c3p1*c5p1*c9p5*ss[56] + 105*c3p1*c5p2*c7p4*ss[44] + 105*c3p1*c5p2*c9p4*ss[52] + 140*c3p1*c5p3*c7p3*ss[42] + 140*c3p1*c5p3*c9p3*ss[48] + 105*c3p1*c5p4*c7p2*ss[40] + 105*c3p1*c5p4*c9p2*ss[44] + 42*c3p1*c5p5*c7p1*ss[38] + 42*c3p1*c5p5*c9p1*ss[40] + 42*c3p1*c7p1*c9p5*ss[58] + 105*c3p1*c7p2*c9p4*ss[56] + 140*c3p1*c7p3*c9p3*ss[54] + 105*c3p1*c7p4*c9p2*ss[52] + 42*c3p1*c7p5*c9p1*ss[50] + 105*c3p2*c5p1*c7p4*ss[42] + 105*c3p2*c5p1*c9p4*ss[50] + 210*c3p2*c5p2*c7p3*ss[40] + 210*c3p2*c5p2*c9p3*ss[46] + 210*c3p2*c5p3*c7p2*ss[38] + 210*c3p2*c5p3*c9p2*ss[42] + 105*c3p2*c5p4*c7p1*ss[36] + 105*c3p2*c5p4*c9p1*ss[38] + 105*c3p2*c7p1*c9p4*ss[52] + 210*c3p2*c7p2*c9p3*ss[50] + 210*c3p2*c7p3*c9p2*ss[48] + 105*c3p2*c7p4*c9p1*ss[46] + 140*c3p3*c5p1*c7p3*ss[38] + 140*c3p3*c5p1*c9p3*ss[44] + 210*c3p3*c5p2*c7p2*ss[36] + 210*c3p3*c5p2*c9p2*ss[40] + 140*c3p3*c5p3*c7p1*ss[34] + 140*c3p3*c5p3*c9p1*ss[36] + 140*c3p3*c7p1*c9p3*ss[46] + 210*c3p3*c7p2*c9p2*ss[44] + 140*c3p3*c7p3*c9p1*ss[42] + 105*c3p4*c5p1*c7p2*ss[34] + 105*c3p4*c5p1*c9p2*ss[38] + 105*c3p4*c5p2*c7p1*ss[32] + 105*c3p4*c5p2*c9p1*ss[34] + 105*c3p4*c7p1*c9p2*ss[40] + 105*c3p4*c7p2*c9p1*ss[38] + 42*c3p5*c5p1*c7p1*ss[30] + 42*c3p5*c5p1*c9p1*ss[32] + 42*c3p5*c7p1*c9p1*ss[34] + 42*c5p1*c7p1*c9p5*ss[60] + 105*c5p1*c7p2*c9p4*ss[58] + 140*c5p1*c7p3*c9p3*ss[56] + 105*c5p1*c7p4*c9p2*ss[54] + 42*c5p1*c7p5*c9p1*ss[52] + 105*c5p2*c7p1*c9p4*ss[56] + 210*c5p2*c7p2*c9p3*ss[54] + 210*c5p2*c7p3*c9p2*ss[52] + 105*c5p2*c7p4*c9p1*ss[50] + 140*c5p3*c7p1*c9p3*ss[52] + 210*c5p3*c7p2*c9p2*ss[50] + 140*c5p3*c7p3*c9p1*ss[48] + 105*c5p4*c7p1*c9p2*ss[48] + 105*c5p4*c7p2*c9p1*ss[46] + 42*c5p5*c7p1*c9p1*ss[44] + 7*c1p1*c3p6*ss[22] + 7*c1p1*c5p6*ss[34] + 7*c1p1*c7p6*ss[46] + 7*c1p1*c9p6*ss[58] + 21*c1p2*c3p5*ss[20] + 21*c1p2*c5p5*ss[30] + 21*c1p2*c7p5*ss[40] + 21*c1p2*c9p5*ss[50] + 35*c1p3*c3p4*ss[18] + 35*c1p3*c5p4*ss[26] + 35*c1p3*c7p4*ss[34] + 35*c1p3*c9p4*ss[42] + 35*c1p4*c3p3*ss[16] + 35*c1p4*c5p3*ss[22] + 35*c1p4*c7p3*ss[28] + 35*c1p4*c9p3*ss[34] + 21*c1p5*c3p2*ss[14] + 21*c1p5*c5p2*ss[18] + 21*c1p5*c7p2*ss[22] + 21*c1p5*c9p2*ss[26] + 7*c1p6*c3p1*ss[12] + 7*c1p6*c5p1*ss[14] + 7*c1p6*c7p1*ss[16] + 7*c1p6*c9p1*ss[18] + 7*c3p1*c5p6*ss[36] + 7*c3p1*c7p6*ss[48] + 7*c3p1*c9p6*ss[60] + 21*c3p2*c5p5*ss[34] + 21*c3p2*c7p5*ss[44] + 21*c3p2*c9p5*ss[54] + 35*c3p3*c5p4*ss[32] + 35*c3p3*c7p4*ss[40] + 35*c3p3*c9p4*ss[48] + 35*c3p4*c5p3*ss[30] + 35*c3p4*c7p3*ss[36] + 35*c3p4*c9p3*ss[42] + 21*c3p5*c5p2*ss[28] + 21*c3p5*c7p2*ss[32] + 21*c3p5*c9p2*ss[36] + 7*c3p6*c5p1*ss[26] + 7*c3p6*c7p1*ss[28] + 7*c3p6*c9p1*ss[30] + 7*c5p1*c7p6*ss[50] + 7*c5p1*c9p6*ss[62] + 21*c5p2*c7p5*ss[48] + 21*c5p2*c9p5*ss[58] + 35*c5p3*c7p4*ss[46] + 35*c5p3*c9p4*ss[54] + 35*c5p4*c7p3*ss[44] + 35*c5p4*c9p3*ss[50] + 21*c5p5*c7p2*ss[42] + 21*c5p5*c9p2*ss[46] + 7*c5p6*c7p1*ss[40] + 7*c5p6*c9p1*ss[42] + 7*c7p1*c9p6*ss[64] + 21*c7p2*c9p5*ss[62] + 35*c7p3*c9p4*ss[60] + 35*c7p4*c9p3*ss[58] + 21*c7p5*c9p2*ss[56] + 7*c7p6*c9p1*ss[54] + c1p7*ss[10] + c3p7*ss[24] + c5p7*ss[38] + c7p7*ss[52] + c9p7*ss[66])*sys_part_3;
  grad_1_num += 10*(3024*c1p1*c3p1*c5p1*c7p1*c9p5*ss[64] + 7560*c1p1*c3p1*c5p1*c7p2*c9p4*ss[62] + 10080*c1p1*c3p1*c5p1*c7p3*c9p3*ss[60] + 7560*c1p1*c3p1*c5p1*c7p4*c9p2*ss[58] + 3024*c1p1*c3p1*c5p1*c7p5*c9p1*ss[56] + 7560*c1p1*c3p1*c5p2*c7p1*c9p4*ss[60] + 15120*c1p1*c3p1*c5p2*c7p2*c9p3*ss[58] + 15120*c1p1*c3p1*c5p2*c7p3*c9p2*ss[56] + 7560*c1p1*c3p1*c5p2*c7p4*c9p1*ss[54] + 10080*c1p1*c3p1*c5p3*c7p1*c9p3*ss[56] + 15120*c1p1*c3p1*c5p3*c7p2*c9p2*ss[54] + 10080*c1p1*c3p1*c5p3*c7p3*c9p1*ss[52] + 7560*c1p1*c3p1*c5p4*c7p1*c9p2*ss[52] + 7560*c1p1*c3p1*c5p4*c7p2*c9p1*ss[50] + 3024*c1p1*c3p1*c5p5*c7p1*c9p1*ss[48] + 7560*c1p1*c3p2*c5p1*c7p1*c9p4*ss[58] + 15120*c1p1*c3p2*c5p1*c7p2*c9p3*ss[56] + 15120*c1p1*c3p2*c5p1*c7p3*c9p2*ss[54] + 7560*c1p1*c3p2*c5p1*c7p4*c9p1*ss[52] + 15120*c1p1*c3p2*c5p2*c7p1*c9p3*ss[54] + 22680*c1p1*c3p2*c5p2*c7p2*c9p2*ss[52] + 15120*c1p1*c3p2*c5p2*c7p3*c9p1*ss[50] + 15120*c1p1*c3p2*c5p3*c7p1*c9p2*ss[50] + 15120*c1p1*c3p2*c5p3*c7p2*c9p1*ss[48] + 7560*c1p1*c3p2*c5p4*c7p1*c9p1*ss[46] + 10080*c1p1*c3p3*c5p1*c7p1*c9p3*ss[52] + 15120*c1p1*c3p3*c5p1*c7p2*c9p2*ss[50] + 10080*c1p1*c3p3*c5p1*c7p3*c9p1*ss[48] + 15120*c1p1*c3p3*c5p2*c7p1*c9p2*ss[48] + 15120*c1p1*c3p3*c5p2*c7p2*c9p1*ss[46] + 10080*c1p1*c3p3*c5p3*c7p1*c9p1*ss[44] + 7560*c1p1*c3p4*c5p1*c7p1*c9p2*ss[46] + 7560*c1p1*c3p4*c5p1*c7p2*c9p1*ss[44] + 7560*c1p1*c3p4*c5p2*c7p1*c9p1*ss[42] + 3024*c1p1*c3p5*c5p1*c7p1*c9p1*ss[40] + 7560*c1p2*c3p1*c5p1*c7p1*c9p4*ss[56] + 15120*c1p2*c3p1*c5p1*c7p2*c9p3*ss[54] + 15120*c1p2*c3p1*c5p1*c7p3*c9p2*ss[52] + 7560*c1p2*c3p1*c5p1*c7p4*c9p1*ss[50] + 15120*c1p2*c3p1*c5p2*c7p1*c9p3*ss[52] + 22680*c1p2*c3p1*c5p2*c7p2*c9p2*ss[50] + 15120*c1p2*c3p1*c5p2*c7p3*c9p1*ss[48] + 15120*c1p2*c3p1*c5p3*c7p1*c9p2*ss[48] + 15120*c1p2*c3p1*c5p3*c7p2*c9p1*ss[46] + 7560*c1p2*c3p1*c5p4*c7p1*c9p1*ss[44] + 15120*c1p2*c3p2*c5p1*c7p1*c9p3*ss[50] + 22680*c1p2*c3p2*c5p1*c7p2*c9p2*ss[48] + 15120*c1p2*c3p2*c5p1*c7p3*c9p1*ss[46] + 22680*c1p2*c3p2*c5p2*c7p1*c9p2*ss[46] + 22680*c1p2*c3p2*c5p2*c7p2*c9p1*ss[44] + 15120*c1p2*c3p2*c5p3*c7p1*c9p1*ss[42] + 15120*c1p2*c3p3*c5p1*c7p1*c9p2*ss[44] + 15120*c1p2*c3p3*c5p1*c7p2*c9p1*ss[42] + 15120*c1p2*c3p3*c5p2*c7p1*c9p1*ss[40] + 7560*c1p2*c3p4*c5p1*c7p1*c9p1*ss[38] + 10080*c1p3*c3p1*c5p1*c7p1*c9p3*ss[48] + 15120*c1p3*c3p1*c5p1*c7p2*c9p2*ss[46] + 10080*c1p3*c3p1*c5p1*c7p3*c9p1*ss[44] + 15120*c1p3*c3p1*c5p2*c7p1*c9p2*ss[44] + 15120*c1p3*c3p1*c5p2*c7p2*c9p1*ss[42] + 10080*c1p3*c3p1*c5p3*c7p1*c9p1*ss[40] + 15120*c1p3*c3p2*c5p1*c7p1*c9p2*ss[42] + 15120*c1p3*c3p2*c5p1*c7p2*c9p1*ss[40] + 15120*c1p3*c3p2*c5p2*c7p1*c9p1*ss[38] + 10080*c1p3*c3p3*c5p1*c7p1*c9p1*ss[36] + 7560*c1p4*c3p1*c5p1*c7p1*c9p2*ss[40] + 7560*c1p4*c3p1*c5p1*c7p2*c9p1*ss[38] + 7560*c1p4*c3p1*c5p2*c7p1*c9p1*ss[36] + 7560*c1p4*c3p2*c5p1*c7p1*c9p1*ss[34] + 3024*c1p5*c3p1*c5p1*c7p1*c9p1*ss[32] + 504*c1p1*c3p1*c5p1*c7p6*ss[54] + 504*c1p1*c3p1*c5p1*c9p6*ss[66] + 1512*c1p1*c3p1*c5p2*c7p5*ss[52] + 1512*c1p1*c3p1*c5p2*c9p5*ss[62] + 2520*c1p1*c3p1*c5p3*c7p4*ss[50] + 2520*c1p1*c3p1*c5p3*c9p4*ss[58] + 2520*c1p1*c3p1*c5p4*c7p3*ss[48] + 2520*c1p1*c3p1*c5p4*c9p3*ss[54] + 1512*c1p1*c3p1*c5p5*c7p2*ss[46] + 1512*c1p1*c3p1*c5p5*c9p2*ss[50] + 504*c1p1*c3p1*c5p6*c7p1*ss[44] + 504*c1p1*c3p1*c5p6*c9p1*ss[46] + 504*c1p1*c3p1*c7p1*c9p6*ss[68] + 1512*c1p1*c3p1*c7p2*c9p5*ss[66] + 2520*c1p1*c3p1*c7p3*c9p4*ss[64] + 2520*c1p1*c3p1*c7p4*c9p3*ss[62] + 1512*c1p1*c3p1*c7p5*c9p2*ss[60] + 504*c1p1*c3p1*c7p6*c9p1*ss[58] + 1512*c1p1*c3p2*c5p1*c7p5*ss[50] + 1512*c1p1*c3p2*c5p1*c9p5*ss[60] + 3780*c1p1*c3p2*c5p2*c7p4*ss[48] + 3780*c1p1*c3p2*c5p2*c9p4*ss[56] + 5040*c1p1*c3p2*c5p3*c7p3*ss[46] + 5040*c1p1*c3p2*c5p3*c9p3*ss[52] + 3780*c1p1*c3p2*c5p4*c7p2*ss[44] + 3780*c1p1*c3p2*c5p4*c9p2*ss[48] + 1512*c1p1*c3p2*c5p5*c7p1*ss[42] + 1512*c1p1*c3p2*c5p5*c9p1*ss[44] + 1512*c1p1*c3p2*c7p1*c9p5*ss[62] + 3780*c1p1*c3p2*c7p2*c9p4*ss[60] + 5040*c1p1*c3p2*c7p3*c9p3*ss[58] + 3780*c1p1*c3p2*c7p4*c9p2*ss[56] + 1512*c1p1*c3p2*c7p5*c9p1*ss[54] + 2520*c1p1*c3p3*c5p1*c7p4*ss[46] + 2520*c1p1*c3p3*c5p1*c9p4*ss[54] + 5040*c1p1*c3p3*c5p2*c7p3*ss[44] + 5040*c1p1*c3p3*c5p2*c9p3*ss[50] + 5040*c1p1*c3p3*c5p3*c7p2*ss[42] + 5040*c1p1*c3p3*c5p3*c9p2*ss[46] + 2520*c1p1*c3p3*c5p4*c7p1*ss[40] + 2520*c1p1*c3p3*c5p4*c9p1*ss[42] + 2520*c1p1*c3p3*c7p1*c9p4*ss[56] + 5040*c1p1*c3p3*c7p2*c9p3*ss[54] + 5040*c1p1*c3p3*c7p3*c9p2*ss[52] + 2520*c1p1*c3p3*c7p4*c9p1*ss[50] + 2520*c1p1*c3p4*c5p1*c7p3*ss[42] + 2520*c1p1*c3p4*c5p1*c9p3*ss[48] + 3780*c1p1*c3p4*c5p2*c7p2*ss[40] + 3780*c1p1*c3p4*c5p2*c9p2*ss[44] + 2520*c1p1*c3p4*c5p3*c7p1*ss[38] + 2520*c1p1*c3p4*c5p3*c9p1*ss[40] + 2520*c1p1*c3p4*c7p1*c9p3*ss[50] + 3780*c1p1*c3p4*c7p2*c9p2*ss[48] + 2520*c1p1*c3p4*c7p3*c9p1*ss[46] + 1512*c1p1*c3p5*c5p1*c7p2*ss[38] + 1512*c1p1*c3p5*c5p1*c9p2*ss[42] + 1512*c1p1*c3p5*c5p2*c7p1*ss[36] + 1512*c1p1*c3p5*c5p2*c9p1*ss[38] + 1512*c1p1*c3p5*c7p1*c9p2*ss[44] + 1512*c1p1*c3p5*c7p2*c9p1*ss[42] + 504*c1p1*c3p6*c5p1*c7p1*ss[34] + 504*c1p1*c3p6*c5p1*c9p1*ss[36] + 504*c1p1*c3p6*c7p1*c9p1*ss[38] + 504*c1p1*c5p1*c7p1*c9p6*ss[70] + 1512*c1p1*c5p1*c7p2*c9p5*ss[68] + 2520*c1p1*c5p1*c7p3*c9p4*ss[66] + 2520*c1p1*c5p1*c7p4*c9p3*ss[64] + 1512*c1p1*c5p1*c7p5*c9p2*ss[62] + 504*c1p1*c5p1*c7p6*c9p1*ss[60] + 1512*c1p1*c5p2*c7p1*c9p5*ss[66] + 3780*c1p1*c5p2*c7p2*c9p4*ss[64] + 5040*c1p1*c5p2*c7p3*c9p3*ss[62] + 3780*c1p1*c5p2*c7p4*c9p2*ss[60] + 1512*c1p1*c5p2*c7p5*c9p1*ss[58] + 2520*c1p1*c5p3*c7p1*c9p4*ss[62] + 5040*c1p1*c5p3*c7p2*c9p3*ss[60] + 5040*c1p1*c5p3*c7p3*c9p2*ss[58] + 2520*c1p1*c5p3*c7p4*c9p1*ss[56] + 2520*c1p1*c5p4*c7p1*c9p3*ss[58] + 3780*c1p1*c5p4*c7p2*c9p2*ss[56] + 2520*c1p1*c5p4*c7p3*c9p1*ss[54] + 1512*c1p1*c5p5*c7p1*c9p2*ss[54] + 1512*c1p1*c5p5*c7p2*c9p1*ss[52] + 504*c1p1*c5p6*c7p1*c9p1*ss[50] + 1512*c1p2*c3p1*c5p1*c7p5*ss[48] + 1512*c1p2*c3p1*c5p1*c9p5*ss[58] + 3780*c1p2*c3p1*c5p2*c7p4*ss[46] + 3780*c1p2*c3p1*c5p2*c9p4*ss[54] + 5040*c1p2*c3p1*c5p3*c7p3*ss[44] + 5040*c1p2*c3p1*c5p3*c9p3*ss[50] + 3780*c1p2*c3p1*c5p4*c7p2*ss[42] + 3780*c1p2*c3p1*c5p4*c9p2*ss[46] + 1512*c1p2*c3p1*c5p5*c7p1*ss[40] + 1512*c1p2*c3p1*c5p5*c9p1*ss[42] + 1512*c1p2*c3p1*c7p1*c9p5*ss[60] + 3780*c1p2*c3p1*c7p2*c9p4*ss[58] + 5040*c1p2*c3p1*c7p3*c9p3*ss[56] + 3780*c1p2*c3p1*c7p4*c9p2*ss[54] + 1512*c1p2*c3p1*c7p5*c9p1*ss[52] + 3780*c1p2*c3p2*c5p1*c7p4*ss[44] + 3780*c1p2*c3p2*c5p1*c9p4*ss[52] + 7560*c1p2*c3p2*c5p2*c7p3*ss[42] + 7560*c1p2*c3p2*c5p2*c9p3*ss[48] + 7560*c1p2*c3p2*c5p3*c7p2*ss[40] + 7560*c1p2*c3p2*c5p3*c9p2*ss[44] + 3780*c1p2*c3p2*c5p4*c7p1*ss[38] + 3780*c1p2*c3p2*c5p4*c9p1*ss[40] + 3780*c1p2*c3p2*c7p1*c9p4*ss[54] + 7560*c1p2*c3p2*c7p2*c9p3*ss[52] + 7560*c1p2*c3p2*c7p3*c9p2*ss[50] + 3780*c1p2*c3p2*c7p4*c9p1*ss[48] + 5040*c1p2*c3p3*c5p1*c7p3*ss[40] + 5040*c1p2*c3p3*c5p1*c9p3*ss[46] + 7560*c1p2*c3p3*c5p2*c7p2*ss[38] + 7560*c1p2*c3p3*c5p2*c9p2*ss[42] + 5040*c1p2*c3p3*c5p3*c7p1*ss[36] + 5040*c1p2*c3p3*c5p3*c9p1*ss[38] + 5040*c1p2*c3p3*c7p1*c9p3*ss[48] + 7560*c1p2*c3p3*c7p2*c9p2*ss[46] + 5040*c1p2*c3p3*c7p3*c9p1*ss[44] + 3780*c1p2*c3p4*c5p1*c7p2*ss[36] + 3780*c1p2*c3p4*c5p1*c9p2*ss[40] + 3780*c1p2*c3p4*c5p2*c7p1*ss[34] + 3780*c1p2*c3p4*c5p2*c9p1*ss[36] + 3780*c1p2*c3p4*c7p1*c9p2*ss[42] + 3780*c1p2*c3p4*c7p2*c9p1*ss[40] + 1512*c1p2*c3p5*c5p1*c7p1*ss[32] + 1512*c1p2*c3p5*c5p1*c9p1*ss[34] + 1512*c1p2*c3p5*c7p1*c9p1*ss[36] + 1512*c1p2*c5p1*c7p1*c9p5*ss[62] + 3780*c1p2*c5p1*c7p2*c9p4*ss[60] + 5040*c1p2*c5p1*c7p3*c9p3*ss[58] + 3780*c1p2*c5p1*c7p4*c9p2*ss[56] + 1512*c1p2*c5p1*c7p5*c9p1*ss[54] + 3780*c1p2*c5p2*c7p1*c9p4*ss[58] + 7560*c1p2*c5p2*c7p2*c9p3*ss[56] + 7560*c1p2*c5p2*c7p3*c9p2*ss[54] + 3780*c1p2*c5p2*c7p4*c9p1*ss[52] + 5040*c1p2*c5p3*c7p1*c9p3*ss[54] + 7560*c1p2*c5p3*c7p2*c9p2*ss[52] + 5040*c1p2*c5p3*c7p3*c9p1*ss[50] + 3780*c1p2*c5p4*c7p1*c9p2*ss[50] + 3780*c1p2*c5p4*c7p2*c9p1*ss[48] + 1512*c1p2*c5p5*c7p1*c9p1*ss[46] + 2520*c1p3*c3p1*c5p1*c7p4*ss[42] + 2520*c1p3*c3p1*c5p1*c9p4*ss[50] + 5040*c1p3*c3p1*c5p2*c7p3*ss[40] + 5040*c1p3*c3p1*c5p2*c9p3*ss[46] + 5040*c1p3*c3p1*c5p3*c7p2*ss[38] + 5040*c1p3*c3p1*c5p3*c9p2*ss[42] + 2520*c1p3*c3p1*c5p4*c7p1*ss[36] + 2520*c1p3*c3p1*c5p4*c9p1*ss[38] + 2520*c1p3*c3p1*c7p1*c9p4*ss[52] + 5040*c1p3*c3p1*c7p2*c9p3*ss[50] + 5040*c1p3*c3p1*c7p3*c9p2*ss[48] + 2520*c1p3*c3p1*c7p4*c9p1*ss[46] + 5040*c1p3*c3p2*c5p1*c7p3*ss[38] + 5040*c1p3*c3p2*c5p1*c9p3*ss[44] + 7560*c1p3*c3p2*c5p2*c7p2*ss[36] + 7560*c1p3*c3p2*c5p2*c9p2*ss[40] + 5040*c1p3*c3p2*c5p3*c7p1*ss[34] + 5040*c1p3*c3p2*c5p3*c9p1*ss[36] + 5040*c1p3*c3p2*c7p1*c9p3*ss[46] + 7560*c1p3*c3p2*c7p2*c9p2*ss[44] + 5040*c1p3*c3p2*c7p3*c9p1*ss[42] + 5040*c1p3*c3p3*c5p1*c7p2*ss[34] + 5040*c1p3*c3p3*c5p1*c9p2*ss[38] + 5040*c1p3*c3p3*c5p2*c7p1*ss[32] + 5040*c1p3*c3p3*c5p2*c9p1*ss[34] + 5040*c1p3*c3p3*c7p1*c9p2*ss[40] + 5040*c1p3*c3p3*c7p2*c9p1*ss[38] + 2520*c1p3*c3p4*c5p1*c7p1*ss[30] + 2520*c1p3*c3p4*c5p1*c9p1*ss[32] + 2520*c1p3*c3p4*c7p1*c9p1*ss[34] + 2520*c1p3*c5p1*c7p1*c9p4*ss[54] + 5040*c1p3*c5p1*c7p2*c9p3*ss[52] + 5040*c1p3*c5p1*c7p3*c9p2*ss[50] + 2520*c1p3*c5p1*c7p4*c9p1*ss[48] + 5040*c1p3*c5p2*c7p1*c9p3*ss[50] + 7560*c1p3*c5p2*c7p2*c9p2*ss[48] + 5040*c1p3*c5p2*c7p3*c9p1*ss[46] + 5040*c1p3*c5p3*c7p1*c9p2*ss[46] + 5040*c1p3*c5p3*c7p2*c9p1*ss[44] + 2520*c1p3*c5p4*c7p1*c9p1*ss[42] + 2520*c1p4*c3p1*c5p1*c7p3*ss[36] + 2520*c1p4*c3p1*c5p1*c9p3*ss[42] + 3780*c1p4*c3p1*c5p2*c7p2*ss[34] + 3780*c1p4*c3p1*c5p2*c9p2*ss[38] + 2520*c1p4*c3p1*c5p3*c7p1*ss[32] + 2520*c1p4*c3p1*c5p3*c9p1*ss[34] + 2520*c1p4*c3p1*c7p1*c9p3*ss[44] + 3780*c1p4*c3p1*c7p2*c9p2*ss[42] + 2520*c1p4*c3p1*c7p3*c9p1*ss[40] + 3780*c1p4*c3p2*c5p1*c7p2*ss[32] + 3780*c1p4*c3p2*c5p1*c9p2*ss[36] + 3780*c1p4*c3p2*c5p2*c7p1*ss[30] + 3780*c1p4*c3p2*c5p2*c9p1*ss[32] + 3780*c1p4*c3p2*c7p1*c9p2*ss[38] + 3780*c1p4*c3p2*c7p2*c9p1*ss[36] + 2520*c1p4*c3p3*c5p1*c7p1*ss[28] + 2520*c1p4*c3p3*c5p1*c9p1*ss[30] + 2520*c1p4*c3p3*c7p1*c9p1*ss[32] + 2520*c1p4*c5p1*c7p1*c9p3*ss[46] + 3780*c1p4*c5p1*c7p2*c9p2*ss[44] + 2520*c1p4*c5p1*c7p3*c9p1*ss[42] + 3780*c1p4*c5p2*c7p1*c9p2*ss[42] + 3780*c1p4*c5p2*c7p2*c9p1*ss[40] + 2520*c1p4*c5p3*c7p1*c9p1*ss[38] + 1512*c1p5*c3p1*c5p1*c7p2*ss[30] + 1512*c1p5*c3p1*c5p1*c9p2*ss[34] + 1512*c1p5*c3p1*c5p2*c7p1*ss[28] + 1512*c1p5*c3p1*c5p2*c9p1*ss[30] + 1512*c1p5*c3p1*c7p1*c9p2*ss[36] + 1512*c1p5*c3p1*c7p2*c9p1*ss[34] + 1512*c1p5*c3p2*c5p1*c7p1*ss[26] + 1512*c1p5*c3p2*c5p1*c9p1*ss[28] + 1512*c1p5*c3p2*c7p1*c9p1*ss[30] + 1512*c1p5*c5p1*c7p1*c9p2*ss[38] + 1512*c1p5*c5p1*c7p2*c9p1*ss[36] + 1512*c1p5*c5p2*c7p1*c9p1*ss[34] + 504*c1p6*c3p1*c5p1*c7p1*ss[24] + 504*c1p6*c3p1*c5p1*c9p1*ss[26] + 504*c1p6*c3p1*c7p1*c9p1*ss[28] + 504*c1p6*c5p1*c7p1*c9p1*ss[30] + 504*c3p1*c5p1*c7p1*c9p6*ss[72] + 1512*c3p1*c5p1*c7p2*c9p5*ss[70] + 2520*c3p1*c5p1*c7p3*c9p4*ss[68] + 2520*c3p1*c5p1*c7p4*c9p3*ss[66] + 1512*c3p1*c5p1*c7p5*c9p2*ss[64] + 504*c3p1*c5p1*c7p6*c9p1*ss[62] + 1512*c3p1*c5p2*c7p1*c9p5*ss[68] + 3780*c3p1*c5p2*c7p2*c9p4*ss[66] + 5040*c3p1*c5p2*c7p3*c9p3*ss[64] + 3780*c3p1*c5p2*c7p4*c9p2*ss[62] + 1512*c3p1*c5p2*c7p5*c9p1*ss[60] + 2520*c3p1*c5p3*c7p1*c9p4*ss[64] + 5040*c3p1*c5p3*c7p2*c9p3*ss[62] + 5040*c3p1*c5p3*c7p3*c9p2*ss[60] + 2520*c3p1*c5p3*c7p4*c9p1*ss[58] + 2520*c3p1*c5p4*c7p1*c9p3*ss[60] + 3780*c3p1*c5p4*c7p2*c9p2*ss[58] + 2520*c3p1*c5p4*c7p3*c9p1*ss[56] + 1512*c3p1*c5p5*c7p1*c9p2*ss[56] + 1512*c3p1*c5p5*c7p2*c9p1*ss[54] + 504*c3p1*c5p6*c7p1*c9p1*ss[52] + 1512*c3p2*c5p1*c7p1*c9p5*ss[66] + 3780*c3p2*c5p1*c7p2*c9p4*ss[64] + 5040*c3p2*c5p1*c7p3*c9p3*ss[62] + 3780*c3p2*c5p1*c7p4*c9p2*ss[60] + 1512*c3p2*c5p1*c7p5*c9p1*ss[58] + 3780*c3p2*c5p2*c7p1*c9p4*ss[62] + 7560*c3p2*c5p2*c7p2*c9p3*ss[60] + 7560*c3p2*c5p2*c7p3*c9p2*ss[58] + 3780*c3p2*c5p2*c7p4*c9p1*ss[56] + 5040*c3p2*c5p3*c7p1*c9p3*ss[58] + 7560*c3p2*c5p3*c7p2*c9p2*ss[56] + 5040*c3p2*c5p3*c7p3*c9p1*ss[54] + 3780*c3p2*c5p4*c7p1*c9p2*ss[54] + 3780*c3p2*c5p4*c7p2*c9p1*ss[52] + 1512*c3p2*c5p5*c7p1*c9p1*ss[50] + 2520*c3p3*c5p1*c7p1*c9p4*ss[60] + 5040*c3p3*c5p1*c7p2*c9p3*ss[58] + 5040*c3p3*c5p1*c7p3*c9p2*ss[56] + 2520*c3p3*c5p1*c7p4*c9p1*ss[54] + 5040*c3p3*c5p2*c7p1*c9p3*ss[56] + 7560*c3p3*c5p2*c7p2*c9p2*ss[54] + 5040*c3p3*c5p2*c7p3*c9p1*ss[52] + 5040*c3p3*c5p3*c7p1*c9p2*ss[52] + 5040*c3p3*c5p3*c7p2*c9p1*ss[50] + 2520*c3p3*c5p4*c7p1*c9p1*ss[48] + 2520*c3p4*c5p1*c7p1*c9p3*ss[54] + 3780*c3p4*c5p1*c7p2*c9p2*ss[52] + 2520*c3p4*c5p1*c7p3*c9p1*ss[50] + 3780*c3p4*c5p2*c7p1*c9p2*ss[50] + 3780*c3p4*c5p2*c7p2*c9p1*ss[48] + 2520*c3p4*c5p3*c7p1*c9p1*ss[46] + 1512*c3p5*c5p1*c7p1*c9p2*ss[48] + 1512*c3p5*c5p1*c7p2*c9p1*ss[46] + 1512*c3p5*c5p2*c7p1*c9p1*ss[44] + 504*c3p6*c5p1*c7p1*c9p1*ss[42] + 72*c1p1*c3p1*c5p7*ss[42] + 72*c1p1*c3p1*c7p7*ss[56] + 72*c1p1*c3p1*c9p7*ss[70] + 252*c1p1*c3p2*c5p6*ss[40] + 252*c1p1*c3p2*c7p6*ss[52] + 252*c1p1*c3p2*c9p6*ss[64] + 504*c1p1*c3p3*c5p5*ss[38] + 504*c1p1*c3p3*c7p5*ss[48] + 504*c1p1*c3p3*c9p5*ss[58] + 630*c1p1*c3p4*c5p4*ss[36] + 630*c1p1*c3p4*c7p4*ss[44] + 630*c1p1*c3p4*c9p4*ss[52] + 504*c1p1*c3p5*c5p3*ss[34] + 504*c1p1*c3p5*c7p3*ss[40] + 504*c1p1*c3p5*c9p3*ss[46] + 252*c1p1*c3p6*c5p2*ss[32] + 252*c1p1*c3p6*c7p2*ss[36] + 252*c1p1*c3p6*c9p2*ss[40] + 72*c1p1*c3p7*c5p1*ss[30] + 72*c1p1*c3p7*c7p1*ss[32] + 72*c1p1*c3p7*c9p1*ss[34] + 72*c1p1*c5p1*c7p7*ss[58] + 72*c1p1*c5p1*c9p7*ss[72] + 252*c1p1*c5p2*c7p6*ss[56] + 252*c1p1*c5p2*c9p6*ss[68] + 504*c1p1*c5p3*c7p5*ss[54] + 504*c1p1*c5p3*c9p5*ss[64] + 630*c1p1*c5p4*c7p4*ss[52] + 630*c1p1*c5p4*c9p4*ss[60] + 504*c1p1*c5p5*c7p3*ss[50] + 504*c1p1*c5p5*c9p3*ss[56] + 252*c1p1*c5p6*c7p2*ss[48] + 252*c1p1*c5p6*c9p2*ss[52] + 72*c1p1*c5p7*c7p1*ss[46] + 72*c1p1*c5p7*c9p1*ss[48] + 72*c1p1*c7p1*c9p7*ss[74] + 252*c1p1*c7p2*c9p6*ss[72] + 504*c1p1*c7p3*c9p5*ss[70] + 630*c1p1*c7p4*c9p4*ss[68] + 504*c1p1*c7p5*c9p3*ss[66] + 252*c1p1*c7p6*c9p2*ss[64] + 72*c1p1*c7p7*c9p1*ss[62] + 252*c1p2*c3p1*c5p6*ss[38] + 252*c1p2*c3p1*c7p6*ss[50] + 252*c1p2*c3p1*c9p6*ss[62] + 756*c1p2*c3p2*c5p5*ss[36] + 756*c1p2*c3p2*c7p5*ss[46] + 756*c1p2*c3p2*c9p5*ss[56] + 1260*c1p2*c3p3*c5p4*ss[34] + 1260*c1p2*c3p3*c7p4*ss[42] + 1260*c1p2*c3p3*c9p4*ss[50] + 1260*c1p2*c3p4*c5p3*ss[32] + 1260*c1p2*c3p4*c7p3*ss[38] + 1260*c1p2*c3p4*c9p3*ss[44] + 756*c1p2*c3p5*c5p2*ss[30] + 756*c1p2*c3p5*c7p2*ss[34] + 756*c1p2*c3p5*c9p2*ss[38] + 252*c1p2*c3p6*c5p1*ss[28] + 252*c1p2*c3p6*c7p1*ss[30] + 252*c1p2*c3p6*c9p1*ss[32] + 252*c1p2*c5p1*c7p6*ss[52] + 252*c1p2*c5p1*c9p6*ss[64] + 756*c1p2*c5p2*c7p5*ss[50] + 756*c1p2*c5p2*c9p5*ss[60] + 1260*c1p2*c5p3*c7p4*ss[48] + 1260*c1p2*c5p3*c9p4*ss[56] + 1260*c1p2*c5p4*c7p3*ss[46] + 1260*c1p2*c5p4*c9p3*ss[52] + 756*c1p2*c5p5*c7p2*ss[44] + 756*c1p2*c5p5*c9p2*ss[48] + 252*c1p2*c5p6*c7p1*ss[42] + 252*c1p2*c5p6*c9p1*ss[44] + 252*c1p2*c7p1*c9p6*ss[66] + 756*c1p2*c7p2*c9p5*ss[64] + 1260*c1p2*c7p3*c9p4*ss[62] + 1260*c1p2*c7p4*c9p3*ss[60] + 756*c1p2*c7p5*c9p2*ss[58] + 252*c1p2*c7p6*c9p1*ss[56] + 504*c1p3*c3p1*c5p5*ss[34] + 504*c1p3*c3p1*c7p5*ss[44] + 504*c1p3*c3p1*c9p5*ss[54] + 1260*c1p3*c3p2*c5p4*ss[32] + 1260*c1p3*c3p2*c7p4*ss[40] + 1260*c1p3*c3p2*c9p4*ss[48] + 1680*c1p3*c3p3*c5p3*ss[30] + 1680*c1p3*c3p3*c7p3*ss[36] + 1680*c1p3*c3p3*c9p3*ss[42] + 1260*c1p3*c3p4*c5p2*ss[28] + 1260*c1p3*c3p4*c7p2*ss[32] + 1260*c1p3*c3p4*c9p2*ss[36] + 504*c1p3*c3p5*c5p1*ss[26] + 504*c1p3*c3p5*c7p1*ss[28] + 504*c1p3*c3p5*c9p1*ss[30] + 504*c1p3*c5p1*c7p5*ss[46] + 504*c1p3*c5p1*c9p5*ss[56] + 1260*c1p3*c5p2*c7p4*ss[44] + 1260*c1p3*c5p2*c9p4*ss[52] + 1680*c1p3*c5p3*c7p3*ss[42] + 1680*c1p3*c5p3*c9p3*ss[48] + 1260*c1p3*c5p4*c7p2*ss[40] + 1260*c1p3*c5p4*c9p2*ss[44] + 504*c1p3*c5p5*c7p1*ss[38] + 504*c1p3*c5p5*c9p1*ss[40] + 504*c1p3*c7p1*c9p5*ss[58] + 1260*c1p3*c7p2*c9p4*ss[56] + 1680*c1p3*c7p3*c9p3*ss[54] + 1260*c1p3*c7p4*c9p2*ss[52] + 504*c1p3*c7p5*c9p1*ss[50] + 630*c1p4*c3p1*c5p4*ss[30] + 630*c1p4*c3p1*c7p4*ss[38] + 630*c1p4*c3p1*c9p4*ss[46] + 1260*c1p4*c3p2*c5p3*ss[28] + 1260*c1p4*c3p2*c7p3*ss[34] + 1260*c1p4*c3p2*c9p3*ss[40] + 1260*c1p4*c3p3*c5p2*ss[26] + 1260*c1p4*c3p3*c7p2*ss[30] + 1260*c1p4*c3p3*c9p2*ss[34] + 630*c1p4*c3p4*c5p1*ss[24] + 630*c1p4*c3p4*c7p1*ss[26] + 630*c1p4*c3p4*c9p1*ss[28] + 630*c1p4*c5p1*c7p4*ss[40] + 630*c1p4*c5p1*c9p4*ss[48] + 1260*c1p4*c5p2*c7p3*ss[38] + 1260*c1p4*c5p2*c9p3*ss[44] + 1260*c1p4*c5p3*c7p2*ss[36] + 1260*c1p4*c5p3*c9p2*ss[40] + 630*c1p4*c5p4*c7p1*ss[34] + 630*c1p4*c5p4*c9p1*ss[36] + 630*c1p4*c7p1*c9p4*ss[50] + 1260*c1p4*c7p2*c9p3*ss[48] + 1260*c1p4*c7p3*c9p2*ss[46] + 630*c1p4*c7p4*c9p1*ss[44] + 504*c1p5*c3p1*c5p3*ss[26] + 504*c1p5*c3p1*c7p3*ss[32] + 504*c1p5*c3p1*c9p3*ss[38] + 756*c1p5*c3p2*c5p2*ss[24] + 756*c1p5*c3p2*c7p2*ss[28] + 756*c1p5*c3p2*c9p2*ss[32] + 504*c1p5*c3p3*c5p1*ss[22] + 504*c1p5*c3p3*c7p1*ss[24] + 504*c1p5*c3p3*c9p1*ss[26] + 504*c1p5*c5p1*c7p3*ss[34] + 504*c1p5*c5p1*c9p3*ss[40] + 756*c1p5*c5p2*c7p2*ss[32] + 756*c1p5*c5p2*c9p2*ss[36] + 504*c1p5*c5p3*c7p1*ss[30] + 504*c1p5*c5p3*c9p1*ss[32] + 504*c1p5*c7p1*c9p3*ss[42] + 756*c1p5*c7p2*c9p2*ss[40] + 504*c1p5*c7p3*c9p1*ss[38] + 252*c1p6*c3p1*c5p2*ss[22] + 252*c1p6*c3p1*c7p2*ss[26] + 252*c1p6*c3p1*c9p2*ss[30] + 252*c1p6*c3p2*c5p1*ss[20] + 252*c1p6*c3p2*c7p1*ss[22] + 252*c1p6*c3p2*c9p1*ss[24] + 252*c1p6*c5p1*c7p2*ss[28] + 252*c1p6*c5p1*c9p2*ss[32] + 252*c1p6*c5p2*c7p1*ss[26] + 252*c1p6*c5p2*c9p1*ss[28] + 252*c1p6*c7p1*c9p2*ss[34] + 252*c1p6*c7p2*c9p1*ss[32] + 72*c1p7*c3p1*c5p1*ss[18] + 72*c1p7*c3p1*c7p1*ss[20] + 72*c1p7*c3p1*c9p1*ss[22] + 72*c1p7*c5p1*c7p1*ss[22] + 72*c1p7*c5p1*c9p1*ss[24] + 72*c1p7*c7p1*c9p1*ss[26] + 72*c3p1*c5p1*c7p7*ss[60] + 72*c3p1*c5p1*c9p7*ss[74] + 252*c3p1*c5p2*c7p6*ss[58] + 252*c3p1*c5p2*c9p6*ss[70] + 504*c3p1*c5p3*c7p5*ss[56] + 504*c3p1*c5p3*c9p5*ss[66] + 630*c3p1*c5p4*c7p4*ss[54] + 630*c3p1*c5p4*c9p4*ss[62] + 504*c3p1*c5p5*c7p3*ss[52] + 504*c3p1*c5p5*c9p3*ss[58] + 252*c3p1*c5p6*c7p2*ss[50] + 252*c3p1*c5p6*c9p2*ss[54] + 72*c3p1*c5p7*c7p1*ss[48] + 72*c3p1*c5p7*c9p1*ss[50] + 72*c3p1*c7p1*c9p7*ss[76] + 252*c3p1*c7p2*c9p6*ss[74] + 504*c3p1*c7p3*c9p5*ss[72] + 630*c3p1*c7p4*c9p4*ss[70] + 504*c3p1*c7p5*c9p3*ss[68] + 252*c3p1*c7p6*c9p2*ss[66] + 72*c3p1*c7p7*c9p1*ss[64] + 252*c3p2*c5p1*c7p6*ss[56] + 252*c3p2*c5p1*c9p6*ss[68] + 756*c3p2*c5p2*c7p5*ss[54] + 756*c3p2*c5p2*c9p5*ss[64] + 1260*c3p2*c5p3*c7p4*ss[52] + 1260*c3p2*c5p3*c9p4*ss[60] + 1260*c3p2*c5p4*c7p3*ss[50] + 1260*c3p2*c5p4*c9p3*ss[56] + 756*c3p2*c5p5*c7p2*ss[48] + 756*c3p2*c5p5*c9p2*ss[52] + 252*c3p2*c5p6*c7p1*ss[46] + 252*c3p2*c5p6*c9p1*ss[48] + 252*c3p2*c7p1*c9p6*ss[70] + 756*c3p2*c7p2*c9p5*ss[68] + 1260*c3p2*c7p3*c9p4*ss[66] + 1260*c3p2*c7p4*c9p3*ss[64] + 756*c3p2*c7p5*c9p2*ss[62] + 252*c3p2*c7p6*c9p1*ss[60] + 504*c3p3*c5p1*c7p5*ss[52] + 504*c3p3*c5p1*c9p5*ss[62] + 1260*c3p3*c5p2*c7p4*ss[50] + 1260*c3p3*c5p2*c9p4*ss[58] + 1680*c3p3*c5p3*c7p3*ss[48] + 1680*c3p3*c5p3*c9p3*ss[54] + 1260*c3p3*c5p4*c7p2*ss[46] + 1260*c3p3*c5p4*c9p2*ss[50] + 504*c3p3*c5p5*c7p1*ss[44] + 504*c3p3*c5p5*c9p1*ss[46] + 504*c3p3*c7p1*c9p5*ss[64] + 1260*c3p3*c7p2*c9p4*ss[62] + 1680*c3p3*c7p3*c9p3*ss[60] + 1260*c3p3*c7p4*c9p2*ss[58] + 504*c3p3*c7p5*c9p1*ss[56] + 630*c3p4*c5p1*c7p4*ss[48] + 630*c3p4*c5p1*c9p4*ss[56] + 1260*c3p4*c5p2*c7p3*ss[46] + 1260*c3p4*c5p2*c9p3*ss[52] + 1260*c3p4*c5p3*c7p2*ss[44] + 1260*c3p4*c5p3*c9p2*ss[48] + 630*c3p4*c5p4*c7p1*ss[42] + 630*c3p4*c5p4*c9p1*ss[44] + 630*c3p4*c7p1*c9p4*ss[58] + 1260*c3p4*c7p2*c9p3*ss[56] + 1260*c3p4*c7p3*c9p2*ss[54] + 630*c3p4*c7p4*c9p1*ss[52] + 504*c3p5*c5p1*c7p3*ss[44] + 504*c3p5*c5p1*c9p3*ss[50] + 756*c3p5*c5p2*c7p2*ss[42] + 756*c3p5*c5p2*c9p2*ss[46] + 504*c3p5*c5p3*c7p1*ss[40] + 504*c3p5*c5p3*c9p1*ss[42] + 504*c3p5*c7p1*c9p3*ss[52] + 756*c3p5*c7p2*c9p2*ss[50] + 504*c3p5*c7p3*c9p1*ss[48] + 252*c3p6*c5p1*c7p2*ss[40] + 252*c3p6*c5p1*c9p2*ss[44] + 252*c3p6*c5p2*c7p1*ss[38] + 252*c3p6*c5p2*c9p1*ss[40] + 252*c3p6*c7p1*c9p2*ss[46] + 252*c3p6*c7p2*c9p1*ss[44] + 72*c3p7*c5p1*c7p1*ss[36] + 72*c3p7*c5p1*c9p1*ss[38] + 72*c3p7*c7p1*c9p1*ss[40] + 72*c5p1*c7p1*c9p7*ss[78] + 252*c5p1*c7p2*c9p6*ss[76] + 504*c5p1*c7p3*c9p5*ss[74] + 630*c5p1*c7p4*c9p4*ss[72] + 504*c5p1*c7p5*c9p3*ss[70] + 252*c5p1*c7p6*c9p2*ss[68] + 72*c5p1*c7p7*c9p1*ss[66] + 252*c5p2*c7p1*c9p6*ss[74] + 756*c5p2*c7p2*c9p5*ss[72] + 1260*c5p2*c7p3*c9p4*ss[70] + 1260*c5p2*c7p4*c9p3*ss[68] + 756*c5p2*c7p5*c9p2*ss[66] + 252*c5p2*c7p6*c9p1*ss[64] + 504*c5p3*c7p1*c9p5*ss[70] + 1260*c5p3*c7p2*c9p4*ss[68] + 1680*c5p3*c7p3*c9p3*ss[66] + 1260*c5p3*c7p4*c9p2*ss[64] + 504*c5p3*c7p5*c9p1*ss[62] + 630*c5p4*c7p1*c9p4*ss[66] + 1260*c5p4*c7p2*c9p3*ss[64] + 1260*c5p4*c7p3*c9p2*ss[62] + 630*c5p4*c7p4*c9p1*ss[60] + 504*c5p5*c7p1*c9p3*ss[62] + 756*c5p5*c7p2*c9p2*ss[60] + 504*c5p5*c7p3*c9p1*ss[58] + 252*c5p6*c7p1*c9p2*ss[58] + 252*c5p6*c7p2*c9p1*ss[56] + 72*c5p7*c7p1*c9p1*ss[54] + 9*c1p1*c3p8*ss[28] + 9*c1p1*c5p8*ss[44] + 9*c1p1*c7p8*ss[60] + 9*c1p1*c9p8*ss[76] + 36*c1p2*c3p7*ss[26] + 36*c1p2*c5p7*ss[40] + 36*c1p2*c7p7*ss[54] + 36*c1p2*c9p7*ss[68] + 84*c1p3*c3p6*ss[24] + 84*c1p3*c5p6*ss[36] + 84*c1p3*c7p6*ss[48] + 84*c1p3*c9p6*ss[60] + 126*c1p4*c3p5*ss[22] + 126*c1p4*c5p5*ss[32] + 126*c1p4*c7p5*ss[42] + 126*c1p4*c9p5*ss[52] + 126*c1p5*c3p4*ss[20] + 126*c1p5*c5p4*ss[28] + 126*c1p5*c7p4*ss[36] + 126*c1p5*c9p4*ss[44] + 84*c1p6*c3p3*ss[18] + 84*c1p6*c5p3*ss[24] + 84*c1p6*c7p3*ss[30] + 84*c1p6*c9p3*ss[36] + 36*c1p7*c3p2*ss[16] + 36*c1p7*c5p2*ss[20] + 36*c1p7*c7p2*ss[24] + 36*c1p7*c9p2*ss[28] + 9*c1p8*c3p1*ss[14] + 9*c1p8*c5p1*ss[16] + 9*c1p8*c7p1*ss[18] + 9*c1p8*c9p1*ss[20] + 9*c3p1*c5p8*ss[46] + 9*c3p1*c7p8*ss[62] + 9*c3p1*c9p8*ss[78] + 36*c3p2*c5p7*ss[44] + 36*c3p2*c7p7*ss[58] + 36*c3p2*c9p7*ss[72] + 84*c3p3*c5p6*ss[42] + 84*c3p3*c7p6*ss[54] + 84*c3p3*c9p6*ss[66] + 126*c3p4*c5p5*ss[40] + 126*c3p4*c7p5*ss[50] + 126*c3p4*c9p5*ss[60] + 126*c3p5*c5p4*ss[38] + 126*c3p5*c7p4*ss[46] + 126*c3p5*c9p4*ss[54] + 84*c3p6*c5p3*ss[36] + 84*c3p6*c7p3*ss[42] + 84*c3p6*c9p3*ss[48] + 36*c3p7*c5p2*ss[34] + 36*c3p7*c7p2*ss[38] + 36*c3p7*c9p2*ss[42] + 9*c3p8*c5p1*ss[32] + 9*c3p8*c7p1*ss[34] + 9*c3p8*c9p1*ss[36] + 9*c5p1*c7p8*ss[64] + 9*c5p1*c9p8*ss[80] + 36*c5p2*c7p7*ss[62] + 36*c5p2*c9p7*ss[76] + 84*c5p3*c7p6*ss[60] + 84*c5p3*c9p6*ss[72] + 126*c5p4*c7p5*ss[58] + 126*c5p4*c9p5*ss[68] + 126*c5p5*c7p4*ss[56] + 126*c5p5*c9p4*ss[64] + 84*c5p6*c7p3*ss[54] + 84*c5p6*c9p3*ss[60] + 36*c5p7*c7p2*ss[52] + 36*c5p7*c9p2*ss[56] + 9*c5p8*c7p1*ss[50] + 9*c5p8*c9p1*ss[52] + 9*c7p1*c9p8*ss[82] + 36*c7p2*c9p7*ss[80] + 84*c7p3*c9p6*ss[78] + 126*c7p4*c9p5*ss[76] + 126*c7p5*c9p4*ss[74] + 84*c7p6*c9p3*ss[72] + 36*c7p7*c9p2*ss[70] + 9*c7p8*c9p1*ss[68] + c1p9*ss[12] + c3p9*ss[30] + c5p9*ss[48] + c7p9*ss[66] + c9p9*ss[84])*sys_part_4;
  TACC grad_1_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4)));
  TACC grad_2_num = convert<TACC>(2*(c1p1*ss[6] + c3p1*ss[8] + c5p1*ss[10] + c7p1*ss[12] + c9p1*ss[14])*sys_part_0);
  grad_2_num += 4*(6*c1p1*c3p1*c5p1*ss[14] + 6*c1p1*c3p1*c7p1*ss[16] + 6*c1p1*c3p1*c9p1*ss[18] + 6*c1p1*c5p1*c7p1*ss[18] + 6*c1p1*c5p1*c9p1*ss[20] + 6*c1p1*c7p1*c9p1*ss[22] + 6*c3p1*c5p1*c7p1*ss[20] + 6*c3p1*c5p1*c9p1*ss[22] + 6*c3p1*c7p1*c9p1*ss[24] + 6*c5p1*c7p1*c9p1*ss[26] + 3*c1p1*c3p2*ss[12] + 3*c1p1*c5p2*ss[16] + 3*c1p1*c7p2*ss[20] + 3*c1p1*c9p2*ss[24] + 3*c1p2*c3p1*ss[10] + 3*c1p2*c5p1*ss[12] + 3*c1p2*c7p1*ss[14] + 3*c1p2*c9p1*ss[16] + 3*c3p1*c5p2*ss[18] + 3*c3p1*c7p2*ss[22] + 3*c3p1*c9p2*ss[26] + 3*c3p2*c5p1*ss[16] + 3*c3p2*c7p1*ss[18] + 3*c3p2*c9p1*ss[20] + 3*c5p1*c7p2*ss[24] + 3*c5p1*c9p2*ss[28] + 3*c5p2*c7p1*ss[22] + 3*c5p2*c9p1*ss[24] + 3*c7p1*c9p2*ss[30] + 3*c7p2*c9p1*ss[28] + c1p3*ss[8] + c3p3*ss[14] + c5p3*ss[20] + c7p3*ss[26] + c9p3*ss[32])*sys_part_1;
  grad_2_num += 6*(120*c1p1*c3p1*c5p1*c7p1*c9p1*ss[30] + 60*c1p1*c3p1*c5p1*c7p2*ss[28] + 60*c1p1*c3p1*c5p1*c9p2*ss[32] + 60*c1p1*c3p1*c5p2*c7p1*ss[26] + 60*c1p1*c3p1*c5p2*c9p1*ss[28] + 60*c1p1*c3p1*c7p1*c9p2*ss[34] + 60*c1p1*c3p1*c7p2*c9p1*ss[32] + 60*c1p1*c3p2*c5p1*c7p1*ss[24] + 60*c1p1*c3p2*c5p1*c9p1*ss[26] + 60*c1p1*c3p2*c7p1*c9p1*ss[28] + 60*c1p1*c5p1*c7p1*c9p2*ss[36] + 60*c1p1*c5p1*c7p2*c9p1*ss[34] + 60*c1p1*c5p2*c7p1*c9p1*ss[32] + 60*c1p2*c3p1*c5p1*c7p1*ss[22] + 60*c1p2*c3p1*c5p1*c9p1*ss[24] + 60*c1p2*c3p1*c7p1*c9p1*ss[26] + 60*c1p2*c5p1*c7p1*c9p1*ss[28] + 60*c3p1*c5p1*c7p1*c9p2*ss[38] + 60*c3p1*c5p1*c7p2*c9p1*ss[36] + 60*c3p1*c5p2*c7p1*c9p1*ss[34] + 60*c3p2*c5p1*c7p1*c9p1*ss[32] + 20*c1p1*c3p1*c5p3*ss[24] + 20*c1p1*c3p1*c7p3*ss[30] + 20*c1p1*c3p1*c9p3*ss[36] + 30*c1p1*c3p2*c5p2*ss[22] + 30*c1p1*c3p2*c7p2*ss[26] + 30*c1p1*c3p2*c9p2*ss[30] + 20*c1p1*c3p3*c5p1*ss[20] + 20*c1p1*c3p3*c7p1*ss[22] + 20*c1p1*c3p3*c9p1*ss[24] + 20*c1p1*c5p1*c7p3*ss[32] + 20*c1p1*c5p1*c9p3*ss[38] + 30*c1p1*c5p2*c7p2*ss[30] + 30*c1p1*c5p2*c9p2*ss[34] + 20*c1p1*c5p3*c7p1*ss[28] + 20*c1p1*c5p3*c9p1*ss[30] + 20*c1p1*c7p1*c9p3*ss[40] + 30*c1p1*c7p2*c9p2*ss[38] + 20*c1p1*c7p3*c9p1*ss[36] + 30*c1p2*c3p1*c5p2*ss[20] + 30*c1p2*c3p1*c7p2*ss[24] + 30*c1p2*c3p1*c9p2*ss[28] + 30*c1p2*c3p2*c5p1*ss[18] + 30*c1p2*c3p2*c7p1*ss[20] + 30*c1p2*c3p2*c9p1*ss[22] + 30*c1p2*c5p1*c7p2*ss[26] + 30*c1p2*c5p1*c9p2*ss[30] + 30*c1p2*c5p2*c7p1*ss[24] + 30*c1p2*c5p2*c9p1*ss[26] + 30*c1p2*c7p1*c9p2*ss[32] + 30*c1p2*c7p2*c9p1*ss[30] + 20*c1p3*c3p1*c5p1*ss[16] + 20*c1p3*c3p1*c7p1*ss[18] + 20*c1p3*c3p1*c9p1*ss[20] + 20*c1p3*c5p1*c7p1*ss[20] + 20*c1p3*c5p1*c9p1*ss[22] + 20*c1p3*c7p1*c9p1*ss[24] + 20*c3p1*c5p1*c7p3*ss[34] + 20*c3p1*c5p1*c9p3*ss[40] + 30*c3p1*c5p2*c7p2*ss[32] + 30*c3p1*c5p2*c9p2*ss[36] + 20*c3p1*c5p3*c7p1*ss[30] + 20*c3p1*c5p3*c9p1*ss[32] + 20*c3p1*c7p1*c9p3*ss[42] + 30*c3p1*c7p2*c9p2*ss[40] + 20*c3p1*c7p3*c9p1*ss[38] + 30*c3p2*c5p1*c7p2*ss[30] + 30*c3p2*c5p1*c9p2*ss[34] + 30*c3p2*c5p2*c7p1*ss[28] + 30*c3p2*c5p2*c9p1*ss[30] + 30*c3p2*c7p1*c9p2*ss[36] + 30*c3p2*c7p2*c9p1*ss[34] + 20*c3p3*c5p1*c7p1*ss[26] + 20*c3p3*c5p1*c9p1*ss[28] + 20*c3p3*c7p1*c9p1*ss[30] + 20*c5p1*c7p1*c9p3*ss[44] + 30*c5p1*c7p2*c9p2*ss[42] + 20*c5p1*c7p3*c9p1*ss[40] + 30*c5p2*c7p1*c9p2*ss[40] + 30*c5p2*c7p2*c9p1*ss[38] + 20*c5p3*c7p1*c9p1*ss[36] + 5*c1p1*c3p4*ss[18] + 5*c1p1*c5p4*ss[26] + 5*c1p1*c7p4*ss[34] + 5*c1p1*c9p4*ss[42] + 10*c1p2*c3p3*ss[16] + 10*c1p2*c5p3*ss[22] + 10*c1p2*c7p3*ss[28] + 10*c1p2*c9p3*ss[34] + 10*c1p3*c3p2*ss[14] + 10*c1p3*c5p2*ss[18] + 10*c1p3*c7p2*ss[22] + 10*c1p3*c9p2*ss[26] + 5*c1p4*c3p1*ss[12] + 5*c1p4*c5p1*ss[14] + 5*c1p4*c7p1*ss[16] + 5*c1p4*c9p1*ss[18] + 5*c3p1*c5p4*ss[28] + 5*c3p1*c7p4*ss[36] + 5*c3p1*c9p4*ss[44] + 10*c3p2*c5p3*ss[26] + 10*c3p2*c7p3*ss[32] + 10*c3p2*c9p3*ss[38] + 10*c3p3*c5p2*ss[24] + 10*c3p3*c7p2*ss[28] + 10*c3p3*c9p2*ss[32] + 5*c3p4*c5p1*ss[22] + 5*c3p4*c7p1*ss[24] + 5*c3p4*c9p1*ss[26] + 5*c5p1*c7p4*ss[38] + 5*c5p1*c9p4*ss[46] + 10*c5p2*c7p3*ss[36] + 10*c5p2*c9p3*ss[42] + 10*c5p3*c7p2*ss[34] + 10*c5p3*c9p2*ss[38] + 5*c5p4*c7p1*ss[32] + 5*c5p4*c9p1*ss[34] + 5*c7p1*c9p4*ss[48] + 10*c7p2*c9p3*ss[46] + 10*c7p3*c9p2*ss[44] + 5*c7p4*c9p1*ss[42] + c1p5*ss[10] + c3p5*ss[20] + c5p5*ss[30] + c7p5*ss[40] + c9p5*ss[50])*sys_part_2;
  grad_2_num += 8*(840*c1p1*c3p1*c5p1*c7p1*c9p3*ss[48] + 1260*c1p1*c3p1*c5p1*c7p2*c9p2*ss[46] + 840*c1p1*c3p1*c5p1*c7p3*c9p1*ss[44] + 1260*c1p1*c3p1*c5p2*c7p1*c9p2*ss[44] + 1260*c1p1*c3p1*c5p2*c7p2*c9p1*ss[42] + 840*c1p1*c3p1*c5p3*c7p1*c9p1*ss[40] + 1260*c1p1*c3p2*c5p1*c7p1*c9p2*ss[42] + 1260*c1p1*c3p2*c5p1*c7p2*c9p1*ss[40] + 1260*c1p1*c3p2*c5p2*c7p1*c9p1*ss[38] + 840*c1p1*c3p3*c5p1*c7p1*c9p1*ss[36] + 1260*c1p2*c3p1*c5p1*c7p1*c9p2*ss[40] + 1260*c1p2*c3p1*c5p1*c7p2*c9p1*ss[38] + 1260*c1p2*c3p1*c5p2*c7p1*c9p1*ss[36] + 1260*c1p2*c3p2*c5p1*c7p1*c9p1*ss[34] + 840*c1p3*c3p1*c5p1*c7p1*c9p1*ss[32] + 210*c1p1*c3p1*c5p1*c7p4*ss[42] + 210*c1p1*c3p1*c5p1*c9p4*ss[50] + 420*c1p1*c3p1*c5p2*c7p3*ss[40] + 420*c1p1*c3p1*c5p2*c9p3*ss[46] + 420*c1p1*c3p1*c5p3*c7p2*ss[38] + 420*c1p1*c3p1*c5p3*c9p2*ss[42] + 210*c1p1*c3p1*c5p4*c7p1*ss[36] + 210*c1p1*c3p1*c5p4*c9p1*ss[38] + 210*c1p1*c3p1*c7p1*c9p4*ss[52] + 420*c1p1*c3p1*c7p2*c9p3*ss[50] + 420*c1p1*c3p1*c7p3*c9p2*ss[48] + 210*c1p1*c3p1*c7p4*c9p1*ss[46] + 420*c1p1*c3p2*c5p1*c7p3*ss[38] + 420*c1p1*c3p2*c5p1*c9p3*ss[44] + 630*c1p1*c3p2*c5p2*c7p2*ss[36] + 630*c1p1*c3p2*c5p2*c9p2*ss[40] + 420*c1p1*c3p2*c5p3*c7p1*ss[34] + 420*c1p1*c3p2*c5p3*c9p1*ss[36] + 420*c1p1*c3p2*c7p1*c9p3*ss[46] + 630*c1p1*c3p2*c7p2*c9p2*ss[44] + 420*c1p1*c3p2*c7p3*c9p1*ss[42] + 420*c1p1*c3p3*c5p1*c7p2*ss[34] + 420*c1p1*c3p3*c5p1*c9p2*ss[38] + 420*c1p1*c3p3*c5p2*c7p1*ss[32] + 420*c1p1*c3p3*c5p2*c9p1*ss[34] + 420*c1p1*c3p3*c7p1*c9p2*ss[40] + 420*c1p1*c3p3*c7p2*c9p1*ss[38] + 210*c1p1*c3p4*c5p1*c7p1*ss[30] + 210*c1p1*c3p4*c5p1*c9p1*ss[32] + 210*c1p1*c3p4*c7p1*c9p1*ss[34] + 210*c1p1*c5p1*c7p1*c9p4*ss[54] + 420*c1p1*c5p1*c7p2*c9p3*ss[52] + 420*c1p1*c5p1*c7p3*c9p2*ss[50] + 210*c1p1*c5p1*c7p4*c9p1*ss[48] + 420*c1p1*c5p2*c7p1*c9p3*ss[50] + 630*c1p1*c5p2*c7p2*c9p2*ss[48] + 420*c1p1*c5p2*c7p3*c9p1*ss[46] + 420*c1p1*c5p3*c7p1*c9p2*ss[46] + 420*c1p1*c5p3*c7p2*c9p1*ss[44] + 210*c1p1*c5p4*c7p1*c9p1*ss[42] + 420*c1p2*c3p1*c5p1*c7p3*ss[36] + 420*c1p2*c3p1*c5p1*c9p3*ss[42] + 630*c1p2*c3p1*c5p2*c7p2*ss[34] + 630*c1p2*c3p1*c5p2*c9p2*ss[38] + 420*c1p2*c3p1*c5p3*c7p1*ss[32] + 420*c1p2*c3p1*c5p3*c9p1*ss[34] + 420*c1p2*c3p1*c7p1*c9p3*ss[44] + 630*c1p2*c3p1*c7p2*c9p2*ss[42] + 420*c1p2*c3p1*c7p3*c9p1*ss[40] + 630*c1p2*c3p2*c5p1*c7p2*ss[32] + 630*c1p2*c3p2*c5p1*c9p2*ss[36] + 630*c1p2*c3p2*c5p2*c7p1*ss[30] + 630*c1p2*c3p2*c5p2*c9p1*ss[32] + 630*c1p2*c3p2*c7p1*c9p2*ss[38] + 630*c1p2*c3p2*c7p2*c9p1*ss[36] + 420*c1p2*c3p3*c5p1*c7p1*ss[28] + 420*c1p2*c3p3*c5p1*c9p1*ss[30] + 420*c1p2*c3p3*c7p1*c9p1*ss[32] + 420*c1p2*c5p1*c7p1*c9p3*ss[46] + 630*c1p2*c5p1*c7p2*c9p2*ss[44] + 420*c1p2*c5p1*c7p3*c9p1*ss[42] + 630*c1p2*c5p2*c7p1*c9p2*ss[42] + 630*c1p2*c5p2*c7p2*c9p1*ss[40] + 420*c1p2*c5p3*c7p1*c9p1*ss[38] + 420*c1p3*c3p1*c5p1*c7p2*ss[30] + 420*c1p3*c3p1*c5p1*c9p2*ss[34] + 420*c1p3*c3p1*c5p2*c7p1*ss[28] + 420*c1p3*c3p1*c5p2*c9p1*ss[30] + 420*c1p3*c3p1*c7p1*c9p2*ss[36] + 420*c1p3*c3p1*c7p2*c9p1*ss[34] + 420*c1p3*c3p2*c5p1*c7p1*ss[26] + 420*c1p3*c3p2*c5p1*c9p1*ss[28] + 420*c1p3*c3p2*c7p1*c9p1*ss[30] + 420*c1p3*c5p1*c7p1*c9p2*ss[38] + 420*c1p3*c5p1*c7p2*c9p1*ss[36] + 420*c1p3*c5p2*c7p1*c9p1*ss[34] + 210*c1p4*c3p1*c5p1*c7p1*ss[24] + 210*c1p4*c3p1*c5p1*c9p1*ss[26] + 210*c1p4*c3p1*c7p1*c9p1*ss[28] + 210*c1p4*c5p1*c7p1*c9p1*ss[30] + 210*c3p1*c5p1*c7p1*c9p4*ss[56] + 420*c3p1*c5p1*c7p2*c9p3*ss[54] + 420*c3p1*c5p1*c7p3*c9p2*ss[52] + 210*c3p1*c5p1*c7p4*c9p1*ss[50] + 420*c3p1*c5p2*c7p1*c9p3*ss[52] + 630*c3p1*c5p2*c7p2*c9p2*ss[50] + 420*c3p1*c5p2*c7p3*c9p1*ss[48] + 420*c3p1*c5p3*c7p1*c9p2*ss[48] + 420*c3p1*c5p3*c7p2*c9p1*ss[46] + 210*c3p1*c5p4*c7p1*c9p1*ss[44] + 420*c3p2*c5p1*c7p1*c9p3*ss[50] + 630*c3p2*c5p1*c7p2*c9p2*ss[48] + 420*c3p2*c5p1*c7p3*c9p1*ss[46] + 630*c3p2*c5p2*c7p1*c9p2*ss[46] + 630*c3p2*c5p2*c7p2*c9p1*ss[44] + 420*c3p2*c5p3*c7p1*c9p1*ss[42] + 420*c3p3*c5p1*c7p1*c9p2*ss[44] + 420*c3p3*c5p1*c7p2*c9p1*ss[42] + 420*c3p3*c5p2*c7p1*c9p1*ss[40] + 210*c3p4*c5p1*c7p1*c9p1*ss[38] + 42*c1p1*c3p1*c5p5*ss[34] + 42*c1p1*c3p1*c7p5*ss[44] + 42*c1p1*c3p1*c9p5*ss[54] + 105*c1p1*c3p2*c5p4*ss[32] + 105*c1p1*c3p2*c7p4*ss[40] + 105*c1p1*c3p2*c9p4*ss[48] + 140*c1p1*c3p3*c5p3*ss[30] + 140*c1p1*c3p3*c7p3*ss[36] + 140*c1p1*c3p3*c9p3*ss[42] + 105*c1p1*c3p4*c5p2*ss[28] + 105*c1p1*c3p4*c7p2*ss[32] + 105*c1p1*c3p4*c9p2*ss[36] + 42*c1p1*c3p5*c5p1*ss[26] + 42*c1p1*c3p5*c7p1*ss[28] + 42*c1p1*c3p5*c9p1*ss[30] + 42*c1p1*c5p1*c7p5*ss[46] + 42*c1p1*c5p1*c9p5*ss[56] + 105*c1p1*c5p2*c7p4*ss[44] + 105*c1p1*c5p2*c9p4*ss[52] + 140*c1p1*c5p3*c7p3*ss[42] + 140*c1p1*c5p3*c9p3*ss[48] + 105*c1p1*c5p4*c7p2*ss[40] + 105*c1p1*c5p4*c9p2*ss[44] + 42*c1p1*c5p5*c7p1*ss[38] + 42*c1p1*c5p5*c9p1*ss[40] + 42*c1p1*c7p1*c9p5*ss[58] + 105*c1p1*c7p2*c9p4*ss[56] + 140*c1p1*c7p3*c9p3*ss[54] + 105*c1p1*c7p4*c9p2*ss[52] + 42*c1p1*c7p5*c9p1*ss[50] + 105*c1p2*c3p1*c5p4*ss[30] + 105*c1p2*c3p1*c7p4*ss[38] + 105*c1p2*c3p1*c9p4*ss[46] + 210*c1p2*c3p2*c5p3*ss[28] + 210*c1p2*c3p2*c7p3*ss[34] + 210*c1p2*c3p2*c9p3*ss[40] + 210*c1p2*c3p3*c5p2*ss[26] + 210*c1p2*c3p3*c7p2*ss[30] + 210*c1p2*c3p3*c9p2*ss[34] + 105*c1p2*c3p4*c5p1*ss[24] + 105*c1p2*c3p4*c7p1*ss[26] + 105*c1p2*c3p4*c9p1*ss[28] + 105*c1p2*c5p1*c7p4*ss[40] + 105*c1p2*c5p1*c9p4*ss[48] + 210*c1p2*c5p2*c7p3*ss[38] + 210*c1p2*c5p2*c9p3*ss[44] + 210*c1p2*c5p3*c7p2*ss[36] + 210*c1p2*c5p3*c9p2*ss[40] + 105*c1p2*c5p4*c7p1*ss[34] + 105*c1p2*c5p4*c9p1*ss[36] + 105*c1p2*c7p1*c9p4*ss[50] + 210*c1p2*c7p2*c9p3*ss[48] + 210*c1p2*c7p3*c9p2*ss[46] + 105*c1p2*c7p4*c9p1*ss[44] + 140*c1p3*c3p1*c5p3*ss[26] + 140*c1p3*c3p1*c7p3*ss[32] + 140*c1p3*c3p1*c9p3*ss[38] + 210*c1p3*c3p2*c5p2*ss[24] + 210*c1p3*c3p2*c7p2*ss[28] + 210*c1p3*c3p2*c9p2*ss[32] + 140*c1p3*c3p3*c5p1*ss[22] + 140*c1p3*c3p3*c7p1*ss[24] + 140*c1p3*c3p3*c9p1*ss[26] + 140*c1p3*c5p1*c7p3*ss[34] + 140*c1p3*c5p1*c9p3*ss[40] + 210*c1p3*c5p2*c7p2*ss[32] + 210*c1p3*c5p2*c9p2*ss[36] + 140*c1p3*c5p3*c7p1*ss[30] + 140*c1p3*c5p3*c9p1*ss[32] + 140*c1p3*c7p1*c9p3*ss[42] + 210*c1p3*c7p2*c9p2*ss[40] + 140*c1p3*c7p3*c9p1*ss[38] + 105*c1p4*c3p1*c5p2*ss[22] + 105*c1p4*c3p1*c7p2*ss[26] + 105*c1p4*c3p1*c9p2*ss[30] + 105*c1p4*c3p2*c5p1*ss[20] + 105*c1p4*c3p2*c7p1*ss[22] + 105*c1p4*c3p2*c9p1*ss[24] + 105*c1p4*c5p1*c7p2*ss[28] + 105*c1p4*c5p1*c9p2*ss[32] + 105*c1p4*c5p2*c7p1*ss[26] + 105*c1p4*c5p2*c9p1*ss[28] + 105*c1p4*c7p1*c9p2*ss[34] + 105*c1p4*c7p2*c9p1*ss[32] + 42*c1p5*c3p1*c5p1*ss[18] + 42*c1p5*c3p1*c7p1*ss[20] + 42*c1p5*c3p1*c9p1*ss[22] + 42*c1p5*c5p1*c7p1*ss[22] + 42*c1p5*c5p1*c9p1*ss[24] + 42*c1p5*c7p1*c9p1*ss[26] + 42*c3p1*c5p1*c7p5*ss[48] + 42*c3p1*c5p1*c9p5*ss[58] + 105*c3p1*c5p2*c7p4*ss[46] + 105*c3p1*c5p2*c9p4*ss[54] + 140*c3p1*c5p3*c7p3*ss[44] + 140*c3p1*c5p3*c9p3*ss[50] + 105*c3p1*c5p4*c7p2*ss[42] + 105*c3p1*c5p4*c9p2*ss[46] + 42*c3p1*c5p5*c7p1*ss[40] + 42*c3p1*c5p5*c9p1*ss[42] + 42*c3p1*c7p1*c9p5*ss[60] + 105*c3p1*c7p2*c9p4*ss[58] + 140*c3p1*c7p3*c9p3*ss[56] + 105*c3p1*c7p4*c9p2*ss[54] + 42*c3p1*c7p5*c9p1*ss[52] + 105*c3p2*c5p1*c7p4*ss[44] + 105*c3p2*c5p1*c9p4*ss[52] + 210*c3p2*c5p2*c7p3*ss[42] + 210*c3p2*c5p2*c9p3*ss[48] + 210*c3p2*c5p3*c7p2*ss[40] + 210*c3p2*c5p3*c9p2*ss[44] + 105*c3p2*c5p4*c7p1*ss[38] + 105*c3p2*c5p4*c9p1*ss[40] + 105*c3p2*c7p1*c9p4*ss[54] + 210*c3p2*c7p2*c9p3*ss[52] + 210*c3p2*c7p3*c9p2*ss[50] + 105*c3p2*c7p4*c9p1*ss[48] + 140*c3p3*c5p1*c7p3*ss[40] + 140*c3p3*c5p1*c9p3*ss[46] + 210*c3p3*c5p2*c7p2*ss[38] + 210*c3p3*c5p2*c9p2*ss[42] + 140*c3p3*c5p3*c7p1*ss[36] + 140*c3p3*c5p3*c9p1*ss[38] + 140*c3p3*c7p1*c9p3*ss[48] + 210*c3p3*c7p2*c9p2*ss[46] + 140*c3p3*c7p3*c9p1*ss[44] + 105*c3p4*c5p1*c7p2*ss[36] + 105*c3p4*c5p1*c9p2*ss[40] + 105*c3p4*c5p2*c7p1*ss[34] + 105*c3p4*c5p2*c9p1*ss[36] + 105*c3p4*c7p1*c9p2*ss[42] + 105*c3p4*c7p2*c9p1*ss[40] + 42*c3p5*c5p1*c7p1*ss[32] + 42*c3p5*c5p1*c9p1*ss[34] + 42*c3p5*c7p1*c9p1*ss[36] + 42*c5p1*c7p1*c9p5*ss[62] + 105*c5p1*c7p2*c9p4*ss[60] + 140*c5p1*c7p3*c9p3*ss[58] + 105*c5p1*c7p4*c9p2*ss[56] + 42*c5p1*c7p5*c9p1*ss[54] + 105*c5p2*c7p1*c9p4*ss[58] + 210*c5p2*c7p2*c9p3*ss[56] + 210*c5p2*c7p3*c9p2*ss[54] + 105*c5p2*c7p4*c9p1*ss[52] + 140*c5p3*c7p1*c9p3*ss[54] + 210*c5p3*c7p2*c9p2*ss[52] + 140*c5p3*c7p3*c9p1*ss[50] + 105*c5p4*c7p1*c9p2*ss[50] + 105*c5p4*c7p2*c9p1*ss[48] + 42*c5p5*c7p1*c9p1*ss[46] + 7*c1p1*c3p6*ss[24] + 7*c1p1*c5p6*ss[36] + 7*c1p1*c7p6*ss[48] + 7*c1p1*c9p6*ss[60] + 21*c1p2*c3p5*ss[22] + 21*c1p2*c5p5*ss[32] + 21*c1p2*c7p5*ss[42] + 21*c1p2*c9p5*ss[52] + 35*c1p3*c3p4*ss[20] + 35*c1p3*c5p4*ss[28] + 35*c1p3*c7p4*ss[36] + 35*c1p3*c9p4*ss[44] + 35*c1p4*c3p3*ss[18] + 35*c1p4*c5p3*ss[24] + 35*c1p4*c7p3*ss[30] + 35*c1p4*c9p3*ss[36] + 21*c1p5*c3p2*ss[16] + 21*c1p5*c5p2*ss[20] + 21*c1p5*c7p2*ss[24] + 21*c1p5*c9p2*ss[28] + 7*c1p6*c3p1*ss[14] + 7*c1p6*c5p1*ss[16] + 7*c1p6*c7p1*ss[18] + 7*c1p6*c9p1*ss[20] + 7*c3p1*c5p6*ss[38] + 7*c3p1*c7p6*ss[50] + 7*c3p1*c9p6*ss[62] + 21*c3p2*c5p5*ss[36] + 21*c3p2*c7p5*ss[46] + 21*c3p2*c9p5*ss[56] + 35*c3p3*c5p4*ss[34] + 35*c3p3*c7p4*ss[42] + 35*c3p3*c9p4*ss[50] + 35*c3p4*c5p3*ss[32] + 35*c3p4*c7p3*ss[38] + 35*c3p4*c9p3*ss[44] + 21*c3p5*c5p2*ss[30] + 21*c3p5*c7p2*ss[34] + 21*c3p5*c9p2*ss[38] + 7*c3p6*c5p1*ss[28] + 7*c3p6*c7p1*ss[30] + 7*c3p6*c9p1*ss[32] + 7*c5p1*c7p6*ss[52] + 7*c5p1*c9p6*ss[64] + 21*c5p2*c7p5*ss[50] + 21*c5p2*c9p5*ss[60] + 35*c5p3*c7p4*ss[48] + 35*c5p3*c9p4*ss[56] + 35*c5p4*c7p3*ss[46] + 35*c5p4*c9p3*ss[52] + 21*c5p5*c7p2*ss[44] + 21*c5p5*c9p2*ss[48] + 7*c5p6*c7p1*ss[42] + 7*c5p6*c9p1*ss[44] + 7*c7p1*c9p6*ss[66] + 21*c7p2*c9p5*ss[64] + 35*c7p3*c9p4*ss[62] + 35*c7p4*c9p3*ss[60] + 21*c7p5*c9p2*ss[58] + 7*c7p6*c9p1*ss[56] + c1p7*ss[12] + c3p7*ss[26] + c5p7*ss[40] + c7p7*ss[54] + c9p7*ss[68])*sys_part_3;
  grad_2_num += 10*(3024*c1p1*c3p1*c5p1*c7p1*c9p5*ss[66] + 7560*c1p1*c3p1*c5p1*c7p2*c9p4*ss[64] + 10080*c1p1*c3p1*c5p1*c7p3*c9p3*ss[62] + 7560*c1p1*c3p1*c5p1*c7p4*c9p2*ss[60] + 3024*c1p1*c3p1*c5p1*c7p5*c9p1*ss[58] + 7560*c1p1*c3p1*c5p2*c7p1*c9p4*ss[62] + 15120*c1p1*c3p1*c5p2*c7p2*c9p3*ss[60] + 15120*c1p1*c3p1*c5p2*c7p3*c9p2*ss[58] + 7560*c1p1*c3p1*c5p2*c7p4*c9p1*ss[56] + 10080*c1p1*c3p1*c5p3*c7p1*c9p3*ss[58] + 15120*c1p1*c3p1*c5p3*c7p2*c9p2*ss[56] + 10080*c1p1*c3p1*c5p3*c7p3*c9p1*ss[54] + 7560*c1p1*c3p1*c5p4*c7p1*c9p2*ss[54] + 7560*c1p1*c3p1*c5p4*c7p2*c9p1*ss[52] + 3024*c1p1*c3p1*c5p5*c7p1*c9p1*ss[50] + 7560*c1p1*c3p2*c5p1*c7p1*c9p4*ss[60] + 15120*c1p1*c3p2*c5p1*c7p2*c9p3*ss[58] + 15120*c1p1*c3p2*c5p1*c7p3*c9p2*ss[56] + 7560*c1p1*c3p2*c5p1*c7p4*c9p1*ss[54] + 15120*c1p1*c3p2*c5p2*c7p1*c9p3*ss[56] + 22680*c1p1*c3p2*c5p2*c7p2*c9p2*ss[54] + 15120*c1p1*c3p2*c5p2*c7p3*c9p1*ss[52] + 15120*c1p1*c3p2*c5p3*c7p1*c9p2*ss[52] + 15120*c1p1*c3p2*c5p3*c7p2*c9p1*ss[50] + 7560*c1p1*c3p2*c5p4*c7p1*c9p1*ss[48] + 10080*c1p1*c3p3*c5p1*c7p1*c9p3*ss[54] + 15120*c1p1*c3p3*c5p1*c7p2*c9p2*ss[52] + 10080*c1p1*c3p3*c5p1*c7p3*c9p1*ss[50] + 15120*c1p1*c3p3*c5p2*c7p1*c9p2*ss[50] + 15120*c1p1*c3p3*c5p2*c7p2*c9p1*ss[48] + 10080*c1p1*c3p3*c5p3*c7p1*c9p1*ss[46] + 7560*c1p1*c3p4*c5p1*c7p1*c9p2*ss[48] + 7560*c1p1*c3p4*c5p1*c7p2*c9p1*ss[46] + 7560*c1p1*c3p4*c5p2*c7p1*c9p1*ss[44] + 3024*c1p1*c3p5*c5p1*c7p1*c9p1*ss[42] + 7560*c1p2*c3p1*c5p1*c7p1*c9p4*ss[58] + 15120*c1p2*c3p1*c5p1*c7p2*c9p3*ss[56] + 15120*c1p2*c3p1*c5p1*c7p3*c9p2*ss[54] + 7560*c1p2*c3p1*c5p1*c7p4*c9p1*ss[52] + 15120*c1p2*c3p1*c5p2*c7p1*c9p3*ss[54] + 22680*c1p2*c3p1*c5p2*c7p2*c9p2*ss[52] + 15120*c1p2*c3p1*c5p2*c7p3*c9p1*ss[50] + 15120*c1p2*c3p1*c5p3*c7p1*c9p2*ss[50] + 15120*c1p2*c3p1*c5p3*c7p2*c9p1*ss[48] + 7560*c1p2*c3p1*c5p4*c7p1*c9p1*ss[46] + 15120*c1p2*c3p2*c5p1*c7p1*c9p3*ss[52] + 22680*c1p2*c3p2*c5p1*c7p2*c9p2*ss[50] + 15120*c1p2*c3p2*c5p1*c7p3*c9p1*ss[48] + 22680*c1p2*c3p2*c5p2*c7p1*c9p2*ss[48] + 22680*c1p2*c3p2*c5p2*c7p2*c9p1*ss[46] + 15120*c1p2*c3p2*c5p3*c7p1*c9p1*ss[44] + 15120*c1p2*c3p3*c5p1*c7p1*c9p2*ss[46] + 15120*c1p2*c3p3*c5p1*c7p2*c9p1*ss[44] + 15120*c1p2*c3p3*c5p2*c7p1*c9p1*ss[42] + 7560*c1p2*c3p4*c5p1*c7p1*c9p1*ss[40] + 10080*c1p3*c3p1*c5p1*c7p1*c9p3*ss[50] + 15120*c1p3*c3p1*c5p1*c7p2*c9p2*ss[48] + 10080*c1p3*c3p1*c5p1*c7p3*c9p1*ss[46] + 15120*c1p3*c3p1*c5p2*c7p1*c9p2*ss[46] + 15120*c1p3*c3p1*c5p2*c7p2*c9p1*ss[44] + 10080*c1p3*c3p1*c5p3*c7p1*c9p1*ss[42] + 15120*c1p3*c3p2*c5p1*c7p1*c9p2*ss[44] + 15120*c1p3*c3p2*c5p1*c7p2*c9p1*ss[42] + 15120*c1p3*c3p2*c5p2*c7p1*c9p1*ss[40] + 10080*c1p3*c3p3*c5p1*c7p1*c9p1*ss[38] + 7560*c1p4*c3p1*c5p1*c7p1*c9p2*ss[42] + 7560*c1p4*c3p1*c5p1*c7p2*c9p1*ss[40] + 7560*c1p4*c3p1*c5p2*c7p1*c9p1*ss[38] + 7560*c1p4*c3p2*c5p1*c7p1*c9p1*ss[36] + 3024*c1p5*c3p1*c5p1*c7p1*c9p1*ss[34] + 504*c1p1*c3p1*c5p1*c7p6*ss[56] + 504*c1p1*c3p1*c5p1*c9p6*ss[68] + 1512*c1p1*c3p1*c5p2*c7p5*ss[54] + 1512*c1p1*c3p1*c5p2*c9p5*ss[64] + 2520*c1p1*c3p1*c5p3*c7p4*ss[52] + 2520*c1p1*c3p1*c5p3*c9p4*ss[60] + 2520*c1p1*c3p1*c5p4*c7p3*ss[50] + 2520*c1p1*c3p1*c5p4*c9p3*ss[56] + 1512*c1p1*c3p1*c5p5*c7p2*ss[48] + 1512*c1p1*c3p1*c5p5*c9p2*ss[52] + 504*c1p1*c3p1*c5p6*c7p1*ss[46] + 504*c1p1*c3p1*c5p6*c9p1*ss[48] + 504*c1p1*c3p1*c7p1*c9p6*ss[70] + 1512*c1p1*c3p1*c7p2*c9p5*ss[68] + 2520*c1p1*c3p1*c7p3*c9p4*ss[66] + 2520*c1p1*c3p1*c7p4*c9p3*ss[64] + 1512*c1p1*c3p1*c7p5*c9p2*ss[62] + 504*c1p1*c3p1*c7p6*c9p1*ss[60] + 1512*c1p1*c3p2*c5p1*c7p5*ss[52] + 1512*c1p1*c3p2*c5p1*c9p5*ss[62] + 3780*c1p1*c3p2*c5p2*c7p4*ss[50] + 3780*c1p1*c3p2*c5p2*c9p4*ss[58] + 5040*c1p1*c3p2*c5p3*c7p3*ss[48] + 5040*c1p1*c3p2*c5p3*c9p3*ss[54] + 3780*c1p1*c3p2*c5p4*c7p2*ss[46] + 3780*c1p1*c3p2*c5p4*c9p2*ss[50] + 1512*c1p1*c3p2*c5p5*c7p1*ss[44] + 1512*c1p1*c3p2*c5p5*c9p1*ss[46] + 1512*c1p1*c3p2*c7p1*c9p5*ss[64] + 3780*c1p1*c3p2*c7p2*c9p4*ss[62] + 5040*c1p1*c3p2*c7p3*c9p3*ss[60] + 3780*c1p1*c3p2*c7p4*c9p2*ss[58] + 1512*c1p1*c3p2*c7p5*c9p1*ss[56] + 2520*c1p1*c3p3*c5p1*c7p4*ss[48] + 2520*c1p1*c3p3*c5p1*c9p4*ss[56] + 5040*c1p1*c3p3*c5p2*c7p3*ss[46] + 5040*c1p1*c3p3*c5p2*c9p3*ss[52] + 5040*c1p1*c3p3*c5p3*c7p2*ss[44] + 5040*c1p1*c3p3*c5p3*c9p2*ss[48] + 2520*c1p1*c3p3*c5p4*c7p1*ss[42] + 2520*c1p1*c3p3*c5p4*c9p1*ss[44] + 2520*c1p1*c3p3*c7p1*c9p4*ss[58] + 5040*c1p1*c3p3*c7p2*c9p3*ss[56] + 5040*c1p1*c3p3*c7p3*c9p2*ss[54] + 2520*c1p1*c3p3*c7p4*c9p1*ss[52] + 2520*c1p1*c3p4*c5p1*c7p3*ss[44] + 2520*c1p1*c3p4*c5p1*c9p3*ss[50] + 3780*c1p1*c3p4*c5p2*c7p2*ss[42] + 3780*c1p1*c3p4*c5p2*c9p2*ss[46] + 2520*c1p1*c3p4*c5p3*c7p1*ss[40] + 2520*c1p1*c3p4*c5p3*c9p1*ss[42] + 2520*c1p1*c3p4*c7p1*c9p3*ss[52] + 3780*c1p1*c3p4*c7p2*c9p2*ss[50] + 2520*c1p1*c3p4*c7p3*c9p1*ss[48] + 1512*c1p1*c3p5*c5p1*c7p2*ss[40] + 1512*c1p1*c3p5*c5p1*c9p2*ss[44] + 1512*c1p1*c3p5*c5p2*c7p1*ss[38] + 1512*c1p1*c3p5*c5p2*c9p1*ss[40] + 1512*c1p1*c3p5*c7p1*c9p2*ss[46] + 1512*c1p1*c3p5*c7p2*c9p1*ss[44] + 504*c1p1*c3p6*c5p1*c7p1*ss[36] + 504*c1p1*c3p6*c5p1*c9p1*ss[38] + 504*c1p1*c3p6*c7p1*c9p1*ss[40] + 504*c1p1*c5p1*c7p1*c9p6*ss[72] + 1512*c1p1*c5p1*c7p2*c9p5*ss[70] + 2520*c1p1*c5p1*c7p3*c9p4*ss[68] + 2520*c1p1*c5p1*c7p4*c9p3*ss[66] + 1512*c1p1*c5p1*c7p5*c9p2*ss[64] + 504*c1p1*c5p1*c7p6*c9p1*ss[62] + 1512*c1p1*c5p2*c7p1*c9p5*ss[68] + 3780*c1p1*c5p2*c7p2*c9p4*ss[66] + 5040*c1p1*c5p2*c7p3*c9p3*ss[64] + 3780*c1p1*c5p2*c7p4*c9p2*ss[62] + 1512*c1p1*c5p2*c7p5*c9p1*ss[60] + 2520*c1p1*c5p3*c7p1*c9p4*ss[64] + 5040*c1p1*c5p3*c7p2*c9p3*ss[62] + 5040*c1p1*c5p3*c7p3*c9p2*ss[60] + 2520*c1p1*c5p3*c7p4*c9p1*ss[58] + 2520*c1p1*c5p4*c7p1*c9p3*ss[60] + 3780*c1p1*c5p4*c7p2*c9p2*ss[58] + 2520*c1p1*c5p4*c7p3*c9p1*ss[56] + 1512*c1p1*c5p5*c7p1*c9p2*ss[56] + 1512*c1p1*c5p5*c7p2*c9p1*ss[54] + 504*c1p1*c5p6*c7p1*c9p1*ss[52] + 1512*c1p2*c3p1*c5p1*c7p5*ss[50] + 1512*c1p2*c3p1*c5p1*c9p5*ss[60] + 3780*c1p2*c3p1*c5p2*c7p4*ss[48] + 3780*c1p2*c3p1*c5p2*c9p4*ss[56] + 5040*c1p2*c3p1*c5p3*c7p3*ss[46] + 5040*c1p2*c3p1*c5p3*c9p3*ss[52] + 3780*c1p2*c3p1*c5p4*c7p2*ss[44] + 3780*c1p2*c3p1*c5p4*c9p2*ss[48] + 1512*c1p2*c3p1*c5p5*c7p1*ss[42] + 1512*c1p2*c3p1*c5p5*c9p1*ss[44] + 1512*c1p2*c3p1*c7p1*c9p5*ss[62] + 3780*c1p2*c3p1*c7p2*c9p4*ss[60] + 5040*c1p2*c3p1*c7p3*c9p3*ss[58] + 3780*c1p2*c3p1*c7p4*c9p2*ss[56] + 1512*c1p2*c3p1*c7p5*c9p1*ss[54] + 3780*c1p2*c3p2*c5p1*c7p4*ss[46] + 3780*c1p2*c3p2*c5p1*c9p4*ss[54] + 7560*c1p2*c3p2*c5p2*c7p3*ss[44] + 7560*c1p2*c3p2*c5p2*c9p3*ss[50] + 7560*c1p2*c3p2*c5p3*c7p2*ss[42] + 7560*c1p2*c3p2*c5p3*c9p2*ss[46] + 3780*c1p2*c3p2*c5p4*c7p1*ss[40] + 3780*c1p2*c3p2*c5p4*c9p1*ss[42] + 3780*c1p2*c3p2*c7p1*c9p4*ss[56] + 7560*c1p2*c3p2*c7p2*c9p3*ss[54] + 7560*c1p2*c3p2*c7p3*c9p2*ss[52] + 3780*c1p2*c3p2*c7p4*c9p1*ss[50] + 5040*c1p2*c3p3*c5p1*c7p3*ss[42] + 5040*c1p2*c3p3*c5p1*c9p3*ss[48] + 7560*c1p2*c3p3*c5p2*c7p2*ss[40] + 7560*c1p2*c3p3*c5p2*c9p2*ss[44] + 5040*c1p2*c3p3*c5p3*c7p1*ss[38] + 5040*c1p2*c3p3*c5p3*c9p1*ss[40] + 5040*c1p2*c3p3*c7p1*c9p3*ss[50] + 7560*c1p2*c3p3*c7p2*c9p2*ss[48] + 5040*c1p2*c3p3*c7p3*c9p1*ss[46] + 3780*c1p2*c3p4*c5p1*c7p2*ss[38] + 3780*c1p2*c3p4*c5p1*c9p2*ss[42] + 3780*c1p2*c3p4*c5p2*c7p1*ss[36] + 3780*c1p2*c3p4*c5p2*c9p1*ss[38] + 3780*c1p2*c3p4*c7p1*c9p2*ss[44] + 3780*c1p2*c3p4*c7p2*c9p1*ss[42] + 1512*c1p2*c3p5*c5p1*c7p1*ss[34] + 1512*c1p2*c3p5*c5p1*c9p1*ss[36] + 1512*c1p2*c3p5*c7p1*c9p1*ss[38] + 1512*c1p2*c5p1*c7p1*c9p5*ss[64] + 3780*c1p2*c5p1*c7p2*c9p4*ss[62] + 5040*c1p2*c5p1*c7p3*c9p3*ss[60] + 3780*c1p2*c5p1*c7p4*c9p2*ss[58] + 1512*c1p2*c5p1*c7p5*c9p1*ss[56] + 3780*c1p2*c5p2*c7p1*c9p4*ss[60] + 7560*c1p2*c5p2*c7p2*c9p3*ss[58] + 7560*c1p2*c5p2*c7p3*c9p2*ss[56] + 3780*c1p2*c5p2*c7p4*c9p1*ss[54] + 5040*c1p2*c5p3*c7p1*c9p3*ss[56] + 7560*c1p2*c5p3*c7p2*c9p2*ss[54] + 5040*c1p2*c5p3*c7p3*c9p1*ss[52] + 3780*c1p2*c5p4*c7p1*c9p2*ss[52] + 3780*c1p2*c5p4*c7p2*c9p1*ss[50] + 1512*c1p2*c5p5*c7p1*c9p1*ss[48] + 2520*c1p3*c3p1*c5p1*c7p4*ss[44] + 2520*c1p3*c3p1*c5p1*c9p4*ss[52] + 5040*c1p3*c3p1*c5p2*c7p3*ss[42] + 5040*c1p3*c3p1*c5p2*c9p3*ss[48] + 5040*c1p3*c3p1*c5p3*c7p2*ss[40] + 5040*c1p3*c3p1*c5p3*c9p2*ss[44] + 2520*c1p3*c3p1*c5p4*c7p1*ss[38] + 2520*c1p3*c3p1*c5p4*c9p1*ss[40] + 2520*c1p3*c3p1*c7p1*c9p4*ss[54] + 5040*c1p3*c3p1*c7p2*c9p3*ss[52] + 5040*c1p3*c3p1*c7p3*c9p2*ss[50] + 2520*c1p3*c3p1*c7p4*c9p1*ss[48] + 5040*c1p3*c3p2*c5p1*c7p3*ss[40] + 5040*c1p3*c3p2*c5p1*c9p3*ss[46] + 7560*c1p3*c3p2*c5p2*c7p2*ss[38] + 7560*c1p3*c3p2*c5p2*c9p2*ss[42] + 5040*c1p3*c3p2*c5p3*c7p1*ss[36] + 5040*c1p3*c3p2*c5p3*c9p1*ss[38] + 5040*c1p3*c3p2*c7p1*c9p3*ss[48] + 7560*c1p3*c3p2*c7p2*c9p2*ss[46] + 5040*c1p3*c3p2*c7p3*c9p1*ss[44] + 5040*c1p3*c3p3*c5p1*c7p2*ss[36] + 5040*c1p3*c3p3*c5p1*c9p2*ss[40] + 5040*c1p3*c3p3*c5p2*c7p1*ss[34] + 5040*c1p3*c3p3*c5p2*c9p1*ss[36] + 5040*c1p3*c3p3*c7p1*c9p2*ss[42] + 5040*c1p3*c3p3*c7p2*c9p1*ss[40] + 2520*c1p3*c3p4*c5p1*c7p1*ss[32] + 2520*c1p3*c3p4*c5p1*c9p1*ss[34] + 2520*c1p3*c3p4*c7p1*c9p1*ss[36] + 2520*c1p3*c5p1*c7p1*c9p4*ss[56] + 5040*c1p3*c5p1*c7p2*c9p3*ss[54] + 5040*c1p3*c5p1*c7p3*c9p2*ss[52] + 2520*c1p3*c5p1*c7p4*c9p1*ss[50] + 5040*c1p3*c5p2*c7p1*c9p3*ss[52] + 7560*c1p3*c5p2*c7p2*c9p2*ss[50] + 5040*c1p3*c5p2*c7p3*c9p1*ss[48] + 5040*c1p3*c5p3*c7p1*c9p2*ss[48] + 5040*c1p3*c5p3*c7p2*c9p1*ss[46] + 2520*c1p3*c5p4*c7p1*c9p1*ss[44] + 2520*c1p4*c3p1*c5p1*c7p3*ss[38] + 2520*c1p4*c3p1*c5p1*c9p3*ss[44] + 3780*c1p4*c3p1*c5p2*c7p2*ss[36] + 3780*c1p4*c3p1*c5p2*c9p2*ss[40] + 2520*c1p4*c3p1*c5p3*c7p1*ss[34] + 2520*c1p4*c3p1*c5p3*c9p1*ss[36] + 2520*c1p4*c3p1*c7p1*c9p3*ss[46] + 3780*c1p4*c3p1*c7p2*c9p2*ss[44] + 2520*c1p4*c3p1*c7p3*c9p1*ss[42] + 3780*c1p4*c3p2*c5p1*c7p2*ss[34] + 3780*c1p4*c3p2*c5p1*c9p2*ss[38] + 3780*c1p4*c3p2*c5p2*c7p1*ss[32] + 3780*c1p4*c3p2*c5p2*c9p1*ss[34] + 3780*c1p4*c3p2*c7p1*c9p2*ss[40] + 3780*c1p4*c3p2*c7p2*c9p1*ss[38] + 2520*c1p4*c3p3*c5p1*c7p1*ss[30] + 2520*c1p4*c3p3*c5p1*c9p1*ss[32] + 2520*c1p4*c3p3*c7p1*c9p1*ss[34] + 2520*c1p4*c5p1*c7p1*c9p3*ss[48] + 3780*c1p4*c5p1*c7p2*c9p2*ss[46] + 2520*c1p4*c5p1*c7p3*c9p1*ss[44] + 3780*c1p4*c5p2*c7p1*c9p2*ss[44] + 3780*c1p4*c5p2*c7p2*c9p1*ss[42] + 2520*c1p4*c5p3*c7p1*c9p1*ss[40] + 1512*c1p5*c3p1*c5p1*c7p2*ss[32] + 1512*c1p5*c3p1*c5p1*c9p2*ss[36] + 1512*c1p5*c3p1*c5p2*c7p1*ss[30] + 1512*c1p5*c3p1*c5p2*c9p1*ss[32] + 1512*c1p5*c3p1*c7p1*c9p2*ss[38] + 1512*c1p5*c3p1*c7p2*c9p1*ss[36] + 1512*c1p5*c3p2*c5p1*c7p1*ss[28] + 1512*c1p5*c3p2*c5p1*c9p1*ss[30] + 1512*c1p5*c3p2*c7p1*c9p1*ss[32] + 1512*c1p5*c5p1*c7p1*c9p2*ss[40] + 1512*c1p5*c5p1*c7p2*c9p1*ss[38] + 1512*c1p5*c5p2*c7p1*c9p1*ss[36] + 504*c1p6*c3p1*c5p1*c7p1*ss[26] + 504*c1p6*c3p1*c5p1*c9p1*ss[28] + 504*c1p6*c3p1*c7p1*c9p1*ss[30] + 504*c1p6*c5p1*c7p1*c9p1*ss[32] + 504*c3p1*c5p1*c7p1*c9p6*ss[74] + 1512*c3p1*c5p1*c7p2*c9p5*ss[72] + 2520*c3p1*c5p1*c7p3*c9p4*ss[70] + 2520*c3p1*c5p1*c7p4*c9p3*ss[68] + 1512*c3p1*c5p1*c7p5*c9p2*ss[66] + 504*c3p1*c5p1*c7p6*c9p1*ss[64] + 1512*c3p1*c5p2*c7p1*c9p5*ss[70] + 3780*c3p1*c5p2*c7p2*c9p4*ss[68] + 5040*c3p1*c5p2*c7p3*c9p3*ss[66] + 3780*c3p1*c5p2*c7p4*c9p2*ss[64] + 1512*c3p1*c5p2*c7p5*c9p1*ss[62] + 2520*c3p1*c5p3*c7p1*c9p4*ss[66] + 5040*c3p1*c5p3*c7p2*c9p3*ss[64] + 5040*c3p1*c5p3*c7p3*c9p2*ss[62] + 2520*c3p1*c5p3*c7p4*c9p1*ss[60] + 2520*c3p1*c5p4*c7p1*c9p3*ss[62] + 3780*c3p1*c5p4*c7p2*c9p2*ss[60] + 2520*c3p1*c5p4*c7p3*c9p1*ss[58] + 1512*c3p1*c5p5*c7p1*c9p2*ss[58] + 1512*c3p1*c5p5*c7p2*c9p1*ss[56] + 504*c3p1*c5p6*c7p1*c9p1*ss[54] + 1512*c3p2*c5p1*c7p1*c9p5*ss[68] + 3780*c3p2*c5p1*c7p2*c9p4*ss[66] + 5040*c3p2*c5p1*c7p3*c9p3*ss[64] + 3780*c3p2*c5p1*c7p4*c9p2*ss[62] + 1512*c3p2*c5p1*c7p5*c9p1*ss[60] + 3780*c3p2*c5p2*c7p1*c9p4*ss[64] + 7560*c3p2*c5p2*c7p2*c9p3*ss[62] + 7560*c3p2*c5p2*c7p3*c9p2*ss[60] + 3780*c3p2*c5p2*c7p4*c9p1*ss[58] + 5040*c3p2*c5p3*c7p1*c9p3*ss[60] + 7560*c3p2*c5p3*c7p2*c9p2*ss[58] + 5040*c3p2*c5p3*c7p3*c9p1*ss[56] + 3780*c3p2*c5p4*c7p1*c9p2*ss[56] + 3780*c3p2*c5p4*c7p2*c9p1*ss[54] + 1512*c3p2*c5p5*c7p1*c9p1*ss[52] + 2520*c3p3*c5p1*c7p1*c9p4*ss[62] + 5040*c3p3*c5p1*c7p2*c9p3*ss[60] + 5040*c3p3*c5p1*c7p3*c9p2*ss[58] + 2520*c3p3*c5p1*c7p4*c9p1*ss[56] + 5040*c3p3*c5p2*c7p1*c9p3*ss[58] + 7560*c3p3*c5p2*c7p2*c9p2*ss[56] + 5040*c3p3*c5p2*c7p3*c9p1*ss[54] + 5040*c3p3*c5p3*c7p1*c9p2*ss[54] + 5040*c3p3*c5p3*c7p2*c9p1*ss[52] + 2520*c3p3*c5p4*c7p1*c9p1*ss[50] + 2520*c3p4*c5p1*c7p1*c9p3*ss[56] + 3780*c3p4*c5p1*c7p2*c9p2*ss[54] + 2520*c3p4*c5p1*c7p3*c9p1*ss[52] + 3780*c3p4*c5p2*c7p1*c9p2*ss[52] + 3780*c3p4*c5p2*c7p2*c9p1*ss[50] + 2520*c3p4*c5p3*c7p1*c9p1*ss[48] + 1512*c3p5*c5p1*c7p1*c9p2*ss[50] + 1512*c3p5*c5p1*c7p2*c9p1*ss[48] + 1512*c3p5*c5p2*c7p1*c9p1*ss[46] + 504*c3p6*c5p1*c7p1*c9p1*ss[44] + 72*c1p1*c3p1*c5p7*ss[44] + 72*c1p1*c3p1*c7p7*ss[58] + 72*c1p1*c3p1*c9p7*ss[72] + 252*c1p1*c3p2*c5p6*ss[42] + 252*c1p1*c3p2*c7p6*ss[54] + 252*c1p1*c3p2*c9p6*ss[66] + 504*c1p1*c3p3*c5p5*ss[40] + 504*c1p1*c3p3*c7p5*ss[50] + 504*c1p1*c3p3*c9p5*ss[60] + 630*c1p1*c3p4*c5p4*ss[38] + 630*c1p1*c3p4*c7p4*ss[46] + 630*c1p1*c3p4*c9p4*ss[54] + 504*c1p1*c3p5*c5p3*ss[36] + 504*c1p1*c3p5*c7p3*ss[42] + 504*c1p1*c3p5*c9p3*ss[48] + 252*c1p1*c3p6*c5p2*ss[34] + 252*c1p1*c3p6*c7p2*ss[38] + 252*c1p1*c3p6*c9p2*ss[42] + 72*c1p1*c3p7*c5p1*ss[32] + 72*c1p1*c3p7*c7p1*ss[34] + 72*c1p1*c3p7*c9p1*ss[36] + 72*c1p1*c5p1*c7p7*ss[60] + 72*c1p1*c5p1*c9p7*ss[74] + 252*c1p1*c5p2*c7p6*ss[58] + 252*c1p1*c5p2*c9p6*ss[70] + 504*c1p1*c5p3*c7p5*ss[56] + 504*c1p1*c5p3*c9p5*ss[66] + 630*c1p1*c5p4*c7p4*ss[54] + 630*c1p1*c5p4*c9p4*ss[62] + 504*c1p1*c5p5*c7p3*ss[52] + 504*c1p1*c5p5*c9p3*ss[58] + 252*c1p1*c5p6*c7p2*ss[50] + 252*c1p1*c5p6*c9p2*ss[54] + 72*c1p1*c5p7*c7p1*ss[48] + 72*c1p1*c5p7*c9p1*ss[50] + 72*c1p1*c7p1*c9p7*ss[76] + 252*c1p1*c7p2*c9p6*ss[74] + 504*c1p1*c7p3*c9p5*ss[72] + 630*c1p1*c7p4*c9p4*ss[70] + 504*c1p1*c7p5*c9p3*ss[68] + 252*c1p1*c7p6*c9p2*ss[66] + 72*c1p1*c7p7*c9p1*ss[64] + 252*c1p2*c3p1*c5p6*ss[40] + 252*c1p2*c3p1*c7p6*ss[52] + 252*c1p2*c3p1*c9p6*ss[64] + 756*c1p2*c3p2*c5p5*ss[38] + 756*c1p2*c3p2*c7p5*ss[48] + 756*c1p2*c3p2*c9p5*ss[58] + 1260*c1p2*c3p3*c5p4*ss[36] + 1260*c1p2*c3p3*c7p4*ss[44] + 1260*c1p2*c3p3*c9p4*ss[52] + 1260*c1p2*c3p4*c5p3*ss[34] + 1260*c1p2*c3p4*c7p3*ss[40] + 1260*c1p2*c3p4*c9p3*ss[46] + 756*c1p2*c3p5*c5p2*ss[32] + 756*c1p2*c3p5*c7p2*ss[36] + 756*c1p2*c3p5*c9p2*ss[40] + 252*c1p2*c3p6*c5p1*ss[30] + 252*c1p2*c3p6*c7p1*ss[32] + 252*c1p2*c3p6*c9p1*ss[34] + 252*c1p2*c5p1*c7p6*ss[54] + 252*c1p2*c5p1*c9p6*ss[66] + 756*c1p2*c5p2*c7p5*ss[52] + 756*c1p2*c5p2*c9p5*ss[62] + 1260*c1p2*c5p3*c7p4*ss[50] + 1260*c1p2*c5p3*c9p4*ss[58] + 1260*c1p2*c5p4*c7p3*ss[48] + 1260*c1p2*c5p4*c9p3*ss[54] + 756*c1p2*c5p5*c7p2*ss[46] + 756*c1p2*c5p5*c9p2*ss[50] + 252*c1p2*c5p6*c7p1*ss[44] + 252*c1p2*c5p6*c9p1*ss[46] + 252*c1p2*c7p1*c9p6*ss[68] + 756*c1p2*c7p2*c9p5*ss[66] + 1260*c1p2*c7p3*c9p4*ss[64] + 1260*c1p2*c7p4*c9p3*ss[62] + 756*c1p2*c7p5*c9p2*ss[60] + 252*c1p2*c7p6*c9p1*ss[58] + 504*c1p3*c3p1*c5p5*ss[36] + 504*c1p3*c3p1*c7p5*ss[46] + 504*c1p3*c3p1*c9p5*ss[56] + 1260*c1p3*c3p2*c5p4*ss[34] + 1260*c1p3*c3p2*c7p4*ss[42] + 1260*c1p3*c3p2*c9p4*ss[50] + 1680*c1p3*c3p3*c5p3*ss[32] + 1680*c1p3*c3p3*c7p3*ss[38] + 1680*c1p3*c3p3*c9p3*ss[44] + 1260*c1p3*c3p4*c5p2*ss[30] + 1260*c1p3*c3p4*c7p2*ss[34] + 1260*c1p3*c3p4*c9p2*ss[38] + 504*c1p3*c3p5*c5p1*ss[28] + 504*c1p3*c3p5*c7p1*ss[30] + 504*c1p3*c3p5*c9p1*ss[32] + 504*c1p3*c5p1*c7p5*ss[48] + 504*c1p3*c5p1*c9p5*ss[58] + 1260*c1p3*c5p2*c7p4*ss[46] + 1260*c1p3*c5p2*c9p4*ss[54] + 1680*c1p3*c5p3*c7p3*ss[44] + 1680*c1p3*c5p3*c9p3*ss[50] + 1260*c1p3*c5p4*c7p2*ss[42] + 1260*c1p3*c5p4*c9p2*ss[46] + 504*c1p3*c5p5*c7p1*ss[40] + 504*c1p3*c5p5*c9p1*ss[42] + 504*c1p3*c7p1*c9p5*ss[60] + 1260*c1p3*c7p2*c9p4*ss[58] + 1680*c1p3*c7p3*c9p3*ss[56] + 1260*c1p3*c7p4*c9p2*ss[54] + 504*c1p3*c7p5*c9p1*ss[52] + 630*c1p4*c3p1*c5p4*ss[32] + 630*c1p4*c3p1*c7p4*ss[40] + 630*c1p4*c3p1*c9p4*ss[48] + 1260*c1p4*c3p2*c5p3*ss[30] + 1260*c1p4*c3p2*c7p3*ss[36] + 1260*c1p4*c3p2*c9p3*ss[42] + 1260*c1p4*c3p3*c5p2*ss[28] + 1260*c1p4*c3p3*c7p2*ss[32] + 1260*c1p4*c3p3*c9p2*ss[36] + 630*c1p4*c3p4*c5p1*ss[26] + 630*c1p4*c3p4*c7p1*ss[28] + 630*c1p4*c3p4*c9p1*ss[30] + 630*c1p4*c5p1*c7p4*ss[42] + 630*c1p4*c5p1*c9p4*ss[50] + 1260*c1p4*c5p2*c7p3*ss[40] + 1260*c1p4*c5p2*c9p3*ss[46] + 1260*c1p4*c5p3*c7p2*ss[38] + 1260*c1p4*c5p3*c9p2*ss[42] + 630*c1p4*c5p4*c7p1*ss[36] + 630*c1p4*c5p4*c9p1*ss[38] + 630*c1p4*c7p1*c9p4*ss[52] + 1260*c1p4*c7p2*c9p3*ss[50] + 1260*c1p4*c7p3*c9p2*ss[48] + 630*c1p4*c7p4*c9p1*ss[46] + 504*c1p5*c3p1*c5p3*ss[28] + 504*c1p5*c3p1*c7p3*ss[34] + 504*c1p5*c3p1*c9p3*ss[40] + 756*c1p5*c3p2*c5p2*ss[26] + 756*c1p5*c3p2*c7p2*ss[30] + 756*c1p5*c3p2*c9p2*ss[34] + 504*c1p5*c3p3*c5p1*ss[24] + 504*c1p5*c3p3*c7p1*ss[26] + 504*c1p5*c3p3*c9p1*ss[28] + 504*c1p5*c5p1*c7p3*ss[36] + 504*c1p5*c5p1*c9p3*ss[42] + 756*c1p5*c5p2*c7p2*ss[34] + 756*c1p5*c5p2*c9p2*ss[38] + 504*c1p5*c5p3*c7p1*ss[32] + 504*c1p5*c5p3*c9p1*ss[34] + 504*c1p5*c7p1*c9p3*ss[44] + 756*c1p5*c7p2*c9p2*ss[42] + 504*c1p5*c7p3*c9p1*ss[40] + 252*c1p6*c3p1*c5p2*ss[24] + 252*c1p6*c3p1*c7p2*ss[28] + 252*c1p6*c3p1*c9p2*ss[32] + 252*c1p6*c3p2*c5p1*ss[22] + 252*c1p6*c3p2*c7p1*ss[24] + 252*c1p6*c3p2*c9p1*ss[26] + 252*c1p6*c5p1*c7p2*ss[30] + 252*c1p6*c5p1*c9p2*ss[34] + 252*c1p6*c5p2*c7p1*ss[28] + 252*c1p6*c5p2*c9p1*ss[30] + 252*c1p6*c7p1*c9p2*ss[36] + 252*c1p6*c7p2*c9p1*ss[34] + 72*c1p7*c3p1*c5p1*ss[20] + 72*c1p7*c3p1*c7p1*ss[22] + 72*c1p7*c3p1*c9p1*ss[24] + 72*c1p7*c5p1*c7p1*ss[24] + 72*c1p7*c5p1*c9p1*ss[26] + 72*c1p7*c7p1*c9p1*ss[28] + 72*c3p1*c5p1*c7p7*ss[62] + 72*c3p1*c5p1*c9p7*ss[76] + 252*c3p1*c5p2*c7p6*ss[60] + 252*c3p1*c5p2*c9p6*ss[72] + 504*c3p1*c5p3*c7p5*ss[58] + 504*c3p1*c5p3*c9p5*ss[68] + 630*c3p1*c5p4*c7p4*ss[56] + 630*c3p1*c5p4*c9p4*ss[64] + 504*c3p1*c5p5*c7p3*ss[54] + 504*c3p1*c5p5*c9p3*ss[60] + 252*c3p1*c5p6*c7p2*ss[52] + 252*c3p1*c5p6*c9p2*ss[56] + 72*c3p1*c5p7*c7p1*ss[50] + 72*c3p1*c5p7*c9p1*ss[52] + 72*c3p1*c7p1*c9p7*ss[78] + 252*c3p1*c7p2*c9p6*ss[76] + 504*c3p1*c7p3*c9p5*ss[74] + 630*c3p1*c7p4*c9p4*ss[72] + 504*c3p1*c7p5*c9p3*ss[70] + 252*c3p1*c7p6*c9p2*ss[68] + 72*c3p1*c7p7*c9p1*ss[66] + 252*c3p2*c5p1*c7p6*ss[58] + 252*c3p2*c5p1*c9p6*ss[70] + 756*c3p2*c5p2*c7p5*ss[56] + 756*c3p2*c5p2*c9p5*ss[66] + 1260*c3p2*c5p3*c7p4*ss[54] + 1260*c3p2*c5p3*c9p4*ss[62] + 1260*c3p2*c5p4*c7p3*ss[52] + 1260*c3p2*c5p4*c9p3*ss[58] + 756*c3p2*c5p5*c7p2*ss[50] + 756*c3p2*c5p5*c9p2*ss[54] + 252*c3p2*c5p6*c7p1*ss[48] + 252*c3p2*c5p6*c9p1*ss[50] + 252*c3p2*c7p1*c9p6*ss[72] + 756*c3p2*c7p2*c9p5*ss[70] + 1260*c3p2*c7p3*c9p4*ss[68] + 1260*c3p2*c7p4*c9p3*ss[66] + 756*c3p2*c7p5*c9p2*ss[64] + 252*c3p2*c7p6*c9p1*ss[62] + 504*c3p3*c5p1*c7p5*ss[54] + 504*c3p3*c5p1*c9p5*ss[64] + 1260*c3p3*c5p2*c7p4*ss[52] + 1260*c3p3*c5p2*c9p4*ss[60] + 1680*c3p3*c5p3*c7p3*ss[50] + 1680*c3p3*c5p3*c9p3*ss[56] + 1260*c3p3*c5p4*c7p2*ss[48] + 1260*c3p3*c5p4*c9p2*ss[52] + 504*c3p3*c5p5*c7p1*ss[46] + 504*c3p3*c5p5*c9p1*ss[48] + 504*c3p3*c7p1*c9p5*ss[66] + 1260*c3p3*c7p2*c9p4*ss[64] + 1680*c3p3*c7p3*c9p3*ss[62] + 1260*c3p3*c7p4*c9p2*ss[60] + 504*c3p3*c7p5*c9p1*ss[58] + 630*c3p4*c5p1*c7p4*ss[50] + 630*c3p4*c5p1*c9p4*ss[58] + 1260*c3p4*c5p2*c7p3*ss[48] + 1260*c3p4*c5p2*c9p3*ss[54] + 1260*c3p4*c5p3*c7p2*ss[46] + 1260*c3p4*c5p3*c9p2*ss[50] + 630*c3p4*c5p4*c7p1*ss[44] + 630*c3p4*c5p4*c9p1*ss[46] + 630*c3p4*c7p1*c9p4*ss[60] + 1260*c3p4*c7p2*c9p3*ss[58] + 1260*c3p4*c7p3*c9p2*ss[56] + 630*c3p4*c7p4*c9p1*ss[54] + 504*c3p5*c5p1*c7p3*ss[46] + 504*c3p5*c5p1*c9p3*ss[52] + 756*c3p5*c5p2*c7p2*ss[44] + 756*c3p5*c5p2*c9p2*ss[48] + 504*c3p5*c5p3*c7p1*ss[42] + 504*c3p5*c5p3*c9p1*ss[44] + 504*c3p5*c7p1*c9p3*ss[54] + 756*c3p5*c7p2*c9p2*ss[52] + 504*c3p5*c7p3*c9p1*ss[50] + 252*c3p6*c5p1*c7p2*ss[42] + 252*c3p6*c5p1*c9p2*ss[46] + 252*c3p6*c5p2*c7p1*ss[40] + 252*c3p6*c5p2*c9p1*ss[42] + 252*c3p6*c7p1*c9p2*ss[48] + 252*c3p6*c7p2*c9p1*ss[46] + 72*c3p7*c5p1*c7p1*ss[38] + 72*c3p7*c5p1*c9p1*ss[40] + 72*c3p7*c7p1*c9p1*ss[42] + 72*c5p1*c7p1*c9p7*ss[80] + 252*c5p1*c7p2*c9p6*ss[78] + 504*c5p1*c7p3*c9p5*ss[76] + 630*c5p1*c7p4*c9p4*ss[74] + 504*c5p1*c7p5*c9p3*ss[72] + 252*c5p1*c7p6*c9p2*ss[70] + 72*c5p1*c7p7*c9p1*ss[68] + 252*c5p2*c7p1*c9p6*ss[76] + 756*c5p2*c7p2*c9p5*ss[74] + 1260*c5p2*c7p3*c9p4*ss[72] + 1260*c5p2*c7p4*c9p3*ss[70] + 756*c5p2*c7p5*c9p2*ss[68] + 252*c5p2*c7p6*c9p1*ss[66] + 504*c5p3*c7p1*c9p5*ss[72] + 1260*c5p3*c7p2*c9p4*ss[70] + 1680*c5p3*c7p3*c9p3*ss[68] + 1260*c5p3*c7p4*c9p2*ss[66] + 504*c5p3*c7p5*c9p1*ss[64] + 630*c5p4*c7p1*c9p4*ss[68] + 1260*c5p4*c7p2*c9p3*ss[66] + 1260*c5p4*c7p3*c9p2*ss[64] + 630*c5p4*c7p4*c9p1*ss[62] + 504*c5p5*c7p1*c9p3*ss[64] + 756*c5p5*c7p2*c9p2*ss[62] + 504*c5p5*c7p3*c9p1*ss[60] + 252*c5p6*c7p1*c9p2*ss[60] + 252*c5p6*c7p2*c9p1*ss[58] + 72*c5p7*c7p1*c9p1*ss[56] + 9*c1p1*c3p8*ss[30] + 9*c1p1*c5p8*ss[46] + 9*c1p1*c7p8*ss[62] + 9*c1p1*c9p8*ss[78] + 36*c1p2*c3p7*ss[28] + 36*c1p2*c5p7*ss[42] + 36*c1p2*c7p7*ss[56] + 36*c1p2*c9p7*ss[70] + 84*c1p3*c3p6*ss[26] + 84*c1p3*c5p6*ss[38] + 84*c1p3*c7p6*ss[50] + 84*c1p3*c9p6*ss[62] + 126*c1p4*c3p5*ss[24] + 126*c1p4*c5p5*ss[34] + 126*c1p4*c7p5*ss[44] + 126*c1p4*c9p5*ss[54] + 126*c1p5*c3p4*ss[22] + 126*c1p5*c5p4*ss[30] + 126*c1p5*c7p4*ss[38] + 126*c1p5*c9p4*ss[46] + 84*c1p6*c3p3*ss[20] + 84*c1p6*c5p3*ss[26] + 84*c1p6*c7p3*ss[32] + 84*c1p6*c9p3*ss[38] + 36*c1p7*c3p2*ss[18] + 36*c1p7*c5p2*ss[22] + 36*c1p7*c7p2*ss[26] + 36*c1p7*c9p2*ss[30] + 9*c1p8*c3p1*ss[16] + 9*c1p8*c5p1*ss[18] + 9*c1p8*c7p1*ss[20] + 9*c1p8*c9p1*ss[22] + 9*c3p1*c5p8*ss[48] + 9*c3p1*c7p8*ss[64] + 9*c3p1*c9p8*ss[80] + 36*c3p2*c5p7*ss[46] + 36*c3p2*c7p7*ss[60] + 36*c3p2*c9p7*ss[74] + 84*c3p3*c5p6*ss[44] + 84*c3p3*c7p6*ss[56] + 84*c3p3*c9p6*ss[68] + 126*c3p4*c5p5*ss[42] + 126*c3p4*c7p5*ss[52] + 126*c3p4*c9p5*ss[62] + 126*c3p5*c5p4*ss[40] + 126*c3p5*c7p4*ss[48] + 126*c3p5*c9p4*ss[56] + 84*c3p6*c5p3*ss[38] + 84*c3p6*c7p3*ss[44] + 84*c3p6*c9p3*ss[50] + 36*c3p7*c5p2*ss[36] + 36*c3p7*c7p2*ss[40] + 36*c3p7*c9p2*ss[44] + 9*c3p8*c5p1*ss[34] + 9*c3p8*c7p1*ss[36] + 9*c3p8*c9p1*ss[38] + 9*c5p1*c7p8*ss[66] + 9*c5p1*c9p8*ss[82] + 36*c5p2*c7p7*ss[64] + 36*c5p2*c9p7*ss[78] + 84*c5p3*c7p6*ss[62] + 84*c5p3*c9p6*ss[74] + 126*c5p4*c7p5*ss[60] + 126*c5p4*c9p5*ss[70] + 126*c5p5*c7p4*ss[58] + 126*c5p5*c9p4*ss[66] + 84*c5p6*c7p3*ss[56] + 84*c5p6*c9p3*ss[62] + 36*c5p7*c7p2*ss[54] + 36*c5p7*c9p2*ss[58] + 9*c5p8*c7p1*ss[52] + 9*c5p8*c9p1*ss[54] + 9*c7p1*c9p8*ss[84] + 36*c7p2*c9p7*ss[82] + 84*c7p3*c9p6*ss[80] + 126*c7p4*c9p5*ss[78] + 126*c7p5*c9p4*ss[76] + 84*c7p6*c9p3*ss[74] + 36*c7p7*c9p2*ss[72] + 9*c7p8*c9p1*ss[70] + c1p9*ss[14] + c3p9*ss[32] + c5p9*ss[50] + c7p9*ss[68] + c9p9*ss[86])*sys_part_4;
  TACC grad_2_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4)));
  TACC grad_3_num = convert<TACC>(2*(c1p1*ss[8] + c3p1*ss[10] + c5p1*ss[12] + c7p1*ss[14] + c9p1*ss[16])*sys_part_0);
  grad_3_num += 4*(6*c1p1*c3p1*c5p1*ss[16] + 6*c1p1*c3p1*c7p1*ss[18] + 6*c1p1*c3p1*c9p1*ss[20] + 6*c1p1*c5p1*c7p1*ss[20] + 6*c1p1*c5p1*c9p1*ss[22] + 6*c1p1*c7p1*c9p1*ss[24] + 6*c3p1*c5p1*c7p1*ss[22] + 6*c3p1*c5p1*c9p1*ss[24] + 6*c3p1*c7p1*c9p1*ss[26] + 6*c5p1*c7p1*c9p1*ss[28] + 3*c1p1*c3p2*ss[14] + 3*c1p1*c5p2*ss[18] + 3*c1p1*c7p2*ss[22] + 3*c1p1*c9p2*ss[26] + 3*c1p2*c3p1*ss[12] + 3*c1p2*c5p1*ss[14] + 3*c1p2*c7p1*ss[16] + 3*c1p2*c9p1*ss[18] + 3*c3p1*c5p2*ss[20] + 3*c3p1*c7p2*ss[24] + 3*c3p1*c9p2*ss[28] + 3*c3p2*c5p1*ss[18] + 3*c3p2*c7p1*ss[20] + 3*c3p2*c9p1*ss[22] + 3*c5p1*c7p2*ss[26] + 3*c5p1*c9p2*ss[30] + 3*c5p2*c7p1*ss[24] + 3*c5p2*c9p1*ss[26] + 3*c7p1*c9p2*ss[32] + 3*c7p2*c9p1*ss[30] + c1p3*ss[10] + c3p3*ss[16] + c5p3*ss[22] + c7p3*ss[28] + c9p3*ss[34])*sys_part_1;
  grad_3_num += 6*(120*c1p1*c3p1*c5p1*c7p1*c9p1*ss[32] + 60*c1p1*c3p1*c5p1*c7p2*ss[30] + 60*c1p1*c3p1*c5p1*c9p2*ss[34] + 60*c1p1*c3p1*c5p2*c7p1*ss[28] + 60*c1p1*c3p1*c5p2*c9p1*ss[30] + 60*c1p1*c3p1*c7p1*c9p2*ss[36] + 60*c1p1*c3p1*c7p2*c9p1*ss[34] + 60*c1p1*c3p2*c5p1*c7p1*ss[26] + 60*c1p1*c3p2*c5p1*c9p1*ss[28] + 60*c1p1*c3p2*c7p1*c9p1*ss[30] + 60*c1p1*c5p1*c7p1*c9p2*ss[38] + 60*c1p1*c5p1*c7p2*c9p1*ss[36] + 60*c1p1*c5p2*c7p1*c9p1*ss[34] + 60*c1p2*c3p1*c5p1*c7p1*ss[24] + 60*c1p2*c3p1*c5p1*c9p1*ss[26] + 60*c1p2*c3p1*c7p1*c9p1*ss[28] + 60*c1p2*c5p1*c7p1*c9p1*ss[30] + 60*c3p1*c5p1*c7p1*c9p2*ss[40] + 60*c3p1*c5p1*c7p2*c9p1*ss[38] + 60*c3p1*c5p2*c7p1*c9p1*ss[36] + 60*c3p2*c5p1*c7p1*c9p1*ss[34] + 20*c1p1*c3p1*c5p3*ss[26] + 20*c1p1*c3p1*c7p3*ss[32] + 20*c1p1*c3p1*c9p3*ss[38] + 30*c1p1*c3p2*c5p2*ss[24] + 30*c1p1*c3p2*c7p2*ss[28] + 30*c1p1*c3p2*c9p2*ss[32] + 20*c1p1*c3p3*c5p1*ss[22] + 20*c1p1*c3p3*c7p1*ss[24] + 20*c1p1*c3p3*c9p1*ss[26] + 20*c1p1*c5p1*c7p3*ss[34] + 20*c1p1*c5p1*c9p3*ss[40] + 30*c1p1*c5p2*c7p2*ss[32] + 30*c1p1*c5p2*c9p2*ss[36] + 20*c1p1*c5p3*c7p1*ss[30] + 20*c1p1*c5p3*c9p1*ss[32] + 20*c1p1*c7p1*c9p3*ss[42] + 30*c1p1*c7p2*c9p2*ss[40] + 20*c1p1*c7p3*c9p1*ss[38] + 30*c1p2*c3p1*c5p2*ss[22] + 30*c1p2*c3p1*c7p2*ss[26] + 30*c1p2*c3p1*c9p2*ss[30] + 30*c1p2*c3p2*c5p1*ss[20] + 30*c1p2*c3p2*c7p1*ss[22] + 30*c1p2*c3p2*c9p1*ss[24] + 30*c1p2*c5p1*c7p2*ss[28] + 30*c1p2*c5p1*c9p2*ss[32] + 30*c1p2*c5p2*c7p1*ss[26] + 30*c1p2*c5p2*c9p1*ss[28] + 30*c1p2*c7p1*c9p2*ss[34] + 30*c1p2*c7p2*c9p1*ss[32] + 20*c1p3*c3p1*c5p1*ss[18] + 20*c1p3*c3p1*c7p1*ss[20] + 20*c1p3*c3p1*c9p1*ss[22] + 20*c1p3*c5p1*c7p1*ss[22] + 20*c1p3*c5p1*c9p1*ss[24] + 20*c1p3*c7p1*c9p1*ss[26] + 20*c3p1*c5p1*c7p3*ss[36] + 20*c3p1*c5p1*c9p3*ss[42] + 30*c3p1*c5p2*c7p2*ss[34] + 30*c3p1*c5p2*c9p2*ss[38] + 20*c3p1*c5p3*c7p1*ss[32] + 20*c3p1*c5p3*c9p1*ss[34] + 20*c3p1*c7p1*c9p3*ss[44] + 30*c3p1*c7p2*c9p2*ss[42] + 20*c3p1*c7p3*c9p1*ss[40] + 30*c3p2*c5p1*c7p2*ss[32] + 30*c3p2*c5p1*c9p2*ss[36] + 30*c3p2*c5p2*c7p1*ss[30] + 30*c3p2*c5p2*c9p1*ss[32] + 30*c3p2*c7p1*c9p2*ss[38] + 30*c3p2*c7p2*c9p1*ss[36] + 20*c3p3*c5p1*c7p1*ss[28] + 20*c3p3*c5p1*c9p1*ss[30] + 20*c3p3*c7p1*c9p1*ss[32] + 20*c5p1*c7p1*c9p3*ss[46] + 30*c5p1*c7p2*c9p2*ss[44] + 20*c5p1*c7p3*c9p1*ss[42] + 30*c5p2*c7p1*c9p2*ss[42] + 30*c5p2*c7p2*c9p1*ss[40] + 20*c5p3*c7p1*c9p1*ss[38] + 5*c1p1*c3p4*ss[20] + 5*c1p1*c5p4*ss[28] + 5*c1p1*c7p4*ss[36] + 5*c1p1*c9p4*ss[44] + 10*c1p2*c3p3*ss[18] + 10*c1p2*c5p3*ss[24] + 10*c1p2*c7p3*ss[30] + 10*c1p2*c9p3*ss[36] + 10*c1p3*c3p2*ss[16] + 10*c1p3*c5p2*ss[20] + 10*c1p3*c7p2*ss[24] + 10*c1p3*c9p2*ss[28] + 5*c1p4*c3p1*ss[14] + 5*c1p4*c5p1*ss[16] + 5*c1p4*c7p1*ss[18] + 5*c1p4*c9p1*ss[20] + 5*c3p1*c5p4*ss[30] + 5*c3p1*c7p4*ss[38] + 5*c3p1*c9p4*ss[46] + 10*c3p2*c5p3*ss[28] + 10*c3p2*c7p3*ss[34] + 10*c3p2*c9p3*ss[40] + 10*c3p3*c5p2*ss[26] + 10*c3p3*c7p2*ss[30] + 10*c3p3*c9p2*ss[34] + 5*c3p4*c5p1*ss[24] + 5*c3p4*c7p1*ss[26] + 5*c3p4*c9p1*ss[28] + 5*c5p1*c7p4*ss[40] + 5*c5p1*c9p4*ss[48] + 10*c5p2*c7p3*ss[38] + 10*c5p2*c9p3*ss[44] + 10*c5p3*c7p2*ss[36] + 10*c5p3*c9p2*ss[40] + 5*c5p4*c7p1*ss[34] + 5*c5p4*c9p1*ss[36] + 5*c7p1*c9p4*ss[50] + 10*c7p2*c9p3*ss[48] + 10*c7p3*c9p2*ss[46] + 5*c7p4*c9p1*ss[44] + c1p5*ss[12] + c3p5*ss[22] + c5p5*ss[32] + c7p5*ss[42] + c9p5*ss[52])*sys_part_2;
  grad_3_num += 8*(840*c1p1*c3p1*c5p1*c7p1*c9p3*ss[50] + 1260*c1p1*c3p1*c5p1*c7p2*c9p2*ss[48] + 840*c1p1*c3p1*c5p1*c7p3*c9p1*ss[46] + 1260*c1p1*c3p1*c5p2*c7p1*c9p2*ss[46] + 1260*c1p1*c3p1*c5p2*c7p2*c9p1*ss[44] + 840*c1p1*c3p1*c5p3*c7p1*c9p1*ss[42] + 1260*c1p1*c3p2*c5p1*c7p1*c9p2*ss[44] + 1260*c1p1*c3p2*c5p1*c7p2*c9p1*ss[42] + 1260*c1p1*c3p2*c5p2*c7p1*c9p1*ss[40] + 840*c1p1*c3p3*c5p1*c7p1*c9p1*ss[38] + 1260*c1p2*c3p1*c5p1*c7p1*c9p2*ss[42] + 1260*c1p2*c3p1*c5p1*c7p2*c9p1*ss[40] + 1260*c1p2*c3p1*c5p2*c7p1*c9p1*ss[38] + 1260*c1p2*c3p2*c5p1*c7p1*c9p1*ss[36] + 840*c1p3*c3p1*c5p1*c7p1*c9p1*ss[34] + 210*c1p1*c3p1*c5p1*c7p4*ss[44] + 210*c1p1*c3p1*c5p1*c9p4*ss[52] + 420*c1p1*c3p1*c5p2*c7p3*ss[42] + 420*c1p1*c3p1*c5p2*c9p3*ss[48] + 420*c1p1*c3p1*c5p3*c7p2*ss[40] + 420*c1p1*c3p1*c5p3*c9p2*ss[44] + 210*c1p1*c3p1*c5p4*c7p1*ss[38] + 210*c1p1*c3p1*c5p4*c9p1*ss[40] + 210*c1p1*c3p1*c7p1*c9p4*ss[54] + 420*c1p1*c3p1*c7p2*c9p3*ss[52] + 420*c1p1*c3p1*c7p3*c9p2*ss[50] + 210*c1p1*c3p1*c7p4*c9p1*ss[48] + 420*c1p1*c3p2*c5p1*c7p3*ss[40] + 420*c1p1*c3p2*c5p1*c9p3*ss[46] + 630*c1p1*c3p2*c5p2*c7p2*ss[38] + 630*c1p1*c3p2*c5p2*c9p2*ss[42] + 420*c1p1*c3p2*c5p3*c7p1*ss[36] + 420*c1p1*c3p2*c5p3*c9p1*ss[38] + 420*c1p1*c3p2*c7p1*c9p3*ss[48] + 630*c1p1*c3p2*c7p2*c9p2*ss[46] + 420*c1p1*c3p2*c7p3*c9p1*ss[44] + 420*c1p1*c3p3*c5p1*c7p2*ss[36] + 420*c1p1*c3p3*c5p1*c9p2*ss[40] + 420*c1p1*c3p3*c5p2*c7p1*ss[34] + 420*c1p1*c3p3*c5p2*c9p1*ss[36] + 420*c1p1*c3p3*c7p1*c9p2*ss[42] + 420*c1p1*c3p3*c7p2*c9p1*ss[40] + 210*c1p1*c3p4*c5p1*c7p1*ss[32] + 210*c1p1*c3p4*c5p1*c9p1*ss[34] + 210*c1p1*c3p4*c7p1*c9p1*ss[36] + 210*c1p1*c5p1*c7p1*c9p4*ss[56] + 420*c1p1*c5p1*c7p2*c9p3*ss[54] + 420*c1p1*c5p1*c7p3*c9p2*ss[52] + 210*c1p1*c5p1*c7p4*c9p1*ss[50] + 420*c1p1*c5p2*c7p1*c9p3*ss[52] + 630*c1p1*c5p2*c7p2*c9p2*ss[50] + 420*c1p1*c5p2*c7p3*c9p1*ss[48] + 420*c1p1*c5p3*c7p1*c9p2*ss[48] + 420*c1p1*c5p3*c7p2*c9p1*ss[46] + 210*c1p1*c5p4*c7p1*c9p1*ss[44] + 420*c1p2*c3p1*c5p1*c7p3*ss[38] + 420*c1p2*c3p1*c5p1*c9p3*ss[44] + 630*c1p2*c3p1*c5p2*c7p2*ss[36] + 630*c1p2*c3p1*c5p2*c9p2*ss[40] + 420*c1p2*c3p1*c5p3*c7p1*ss[34] + 420*c1p2*c3p1*c5p3*c9p1*ss[36] + 420*c1p2*c3p1*c7p1*c9p3*ss[46] + 630*c1p2*c3p1*c7p2*c9p2*ss[44] + 420*c1p2*c3p1*c7p3*c9p1*ss[42] + 630*c1p2*c3p2*c5p1*c7p2*ss[34] + 630*c1p2*c3p2*c5p1*c9p2*ss[38] + 630*c1p2*c3p2*c5p2*c7p1*ss[32] + 630*c1p2*c3p2*c5p2*c9p1*ss[34] + 630*c1p2*c3p2*c7p1*c9p2*ss[40] + 630*c1p2*c3p2*c7p2*c9p1*ss[38] + 420*c1p2*c3p3*c5p1*c7p1*ss[30] + 420*c1p2*c3p3*c5p1*c9p1*ss[32] + 420*c1p2*c3p3*c7p1*c9p1*ss[34] + 420*c1p2*c5p1*c7p1*c9p3*ss[48] + 630*c1p2*c5p1*c7p2*c9p2*ss[46] + 420*c1p2*c5p1*c7p3*c9p1*ss[44] + 630*c1p2*c5p2*c7p1*c9p2*ss[44] + 630*c1p2*c5p2*c7p2*c9p1*ss[42] + 420*c1p2*c5p3*c7p1*c9p1*ss[40] + 420*c1p3*c3p1*c5p1*c7p2*ss[32] + 420*c1p3*c3p1*c5p1*c9p2*ss[36] + 420*c1p3*c3p1*c5p2*c7p1*ss[30] + 420*c1p3*c3p1*c5p2*c9p1*ss[32] + 420*c1p3*c3p1*c7p1*c9p2*ss[38] + 420*c1p3*c3p1*c7p2*c9p1*ss[36] + 420*c1p3*c3p2*c5p1*c7p1*ss[28] + 420*c1p3*c3p2*c5p1*c9p1*ss[30] + 420*c1p3*c3p2*c7p1*c9p1*ss[32] + 420*c1p3*c5p1*c7p1*c9p2*ss[40] + 420*c1p3*c5p1*c7p2*c9p1*ss[38] + 420*c1p3*c5p2*c7p1*c9p1*ss[36] + 210*c1p4*c3p1*c5p1*c7p1*ss[26] + 210*c1p4*c3p1*c5p1*c9p1*ss[28] + 210*c1p4*c3p1*c7p1*c9p1*ss[30] + 210*c1p4*c5p1*c7p1*c9p1*ss[32] + 210*c3p1*c5p1*c7p1*c9p4*ss[58] + 420*c3p1*c5p1*c7p2*c9p3*ss[56] + 420*c3p1*c5p1*c7p3*c9p2*ss[54] + 210*c3p1*c5p1*c7p4*c9p1*ss[52] + 420*c3p1*c5p2*c7p1*c9p3*ss[54] + 630*c3p1*c5p2*c7p2*c9p2*ss[52] + 420*c3p1*c5p2*c7p3*c9p1*ss[50] + 420*c3p1*c5p3*c7p1*c9p2*ss[50] + 420*c3p1*c5p3*c7p2*c9p1*ss[48] + 210*c3p1*c5p4*c7p1*c9p1*ss[46] + 420*c3p2*c5p1*c7p1*c9p3*ss[52] + 630*c3p2*c5p1*c7p2*c9p2*ss[50] + 420*c3p2*c5p1*c7p3*c9p1*ss[48] + 630*c3p2*c5p2*c7p1*c9p2*ss[48] + 630*c3p2*c5p2*c7p2*c9p1*ss[46] + 420*c3p2*c5p3*c7p1*c9p1*ss[44] + 420*c3p3*c5p1*c7p1*c9p2*ss[46] + 420*c3p3*c5p1*c7p2*c9p1*ss[44] + 420*c3p3*c5p2*c7p1*c9p1*ss[42] + 210*c3p4*c5p1*c7p1*c9p1*ss[40] + 42*c1p1*c3p1*c5p5*ss[36] + 42*c1p1*c3p1*c7p5*ss[46] + 42*c1p1*c3p1*c9p5*ss[56] + 105*c1p1*c3p2*c5p4*ss[34] + 105*c1p1*c3p2*c7p4*ss[42] + 105*c1p1*c3p2*c9p4*ss[50] + 140*c1p1*c3p3*c5p3*ss[32] + 140*c1p1*c3p3*c7p3*ss[38] + 140*c1p1*c3p3*c9p3*ss[44] + 105*c1p1*c3p4*c5p2*ss[30] + 105*c1p1*c3p4*c7p2*ss[34] + 105*c1p1*c3p4*c9p2*ss[38] + 42*c1p1*c3p5*c5p1*ss[28] + 42*c1p1*c3p5*c7p1*ss[30] + 42*c1p1*c3p5*c9p1*ss[32] + 42*c1p1*c5p1*c7p5*ss[48] + 42*c1p1*c5p1*c9p5*ss[58] + 105*c1p1*c5p2*c7p4*ss[46] + 105*c1p1*c5p2*c9p4*ss[54] + 140*c1p1*c5p3*c7p3*ss[44] + 140*c1p1*c5p3*c9p3*ss[50] + 105*c1p1*c5p4*c7p2*ss[42] + 105*c1p1*c5p4*c9p2*ss[46] + 42*c1p1*c5p5*c7p1*ss[40] + 42*c1p1*c5p5*c9p1*ss[42] + 42*c1p1*c7p1*c9p5*ss[60] + 105*c1p1*c7p2*c9p4*ss[58] + 140*c1p1*c7p3*c9p3*ss[56] + 105*c1p1*c7p4*c9p2*ss[54] + 42*c1p1*c7p5*c9p1*ss[52] + 105*c1p2*c3p1*c5p4*ss[32] + 105*c1p2*c3p1*c7p4*ss[40] + 105*c1p2*c3p1*c9p4*ss[48] + 210*c1p2*c3p2*c5p3*ss[30] + 210*c1p2*c3p2*c7p3*ss[36] + 210*c1p2*c3p2*c9p3*ss[42] + 210*c1p2*c3p3*c5p2*ss[28] + 210*c1p2*c3p3*c7p2*ss[32] + 210*c1p2*c3p3*c9p2*ss[36] + 105*c1p2*c3p4*c5p1*ss[26] + 105*c1p2*c3p4*c7p1*ss[28] + 105*c1p2*c3p4*c9p1*ss[30] + 105*c1p2*c5p1*c7p4*ss[42] + 105*c1p2*c5p1*c9p4*ss[50] + 210*c1p2*c5p2*c7p3*ss[40] + 210*c1p2*c5p2*c9p3*ss[46] + 210*c1p2*c5p3*c7p2*ss[38] + 210*c1p2*c5p3*c9p2*ss[42] + 105*c1p2*c5p4*c7p1*ss[36] + 105*c1p2*c5p4*c9p1*ss[38] + 105*c1p2*c7p1*c9p4*ss[52] + 210*c1p2*c7p2*c9p3*ss[50] + 210*c1p2*c7p3*c9p2*ss[48] + 105*c1p2*c7p4*c9p1*ss[46] + 140*c1p3*c3p1*c5p3*ss[28] + 140*c1p3*c3p1*c7p3*ss[34] + 140*c1p3*c3p1*c9p3*ss[40] + 210*c1p3*c3p2*c5p2*ss[26] + 210*c1p3*c3p2*c7p2*ss[30] + 210*c1p3*c3p2*c9p2*ss[34] + 140*c1p3*c3p3*c5p1*ss[24] + 140*c1p3*c3p3*c7p1*ss[26] + 140*c1p3*c3p3*c9p1*ss[28] + 140*c1p3*c5p1*c7p3*ss[36] + 140*c1p3*c5p1*c9p3*ss[42] + 210*c1p3*c5p2*c7p2*ss[34] + 210*c1p3*c5p2*c9p2*ss[38] + 140*c1p3*c5p3*c7p1*ss[32] + 140*c1p3*c5p3*c9p1*ss[34] + 140*c1p3*c7p1*c9p3*ss[44] + 210*c1p3*c7p2*c9p2*ss[42] + 140*c1p3*c7p3*c9p1*ss[40] + 105*c1p4*c3p1*c5p2*ss[24] + 105*c1p4*c3p1*c7p2*ss[28] + 105*c1p4*c3p1*c9p2*ss[32] + 105*c1p4*c3p2*c5p1*ss[22] + 105*c1p4*c3p2*c7p1*ss[24] + 105*c1p4*c3p2*c9p1*ss[26] + 105*c1p4*c5p1*c7p2*ss[30] + 105*c1p4*c5p1*c9p2*ss[34] + 105*c1p4*c5p2*c7p1*ss[28] + 105*c1p4*c5p2*c9p1*ss[30] + 105*c1p4*c7p1*c9p2*ss[36] + 105*c1p4*c7p2*c9p1*ss[34] + 42*c1p5*c3p1*c5p1*ss[20] + 42*c1p5*c3p1*c7p1*ss[22] + 42*c1p5*c3p1*c9p1*ss[24] + 42*c1p5*c5p1*c7p1*ss[24] + 42*c1p5*c5p1*c9p1*ss[26] + 42*c1p5*c7p1*c9p1*ss[28] + 42*c3p1*c5p1*c7p5*ss[50] + 42*c3p1*c5p1*c9p5*ss[60] + 105*c3p1*c5p2*c7p4*ss[48] + 105*c3p1*c5p2*c9p4*ss[56] + 140*c3p1*c5p3*c7p3*ss[46] + 140*c3p1*c5p3*c9p3*ss[52] + 105*c3p1*c5p4*c7p2*ss[44] + 105*c3p1*c5p4*c9p2*ss[48] + 42*c3p1*c5p5*c7p1*ss[42] + 42*c3p1*c5p5*c9p1*ss[44] + 42*c3p1*c7p1*c9p5*ss[62] + 105*c3p1*c7p2*c9p4*ss[60] + 140*c3p1*c7p3*c9p3*ss[58] + 105*c3p1*c7p4*c9p2*ss[56] + 42*c3p1*c7p5*c9p1*ss[54] + 105*c3p2*c5p1*c7p4*ss[46] + 105*c3p2*c5p1*c9p4*ss[54] + 210*c3p2*c5p2*c7p3*ss[44] + 210*c3p2*c5p2*c9p3*ss[50] + 210*c3p2*c5p3*c7p2*ss[42] + 210*c3p2*c5p3*c9p2*ss[46] + 105*c3p2*c5p4*c7p1*ss[40] + 105*c3p2*c5p4*c9p1*ss[42] + 105*c3p2*c7p1*c9p4*ss[56] + 210*c3p2*c7p2*c9p3*ss[54] + 210*c3p2*c7p3*c9p2*ss[52] + 105*c3p2*c7p4*c9p1*ss[50] + 140*c3p3*c5p1*c7p3*ss[42] + 140*c3p3*c5p1*c9p3*ss[48] + 210*c3p3*c5p2*c7p2*ss[40] + 210*c3p3*c5p2*c9p2*ss[44] + 140*c3p3*c5p3*c7p1*ss[38] + 140*c3p3*c5p3*c9p1*ss[40] + 140*c3p3*c7p1*c9p3*ss[50] + 210*c3p3*c7p2*c9p2*ss[48] + 140*c3p3*c7p3*c9p1*ss[46] + 105*c3p4*c5p1*c7p2*ss[38] + 105*c3p4*c5p1*c9p2*ss[42] + 105*c3p4*c5p2*c7p1*ss[36] + 105*c3p4*c5p2*c9p1*ss[38] + 105*c3p4*c7p1*c9p2*ss[44] + 105*c3p4*c7p2*c9p1*ss[42] + 42*c3p5*c5p1*c7p1*ss[34] + 42*c3p5*c5p1*c9p1*ss[36] + 42*c3p5*c7p1*c9p1*ss[38] + 42*c5p1*c7p1*c9p5*ss[64] + 105*c5p1*c7p2*c9p4*ss[62] + 140*c5p1*c7p3*c9p3*ss[60] + 105*c5p1*c7p4*c9p2*ss[58] + 42*c5p1*c7p5*c9p1*ss[56] + 105*c5p2*c7p1*c9p4*ss[60] + 210*c5p2*c7p2*c9p3*ss[58] + 210*c5p2*c7p3*c9p2*ss[56] + 105*c5p2*c7p4*c9p1*ss[54] + 140*c5p3*c7p1*c9p3*ss[56] + 210*c5p3*c7p2*c9p2*ss[54] + 140*c5p3*c7p3*c9p1*ss[52] + 105*c5p4*c7p1*c9p2*ss[52] + 105*c5p4*c7p2*c9p1*ss[50] + 42*c5p5*c7p1*c9p1*ss[48] + 7*c1p1*c3p6*ss[26] + 7*c1p1*c5p6*ss[38] + 7*c1p1*c7p6*ss[50] + 7*c1p1*c9p6*ss[62] + 21*c1p2*c3p5*ss[24] + 21*c1p2*c5p5*ss[34] + 21*c1p2*c7p5*ss[44] + 21*c1p2*c9p5*ss[54] + 35*c1p3*c3p4*ss[22] + 35*c1p3*c5p4*ss[30] + 35*c1p3*c7p4*ss[38] + 35*c1p3*c9p4*ss[46] + 35*c1p4*c3p3*ss[20] + 35*c1p4*c5p3*ss[26] + 35*c1p4*c7p3*ss[32] + 35*c1p4*c9p3*ss[38] + 21*c1p5*c3p2*ss[18] + 21*c1p5*c5p2*ss[22] + 21*c1p5*c7p2*ss[26] + 21*c1p5*c9p2*ss[30] + 7*c1p6*c3p1*ss[16] + 7*c1p6*c5p1*ss[18] + 7*c1p6*c7p1*ss[20] + 7*c1p6*c9p1*ss[22] + 7*c3p1*c5p6*ss[40] + 7*c3p1*c7p6*ss[52] + 7*c3p1*c9p6*ss[64] + 21*c3p2*c5p5*ss[38] + 21*c3p2*c7p5*ss[48] + 21*c3p2*c9p5*ss[58] + 35*c3p3*c5p4*ss[36] + 35*c3p3*c7p4*ss[44] + 35*c3p3*c9p4*ss[52] + 35*c3p4*c5p3*ss[34] + 35*c3p4*c7p3*ss[40] + 35*c3p4*c9p3*ss[46] + 21*c3p5*c5p2*ss[32] + 21*c3p5*c7p2*ss[36] + 21*c3p5*c9p2*ss[40] + 7*c3p6*c5p1*ss[30] + 7*c3p6*c7p1*ss[32] + 7*c3p6*c9p1*ss[34] + 7*c5p1*c7p6*ss[54] + 7*c5p1*c9p6*ss[66] + 21*c5p2*c7p5*ss[52] + 21*c5p2*c9p5*ss[62] + 35*c5p3*c7p4*ss[50] + 35*c5p3*c9p4*ss[58] + 35*c5p4*c7p3*ss[48] + 35*c5p4*c9p3*ss[54] + 21*c5p5*c7p2*ss[46] + 21*c5p5*c9p2*ss[50] + 7*c5p6*c7p1*ss[44] + 7*c5p6*c9p1*ss[46] + 7*c7p1*c9p6*ss[68] + 21*c7p2*c9p5*ss[66] + 35*c7p3*c9p4*ss[64] + 35*c7p4*c9p3*ss[62] + 21*c7p5*c9p2*ss[60] + 7*c7p6*c9p1*ss[58] + c1p7*ss[14] + c3p7*ss[28] + c5p7*ss[42] + c7p7*ss[56] + c9p7*ss[70])*sys_part_3;
  grad_3_num += 10*(3024*c1p1*c3p1*c5p1*c7p1*c9p5*ss[68] + 7560*c1p1*c3p1*c5p1*c7p2*c9p4*ss[66] + 10080*c1p1*c3p1*c5p1*c7p3*c9p3*ss[64] + 7560*c1p1*c3p1*c5p1*c7p4*c9p2*ss[62] + 3024*c1p1*c3p1*c5p1*c7p5*c9p1*ss[60] + 7560*c1p1*c3p1*c5p2*c7p1*c9p4*ss[64] + 15120*c1p1*c3p1*c5p2*c7p2*c9p3*ss[62] + 15120*c1p1*c3p1*c5p2*c7p3*c9p2*ss[60] + 7560*c1p1*c3p1*c5p2*c7p4*c9p1*ss[58] + 10080*c1p1*c3p1*c5p3*c7p1*c9p3*ss[60] + 15120*c1p1*c3p1*c5p3*c7p2*c9p2*ss[58] + 10080*c1p1*c3p1*c5p3*c7p3*c9p1*ss[56] + 7560*c1p1*c3p1*c5p4*c7p1*c9p2*ss[56] + 7560*c1p1*c3p1*c5p4*c7p2*c9p1*ss[54] + 3024*c1p1*c3p1*c5p5*c7p1*c9p1*ss[52] + 7560*c1p1*c3p2*c5p1*c7p1*c9p4*ss[62] + 15120*c1p1*c3p2*c5p1*c7p2*c9p3*ss[60] + 15120*c1p1*c3p2*c5p1*c7p3*c9p2*ss[58] + 7560*c1p1*c3p2*c5p1*c7p4*c9p1*ss[56] + 15120*c1p1*c3p2*c5p2*c7p1*c9p3*ss[58] + 22680*c1p1*c3p2*c5p2*c7p2*c9p2*ss[56] + 15120*c1p1*c3p2*c5p2*c7p3*c9p1*ss[54] + 15120*c1p1*c3p2*c5p3*c7p1*c9p2*ss[54] + 15120*c1p1*c3p2*c5p3*c7p2*c9p1*ss[52] + 7560*c1p1*c3p2*c5p4*c7p1*c9p1*ss[50] + 10080*c1p1*c3p3*c5p1*c7p1*c9p3*ss[56] + 15120*c1p1*c3p3*c5p1*c7p2*c9p2*ss[54] + 10080*c1p1*c3p3*c5p1*c7p3*c9p1*ss[52] + 15120*c1p1*c3p3*c5p2*c7p1*c9p2*ss[52] + 15120*c1p1*c3p3*c5p2*c7p2*c9p1*ss[50] + 10080*c1p1*c3p3*c5p3*c7p1*c9p1*ss[48] + 7560*c1p1*c3p4*c5p1*c7p1*c9p2*ss[50] + 7560*c1p1*c3p4*c5p1*c7p2*c9p1*ss[48] + 7560*c1p1*c3p4*c5p2*c7p1*c9p1*ss[46] + 3024*c1p1*c3p5*c5p1*c7p1*c9p1*ss[44] + 7560*c1p2*c3p1*c5p1*c7p1*c9p4*ss[60] + 15120*c1p2*c3p1*c5p1*c7p2*c9p3*ss[58] + 15120*c1p2*c3p1*c5p1*c7p3*c9p2*ss[56] + 7560*c1p2*c3p1*c5p1*c7p4*c9p1*ss[54] + 15120*c1p2*c3p1*c5p2*c7p1*c9p3*ss[56] + 22680*c1p2*c3p1*c5p2*c7p2*c9p2*ss[54] + 15120*c1p2*c3p1*c5p2*c7p3*c9p1*ss[52] + 15120*c1p2*c3p1*c5p3*c7p1*c9p2*ss[52] + 15120*c1p2*c3p1*c5p3*c7p2*c9p1*ss[50] + 7560*c1p2*c3p1*c5p4*c7p1*c9p1*ss[48] + 15120*c1p2*c3p2*c5p1*c7p1*c9p3*ss[54] + 22680*c1p2*c3p2*c5p1*c7p2*c9p2*ss[52] + 15120*c1p2*c3p2*c5p1*c7p3*c9p1*ss[50] + 22680*c1p2*c3p2*c5p2*c7p1*c9p2*ss[50] + 22680*c1p2*c3p2*c5p2*c7p2*c9p1*ss[48] + 15120*c1p2*c3p2*c5p3*c7p1*c9p1*ss[46] + 15120*c1p2*c3p3*c5p1*c7p1*c9p2*ss[48] + 15120*c1p2*c3p3*c5p1*c7p2*c9p1*ss[46] + 15120*c1p2*c3p3*c5p2*c7p1*c9p1*ss[44] + 7560*c1p2*c3p4*c5p1*c7p1*c9p1*ss[42] + 10080*c1p3*c3p1*c5p1*c7p1*c9p3*ss[52] + 15120*c1p3*c3p1*c5p1*c7p2*c9p2*ss[50] + 10080*c1p3*c3p1*c5p1*c7p3*c9p1*ss[48] + 15120*c1p3*c3p1*c5p2*c7p1*c9p2*ss[48] + 15120*c1p3*c3p1*c5p2*c7p2*c9p1*ss[46] + 10080*c1p3*c3p1*c5p3*c7p1*c9p1*ss[44] + 15120*c1p3*c3p2*c5p1*c7p1*c9p2*ss[46] + 15120*c1p3*c3p2*c5p1*c7p2*c9p1*ss[44] + 15120*c1p3*c3p2*c5p2*c7p1*c9p1*ss[42] + 10080*c1p3*c3p3*c5p1*c7p1*c9p1*ss[40] + 7560*c1p4*c3p1*c5p1*c7p1*c9p2*ss[44] + 7560*c1p4*c3p1*c5p1*c7p2*c9p1*ss[42] + 7560*c1p4*c3p1*c5p2*c7p1*c9p1*ss[40] + 7560*c1p4*c3p2*c5p1*c7p1*c9p1*ss[38] + 3024*c1p5*c3p1*c5p1*c7p1*c9p1*ss[36] + 504*c1p1*c3p1*c5p1*c7p6*ss[58] + 504*c1p1*c3p1*c5p1*c9p6*ss[70] + 1512*c1p1*c3p1*c5p2*c7p5*ss[56] + 1512*c1p1*c3p1*c5p2*c9p5*ss[66] + 2520*c1p1*c3p1*c5p3*c7p4*ss[54] + 2520*c1p1*c3p1*c5p3*c9p4*ss[62] + 2520*c1p1*c3p1*c5p4*c7p3*ss[52] + 2520*c1p1*c3p1*c5p4*c9p3*ss[58] + 1512*c1p1*c3p1*c5p5*c7p2*ss[50] + 1512*c1p1*c3p1*c5p5*c9p2*ss[54] + 504*c1p1*c3p1*c5p6*c7p1*ss[48] + 504*c1p1*c3p1*c5p6*c9p1*ss[50] + 504*c1p1*c3p1*c7p1*c9p6*ss[72] + 1512*c1p1*c3p1*c7p2*c9p5*ss[70] + 2520*c1p1*c3p1*c7p3*c9p4*ss[68] + 2520*c1p1*c3p1*c7p4*c9p3*ss[66] + 1512*c1p1*c3p1*c7p5*c9p2*ss[64] + 504*c1p1*c3p1*c7p6*c9p1*ss[62] + 1512*c1p1*c3p2*c5p1*c7p5*ss[54] + 1512*c1p1*c3p2*c5p1*c9p5*ss[64] + 3780*c1p1*c3p2*c5p2*c7p4*ss[52] + 3780*c1p1*c3p2*c5p2*c9p4*ss[60] + 5040*c1p1*c3p2*c5p3*c7p3*ss[50] + 5040*c1p1*c3p2*c5p3*c9p3*ss[56] + 3780*c1p1*c3p2*c5p4*c7p2*ss[48] + 3780*c1p1*c3p2*c5p4*c9p2*ss[52] + 1512*c1p1*c3p2*c5p5*c7p1*ss[46] + 1512*c1p1*c3p2*c5p5*c9p1*ss[48] + 1512*c1p1*c3p2*c7p1*c9p5*ss[66] + 3780*c1p1*c3p2*c7p2*c9p4*ss[64] + 5040*c1p1*c3p2*c7p3*c9p3*ss[62] + 3780*c1p1*c3p2*c7p4*c9p2*ss[60] + 1512*c1p1*c3p2*c7p5*c9p1*ss[58] + 2520*c1p1*c3p3*c5p1*c7p4*ss[50] + 2520*c1p1*c3p3*c5p1*c9p4*ss[58] + 5040*c1p1*c3p3*c5p2*c7p3*ss[48] + 5040*c1p1*c3p3*c5p2*c9p3*ss[54] + 5040*c1p1*c3p3*c5p3*c7p2*ss[46] + 5040*c1p1*c3p3*c5p3*c9p2*ss[50] + 2520*c1p1*c3p3*c5p4*c7p1*ss[44] + 2520*c1p1*c3p3*c5p4*c9p1*ss[46] + 2520*c1p1*c3p3*c7p1*c9p4*ss[60] + 5040*c1p1*c3p3*c7p2*c9p3*ss[58] + 5040*c1p1*c3p3*c7p3*c9p2*ss[56] + 2520*c1p1*c3p3*c7p4*c9p1*ss[54] + 2520*c1p1*c3p4*c5p1*c7p3*ss[46] + 2520*c1p1*c3p4*c5p1*c9p3*ss[52] + 3780*c1p1*c3p4*c5p2*c7p2*ss[44] + 3780*c1p1*c3p4*c5p2*c9p2*ss[48] + 2520*c1p1*c3p4*c5p3*c7p1*ss[42] + 2520*c1p1*c3p4*c5p3*c9p1*ss[44] + 2520*c1p1*c3p4*c7p1*c9p3*ss[54] + 3780*c1p1*c3p4*c7p2*c9p2*ss[52] + 2520*c1p1*c3p4*c7p3*c9p1*ss[50] + 1512*c1p1*c3p5*c5p1*c7p2*ss[42] + 1512*c1p1*c3p5*c5p1*c9p2*ss[46] + 1512*c1p1*c3p5*c5p2*c7p1*ss[40] + 1512*c1p1*c3p5*c5p2*c9p1*ss[42] + 1512*c1p1*c3p5*c7p1*c9p2*ss[48] + 1512*c1p1*c3p5*c7p2*c9p1*ss[46] + 504*c1p1*c3p6*c5p1*c7p1*ss[38] + 504*c1p1*c3p6*c5p1*c9p1*ss[40] + 504*c1p1*c3p6*c7p1*c9p1*ss[42] + 504*c1p1*c5p1*c7p1*c9p6*ss[74] + 1512*c1p1*c5p1*c7p2*c9p5*ss[72] + 2520*c1p1*c5p1*c7p3*c9p4*ss[70] + 2520*c1p1*c5p1*c7p4*c9p3*ss[68] + 1512*c1p1*c5p1*c7p5*c9p2*ss[66] + 504*c1p1*c5p1*c7p6*c9p1*ss[64] + 1512*c1p1*c5p2*c7p1*c9p5*ss[70] + 3780*c1p1*c5p2*c7p2*c9p4*ss[68] + 5040*c1p1*c5p2*c7p3*c9p3*ss[66] + 3780*c1p1*c5p2*c7p4*c9p2*ss[64] + 1512*c1p1*c5p2*c7p5*c9p1*ss[62] + 2520*c1p1*c5p3*c7p1*c9p4*ss[66] + 5040*c1p1*c5p3*c7p2*c9p3*ss[64] + 5040*c1p1*c5p3*c7p3*c9p2*ss[62] + 2520*c1p1*c5p3*c7p4*c9p1*ss[60] + 2520*c1p1*c5p4*c7p1*c9p3*ss[62] + 3780*c1p1*c5p4*c7p2*c9p2*ss[60] + 2520*c1p1*c5p4*c7p3*c9p1*ss[58] + 1512*c1p1*c5p5*c7p1*c9p2*ss[58] + 1512*c1p1*c5p5*c7p2*c9p1*ss[56] + 504*c1p1*c5p6*c7p1*c9p1*ss[54] + 1512*c1p2*c3p1*c5p1*c7p5*ss[52] + 1512*c1p2*c3p1*c5p1*c9p5*ss[62] + 3780*c1p2*c3p1*c5p2*c7p4*ss[50] + 3780*c1p2*c3p1*c5p2*c9p4*ss[58] + 5040*c1p2*c3p1*c5p3*c7p3*ss[48] + 5040*c1p2*c3p1*c5p3*c9p3*ss[54] + 3780*c1p2*c3p1*c5p4*c7p2*ss[46] + 3780*c1p2*c3p1*c5p4*c9p2*ss[50] + 1512*c1p2*c3p1*c5p5*c7p1*ss[44] + 1512*c1p2*c3p1*c5p5*c9p1*ss[46] + 1512*c1p2*c3p1*c7p1*c9p5*ss[64] + 3780*c1p2*c3p1*c7p2*c9p4*ss[62] + 5040*c1p2*c3p1*c7p3*c9p3*ss[60] + 3780*c1p2*c3p1*c7p4*c9p2*ss[58] + 1512*c1p2*c3p1*c7p5*c9p1*ss[56] + 3780*c1p2*c3p2*c5p1*c7p4*ss[48] + 3780*c1p2*c3p2*c5p1*c9p4*ss[56] + 7560*c1p2*c3p2*c5p2*c7p3*ss[46] + 7560*c1p2*c3p2*c5p2*c9p3*ss[52] + 7560*c1p2*c3p2*c5p3*c7p2*ss[44] + 7560*c1p2*c3p2*c5p3*c9p2*ss[48] + 3780*c1p2*c3p2*c5p4*c7p1*ss[42] + 3780*c1p2*c3p2*c5p4*c9p1*ss[44] + 3780*c1p2*c3p2*c7p1*c9p4*ss[58] + 7560*c1p2*c3p2*c7p2*c9p3*ss[56] + 7560*c1p2*c3p2*c7p3*c9p2*ss[54] + 3780*c1p2*c3p2*c7p4*c9p1*ss[52] + 5040*c1p2*c3p3*c5p1*c7p3*ss[44] + 5040*c1p2*c3p3*c5p1*c9p3*ss[50] + 7560*c1p2*c3p3*c5p2*c7p2*ss[42] + 7560*c1p2*c3p3*c5p2*c9p2*ss[46] + 5040*c1p2*c3p3*c5p3*c7p1*ss[40] + 5040*c1p2*c3p3*c5p3*c9p1*ss[42] + 5040*c1p2*c3p3*c7p1*c9p3*ss[52] + 7560*c1p2*c3p3*c7p2*c9p2*ss[50] + 5040*c1p2*c3p3*c7p3*c9p1*ss[48] + 3780*c1p2*c3p4*c5p1*c7p2*ss[40] + 3780*c1p2*c3p4*c5p1*c9p2*ss[44] + 3780*c1p2*c3p4*c5p2*c7p1*ss[38] + 3780*c1p2*c3p4*c5p2*c9p1*ss[40] + 3780*c1p2*c3p4*c7p1*c9p2*ss[46] + 3780*c1p2*c3p4*c7p2*c9p1*ss[44] + 1512*c1p2*c3p5*c5p1*c7p1*ss[36] + 1512*c1p2*c3p5*c5p1*c9p1*ss[38] + 1512*c1p2*c3p5*c7p1*c9p1*ss[40] + 1512*c1p2*c5p1*c7p1*c9p5*ss[66] + 3780*c1p2*c5p1*c7p2*c9p4*ss[64] + 5040*c1p2*c5p1*c7p3*c9p3*ss[62] + 3780*c1p2*c5p1*c7p4*c9p2*ss[60] + 1512*c1p2*c5p1*c7p5*c9p1*ss[58] + 3780*c1p2*c5p2*c7p1*c9p4*ss[62] + 7560*c1p2*c5p2*c7p2*c9p3*ss[60] + 7560*c1p2*c5p2*c7p3*c9p2*ss[58] + 3780*c1p2*c5p2*c7p4*c9p1*ss[56] + 5040*c1p2*c5p3*c7p1*c9p3*ss[58] + 7560*c1p2*c5p3*c7p2*c9p2*ss[56] + 5040*c1p2*c5p3*c7p3*c9p1*ss[54] + 3780*c1p2*c5p4*c7p1*c9p2*ss[54] + 3780*c1p2*c5p4*c7p2*c9p1*ss[52] + 1512*c1p2*c5p5*c7p1*c9p1*ss[50] + 2520*c1p3*c3p1*c5p1*c7p4*ss[46] + 2520*c1p3*c3p1*c5p1*c9p4*ss[54] + 5040*c1p3*c3p1*c5p2*c7p3*ss[44] + 5040*c1p3*c3p1*c5p2*c9p3*ss[50] + 5040*c1p3*c3p1*c5p3*c7p2*ss[42] + 5040*c1p3*c3p1*c5p3*c9p2*ss[46] + 2520*c1p3*c3p1*c5p4*c7p1*ss[40] + 2520*c1p3*c3p1*c5p4*c9p1*ss[42] + 2520*c1p3*c3p1*c7p1*c9p4*ss[56] + 5040*c1p3*c3p1*c7p2*c9p3*ss[54] + 5040*c1p3*c3p1*c7p3*c9p2*ss[52] + 2520*c1p3*c3p1*c7p4*c9p1*ss[50] + 5040*c1p3*c3p2*c5p1*c7p3*ss[42] + 5040*c1p3*c3p2*c5p1*c9p3*ss[48] + 7560*c1p3*c3p2*c5p2*c7p2*ss[40] + 7560*c1p3*c3p2*c5p2*c9p2*ss[44] + 5040*c1p3*c3p2*c5p3*c7p1*ss[38] + 5040*c1p3*c3p2*c5p3*c9p1*ss[40] + 5040*c1p3*c3p2*c7p1*c9p3*ss[50] + 7560*c1p3*c3p2*c7p2*c9p2*ss[48] + 5040*c1p3*c3p2*c7p3*c9p1*ss[46] + 5040*c1p3*c3p3*c5p1*c7p2*ss[38] + 5040*c1p3*c3p3*c5p1*c9p2*ss[42] + 5040*c1p3*c3p3*c5p2*c7p1*ss[36] + 5040*c1p3*c3p3*c5p2*c9p1*ss[38] + 5040*c1p3*c3p3*c7p1*c9p2*ss[44] + 5040*c1p3*c3p3*c7p2*c9p1*ss[42] + 2520*c1p3*c3p4*c5p1*c7p1*ss[34] + 2520*c1p3*c3p4*c5p1*c9p1*ss[36] + 2520*c1p3*c3p4*c7p1*c9p1*ss[38] + 2520*c1p3*c5p1*c7p1*c9p4*ss[58] + 5040*c1p3*c5p1*c7p2*c9p3*ss[56] + 5040*c1p3*c5p1*c7p3*c9p2*ss[54] + 2520*c1p3*c5p1*c7p4*c9p1*ss[52] + 5040*c1p3*c5p2*c7p1*c9p3*ss[54] + 7560*c1p3*c5p2*c7p2*c9p2*ss[52] + 5040*c1p3*c5p2*c7p3*c9p1*ss[50] + 5040*c1p3*c5p3*c7p1*c9p2*ss[50] + 5040*c1p3*c5p3*c7p2*c9p1*ss[48] + 2520*c1p3*c5p4*c7p1*c9p1*ss[46] + 2520*c1p4*c3p1*c5p1*c7p3*ss[40] + 2520*c1p4*c3p1*c5p1*c9p3*ss[46] + 3780*c1p4*c3p1*c5p2*c7p2*ss[38] + 3780*c1p4*c3p1*c5p2*c9p2*ss[42] + 2520*c1p4*c3p1*c5p3*c7p1*ss[36] + 2520*c1p4*c3p1*c5p3*c9p1*ss[38] + 2520*c1p4*c3p1*c7p1*c9p3*ss[48] + 3780*c1p4*c3p1*c7p2*c9p2*ss[46] + 2520*c1p4*c3p1*c7p3*c9p1*ss[44] + 3780*c1p4*c3p2*c5p1*c7p2*ss[36] + 3780*c1p4*c3p2*c5p1*c9p2*ss[40] + 3780*c1p4*c3p2*c5p2*c7p1*ss[34] + 3780*c1p4*c3p2*c5p2*c9p1*ss[36] + 3780*c1p4*c3p2*c7p1*c9p2*ss[42] + 3780*c1p4*c3p2*c7p2*c9p1*ss[40] + 2520*c1p4*c3p3*c5p1*c7p1*ss[32] + 2520*c1p4*c3p3*c5p1*c9p1*ss[34] + 2520*c1p4*c3p3*c7p1*c9p1*ss[36] + 2520*c1p4*c5p1*c7p1*c9p3*ss[50] + 3780*c1p4*c5p1*c7p2*c9p2*ss[48] + 2520*c1p4*c5p1*c7p3*c9p1*ss[46] + 3780*c1p4*c5p2*c7p1*c9p2*ss[46] + 3780*c1p4*c5p2*c7p2*c9p1*ss[44] + 2520*c1p4*c5p3*c7p1*c9p1*ss[42] + 1512*c1p5*c3p1*c5p1*c7p2*ss[34] + 1512*c1p5*c3p1*c5p1*c9p2*ss[38] + 1512*c1p5*c3p1*c5p2*c7p1*ss[32] + 1512*c1p5*c3p1*c5p2*c9p1*ss[34] + 1512*c1p5*c3p1*c7p1*c9p2*ss[40] + 1512*c1p5*c3p1*c7p2*c9p1*ss[38] + 1512*c1p5*c3p2*c5p1*c7p1*ss[30] + 1512*c1p5*c3p2*c5p1*c9p1*ss[32] + 1512*c1p5*c3p2*c7p1*c9p1*ss[34] + 1512*c1p5*c5p1*c7p1*c9p2*ss[42] + 1512*c1p5*c5p1*c7p2*c9p1*ss[40] + 1512*c1p5*c5p2*c7p1*c9p1*ss[38] + 504*c1p6*c3p1*c5p1*c7p1*ss[28] + 504*c1p6*c3p1*c5p1*c9p1*ss[30] + 504*c1p6*c3p1*c7p1*c9p1*ss[32] + 504*c1p6*c5p1*c7p1*c9p1*ss[34] + 504*c3p1*c5p1*c7p1*c9p6*ss[76] + 1512*c3p1*c5p1*c7p2*c9p5*ss[74] + 2520*c3p1*c5p1*c7p3*c9p4*ss[72] + 2520*c3p1*c5p1*c7p4*c9p3*ss[70] + 1512*c3p1*c5p1*c7p5*c9p2*ss[68] + 504*c3p1*c5p1*c7p6*c9p1*ss[66] + 1512*c3p1*c5p2*c7p1*c9p5*ss[72] + 3780*c3p1*c5p2*c7p2*c9p4*ss[70] + 5040*c3p1*c5p2*c7p3*c9p3*ss[68] + 3780*c3p1*c5p2*c7p4*c9p2*ss[66] + 1512*c3p1*c5p2*c7p5*c9p1*ss[64] + 2520*c3p1*c5p3*c7p1*c9p4*ss[68] + 5040*c3p1*c5p3*c7p2*c9p3*ss[66] + 5040*c3p1*c5p3*c7p3*c9p2*ss[64] + 2520*c3p1*c5p3*c7p4*c9p1*ss[62] + 2520*c3p1*c5p4*c7p1*c9p3*ss[64] + 3780*c3p1*c5p4*c7p2*c9p2*ss[62] + 2520*c3p1*c5p4*c7p3*c9p1*ss[60] + 1512*c3p1*c5p5*c7p1*c9p2*ss[60] + 1512*c3p1*c5p5*c7p2*c9p1*ss[58] + 504*c3p1*c5p6*c7p1*c9p1*ss[56] + 1512*c3p2*c5p1*c7p1*c9p5*ss[70] + 3780*c3p2*c5p1*c7p2*c9p4*ss[68] + 5040*c3p2*c5p1*c7p3*c9p3*ss[66] + 3780*c3p2*c5p1*c7p4*c9p2*ss[64] + 1512*c3p2*c5p1*c7p5*c9p1*ss[62] + 3780*c3p2*c5p2*c7p1*c9p4*ss[66] + 7560*c3p2*c5p2*c7p2*c9p3*ss[64] + 7560*c3p2*c5p2*c7p3*c9p2*ss[62] + 3780*c3p2*c5p2*c7p4*c9p1*ss[60] + 5040*c3p2*c5p3*c7p1*c9p3*ss[62] + 7560*c3p2*c5p3*c7p2*c9p2*ss[60] + 5040*c3p2*c5p3*c7p3*c9p1*ss[58] + 3780*c3p2*c5p4*c7p1*c9p2*ss[58] + 3780*c3p2*c5p4*c7p2*c9p1*ss[56] + 1512*c3p2*c5p5*c7p1*c9p1*ss[54] + 2520*c3p3*c5p1*c7p1*c9p4*ss[64] + 5040*c3p3*c5p1*c7p2*c9p3*ss[62] + 5040*c3p3*c5p1*c7p3*c9p2*ss[60] + 2520*c3p3*c5p1*c7p4*c9p1*ss[58] + 5040*c3p3*c5p2*c7p1*c9p3*ss[60] + 7560*c3p3*c5p2*c7p2*c9p2*ss[58] + 5040*c3p3*c5p2*c7p3*c9p1*ss[56] + 5040*c3p3*c5p3*c7p1*c9p2*ss[56] + 5040*c3p3*c5p3*c7p2*c9p1*ss[54] + 2520*c3p3*c5p4*c7p1*c9p1*ss[52] + 2520*c3p4*c5p1*c7p1*c9p3*ss[58] + 3780*c3p4*c5p1*c7p2*c9p2*ss[56] + 2520*c3p4*c5p1*c7p3*c9p1*ss[54] + 3780*c3p4*c5p2*c7p1*c9p2*ss[54] + 3780*c3p4*c5p2*c7p2*c9p1*ss[52] + 2520*c3p4*c5p3*c7p1*c9p1*ss[50] + 1512*c3p5*c5p1*c7p1*c9p2*ss[52] + 1512*c3p5*c5p1*c7p2*c9p1*ss[50] + 1512*c3p5*c5p2*c7p1*c9p1*ss[48] + 504*c3p6*c5p1*c7p1*c9p1*ss[46] + 72*c1p1*c3p1*c5p7*ss[46] + 72*c1p1*c3p1*c7p7*ss[60] + 72*c1p1*c3p1*c9p7*ss[74] + 252*c1p1*c3p2*c5p6*ss[44] + 252*c1p1*c3p2*c7p6*ss[56] + 252*c1p1*c3p2*c9p6*ss[68] + 504*c1p1*c3p3*c5p5*ss[42] + 504*c1p1*c3p3*c7p5*ss[52] + 504*c1p1*c3p3*c9p5*ss[62] + 630*c1p1*c3p4*c5p4*ss[40] + 630*c1p1*c3p4*c7p4*ss[48] + 630*c1p1*c3p4*c9p4*ss[56] + 504*c1p1*c3p5*c5p3*ss[38] + 504*c1p1*c3p5*c7p3*ss[44] + 504*c1p1*c3p5*c9p3*ss[50] + 252*c1p1*c3p6*c5p2*ss[36] + 252*c1p1*c3p6*c7p2*ss[40] + 252*c1p1*c3p6*c9p2*ss[44] + 72*c1p1*c3p7*c5p1*ss[34] + 72*c1p1*c3p7*c7p1*ss[36] + 72*c1p1*c3p7*c9p1*ss[38] + 72*c1p1*c5p1*c7p7*ss[62] + 72*c1p1*c5p1*c9p7*ss[76] + 252*c1p1*c5p2*c7p6*ss[60] + 252*c1p1*c5p2*c9p6*ss[72] + 504*c1p1*c5p3*c7p5*ss[58] + 504*c1p1*c5p3*c9p5*ss[68] + 630*c1p1*c5p4*c7p4*ss[56] + 630*c1p1*c5p4*c9p4*ss[64] + 504*c1p1*c5p5*c7p3*ss[54] + 504*c1p1*c5p5*c9p3*ss[60] + 252*c1p1*c5p6*c7p2*ss[52] + 252*c1p1*c5p6*c9p2*ss[56] + 72*c1p1*c5p7*c7p1*ss[50] + 72*c1p1*c5p7*c9p1*ss[52] + 72*c1p1*c7p1*c9p7*ss[78] + 252*c1p1*c7p2*c9p6*ss[76] + 504*c1p1*c7p3*c9p5*ss[74] + 630*c1p1*c7p4*c9p4*ss[72] + 504*c1p1*c7p5*c9p3*ss[70] + 252*c1p1*c7p6*c9p2*ss[68] + 72*c1p1*c7p7*c9p1*ss[66] + 252*c1p2*c3p1*c5p6*ss[42] + 252*c1p2*c3p1*c7p6*ss[54] + 252*c1p2*c3p1*c9p6*ss[66] + 756*c1p2*c3p2*c5p5*ss[40] + 756*c1p2*c3p2*c7p5*ss[50] + 756*c1p2*c3p2*c9p5*ss[60] + 1260*c1p2*c3p3*c5p4*ss[38] + 1260*c1p2*c3p3*c7p4*ss[46] + 1260*c1p2*c3p3*c9p4*ss[54] + 1260*c1p2*c3p4*c5p3*ss[36] + 1260*c1p2*c3p4*c7p3*ss[42] + 1260*c1p2*c3p4*c9p3*ss[48] + 756*c1p2*c3p5*c5p2*ss[34] + 756*c1p2*c3p5*c7p2*ss[38] + 756*c1p2*c3p5*c9p2*ss[42] + 252*c1p2*c3p6*c5p1*ss[32] + 252*c1p2*c3p6*c7p1*ss[34] + 252*c1p2*c3p6*c9p1*ss[36] + 252*c1p2*c5p1*c7p6*ss[56] + 252*c1p2*c5p1*c9p6*ss[68] + 756*c1p2*c5p2*c7p5*ss[54] + 756*c1p2*c5p2*c9p5*ss[64] + 1260*c1p2*c5p3*c7p4*ss[52] + 1260*c1p2*c5p3*c9p4*ss[60] + 1260*c1p2*c5p4*c7p3*ss[50] + 1260*c1p2*c5p4*c9p3*ss[56] + 756*c1p2*c5p5*c7p2*ss[48] + 756*c1p2*c5p5*c9p2*ss[52] + 252*c1p2*c5p6*c7p1*ss[46] + 252*c1p2*c5p6*c9p1*ss[48] + 252*c1p2*c7p1*c9p6*ss[70] + 756*c1p2*c7p2*c9p5*ss[68] + 1260*c1p2*c7p3*c9p4*ss[66] + 1260*c1p2*c7p4*c9p3*ss[64] + 756*c1p2*c7p5*c9p2*ss[62] + 252*c1p2*c7p6*c9p1*ss[60] + 504*c1p3*c3p1*c5p5*ss[38] + 504*c1p3*c3p1*c7p5*ss[48] + 504*c1p3*c3p1*c9p5*ss[58] + 1260*c1p3*c3p2*c5p4*ss[36] + 1260*c1p3*c3p2*c7p4*ss[44] + 1260*c1p3*c3p2*c9p4*ss[52] + 1680*c1p3*c3p3*c5p3*ss[34] + 1680*c1p3*c3p3*c7p3*ss[40] + 1680*c1p3*c3p3*c9p3*ss[46] + 1260*c1p3*c3p4*c5p2*ss[32] + 1260*c1p3*c3p4*c7p2*ss[36] + 1260*c1p3*c3p4*c9p2*ss[40] + 504*c1p3*c3p5*c5p1*ss[30] + 504*c1p3*c3p5*c7p1*ss[32] + 504*c1p3*c3p5*c9p1*ss[34] + 504*c1p3*c5p1*c7p5*ss[50] + 504*c1p3*c5p1*c9p5*ss[60] + 1260*c1p3*c5p2*c7p4*ss[48] + 1260*c1p3*c5p2*c9p4*ss[56] + 1680*c1p3*c5p3*c7p3*ss[46] + 1680*c1p3*c5p3*c9p3*ss[52] + 1260*c1p3*c5p4*c7p2*ss[44] + 1260*c1p3*c5p4*c9p2*ss[48] + 504*c1p3*c5p5*c7p1*ss[42] + 504*c1p3*c5p5*c9p1*ss[44] + 504*c1p3*c7p1*c9p5*ss[62] + 1260*c1p3*c7p2*c9p4*ss[60] + 1680*c1p3*c7p3*c9p3*ss[58] + 1260*c1p3*c7p4*c9p2*ss[56] + 504*c1p3*c7p5*c9p1*ss[54] + 630*c1p4*c3p1*c5p4*ss[34] + 630*c1p4*c3p1*c7p4*ss[42] + 630*c1p4*c3p1*c9p4*ss[50] + 1260*c1p4*c3p2*c5p3*ss[32] + 1260*c1p4*c3p2*c7p3*ss[38] + 1260*c1p4*c3p2*c9p3*ss[44] + 1260*c1p4*c3p3*c5p2*ss[30] + 1260*c1p4*c3p3*c7p2*ss[34] + 1260*c1p4*c3p3*c9p2*ss[38] + 630*c1p4*c3p4*c5p1*ss[28] + 630*c1p4*c3p4*c7p1*ss[30] + 630*c1p4*c3p4*c9p1*ss[32] + 630*c1p4*c5p1*c7p4*ss[44] + 630*c1p4*c5p1*c9p4*ss[52] + 1260*c1p4*c5p2*c7p3*ss[42] + 1260*c1p4*c5p2*c9p3*ss[48] + 1260*c1p4*c5p3*c7p2*ss[40] + 1260*c1p4*c5p3*c9p2*ss[44] + 630*c1p4*c5p4*c7p1*ss[38] + 630*c1p4*c5p4*c9p1*ss[40] + 630*c1p4*c7p1*c9p4*ss[54] + 1260*c1p4*c7p2*c9p3*ss[52] + 1260*c1p4*c7p3*c9p2*ss[50] + 630*c1p4*c7p4*c9p1*ss[48] + 504*c1p5*c3p1*c5p3*ss[30] + 504*c1p5*c3p1*c7p3*ss[36] + 504*c1p5*c3p1*c9p3*ss[42] + 756*c1p5*c3p2*c5p2*ss[28] + 756*c1p5*c3p2*c7p2*ss[32] + 756*c1p5*c3p2*c9p2*ss[36] + 504*c1p5*c3p3*c5p1*ss[26] + 504*c1p5*c3p3*c7p1*ss[28] + 504*c1p5*c3p3*c9p1*ss[30] + 504*c1p5*c5p1*c7p3*ss[38] + 504*c1p5*c5p1*c9p3*ss[44] + 756*c1p5*c5p2*c7p2*ss[36] + 756*c1p5*c5p2*c9p2*ss[40] + 504*c1p5*c5p3*c7p1*ss[34] + 504*c1p5*c5p3*c9p1*ss[36] + 504*c1p5*c7p1*c9p3*ss[46] + 756*c1p5*c7p2*c9p2*ss[44] + 504*c1p5*c7p3*c9p1*ss[42] + 252*c1p6*c3p1*c5p2*ss[26] + 252*c1p6*c3p1*c7p2*ss[30] + 252*c1p6*c3p1*c9p2*ss[34] + 252*c1p6*c3p2*c5p1*ss[24] + 252*c1p6*c3p2*c7p1*ss[26] + 252*c1p6*c3p2*c9p1*ss[28] + 252*c1p6*c5p1*c7p2*ss[32] + 252*c1p6*c5p1*c9p2*ss[36] + 252*c1p6*c5p2*c7p1*ss[30] + 252*c1p6*c5p2*c9p1*ss[32] + 252*c1p6*c7p1*c9p2*ss[38] + 252*c1p6*c7p2*c9p1*ss[36] + 72*c1p7*c3p1*c5p1*ss[22] + 72*c1p7*c3p1*c7p1*ss[24] + 72*c1p7*c3p1*c9p1*ss[26] + 72*c1p7*c5p1*c7p1*ss[26] + 72*c1p7*c5p1*c9p1*ss[28] + 72*c1p7*c7p1*c9p1*ss[30] + 72*c3p1*c5p1*c7p7*ss[64] + 72*c3p1*c5p1*c9p7*ss[78] + 252*c3p1*c5p2*c7p6*ss[62] + 252*c3p1*c5p2*c9p6*ss[74] + 504*c3p1*c5p3*c7p5*ss[60] + 504*c3p1*c5p3*c9p5*ss[70] + 630*c3p1*c5p4*c7p4*ss[58] + 630*c3p1*c5p4*c9p4*ss[66] + 504*c3p1*c5p5*c7p3*ss[56] + 504*c3p1*c5p5*c9p3*ss[62] + 252*c3p1*c5p6*c7p2*ss[54] + 252*c3p1*c5p6*c9p2*ss[58] + 72*c3p1*c5p7*c7p1*ss[52] + 72*c3p1*c5p7*c9p1*ss[54] + 72*c3p1*c7p1*c9p7*ss[80] + 252*c3p1*c7p2*c9p6*ss[78] + 504*c3p1*c7p3*c9p5*ss[76] + 630*c3p1*c7p4*c9p4*ss[74] + 504*c3p1*c7p5*c9p3*ss[72] + 252*c3p1*c7p6*c9p2*ss[70] + 72*c3p1*c7p7*c9p1*ss[68] + 252*c3p2*c5p1*c7p6*ss[60] + 252*c3p2*c5p1*c9p6*ss[72] + 756*c3p2*c5p2*c7p5*ss[58] + 756*c3p2*c5p2*c9p5*ss[68] + 1260*c3p2*c5p3*c7p4*ss[56] + 1260*c3p2*c5p3*c9p4*ss[64] + 1260*c3p2*c5p4*c7p3*ss[54] + 1260*c3p2*c5p4*c9p3*ss[60] + 756*c3p2*c5p5*c7p2*ss[52] + 756*c3p2*c5p5*c9p2*ss[56] + 252*c3p2*c5p6*c7p1*ss[50] + 252*c3p2*c5p6*c9p1*ss[52] + 252*c3p2*c7p1*c9p6*ss[74] + 756*c3p2*c7p2*c9p5*ss[72] + 1260*c3p2*c7p3*c9p4*ss[70] + 1260*c3p2*c7p4*c9p3*ss[68] + 756*c3p2*c7p5*c9p2*ss[66] + 252*c3p2*c7p6*c9p1*ss[64] + 504*c3p3*c5p1*c7p5*ss[56] + 504*c3p3*c5p1*c9p5*ss[66] + 1260*c3p3*c5p2*c7p4*ss[54] + 1260*c3p3*c5p2*c9p4*ss[62] + 1680*c3p3*c5p3*c7p3*ss[52] + 1680*c3p3*c5p3*c9p3*ss[58] + 1260*c3p3*c5p4*c7p2*ss[50] + 1260*c3p3*c5p4*c9p2*ss[54] + 504*c3p3*c5p5*c7p1*ss[48] + 504*c3p3*c5p5*c9p1*ss[50] + 504*c3p3*c7p1*c9p5*ss[68] + 1260*c3p3*c7p2*c9p4*ss[66] + 1680*c3p3*c7p3*c9p3*ss[64] + 1260*c3p3*c7p4*c9p2*ss[62] + 504*c3p3*c7p5*c9p1*ss[60] + 630*c3p4*c5p1*c7p4*ss[52] + 630*c3p4*c5p1*c9p4*ss[60] + 1260*c3p4*c5p2*c7p3*ss[50] + 1260*c3p4*c5p2*c9p3*ss[56] + 1260*c3p4*c5p3*c7p2*ss[48] + 1260*c3p4*c5p3*c9p2*ss[52] + 630*c3p4*c5p4*c7p1*ss[46] + 630*c3p4*c5p4*c9p1*ss[48] + 630*c3p4*c7p1*c9p4*ss[62] + 1260*c3p4*c7p2*c9p3*ss[60] + 1260*c3p4*c7p3*c9p2*ss[58] + 630*c3p4*c7p4*c9p1*ss[56] + 504*c3p5*c5p1*c7p3*ss[48] + 504*c3p5*c5p1*c9p3*ss[54] + 756*c3p5*c5p2*c7p2*ss[46] + 756*c3p5*c5p2*c9p2*ss[50] + 504*c3p5*c5p3*c7p1*ss[44] + 504*c3p5*c5p3*c9p1*ss[46] + 504*c3p5*c7p1*c9p3*ss[56] + 756*c3p5*c7p2*c9p2*ss[54] + 504*c3p5*c7p3*c9p1*ss[52] + 252*c3p6*c5p1*c7p2*ss[44] + 252*c3p6*c5p1*c9p2*ss[48] + 252*c3p6*c5p2*c7p1*ss[42] + 252*c3p6*c5p2*c9p1*ss[44] + 252*c3p6*c7p1*c9p2*ss[50] + 252*c3p6*c7p2*c9p1*ss[48] + 72*c3p7*c5p1*c7p1*ss[40] + 72*c3p7*c5p1*c9p1*ss[42] + 72*c3p7*c7p1*c9p1*ss[44] + 72*c5p1*c7p1*c9p7*ss[82] + 252*c5p1*c7p2*c9p6*ss[80] + 504*c5p1*c7p3*c9p5*ss[78] + 630*c5p1*c7p4*c9p4*ss[76] + 504*c5p1*c7p5*c9p3*ss[74] + 252*c5p1*c7p6*c9p2*ss[72] + 72*c5p1*c7p7*c9p1*ss[70] + 252*c5p2*c7p1*c9p6*ss[78] + 756*c5p2*c7p2*c9p5*ss[76] + 1260*c5p2*c7p3*c9p4*ss[74] + 1260*c5p2*c7p4*c9p3*ss[72] + 756*c5p2*c7p5*c9p2*ss[70] + 252*c5p2*c7p6*c9p1*ss[68] + 504*c5p3*c7p1*c9p5*ss[74] + 1260*c5p3*c7p2*c9p4*ss[72] + 1680*c5p3*c7p3*c9p3*ss[70] + 1260*c5p3*c7p4*c9p2*ss[68] + 504*c5p3*c7p5*c9p1*ss[66] + 630*c5p4*c7p1*c9p4*ss[70] + 1260*c5p4*c7p2*c9p3*ss[68] + 1260*c5p4*c7p3*c9p2*ss[66] + 630*c5p4*c7p4*c9p1*ss[64] + 504*c5p5*c7p1*c9p3*ss[66] + 756*c5p5*c7p2*c9p2*ss[64] + 504*c5p5*c7p3*c9p1*ss[62] + 252*c5p6*c7p1*c9p2*ss[62] + 252*c5p6*c7p2*c9p1*ss[60] + 72*c5p7*c7p1*c9p1*ss[58] + 9*c1p1*c3p8*ss[32] + 9*c1p1*c5p8*ss[48] + 9*c1p1*c7p8*ss[64] + 9*c1p1*c9p8*ss[80] + 36*c1p2*c3p7*ss[30] + 36*c1p2*c5p7*ss[44] + 36*c1p2*c7p7*ss[58] + 36*c1p2*c9p7*ss[72] + 84*c1p3*c3p6*ss[28] + 84*c1p3*c5p6*ss[40] + 84*c1p3*c7p6*ss[52] + 84*c1p3*c9p6*ss[64] + 126*c1p4*c3p5*ss[26] + 126*c1p4*c5p5*ss[36] + 126*c1p4*c7p5*ss[46] + 126*c1p4*c9p5*ss[56] + 126*c1p5*c3p4*ss[24] + 126*c1p5*c5p4*ss[32] + 126*c1p5*c7p4*ss[40] + 126*c1p5*c9p4*ss[48] + 84*c1p6*c3p3*ss[22] + 84*c1p6*c5p3*ss[28] + 84*c1p6*c7p3*ss[34] + 84*c1p6*c9p3*ss[40] + 36*c1p7*c3p2*ss[20] + 36*c1p7*c5p2*ss[24] + 36*c1p7*c7p2*ss[28] + 36*c1p7*c9p2*ss[32] + 9*c1p8*c3p1*ss[18] + 9*c1p8*c5p1*ss[20] + 9*c1p8*c7p1*ss[22] + 9*c1p8*c9p1*ss[24] + 9*c3p1*c5p8*ss[50] + 9*c3p1*c7p8*ss[66] + 9*c3p1*c9p8*ss[82] + 36*c3p2*c5p7*ss[48] + 36*c3p2*c7p7*ss[62] + 36*c3p2*c9p7*ss[76] + 84*c3p3*c5p6*ss[46] + 84*c3p3*c7p6*ss[58] + 84*c3p3*c9p6*ss[70] + 126*c3p4*c5p5*ss[44] + 126*c3p4*c7p5*ss[54] + 126*c3p4*c9p5*ss[64] + 126*c3p5*c5p4*ss[42] + 126*c3p5*c7p4*ss[50] + 126*c3p5*c9p4*ss[58] + 84*c3p6*c5p3*ss[40] + 84*c3p6*c7p3*ss[46] + 84*c3p6*c9p3*ss[52] + 36*c3p7*c5p2*ss[38] + 36*c3p7*c7p2*ss[42] + 36*c3p7*c9p2*ss[46] + 9*c3p8*c5p1*ss[36] + 9*c3p8*c7p1*ss[38] + 9*c3p8*c9p1*ss[40] + 9*c5p1*c7p8*ss[68] + 9*c5p1*c9p8*ss[84] + 36*c5p2*c7p7*ss[66] + 36*c5p2*c9p7*ss[80] + 84*c5p3*c7p6*ss[64] + 84*c5p3*c9p6*ss[76] + 126*c5p4*c7p5*ss[62] + 126*c5p4*c9p5*ss[72] + 126*c5p5*c7p4*ss[60] + 126*c5p5*c9p4*ss[68] + 84*c5p6*c7p3*ss[58] + 84*c5p6*c9p3*ss[64] + 36*c5p7*c7p2*ss[56] + 36*c5p7*c9p2*ss[60] + 9*c5p8*c7p1*ss[54] + 9*c5p8*c9p1*ss[56] + 9*c7p1*c9p8*ss[86] + 36*c7p2*c9p7*ss[84] + 84*c7p3*c9p6*ss[82] + 126*c7p4*c9p5*ss[80] + 126*c7p5*c9p4*ss[78] + 84*c7p6*c9p3*ss[76] + 36*c7p7*c9p2*ss[74] + 9*c7p8*c9p1*ss[72] + c1p9*ss[16] + c3p9*ss[34] + c5p9*ss[52] + c7p9*ss[70] + c9p9*ss[88])*sys_part_4;
  TACC grad_3_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4)));
  TACC grad_4_num = convert<TACC>(2*(c1p1*ss[10] + c3p1*ss[12] + c5p1*ss[14] + c7p1*ss[16] + c9p1*ss[18])*sys_part_0);
  grad_4_num += 4*(6*c1p1*c3p1*c5p1*ss[18] + 6*c1p1*c3p1*c7p1*ss[20] + 6*c1p1*c3p1*c9p1*ss[22] + 6*c1p1*c5p1*c7p1*ss[22] + 6*c1p1*c5p1*c9p1*ss[24] + 6*c1p1*c7p1*c9p1*ss[26] + 6*c3p1*c5p1*c7p1*ss[24] + 6*c3p1*c5p1*c9p1*ss[26] + 6*c3p1*c7p1*c9p1*ss[28] + 6*c5p1*c7p1*c9p1*ss[30] + 3*c1p1*c3p2*ss[16] + 3*c1p1*c5p2*ss[20] + 3*c1p1*c7p2*ss[24] + 3*c1p1*c9p2*ss[28] + 3*c1p2*c3p1*ss[14] + 3*c1p2*c5p1*ss[16] + 3*c1p2*c7p1*ss[18] + 3*c1p2*c9p1*ss[20] + 3*c3p1*c5p2*ss[22] + 3*c3p1*c7p2*ss[26] + 3*c3p1*c9p2*ss[30] + 3*c3p2*c5p1*ss[20] + 3*c3p2*c7p1*ss[22] + 3*c3p2*c9p1*ss[24] + 3*c5p1*c7p2*ss[28] + 3*c5p1*c9p2*ss[32] + 3*c5p2*c7p1*ss[26] + 3*c5p2*c9p1*ss[28] + 3*c7p1*c9p2*ss[34] + 3*c7p2*c9p1*ss[32] + c1p3*ss[12] + c3p3*ss[18] + c5p3*ss[24] + c7p3*ss[30] + c9p3*ss[36])*sys_part_1;
  grad_4_num += 6*(120*c1p1*c3p1*c5p1*c7p1*c9p1*ss[34] + 60*c1p1*c3p1*c5p1*c7p2*ss[32] + 60*c1p1*c3p1*c5p1*c9p2*ss[36] + 60*c1p1*c3p1*c5p2*c7p1*ss[30] + 60*c1p1*c3p1*c5p2*c9p1*ss[32] + 60*c1p1*c3p1*c7p1*c9p2*ss[38] + 60*c1p1*c3p1*c7p2*c9p1*ss[36] + 60*c1p1*c3p2*c5p1*c7p1*ss[28] + 60*c1p1*c3p2*c5p1*c9p1*ss[30] + 60*c1p1*c3p2*c7p1*c9p1*ss[32] + 60*c1p1*c5p1*c7p1*c9p2*ss[40] + 60*c1p1*c5p1*c7p2*c9p1*ss[38] + 60*c1p1*c5p2*c7p1*c9p1*ss[36] + 60*c1p2*c3p1*c5p1*c7p1*ss[26] + 60*c1p2*c3p1*c5p1*c9p1*ss[28] + 60*c1p2*c3p1*c7p1*c9p1*ss[30] + 60*c1p2*c5p1*c7p1*c9p1*ss[32] + 60*c3p1*c5p1*c7p1*c9p2*ss[42] + 60*c3p1*c5p1*c7p2*c9p1*ss[40] + 60*c3p1*c5p2*c7p1*c9p1*ss[38] + 60*c3p2*c5p1*c7p1*c9p1*ss[36] + 20*c1p1*c3p1*c5p3*ss[28] + 20*c1p1*c3p1*c7p3*ss[34] + 20*c1p1*c3p1*c9p3*ss[40] + 30*c1p1*c3p2*c5p2*ss[26] + 30*c1p1*c3p2*c7p2*ss[30] + 30*c1p1*c3p2*c9p2*ss[34] + 20*c1p1*c3p3*c5p1*ss[24] + 20*c1p1*c3p3*c7p1*ss[26] + 20*c1p1*c3p3*c9p1*ss[28] + 20*c1p1*c5p1*c7p3*ss[36] + 20*c1p1*c5p1*c9p3*ss[42] + 30*c1p1*c5p2*c7p2*ss[34] + 30*c1p1*c5p2*c9p2*ss[38] + 20*c1p1*c5p3*c7p1*ss[32] + 20*c1p1*c5p3*c9p1*ss[34] + 20*c1p1*c7p1*c9p3*ss[44] + 30*c1p1*c7p2*c9p2*ss[42] + 20*c1p1*c7p3*c9p1*ss[40] + 30*c1p2*c3p1*c5p2*ss[24] + 30*c1p2*c3p1*c7p2*ss[28] + 30*c1p2*c3p1*c9p2*ss[32] + 30*c1p2*c3p2*c5p1*ss[22] + 30*c1p2*c3p2*c7p1*ss[24] + 30*c1p2*c3p2*c9p1*ss[26] + 30*c1p2*c5p1*c7p2*ss[30] + 30*c1p2*c5p1*c9p2*ss[34] + 30*c1p2*c5p2*c7p1*ss[28] + 30*c1p2*c5p2*c9p1*ss[30] + 30*c1p2*c7p1*c9p2*ss[36] + 30*c1p2*c7p2*c9p1*ss[34] + 20*c1p3*c3p1*c5p1*ss[20] + 20*c1p3*c3p1*c7p1*ss[22] + 20*c1p3*c3p1*c9p1*ss[24] + 20*c1p3*c5p1*c7p1*ss[24] + 20*c1p3*c5p1*c9p1*ss[26] + 20*c1p3*c7p1*c9p1*ss[28] + 20*c3p1*c5p1*c7p3*ss[38] + 20*c3p1*c5p1*c9p3*ss[44] + 30*c3p1*c5p2*c7p2*ss[36] + 30*c3p1*c5p2*c9p2*ss[40] + 20*c3p1*c5p3*c7p1*ss[34] + 20*c3p1*c5p3*c9p1*ss[36] + 20*c3p1*c7p1*c9p3*ss[46] + 30*c3p1*c7p2*c9p2*ss[44] + 20*c3p1*c7p3*c9p1*ss[42] + 30*c3p2*c5p1*c7p2*ss[34] + 30*c3p2*c5p1*c9p2*ss[38] + 30*c3p2*c5p2*c7p1*ss[32] + 30*c3p2*c5p2*c9p1*ss[34] + 30*c3p2*c7p1*c9p2*ss[40] + 30*c3p2*c7p2*c9p1*ss[38] + 20*c3p3*c5p1*c7p1*ss[30] + 20*c3p3*c5p1*c9p1*ss[32] + 20*c3p3*c7p1*c9p1*ss[34] + 20*c5p1*c7p1*c9p3*ss[48] + 30*c5p1*c7p2*c9p2*ss[46] + 20*c5p1*c7p3*c9p1*ss[44] + 30*c5p2*c7p1*c9p2*ss[44] + 30*c5p2*c7p2*c9p1*ss[42] + 20*c5p3*c7p1*c9p1*ss[40] + 5*c1p1*c3p4*ss[22] + 5*c1p1*c5p4*ss[30] + 5*c1p1*c7p4*ss[38] + 5*c1p1*c9p4*ss[46] + 10*c1p2*c3p3*ss[20] + 10*c1p2*c5p3*ss[26] + 10*c1p2*c7p3*ss[32] + 10*c1p2*c9p3*ss[38] + 10*c1p3*c3p2*ss[18] + 10*c1p3*c5p2*ss[22] + 10*c1p3*c7p2*ss[26] + 10*c1p3*c9p2*ss[30] + 5*c1p4*c3p1*ss[16] + 5*c1p4*c5p1*ss[18] + 5*c1p4*c7p1*ss[20] + 5*c1p4*c9p1*ss[22] + 5*c3p1*c5p4*ss[32] + 5*c3p1*c7p4*ss[40] + 5*c3p1*c9p4*ss[48] + 10*c3p2*c5p3*ss[30] + 10*c3p2*c7p3*ss[36] + 10*c3p2*c9p3*ss[42] + 10*c3p3*c5p2*ss[28] + 10*c3p3*c7p2*ss[32] + 10*c3p3*c9p2*ss[36] + 5*c3p4*c5p1*ss[26] + 5*c3p4*c7p1*ss[28] + 5*c3p4*c9p1*ss[30] + 5*c5p1*c7p4*ss[42] + 5*c5p1*c9p4*ss[50] + 10*c5p2*c7p3*ss[40] + 10*c5p2*c9p3*ss[46] + 10*c5p3*c7p2*ss[38] + 10*c5p3*c9p2*ss[42] + 5*c5p4*c7p1*ss[36] + 5*c5p4*c9p1*ss[38] + 5*c7p1*c9p4*ss[52] + 10*c7p2*c9p3*ss[50] + 10*c7p3*c9p2*ss[48] + 5*c7p4*c9p1*ss[46] + c1p5*ss[14] + c3p5*ss[24] + c5p5*ss[34] + c7p5*ss[44] + c9p5*ss[54])*sys_part_2;
  grad_4_num += 8*(840*c1p1*c3p1*c5p1*c7p1*c9p3*ss[52] + 1260*c1p1*c3p1*c5p1*c7p2*c9p2*ss[50] + 840*c1p1*c3p1*c5p1*c7p3*c9p1*ss[48] + 1260*c1p1*c3p1*c5p2*c7p1*c9p2*ss[48] + 1260*c1p1*c3p1*c5p2*c7p2*c9p1*ss[46] + 840*c1p1*c3p1*c5p3*c7p1*c9p1*ss[44] + 1260*c1p1*c3p2*c5p1*c7p1*c9p2*ss[46] + 1260*c1p1*c3p2*c5p1*c7p2*c9p1*ss[44] + 1260*c1p1*c3p2*c5p2*c7p1*c9p1*ss[42] + 840*c1p1*c3p3*c5p1*c7p1*c9p1*ss[40] + 1260*c1p2*c3p1*c5p1*c7p1*c9p2*ss[44] + 1260*c1p2*c3p1*c5p1*c7p2*c9p1*ss[42] + 1260*c1p2*c3p1*c5p2*c7p1*c9p1*ss[40] + 1260*c1p2*c3p2*c5p1*c7p1*c9p1*ss[38] + 840*c1p3*c3p1*c5p1*c7p1*c9p1*ss[36] + 210*c1p1*c3p1*c5p1*c7p4*ss[46] + 210*c1p1*c3p1*c5p1*c9p4*ss[54] + 420*c1p1*c3p1*c5p2*c7p3*ss[44] + 420*c1p1*c3p1*c5p2*c9p3*ss[50] + 420*c1p1*c3p1*c5p3*c7p2*ss[42] + 420*c1p1*c3p1*c5p3*c9p2*ss[46] + 210*c1p1*c3p1*c5p4*c7p1*ss[40] + 210*c1p1*c3p1*c5p4*c9p1*ss[42] + 210*c1p1*c3p1*c7p1*c9p4*ss[56] + 420*c1p1*c3p1*c7p2*c9p3*ss[54] + 420*c1p1*c3p1*c7p3*c9p2*ss[52] + 210*c1p1*c3p1*c7p4*c9p1*ss[50] + 420*c1p1*c3p2*c5p1*c7p3*ss[42] + 420*c1p1*c3p2*c5p1*c9p3*ss[48] + 630*c1p1*c3p2*c5p2*c7p2*ss[40] + 630*c1p1*c3p2*c5p2*c9p2*ss[44] + 420*c1p1*c3p2*c5p3*c7p1*ss[38] + 420*c1p1*c3p2*c5p3*c9p1*ss[40] + 420*c1p1*c3p2*c7p1*c9p3*ss[50] + 630*c1p1*c3p2*c7p2*c9p2*ss[48] + 420*c1p1*c3p2*c7p3*c9p1*ss[46] + 420*c1p1*c3p3*c5p1*c7p2*ss[38] + 420*c1p1*c3p3*c5p1*c9p2*ss[42] + 420*c1p1*c3p3*c5p2*c7p1*ss[36] + 420*c1p1*c3p3*c5p2*c9p1*ss[38] + 420*c1p1*c3p3*c7p1*c9p2*ss[44] + 420*c1p1*c3p3*c7p2*c9p1*ss[42] + 210*c1p1*c3p4*c5p1*c7p1*ss[34] + 210*c1p1*c3p4*c5p1*c9p1*ss[36] + 210*c1p1*c3p4*c7p1*c9p1*ss[38] + 210*c1p1*c5p1*c7p1*c9p4*ss[58] + 420*c1p1*c5p1*c7p2*c9p3*ss[56] + 420*c1p1*c5p1*c7p3*c9p2*ss[54] + 210*c1p1*c5p1*c7p4*c9p1*ss[52] + 420*c1p1*c5p2*c7p1*c9p3*ss[54] + 630*c1p1*c5p2*c7p2*c9p2*ss[52] + 420*c1p1*c5p2*c7p3*c9p1*ss[50] + 420*c1p1*c5p3*c7p1*c9p2*ss[50] + 420*c1p1*c5p3*c7p2*c9p1*ss[48] + 210*c1p1*c5p4*c7p1*c9p1*ss[46] + 420*c1p2*c3p1*c5p1*c7p3*ss[40] + 420*c1p2*c3p1*c5p1*c9p3*ss[46] + 630*c1p2*c3p1*c5p2*c7p2*ss[38] + 630*c1p2*c3p1*c5p2*c9p2*ss[42] + 420*c1p2*c3p1*c5p3*c7p1*ss[36] + 420*c1p2*c3p1*c5p3*c9p1*ss[38] + 420*c1p2*c3p1*c7p1*c9p3*ss[48] + 630*c1p2*c3p1*c7p2*c9p2*ss[46] + 420*c1p2*c3p1*c7p3*c9p1*ss[44] + 630*c1p2*c3p2*c5p1*c7p2*ss[36] + 630*c1p2*c3p2*c5p1*c9p2*ss[40] + 630*c1p2*c3p2*c5p2*c7p1*ss[34] + 630*c1p2*c3p2*c5p2*c9p1*ss[36] + 630*c1p2*c3p2*c7p1*c9p2*ss[42] + 630*c1p2*c3p2*c7p2*c9p1*ss[40] + 420*c1p2*c3p3*c5p1*c7p1*ss[32] + 420*c1p2*c3p3*c5p1*c9p1*ss[34] + 420*c1p2*c3p3*c7p1*c9p1*ss[36] + 420*c1p2*c5p1*c7p1*c9p3*ss[50] + 630*c1p2*c5p1*c7p2*c9p2*ss[48] + 420*c1p2*c5p1*c7p3*c9p1*ss[46] + 630*c1p2*c5p2*c7p1*c9p2*ss[46] + 630*c1p2*c5p2*c7p2*c9p1*ss[44] + 420*c1p2*c5p3*c7p1*c9p1*ss[42] + 420*c1p3*c3p1*c5p1*c7p2*ss[34] + 420*c1p3*c3p1*c5p1*c9p2*ss[38] + 420*c1p3*c3p1*c5p2*c7p1*ss[32] + 420*c1p3*c3p1*c5p2*c9p1*ss[34] + 420*c1p3*c3p1*c7p1*c9p2*ss[40] + 420*c1p3*c3p1*c7p2*c9p1*ss[38] + 420*c1p3*c3p2*c5p1*c7p1*ss[30] + 420*c1p3*c3p2*c5p1*c9p1*ss[32] + 420*c1p3*c3p2*c7p1*c9p1*ss[34] + 420*c1p3*c5p1*c7p1*c9p2*ss[42] + 420*c1p3*c5p1*c7p2*c9p1*ss[40] + 420*c1p3*c5p2*c7p1*c9p1*ss[38] + 210*c1p4*c3p1*c5p1*c7p1*ss[28] + 210*c1p4*c3p1*c5p1*c9p1*ss[30] + 210*c1p4*c3p1*c7p1*c9p1*ss[32] + 210*c1p4*c5p1*c7p1*c9p1*ss[34] + 210*c3p1*c5p1*c7p1*c9p4*ss[60] + 420*c3p1*c5p1*c7p2*c9p3*ss[58] + 420*c3p1*c5p1*c7p3*c9p2*ss[56] + 210*c3p1*c5p1*c7p4*c9p1*ss[54] + 420*c3p1*c5p2*c7p1*c9p3*ss[56] + 630*c3p1*c5p2*c7p2*c9p2*ss[54] + 420*c3p1*c5p2*c7p3*c9p1*ss[52] + 420*c3p1*c5p3*c7p1*c9p2*ss[52] + 420*c3p1*c5p3*c7p2*c9p1*ss[50] + 210*c3p1*c5p4*c7p1*c9p1*ss[48] + 420*c3p2*c5p1*c7p1*c9p3*ss[54] + 630*c3p2*c5p1*c7p2*c9p2*ss[52] + 420*c3p2*c5p1*c7p3*c9p1*ss[50] + 630*c3p2*c5p2*c7p1*c9p2*ss[50] + 630*c3p2*c5p2*c7p2*c9p1*ss[48] + 420*c3p2*c5p3*c7p1*c9p1*ss[46] + 420*c3p3*c5p1*c7p1*c9p2*ss[48] + 420*c3p3*c5p1*c7p2*c9p1*ss[46] + 420*c3p3*c5p2*c7p1*c9p1*ss[44] + 210*c3p4*c5p1*c7p1*c9p1*ss[42] + 42*c1p1*c3p1*c5p5*ss[38] + 42*c1p1*c3p1*c7p5*ss[48] + 42*c1p1*c3p1*c9p5*ss[58] + 105*c1p1*c3p2*c5p4*ss[36] + 105*c1p1*c3p2*c7p4*ss[44] + 105*c1p1*c3p2*c9p4*ss[52] + 140*c1p1*c3p3*c5p3*ss[34] + 140*c1p1*c3p3*c7p3*ss[40] + 140*c1p1*c3p3*c9p3*ss[46] + 105*c1p1*c3p4*c5p2*ss[32] + 105*c1p1*c3p4*c7p2*ss[36] + 105*c1p1*c3p4*c9p2*ss[40] + 42*c1p1*c3p5*c5p1*ss[30] + 42*c1p1*c3p5*c7p1*ss[32] + 42*c1p1*c3p5*c9p1*ss[34] + 42*c1p1*c5p1*c7p5*ss[50] + 42*c1p1*c5p1*c9p5*ss[60] + 105*c1p1*c5p2*c7p4*ss[48] + 105*c1p1*c5p2*c9p4*ss[56] + 140*c1p1*c5p3*c7p3*ss[46] + 140*c1p1*c5p3*c9p3*ss[52] + 105*c1p1*c5p4*c7p2*ss[44] + 105*c1p1*c5p4*c9p2*ss[48] + 42*c1p1*c5p5*c7p1*ss[42] + 42*c1p1*c5p5*c9p1*ss[44] + 42*c1p1*c7p1*c9p5*ss[62] + 105*c1p1*c7p2*c9p4*ss[60] + 140*c1p1*c7p3*c9p3*ss[58] + 105*c1p1*c7p4*c9p2*ss[56] + 42*c1p1*c7p5*c9p1*ss[54] + 105*c1p2*c3p1*c5p4*ss[34] + 105*c1p2*c3p1*c7p4*ss[42] + 105*c1p2*c3p1*c9p4*ss[50] + 210*c1p2*c3p2*c5p3*ss[32] + 210*c1p2*c3p2*c7p3*ss[38] + 210*c1p2*c3p2*c9p3*ss[44] + 210*c1p2*c3p3*c5p2*ss[30] + 210*c1p2*c3p3*c7p2*ss[34] + 210*c1p2*c3p3*c9p2*ss[38] + 105*c1p2*c3p4*c5p1*ss[28] + 105*c1p2*c3p4*c7p1*ss[30] + 105*c1p2*c3p4*c9p1*ss[32] + 105*c1p2*c5p1*c7p4*ss[44] + 105*c1p2*c5p1*c9p4*ss[52] + 210*c1p2*c5p2*c7p3*ss[42] + 210*c1p2*c5p2*c9p3*ss[48] + 210*c1p2*c5p3*c7p2*ss[40] + 210*c1p2*c5p3*c9p2*ss[44] + 105*c1p2*c5p4*c7p1*ss[38] + 105*c1p2*c5p4*c9p1*ss[40] + 105*c1p2*c7p1*c9p4*ss[54] + 210*c1p2*c7p2*c9p3*ss[52] + 210*c1p2*c7p3*c9p2*ss[50] + 105*c1p2*c7p4*c9p1*ss[48] + 140*c1p3*c3p1*c5p3*ss[30] + 140*c1p3*c3p1*c7p3*ss[36] + 140*c1p3*c3p1*c9p3*ss[42] + 210*c1p3*c3p2*c5p2*ss[28] + 210*c1p3*c3p2*c7p2*ss[32] + 210*c1p3*c3p2*c9p2*ss[36] + 140*c1p3*c3p3*c5p1*ss[26] + 140*c1p3*c3p3*c7p1*ss[28] + 140*c1p3*c3p3*c9p1*ss[30] + 140*c1p3*c5p1*c7p3*ss[38] + 140*c1p3*c5p1*c9p3*ss[44] + 210*c1p3*c5p2*c7p2*ss[36] + 210*c1p3*c5p2*c9p2*ss[40] + 140*c1p3*c5p3*c7p1*ss[34] + 140*c1p3*c5p3*c9p1*ss[36] + 140*c1p3*c7p1*c9p3*ss[46] + 210*c1p3*c7p2*c9p2*ss[44] + 140*c1p3*c7p3*c9p1*ss[42] + 105*c1p4*c3p1*c5p2*ss[26] + 105*c1p4*c3p1*c7p2*ss[30] + 105*c1p4*c3p1*c9p2*ss[34] + 105*c1p4*c3p2*c5p1*ss[24] + 105*c1p4*c3p2*c7p1*ss[26] + 105*c1p4*c3p2*c9p1*ss[28] + 105*c1p4*c5p1*c7p2*ss[32] + 105*c1p4*c5p1*c9p2*ss[36] + 105*c1p4*c5p2*c7p1*ss[30] + 105*c1p4*c5p2*c9p1*ss[32] + 105*c1p4*c7p1*c9p2*ss[38] + 105*c1p4*c7p2*c9p1*ss[36] + 42*c1p5*c3p1*c5p1*ss[22] + 42*c1p5*c3p1*c7p1*ss[24] + 42*c1p5*c3p1*c9p1*ss[26] + 42*c1p5*c5p1*c7p1*ss[26] + 42*c1p5*c5p1*c9p1*ss[28] + 42*c1p5*c7p1*c9p1*ss[30] + 42*c3p1*c5p1*c7p5*ss[52] + 42*c3p1*c5p1*c9p5*ss[62] + 105*c3p1*c5p2*c7p4*ss[50] + 105*c3p1*c5p2*c9p4*ss[58] + 140*c3p1*c5p3*c7p3*ss[48] + 140*c3p1*c5p3*c9p3*ss[54] + 105*c3p1*c5p4*c7p2*ss[46] + 105*c3p1*c5p4*c9p2*ss[50] + 42*c3p1*c5p5*c7p1*ss[44] + 42*c3p1*c5p5*c9p1*ss[46] + 42*c3p1*c7p1*c9p5*ss[64] + 105*c3p1*c7p2*c9p4*ss[62] + 140*c3p1*c7p3*c9p3*ss[60] + 105*c3p1*c7p4*c9p2*ss[58] + 42*c3p1*c7p5*c9p1*ss[56] + 105*c3p2*c5p1*c7p4*ss[48] + 105*c3p2*c5p1*c9p4*ss[56] + 210*c3p2*c5p2*c7p3*ss[46] + 210*c3p2*c5p2*c9p3*ss[52] + 210*c3p2*c5p3*c7p2*ss[44] + 210*c3p2*c5p3*c9p2*ss[48] + 105*c3p2*c5p4*c7p1*ss[42] + 105*c3p2*c5p4*c9p1*ss[44] + 105*c3p2*c7p1*c9p4*ss[58] + 210*c3p2*c7p2*c9p3*ss[56] + 210*c3p2*c7p3*c9p2*ss[54] + 105*c3p2*c7p4*c9p1*ss[52] + 140*c3p3*c5p1*c7p3*ss[44] + 140*c3p3*c5p1*c9p3*ss[50] + 210*c3p3*c5p2*c7p2*ss[42] + 210*c3p3*c5p2*c9p2*ss[46] + 140*c3p3*c5p3*c7p1*ss[40] + 140*c3p3*c5p3*c9p1*ss[42] + 140*c3p3*c7p1*c9p3*ss[52] + 210*c3p3*c7p2*c9p2*ss[50] + 140*c3p3*c7p3*c9p1*ss[48] + 105*c3p4*c5p1*c7p2*ss[40] + 105*c3p4*c5p1*c9p2*ss[44] + 105*c3p4*c5p2*c7p1*ss[38] + 105*c3p4*c5p2*c9p1*ss[40] + 105*c3p4*c7p1*c9p2*ss[46] + 105*c3p4*c7p2*c9p1*ss[44] + 42*c3p5*c5p1*c7p1*ss[36] + 42*c3p5*c5p1*c9p1*ss[38] + 42*c3p5*c7p1*c9p1*ss[40] + 42*c5p1*c7p1*c9p5*ss[66] + 105*c5p1*c7p2*c9p4*ss[64] + 140*c5p1*c7p3*c9p3*ss[62] + 105*c5p1*c7p4*c9p2*ss[60] + 42*c5p1*c7p5*c9p1*ss[58] + 105*c5p2*c7p1*c9p4*ss[62] + 210*c5p2*c7p2*c9p3*ss[60] + 210*c5p2*c7p3*c9p2*ss[58] + 105*c5p2*c7p4*c9p1*ss[56] + 140*c5p3*c7p1*c9p3*ss[58] + 210*c5p3*c7p2*c9p2*ss[56] + 140*c5p3*c7p3*c9p1*ss[54] + 105*c5p4*c7p1*c9p2*ss[54] + 105*c5p4*c7p2*c9p1*ss[52] + 42*c5p5*c7p1*c9p1*ss[50] + 7*c1p1*c3p6*ss[28] + 7*c1p1*c5p6*ss[40] + 7*c1p1*c7p6*ss[52] + 7*c1p1*c9p6*ss[64] + 21*c1p2*c3p5*ss[26] + 21*c1p2*c5p5*ss[36] + 21*c1p2*c7p5*ss[46] + 21*c1p2*c9p5*ss[56] + 35*c1p3*c3p4*ss[24] + 35*c1p3*c5p4*ss[32] + 35*c1p3*c7p4*ss[40] + 35*c1p3*c9p4*ss[48] + 35*c1p4*c3p3*ss[22] + 35*c1p4*c5p3*ss[28] + 35*c1p4*c7p3*ss[34] + 35*c1p4*c9p3*ss[40] + 21*c1p5*c3p2*ss[20] + 21*c1p5*c5p2*ss[24] + 21*c1p5*c7p2*ss[28] + 21*c1p5*c9p2*ss[32] + 7*c1p6*c3p1*ss[18] + 7*c1p6*c5p1*ss[20] + 7*c1p6*c7p1*ss[22] + 7*c1p6*c9p1*ss[24] + 7*c3p1*c5p6*ss[42] + 7*c3p1*c7p6*ss[54] + 7*c3p1*c9p6*ss[66] + 21*c3p2*c5p5*ss[40] + 21*c3p2*c7p5*ss[50] + 21*c3p2*c9p5*ss[60] + 35*c3p3*c5p4*ss[38] + 35*c3p3*c7p4*ss[46] + 35*c3p3*c9p4*ss[54] + 35*c3p4*c5p3*ss[36] + 35*c3p4*c7p3*ss[42] + 35*c3p4*c9p3*ss[48] + 21*c3p5*c5p2*ss[34] + 21*c3p5*c7p2*ss[38] + 21*c3p5*c9p2*ss[42] + 7*c3p6*c5p1*ss[32] + 7*c3p6*c7p1*ss[34] + 7*c3p6*c9p1*ss[36] + 7*c5p1*c7p6*ss[56] + 7*c5p1*c9p6*ss[68] + 21*c5p2*c7p5*ss[54] + 21*c5p2*c9p5*ss[64] + 35*c5p3*c7p4*ss[52] + 35*c5p3*c9p4*ss[60] + 35*c5p4*c7p3*ss[50] + 35*c5p4*c9p3*ss[56] + 21*c5p5*c7p2*ss[48] + 21*c5p5*c9p2*ss[52] + 7*c5p6*c7p1*ss[46] + 7*c5p6*c9p1*ss[48] + 7*c7p1*c9p6*ss[70] + 21*c7p2*c9p5*ss[68] + 35*c7p3*c9p4*ss[66] + 35*c7p4*c9p3*ss[64] + 21*c7p5*c9p2*ss[62] + 7*c7p6*c9p1*ss[60] + c1p7*ss[16] + c3p7*ss[30] + c5p7*ss[44] + c7p7*ss[58] + c9p7*ss[72])*sys_part_3;
  grad_4_num += 10*(3024*c1p1*c3p1*c5p1*c7p1*c9p5*ss[70] + 7560*c1p1*c3p1*c5p1*c7p2*c9p4*ss[68] + 10080*c1p1*c3p1*c5p1*c7p3*c9p3*ss[66] + 7560*c1p1*c3p1*c5p1*c7p4*c9p2*ss[64] + 3024*c1p1*c3p1*c5p1*c7p5*c9p1*ss[62] + 7560*c1p1*c3p1*c5p2*c7p1*c9p4*ss[66] + 15120*c1p1*c3p1*c5p2*c7p2*c9p3*ss[64] + 15120*c1p1*c3p1*c5p2*c7p3*c9p2*ss[62] + 7560*c1p1*c3p1*c5p2*c7p4*c9p1*ss[60] + 10080*c1p1*c3p1*c5p3*c7p1*c9p3*ss[62] + 15120*c1p1*c3p1*c5p3*c7p2*c9p2*ss[60] + 10080*c1p1*c3p1*c5p3*c7p3*c9p1*ss[58] + 7560*c1p1*c3p1*c5p4*c7p1*c9p2*ss[58] + 7560*c1p1*c3p1*c5p4*c7p2*c9p1*ss[56] + 3024*c1p1*c3p1*c5p5*c7p1*c9p1*ss[54] + 7560*c1p1*c3p2*c5p1*c7p1*c9p4*ss[64] + 15120*c1p1*c3p2*c5p1*c7p2*c9p3*ss[62] + 15120*c1p1*c3p2*c5p1*c7p3*c9p2*ss[60] + 7560*c1p1*c3p2*c5p1*c7p4*c9p1*ss[58] + 15120*c1p1*c3p2*c5p2*c7p1*c9p3*ss[60] + 22680*c1p1*c3p2*c5p2*c7p2*c9p2*ss[58] + 15120*c1p1*c3p2*c5p2*c7p3*c9p1*ss[56] + 15120*c1p1*c3p2*c5p3*c7p1*c9p2*ss[56] + 15120*c1p1*c3p2*c5p3*c7p2*c9p1*ss[54] + 7560*c1p1*c3p2*c5p4*c7p1*c9p1*ss[52] + 10080*c1p1*c3p3*c5p1*c7p1*c9p3*ss[58] + 15120*c1p1*c3p3*c5p1*c7p2*c9p2*ss[56] + 10080*c1p1*c3p3*c5p1*c7p3*c9p1*ss[54] + 15120*c1p1*c3p3*c5p2*c7p1*c9p2*ss[54] + 15120*c1p1*c3p3*c5p2*c7p2*c9p1*ss[52] + 10080*c1p1*c3p3*c5p3*c7p1*c9p1*ss[50] + 7560*c1p1*c3p4*c5p1*c7p1*c9p2*ss[52] + 7560*c1p1*c3p4*c5p1*c7p2*c9p1*ss[50] + 7560*c1p1*c3p4*c5p2*c7p1*c9p1*ss[48] + 3024*c1p1*c3p5*c5p1*c7p1*c9p1*ss[46] + 7560*c1p2*c3p1*c5p1*c7p1*c9p4*ss[62] + 15120*c1p2*c3p1*c5p1*c7p2*c9p3*ss[60] + 15120*c1p2*c3p1*c5p1*c7p3*c9p2*ss[58] + 7560*c1p2*c3p1*c5p1*c7p4*c9p1*ss[56] + 15120*c1p2*c3p1*c5p2*c7p1*c9p3*ss[58] + 22680*c1p2*c3p1*c5p2*c7p2*c9p2*ss[56] + 15120*c1p2*c3p1*c5p2*c7p3*c9p1*ss[54] + 15120*c1p2*c3p1*c5p3*c7p1*c9p2*ss[54] + 15120*c1p2*c3p1*c5p3*c7p2*c9p1*ss[52] + 7560*c1p2*c3p1*c5p4*c7p1*c9p1*ss[50] + 15120*c1p2*c3p2*c5p1*c7p1*c9p3*ss[56] + 22680*c1p2*c3p2*c5p1*c7p2*c9p2*ss[54] + 15120*c1p2*c3p2*c5p1*c7p3*c9p1*ss[52] + 22680*c1p2*c3p2*c5p2*c7p1*c9p2*ss[52] + 22680*c1p2*c3p2*c5p2*c7p2*c9p1*ss[50] + 15120*c1p2*c3p2*c5p3*c7p1*c9p1*ss[48] + 15120*c1p2*c3p3*c5p1*c7p1*c9p2*ss[50] + 15120*c1p2*c3p3*c5p1*c7p2*c9p1*ss[48] + 15120*c1p2*c3p3*c5p2*c7p1*c9p1*ss[46] + 7560*c1p2*c3p4*c5p1*c7p1*c9p1*ss[44] + 10080*c1p3*c3p1*c5p1*c7p1*c9p3*ss[54] + 15120*c1p3*c3p1*c5p1*c7p2*c9p2*ss[52] + 10080*c1p3*c3p1*c5p1*c7p3*c9p1*ss[50] + 15120*c1p3*c3p1*c5p2*c7p1*c9p2*ss[50] + 15120*c1p3*c3p1*c5p2*c7p2*c9p1*ss[48] + 10080*c1p3*c3p1*c5p3*c7p1*c9p1*ss[46] + 15120*c1p3*c3p2*c5p1*c7p1*c9p2*ss[48] + 15120*c1p3*c3p2*c5p1*c7p2*c9p1*ss[46] + 15120*c1p3*c3p2*c5p2*c7p1*c9p1*ss[44] + 10080*c1p3*c3p3*c5p1*c7p1*c9p1*ss[42] + 7560*c1p4*c3p1*c5p1*c7p1*c9p2*ss[46] + 7560*c1p4*c3p1*c5p1*c7p2*c9p1*ss[44] + 7560*c1p4*c3p1*c5p2*c7p1*c9p1*ss[42] + 7560*c1p4*c3p2*c5p1*c7p1*c9p1*ss[40] + 3024*c1p5*c3p1*c5p1*c7p1*c9p1*ss[38] + 504*c1p1*c3p1*c5p1*c7p6*ss[60] + 504*c1p1*c3p1*c5p1*c9p6*ss[72] + 1512*c1p1*c3p1*c5p2*c7p5*ss[58] + 1512*c1p1*c3p1*c5p2*c9p5*ss[68] + 2520*c1p1*c3p1*c5p3*c7p4*ss[56] + 2520*c1p1*c3p1*c5p3*c9p4*ss[64] + 2520*c1p1*c3p1*c5p4*c7p3*ss[54] + 2520*c1p1*c3p1*c5p4*c9p3*ss[60] + 1512*c1p1*c3p1*c5p5*c7p2*ss[52] + 1512*c1p1*c3p1*c5p5*c9p2*ss[56] + 504*c1p1*c3p1*c5p6*c7p1*ss[50] + 504*c1p1*c3p1*c5p6*c9p1*ss[52] + 504*c1p1*c3p1*c7p1*c9p6*ss[74] + 1512*c1p1*c3p1*c7p2*c9p5*ss[72] + 2520*c1p1*c3p1*c7p3*c9p4*ss[70] + 2520*c1p1*c3p1*c7p4*c9p3*ss[68] + 1512*c1p1*c3p1*c7p5*c9p2*ss[66] + 504*c1p1*c3p1*c7p6*c9p1*ss[64] + 1512*c1p1*c3p2*c5p1*c7p5*ss[56] + 1512*c1p1*c3p2*c5p1*c9p5*ss[66] + 3780*c1p1*c3p2*c5p2*c7p4*ss[54] + 3780*c1p1*c3p2*c5p2*c9p4*ss[62] + 5040*c1p1*c3p2*c5p3*c7p3*ss[52] + 5040*c1p1*c3p2*c5p3*c9p3*ss[58] + 3780*c1p1*c3p2*c5p4*c7p2*ss[50] + 3780*c1p1*c3p2*c5p4*c9p2*ss[54] + 1512*c1p1*c3p2*c5p5*c7p1*ss[48] + 1512*c1p1*c3p2*c5p5*c9p1*ss[50] + 1512*c1p1*c3p2*c7p1*c9p5*ss[68] + 3780*c1p1*c3p2*c7p2*c9p4*ss[66] + 5040*c1p1*c3p2*c7p3*c9p3*ss[64] + 3780*c1p1*c3p2*c7p4*c9p2*ss[62] + 1512*c1p1*c3p2*c7p5*c9p1*ss[60] + 2520*c1p1*c3p3*c5p1*c7p4*ss[52] + 2520*c1p1*c3p3*c5p1*c9p4*ss[60] + 5040*c1p1*c3p3*c5p2*c7p3*ss[50] + 5040*c1p1*c3p3*c5p2*c9p3*ss[56] + 5040*c1p1*c3p3*c5p3*c7p2*ss[48] + 5040*c1p1*c3p3*c5p3*c9p2*ss[52] + 2520*c1p1*c3p3*c5p4*c7p1*ss[46] + 2520*c1p1*c3p3*c5p4*c9p1*ss[48] + 2520*c1p1*c3p3*c7p1*c9p4*ss[62] + 5040*c1p1*c3p3*c7p2*c9p3*ss[60] + 5040*c1p1*c3p3*c7p3*c9p2*ss[58] + 2520*c1p1*c3p3*c7p4*c9p1*ss[56] + 2520*c1p1*c3p4*c5p1*c7p3*ss[48] + 2520*c1p1*c3p4*c5p1*c9p3*ss[54] + 3780*c1p1*c3p4*c5p2*c7p2*ss[46] + 3780*c1p1*c3p4*c5p2*c9p2*ss[50] + 2520*c1p1*c3p4*c5p3*c7p1*ss[44] + 2520*c1p1*c3p4*c5p3*c9p1*ss[46] + 2520*c1p1*c3p4*c7p1*c9p3*ss[56] + 3780*c1p1*c3p4*c7p2*c9p2*ss[54] + 2520*c1p1*c3p4*c7p3*c9p1*ss[52] + 1512*c1p1*c3p5*c5p1*c7p2*ss[44] + 1512*c1p1*c3p5*c5p1*c9p2*ss[48] + 1512*c1p1*c3p5*c5p2*c7p1*ss[42] + 1512*c1p1*c3p5*c5p2*c9p1*ss[44] + 1512*c1p1*c3p5*c7p1*c9p2*ss[50] + 1512*c1p1*c3p5*c7p2*c9p1*ss[48] + 504*c1p1*c3p6*c5p1*c7p1*ss[40] + 504*c1p1*c3p6*c5p1*c9p1*ss[42] + 504*c1p1*c3p6*c7p1*c9p1*ss[44] + 504*c1p1*c5p1*c7p1*c9p6*ss[76] + 1512*c1p1*c5p1*c7p2*c9p5*ss[74] + 2520*c1p1*c5p1*c7p3*c9p4*ss[72] + 2520*c1p1*c5p1*c7p4*c9p3*ss[70] + 1512*c1p1*c5p1*c7p5*c9p2*ss[68] + 504*c1p1*c5p1*c7p6*c9p1*ss[66] + 1512*c1p1*c5p2*c7p1*c9p5*ss[72] + 3780*c1p1*c5p2*c7p2*c9p4*ss[70] + 5040*c1p1*c5p2*c7p3*c9p3*ss[68] + 3780*c1p1*c5p2*c7p4*c9p2*ss[66] + 1512*c1p1*c5p2*c7p5*c9p1*ss[64] + 2520*c1p1*c5p3*c7p1*c9p4*ss[68] + 5040*c1p1*c5p3*c7p2*c9p3*ss[66] + 5040*c1p1*c5p3*c7p3*c9p2*ss[64] + 2520*c1p1*c5p3*c7p4*c9p1*ss[62] + 2520*c1p1*c5p4*c7p1*c9p3*ss[64] + 3780*c1p1*c5p4*c7p2*c9p2*ss[62] + 2520*c1p1*c5p4*c7p3*c9p1*ss[60] + 1512*c1p1*c5p5*c7p1*c9p2*ss[60] + 1512*c1p1*c5p5*c7p2*c9p1*ss[58] + 504*c1p1*c5p6*c7p1*c9p1*ss[56] + 1512*c1p2*c3p1*c5p1*c7p5*ss[54] + 1512*c1p2*c3p1*c5p1*c9p5*ss[64] + 3780*c1p2*c3p1*c5p2*c7p4*ss[52] + 3780*c1p2*c3p1*c5p2*c9p4*ss[60] + 5040*c1p2*c3p1*c5p3*c7p3*ss[50] + 5040*c1p2*c3p1*c5p3*c9p3*ss[56] + 3780*c1p2*c3p1*c5p4*c7p2*ss[48] + 3780*c1p2*c3p1*c5p4*c9p2*ss[52] + 1512*c1p2*c3p1*c5p5*c7p1*ss[46] + 1512*c1p2*c3p1*c5p5*c9p1*ss[48] + 1512*c1p2*c3p1*c7p1*c9p5*ss[66] + 3780*c1p2*c3p1*c7p2*c9p4*ss[64] + 5040*c1p2*c3p1*c7p3*c9p3*ss[62] + 3780*c1p2*c3p1*c7p4*c9p2*ss[60] + 1512*c1p2*c3p1*c7p5*c9p1*ss[58] + 3780*c1p2*c3p2*c5p1*c7p4*ss[50] + 3780*c1p2*c3p2*c5p1*c9p4*ss[58] + 7560*c1p2*c3p2*c5p2*c7p3*ss[48] + 7560*c1p2*c3p2*c5p2*c9p3*ss[54] + 7560*c1p2*c3p2*c5p3*c7p2*ss[46] + 7560*c1p2*c3p2*c5p3*c9p2*ss[50] + 3780*c1p2*c3p2*c5p4*c7p1*ss[44] + 3780*c1p2*c3p2*c5p4*c9p1*ss[46] + 3780*c1p2*c3p2*c7p1*c9p4*ss[60] + 7560*c1p2*c3p2*c7p2*c9p3*ss[58] + 7560*c1p2*c3p2*c7p3*c9p2*ss[56] + 3780*c1p2*c3p2*c7p4*c9p1*ss[54] + 5040*c1p2*c3p3*c5p1*c7p3*ss[46] + 5040*c1p2*c3p3*c5p1*c9p3*ss[52] + 7560*c1p2*c3p3*c5p2*c7p2*ss[44] + 7560*c1p2*c3p3*c5p2*c9p2*ss[48] + 5040*c1p2*c3p3*c5p3*c7p1*ss[42] + 5040*c1p2*c3p3*c5p3*c9p1*ss[44] + 5040*c1p2*c3p3*c7p1*c9p3*ss[54] + 7560*c1p2*c3p3*c7p2*c9p2*ss[52] + 5040*c1p2*c3p3*c7p3*c9p1*ss[50] + 3780*c1p2*c3p4*c5p1*c7p2*ss[42] + 3780*c1p2*c3p4*c5p1*c9p2*ss[46] + 3780*c1p2*c3p4*c5p2*c7p1*ss[40] + 3780*c1p2*c3p4*c5p2*c9p1*ss[42] + 3780*c1p2*c3p4*c7p1*c9p2*ss[48] + 3780*c1p2*c3p4*c7p2*c9p1*ss[46] + 1512*c1p2*c3p5*c5p1*c7p1*ss[38] + 1512*c1p2*c3p5*c5p1*c9p1*ss[40] + 1512*c1p2*c3p5*c7p1*c9p1*ss[42] + 1512*c1p2*c5p1*c7p1*c9p5*ss[68] + 3780*c1p2*c5p1*c7p2*c9p4*ss[66] + 5040*c1p2*c5p1*c7p3*c9p3*ss[64] + 3780*c1p2*c5p1*c7p4*c9p2*ss[62] + 1512*c1p2*c5p1*c7p5*c9p1*ss[60] + 3780*c1p2*c5p2*c7p1*c9p4*ss[64] + 7560*c1p2*c5p2*c7p2*c9p3*ss[62] + 7560*c1p2*c5p2*c7p3*c9p2*ss[60] + 3780*c1p2*c5p2*c7p4*c9p1*ss[58] + 5040*c1p2*c5p3*c7p1*c9p3*ss[60] + 7560*c1p2*c5p3*c7p2*c9p2*ss[58] + 5040*c1p2*c5p3*c7p3*c9p1*ss[56] + 3780*c1p2*c5p4*c7p1*c9p2*ss[56] + 3780*c1p2*c5p4*c7p2*c9p1*ss[54] + 1512*c1p2*c5p5*c7p1*c9p1*ss[52] + 2520*c1p3*c3p1*c5p1*c7p4*ss[48] + 2520*c1p3*c3p1*c5p1*c9p4*ss[56] + 5040*c1p3*c3p1*c5p2*c7p3*ss[46] + 5040*c1p3*c3p1*c5p2*c9p3*ss[52] + 5040*c1p3*c3p1*c5p3*c7p2*ss[44] + 5040*c1p3*c3p1*c5p3*c9p2*ss[48] + 2520*c1p3*c3p1*c5p4*c7p1*ss[42] + 2520*c1p3*c3p1*c5p4*c9p1*ss[44] + 2520*c1p3*c3p1*c7p1*c9p4*ss[58] + 5040*c1p3*c3p1*c7p2*c9p3*ss[56] + 5040*c1p3*c3p1*c7p3*c9p2*ss[54] + 2520*c1p3*c3p1*c7p4*c9p1*ss[52] + 5040*c1p3*c3p2*c5p1*c7p3*ss[44] + 5040*c1p3*c3p2*c5p1*c9p3*ss[50] + 7560*c1p3*c3p2*c5p2*c7p2*ss[42] + 7560*c1p3*c3p2*c5p2*c9p2*ss[46] + 5040*c1p3*c3p2*c5p3*c7p1*ss[40] + 5040*c1p3*c3p2*c5p3*c9p1*ss[42] + 5040*c1p3*c3p2*c7p1*c9p3*ss[52] + 7560*c1p3*c3p2*c7p2*c9p2*ss[50] + 5040*c1p3*c3p2*c7p3*c9p1*ss[48] + 5040*c1p3*c3p3*c5p1*c7p2*ss[40] + 5040*c1p3*c3p3*c5p1*c9p2*ss[44] + 5040*c1p3*c3p3*c5p2*c7p1*ss[38] + 5040*c1p3*c3p3*c5p2*c9p1*ss[40] + 5040*c1p3*c3p3*c7p1*c9p2*ss[46] + 5040*c1p3*c3p3*c7p2*c9p1*ss[44] + 2520*c1p3*c3p4*c5p1*c7p1*ss[36] + 2520*c1p3*c3p4*c5p1*c9p1*ss[38] + 2520*c1p3*c3p4*c7p1*c9p1*ss[40] + 2520*c1p3*c5p1*c7p1*c9p4*ss[60] + 5040*c1p3*c5p1*c7p2*c9p3*ss[58] + 5040*c1p3*c5p1*c7p3*c9p2*ss[56] + 2520*c1p3*c5p1*c7p4*c9p1*ss[54] + 5040*c1p3*c5p2*c7p1*c9p3*ss[56] + 7560*c1p3*c5p2*c7p2*c9p2*ss[54] + 5040*c1p3*c5p2*c7p3*c9p1*ss[52] + 5040*c1p3*c5p3*c7p1*c9p2*ss[52] + 5040*c1p3*c5p3*c7p2*c9p1*ss[50] + 2520*c1p3*c5p4*c7p1*c9p1*ss[48] + 2520*c1p4*c3p1*c5p1*c7p3*ss[42] + 2520*c1p4*c3p1*c5p1*c9p3*ss[48] + 3780*c1p4*c3p1*c5p2*c7p2*ss[40] + 3780*c1p4*c3p1*c5p2*c9p2*ss[44] + 2520*c1p4*c3p1*c5p3*c7p1*ss[38] + 2520*c1p4*c3p1*c5p3*c9p1*ss[40] + 2520*c1p4*c3p1*c7p1*c9p3*ss[50] + 3780*c1p4*c3p1*c7p2*c9p2*ss[48] + 2520*c1p4*c3p1*c7p3*c9p1*ss[46] + 3780*c1p4*c3p2*c5p1*c7p2*ss[38] + 3780*c1p4*c3p2*c5p1*c9p2*ss[42] + 3780*c1p4*c3p2*c5p2*c7p1*ss[36] + 3780*c1p4*c3p2*c5p2*c9p1*ss[38] + 3780*c1p4*c3p2*c7p1*c9p2*ss[44] + 3780*c1p4*c3p2*c7p2*c9p1*ss[42] + 2520*c1p4*c3p3*c5p1*c7p1*ss[34] + 2520*c1p4*c3p3*c5p1*c9p1*ss[36] + 2520*c1p4*c3p3*c7p1*c9p1*ss[38] + 2520*c1p4*c5p1*c7p1*c9p3*ss[52] + 3780*c1p4*c5p1*c7p2*c9p2*ss[50] + 2520*c1p4*c5p1*c7p3*c9p1*ss[48] + 3780*c1p4*c5p2*c7p1*c9p2*ss[48] + 3780*c1p4*c5p2*c7p2*c9p1*ss[46] + 2520*c1p4*c5p3*c7p1*c9p1*ss[44] + 1512*c1p5*c3p1*c5p1*c7p2*ss[36] + 1512*c1p5*c3p1*c5p1*c9p2*ss[40] + 1512*c1p5*c3p1*c5p2*c7p1*ss[34] + 1512*c1p5*c3p1*c5p2*c9p1*ss[36] + 1512*c1p5*c3p1*c7p1*c9p2*ss[42] + 1512*c1p5*c3p1*c7p2*c9p1*ss[40] + 1512*c1p5*c3p2*c5p1*c7p1*ss[32] + 1512*c1p5*c3p2*c5p1*c9p1*ss[34] + 1512*c1p5*c3p2*c7p1*c9p1*ss[36] + 1512*c1p5*c5p1*c7p1*c9p2*ss[44] + 1512*c1p5*c5p1*c7p2*c9p1*ss[42] + 1512*c1p5*c5p2*c7p1*c9p1*ss[40] + 504*c1p6*c3p1*c5p1*c7p1*ss[30] + 504*c1p6*c3p1*c5p1*c9p1*ss[32] + 504*c1p6*c3p1*c7p1*c9p1*ss[34] + 504*c1p6*c5p1*c7p1*c9p1*ss[36] + 504*c3p1*c5p1*c7p1*c9p6*ss[78] + 1512*c3p1*c5p1*c7p2*c9p5*ss[76] + 2520*c3p1*c5p1*c7p3*c9p4*ss[74] + 2520*c3p1*c5p1*c7p4*c9p3*ss[72] + 1512*c3p1*c5p1*c7p5*c9p2*ss[70] + 504*c3p1*c5p1*c7p6*c9p1*ss[68] + 1512*c3p1*c5p2*c7p1*c9p5*ss[74] + 3780*c3p1*c5p2*c7p2*c9p4*ss[72] + 5040*c3p1*c5p2*c7p3*c9p3*ss[70] + 3780*c3p1*c5p2*c7p4*c9p2*ss[68] + 1512*c3p1*c5p2*c7p5*c9p1*ss[66] + 2520*c3p1*c5p3*c7p1*c9p4*ss[70] + 5040*c3p1*c5p3*c7p2*c9p3*ss[68] + 5040*c3p1*c5p3*c7p3*c9p2*ss[66] + 2520*c3p1*c5p3*c7p4*c9p1*ss[64] + 2520*c3p1*c5p4*c7p1*c9p3*ss[66] + 3780*c3p1*c5p4*c7p2*c9p2*ss[64] + 2520*c3p1*c5p4*c7p3*c9p1*ss[62] + 1512*c3p1*c5p5*c7p1*c9p2*ss[62] + 1512*c3p1*c5p5*c7p2*c9p1*ss[60] + 504*c3p1*c5p6*c7p1*c9p1*ss[58] + 1512*c3p2*c5p1*c7p1*c9p5*ss[72] + 3780*c3p2*c5p1*c7p2*c9p4*ss[70] + 5040*c3p2*c5p1*c7p3*c9p3*ss[68] + 3780*c3p2*c5p1*c7p4*c9p2*ss[66] + 1512*c3p2*c5p1*c7p5*c9p1*ss[64] + 3780*c3p2*c5p2*c7p1*c9p4*ss[68] + 7560*c3p2*c5p2*c7p2*c9p3*ss[66] + 7560*c3p2*c5p2*c7p3*c9p2*ss[64] + 3780*c3p2*c5p2*c7p4*c9p1*ss[62] + 5040*c3p2*c5p3*c7p1*c9p3*ss[64] + 7560*c3p2*c5p3*c7p2*c9p2*ss[62] + 5040*c3p2*c5p3*c7p3*c9p1*ss[60] + 3780*c3p2*c5p4*c7p1*c9p2*ss[60] + 3780*c3p2*c5p4*c7p2*c9p1*ss[58] + 1512*c3p2*c5p5*c7p1*c9p1*ss[56] + 2520*c3p3*c5p1*c7p1*c9p4*ss[66] + 5040*c3p3*c5p1*c7p2*c9p3*ss[64] + 5040*c3p3*c5p1*c7p3*c9p2*ss[62] + 2520*c3p3*c5p1*c7p4*c9p1*ss[60] + 5040*c3p3*c5p2*c7p1*c9p3*ss[62] + 7560*c3p3*c5p2*c7p2*c9p2*ss[60] + 5040*c3p3*c5p2*c7p3*c9p1*ss[58] + 5040*c3p3*c5p3*c7p1*c9p2*ss[58] + 5040*c3p3*c5p3*c7p2*c9p1*ss[56] + 2520*c3p3*c5p4*c7p1*c9p1*ss[54] + 2520*c3p4*c5p1*c7p1*c9p3*ss[60] + 3780*c3p4*c5p1*c7p2*c9p2*ss[58] + 2520*c3p4*c5p1*c7p3*c9p1*ss[56] + 3780*c3p4*c5p2*c7p1*c9p2*ss[56] + 3780*c3p4*c5p2*c7p2*c9p1*ss[54] + 2520*c3p4*c5p3*c7p1*c9p1*ss[52] + 1512*c3p5*c5p1*c7p1*c9p2*ss[54] + 1512*c3p5*c5p1*c7p2*c9p1*ss[52] + 1512*c3p5*c5p2*c7p1*c9p1*ss[50] + 504*c3p6*c5p1*c7p1*c9p1*ss[48] + 72*c1p1*c3p1*c5p7*ss[48] + 72*c1p1*c3p1*c7p7*ss[62] + 72*c1p1*c3p1*c9p7*ss[76] + 252*c1p1*c3p2*c5p6*ss[46] + 252*c1p1*c3p2*c7p6*ss[58] + 252*c1p1*c3p2*c9p6*ss[70] + 504*c1p1*c3p3*c5p5*ss[44] + 504*c1p1*c3p3*c7p5*ss[54] + 504*c1p1*c3p3*c9p5*ss[64] + 630*c1p1*c3p4*c5p4*ss[42] + 630*c1p1*c3p4*c7p4*ss[50] + 630*c1p1*c3p4*c9p4*ss[58] + 504*c1p1*c3p5*c5p3*ss[40] + 504*c1p1*c3p5*c7p3*ss[46] + 504*c1p1*c3p5*c9p3*ss[52] + 252*c1p1*c3p6*c5p2*ss[38] + 252*c1p1*c3p6*c7p2*ss[42] + 252*c1p1*c3p6*c9p2*ss[46] + 72*c1p1*c3p7*c5p1*ss[36] + 72*c1p1*c3p7*c7p1*ss[38] + 72*c1p1*c3p7*c9p1*ss[40] + 72*c1p1*c5p1*c7p7*ss[64] + 72*c1p1*c5p1*c9p7*ss[78] + 252*c1p1*c5p2*c7p6*ss[62] + 252*c1p1*c5p2*c9p6*ss[74] + 504*c1p1*c5p3*c7p5*ss[60] + 504*c1p1*c5p3*c9p5*ss[70] + 630*c1p1*c5p4*c7p4*ss[58] + 630*c1p1*c5p4*c9p4*ss[66] + 504*c1p1*c5p5*c7p3*ss[56] + 504*c1p1*c5p5*c9p3*ss[62] + 252*c1p1*c5p6*c7p2*ss[54] + 252*c1p1*c5p6*c9p2*ss[58] + 72*c1p1*c5p7*c7p1*ss[52] + 72*c1p1*c5p7*c9p1*ss[54] + 72*c1p1*c7p1*c9p7*ss[80] + 252*c1p1*c7p2*c9p6*ss[78] + 504*c1p1*c7p3*c9p5*ss[76] + 630*c1p1*c7p4*c9p4*ss[74] + 504*c1p1*c7p5*c9p3*ss[72] + 252*c1p1*c7p6*c9p2*ss[70] + 72*c1p1*c7p7*c9p1*ss[68] + 252*c1p2*c3p1*c5p6*ss[44] + 252*c1p2*c3p1*c7p6*ss[56] + 252*c1p2*c3p1*c9p6*ss[68] + 756*c1p2*c3p2*c5p5*ss[42] + 756*c1p2*c3p2*c7p5*ss[52] + 756*c1p2*c3p2*c9p5*ss[62] + 1260*c1p2*c3p3*c5p4*ss[40] + 1260*c1p2*c3p3*c7p4*ss[48] + 1260*c1p2*c3p3*c9p4*ss[56] + 1260*c1p2*c3p4*c5p3*ss[38] + 1260*c1p2*c3p4*c7p3*ss[44] + 1260*c1p2*c3p4*c9p3*ss[50] + 756*c1p2*c3p5*c5p2*ss[36] + 756*c1p2*c3p5*c7p2*ss[40] + 756*c1p2*c3p5*c9p2*ss[44] + 252*c1p2*c3p6*c5p1*ss[34] + 252*c1p2*c3p6*c7p1*ss[36] + 252*c1p2*c3p6*c9p1*ss[38] + 252*c1p2*c5p1*c7p6*ss[58] + 252*c1p2*c5p1*c9p6*ss[70] + 756*c1p2*c5p2*c7p5*ss[56] + 756*c1p2*c5p2*c9p5*ss[66] + 1260*c1p2*c5p3*c7p4*ss[54] + 1260*c1p2*c5p3*c9p4*ss[62] + 1260*c1p2*c5p4*c7p3*ss[52] + 1260*c1p2*c5p4*c9p3*ss[58] + 756*c1p2*c5p5*c7p2*ss[50] + 756*c1p2*c5p5*c9p2*ss[54] + 252*c1p2*c5p6*c7p1*ss[48] + 252*c1p2*c5p6*c9p1*ss[50] + 252*c1p2*c7p1*c9p6*ss[72] + 756*c1p2*c7p2*c9p5*ss[70] + 1260*c1p2*c7p3*c9p4*ss[68] + 1260*c1p2*c7p4*c9p3*ss[66] + 756*c1p2*c7p5*c9p2*ss[64] + 252*c1p2*c7p6*c9p1*ss[62] + 504*c1p3*c3p1*c5p5*ss[40] + 504*c1p3*c3p1*c7p5*ss[50] + 504*c1p3*c3p1*c9p5*ss[60] + 1260*c1p3*c3p2*c5p4*ss[38] + 1260*c1p3*c3p2*c7p4*ss[46] + 1260*c1p3*c3p2*c9p4*ss[54] + 1680*c1p3*c3p3*c5p3*ss[36] + 1680*c1p3*c3p3*c7p3*ss[42] + 1680*c1p3*c3p3*c9p3*ss[48] + 1260*c1p3*c3p4*c5p2*ss[34] + 1260*c1p3*c3p4*c7p2*ss[38] + 1260*c1p3*c3p4*c9p2*ss[42] + 504*c1p3*c3p5*c5p1*ss[32] + 504*c1p3*c3p5*c7p1*ss[34] + 504*c1p3*c3p5*c9p1*ss[36] + 504*c1p3*c5p1*c7p5*ss[52] + 504*c1p3*c5p1*c9p5*ss[62] + 1260*c1p3*c5p2*c7p4*ss[50] + 1260*c1p3*c5p2*c9p4*ss[58] + 1680*c1p3*c5p3*c7p3*ss[48] + 1680*c1p3*c5p3*c9p3*ss[54] + 1260*c1p3*c5p4*c7p2*ss[46] + 1260*c1p3*c5p4*c9p2*ss[50] + 504*c1p3*c5p5*c7p1*ss[44] + 504*c1p3*c5p5*c9p1*ss[46] + 504*c1p3*c7p1*c9p5*ss[64] + 1260*c1p3*c7p2*c9p4*ss[62] + 1680*c1p3*c7p3*c9p3*ss[60] + 1260*c1p3*c7p4*c9p2*ss[58] + 504*c1p3*c7p5*c9p1*ss[56] + 630*c1p4*c3p1*c5p4*ss[36] + 630*c1p4*c3p1*c7p4*ss[44] + 630*c1p4*c3p1*c9p4*ss[52] + 1260*c1p4*c3p2*c5p3*ss[34] + 1260*c1p4*c3p2*c7p3*ss[40] + 1260*c1p4*c3p2*c9p3*ss[46] + 1260*c1p4*c3p3*c5p2*ss[32] + 1260*c1p4*c3p3*c7p2*ss[36] + 1260*c1p4*c3p3*c9p2*ss[40] + 630*c1p4*c3p4*c5p1*ss[30] + 630*c1p4*c3p4*c7p1*ss[32] + 630*c1p4*c3p4*c9p1*ss[34] + 630*c1p4*c5p1*c7p4*ss[46] + 630*c1p4*c5p1*c9p4*ss[54] + 1260*c1p4*c5p2*c7p3*ss[44] + 1260*c1p4*c5p2*c9p3*ss[50] + 1260*c1p4*c5p3*c7p2*ss[42] + 1260*c1p4*c5p3*c9p2*ss[46] + 630*c1p4*c5p4*c7p1*ss[40] + 630*c1p4*c5p4*c9p1*ss[42] + 630*c1p4*c7p1*c9p4*ss[56] + 1260*c1p4*c7p2*c9p3*ss[54] + 1260*c1p4*c7p3*c9p2*ss[52] + 630*c1p4*c7p4*c9p1*ss[50] + 504*c1p5*c3p1*c5p3*ss[32] + 504*c1p5*c3p1*c7p3*ss[38] + 504*c1p5*c3p1*c9p3*ss[44] + 756*c1p5*c3p2*c5p2*ss[30] + 756*c1p5*c3p2*c7p2*ss[34] + 756*c1p5*c3p2*c9p2*ss[38] + 504*c1p5*c3p3*c5p1*ss[28] + 504*c1p5*c3p3*c7p1*ss[30] + 504*c1p5*c3p3*c9p1*ss[32] + 504*c1p5*c5p1*c7p3*ss[40] + 504*c1p5*c5p1*c9p3*ss[46] + 756*c1p5*c5p2*c7p2*ss[38] + 756*c1p5*c5p2*c9p2*ss[42] + 504*c1p5*c5p3*c7p1*ss[36] + 504*c1p5*c5p3*c9p1*ss[38] + 504*c1p5*c7p1*c9p3*ss[48] + 756*c1p5*c7p2*c9p2*ss[46] + 504*c1p5*c7p3*c9p1*ss[44] + 252*c1p6*c3p1*c5p2*ss[28] + 252*c1p6*c3p1*c7p2*ss[32] + 252*c1p6*c3p1*c9p2*ss[36] + 252*c1p6*c3p2*c5p1*ss[26] + 252*c1p6*c3p2*c7p1*ss[28] + 252*c1p6*c3p2*c9p1*ss[30] + 252*c1p6*c5p1*c7p2*ss[34] + 252*c1p6*c5p1*c9p2*ss[38] + 252*c1p6*c5p2*c7p1*ss[32] + 252*c1p6*c5p2*c9p1*ss[34] + 252*c1p6*c7p1*c9p2*ss[40] + 252*c1p6*c7p2*c9p1*ss[38] + 72*c1p7*c3p1*c5p1*ss[24] + 72*c1p7*c3p1*c7p1*ss[26] + 72*c1p7*c3p1*c9p1*ss[28] + 72*c1p7*c5p1*c7p1*ss[28] + 72*c1p7*c5p1*c9p1*ss[30] + 72*c1p7*c7p1*c9p1*ss[32] + 72*c3p1*c5p1*c7p7*ss[66] + 72*c3p1*c5p1*c9p7*ss[80] + 252*c3p1*c5p2*c7p6*ss[64] + 252*c3p1*c5p2*c9p6*ss[76] + 504*c3p1*c5p3*c7p5*ss[62] + 504*c3p1*c5p3*c9p5*ss[72] + 630*c3p1*c5p4*c7p4*ss[60] + 630*c3p1*c5p4*c9p4*ss[68] + 504*c3p1*c5p5*c7p3*ss[58] + 504*c3p1*c5p5*c9p3*ss[64] + 252*c3p1*c5p6*c7p2*ss[56] + 252*c3p1*c5p6*c9p2*ss[60] + 72*c3p1*c5p7*c7p1*ss[54] + 72*c3p1*c5p7*c9p1*ss[56] + 72*c3p1*c7p1*c9p7*ss[82] + 252*c3p1*c7p2*c9p6*ss[80] + 504*c3p1*c7p3*c9p5*ss[78] + 630*c3p1*c7p4*c9p4*ss[76] + 504*c3p1*c7p5*c9p3*ss[74] + 252*c3p1*c7p6*c9p2*ss[72] + 72*c3p1*c7p7*c9p1*ss[70] + 252*c3p2*c5p1*c7p6*ss[62] + 252*c3p2*c5p1*c9p6*ss[74] + 756*c3p2*c5p2*c7p5*ss[60] + 756*c3p2*c5p2*c9p5*ss[70] + 1260*c3p2*c5p3*c7p4*ss[58] + 1260*c3p2*c5p3*c9p4*ss[66] + 1260*c3p2*c5p4*c7p3*ss[56] + 1260*c3p2*c5p4*c9p3*ss[62] + 756*c3p2*c5p5*c7p2*ss[54] + 756*c3p2*c5p5*c9p2*ss[58] + 252*c3p2*c5p6*c7p1*ss[52] + 252*c3p2*c5p6*c9p1*ss[54] + 252*c3p2*c7p1*c9p6*ss[76] + 756*c3p2*c7p2*c9p5*ss[74] + 1260*c3p2*c7p3*c9p4*ss[72] + 1260*c3p2*c7p4*c9p3*ss[70] + 756*c3p2*c7p5*c9p2*ss[68] + 252*c3p2*c7p6*c9p1*ss[66] + 504*c3p3*c5p1*c7p5*ss[58] + 504*c3p3*c5p1*c9p5*ss[68] + 1260*c3p3*c5p2*c7p4*ss[56] + 1260*c3p3*c5p2*c9p4*ss[64] + 1680*c3p3*c5p3*c7p3*ss[54] + 1680*c3p3*c5p3*c9p3*ss[60] + 1260*c3p3*c5p4*c7p2*ss[52] + 1260*c3p3*c5p4*c9p2*ss[56] + 504*c3p3*c5p5*c7p1*ss[50] + 504*c3p3*c5p5*c9p1*ss[52] + 504*c3p3*c7p1*c9p5*ss[70] + 1260*c3p3*c7p2*c9p4*ss[68] + 1680*c3p3*c7p3*c9p3*ss[66] + 1260*c3p3*c7p4*c9p2*ss[64] + 504*c3p3*c7p5*c9p1*ss[62] + 630*c3p4*c5p1*c7p4*ss[54] + 630*c3p4*c5p1*c9p4*ss[62] + 1260*c3p4*c5p2*c7p3*ss[52] + 1260*c3p4*c5p2*c9p3*ss[58] + 1260*c3p4*c5p3*c7p2*ss[50] + 1260*c3p4*c5p3*c9p2*ss[54] + 630*c3p4*c5p4*c7p1*ss[48] + 630*c3p4*c5p4*c9p1*ss[50] + 630*c3p4*c7p1*c9p4*ss[64] + 1260*c3p4*c7p2*c9p3*ss[62] + 1260*c3p4*c7p3*c9p2*ss[60] + 630*c3p4*c7p4*c9p1*ss[58] + 504*c3p5*c5p1*c7p3*ss[50] + 504*c3p5*c5p1*c9p3*ss[56] + 756*c3p5*c5p2*c7p2*ss[48] + 756*c3p5*c5p2*c9p2*ss[52] + 504*c3p5*c5p3*c7p1*ss[46] + 504*c3p5*c5p3*c9p1*ss[48] + 504*c3p5*c7p1*c9p3*ss[58] + 756*c3p5*c7p2*c9p2*ss[56] + 504*c3p5*c7p3*c9p1*ss[54] + 252*c3p6*c5p1*c7p2*ss[46] + 252*c3p6*c5p1*c9p2*ss[50] + 252*c3p6*c5p2*c7p1*ss[44] + 252*c3p6*c5p2*c9p1*ss[46] + 252*c3p6*c7p1*c9p2*ss[52] + 252*c3p6*c7p2*c9p1*ss[50] + 72*c3p7*c5p1*c7p1*ss[42] + 72*c3p7*c5p1*c9p1*ss[44] + 72*c3p7*c7p1*c9p1*ss[46] + 72*c5p1*c7p1*c9p7*ss[84] + 252*c5p1*c7p2*c9p6*ss[82] + 504*c5p1*c7p3*c9p5*ss[80] + 630*c5p1*c7p4*c9p4*ss[78] + 504*c5p1*c7p5*c9p3*ss[76] + 252*c5p1*c7p6*c9p2*ss[74] + 72*c5p1*c7p7*c9p1*ss[72] + 252*c5p2*c7p1*c9p6*ss[80] + 756*c5p2*c7p2*c9p5*ss[78] + 1260*c5p2*c7p3*c9p4*ss[76] + 1260*c5p2*c7p4*c9p3*ss[74] + 756*c5p2*c7p5*c9p2*ss[72] + 252*c5p2*c7p6*c9p1*ss[70] + 504*c5p3*c7p1*c9p5*ss[76] + 1260*c5p3*c7p2*c9p4*ss[74] + 1680*c5p3*c7p3*c9p3*ss[72] + 1260*c5p3*c7p4*c9p2*ss[70] + 504*c5p3*c7p5*c9p1*ss[68] + 630*c5p4*c7p1*c9p4*ss[72] + 1260*c5p4*c7p2*c9p3*ss[70] + 1260*c5p4*c7p3*c9p2*ss[68] + 630*c5p4*c7p4*c9p1*ss[66] + 504*c5p5*c7p1*c9p3*ss[68] + 756*c5p5*c7p2*c9p2*ss[66] + 504*c5p5*c7p3*c9p1*ss[64] + 252*c5p6*c7p1*c9p2*ss[64] + 252*c5p6*c7p2*c9p1*ss[62] + 72*c5p7*c7p1*c9p1*ss[60] + 9*c1p1*c3p8*ss[34] + 9*c1p1*c5p8*ss[50] + 9*c1p1*c7p8*ss[66] + 9*c1p1*c9p8*ss[82] + 36*c1p2*c3p7*ss[32] + 36*c1p2*c5p7*ss[46] + 36*c1p2*c7p7*ss[60] + 36*c1p2*c9p7*ss[74] + 84*c1p3*c3p6*ss[30] + 84*c1p3*c5p6*ss[42] + 84*c1p3*c7p6*ss[54] + 84*c1p3*c9p6*ss[66] + 126*c1p4*c3p5*ss[28] + 126*c1p4*c5p5*ss[38] + 126*c1p4*c7p5*ss[48] + 126*c1p4*c9p5*ss[58] + 126*c1p5*c3p4*ss[26] + 126*c1p5*c5p4*ss[34] + 126*c1p5*c7p4*ss[42] + 126*c1p5*c9p4*ss[50] + 84*c1p6*c3p3*ss[24] + 84*c1p6*c5p3*ss[30] + 84*c1p6*c7p3*ss[36] + 84*c1p6*c9p3*ss[42] + 36*c1p7*c3p2*ss[22] + 36*c1p7*c5p2*ss[26] + 36*c1p7*c7p2*ss[30] + 36*c1p7*c9p2*ss[34] + 9*c1p8*c3p1*ss[20] + 9*c1p8*c5p1*ss[22] + 9*c1p8*c7p1*ss[24] + 9*c1p8*c9p1*ss[26] + 9*c3p1*c5p8*ss[52] + 9*c3p1*c7p8*ss[68] + 9*c3p1*c9p8*ss[84] + 36*c3p2*c5p7*ss[50] + 36*c3p2*c7p7*ss[64] + 36*c3p2*c9p7*ss[78] + 84*c3p3*c5p6*ss[48] + 84*c3p3*c7p6*ss[60] + 84*c3p3*c9p6*ss[72] + 126*c3p4*c5p5*ss[46] + 126*c3p4*c7p5*ss[56] + 126*c3p4*c9p5*ss[66] + 126*c3p5*c5p4*ss[44] + 126*c3p5*c7p4*ss[52] + 126*c3p5*c9p4*ss[60] + 84*c3p6*c5p3*ss[42] + 84*c3p6*c7p3*ss[48] + 84*c3p6*c9p3*ss[54] + 36*c3p7*c5p2*ss[40] + 36*c3p7*c7p2*ss[44] + 36*c3p7*c9p2*ss[48] + 9*c3p8*c5p1*ss[38] + 9*c3p8*c7p1*ss[40] + 9*c3p8*c9p1*ss[42] + 9*c5p1*c7p8*ss[70] + 9*c5p1*c9p8*ss[86] + 36*c5p2*c7p7*ss[68] + 36*c5p2*c9p7*ss[82] + 84*c5p3*c7p6*ss[66] + 84*c5p3*c9p6*ss[78] + 126*c5p4*c7p5*ss[64] + 126*c5p4*c9p5*ss[74] + 126*c5p5*c7p4*ss[62] + 126*c5p5*c9p4*ss[70] + 84*c5p6*c7p3*ss[60] + 84*c5p6*c9p3*ss[66] + 36*c5p7*c7p2*ss[58] + 36*c5p7*c9p2*ss[62] + 9*c5p8*c7p1*ss[56] + 9*c5p8*c9p1*ss[58] + 9*c7p1*c9p8*ss[88] + 36*c7p2*c9p7*ss[86] + 84*c7p3*c9p6*ss[84] + 126*c7p4*c9p5*ss[82] + 126*c7p5*c9p4*ss[80] + 84*c7p6*c9p3*ss[78] + 36*c7p7*c9p2*ss[76] + 9*c7p8*c9p1*ss[74] + c1p9*ss[18] + c3p9*ss[36] + c5p9*ss[54] + c7p9*ss[72] + c9p9*ss[90])*sys_part_4;
  TACC grad_4_den = convert<TACC>(sqrt(square(sys_part_0) + square(sys_part_1) + square(sys_part_2) + square(sys_part_3) + square(sys_part_4)));
  TACC metric_sqr = convert<TACC>(square(sys_part_0));
  metric_sqr += square(sys_part_1);
  metric_sqr += square(sys_part_2);
  metric_sqr += square(sys_part_3);
  metric_sqr += square(sys_part_4);
  gradient[0]=convert<T>(grad_0_num / grad_0_den);
  gradient[1]=convert<T>(grad_1_num / grad_1_den);
  gradient[2]=convert<T>(grad_2_num / grad_2_den);
  gradient[3]=convert<T>(grad_3_num / grad_3_den);
  gradient[4]=convert<T>(grad_4_num / grad_4_den);
  return convert<T>(sqrt(metric_sqr));
}
void roots(const T *parameters, T *roots) const {
  T c1p1 = parameters[0];
  T c3p1 = parameters[1];
  T c5p1 = parameters[2];
  T c7p1 = parameters[3];
  T c9p1 = parameters[4];
  T c1p2 = c1p1*c1p1;
  T c3p2 = c3p1*c3p1;
  T c5p2 = c5p1*c5p1;
  T c7p2 = c7p1*c7p1;
  T c9p2 = c9p1*c9p1;
  TACC root_0 = convert<TACC>(2*c1p1*c3p1*ss[4]);
  root_0 += 2*c1p1*c5p1*ss[6];
  root_0 += 2*c1p1*c7p1*ss[8];
  root_0 += 2*c1p1*c9p1*ss[10];
  root_0 += 2*c3p1*c5p1*ss[8];
  root_0 += 2*c3p1*c7p1*ss[10];
  root_0 += 2*c3p1*c9p1*ss[12];
  root_0 += 2*c5p1*c7p1*ss[12];
  root_0 += 2*c5p1*c9p1*ss[14];
  root_0 += 2*c7p1*c9p1*ss[16];
  root_0 += c1p2*ss[2];
  root_0 += c3p2*ss[6];
  root_0 += c5p2*ss[10];
  root_0 += c7p2*ss[14];
  root_0 += c9p2*ss[18];
  root_0 += -tmom[2];
  T c1p3 = c1p2*c1p1;
  T c1p4 = c1p3*c1p1;
  T c3p3 = c3p2*c3p1;
  T c3p4 = c3p3*c3p1;
  T c5p3 = c5p2*c5p1;
  T c5p4 = c5p3*c5p1;
  T c7p3 = c7p2*c7p1;
  T c7p4 = c7p3*c7p1;
  T c9p3 = c9p2*c9p1;
  T c9p4 = c9p3*c9p1;
  TACC root_1 = convert<TACC>(24*c1p1*c3p1*c5p1*c7p1*ss[16]);
  root_1 += 24*c1p1*c3p1*c5p1*c9p1*ss[18];
  root_1 += 24*c1p1*c3p1*c7p1*c9p1*ss[20];
  root_1 += 24*c1p1*c5p1*c7p1*c9p1*ss[22];
  root_1 += 24*c3p1*c5p1*c7p1*c9p1*ss[24];
  root_1 += 12*c1p1*c3p1*c5p2*ss[14];
  root_1 += 12*c1p1*c3p1*c7p2*ss[18];
  root_1 += 12*c1p1*c3p1*c9p2*ss[22];
  root_1 += 12*c1p1*c3p2*c5p1*ss[12];
  root_1 += 12*c1p1*c3p2*c7p1*ss[14];
  root_1 += 12*c1p1*c3p2*c9p1*ss[16];
  root_1 += 12*c1p1*c5p1*c7p2*ss[20];
  root_1 += 12*c1p1*c5p1*c9p2*ss[24];
  root_1 += 12*c1p1*c5p2*c7p1*ss[18];
  root_1 += 12*c1p1*c5p2*c9p1*ss[20];
  root_1 += 12*c1p1*c7p1*c9p2*ss[26];
  root_1 += 12*c1p1*c7p2*c9p1*ss[24];
  root_1 += 12*c1p2*c3p1*c5p1*ss[10];
  root_1 += 12*c1p2*c3p1*c7p1*ss[12];
  root_1 += 12*c1p2*c3p1*c9p1*ss[14];
  root_1 += 12*c1p2*c5p1*c7p1*ss[14];
  root_1 += 12*c1p2*c5p1*c9p1*ss[16];
  root_1 += 12*c1p2*c7p1*c9p1*ss[18];
  root_1 += 12*c3p1*c5p1*c7p2*ss[22];
  root_1 += 12*c3p1*c5p1*c9p2*ss[26];
  root_1 += 12*c3p1*c5p2*c7p1*ss[20];
  root_1 += 12*c3p1*c5p2*c9p1*ss[22];
  root_1 += 12*c3p1*c7p1*c9p2*ss[28];
  root_1 += 12*c3p1*c7p2*c9p1*ss[26];
  root_1 += 12*c3p2*c5p1*c7p1*ss[18];
  root_1 += 12*c3p2*c5p1*c9p1*ss[20];
  root_1 += 12*c3p2*c7p1*c9p1*ss[22];
  root_1 += 12*c5p1*c7p1*c9p2*ss[30];
  root_1 += 12*c5p1*c7p2*c9p1*ss[28];
  root_1 += 12*c5p2*c7p1*c9p1*ss[26];
  root_1 += 4*c1p1*c3p3*ss[10];
  root_1 += 4*c1p1*c5p3*ss[16];
  root_1 += 4*c1p1*c7p3*ss[22];
  root_1 += 4*c1p1*c9p3*ss[28];
  root_1 += 6*c1p2*c3p2*ss[8];
  root_1 += 6*c1p2*c5p2*ss[12];
  root_1 += 6*c1p2*c7p2*ss[16];
  root_1 += 6*c1p2*c9p2*ss[20];
  root_1 += 4*c1p3*c3p1*ss[6];
  root_1 += 4*c1p3*c5p1*ss[8];
  root_1 += 4*c1p3*c7p1*ss[10];
  root_1 += 4*c1p3*c9p1*ss[12];
  root_1 += 4*c3p1*c5p3*ss[18];
  root_1 += 4*c3p1*c7p3*ss[24];
  root_1 += 4*c3p1*c9p3*ss[30];
  root_1 += 6*c3p2*c5p2*ss[16];
  root_1 += 6*c3p2*c7p2*ss[20];
  root_1 += 6*c3p2*c9p2*ss[24];
  root_1 += 4*c3p3*c5p1*ss[14];
  root_1 += 4*c3p3*c7p1*ss[16];
  root_1 += 4*c3p3*c9p1*ss[18];
  root_1 += 4*c5p1*c7p3*ss[26];
  root_1 += 4*c5p1*c9p3*ss[32];
  root_1 += 6*c5p2*c7p2*ss[24];
  root_1 += 6*c5p2*c9p2*ss[28];
  root_1 += 4*c5p3*c7p1*ss[22];
  root_1 += 4*c5p3*c9p1*ss[24];
  root_1 += 4*c7p1*c9p3*ss[34];
  root_1 += 6*c7p2*c9p2*ss[32];
  root_1 += 4*c7p3*c9p1*ss[30];
  root_1 += c1p4*ss[4];
  root_1 += c3p4*ss[12];
  root_1 += c5p4*ss[20];
  root_1 += c7p4*ss[28];
  root_1 += c9p4*ss[36];
  root_1 += -tmom[4];
  T c1p5 = c1p4*c1p1;
  T c1p6 = c1p5*c1p1;
  T c3p5 = c3p4*c3p1;
  T c3p6 = c3p5*c3p1;
  T c5p5 = c5p4*c5p1;
  T c5p6 = c5p5*c5p1;
  T c7p5 = c7p4*c7p1;
  T c7p6 = c7p5*c7p1;
  T c9p5 = c9p4*c9p1;
  T c9p6 = c9p5*c9p1;
  TACC root_2 = convert<TACC>(360*c1p1*c3p1*c5p1*c7p1*c9p2*ss[34]);
  root_2 += 360*c1p1*c3p1*c5p1*c7p2*c9p1*ss[32];
  root_2 += 360*c1p1*c3p1*c5p2*c7p1*c9p1*ss[30];
  root_2 += 360*c1p1*c3p2*c5p1*c7p1*c9p1*ss[28];
  root_2 += 360*c1p2*c3p1*c5p1*c7p1*c9p1*ss[26];
  root_2 += 120*c1p1*c3p1*c5p1*c7p3*ss[30];
  root_2 += 120*c1p1*c3p1*c5p1*c9p3*ss[36];
  root_2 += 180*c1p1*c3p1*c5p2*c7p2*ss[28];
  root_2 += 180*c1p1*c3p1*c5p2*c9p2*ss[32];
  root_2 += 120*c1p1*c3p1*c5p3*c7p1*ss[26];
  root_2 += 120*c1p1*c3p1*c5p3*c9p1*ss[28];
  root_2 += 120*c1p1*c3p1*c7p1*c9p3*ss[38];
  root_2 += 180*c1p1*c3p1*c7p2*c9p2*ss[36];
  root_2 += 120*c1p1*c3p1*c7p3*c9p1*ss[34];
  root_2 += 180*c1p1*c3p2*c5p1*c7p2*ss[26];
  root_2 += 180*c1p1*c3p2*c5p1*c9p2*ss[30];
  root_2 += 180*c1p1*c3p2*c5p2*c7p1*ss[24];
  root_2 += 180*c1p1*c3p2*c5p2*c9p1*ss[26];
  root_2 += 180*c1p1*c3p2*c7p1*c9p2*ss[32];
  root_2 += 180*c1p1*c3p2*c7p2*c9p1*ss[30];
  root_2 += 120*c1p1*c3p3*c5p1*c7p1*ss[22];
  root_2 += 120*c1p1*c3p3*c5p1*c9p1*ss[24];
  root_2 += 120*c1p1*c3p3*c7p1*c9p1*ss[26];
  root_2 += 120*c1p1*c5p1*c7p1*c9p3*ss[40];
  root_2 += 180*c1p1*c5p1*c7p2*c9p2*ss[38];
  root_2 += 120*c1p1*c5p1*c7p3*c9p1*ss[36];
  root_2 += 180*c1p1*c5p2*c7p1*c9p2*ss[36];
  root_2 += 180*c1p1*c5p2*c7p2*c9p1*ss[34];
  root_2 += 120*c1p1*c5p3*c7p1*c9p1*ss[32];
  root_2 += 180*c1p2*c3p1*c5p1*c7p2*ss[24];
  root_2 += 180*c1p2*c3p1*c5p1*c9p2*ss[28];
  root_2 += 180*c1p2*c3p1*c5p2*c7p1*ss[22];
  root_2 += 180*c1p2*c3p1*c5p2*c9p1*ss[24];
  root_2 += 180*c1p2*c3p1*c7p1*c9p2*ss[30];
  root_2 += 180*c1p2*c3p1*c7p2*c9p1*ss[28];
  root_2 += 180*c1p2*c3p2*c5p1*c7p1*ss[20];
  root_2 += 180*c1p2*c3p2*c5p1*c9p1*ss[22];
  root_2 += 180*c1p2*c3p2*c7p1*c9p1*ss[24];
  root_2 += 180*c1p2*c5p1*c7p1*c9p2*ss[32];
  root_2 += 180*c1p2*c5p1*c7p2*c9p1*ss[30];
  root_2 += 180*c1p2*c5p2*c7p1*c9p1*ss[28];
  root_2 += 120*c1p3*c3p1*c5p1*c7p1*ss[18];
  root_2 += 120*c1p3*c3p1*c5p1*c9p1*ss[20];
  root_2 += 120*c1p3*c3p1*c7p1*c9p1*ss[22];
  root_2 += 120*c1p3*c5p1*c7p1*c9p1*ss[24];
  root_2 += 120*c3p1*c5p1*c7p1*c9p3*ss[42];
  root_2 += 180*c3p1*c5p1*c7p2*c9p2*ss[40];
  root_2 += 120*c3p1*c5p1*c7p3*c9p1*ss[38];
  root_2 += 180*c3p1*c5p2*c7p1*c9p2*ss[38];
  root_2 += 180*c3p1*c5p2*c7p2*c9p1*ss[36];
  root_2 += 120*c3p1*c5p3*c7p1*c9p1*ss[34];
  root_2 += 180*c3p2*c5p1*c7p1*c9p2*ss[36];
  root_2 += 180*c3p2*c5p1*c7p2*c9p1*ss[34];
  root_2 += 180*c3p2*c5p2*c7p1*c9p1*ss[32];
  root_2 += 120*c3p3*c5p1*c7p1*c9p1*ss[30];
  root_2 += 30*c1p1*c3p1*c5p4*ss[24];
  root_2 += 30*c1p1*c3p1*c7p4*ss[32];
  root_2 += 30*c1p1*c3p1*c9p4*ss[40];
  root_2 += 60*c1p1*c3p2*c5p3*ss[22];
  root_2 += 60*c1p1*c3p2*c7p3*ss[28];
  root_2 += 60*c1p1*c3p2*c9p3*ss[34];
  root_2 += 60*c1p1*c3p3*c5p2*ss[20];
  root_2 += 60*c1p1*c3p3*c7p2*ss[24];
  root_2 += 60*c1p1*c3p3*c9p2*ss[28];
  root_2 += 30*c1p1*c3p4*c5p1*ss[18];
  root_2 += 30*c1p1*c3p4*c7p1*ss[20];
  root_2 += 30*c1p1*c3p4*c9p1*ss[22];
  root_2 += 30*c1p1*c5p1*c7p4*ss[34];
  root_2 += 30*c1p1*c5p1*c9p4*ss[42];
  root_2 += 60*c1p1*c5p2*c7p3*ss[32];
  root_2 += 60*c1p1*c5p2*c9p3*ss[38];
  root_2 += 60*c1p1*c5p3*c7p2*ss[30];
  root_2 += 60*c1p1*c5p3*c9p2*ss[34];
  root_2 += 30*c1p1*c5p4*c7p1*ss[28];
  root_2 += 30*c1p1*c5p4*c9p1*ss[30];
  root_2 += 30*c1p1*c7p1*c9p4*ss[44];
  root_2 += 60*c1p1*c7p2*c9p3*ss[42];
  root_2 += 60*c1p1*c7p3*c9p2*ss[40];
  root_2 += 30*c1p1*c7p4*c9p1*ss[38];
  root_2 += 60*c1p2*c3p1*c5p3*ss[20];
  root_2 += 60*c1p2*c3p1*c7p3*ss[26];
  root_2 += 60*c1p2*c3p1*c9p3*ss[32];
  root_2 += 90*c1p2*c3p2*c5p2*ss[18];
  root_2 += 90*c1p2*c3p2*c7p2*ss[22];
  root_2 += 90*c1p2*c3p2*c9p2*ss[26];
  root_2 += 60*c1p2*c3p3*c5p1*ss[16];
  root_2 += 60*c1p2*c3p3*c7p1*ss[18];
  root_2 += 60*c1p2*c3p3*c9p1*ss[20];
  root_2 += 60*c1p2*c5p1*c7p3*ss[28];
  root_2 += 60*c1p2*c5p1*c9p3*ss[34];
  root_2 += 90*c1p2*c5p2*c7p2*ss[26];
  root_2 += 90*c1p2*c5p2*c9p2*ss[30];
  root_2 += 60*c1p2*c5p3*c7p1*ss[24];
  root_2 += 60*c1p2*c5p3*c9p1*ss[26];
  root_2 += 60*c1p2*c7p1*c9p3*ss[36];
  root_2 += 90*c1p2*c7p2*c9p2*ss[34];
  root_2 += 60*c1p2*c7p3*c9p1*ss[32];
  root_2 += 60*c1p3*c3p1*c5p2*ss[16];
  root_2 += 60*c1p3*c3p1*c7p2*ss[20];
  root_2 += 60*c1p3*c3p1*c9p2*ss[24];
  root_2 += 60*c1p3*c3p2*c5p1*ss[14];
  root_2 += 60*c1p3*c3p2*c7p1*ss[16];
  root_2 += 60*c1p3*c3p2*c9p1*ss[18];
  root_2 += 60*c1p3*c5p1*c7p2*ss[22];
  root_2 += 60*c1p3*c5p1*c9p2*ss[26];
  root_2 += 60*c1p3*c5p2*c7p1*ss[20];
  root_2 += 60*c1p3*c5p2*c9p1*ss[22];
  root_2 += 60*c1p3*c7p1*c9p2*ss[28];
  root_2 += 60*c1p3*c7p2*c9p1*ss[26];
  root_2 += 30*c1p4*c3p1*c5p1*ss[12];
  root_2 += 30*c1p4*c3p1*c7p1*ss[14];
  root_2 += 30*c1p4*c3p1*c9p1*ss[16];
  root_2 += 30*c1p4*c5p1*c7p1*ss[16];
  root_2 += 30*c1p4*c5p1*c9p1*ss[18];
  root_2 += 30*c1p4*c7p1*c9p1*ss[20];
  root_2 += 30*c3p1*c5p1*c7p4*ss[36];
  root_2 += 30*c3p1*c5p1*c9p4*ss[44];
  root_2 += 60*c3p1*c5p2*c7p3*ss[34];
  root_2 += 60*c3p1*c5p2*c9p3*ss[40];
  root_2 += 60*c3p1*c5p3*c7p2*ss[32];
  root_2 += 60*c3p1*c5p3*c9p2*ss[36];
  root_2 += 30*c3p1*c5p4*c7p1*ss[30];
  root_2 += 30*c3p1*c5p4*c9p1*ss[32];
  root_2 += 30*c3p1*c7p1*c9p4*ss[46];
  root_2 += 60*c3p1*c7p2*c9p3*ss[44];
  root_2 += 60*c3p1*c7p3*c9p2*ss[42];
  root_2 += 30*c3p1*c7p4*c9p1*ss[40];
  root_2 += 60*c3p2*c5p1*c7p3*ss[32];
  root_2 += 60*c3p2*c5p1*c9p3*ss[38];
  root_2 += 90*c3p2*c5p2*c7p2*ss[30];
  root_2 += 90*c3p2*c5p2*c9p2*ss[34];
  root_2 += 60*c3p2*c5p3*c7p1*ss[28];
  root_2 += 60*c3p2*c5p3*c9p1*ss[30];
  root_2 += 60*c3p2*c7p1*c9p3*ss[40];
  root_2 += 90*c3p2*c7p2*c9p2*ss[38];
  root_2 += 60*c3p2*c7p3*c9p1*ss[36];
  root_2 += 60*c3p3*c5p1*c7p2*ss[28];
  root_2 += 60*c3p3*c5p1*c9p2*ss[32];
  root_2 += 60*c3p3*c5p2*c7p1*ss[26];
  root_2 += 60*c3p3*c5p2*c9p1*ss[28];
  root_2 += 60*c3p3*c7p1*c9p2*ss[34];
  root_2 += 60*c3p3*c7p2*c9p1*ss[32];
  root_2 += 30*c3p4*c5p1*c7p1*ss[24];
  root_2 += 30*c3p4*c5p1*c9p1*ss[26];
  root_2 += 30*c3p4*c7p1*c9p1*ss[28];
  root_2 += 30*c5p1*c7p1*c9p4*ss[48];
  root_2 += 60*c5p1*c7p2*c9p3*ss[46];
  root_2 += 60*c5p1*c7p3*c9p2*ss[44];
  root_2 += 30*c5p1*c7p4*c9p1*ss[42];
  root_2 += 60*c5p2*c7p1*c9p3*ss[44];
  root_2 += 90*c5p2*c7p2*c9p2*ss[42];
  root_2 += 60*c5p2*c7p3*c9p1*ss[40];
  root_2 += 60*c5p3*c7p1*c9p2*ss[40];
  root_2 += 60*c5p3*c7p2*c9p1*ss[38];
  root_2 += 30*c5p4*c7p1*c9p1*ss[36];
  root_2 += 6*c1p1*c3p5*ss[16];
  root_2 += 6*c1p1*c5p5*ss[26];
  root_2 += 6*c1p1*c7p5*ss[36];
  root_2 += 6*c1p1*c9p5*ss[46];
  root_2 += 15*c1p2*c3p4*ss[14];
  root_2 += 15*c1p2*c5p4*ss[22];
  root_2 += 15*c1p2*c7p4*ss[30];
  root_2 += 15*c1p2*c9p4*ss[38];
  root_2 += 20*c1p3*c3p3*ss[12];
  root_2 += 20*c1p3*c5p3*ss[18];
  root_2 += 20*c1p3*c7p3*ss[24];
  root_2 += 20*c1p3*c9p3*ss[30];
  root_2 += 15*c1p4*c3p2*ss[10];
  root_2 += 15*c1p4*c5p2*ss[14];
  root_2 += 15*c1p4*c7p2*ss[18];
  root_2 += 15*c1p4*c9p2*ss[22];
  root_2 += 6*c1p5*c3p1*ss[8];
  root_2 += 6*c1p5*c5p1*ss[10];
  root_2 += 6*c1p5*c7p1*ss[12];
  root_2 += 6*c1p5*c9p1*ss[14];
  root_2 += 6*c3p1*c5p5*ss[28];
  root_2 += 6*c3p1*c7p5*ss[38];
  root_2 += 6*c3p1*c9p5*ss[48];
  root_2 += 15*c3p2*c5p4*ss[26];
  root_2 += 15*c3p2*c7p4*ss[34];
  root_2 += 15*c3p2*c9p4*ss[42];
  root_2 += 20*c3p3*c5p3*ss[24];
  root_2 += 20*c3p3*c7p3*ss[30];
  root_2 += 20*c3p3*c9p3*ss[36];
  root_2 += 15*c3p4*c5p2*ss[22];
  root_2 += 15*c3p4*c7p2*ss[26];
  root_2 += 15*c3p4*c9p2*ss[30];
  root_2 += 6*c3p5*c5p1*ss[20];
  root_2 += 6*c3p5*c7p1*ss[22];
  root_2 += 6*c3p5*c9p1*ss[24];
  root_2 += 6*c5p1*c7p5*ss[40];
  root_2 += 6*c5p1*c9p5*ss[50];
  root_2 += 15*c5p2*c7p4*ss[38];
  root_2 += 15*c5p2*c9p4*ss[46];
  root_2 += 20*c5p3*c7p3*ss[36];
  root_2 += 20*c5p3*c9p3*ss[42];
  root_2 += 15*c5p4*c7p2*ss[34];
  root_2 += 15*c5p4*c9p2*ss[38];
  root_2 += 6*c5p5*c7p1*ss[32];
  root_2 += 6*c5p5*c9p1*ss[34];
  root_2 += 6*c7p1*c9p5*ss[52];
  root_2 += 15*c7p2*c9p4*ss[50];
  root_2 += 20*c7p3*c9p3*ss[48];
  root_2 += 15*c7p4*c9p2*ss[46];
  root_2 += 6*c7p5*c9p1*ss[44];
  root_2 += c1p6*ss[6];
  root_2 += c3p6*ss[18];
  root_2 += c5p6*ss[30];
  root_2 += c7p6*ss[42];
  root_2 += c9p6*ss[54];
  root_2 += -tmom[6];
  T c1p7 = c1p6*c1p1;
  T c1p8 = c1p7*c1p1;
  T c3p7 = c3p6*c3p1;
  T c3p8 = c3p7*c3p1;
  T c5p7 = c5p6*c5p1;
  T c5p8 = c5p7*c5p1;
  T c7p7 = c7p6*c7p1;
  T c7p8 = c7p7*c7p1;
  T c9p7 = c9p6*c9p1;
  T c9p8 = c9p7*c9p1;
  TACC root_3 = convert<TACC>(1680*c1p1*c3p1*c5p1*c7p1*c9p4*ss[52]);
  root_3 += 3360*c1p1*c3p1*c5p1*c7p2*c9p3*ss[50];
  root_3 += 3360*c1p1*c3p1*c5p1*c7p3*c9p2*ss[48];
  root_3 += 1680*c1p1*c3p1*c5p1*c7p4*c9p1*ss[46];
  root_3 += 3360*c1p1*c3p1*c5p2*c7p1*c9p3*ss[48];
  root_3 += 5040*c1p1*c3p1*c5p2*c7p2*c9p2*ss[46];
  root_3 += 3360*c1p1*c3p1*c5p2*c7p3*c9p1*ss[44];
  root_3 += 3360*c1p1*c3p1*c5p3*c7p1*c9p2*ss[44];
  root_3 += 3360*c1p1*c3p1*c5p3*c7p2*c9p1*ss[42];
  root_3 += 1680*c1p1*c3p1*c5p4*c7p1*c9p1*ss[40];
  root_3 += 3360*c1p1*c3p2*c5p1*c7p1*c9p3*ss[46];
  root_3 += 5040*c1p1*c3p2*c5p1*c7p2*c9p2*ss[44];
  root_3 += 3360*c1p1*c3p2*c5p1*c7p3*c9p1*ss[42];
  root_3 += 5040*c1p1*c3p2*c5p2*c7p1*c9p2*ss[42];
  root_3 += 5040*c1p1*c3p2*c5p2*c7p2*c9p1*ss[40];
  root_3 += 3360*c1p1*c3p2*c5p3*c7p1*c9p1*ss[38];
  root_3 += 3360*c1p1*c3p3*c5p1*c7p1*c9p2*ss[40];
  root_3 += 3360*c1p1*c3p3*c5p1*c7p2*c9p1*ss[38];
  root_3 += 3360*c1p1*c3p3*c5p2*c7p1*c9p1*ss[36];
  root_3 += 1680*c1p1*c3p4*c5p1*c7p1*c9p1*ss[34];
  root_3 += 3360*c1p2*c3p1*c5p1*c7p1*c9p3*ss[44];
  root_3 += 5040*c1p2*c3p1*c5p1*c7p2*c9p2*ss[42];
  root_3 += 3360*c1p2*c3p1*c5p1*c7p3*c9p1*ss[40];
  root_3 += 5040*c1p2*c3p1*c5p2*c7p1*c9p2*ss[40];
  root_3 += 5040*c1p2*c3p1*c5p2*c7p2*c9p1*ss[38];
  root_3 += 3360*c1p2*c3p1*c5p3*c7p1*c9p1*ss[36];
  root_3 += 5040*c1p2*c3p2*c5p1*c7p1*c9p2*ss[38];
  root_3 += 5040*c1p2*c3p2*c5p1*c7p2*c9p1*ss[36];
  root_3 += 5040*c1p2*c3p2*c5p2*c7p1*c9p1*ss[34];
  root_3 += 3360*c1p2*c3p3*c5p1*c7p1*c9p1*ss[32];
  root_3 += 3360*c1p3*c3p1*c5p1*c7p1*c9p2*ss[36];
  root_3 += 3360*c1p3*c3p1*c5p1*c7p2*c9p1*ss[34];
  root_3 += 3360*c1p3*c3p1*c5p2*c7p1*c9p1*ss[32];
  root_3 += 3360*c1p3*c3p2*c5p1*c7p1*c9p1*ss[30];
  root_3 += 1680*c1p4*c3p1*c5p1*c7p1*c9p1*ss[28];
  root_3 += 336*c1p1*c3p1*c5p1*c7p5*ss[44];
  root_3 += 336*c1p1*c3p1*c5p1*c9p5*ss[54];
  root_3 += 840*c1p1*c3p1*c5p2*c7p4*ss[42];
  root_3 += 840*c1p1*c3p1*c5p2*c9p4*ss[50];
  root_3 += 1120*c1p1*c3p1*c5p3*c7p3*ss[40];
  root_3 += 1120*c1p1*c3p1*c5p3*c9p3*ss[46];
  root_3 += 840*c1p1*c3p1*c5p4*c7p2*ss[38];
  root_3 += 840*c1p1*c3p1*c5p4*c9p2*ss[42];
  root_3 += 336*c1p1*c3p1*c5p5*c7p1*ss[36];
  root_3 += 336*c1p1*c3p1*c5p5*c9p1*ss[38];
  root_3 += 336*c1p1*c3p1*c7p1*c9p5*ss[56];
  root_3 += 840*c1p1*c3p1*c7p2*c9p4*ss[54];
  root_3 += 1120*c1p1*c3p1*c7p3*c9p3*ss[52];
  root_3 += 840*c1p1*c3p1*c7p4*c9p2*ss[50];
  root_3 += 336*c1p1*c3p1*c7p5*c9p1*ss[48];
  root_3 += 840*c1p1*c3p2*c5p1*c7p4*ss[40];
  root_3 += 840*c1p1*c3p2*c5p1*c9p4*ss[48];
  root_3 += 1680*c1p1*c3p2*c5p2*c7p3*ss[38];
  root_3 += 1680*c1p1*c3p2*c5p2*c9p3*ss[44];
  root_3 += 1680*c1p1*c3p2*c5p3*c7p2*ss[36];
  root_3 += 1680*c1p1*c3p2*c5p3*c9p2*ss[40];
  root_3 += 840*c1p1*c3p2*c5p4*c7p1*ss[34];
  root_3 += 840*c1p1*c3p2*c5p4*c9p1*ss[36];
  root_3 += 840*c1p1*c3p2*c7p1*c9p4*ss[50];
  root_3 += 1680*c1p1*c3p2*c7p2*c9p3*ss[48];
  root_3 += 1680*c1p1*c3p2*c7p3*c9p2*ss[46];
  root_3 += 840*c1p1*c3p2*c7p4*c9p1*ss[44];
  root_3 += 1120*c1p1*c3p3*c5p1*c7p3*ss[36];
  root_3 += 1120*c1p1*c3p3*c5p1*c9p3*ss[42];
  root_3 += 1680*c1p1*c3p3*c5p2*c7p2*ss[34];
  root_3 += 1680*c1p1*c3p3*c5p2*c9p2*ss[38];
  root_3 += 1120*c1p1*c3p3*c5p3*c7p1*ss[32];
  root_3 += 1120*c1p1*c3p3*c5p3*c9p1*ss[34];
  root_3 += 1120*c1p1*c3p3*c7p1*c9p3*ss[44];
  root_3 += 1680*c1p1*c3p3*c7p2*c9p2*ss[42];
  root_3 += 1120*c1p1*c3p3*c7p3*c9p1*ss[40];
  root_3 += 840*c1p1*c3p4*c5p1*c7p2*ss[32];
  root_3 += 840*c1p1*c3p4*c5p1*c9p2*ss[36];
  root_3 += 840*c1p1*c3p4*c5p2*c7p1*ss[30];
  root_3 += 840*c1p1*c3p4*c5p2*c9p1*ss[32];
  root_3 += 840*c1p1*c3p4*c7p1*c9p2*ss[38];
  root_3 += 840*c1p1*c3p4*c7p2*c9p1*ss[36];
  root_3 += 336*c1p1*c3p5*c5p1*c7p1*ss[28];
  root_3 += 336*c1p1*c3p5*c5p1*c9p1*ss[30];
  root_3 += 336*c1p1*c3p5*c7p1*c9p1*ss[32];
  root_3 += 336*c1p1*c5p1*c7p1*c9p5*ss[58];
  root_3 += 840*c1p1*c5p1*c7p2*c9p4*ss[56];
  root_3 += 1120*c1p1*c5p1*c7p3*c9p3*ss[54];
  root_3 += 840*c1p1*c5p1*c7p4*c9p2*ss[52];
  root_3 += 336*c1p1*c5p1*c7p5*c9p1*ss[50];
  root_3 += 840*c1p1*c5p2*c7p1*c9p4*ss[54];
  root_3 += 1680*c1p1*c5p2*c7p2*c9p3*ss[52];
  root_3 += 1680*c1p1*c5p2*c7p3*c9p2*ss[50];
  root_3 += 840*c1p1*c5p2*c7p4*c9p1*ss[48];
  root_3 += 1120*c1p1*c5p3*c7p1*c9p3*ss[50];
  root_3 += 1680*c1p1*c5p3*c7p2*c9p2*ss[48];
  root_3 += 1120*c1p1*c5p3*c7p3*c9p1*ss[46];
  root_3 += 840*c1p1*c5p4*c7p1*c9p2*ss[46];
  root_3 += 840*c1p1*c5p4*c7p2*c9p1*ss[44];
  root_3 += 336*c1p1*c5p5*c7p1*c9p1*ss[42];
  root_3 += 840*c1p2*c3p1*c5p1*c7p4*ss[38];
  root_3 += 840*c1p2*c3p1*c5p1*c9p4*ss[46];
  root_3 += 1680*c1p2*c3p1*c5p2*c7p3*ss[36];
  root_3 += 1680*c1p2*c3p1*c5p2*c9p3*ss[42];
  root_3 += 1680*c1p2*c3p1*c5p3*c7p2*ss[34];
  root_3 += 1680*c1p2*c3p1*c5p3*c9p2*ss[38];
  root_3 += 840*c1p2*c3p1*c5p4*c7p1*ss[32];
  root_3 += 840*c1p2*c3p1*c5p4*c9p1*ss[34];
  root_3 += 840*c1p2*c3p1*c7p1*c9p4*ss[48];
  root_3 += 1680*c1p2*c3p1*c7p2*c9p3*ss[46];
  root_3 += 1680*c1p2*c3p1*c7p3*c9p2*ss[44];
  root_3 += 840*c1p2*c3p1*c7p4*c9p1*ss[42];
  root_3 += 1680*c1p2*c3p2*c5p1*c7p3*ss[34];
  root_3 += 1680*c1p2*c3p2*c5p1*c9p3*ss[40];
  root_3 += 2520*c1p2*c3p2*c5p2*c7p2*ss[32];
  root_3 += 2520*c1p2*c3p2*c5p2*c9p2*ss[36];
  root_3 += 1680*c1p2*c3p2*c5p3*c7p1*ss[30];
  root_3 += 1680*c1p2*c3p2*c5p3*c9p1*ss[32];
  root_3 += 1680*c1p2*c3p2*c7p1*c9p3*ss[42];
  root_3 += 2520*c1p2*c3p2*c7p2*c9p2*ss[40];
  root_3 += 1680*c1p2*c3p2*c7p3*c9p1*ss[38];
  root_3 += 1680*c1p2*c3p3*c5p1*c7p2*ss[30];
  root_3 += 1680*c1p2*c3p3*c5p1*c9p2*ss[34];
  root_3 += 1680*c1p2*c3p3*c5p2*c7p1*ss[28];
  root_3 += 1680*c1p2*c3p3*c5p2*c9p1*ss[30];
  root_3 += 1680*c1p2*c3p3*c7p1*c9p2*ss[36];
  root_3 += 1680*c1p2*c3p3*c7p2*c9p1*ss[34];
  root_3 += 840*c1p2*c3p4*c5p1*c7p1*ss[26];
  root_3 += 840*c1p2*c3p4*c5p1*c9p1*ss[28];
  root_3 += 840*c1p2*c3p4*c7p1*c9p1*ss[30];
  root_3 += 840*c1p2*c5p1*c7p1*c9p4*ss[50];
  root_3 += 1680*c1p2*c5p1*c7p2*c9p3*ss[48];
  root_3 += 1680*c1p2*c5p1*c7p3*c9p2*ss[46];
  root_3 += 840*c1p2*c5p1*c7p4*c9p1*ss[44];
  root_3 += 1680*c1p2*c5p2*c7p1*c9p3*ss[46];
  root_3 += 2520*c1p2*c5p2*c7p2*c9p2*ss[44];
  root_3 += 1680*c1p2*c5p2*c7p3*c9p1*ss[42];
  root_3 += 1680*c1p2*c5p3*c7p1*c9p2*ss[42];
  root_3 += 1680*c1p2*c5p3*c7p2*c9p1*ss[40];
  root_3 += 840*c1p2*c5p4*c7p1*c9p1*ss[38];
  root_3 += 1120*c1p3*c3p1*c5p1*c7p3*ss[32];
  root_3 += 1120*c1p3*c3p1*c5p1*c9p3*ss[38];
  root_3 += 1680*c1p3*c3p1*c5p2*c7p2*ss[30];
  root_3 += 1680*c1p3*c3p1*c5p2*c9p2*ss[34];
  root_3 += 1120*c1p3*c3p1*c5p3*c7p1*ss[28];
  root_3 += 1120*c1p3*c3p1*c5p3*c9p1*ss[30];
  root_3 += 1120*c1p3*c3p1*c7p1*c9p3*ss[40];
  root_3 += 1680*c1p3*c3p1*c7p2*c9p2*ss[38];
  root_3 += 1120*c1p3*c3p1*c7p3*c9p1*ss[36];
  root_3 += 1680*c1p3*c3p2*c5p1*c7p2*ss[28];
  root_3 += 1680*c1p3*c3p2*c5p1*c9p2*ss[32];
  root_3 += 1680*c1p3*c3p2*c5p2*c7p1*ss[26];
  root_3 += 1680*c1p3*c3p2*c5p2*c9p1*ss[28];
  root_3 += 1680*c1p3*c3p2*c7p1*c9p2*ss[34];
  root_3 += 1680*c1p3*c3p2*c7p2*c9p1*ss[32];
  root_3 += 1120*c1p3*c3p3*c5p1*c7p1*ss[24];
  root_3 += 1120*c1p3*c3p3*c5p1*c9p1*ss[26];
  root_3 += 1120*c1p3*c3p3*c7p1*c9p1*ss[28];
  root_3 += 1120*c1p3*c5p1*c7p1*c9p3*ss[42];
  root_3 += 1680*c1p3*c5p1*c7p2*c9p2*ss[40];
  root_3 += 1120*c1p3*c5p1*c7p3*c9p1*ss[38];
  root_3 += 1680*c1p3*c5p2*c7p1*c9p2*ss[38];
  root_3 += 1680*c1p3*c5p2*c7p2*c9p1*ss[36];
  root_3 += 1120*c1p3*c5p3*c7p1*c9p1*ss[34];
  root_3 += 840*c1p4*c3p1*c5p1*c7p2*ss[26];
  root_3 += 840*c1p4*c3p1*c5p1*c9p2*ss[30];
  root_3 += 840*c1p4*c3p1*c5p2*c7p1*ss[24];
  root_3 += 840*c1p4*c3p1*c5p2*c9p1*ss[26];
  root_3 += 840*c1p4*c3p1*c7p1*c9p2*ss[32];
  root_3 += 840*c1p4*c3p1*c7p2*c9p1*ss[30];
  root_3 += 840*c1p4*c3p2*c5p1*c7p1*ss[22];
  root_3 += 840*c1p4*c3p2*c5p1*c9p1*ss[24];
  root_3 += 840*c1p4*c3p2*c7p1*c9p1*ss[26];
  root_3 += 840*c1p4*c5p1*c7p1*c9p2*ss[34];
  root_3 += 840*c1p4*c5p1*c7p2*c9p1*ss[32];
  root_3 += 840*c1p4*c5p2*c7p1*c9p1*ss[30];
  root_3 += 336*c1p5*c3p1*c5p1*c7p1*ss[20];
  root_3 += 336*c1p5*c3p1*c5p1*c9p1*ss[22];
  root_3 += 336*c1p5*c3p1*c7p1*c9p1*ss[24];
  root_3 += 336*c1p5*c5p1*c7p1*c9p1*ss[26];
  root_3 += 336*c3p1*c5p1*c7p1*c9p5*ss[60];
  root_3 += 840*c3p1*c5p1*c7p2*c9p4*ss[58];
  root_3 += 1120*c3p1*c5p1*c7p3*c9p3*ss[56];
  root_3 += 840*c3p1*c5p1*c7p4*c9p2*ss[54];
  root_3 += 336*c3p1*c5p1*c7p5*c9p1*ss[52];
  root_3 += 840*c3p1*c5p2*c7p1*c9p4*ss[56];
  root_3 += 1680*c3p1*c5p2*c7p2*c9p3*ss[54];
  root_3 += 1680*c3p1*c5p2*c7p3*c9p2*ss[52];
  root_3 += 840*c3p1*c5p2*c7p4*c9p1*ss[50];
  root_3 += 1120*c3p1*c5p3*c7p1*c9p3*ss[52];
  root_3 += 1680*c3p1*c5p3*c7p2*c9p2*ss[50];
  root_3 += 1120*c3p1*c5p3*c7p3*c9p1*ss[48];
  root_3 += 840*c3p1*c5p4*c7p1*c9p2*ss[48];
  root_3 += 840*c3p1*c5p4*c7p2*c9p1*ss[46];
  root_3 += 336*c3p1*c5p5*c7p1*c9p1*ss[44];
  root_3 += 840*c3p2*c5p1*c7p1*c9p4*ss[54];
  root_3 += 1680*c3p2*c5p1*c7p2*c9p3*ss[52];
  root_3 += 1680*c3p2*c5p1*c7p3*c9p2*ss[50];
  root_3 += 840*c3p2*c5p1*c7p4*c9p1*ss[48];
  root_3 += 1680*c3p2*c5p2*c7p1*c9p3*ss[50];
  root_3 += 2520*c3p2*c5p2*c7p2*c9p2*ss[48];
  root_3 += 1680*c3p2*c5p2*c7p3*c9p1*ss[46];
  root_3 += 1680*c3p2*c5p3*c7p1*c9p2*ss[46];
  root_3 += 1680*c3p2*c5p3*c7p2*c9p1*ss[44];
  root_3 += 840*c3p2*c5p4*c7p1*c9p1*ss[42];
  root_3 += 1120*c3p3*c5p1*c7p1*c9p3*ss[48];
  root_3 += 1680*c3p3*c5p1*c7p2*c9p2*ss[46];
  root_3 += 1120*c3p3*c5p1*c7p3*c9p1*ss[44];
  root_3 += 1680*c3p3*c5p2*c7p1*c9p2*ss[44];
  root_3 += 1680*c3p3*c5p2*c7p2*c9p1*ss[42];
  root_3 += 1120*c3p3*c5p3*c7p1*c9p1*ss[40];
  root_3 += 840*c3p4*c5p1*c7p1*c9p2*ss[42];
  root_3 += 840*c3p4*c5p1*c7p2*c9p1*ss[40];
  root_3 += 840*c3p4*c5p2*c7p1*c9p1*ss[38];
  root_3 += 336*c3p5*c5p1*c7p1*c9p1*ss[36];
  root_3 += 56*c1p1*c3p1*c5p6*ss[34];
  root_3 += 56*c1p1*c3p1*c7p6*ss[46];
  root_3 += 56*c1p1*c3p1*c9p6*ss[58];
  root_3 += 168*c1p1*c3p2*c5p5*ss[32];
  root_3 += 168*c1p1*c3p2*c7p5*ss[42];
  root_3 += 168*c1p1*c3p2*c9p5*ss[52];
  root_3 += 280*c1p1*c3p3*c5p4*ss[30];
  root_3 += 280*c1p1*c3p3*c7p4*ss[38];
  root_3 += 280*c1p1*c3p3*c9p4*ss[46];
  root_3 += 280*c1p1*c3p4*c5p3*ss[28];
  root_3 += 280*c1p1*c3p4*c7p3*ss[34];
  root_3 += 280*c1p1*c3p4*c9p3*ss[40];
  root_3 += 168*c1p1*c3p5*c5p2*ss[26];
  root_3 += 168*c1p1*c3p5*c7p2*ss[30];
  root_3 += 168*c1p1*c3p5*c9p2*ss[34];
  root_3 += 56*c1p1*c3p6*c5p1*ss[24];
  root_3 += 56*c1p1*c3p6*c7p1*ss[26];
  root_3 += 56*c1p1*c3p6*c9p1*ss[28];
  root_3 += 56*c1p1*c5p1*c7p6*ss[48];
  root_3 += 56*c1p1*c5p1*c9p6*ss[60];
  root_3 += 168*c1p1*c5p2*c7p5*ss[46];
  root_3 += 168*c1p1*c5p2*c9p5*ss[56];
  root_3 += 280*c1p1*c5p3*c7p4*ss[44];
  root_3 += 280*c1p1*c5p3*c9p4*ss[52];
  root_3 += 280*c1p1*c5p4*c7p3*ss[42];
  root_3 += 280*c1p1*c5p4*c9p3*ss[48];
  root_3 += 168*c1p1*c5p5*c7p2*ss[40];
  root_3 += 168*c1p1*c5p5*c9p2*ss[44];
  root_3 += 56*c1p1*c5p6*c7p1*ss[38];
  root_3 += 56*c1p1*c5p6*c9p1*ss[40];
  root_3 += 56*c1p1*c7p1*c9p6*ss[62];
  root_3 += 168*c1p1*c7p2*c9p5*ss[60];
  root_3 += 280*c1p1*c7p3*c9p4*ss[58];
  root_3 += 280*c1p1*c7p4*c9p3*ss[56];
  root_3 += 168*c1p1*c7p5*c9p2*ss[54];
  root_3 += 56*c1p1*c7p6*c9p1*ss[52];
  root_3 += 168*c1p2*c3p1*c5p5*ss[30];
  root_3 += 168*c1p2*c3p1*c7p5*ss[40];
  root_3 += 168*c1p2*c3p1*c9p5*ss[50];
  root_3 += 420*c1p2*c3p2*c5p4*ss[28];
  root_3 += 420*c1p2*c3p2*c7p4*ss[36];
  root_3 += 420*c1p2*c3p2*c9p4*ss[44];
  root_3 += 560*c1p2*c3p3*c5p3*ss[26];
  root_3 += 560*c1p2*c3p3*c7p3*ss[32];
  root_3 += 560*c1p2*c3p3*c9p3*ss[38];
  root_3 += 420*c1p2*c3p4*c5p2*ss[24];
  root_3 += 420*c1p2*c3p4*c7p2*ss[28];
  root_3 += 420*c1p2*c3p4*c9p2*ss[32];
  root_3 += 168*c1p2*c3p5*c5p1*ss[22];
  root_3 += 168*c1p2*c3p5*c7p1*ss[24];
  root_3 += 168*c1p2*c3p5*c9p1*ss[26];
  root_3 += 168*c1p2*c5p1*c7p5*ss[42];
  root_3 += 168*c1p2*c5p1*c9p5*ss[52];
  root_3 += 420*c1p2*c5p2*c7p4*ss[40];
  root_3 += 420*c1p2*c5p2*c9p4*ss[48];
  root_3 += 560*c1p2*c5p3*c7p3*ss[38];
  root_3 += 560*c1p2*c5p3*c9p3*ss[44];
  root_3 += 420*c1p2*c5p4*c7p2*ss[36];
  root_3 += 420*c1p2*c5p4*c9p2*ss[40];
  root_3 += 168*c1p2*c5p5*c7p1*ss[34];
  root_3 += 168*c1p2*c5p5*c9p1*ss[36];
  root_3 += 168*c1p2*c7p1*c9p5*ss[54];
  root_3 += 420*c1p2*c7p2*c9p4*ss[52];
  root_3 += 560*c1p2*c7p3*c9p3*ss[50];
  root_3 += 420*c1p2*c7p4*c9p2*ss[48];
  root_3 += 168*c1p2*c7p5*c9p1*ss[46];
  root_3 += 280*c1p3*c3p1*c5p4*ss[26];
  root_3 += 280*c1p3*c3p1*c7p4*ss[34];
  root_3 += 280*c1p3*c3p1*c9p4*ss[42];
  root_3 += 560*c1p3*c3p2*c5p3*ss[24];
  root_3 += 560*c1p3*c3p2*c7p3*ss[30];
  root_3 += 560*c1p3*c3p2*c9p3*ss[36];
  root_3 += 560*c1p3*c3p3*c5p2*ss[22];
  root_3 += 560*c1p3*c3p3*c7p2*ss[26];
  root_3 += 560*c1p3*c3p3*c9p2*ss[30];
  root_3 += 280*c1p3*c3p4*c5p1*ss[20];
  root_3 += 280*c1p3*c3p4*c7p1*ss[22];
  root_3 += 280*c1p3*c3p4*c9p1*ss[24];
  root_3 += 280*c1p3*c5p1*c7p4*ss[36];
  root_3 += 280*c1p3*c5p1*c9p4*ss[44];
  root_3 += 560*c1p3*c5p2*c7p3*ss[34];
  root_3 += 560*c1p3*c5p2*c9p3*ss[40];
  root_3 += 560*c1p3*c5p3*c7p2*ss[32];
  root_3 += 560*c1p3*c5p3*c9p2*ss[36];
  root_3 += 280*c1p3*c5p4*c7p1*ss[30];
  root_3 += 280*c1p3*c5p4*c9p1*ss[32];
  root_3 += 280*c1p3*c7p1*c9p4*ss[46];
  root_3 += 560*c1p3*c7p2*c9p3*ss[44];
  root_3 += 560*c1p3*c7p3*c9p2*ss[42];
  root_3 += 280*c1p3*c7p4*c9p1*ss[40];
  root_3 += 280*c1p4*c3p1*c5p3*ss[22];
  root_3 += 280*c1p4*c3p1*c7p3*ss[28];
  root_3 += 280*c1p4*c3p1*c9p3*ss[34];
  root_3 += 420*c1p4*c3p2*c5p2*ss[20];
  root_3 += 420*c1p4*c3p2*c7p2*ss[24];
  root_3 += 420*c1p4*c3p2*c9p2*ss[28];
  root_3 += 280*c1p4*c3p3*c5p1*ss[18];
  root_3 += 280*c1p4*c3p3*c7p1*ss[20];
  root_3 += 280*c1p4*c3p3*c9p1*ss[22];
  root_3 += 280*c1p4*c5p1*c7p3*ss[30];
  root_3 += 280*c1p4*c5p1*c9p3*ss[36];
  root_3 += 420*c1p4*c5p2*c7p2*ss[28];
  root_3 += 420*c1p4*c5p2*c9p2*ss[32];
  root_3 += 280*c1p4*c5p3*c7p1*ss[26];
  root_3 += 280*c1p4*c5p3*c9p1*ss[28];
  root_3 += 280*c1p4*c7p1*c9p3*ss[38];
  root_3 += 420*c1p4*c7p2*c9p2*ss[36];
  root_3 += 280*c1p4*c7p3*c9p1*ss[34];
  root_3 += 168*c1p5*c3p1*c5p2*ss[18];
  root_3 += 168*c1p5*c3p1*c7p2*ss[22];
  root_3 += 168*c1p5*c3p1*c9p2*ss[26];
  root_3 += 168*c1p5*c3p2*c5p1*ss[16];
  root_3 += 168*c1p5*c3p2*c7p1*ss[18];
  root_3 += 168*c1p5*c3p2*c9p1*ss[20];
  root_3 += 168*c1p5*c5p1*c7p2*ss[24];
  root_3 += 168*c1p5*c5p1*c9p2*ss[28];
  root_3 += 168*c1p5*c5p2*c7p1*ss[22];
  root_3 += 168*c1p5*c5p2*c9p1*ss[24];
  root_3 += 168*c1p5*c7p1*c9p2*ss[30];
  root_3 += 168*c1p5*c7p2*c9p1*ss[28];
  root_3 += 56*c1p6*c3p1*c5p1*ss[14];
  root_3 += 56*c1p6*c3p1*c7p1*ss[16];
  root_3 += 56*c1p6*c3p1*c9p1*ss[18];
  root_3 += 56*c1p6*c5p1*c7p1*ss[18];
  root_3 += 56*c1p6*c5p1*c9p1*ss[20];
  root_3 += 56*c1p6*c7p1*c9p1*ss[22];
  root_3 += 56*c3p1*c5p1*c7p6*ss[50];
  root_3 += 56*c3p1*c5p1*c9p6*ss[62];
  root_3 += 168*c3p1*c5p2*c7p5*ss[48];
  root_3 += 168*c3p1*c5p2*c9p5*ss[58];
  root_3 += 280*c3p1*c5p3*c7p4*ss[46];
  root_3 += 280*c3p1*c5p3*c9p4*ss[54];
  root_3 += 280*c3p1*c5p4*c7p3*ss[44];
  root_3 += 280*c3p1*c5p4*c9p3*ss[50];
  root_3 += 168*c3p1*c5p5*c7p2*ss[42];
  root_3 += 168*c3p1*c5p5*c9p2*ss[46];
  root_3 += 56*c3p1*c5p6*c7p1*ss[40];
  root_3 += 56*c3p1*c5p6*c9p1*ss[42];
  root_3 += 56*c3p1*c7p1*c9p6*ss[64];
  root_3 += 168*c3p1*c7p2*c9p5*ss[62];
  root_3 += 280*c3p1*c7p3*c9p4*ss[60];
  root_3 += 280*c3p1*c7p4*c9p3*ss[58];
  root_3 += 168*c3p1*c7p5*c9p2*ss[56];
  root_3 += 56*c3p1*c7p6*c9p1*ss[54];
  root_3 += 168*c3p2*c5p1*c7p5*ss[46];
  root_3 += 168*c3p2*c5p1*c9p5*ss[56];
  root_3 += 420*c3p2*c5p2*c7p4*ss[44];
  root_3 += 420*c3p2*c5p2*c9p4*ss[52];
  root_3 += 560*c3p2*c5p3*c7p3*ss[42];
  root_3 += 560*c3p2*c5p3*c9p3*ss[48];
  root_3 += 420*c3p2*c5p4*c7p2*ss[40];
  root_3 += 420*c3p2*c5p4*c9p2*ss[44];
  root_3 += 168*c3p2*c5p5*c7p1*ss[38];
  root_3 += 168*c3p2*c5p5*c9p1*ss[40];
  root_3 += 168*c3p2*c7p1*c9p5*ss[58];
  root_3 += 420*c3p2*c7p2*c9p4*ss[56];
  root_3 += 560*c3p2*c7p3*c9p3*ss[54];
  root_3 += 420*c3p2*c7p4*c9p2*ss[52];
  root_3 += 168*c3p2*c7p5*c9p1*ss[50];
  root_3 += 280*c3p3*c5p1*c7p4*ss[42];
  root_3 += 280*c3p3*c5p1*c9p4*ss[50];
  root_3 += 560*c3p3*c5p2*c7p3*ss[40];
  root_3 += 560*c3p3*c5p2*c9p3*ss[46];
  root_3 += 560*c3p3*c5p3*c7p2*ss[38];
  root_3 += 560*c3p3*c5p3*c9p2*ss[42];
  root_3 += 280*c3p3*c5p4*c7p1*ss[36];
  root_3 += 280*c3p3*c5p4*c9p1*ss[38];
  root_3 += 280*c3p3*c7p1*c9p4*ss[52];
  root_3 += 560*c3p3*c7p2*c9p3*ss[50];
  root_3 += 560*c3p3*c7p3*c9p2*ss[48];
  root_3 += 280*c3p3*c7p4*c9p1*ss[46];
  root_3 += 280*c3p4*c5p1*c7p3*ss[38];
  root_3 += 280*c3p4*c5p1*c9p3*ss[44];
  root_3 += 420*c3p4*c5p2*c7p2*ss[36];
  root_3 += 420*c3p4*c5p2*c9p2*ss[40];
  root_3 += 280*c3p4*c5p3*c7p1*ss[34];
  root_3 += 280*c3p4*c5p3*c9p1*ss[36];
  root_3 += 280*c3p4*c7p1*c9p3*ss[46];
  root_3 += 420*c3p4*c7p2*c9p2*ss[44];
  root_3 += 280*c3p4*c7p3*c9p1*ss[42];
  root_3 += 168*c3p5*c5p1*c7p2*ss[34];
  root_3 += 168*c3p5*c5p1*c9p2*ss[38];
  root_3 += 168*c3p5*c5p2*c7p1*ss[32];
  root_3 += 168*c3p5*c5p2*c9p1*ss[34];
  root_3 += 168*c3p5*c7p1*c9p2*ss[40];
  root_3 += 168*c3p5*c7p2*c9p1*ss[38];
  root_3 += 56*c3p6*c5p1*c7p1*ss[30];
  root_3 += 56*c3p6*c5p1*c9p1*ss[32];
  root_3 += 56*c3p6*c7p1*c9p1*ss[34];
  root_3 += 56*c5p1*c7p1*c9p6*ss[66];
  root_3 += 168*c5p1*c7p2*c9p5*ss[64];
  root_3 += 280*c5p1*c7p3*c9p4*ss[62];
  root_3 += 280*c5p1*c7p4*c9p3*ss[60];
  root_3 += 168*c5p1*c7p5*c9p2*ss[58];
  root_3 += 56*c5p1*c7p6*c9p1*ss[56];
  root_3 += 168*c5p2*c7p1*c9p5*ss[62];
  root_3 += 420*c5p2*c7p2*c9p4*ss[60];
  root_3 += 560*c5p2*c7p3*c9p3*ss[58];
  root_3 += 420*c5p2*c7p4*c9p2*ss[56];
  root_3 += 168*c5p2*c7p5*c9p1*ss[54];
  root_3 += 280*c5p3*c7p1*c9p4*ss[58];
  root_3 += 560*c5p3*c7p2*c9p3*ss[56];
  root_3 += 560*c5p3*c7p3*c9p2*ss[54];
  root_3 += 280*c5p3*c7p4*c9p1*ss[52];
  root_3 += 280*c5p4*c7p1*c9p3*ss[54];
  root_3 += 420*c5p4*c7p2*c9p2*ss[52];
  root_3 += 280*c5p4*c7p3*c9p1*ss[50];
  root_3 += 168*c5p5*c7p1*c9p2*ss[50];
  root_3 += 168*c5p5*c7p2*c9p1*ss[48];
  root_3 += 56*c5p6*c7p1*c9p1*ss[46];
  root_3 += 8*c1p1*c3p7*ss[22];
  root_3 += 8*c1p1*c5p7*ss[36];
  root_3 += 8*c1p1*c7p7*ss[50];
  root_3 += 8*c1p1*c9p7*ss[64];
  root_3 += 28*c1p2*c3p6*ss[20];
  root_3 += 28*c1p2*c5p6*ss[32];
  root_3 += 28*c1p2*c7p6*ss[44];
  root_3 += 28*c1p2*c9p6*ss[56];
  root_3 += 56*c1p3*c3p5*ss[18];
  root_3 += 56*c1p3*c5p5*ss[28];
  root_3 += 56*c1p3*c7p5*ss[38];
  root_3 += 56*c1p3*c9p5*ss[48];
  root_3 += 70*c1p4*c3p4*ss[16];
  root_3 += 70*c1p4*c5p4*ss[24];
  root_3 += 70*c1p4*c7p4*ss[32];
  root_3 += 70*c1p4*c9p4*ss[40];
  root_3 += 56*c1p5*c3p3*ss[14];
  root_3 += 56*c1p5*c5p3*ss[20];
  root_3 += 56*c1p5*c7p3*ss[26];
  root_3 += 56*c1p5*c9p3*ss[32];
  root_3 += 28*c1p6*c3p2*ss[12];
  root_3 += 28*c1p6*c5p2*ss[16];
  root_3 += 28*c1p6*c7p2*ss[20];
  root_3 += 28*c1p6*c9p2*ss[24];
  root_3 += 8*c1p7*c3p1*ss[10];
  root_3 += 8*c1p7*c5p1*ss[12];
  root_3 += 8*c1p7*c7p1*ss[14];
  root_3 += 8*c1p7*c9p1*ss[16];
  root_3 += 8*c3p1*c5p7*ss[38];
  root_3 += 8*c3p1*c7p7*ss[52];
  root_3 += 8*c3p1*c9p7*ss[66];
  root_3 += 28*c3p2*c5p6*ss[36];
  root_3 += 28*c3p2*c7p6*ss[48];
  root_3 += 28*c3p2*c9p6*ss[60];
  root_3 += 56*c3p3*c5p5*ss[34];
  root_3 += 56*c3p3*c7p5*ss[44];
  root_3 += 56*c3p3*c9p5*ss[54];
  root_3 += 70*c3p4*c5p4*ss[32];
  root_3 += 70*c3p4*c7p4*ss[40];
  root_3 += 70*c3p4*c9p4*ss[48];
  root_3 += 56*c3p5*c5p3*ss[30];
  root_3 += 56*c3p5*c7p3*ss[36];
  root_3 += 56*c3p5*c9p3*ss[42];
  root_3 += 28*c3p6*c5p2*ss[28];
  root_3 += 28*c3p6*c7p2*ss[32];
  root_3 += 28*c3p6*c9p2*ss[36];
  root_3 += 8*c3p7*c5p1*ss[26];
  root_3 += 8*c3p7*c7p1*ss[28];
  root_3 += 8*c3p7*c9p1*ss[30];
  root_3 += 8*c5p1*c7p7*ss[54];
  root_3 += 8*c5p1*c9p7*ss[68];
  root_3 += 28*c5p2*c7p6*ss[52];
  root_3 += 28*c5p2*c9p6*ss[64];
  root_3 += 56*c5p3*c7p5*ss[50];
  root_3 += 56*c5p3*c9p5*ss[60];
  root_3 += 70*c5p4*c7p4*ss[48];
  root_3 += 70*c5p4*c9p4*ss[56];
  root_3 += 56*c5p5*c7p3*ss[46];
  root_3 += 56*c5p5*c9p3*ss[52];
  root_3 += 28*c5p6*c7p2*ss[44];
  root_3 += 28*c5p6*c9p2*ss[48];
  root_3 += 8*c5p7*c7p1*ss[42];
  root_3 += 8*c5p7*c9p1*ss[44];
  root_3 += 8*c7p1*c9p7*ss[70];
  root_3 += 28*c7p2*c9p6*ss[68];
  root_3 += 56*c7p3*c9p5*ss[66];
  root_3 += 70*c7p4*c9p4*ss[64];
  root_3 += 56*c7p5*c9p3*ss[62];
  root_3 += 28*c7p6*c9p2*ss[60];
  root_3 += 8*c7p7*c9p1*ss[58];
  root_3 += c1p8*ss[8];
  root_3 += c3p8*ss[24];
  root_3 += c5p8*ss[40];
  root_3 += c7p8*ss[56];
  root_3 += c9p8*ss[72];
  root_3 += -tmom[8];
  T c1p9 = c1p8*c1p1;
  T c1p10 = c1p9*c1p1;
  T c3p9 = c3p8*c3p1;
  T c3p10 = c3p9*c3p1;
  T c5p9 = c5p8*c5p1;
  T c5p10 = c5p9*c5p1;
  T c7p9 = c7p8*c7p1;
  T c7p10 = c7p9*c7p1;
  T c9p9 = c9p8*c9p1;
  T c9p10 = c9p9*c9p1;
  TACC root_4 = convert<TACC>(5040*c1p1*c3p1*c5p1*c7p1*c9p6*ss[70]);
  root_4 += 15120*c1p1*c3p1*c5p1*c7p2*c9p5*ss[68];
  root_4 += 25200*c1p1*c3p1*c5p1*c7p3*c9p4*ss[66];
  root_4 += 25200*c1p1*c3p1*c5p1*c7p4*c9p3*ss[64];
  root_4 += 15120*c1p1*c3p1*c5p1*c7p5*c9p2*ss[62];
  root_4 += 5040*c1p1*c3p1*c5p1*c7p6*c9p1*ss[60];
  root_4 += 15120*c1p1*c3p1*c5p2*c7p1*c9p5*ss[66];
  root_4 += 37800*c1p1*c3p1*c5p2*c7p2*c9p4*ss[64];
  root_4 += 50400*c1p1*c3p1*c5p2*c7p3*c9p3*ss[62];
  root_4 += 37800*c1p1*c3p1*c5p2*c7p4*c9p2*ss[60];
  root_4 += 15120*c1p1*c3p1*c5p2*c7p5*c9p1*ss[58];
  root_4 += 25200*c1p1*c3p1*c5p3*c7p1*c9p4*ss[62];
  root_4 += 50400*c1p1*c3p1*c5p3*c7p2*c9p3*ss[60];
  root_4 += 50400*c1p1*c3p1*c5p3*c7p3*c9p2*ss[58];
  root_4 += 25200*c1p1*c3p1*c5p3*c7p4*c9p1*ss[56];
  root_4 += 25200*c1p1*c3p1*c5p4*c7p1*c9p3*ss[58];
  root_4 += 37800*c1p1*c3p1*c5p4*c7p2*c9p2*ss[56];
  root_4 += 25200*c1p1*c3p1*c5p4*c7p3*c9p1*ss[54];
  root_4 += 15120*c1p1*c3p1*c5p5*c7p1*c9p2*ss[54];
  root_4 += 15120*c1p1*c3p1*c5p5*c7p2*c9p1*ss[52];
  root_4 += 5040*c1p1*c3p1*c5p6*c7p1*c9p1*ss[50];
  root_4 += 15120*c1p1*c3p2*c5p1*c7p1*c9p5*ss[64];
  root_4 += 37800*c1p1*c3p2*c5p1*c7p2*c9p4*ss[62];
  root_4 += 50400*c1p1*c3p2*c5p1*c7p3*c9p3*ss[60];
  root_4 += 37800*c1p1*c3p2*c5p1*c7p4*c9p2*ss[58];
  root_4 += 15120*c1p1*c3p2*c5p1*c7p5*c9p1*ss[56];
  root_4 += 37800*c1p1*c3p2*c5p2*c7p1*c9p4*ss[60];
  root_4 += 75600*c1p1*c3p2*c5p2*c7p2*c9p3*ss[58];
  root_4 += 75600*c1p1*c3p2*c5p2*c7p3*c9p2*ss[56];
  root_4 += 37800*c1p1*c3p2*c5p2*c7p4*c9p1*ss[54];
  root_4 += 50400*c1p1*c3p2*c5p3*c7p1*c9p3*ss[56];
  root_4 += 75600*c1p1*c3p2*c5p3*c7p2*c9p2*ss[54];
  root_4 += 50400*c1p1*c3p2*c5p3*c7p3*c9p1*ss[52];
  root_4 += 37800*c1p1*c3p2*c5p4*c7p1*c9p2*ss[52];
  root_4 += 37800*c1p1*c3p2*c5p4*c7p2*c9p1*ss[50];
  root_4 += 15120*c1p1*c3p2*c5p5*c7p1*c9p1*ss[48];
  root_4 += 25200*c1p1*c3p3*c5p1*c7p1*c9p4*ss[58];
  root_4 += 50400*c1p1*c3p3*c5p1*c7p2*c9p3*ss[56];
  root_4 += 50400*c1p1*c3p3*c5p1*c7p3*c9p2*ss[54];
  root_4 += 25200*c1p1*c3p3*c5p1*c7p4*c9p1*ss[52];
  root_4 += 50400*c1p1*c3p3*c5p2*c7p1*c9p3*ss[54];
  root_4 += 75600*c1p1*c3p3*c5p2*c7p2*c9p2*ss[52];
  root_4 += 50400*c1p1*c3p3*c5p2*c7p3*c9p1*ss[50];
  root_4 += 50400*c1p1*c3p3*c5p3*c7p1*c9p2*ss[50];
  root_4 += 50400*c1p1*c3p3*c5p3*c7p2*c9p1*ss[48];
  root_4 += 25200*c1p1*c3p3*c5p4*c7p1*c9p1*ss[46];
  root_4 += 25200*c1p1*c3p4*c5p1*c7p1*c9p3*ss[52];
  root_4 += 37800*c1p1*c3p4*c5p1*c7p2*c9p2*ss[50];
  root_4 += 25200*c1p1*c3p4*c5p1*c7p3*c9p1*ss[48];
  root_4 += 37800*c1p1*c3p4*c5p2*c7p1*c9p2*ss[48];
  root_4 += 37800*c1p1*c3p4*c5p2*c7p2*c9p1*ss[46];
  root_4 += 25200*c1p1*c3p4*c5p3*c7p1*c9p1*ss[44];
  root_4 += 15120*c1p1*c3p5*c5p1*c7p1*c9p2*ss[46];
  root_4 += 15120*c1p1*c3p5*c5p1*c7p2*c9p1*ss[44];
  root_4 += 15120*c1p1*c3p5*c5p2*c7p1*c9p1*ss[42];
  root_4 += 5040*c1p1*c3p6*c5p1*c7p1*c9p1*ss[40];
  root_4 += 15120*c1p2*c3p1*c5p1*c7p1*c9p5*ss[62];
  root_4 += 37800*c1p2*c3p1*c5p1*c7p2*c9p4*ss[60];
  root_4 += 50400*c1p2*c3p1*c5p1*c7p3*c9p3*ss[58];
  root_4 += 37800*c1p2*c3p1*c5p1*c7p4*c9p2*ss[56];
  root_4 += 15120*c1p2*c3p1*c5p1*c7p5*c9p1*ss[54];
  root_4 += 37800*c1p2*c3p1*c5p2*c7p1*c9p4*ss[58];
  root_4 += 75600*c1p2*c3p1*c5p2*c7p2*c9p3*ss[56];
  root_4 += 75600*c1p2*c3p1*c5p2*c7p3*c9p2*ss[54];
  root_4 += 37800*c1p2*c3p1*c5p2*c7p4*c9p1*ss[52];
  root_4 += 50400*c1p2*c3p1*c5p3*c7p1*c9p3*ss[54];
  root_4 += 75600*c1p2*c3p1*c5p3*c7p2*c9p2*ss[52];
  root_4 += 50400*c1p2*c3p1*c5p3*c7p3*c9p1*ss[50];
  root_4 += 37800*c1p2*c3p1*c5p4*c7p1*c9p2*ss[50];
  root_4 += 37800*c1p2*c3p1*c5p4*c7p2*c9p1*ss[48];
  root_4 += 15120*c1p2*c3p1*c5p5*c7p1*c9p1*ss[46];
  root_4 += 37800*c1p2*c3p2*c5p1*c7p1*c9p4*ss[56];
  root_4 += 75600*c1p2*c3p2*c5p1*c7p2*c9p3*ss[54];
  root_4 += 75600*c1p2*c3p2*c5p1*c7p3*c9p2*ss[52];
  root_4 += 37800*c1p2*c3p2*c5p1*c7p4*c9p1*ss[50];
  root_4 += 75600*c1p2*c3p2*c5p2*c7p1*c9p3*ss[52];
  root_4 += 113400*c1p2*c3p2*c5p2*c7p2*c9p2*ss[50];
  root_4 += 75600*c1p2*c3p2*c5p2*c7p3*c9p1*ss[48];
  root_4 += 75600*c1p2*c3p2*c5p3*c7p1*c9p2*ss[48];
  root_4 += 75600*c1p2*c3p2*c5p3*c7p2*c9p1*ss[46];
  root_4 += 37800*c1p2*c3p2*c5p4*c7p1*c9p1*ss[44];
  root_4 += 50400*c1p2*c3p3*c5p1*c7p1*c9p3*ss[50];
  root_4 += 75600*c1p2*c3p3*c5p1*c7p2*c9p2*ss[48];
  root_4 += 50400*c1p2*c3p3*c5p1*c7p3*c9p1*ss[46];
  root_4 += 75600*c1p2*c3p3*c5p2*c7p1*c9p2*ss[46];
  root_4 += 75600*c1p2*c3p3*c5p2*c7p2*c9p1*ss[44];
  root_4 += 50400*c1p2*c3p3*c5p3*c7p1*c9p1*ss[42];
  root_4 += 37800*c1p2*c3p4*c5p1*c7p1*c9p2*ss[44];
  root_4 += 37800*c1p2*c3p4*c5p1*c7p2*c9p1*ss[42];
  root_4 += 37800*c1p2*c3p4*c5p2*c7p1*c9p1*ss[40];
  root_4 += 15120*c1p2*c3p5*c5p1*c7p1*c9p1*ss[38];
  root_4 += 25200*c1p3*c3p1*c5p1*c7p1*c9p4*ss[54];
  root_4 += 50400*c1p3*c3p1*c5p1*c7p2*c9p3*ss[52];
  root_4 += 50400*c1p3*c3p1*c5p1*c7p3*c9p2*ss[50];
  root_4 += 25200*c1p3*c3p1*c5p1*c7p4*c9p1*ss[48];
  root_4 += 50400*c1p3*c3p1*c5p2*c7p1*c9p3*ss[50];
  root_4 += 75600*c1p3*c3p1*c5p2*c7p2*c9p2*ss[48];
  root_4 += 50400*c1p3*c3p1*c5p2*c7p3*c9p1*ss[46];
  root_4 += 50400*c1p3*c3p1*c5p3*c7p1*c9p2*ss[46];
  root_4 += 50400*c1p3*c3p1*c5p3*c7p2*c9p1*ss[44];
  root_4 += 25200*c1p3*c3p1*c5p4*c7p1*c9p1*ss[42];
  root_4 += 50400*c1p3*c3p2*c5p1*c7p1*c9p3*ss[48];
  root_4 += 75600*c1p3*c3p2*c5p1*c7p2*c9p2*ss[46];
  root_4 += 50400*c1p3*c3p2*c5p1*c7p3*c9p1*ss[44];
  root_4 += 75600*c1p3*c3p2*c5p2*c7p1*c9p2*ss[44];
  root_4 += 75600*c1p3*c3p2*c5p2*c7p2*c9p1*ss[42];
  root_4 += 50400*c1p3*c3p2*c5p3*c7p1*c9p1*ss[40];
  root_4 += 50400*c1p3*c3p3*c5p1*c7p1*c9p2*ss[42];
  root_4 += 50400*c1p3*c3p3*c5p1*c7p2*c9p1*ss[40];
  root_4 += 50400*c1p3*c3p3*c5p2*c7p1*c9p1*ss[38];
  root_4 += 25200*c1p3*c3p4*c5p1*c7p1*c9p1*ss[36];
  root_4 += 25200*c1p4*c3p1*c5p1*c7p1*c9p3*ss[46];
  root_4 += 37800*c1p4*c3p1*c5p1*c7p2*c9p2*ss[44];
  root_4 += 25200*c1p4*c3p1*c5p1*c7p3*c9p1*ss[42];
  root_4 += 37800*c1p4*c3p1*c5p2*c7p1*c9p2*ss[42];
  root_4 += 37800*c1p4*c3p1*c5p2*c7p2*c9p1*ss[40];
  root_4 += 25200*c1p4*c3p1*c5p3*c7p1*c9p1*ss[38];
  root_4 += 37800*c1p4*c3p2*c5p1*c7p1*c9p2*ss[40];
  root_4 += 37800*c1p4*c3p2*c5p1*c7p2*c9p1*ss[38];
  root_4 += 37800*c1p4*c3p2*c5p2*c7p1*c9p1*ss[36];
  root_4 += 25200*c1p4*c3p3*c5p1*c7p1*c9p1*ss[34];
  root_4 += 15120*c1p5*c3p1*c5p1*c7p1*c9p2*ss[38];
  root_4 += 15120*c1p5*c3p1*c5p1*c7p2*c9p1*ss[36];
  root_4 += 15120*c1p5*c3p1*c5p2*c7p1*c9p1*ss[34];
  root_4 += 15120*c1p5*c3p2*c5p1*c7p1*c9p1*ss[32];
  root_4 += 5040*c1p6*c3p1*c5p1*c7p1*c9p1*ss[30];
  root_4 += 720*c1p1*c3p1*c5p1*c7p7*ss[58];
  root_4 += 720*c1p1*c3p1*c5p1*c9p7*ss[72];
  root_4 += 2520*c1p1*c3p1*c5p2*c7p6*ss[56];
  root_4 += 2520*c1p1*c3p1*c5p2*c9p6*ss[68];
  root_4 += 5040*c1p1*c3p1*c5p3*c7p5*ss[54];
  root_4 += 5040*c1p1*c3p1*c5p3*c9p5*ss[64];
  root_4 += 6300*c1p1*c3p1*c5p4*c7p4*ss[52];
  root_4 += 6300*c1p1*c3p1*c5p4*c9p4*ss[60];
  root_4 += 5040*c1p1*c3p1*c5p5*c7p3*ss[50];
  root_4 += 5040*c1p1*c3p1*c5p5*c9p3*ss[56];
  root_4 += 2520*c1p1*c3p1*c5p6*c7p2*ss[48];
  root_4 += 2520*c1p1*c3p1*c5p6*c9p2*ss[52];
  root_4 += 720*c1p1*c3p1*c5p7*c7p1*ss[46];
  root_4 += 720*c1p1*c3p1*c5p7*c9p1*ss[48];
  root_4 += 720*c1p1*c3p1*c7p1*c9p7*ss[74];
  root_4 += 2520*c1p1*c3p1*c7p2*c9p6*ss[72];
  root_4 += 5040*c1p1*c3p1*c7p3*c9p5*ss[70];
  root_4 += 6300*c1p1*c3p1*c7p4*c9p4*ss[68];
  root_4 += 5040*c1p1*c3p1*c7p5*c9p3*ss[66];
  root_4 += 2520*c1p1*c3p1*c7p6*c9p2*ss[64];
  root_4 += 720*c1p1*c3p1*c7p7*c9p1*ss[62];
  root_4 += 2520*c1p1*c3p2*c5p1*c7p6*ss[54];
  root_4 += 2520*c1p1*c3p2*c5p1*c9p6*ss[66];
  root_4 += 7560*c1p1*c3p2*c5p2*c7p5*ss[52];
  root_4 += 7560*c1p1*c3p2*c5p2*c9p5*ss[62];
  root_4 += 12600*c1p1*c3p2*c5p3*c7p4*ss[50];
  root_4 += 12600*c1p1*c3p2*c5p3*c9p4*ss[58];
  root_4 += 12600*c1p1*c3p2*c5p4*c7p3*ss[48];
  root_4 += 12600*c1p1*c3p2*c5p4*c9p3*ss[54];
  root_4 += 7560*c1p1*c3p2*c5p5*c7p2*ss[46];
  root_4 += 7560*c1p1*c3p2*c5p5*c9p2*ss[50];
  root_4 += 2520*c1p1*c3p2*c5p6*c7p1*ss[44];
  root_4 += 2520*c1p1*c3p2*c5p6*c9p1*ss[46];
  root_4 += 2520*c1p1*c3p2*c7p1*c9p6*ss[68];
  root_4 += 7560*c1p1*c3p2*c7p2*c9p5*ss[66];
  root_4 += 12600*c1p1*c3p2*c7p3*c9p4*ss[64];
  root_4 += 12600*c1p1*c3p2*c7p4*c9p3*ss[62];
  root_4 += 7560*c1p1*c3p2*c7p5*c9p2*ss[60];
  root_4 += 2520*c1p1*c3p2*c7p6*c9p1*ss[58];
  root_4 += 5040*c1p1*c3p3*c5p1*c7p5*ss[50];
  root_4 += 5040*c1p1*c3p3*c5p1*c9p5*ss[60];
  root_4 += 12600*c1p1*c3p3*c5p2*c7p4*ss[48];
  root_4 += 12600*c1p1*c3p3*c5p2*c9p4*ss[56];
  root_4 += 16800*c1p1*c3p3*c5p3*c7p3*ss[46];
  root_4 += 16800*c1p1*c3p3*c5p3*c9p3*ss[52];
  root_4 += 12600*c1p1*c3p3*c5p4*c7p2*ss[44];
  root_4 += 12600*c1p1*c3p3*c5p4*c9p2*ss[48];
  root_4 += 5040*c1p1*c3p3*c5p5*c7p1*ss[42];
  root_4 += 5040*c1p1*c3p3*c5p5*c9p1*ss[44];
  root_4 += 5040*c1p1*c3p3*c7p1*c9p5*ss[62];
  root_4 += 12600*c1p1*c3p3*c7p2*c9p4*ss[60];
  root_4 += 16800*c1p1*c3p3*c7p3*c9p3*ss[58];
  root_4 += 12600*c1p1*c3p3*c7p4*c9p2*ss[56];
  root_4 += 5040*c1p1*c3p3*c7p5*c9p1*ss[54];
  root_4 += 6300*c1p1*c3p4*c5p1*c7p4*ss[46];
  root_4 += 6300*c1p1*c3p4*c5p1*c9p4*ss[54];
  root_4 += 12600*c1p1*c3p4*c5p2*c7p3*ss[44];
  root_4 += 12600*c1p1*c3p4*c5p2*c9p3*ss[50];
  root_4 += 12600*c1p1*c3p4*c5p3*c7p2*ss[42];
  root_4 += 12600*c1p1*c3p4*c5p3*c9p2*ss[46];
  root_4 += 6300*c1p1*c3p4*c5p4*c7p1*ss[40];
  root_4 += 6300*c1p1*c3p4*c5p4*c9p1*ss[42];
  root_4 += 6300*c1p1*c3p4*c7p1*c9p4*ss[56];
  root_4 += 12600*c1p1*c3p4*c7p2*c9p3*ss[54];
  root_4 += 12600*c1p1*c3p4*c7p3*c9p2*ss[52];
  root_4 += 6300*c1p1*c3p4*c7p4*c9p1*ss[50];
  root_4 += 5040*c1p1*c3p5*c5p1*c7p3*ss[42];
  root_4 += 5040*c1p1*c3p5*c5p1*c9p3*ss[48];
  root_4 += 7560*c1p1*c3p5*c5p2*c7p2*ss[40];
  root_4 += 7560*c1p1*c3p5*c5p2*c9p2*ss[44];
  root_4 += 5040*c1p1*c3p5*c5p3*c7p1*ss[38];
  root_4 += 5040*c1p1*c3p5*c5p3*c9p1*ss[40];
  root_4 += 5040*c1p1*c3p5*c7p1*c9p3*ss[50];
  root_4 += 7560*c1p1*c3p5*c7p2*c9p2*ss[48];
  root_4 += 5040*c1p1*c3p5*c7p3*c9p1*ss[46];
  root_4 += 2520*c1p1*c3p6*c5p1*c7p2*ss[38];
  root_4 += 2520*c1p1*c3p6*c5p1*c9p2*ss[42];
  root_4 += 2520*c1p1*c3p6*c5p2*c7p1*ss[36];
  root_4 += 2520*c1p1*c3p6*c5p2*c9p1*ss[38];
  root_4 += 2520*c1p1*c3p6*c7p1*c9p2*ss[44];
  root_4 += 2520*c1p1*c3p6*c7p2*c9p1*ss[42];
  root_4 += 720*c1p1*c3p7*c5p1*c7p1*ss[34];
  root_4 += 720*c1p1*c3p7*c5p1*c9p1*ss[36];
  root_4 += 720*c1p1*c3p7*c7p1*c9p1*ss[38];
  root_4 += 720*c1p1*c5p1*c7p1*c9p7*ss[76];
  root_4 += 2520*c1p1*c5p1*c7p2*c9p6*ss[74];
  root_4 += 5040*c1p1*c5p1*c7p3*c9p5*ss[72];
  root_4 += 6300*c1p1*c5p1*c7p4*c9p4*ss[70];
  root_4 += 5040*c1p1*c5p1*c7p5*c9p3*ss[68];
  root_4 += 2520*c1p1*c5p1*c7p6*c9p2*ss[66];
  root_4 += 720*c1p1*c5p1*c7p7*c9p1*ss[64];
  root_4 += 2520*c1p1*c5p2*c7p1*c9p6*ss[72];
  root_4 += 7560*c1p1*c5p2*c7p2*c9p5*ss[70];
  root_4 += 12600*c1p1*c5p2*c7p3*c9p4*ss[68];
  root_4 += 12600*c1p1*c5p2*c7p4*c9p3*ss[66];
  root_4 += 7560*c1p1*c5p2*c7p5*c9p2*ss[64];
  root_4 += 2520*c1p1*c5p2*c7p6*c9p1*ss[62];
  root_4 += 5040*c1p1*c5p3*c7p1*c9p5*ss[68];
  root_4 += 12600*c1p1*c5p3*c7p2*c9p4*ss[66];
  root_4 += 16800*c1p1*c5p3*c7p3*c9p3*ss[64];
  root_4 += 12600*c1p1*c5p3*c7p4*c9p2*ss[62];
  root_4 += 5040*c1p1*c5p3*c7p5*c9p1*ss[60];
  root_4 += 6300*c1p1*c5p4*c7p1*c9p4*ss[64];
  root_4 += 12600*c1p1*c5p4*c7p2*c9p3*ss[62];
  root_4 += 12600*c1p1*c5p4*c7p3*c9p2*ss[60];
  root_4 += 6300*c1p1*c5p4*c7p4*c9p1*ss[58];
  root_4 += 5040*c1p1*c5p5*c7p1*c9p3*ss[60];
  root_4 += 7560*c1p1*c5p5*c7p2*c9p2*ss[58];
  root_4 += 5040*c1p1*c5p5*c7p3*c9p1*ss[56];
  root_4 += 2520*c1p1*c5p6*c7p1*c9p2*ss[56];
  root_4 += 2520*c1p1*c5p6*c7p2*c9p1*ss[54];
  root_4 += 720*c1p1*c5p7*c7p1*c9p1*ss[52];
  root_4 += 2520*c1p2*c3p1*c5p1*c7p6*ss[52];
  root_4 += 2520*c1p2*c3p1*c5p1*c9p6*ss[64];
  root_4 += 7560*c1p2*c3p1*c5p2*c7p5*ss[50];
  root_4 += 7560*c1p2*c3p1*c5p2*c9p5*ss[60];
  root_4 += 12600*c1p2*c3p1*c5p3*c7p4*ss[48];
  root_4 += 12600*c1p2*c3p1*c5p3*c9p4*ss[56];
  root_4 += 12600*c1p2*c3p1*c5p4*c7p3*ss[46];
  root_4 += 12600*c1p2*c3p1*c5p4*c9p3*ss[52];
  root_4 += 7560*c1p2*c3p1*c5p5*c7p2*ss[44];
  root_4 += 7560*c1p2*c3p1*c5p5*c9p2*ss[48];
  root_4 += 2520*c1p2*c3p1*c5p6*c7p1*ss[42];
  root_4 += 2520*c1p2*c3p1*c5p6*c9p1*ss[44];
  root_4 += 2520*c1p2*c3p1*c7p1*c9p6*ss[66];
  root_4 += 7560*c1p2*c3p1*c7p2*c9p5*ss[64];
  root_4 += 12600*c1p2*c3p1*c7p3*c9p4*ss[62];
  root_4 += 12600*c1p2*c3p1*c7p4*c9p3*ss[60];
  root_4 += 7560*c1p2*c3p1*c7p5*c9p2*ss[58];
  root_4 += 2520*c1p2*c3p1*c7p6*c9p1*ss[56];
  root_4 += 7560*c1p2*c3p2*c5p1*c7p5*ss[48];
  root_4 += 7560*c1p2*c3p2*c5p1*c9p5*ss[58];
  root_4 += 18900*c1p2*c3p2*c5p2*c7p4*ss[46];
  root_4 += 18900*c1p2*c3p2*c5p2*c9p4*ss[54];
  root_4 += 25200*c1p2*c3p2*c5p3*c7p3*ss[44];
  root_4 += 25200*c1p2*c3p2*c5p3*c9p3*ss[50];
  root_4 += 18900*c1p2*c3p2*c5p4*c7p2*ss[42];
  root_4 += 18900*c1p2*c3p2*c5p4*c9p2*ss[46];
  root_4 += 7560*c1p2*c3p2*c5p5*c7p1*ss[40];
  root_4 += 7560*c1p2*c3p2*c5p5*c9p1*ss[42];
  root_4 += 7560*c1p2*c3p2*c7p1*c9p5*ss[60];
  root_4 += 18900*c1p2*c3p2*c7p2*c9p4*ss[58];
  root_4 += 25200*c1p2*c3p2*c7p3*c9p3*ss[56];
  root_4 += 18900*c1p2*c3p2*c7p4*c9p2*ss[54];
  root_4 += 7560*c1p2*c3p2*c7p5*c9p1*ss[52];
  root_4 += 12600*c1p2*c3p3*c5p1*c7p4*ss[44];
  root_4 += 12600*c1p2*c3p3*c5p1*c9p4*ss[52];
  root_4 += 25200*c1p2*c3p3*c5p2*c7p3*ss[42];
  root_4 += 25200*c1p2*c3p3*c5p2*c9p3*ss[48];
  root_4 += 25200*c1p2*c3p3*c5p3*c7p2*ss[40];
  root_4 += 25200*c1p2*c3p3*c5p3*c9p2*ss[44];
  root_4 += 12600*c1p2*c3p3*c5p4*c7p1*ss[38];
  root_4 += 12600*c1p2*c3p3*c5p4*c9p1*ss[40];
  root_4 += 12600*c1p2*c3p3*c7p1*c9p4*ss[54];
  root_4 += 25200*c1p2*c3p3*c7p2*c9p3*ss[52];
  root_4 += 25200*c1p2*c3p3*c7p3*c9p2*ss[50];
  root_4 += 12600*c1p2*c3p3*c7p4*c9p1*ss[48];
  root_4 += 12600*c1p2*c3p4*c5p1*c7p3*ss[40];
  root_4 += 12600*c1p2*c3p4*c5p1*c9p3*ss[46];
  root_4 += 18900*c1p2*c3p4*c5p2*c7p2*ss[38];
  root_4 += 18900*c1p2*c3p4*c5p2*c9p2*ss[42];
  root_4 += 12600*c1p2*c3p4*c5p3*c7p1*ss[36];
  root_4 += 12600*c1p2*c3p4*c5p3*c9p1*ss[38];
  root_4 += 12600*c1p2*c3p4*c7p1*c9p3*ss[48];
  root_4 += 18900*c1p2*c3p4*c7p2*c9p2*ss[46];
  root_4 += 12600*c1p2*c3p4*c7p3*c9p1*ss[44];
  root_4 += 7560*c1p2*c3p5*c5p1*c7p2*ss[36];
  root_4 += 7560*c1p2*c3p5*c5p1*c9p2*ss[40];
  root_4 += 7560*c1p2*c3p5*c5p2*c7p1*ss[34];
  root_4 += 7560*c1p2*c3p5*c5p2*c9p1*ss[36];
  root_4 += 7560*c1p2*c3p5*c7p1*c9p2*ss[42];
  root_4 += 7560*c1p2*c3p5*c7p2*c9p1*ss[40];
  root_4 += 2520*c1p2*c3p6*c5p1*c7p1*ss[32];
  root_4 += 2520*c1p2*c3p6*c5p1*c9p1*ss[34];
  root_4 += 2520*c1p2*c3p6*c7p1*c9p1*ss[36];
  root_4 += 2520*c1p2*c5p1*c7p1*c9p6*ss[68];
  root_4 += 7560*c1p2*c5p1*c7p2*c9p5*ss[66];
  root_4 += 12600*c1p2*c5p1*c7p3*c9p4*ss[64];
  root_4 += 12600*c1p2*c5p1*c7p4*c9p3*ss[62];
  root_4 += 7560*c1p2*c5p1*c7p5*c9p2*ss[60];
  root_4 += 2520*c1p2*c5p1*c7p6*c9p1*ss[58];
  root_4 += 7560*c1p2*c5p2*c7p1*c9p5*ss[64];
  root_4 += 18900*c1p2*c5p2*c7p2*c9p4*ss[62];
  root_4 += 25200*c1p2*c5p2*c7p3*c9p3*ss[60];
  root_4 += 18900*c1p2*c5p2*c7p4*c9p2*ss[58];
  root_4 += 7560*c1p2*c5p2*c7p5*c9p1*ss[56];
  root_4 += 12600*c1p2*c5p3*c7p1*c9p4*ss[60];
  root_4 += 25200*c1p2*c5p3*c7p2*c9p3*ss[58];
  root_4 += 25200*c1p2*c5p3*c7p3*c9p2*ss[56];
  root_4 += 12600*c1p2*c5p3*c7p4*c9p1*ss[54];
  root_4 += 12600*c1p2*c5p4*c7p1*c9p3*ss[56];
  root_4 += 18900*c1p2*c5p4*c7p2*c9p2*ss[54];
  root_4 += 12600*c1p2*c5p4*c7p3*c9p1*ss[52];
  root_4 += 7560*c1p2*c5p5*c7p1*c9p2*ss[52];
  root_4 += 7560*c1p2*c5p5*c7p2*c9p1*ss[50];
  root_4 += 2520*c1p2*c5p6*c7p1*c9p1*ss[48];
  root_4 += 5040*c1p3*c3p1*c5p1*c7p5*ss[46];
  root_4 += 5040*c1p3*c3p1*c5p1*c9p5*ss[56];
  root_4 += 12600*c1p3*c3p1*c5p2*c7p4*ss[44];
  root_4 += 12600*c1p3*c3p1*c5p2*c9p4*ss[52];
  root_4 += 16800*c1p3*c3p1*c5p3*c7p3*ss[42];
  root_4 += 16800*c1p3*c3p1*c5p3*c9p3*ss[48];
  root_4 += 12600*c1p3*c3p1*c5p4*c7p2*ss[40];
  root_4 += 12600*c1p3*c3p1*c5p4*c9p2*ss[44];
  root_4 += 5040*c1p3*c3p1*c5p5*c7p1*ss[38];
  root_4 += 5040*c1p3*c3p1*c5p5*c9p1*ss[40];
  root_4 += 5040*c1p3*c3p1*c7p1*c9p5*ss[58];
  root_4 += 12600*c1p3*c3p1*c7p2*c9p4*ss[56];
  root_4 += 16800*c1p3*c3p1*c7p3*c9p3*ss[54];
  root_4 += 12600*c1p3*c3p1*c7p4*c9p2*ss[52];
  root_4 += 5040*c1p3*c3p1*c7p5*c9p1*ss[50];
  root_4 += 12600*c1p3*c3p2*c5p1*c7p4*ss[42];
  root_4 += 12600*c1p3*c3p2*c5p1*c9p4*ss[50];
  root_4 += 25200*c1p3*c3p2*c5p2*c7p3*ss[40];
  root_4 += 25200*c1p3*c3p2*c5p2*c9p3*ss[46];
  root_4 += 25200*c1p3*c3p2*c5p3*c7p2*ss[38];
  root_4 += 25200*c1p3*c3p2*c5p3*c9p2*ss[42];
  root_4 += 12600*c1p3*c3p2*c5p4*c7p1*ss[36];
  root_4 += 12600*c1p3*c3p2*c5p4*c9p1*ss[38];
  root_4 += 12600*c1p3*c3p2*c7p1*c9p4*ss[52];
  root_4 += 25200*c1p3*c3p2*c7p2*c9p3*ss[50];
  root_4 += 25200*c1p3*c3p2*c7p3*c9p2*ss[48];
  root_4 += 12600*c1p3*c3p2*c7p4*c9p1*ss[46];
  root_4 += 16800*c1p3*c3p3*c5p1*c7p3*ss[38];
  root_4 += 16800*c1p3*c3p3*c5p1*c9p3*ss[44];
  root_4 += 25200*c1p3*c3p3*c5p2*c7p2*ss[36];
  root_4 += 25200*c1p3*c3p3*c5p2*c9p2*ss[40];
  root_4 += 16800*c1p3*c3p3*c5p3*c7p1*ss[34];
  root_4 += 16800*c1p3*c3p3*c5p3*c9p1*ss[36];
  root_4 += 16800*c1p3*c3p3*c7p1*c9p3*ss[46];
  root_4 += 25200*c1p3*c3p3*c7p2*c9p2*ss[44];
  root_4 += 16800*c1p3*c3p3*c7p3*c9p1*ss[42];
  root_4 += 12600*c1p3*c3p4*c5p1*c7p2*ss[34];
  root_4 += 12600*c1p3*c3p4*c5p1*c9p2*ss[38];
  root_4 += 12600*c1p3*c3p4*c5p2*c7p1*ss[32];
  root_4 += 12600*c1p3*c3p4*c5p2*c9p1*ss[34];
  root_4 += 12600*c1p3*c3p4*c7p1*c9p2*ss[40];
  root_4 += 12600*c1p3*c3p4*c7p2*c9p1*ss[38];
  root_4 += 5040*c1p3*c3p5*c5p1*c7p1*ss[30];
  root_4 += 5040*c1p3*c3p5*c5p1*c9p1*ss[32];
  root_4 += 5040*c1p3*c3p5*c7p1*c9p1*ss[34];
  root_4 += 5040*c1p3*c5p1*c7p1*c9p5*ss[60];
  root_4 += 12600*c1p3*c5p1*c7p2*c9p4*ss[58];
  root_4 += 16800*c1p3*c5p1*c7p3*c9p3*ss[56];
  root_4 += 12600*c1p3*c5p1*c7p4*c9p2*ss[54];
  root_4 += 5040*c1p3*c5p1*c7p5*c9p1*ss[52];
  root_4 += 12600*c1p3*c5p2*c7p1*c9p4*ss[56];
  root_4 += 25200*c1p3*c5p2*c7p2*c9p3*ss[54];
  root_4 += 25200*c1p3*c5p2*c7p3*c9p2*ss[52];
  root_4 += 12600*c1p3*c5p2*c7p4*c9p1*ss[50];
  root_4 += 16800*c1p3*c5p3*c7p1*c9p3*ss[52];
  root_4 += 25200*c1p3*c5p3*c7p2*c9p2*ss[50];
  root_4 += 16800*c1p3*c5p3*c7p3*c9p1*ss[48];
  root_4 += 12600*c1p3*c5p4*c7p1*c9p2*ss[48];
  root_4 += 12600*c1p3*c5p4*c7p2*c9p1*ss[46];
  root_4 += 5040*c1p3*c5p5*c7p1*c9p1*ss[44];
  root_4 += 6300*c1p4*c3p1*c5p1*c7p4*ss[40];
  root_4 += 6300*c1p4*c3p1*c5p1*c9p4*ss[48];
  root_4 += 12600*c1p4*c3p1*c5p2*c7p3*ss[38];
  root_4 += 12600*c1p4*c3p1*c5p2*c9p3*ss[44];
  root_4 += 12600*c1p4*c3p1*c5p3*c7p2*ss[36];
  root_4 += 12600*c1p4*c3p1*c5p3*c9p2*ss[40];
  root_4 += 6300*c1p4*c3p1*c5p4*c7p1*ss[34];
  root_4 += 6300*c1p4*c3p1*c5p4*c9p1*ss[36];
  root_4 += 6300*c1p4*c3p1*c7p1*c9p4*ss[50];
  root_4 += 12600*c1p4*c3p1*c7p2*c9p3*ss[48];
  root_4 += 12600*c1p4*c3p1*c7p3*c9p2*ss[46];
  root_4 += 6300*c1p4*c3p1*c7p4*c9p1*ss[44];
  root_4 += 12600*c1p4*c3p2*c5p1*c7p3*ss[36];
  root_4 += 12600*c1p4*c3p2*c5p1*c9p3*ss[42];
  root_4 += 18900*c1p4*c3p2*c5p2*c7p2*ss[34];
  root_4 += 18900*c1p4*c3p2*c5p2*c9p2*ss[38];
  root_4 += 12600*c1p4*c3p2*c5p3*c7p1*ss[32];
  root_4 += 12600*c1p4*c3p2*c5p3*c9p1*ss[34];
  root_4 += 12600*c1p4*c3p2*c7p1*c9p3*ss[44];
  root_4 += 18900*c1p4*c3p2*c7p2*c9p2*ss[42];
  root_4 += 12600*c1p4*c3p2*c7p3*c9p1*ss[40];
  root_4 += 12600*c1p4*c3p3*c5p1*c7p2*ss[32];
  root_4 += 12600*c1p4*c3p3*c5p1*c9p2*ss[36];
  root_4 += 12600*c1p4*c3p3*c5p2*c7p1*ss[30];
  root_4 += 12600*c1p4*c3p3*c5p2*c9p1*ss[32];
  root_4 += 12600*c1p4*c3p3*c7p1*c9p2*ss[38];
  root_4 += 12600*c1p4*c3p3*c7p2*c9p1*ss[36];
  root_4 += 6300*c1p4*c3p4*c5p1*c7p1*ss[28];
  root_4 += 6300*c1p4*c3p4*c5p1*c9p1*ss[30];
  root_4 += 6300*c1p4*c3p4*c7p1*c9p1*ss[32];
  root_4 += 6300*c1p4*c5p1*c7p1*c9p4*ss[52];
  root_4 += 12600*c1p4*c5p1*c7p2*c9p3*ss[50];
  root_4 += 12600*c1p4*c5p1*c7p3*c9p2*ss[48];
  root_4 += 6300*c1p4*c5p1*c7p4*c9p1*ss[46];
  root_4 += 12600*c1p4*c5p2*c7p1*c9p3*ss[48];
  root_4 += 18900*c1p4*c5p2*c7p2*c9p2*ss[46];
  root_4 += 12600*c1p4*c5p2*c7p3*c9p1*ss[44];
  root_4 += 12600*c1p4*c5p3*c7p1*c9p2*ss[44];
  root_4 += 12600*c1p4*c5p3*c7p2*c9p1*ss[42];
  root_4 += 6300*c1p4*c5p4*c7p1*c9p1*ss[40];
  root_4 += 5040*c1p5*c3p1*c5p1*c7p3*ss[34];
  root_4 += 5040*c1p5*c3p1*c5p1*c9p3*ss[40];
  root_4 += 7560*c1p5*c3p1*c5p2*c7p2*ss[32];
  root_4 += 7560*c1p5*c3p1*c5p2*c9p2*ss[36];
  root_4 += 5040*c1p5*c3p1*c5p3*c7p1*ss[30];
  root_4 += 5040*c1p5*c3p1*c5p3*c9p1*ss[32];
  root_4 += 5040*c1p5*c3p1*c7p1*c9p3*ss[42];
  root_4 += 7560*c1p5*c3p1*c7p2*c9p2*ss[40];
  root_4 += 5040*c1p5*c3p1*c7p3*c9p1*ss[38];
  root_4 += 7560*c1p5*c3p2*c5p1*c7p2*ss[30];
  root_4 += 7560*c1p5*c3p2*c5p1*c9p2*ss[34];
  root_4 += 7560*c1p5*c3p2*c5p2*c7p1*ss[28];
  root_4 += 7560*c1p5*c3p2*c5p2*c9p1*ss[30];
  root_4 += 7560*c1p5*c3p2*c7p1*c9p2*ss[36];
  root_4 += 7560*c1p5*c3p2*c7p2*c9p1*ss[34];
  root_4 += 5040*c1p5*c3p3*c5p1*c7p1*ss[26];
  root_4 += 5040*c1p5*c3p3*c5p1*c9p1*ss[28];
  root_4 += 5040*c1p5*c3p3*c7p1*c9p1*ss[30];
  root_4 += 5040*c1p5*c5p1*c7p1*c9p3*ss[44];
  root_4 += 7560*c1p5*c5p1*c7p2*c9p2*ss[42];
  root_4 += 5040*c1p5*c5p1*c7p3*c9p1*ss[40];
  root_4 += 7560*c1p5*c5p2*c7p1*c9p2*ss[40];
  root_4 += 7560*c1p5*c5p2*c7p2*c9p1*ss[38];
  root_4 += 5040*c1p5*c5p3*c7p1*c9p1*ss[36];
  root_4 += 2520*c1p6*c3p1*c5p1*c7p2*ss[28];
  root_4 += 2520*c1p6*c3p1*c5p1*c9p2*ss[32];
  root_4 += 2520*c1p6*c3p1*c5p2*c7p1*ss[26];
  root_4 += 2520*c1p6*c3p1*c5p2*c9p1*ss[28];
  root_4 += 2520*c1p6*c3p1*c7p1*c9p2*ss[34];
  root_4 += 2520*c1p6*c3p1*c7p2*c9p1*ss[32];
  root_4 += 2520*c1p6*c3p2*c5p1*c7p1*ss[24];
  root_4 += 2520*c1p6*c3p2*c5p1*c9p1*ss[26];
  root_4 += 2520*c1p6*c3p2*c7p1*c9p1*ss[28];
  root_4 += 2520*c1p6*c5p1*c7p1*c9p2*ss[36];
  root_4 += 2520*c1p6*c5p1*c7p2*c9p1*ss[34];
  root_4 += 2520*c1p6*c5p2*c7p1*c9p1*ss[32];
  root_4 += 720*c1p7*c3p1*c5p1*c7p1*ss[22];
  root_4 += 720*c1p7*c3p1*c5p1*c9p1*ss[24];
  root_4 += 720*c1p7*c3p1*c7p1*c9p1*ss[26];
  root_4 += 720*c1p7*c5p1*c7p1*c9p1*ss[28];
  root_4 += 720*c3p1*c5p1*c7p1*c9p7*ss[78];
  root_4 += 2520*c3p1*c5p1*c7p2*c9p6*ss[76];
  root_4 += 5040*c3p1*c5p1*c7p3*c9p5*ss[74];
  root_4 += 6300*c3p1*c5p1*c7p4*c9p4*ss[72];
  root_4 += 5040*c3p1*c5p1*c7p5*c9p3*ss[70];
  root_4 += 2520*c3p1*c5p1*c7p6*c9p2*ss[68];
  root_4 += 720*c3p1*c5p1*c7p7*c9p1*ss[66];
  root_4 += 2520*c3p1*c5p2*c7p1*c9p6*ss[74];
  root_4 += 7560*c3p1*c5p2*c7p2*c9p5*ss[72];
  root_4 += 12600*c3p1*c5p2*c7p3*c9p4*ss[70];
  root_4 += 12600*c3p1*c5p2*c7p4*c9p3*ss[68];
  root_4 += 7560*c3p1*c5p2*c7p5*c9p2*ss[66];
  root_4 += 2520*c3p1*c5p2*c7p6*c9p1*ss[64];
  root_4 += 5040*c3p1*c5p3*c7p1*c9p5*ss[70];
  root_4 += 12600*c3p1*c5p3*c7p2*c9p4*ss[68];
  root_4 += 16800*c3p1*c5p3*c7p3*c9p3*ss[66];
  root_4 += 12600*c3p1*c5p3*c7p4*c9p2*ss[64];
  root_4 += 5040*c3p1*c5p3*c7p5*c9p1*ss[62];
  root_4 += 6300*c3p1*c5p4*c7p1*c9p4*ss[66];
  root_4 += 12600*c3p1*c5p4*c7p2*c9p3*ss[64];
  root_4 += 12600*c3p1*c5p4*c7p3*c9p2*ss[62];
  root_4 += 6300*c3p1*c5p4*c7p4*c9p1*ss[60];
  root_4 += 5040*c3p1*c5p5*c7p1*c9p3*ss[62];
  root_4 += 7560*c3p1*c5p5*c7p2*c9p2*ss[60];
  root_4 += 5040*c3p1*c5p5*c7p3*c9p1*ss[58];
  root_4 += 2520*c3p1*c5p6*c7p1*c9p2*ss[58];
  root_4 += 2520*c3p1*c5p6*c7p2*c9p1*ss[56];
  root_4 += 720*c3p1*c5p7*c7p1*c9p1*ss[54];
  root_4 += 2520*c3p2*c5p1*c7p1*c9p6*ss[72];
  root_4 += 7560*c3p2*c5p1*c7p2*c9p5*ss[70];
  root_4 += 12600*c3p2*c5p1*c7p3*c9p4*ss[68];
  root_4 += 12600*c3p2*c5p1*c7p4*c9p3*ss[66];
  root_4 += 7560*c3p2*c5p1*c7p5*c9p2*ss[64];
  root_4 += 2520*c3p2*c5p1*c7p6*c9p1*ss[62];
  root_4 += 7560*c3p2*c5p2*c7p1*c9p5*ss[68];
  root_4 += 18900*c3p2*c5p2*c7p2*c9p4*ss[66];
  root_4 += 25200*c3p2*c5p2*c7p3*c9p3*ss[64];
  root_4 += 18900*c3p2*c5p2*c7p4*c9p2*ss[62];
  root_4 += 7560*c3p2*c5p2*c7p5*c9p1*ss[60];
  root_4 += 12600*c3p2*c5p3*c7p1*c9p4*ss[64];
  root_4 += 25200*c3p2*c5p3*c7p2*c9p3*ss[62];
  root_4 += 25200*c3p2*c5p3*c7p3*c9p2*ss[60];
  root_4 += 12600*c3p2*c5p3*c7p4*c9p1*ss[58];
  root_4 += 12600*c3p2*c5p4*c7p1*c9p3*ss[60];
  root_4 += 18900*c3p2*c5p4*c7p2*c9p2*ss[58];
  root_4 += 12600*c3p2*c5p4*c7p3*c9p1*ss[56];
  root_4 += 7560*c3p2*c5p5*c7p1*c9p2*ss[56];
  root_4 += 7560*c3p2*c5p5*c7p2*c9p1*ss[54];
  root_4 += 2520*c3p2*c5p6*c7p1*c9p1*ss[52];
  root_4 += 5040*c3p3*c5p1*c7p1*c9p5*ss[66];
  root_4 += 12600*c3p3*c5p1*c7p2*c9p4*ss[64];
  root_4 += 16800*c3p3*c5p1*c7p3*c9p3*ss[62];
  root_4 += 12600*c3p3*c5p1*c7p4*c9p2*ss[60];
  root_4 += 5040*c3p3*c5p1*c7p5*c9p1*ss[58];
  root_4 += 12600*c3p3*c5p2*c7p1*c9p4*ss[62];
  root_4 += 25200*c3p3*c5p2*c7p2*c9p3*ss[60];
  root_4 += 25200*c3p3*c5p2*c7p3*c9p2*ss[58];
  root_4 += 12600*c3p3*c5p2*c7p4*c9p1*ss[56];
  root_4 += 16800*c3p3*c5p3*c7p1*c9p3*ss[58];
  root_4 += 25200*c3p3*c5p3*c7p2*c9p2*ss[56];
  root_4 += 16800*c3p3*c5p3*c7p3*c9p1*ss[54];
  root_4 += 12600*c3p3*c5p4*c7p1*c9p2*ss[54];
  root_4 += 12600*c3p3*c5p4*c7p2*c9p1*ss[52];
  root_4 += 5040*c3p3*c5p5*c7p1*c9p1*ss[50];
  root_4 += 6300*c3p4*c5p1*c7p1*c9p4*ss[60];
  root_4 += 12600*c3p4*c5p1*c7p2*c9p3*ss[58];
  root_4 += 12600*c3p4*c5p1*c7p3*c9p2*ss[56];
  root_4 += 6300*c3p4*c5p1*c7p4*c9p1*ss[54];
  root_4 += 12600*c3p4*c5p2*c7p1*c9p3*ss[56];
  root_4 += 18900*c3p4*c5p2*c7p2*c9p2*ss[54];
  root_4 += 12600*c3p4*c5p2*c7p3*c9p1*ss[52];
  root_4 += 12600*c3p4*c5p3*c7p1*c9p2*ss[52];
  root_4 += 12600*c3p4*c5p3*c7p2*c9p1*ss[50];
  root_4 += 6300*c3p4*c5p4*c7p1*c9p1*ss[48];
  root_4 += 5040*c3p5*c5p1*c7p1*c9p3*ss[54];
  root_4 += 7560*c3p5*c5p1*c7p2*c9p2*ss[52];
  root_4 += 5040*c3p5*c5p1*c7p3*c9p1*ss[50];
  root_4 += 7560*c3p5*c5p2*c7p1*c9p2*ss[50];
  root_4 += 7560*c3p5*c5p2*c7p2*c9p1*ss[48];
  root_4 += 5040*c3p5*c5p3*c7p1*c9p1*ss[46];
  root_4 += 2520*c3p6*c5p1*c7p1*c9p2*ss[48];
  root_4 += 2520*c3p6*c5p1*c7p2*c9p1*ss[46];
  root_4 += 2520*c3p6*c5p2*c7p1*c9p1*ss[44];
  root_4 += 720*c3p7*c5p1*c7p1*c9p1*ss[42];
  root_4 += 90*c1p1*c3p1*c5p8*ss[44];
  root_4 += 90*c1p1*c3p1*c7p8*ss[60];
  root_4 += 90*c1p1*c3p1*c9p8*ss[76];
  root_4 += 360*c1p1*c3p2*c5p7*ss[42];
  root_4 += 360*c1p1*c3p2*c7p7*ss[56];
  root_4 += 360*c1p1*c3p2*c9p7*ss[70];
  root_4 += 840*c1p1*c3p3*c5p6*ss[40];
  root_4 += 840*c1p1*c3p3*c7p6*ss[52];
  root_4 += 840*c1p1*c3p3*c9p6*ss[64];
  root_4 += 1260*c1p1*c3p4*c5p5*ss[38];
  root_4 += 1260*c1p1*c3p4*c7p5*ss[48];
  root_4 += 1260*c1p1*c3p4*c9p5*ss[58];
  root_4 += 1260*c1p1*c3p5*c5p4*ss[36];
  root_4 += 1260*c1p1*c3p5*c7p4*ss[44];
  root_4 += 1260*c1p1*c3p5*c9p4*ss[52];
  root_4 += 840*c1p1*c3p6*c5p3*ss[34];
  root_4 += 840*c1p1*c3p6*c7p3*ss[40];
  root_4 += 840*c1p1*c3p6*c9p3*ss[46];
  root_4 += 360*c1p1*c3p7*c5p2*ss[32];
  root_4 += 360*c1p1*c3p7*c7p2*ss[36];
  root_4 += 360*c1p1*c3p7*c9p2*ss[40];
  root_4 += 90*c1p1*c3p8*c5p1*ss[30];
  root_4 += 90*c1p1*c3p8*c7p1*ss[32];
  root_4 += 90*c1p1*c3p8*c9p1*ss[34];
  root_4 += 90*c1p1*c5p1*c7p8*ss[62];
  root_4 += 90*c1p1*c5p1*c9p8*ss[78];
  root_4 += 360*c1p1*c5p2*c7p7*ss[60];
  root_4 += 360*c1p1*c5p2*c9p7*ss[74];
  root_4 += 840*c1p1*c5p3*c7p6*ss[58];
  root_4 += 840*c1p1*c5p3*c9p6*ss[70];
  root_4 += 1260*c1p1*c5p4*c7p5*ss[56];
  root_4 += 1260*c1p1*c5p4*c9p5*ss[66];
  root_4 += 1260*c1p1*c5p5*c7p4*ss[54];
  root_4 += 1260*c1p1*c5p5*c9p4*ss[62];
  root_4 += 840*c1p1*c5p6*c7p3*ss[52];
  root_4 += 840*c1p1*c5p6*c9p3*ss[58];
  root_4 += 360*c1p1*c5p7*c7p2*ss[50];
  root_4 += 360*c1p1*c5p7*c9p2*ss[54];
  root_4 += 90*c1p1*c5p8*c7p1*ss[48];
  root_4 += 90*c1p1*c5p8*c9p1*ss[50];
  root_4 += 90*c1p1*c7p1*c9p8*ss[80];
  root_4 += 360*c1p1*c7p2*c9p7*ss[78];
  root_4 += 840*c1p1*c7p3*c9p6*ss[76];
  root_4 += 1260*c1p1*c7p4*c9p5*ss[74];
  root_4 += 1260*c1p1*c7p5*c9p4*ss[72];
  root_4 += 840*c1p1*c7p6*c9p3*ss[70];
  root_4 += 360*c1p1*c7p7*c9p2*ss[68];
  root_4 += 90*c1p1*c7p8*c9p1*ss[66];
  root_4 += 360*c1p2*c3p1*c5p7*ss[40];
  root_4 += 360*c1p2*c3p1*c7p7*ss[54];
  root_4 += 360*c1p2*c3p1*c9p7*ss[68];
  root_4 += 1260*c1p2*c3p2*c5p6*ss[38];
  root_4 += 1260*c1p2*c3p2*c7p6*ss[50];
  root_4 += 1260*c1p2*c3p2*c9p6*ss[62];
  root_4 += 2520*c1p2*c3p3*c5p5*ss[36];
  root_4 += 2520*c1p2*c3p3*c7p5*ss[46];
  root_4 += 2520*c1p2*c3p3*c9p5*ss[56];
  root_4 += 3150*c1p2*c3p4*c5p4*ss[34];
  root_4 += 3150*c1p2*c3p4*c7p4*ss[42];
  root_4 += 3150*c1p2*c3p4*c9p4*ss[50];
  root_4 += 2520*c1p2*c3p5*c5p3*ss[32];
  root_4 += 2520*c1p2*c3p5*c7p3*ss[38];
  root_4 += 2520*c1p2*c3p5*c9p3*ss[44];
  root_4 += 1260*c1p2*c3p6*c5p2*ss[30];
  root_4 += 1260*c1p2*c3p6*c7p2*ss[34];
  root_4 += 1260*c1p2*c3p6*c9p2*ss[38];
  root_4 += 360*c1p2*c3p7*c5p1*ss[28];
  root_4 += 360*c1p2*c3p7*c7p1*ss[30];
  root_4 += 360*c1p2*c3p7*c9p1*ss[32];
  root_4 += 360*c1p2*c5p1*c7p7*ss[56];
  root_4 += 360*c1p2*c5p1*c9p7*ss[70];
  root_4 += 1260*c1p2*c5p2*c7p6*ss[54];
  root_4 += 1260*c1p2*c5p2*c9p6*ss[66];
  root_4 += 2520*c1p2*c5p3*c7p5*ss[52];
  root_4 += 2520*c1p2*c5p3*c9p5*ss[62];
  root_4 += 3150*c1p2*c5p4*c7p4*ss[50];
  root_4 += 3150*c1p2*c5p4*c9p4*ss[58];
  root_4 += 2520*c1p2*c5p5*c7p3*ss[48];
  root_4 += 2520*c1p2*c5p5*c9p3*ss[54];
  root_4 += 1260*c1p2*c5p6*c7p2*ss[46];
  root_4 += 1260*c1p2*c5p6*c9p2*ss[50];
  root_4 += 360*c1p2*c5p7*c7p1*ss[44];
  root_4 += 360*c1p2*c5p7*c9p1*ss[46];
  root_4 += 360*c1p2*c7p1*c9p7*ss[72];
  root_4 += 1260*c1p2*c7p2*c9p6*ss[70];
  root_4 += 2520*c1p2*c7p3*c9p5*ss[68];
  root_4 += 3150*c1p2*c7p4*c9p4*ss[66];
  root_4 += 2520*c1p2*c7p5*c9p3*ss[64];
  root_4 += 1260*c1p2*c7p6*c9p2*ss[62];
  root_4 += 360*c1p2*c7p7*c9p1*ss[60];
  root_4 += 840*c1p3*c3p1*c5p6*ss[36];
  root_4 += 840*c1p3*c3p1*c7p6*ss[48];
  root_4 += 840*c1p3*c3p1*c9p6*ss[60];
  root_4 += 2520*c1p3*c3p2*c5p5*ss[34];
  root_4 += 2520*c1p3*c3p2*c7p5*ss[44];
  root_4 += 2520*c1p3*c3p2*c9p5*ss[54];
  root_4 += 4200*c1p3*c3p3*c5p4*ss[32];
  root_4 += 4200*c1p3*c3p3*c7p4*ss[40];
  root_4 += 4200*c1p3*c3p3*c9p4*ss[48];
  root_4 += 4200*c1p3*c3p4*c5p3*ss[30];
  root_4 += 4200*c1p3*c3p4*c7p3*ss[36];
  root_4 += 4200*c1p3*c3p4*c9p3*ss[42];
  root_4 += 2520*c1p3*c3p5*c5p2*ss[28];
  root_4 += 2520*c1p3*c3p5*c7p2*ss[32];
  root_4 += 2520*c1p3*c3p5*c9p2*ss[36];
  root_4 += 840*c1p3*c3p6*c5p1*ss[26];
  root_4 += 840*c1p3*c3p6*c7p1*ss[28];
  root_4 += 840*c1p3*c3p6*c9p1*ss[30];
  root_4 += 840*c1p3*c5p1*c7p6*ss[50];
  root_4 += 840*c1p3*c5p1*c9p6*ss[62];
  root_4 += 2520*c1p3*c5p2*c7p5*ss[48];
  root_4 += 2520*c1p3*c5p2*c9p5*ss[58];
  root_4 += 4200*c1p3*c5p3*c7p4*ss[46];
  root_4 += 4200*c1p3*c5p3*c9p4*ss[54];
  root_4 += 4200*c1p3*c5p4*c7p3*ss[44];
  root_4 += 4200*c1p3*c5p4*c9p3*ss[50];
  root_4 += 2520*c1p3*c5p5*c7p2*ss[42];
  root_4 += 2520*c1p3*c5p5*c9p2*ss[46];
  root_4 += 840*c1p3*c5p6*c7p1*ss[40];
  root_4 += 840*c1p3*c5p6*c9p1*ss[42];
  root_4 += 840*c1p3*c7p1*c9p6*ss[64];
  root_4 += 2520*c1p3*c7p2*c9p5*ss[62];
  root_4 += 4200*c1p3*c7p3*c9p4*ss[60];
  root_4 += 4200*c1p3*c7p4*c9p3*ss[58];
  root_4 += 2520*c1p3*c7p5*c9p2*ss[56];
  root_4 += 840*c1p3*c7p6*c9p1*ss[54];
  root_4 += 1260*c1p4*c3p1*c5p5*ss[32];
  root_4 += 1260*c1p4*c3p1*c7p5*ss[42];
  root_4 += 1260*c1p4*c3p1*c9p5*ss[52];
  root_4 += 3150*c1p4*c3p2*c5p4*ss[30];
  root_4 += 3150*c1p4*c3p2*c7p4*ss[38];
  root_4 += 3150*c1p4*c3p2*c9p4*ss[46];
  root_4 += 4200*c1p4*c3p3*c5p3*ss[28];
  root_4 += 4200*c1p4*c3p3*c7p3*ss[34];
  root_4 += 4200*c1p4*c3p3*c9p3*ss[40];
  root_4 += 3150*c1p4*c3p4*c5p2*ss[26];
  root_4 += 3150*c1p4*c3p4*c7p2*ss[30];
  root_4 += 3150*c1p4*c3p4*c9p2*ss[34];
  root_4 += 1260*c1p4*c3p5*c5p1*ss[24];
  root_4 += 1260*c1p4*c3p5*c7p1*ss[26];
  root_4 += 1260*c1p4*c3p5*c9p1*ss[28];
  root_4 += 1260*c1p4*c5p1*c7p5*ss[44];
  root_4 += 1260*c1p4*c5p1*c9p5*ss[54];
  root_4 += 3150*c1p4*c5p2*c7p4*ss[42];
  root_4 += 3150*c1p4*c5p2*c9p4*ss[50];
  root_4 += 4200*c1p4*c5p3*c7p3*ss[40];
  root_4 += 4200*c1p4*c5p3*c9p3*ss[46];
  root_4 += 3150*c1p4*c5p4*c7p2*ss[38];
  root_4 += 3150*c1p4*c5p4*c9p2*ss[42];
  root_4 += 1260*c1p4*c5p5*c7p1*ss[36];
  root_4 += 1260*c1p4*c5p5*c9p1*ss[38];
  root_4 += 1260*c1p4*c7p1*c9p5*ss[56];
  root_4 += 3150*c1p4*c7p2*c9p4*ss[54];
  root_4 += 4200*c1p4*c7p3*c9p3*ss[52];
  root_4 += 3150*c1p4*c7p4*c9p2*ss[50];
  root_4 += 1260*c1p4*c7p5*c9p1*ss[48];
  root_4 += 1260*c1p5*c3p1*c5p4*ss[28];
  root_4 += 1260*c1p5*c3p1*c7p4*ss[36];
  root_4 += 1260*c1p5*c3p1*c9p4*ss[44];
  root_4 += 2520*c1p5*c3p2*c5p3*ss[26];
  root_4 += 2520*c1p5*c3p2*c7p3*ss[32];
  root_4 += 2520*c1p5*c3p2*c9p3*ss[38];
  root_4 += 2520*c1p5*c3p3*c5p2*ss[24];
  root_4 += 2520*c1p5*c3p3*c7p2*ss[28];
  root_4 += 2520*c1p5*c3p3*c9p2*ss[32];
  root_4 += 1260*c1p5*c3p4*c5p1*ss[22];
  root_4 += 1260*c1p5*c3p4*c7p1*ss[24];
  root_4 += 1260*c1p5*c3p4*c9p1*ss[26];
  root_4 += 1260*c1p5*c5p1*c7p4*ss[38];
  root_4 += 1260*c1p5*c5p1*c9p4*ss[46];
  root_4 += 2520*c1p5*c5p2*c7p3*ss[36];
  root_4 += 2520*c1p5*c5p2*c9p3*ss[42];
  root_4 += 2520*c1p5*c5p3*c7p2*ss[34];
  root_4 += 2520*c1p5*c5p3*c9p2*ss[38];
  root_4 += 1260*c1p5*c5p4*c7p1*ss[32];
  root_4 += 1260*c1p5*c5p4*c9p1*ss[34];
  root_4 += 1260*c1p5*c7p1*c9p4*ss[48];
  root_4 += 2520*c1p5*c7p2*c9p3*ss[46];
  root_4 += 2520*c1p5*c7p3*c9p2*ss[44];
  root_4 += 1260*c1p5*c7p4*c9p1*ss[42];
  root_4 += 840*c1p6*c3p1*c5p3*ss[24];
  root_4 += 840*c1p6*c3p1*c7p3*ss[30];
  root_4 += 840*c1p6*c3p1*c9p3*ss[36];
  root_4 += 1260*c1p6*c3p2*c5p2*ss[22];
  root_4 += 1260*c1p6*c3p2*c7p2*ss[26];
  root_4 += 1260*c1p6*c3p2*c9p2*ss[30];
  root_4 += 840*c1p6*c3p3*c5p1*ss[20];
  root_4 += 840*c1p6*c3p3*c7p1*ss[22];
  root_4 += 840*c1p6*c3p3*c9p1*ss[24];
  root_4 += 840*c1p6*c5p1*c7p3*ss[32];
  root_4 += 840*c1p6*c5p1*c9p3*ss[38];
  root_4 += 1260*c1p6*c5p2*c7p2*ss[30];
  root_4 += 1260*c1p6*c5p2*c9p2*ss[34];
  root_4 += 840*c1p6*c5p3*c7p1*ss[28];
  root_4 += 840*c1p6*c5p3*c9p1*ss[30];
  root_4 += 840*c1p6*c7p1*c9p3*ss[40];
  root_4 += 1260*c1p6*c7p2*c9p2*ss[38];
  root_4 += 840*c1p6*c7p3*c9p1*ss[36];
  root_4 += 360*c1p7*c3p1*c5p2*ss[20];
  root_4 += 360*c1p7*c3p1*c7p2*ss[24];
  root_4 += 360*c1p7*c3p1*c9p2*ss[28];
  root_4 += 360*c1p7*c3p2*c5p1*ss[18];
  root_4 += 360*c1p7*c3p2*c7p1*ss[20];
  root_4 += 360*c1p7*c3p2*c9p1*ss[22];
  root_4 += 360*c1p7*c5p1*c7p2*ss[26];
  root_4 += 360*c1p7*c5p1*c9p2*ss[30];
  root_4 += 360*c1p7*c5p2*c7p1*ss[24];
  root_4 += 360*c1p7*c5p2*c9p1*ss[26];
  root_4 += 360*c1p7*c7p1*c9p2*ss[32];
  root_4 += 360*c1p7*c7p2*c9p1*ss[30];
  root_4 += 90*c1p8*c3p1*c5p1*ss[16];
  root_4 += 90*c1p8*c3p1*c7p1*ss[18];
  root_4 += 90*c1p8*c3p1*c9p1*ss[20];
  root_4 += 90*c1p8*c5p1*c7p1*ss[20];
  root_4 += 90*c1p8*c5p1*c9p1*ss[22];
  root_4 += 90*c1p8*c7p1*c9p1*ss[24];
  root_4 += 90*c3p1*c5p1*c7p8*ss[64];
  root_4 += 90*c3p1*c5p1*c9p8*ss[80];
  root_4 += 360*c3p1*c5p2*c7p7*ss[62];
  root_4 += 360*c3p1*c5p2*c9p7*ss[76];
  root_4 += 840*c3p1*c5p3*c7p6*ss[60];
  root_4 += 840*c3p1*c5p3*c9p6*ss[72];
  root_4 += 1260*c3p1*c5p4*c7p5*ss[58];
  root_4 += 1260*c3p1*c5p4*c9p5*ss[68];
  root_4 += 1260*c3p1*c5p5*c7p4*ss[56];
  root_4 += 1260*c3p1*c5p5*c9p4*ss[64];
  root_4 += 840*c3p1*c5p6*c7p3*ss[54];
  root_4 += 840*c3p1*c5p6*c9p3*ss[60];
  root_4 += 360*c3p1*c5p7*c7p2*ss[52];
  root_4 += 360*c3p1*c5p7*c9p2*ss[56];
  root_4 += 90*c3p1*c5p8*c7p1*ss[50];
  root_4 += 90*c3p1*c5p8*c9p1*ss[52];
  root_4 += 90*c3p1*c7p1*c9p8*ss[82];
  root_4 += 360*c3p1*c7p2*c9p7*ss[80];
  root_4 += 840*c3p1*c7p3*c9p6*ss[78];
  root_4 += 1260*c3p1*c7p4*c9p5*ss[76];
  root_4 += 1260*c3p1*c7p5*c9p4*ss[74];
  root_4 += 840*c3p1*c7p6*c9p3*ss[72];
  root_4 += 360*c3p1*c7p7*c9p2*ss[70];
  root_4 += 90*c3p1*c7p8*c9p1*ss[68];
  root_4 += 360*c3p2*c5p1*c7p7*ss[60];
  root_4 += 360*c3p2*c5p1*c9p7*ss[74];
  root_4 += 1260*c3p2*c5p2*c7p6*ss[58];
  root_4 += 1260*c3p2*c5p2*c9p6*ss[70];
  root_4 += 2520*c3p2*c5p3*c7p5*ss[56];
  root_4 += 2520*c3p2*c5p3*c9p5*ss[66];
  root_4 += 3150*c3p2*c5p4*c7p4*ss[54];
  root_4 += 3150*c3p2*c5p4*c9p4*ss[62];
  root_4 += 2520*c3p2*c5p5*c7p3*ss[52];
  root_4 += 2520*c3p2*c5p5*c9p3*ss[58];
  root_4 += 1260*c3p2*c5p6*c7p2*ss[50];
  root_4 += 1260*c3p2*c5p6*c9p2*ss[54];
  root_4 += 360*c3p2*c5p7*c7p1*ss[48];
  root_4 += 360*c3p2*c5p7*c9p1*ss[50];
  root_4 += 360*c3p2*c7p1*c9p7*ss[76];
  root_4 += 1260*c3p2*c7p2*c9p6*ss[74];
  root_4 += 2520*c3p2*c7p3*c9p5*ss[72];
  root_4 += 3150*c3p2*c7p4*c9p4*ss[70];
  root_4 += 2520*c3p2*c7p5*c9p3*ss[68];
  root_4 += 1260*c3p2*c7p6*c9p2*ss[66];
  root_4 += 360*c3p2*c7p7*c9p1*ss[64];
  root_4 += 840*c3p3*c5p1*c7p6*ss[56];
  root_4 += 840*c3p3*c5p1*c9p6*ss[68];
  root_4 += 2520*c3p3*c5p2*c7p5*ss[54];
  root_4 += 2520*c3p3*c5p2*c9p5*ss[64];
  root_4 += 4200*c3p3*c5p3*c7p4*ss[52];
  root_4 += 4200*c3p3*c5p3*c9p4*ss[60];
  root_4 += 4200*c3p3*c5p4*c7p3*ss[50];
  root_4 += 4200*c3p3*c5p4*c9p3*ss[56];
  root_4 += 2520*c3p3*c5p5*c7p2*ss[48];
  root_4 += 2520*c3p3*c5p5*c9p2*ss[52];
  root_4 += 840*c3p3*c5p6*c7p1*ss[46];
  root_4 += 840*c3p3*c5p6*c9p1*ss[48];
  root_4 += 840*c3p3*c7p1*c9p6*ss[70];
  root_4 += 2520*c3p3*c7p2*c9p5*ss[68];
  root_4 += 4200*c3p3*c7p3*c9p4*ss[66];
  root_4 += 4200*c3p3*c7p4*c9p3*ss[64];
  root_4 += 2520*c3p3*c7p5*c9p2*ss[62];
  root_4 += 840*c3p3*c7p6*c9p1*ss[60];
  root_4 += 1260*c3p4*c5p1*c7p5*ss[52];
  root_4 += 1260*c3p4*c5p1*c9p5*ss[62];
  root_4 += 3150*c3p4*c5p2*c7p4*ss[50];
  root_4 += 3150*c3p4*c5p2*c9p4*ss[58];
  root_4 += 4200*c3p4*c5p3*c7p3*ss[48];
  root_4 += 4200*c3p4*c5p3*c9p3*ss[54];
  root_4 += 3150*c3p4*c5p4*c7p2*ss[46];
  root_4 += 3150*c3p4*c5p4*c9p2*ss[50];
  root_4 += 1260*c3p4*c5p5*c7p1*ss[44];
  root_4 += 1260*c3p4*c5p5*c9p1*ss[46];
  root_4 += 1260*c3p4*c7p1*c9p5*ss[64];
  root_4 += 3150*c3p4*c7p2*c9p4*ss[62];
  root_4 += 4200*c3p4*c7p3*c9p3*ss[60];
  root_4 += 3150*c3p4*c7p4*c9p2*ss[58];
  root_4 += 1260*c3p4*c7p5*c9p1*ss[56];
  root_4 += 1260*c3p5*c5p1*c7p4*ss[48];
  root_4 += 1260*c3p5*c5p1*c9p4*ss[56];
  root_4 += 2520*c3p5*c5p2*c7p3*ss[46];
  root_4 += 2520*c3p5*c5p2*c9p3*ss[52];
  root_4 += 2520*c3p5*c5p3*c7p2*ss[44];
  root_4 += 2520*c3p5*c5p3*c9p2*ss[48];
  root_4 += 1260*c3p5*c5p4*c7p1*ss[42];
  root_4 += 1260*c3p5*c5p4*c9p1*ss[44];
  root_4 += 1260*c3p5*c7p1*c9p4*ss[58];
  root_4 += 2520*c3p5*c7p2*c9p3*ss[56];
  root_4 += 2520*c3p5*c7p3*c9p2*ss[54];
  root_4 += 1260*c3p5*c7p4*c9p1*ss[52];
  root_4 += 840*c3p6*c5p1*c7p3*ss[44];
  root_4 += 840*c3p6*c5p1*c9p3*ss[50];
  root_4 += 1260*c3p6*c5p2*c7p2*ss[42];
  root_4 += 1260*c3p6*c5p2*c9p2*ss[46];
  root_4 += 840*c3p6*c5p3*c7p1*ss[40];
  root_4 += 840*c3p6*c5p3*c9p1*ss[42];
  root_4 += 840*c3p6*c7p1*c9p3*ss[52];
  root_4 += 1260*c3p6*c7p2*c9p2*ss[50];
  root_4 += 840*c3p6*c7p3*c9p1*ss[48];
  root_4 += 360*c3p7*c5p1*c7p2*ss[40];
  root_4 += 360*c3p7*c5p1*c9p2*ss[44];
  root_4 += 360*c3p7*c5p2*c7p1*ss[38];
  root_4 += 360*c3p7*c5p2*c9p1*ss[40];
  root_4 += 360*c3p7*c7p1*c9p2*ss[46];
  root_4 += 360*c3p7*c7p2*c9p1*ss[44];
  root_4 += 90*c3p8*c5p1*c7p1*ss[36];
  root_4 += 90*c3p8*c5p1*c9p1*ss[38];
  root_4 += 90*c3p8*c7p1*c9p1*ss[40];
  root_4 += 90*c5p1*c7p1*c9p8*ss[84];
  root_4 += 360*c5p1*c7p2*c9p7*ss[82];
  root_4 += 840*c5p1*c7p3*c9p6*ss[80];
  root_4 += 1260*c5p1*c7p4*c9p5*ss[78];
  root_4 += 1260*c5p1*c7p5*c9p4*ss[76];
  root_4 += 840*c5p1*c7p6*c9p3*ss[74];
  root_4 += 360*c5p1*c7p7*c9p2*ss[72];
  root_4 += 90*c5p1*c7p8*c9p1*ss[70];
  root_4 += 360*c5p2*c7p1*c9p7*ss[80];
  root_4 += 1260*c5p2*c7p2*c9p6*ss[78];
  root_4 += 2520*c5p2*c7p3*c9p5*ss[76];
  root_4 += 3150*c5p2*c7p4*c9p4*ss[74];
  root_4 += 2520*c5p2*c7p5*c9p3*ss[72];
  root_4 += 1260*c5p2*c7p6*c9p2*ss[70];
  root_4 += 360*c5p2*c7p7*c9p1*ss[68];
  root_4 += 840*c5p3*c7p1*c9p6*ss[76];
  root_4 += 2520*c5p3*c7p2*c9p5*ss[74];
  root_4 += 4200*c5p3*c7p3*c9p4*ss[72];
  root_4 += 4200*c5p3*c7p4*c9p3*ss[70];
  root_4 += 2520*c5p3*c7p5*c9p2*ss[68];
  root_4 += 840*c5p3*c7p6*c9p1*ss[66];
  root_4 += 1260*c5p4*c7p1*c9p5*ss[72];
  root_4 += 3150*c5p4*c7p2*c9p4*ss[70];
  root_4 += 4200*c5p4*c7p3*c9p3*ss[68];
  root_4 += 3150*c5p4*c7p4*c9p2*ss[66];
  root_4 += 1260*c5p4*c7p5*c9p1*ss[64];
  root_4 += 1260*c5p5*c7p1*c9p4*ss[68];
  root_4 += 2520*c5p5*c7p2*c9p3*ss[66];
  root_4 += 2520*c5p5*c7p3*c9p2*ss[64];
  root_4 += 1260*c5p5*c7p4*c9p1*ss[62];
  root_4 += 840*c5p6*c7p1*c9p3*ss[64];
  root_4 += 1260*c5p6*c7p2*c9p2*ss[62];
  root_4 += 840*c5p6*c7p3*c9p1*ss[60];
  root_4 += 360*c5p7*c7p1*c9p2*ss[60];
  root_4 += 360*c5p7*c7p2*c9p1*ss[58];
  root_4 += 90*c5p8*c7p1*c9p1*ss[56];
  root_4 += 10*c1p1*c3p9*ss[28];
  root_4 += 10*c1p1*c5p9*ss[46];
  root_4 += 10*c1p1*c7p9*ss[64];
  root_4 += 10*c1p1*c9p9*ss[82];
  root_4 += 45*c1p2*c3p8*ss[26];
  root_4 += 45*c1p2*c5p8*ss[42];
  root_4 += 45*c1p2*c7p8*ss[58];
  root_4 += 45*c1p2*c9p8*ss[74];
  root_4 += 120*c1p3*c3p7*ss[24];
  root_4 += 120*c1p3*c5p7*ss[38];
  root_4 += 120*c1p3*c7p7*ss[52];
  root_4 += 120*c1p3*c9p7*ss[66];
  root_4 += 210*c1p4*c3p6*ss[22];
  root_4 += 210*c1p4*c5p6*ss[34];
  root_4 += 210*c1p4*c7p6*ss[46];
  root_4 += 210*c1p4*c9p6*ss[58];
  root_4 += 252*c1p5*c3p5*ss[20];
  root_4 += 252*c1p5*c5p5*ss[30];
  root_4 += 252*c1p5*c7p5*ss[40];
  root_4 += 252*c1p5*c9p5*ss[50];
  root_4 += 210*c1p6*c3p4*ss[18];
  root_4 += 210*c1p6*c5p4*ss[26];
  root_4 += 210*c1p6*c7p4*ss[34];
  root_4 += 210*c1p6*c9p4*ss[42];
  root_4 += 120*c1p7*c3p3*ss[16];
  root_4 += 120*c1p7*c5p3*ss[22];
  root_4 += 120*c1p7*c7p3*ss[28];
  root_4 += 120*c1p7*c9p3*ss[34];
  root_4 += 45*c1p8*c3p2*ss[14];
  root_4 += 45*c1p8*c5p2*ss[18];
  root_4 += 45*c1p8*c7p2*ss[22];
  root_4 += 45*c1p8*c9p2*ss[26];
  root_4 += 10*c1p9*c3p1*ss[12];
  root_4 += 10*c1p9*c5p1*ss[14];
  root_4 += 10*c1p9*c7p1*ss[16];
  root_4 += 10*c1p9*c9p1*ss[18];
  root_4 += 10*c3p1*c5p9*ss[48];
  root_4 += 10*c3p1*c7p9*ss[66];
  root_4 += 10*c3p1*c9p9*ss[84];
  root_4 += 45*c3p2*c5p8*ss[46];
  root_4 += 45*c3p2*c7p8*ss[62];
  root_4 += 45*c3p2*c9p8*ss[78];
  root_4 += 120*c3p3*c5p7*ss[44];
  root_4 += 120*c3p3*c7p7*ss[58];
  root_4 += 120*c3p3*c9p7*ss[72];
  root_4 += 210*c3p4*c5p6*ss[42];
  root_4 += 210*c3p4*c7p6*ss[54];
  root_4 += 210*c3p4*c9p6*ss[66];
  root_4 += 252*c3p5*c5p5*ss[40];
  root_4 += 252*c3p5*c7p5*ss[50];
  root_4 += 252*c3p5*c9p5*ss[60];
  root_4 += 210*c3p6*c5p4*ss[38];
  root_4 += 210*c3p6*c7p4*ss[46];
  root_4 += 210*c3p6*c9p4*ss[54];
  root_4 += 120*c3p7*c5p3*ss[36];
  root_4 += 120*c3p7*c7p3*ss[42];
  root_4 += 120*c3p7*c9p3*ss[48];
  root_4 += 45*c3p8*c5p2*ss[34];
  root_4 += 45*c3p8*c7p2*ss[38];
  root_4 += 45*c3p8*c9p2*ss[42];
  root_4 += 10*c3p9*c5p1*ss[32];
  root_4 += 10*c3p9*c7p1*ss[34];
  root_4 += 10*c3p9*c9p1*ss[36];
  root_4 += 10*c5p1*c7p9*ss[68];
  root_4 += 10*c5p1*c9p9*ss[86];
  root_4 += 45*c5p2*c7p8*ss[66];
  root_4 += 45*c5p2*c9p8*ss[82];
  root_4 += 120*c5p3*c7p7*ss[64];
  root_4 += 120*c5p3*c9p7*ss[78];
  root_4 += 210*c5p4*c7p6*ss[62];
  root_4 += 210*c5p4*c9p6*ss[74];
  root_4 += 252*c5p5*c7p5*ss[60];
  root_4 += 252*c5p5*c9p5*ss[70];
  root_4 += 210*c5p6*c7p4*ss[58];
  root_4 += 210*c5p6*c9p4*ss[66];
  root_4 += 120*c5p7*c7p3*ss[56];
  root_4 += 120*c5p7*c9p3*ss[62];
  root_4 += 45*c5p8*c7p2*ss[54];
  root_4 += 45*c5p8*c9p2*ss[58];
  root_4 += 10*c5p9*c7p1*ss[52];
  root_4 += 10*c5p9*c9p1*ss[54];
  root_4 += 10*c7p1*c9p9*ss[88];
  root_4 += 45*c7p2*c9p8*ss[86];
  root_4 += 120*c7p3*c9p7*ss[84];
  root_4 += 210*c7p4*c9p6*ss[82];
  root_4 += 252*c7p5*c9p5*ss[80];
  root_4 += 210*c7p6*c9p4*ss[78];
  root_4 += 120*c7p7*c9p3*ss[76];
  root_4 += 45*c7p8*c9p2*ss[74];
  root_4 += 10*c7p9*c9p1*ss[72];
  root_4 += c1p10*ss[10];
  root_4 += c3p10*ss[30];
  root_4 += c5p10*ss[50];
  root_4 += c7p10*ss[70];
  root_4 += c9p10*ss[90];
  root_4 += -tmom[10];
  roots[0]=convert<T>(root_0);
  roots[1]=convert<T>(root_1);
  roots[2]=convert<T>(root_2);
  roots[3]=convert<T>(root_3);
  roots[4]=convert<T>(root_4);
}
void roots_and_jacobian(const T *parameters, T *roots, T *jacobian) const {
  T c1p1 = parameters[0];
  T c3p1 = parameters[1];
  T c5p1 = parameters[2];
  T c7p1 = parameters[3];
  T c9p1 = parameters[4];
  T c1p2 = c1p1*c1p1;
  T c3p2 = c3p1*c3p1;
  T c5p2 = c5p1*c5p1;
  T c7p2 = c7p1*c7p1;
  T c9p2 = c9p1*c9p1;
  TACC root_0 = convert<TACC>(2*c1p1*c3p1*ss[4]);
  root_0 += 2*c1p1*c5p1*ss[6];
  root_0 += 2*c1p1*c7p1*ss[8];
  root_0 += 2*c1p1*c9p1*ss[10];
  root_0 += 2*c3p1*c5p1*ss[8];
  root_0 += 2*c3p1*c7p1*ss[10];
  root_0 += 2*c3p1*c9p1*ss[12];
  root_0 += 2*c5p1*c7p1*ss[12];
  root_0 += 2*c5p1*c9p1*ss[14];
  root_0 += 2*c7p1*c9p1*ss[16];
  root_0 += c1p2*ss[2];
  root_0 += c3p2*ss[6];
  root_0 += c5p2*ss[10];
  root_0 += c7p2*ss[14];
  root_0 += c9p2*ss[18];
  root_0 += -tmom[2];
  T c1p3 = c1p2*c1p1;
  T c1p4 = c1p3*c1p1;
  T c3p3 = c3p2*c3p1;
  T c3p4 = c3p3*c3p1;
  T c5p3 = c5p2*c5p1;
  T c5p4 = c5p3*c5p1;
  T c7p3 = c7p2*c7p1;
  T c7p4 = c7p3*c7p1;
  T c9p3 = c9p2*c9p1;
  T c9p4 = c9p3*c9p1;
  TACC root_1 = convert<TACC>(24*c1p1*c3p1*c5p1*c7p1*ss[16]);
  root_1 += 24*c1p1*c3p1*c5p1*c9p1*ss[18];
  root_1 += 24*c1p1*c3p1*c7p1*c9p1*ss[20];
  root_1 += 24*c1p1*c5p1*c7p1*c9p1*ss[22];
  root_1 += 24*c3p1*c5p1*c7p1*c9p1*ss[24];
  root_1 += 12*c1p1*c3p1*c5p2*ss[14];
  root_1 += 12*c1p1*c3p1*c7p2*ss[18];
  root_1 += 12*c1p1*c3p1*c9p2*ss[22];
  root_1 += 12*c1p1*c3p2*c5p1*ss[12];
  root_1 += 12*c1p1*c3p2*c7p1*ss[14];
  root_1 += 12*c1p1*c3p2*c9p1*ss[16];
  root_1 += 12*c1p1*c5p1*c7p2*ss[20];
  root_1 += 12*c1p1*c5p1*c9p2*ss[24];
  root_1 += 12*c1p1*c5p2*c7p1*ss[18];
  root_1 += 12*c1p1*c5p2*c9p1*ss[20];
  root_1 += 12*c1p1*c7p1*c9p2*ss[26];
  root_1 += 12*c1p1*c7p2*c9p1*ss[24];
  root_1 += 12*c1p2*c3p1*c5p1*ss[10];
  root_1 += 12*c1p2*c3p1*c7p1*ss[12];
  root_1 += 12*c1p2*c3p1*c9p1*ss[14];
  root_1 += 12*c1p2*c5p1*c7p1*ss[14];
  root_1 += 12*c1p2*c5p1*c9p1*ss[16];
  root_1 += 12*c1p2*c7p1*c9p1*ss[18];
  root_1 += 12*c3p1*c5p1*c7p2*ss[22];
  root_1 += 12*c3p1*c5p1*c9p2*ss[26];
  root_1 += 12*c3p1*c5p2*c7p1*ss[20];
  root_1 += 12*c3p1*c5p2*c9p1*ss[22];
  root_1 += 12*c3p1*c7p1*c9p2*ss[28];
  root_1 += 12*c3p1*c7p2*c9p1*ss[26];
  root_1 += 12*c3p2*c5p1*c7p1*ss[18];
  root_1 += 12*c3p2*c5p1*c9p1*ss[20];
  root_1 += 12*c3p2*c7p1*c9p1*ss[22];
  root_1 += 12*c5p1*c7p1*c9p2*ss[30];
  root_1 += 12*c5p1*c7p2*c9p1*ss[28];
  root_1 += 12*c5p2*c7p1*c9p1*ss[26];
  root_1 += 4*c1p1*c3p3*ss[10];
  root_1 += 4*c1p1*c5p3*ss[16];
  root_1 += 4*c1p1*c7p3*ss[22];
  root_1 += 4*c1p1*c9p3*ss[28];
  root_1 += 6*c1p2*c3p2*ss[8];
  root_1 += 6*c1p2*c5p2*ss[12];
  root_1 += 6*c1p2*c7p2*ss[16];
  root_1 += 6*c1p2*c9p2*ss[20];
  root_1 += 4*c1p3*c3p1*ss[6];
  root_1 += 4*c1p3*c5p1*ss[8];
  root_1 += 4*c1p3*c7p1*ss[10];
  root_1 += 4*c1p3*c9p1*ss[12];
  root_1 += 4*c3p1*c5p3*ss[18];
  root_1 += 4*c3p1*c7p3*ss[24];
  root_1 += 4*c3p1*c9p3*ss[30];
  root_1 += 6*c3p2*c5p2*ss[16];
  root_1 += 6*c3p2*c7p2*ss[20];
  root_1 += 6*c3p2*c9p2*ss[24];
  root_1 += 4*c3p3*c5p1*ss[14];
  root_1 += 4*c3p3*c7p1*ss[16];
  root_1 += 4*c3p3*c9p1*ss[18];
  root_1 += 4*c5p1*c7p3*ss[26];
  root_1 += 4*c5p1*c9p3*ss[32];
  root_1 += 6*c5p2*c7p2*ss[24];
  root_1 += 6*c5p2*c9p2*ss[28];
  root_1 += 4*c5p3*c7p1*ss[22];
  root_1 += 4*c5p3*c9p1*ss[24];
  root_1 += 4*c7p1*c9p3*ss[34];
  root_1 += 6*c7p2*c9p2*ss[32];
  root_1 += 4*c7p3*c9p1*ss[30];
  root_1 += c1p4*ss[4];
  root_1 += c3p4*ss[12];
  root_1 += c5p4*ss[20];
  root_1 += c7p4*ss[28];
  root_1 += c9p4*ss[36];
  root_1 += -tmom[4];
  T c1p5 = c1p4*c1p1;
  T c1p6 = c1p5*c1p1;
  T c3p5 = c3p4*c3p1;
  T c3p6 = c3p5*c3p1;
  T c5p5 = c5p4*c5p1;
  T c5p6 = c5p5*c5p1;
  T c7p5 = c7p4*c7p1;
  T c7p6 = c7p5*c7p1;
  T c9p5 = c9p4*c9p1;
  T c9p6 = c9p5*c9p1;
  TACC root_2 = convert<TACC>(360*c1p1*c3p1*c5p1*c7p1*c9p2*ss[34]);
  root_2 += 360*c1p1*c3p1*c5p1*c7p2*c9p1*ss[32];
  root_2 += 360*c1p1*c3p1*c5p2*c7p1*c9p1*ss[30];
  root_2 += 360*c1p1*c3p2*c5p1*c7p1*c9p1*ss[28];
  root_2 += 360*c1p2*c3p1*c5p1*c7p1*c9p1*ss[26];
  root_2 += 120*c1p1*c3p1*c5p1*c7p3*ss[30];
  root_2 += 120*c1p1*c3p1*c5p1*c9p3*ss[36];
  root_2 += 180*c1p1*c3p1*c5p2*c7p2*ss[28];
  root_2 += 180*c1p1*c3p1*c5p2*c9p2*ss[32];
  root_2 += 120*c1p1*c3p1*c5p3*c7p1*ss[26];
  root_2 += 120*c1p1*c3p1*c5p3*c9p1*ss[28];
  root_2 += 120*c1p1*c3p1*c7p1*c9p3*ss[38];
  root_2 += 180*c1p1*c3p1*c7p2*c9p2*ss[36];
  root_2 += 120*c1p1*c3p1*c7p3*c9p1*ss[34];
  root_2 += 180*c1p1*c3p2*c5p1*c7p2*ss[26];
  root_2 += 180*c1p1*c3p2*c5p1*c9p2*ss[30];
  root_2 += 180*c1p1*c3p2*c5p2*c7p1*ss[24];
  root_2 += 180*c1p1*c3p2*c5p2*c9p1*ss[26];
  root_2 += 180*c1p1*c3p2*c7p1*c9p2*ss[32];
  root_2 += 180*c1p1*c3p2*c7p2*c9p1*ss[30];
  root_2 += 120*c1p1*c3p3*c5p1*c7p1*ss[22];
  root_2 += 120*c1p1*c3p3*c5p1*c9p1*ss[24];
  root_2 += 120*c1p1*c3p3*c7p1*c9p1*ss[26];
  root_2 += 120*c1p1*c5p1*c7p1*c9p3*ss[40];
  root_2 += 180*c1p1*c5p1*c7p2*c9p2*ss[38];
  root_2 += 120*c1p1*c5p1*c7p3*c9p1*ss[36];
  root_2 += 180*c1p1*c5p2*c7p1*c9p2*ss[36];
  root_2 += 180*c1p1*c5p2*c7p2*c9p1*ss[34];
  root_2 += 120*c1p1*c5p3*c7p1*c9p1*ss[32];
  root_2 += 180*c1p2*c3p1*c5p1*c7p2*ss[24];
  root_2 += 180*c1p2*c3p1*c5p1*c9p2*ss[28];
  root_2 += 180*c1p2*c3p1*c5p2*c7p1*ss[22];
  root_2 += 180*c1p2*c3p1*c5p2*c9p1*ss[24];
  root_2 += 180*c1p2*c3p1*c7p1*c9p2*ss[30];
  root_2 += 180*c1p2*c3p1*c7p2*c9p1*ss[28];
  root_2 += 180*c1p2*c3p2*c5p1*c7p1*ss[20];
  root_2 += 180*c1p2*c3p2*c5p1*c9p1*ss[22];
  root_2 += 180*c1p2*c3p2*c7p1*c9p1*ss[24];
  root_2 += 180*c1p2*c5p1*c7p1*c9p2*ss[32];
  root_2 += 180*c1p2*c5p1*c7p2*c9p1*ss[30];
  root_2 += 180*c1p2*c5p2*c7p1*c9p1*ss[28];
  root_2 += 120*c1p3*c3p1*c5p1*c7p1*ss[18];
  root_2 += 120*c1p3*c3p1*c5p1*c9p1*ss[20];
  root_2 += 120*c1p3*c3p1*c7p1*c9p1*ss[22];
  root_2 += 120*c1p3*c5p1*c7p1*c9p1*ss[24];
  root_2 += 120*c3p1*c5p1*c7p1*c9p3*ss[42];
  root_2 += 180*c3p1*c5p1*c7p2*c9p2*ss[40];
  root_2 += 120*c3p1*c5p1*c7p3*c9p1*ss[38];
  root_2 += 180*c3p1*c5p2*c7p1*c9p2*ss[38];
  root_2 += 180*c3p1*c5p2*c7p2*c9p1*ss[36];
  root_2 += 120*c3p1*c5p3*c7p1*c9p1*ss[34];
  root_2 += 180*c3p2*c5p1*c7p1*c9p2*ss[36];
  root_2 += 180*c3p2*c5p1*c7p2*c9p1*ss[34];
  root_2 += 180*c3p2*c5p2*c7p1*c9p1*ss[32];
  root_2 += 120*c3p3*c5p1*c7p1*c9p1*ss[30];
  root_2 += 30*c1p1*c3p1*c5p4*ss[24];
  root_2 += 30*c1p1*c3p1*c7p4*ss[32];
  root_2 += 30*c1p1*c3p1*c9p4*ss[40];
  root_2 += 60*c1p1*c3p2*c5p3*ss[22];
  root_2 += 60*c1p1*c3p2*c7p3*ss[28];
  root_2 += 60*c1p1*c3p2*c9p3*ss[34];
  root_2 += 60*c1p1*c3p3*c5p2*ss[20];
  root_2 += 60*c1p1*c3p3*c7p2*ss[24];
  root_2 += 60*c1p1*c3p3*c9p2*ss[28];
  root_2 += 30*c1p1*c3p4*c5p1*ss[18];
  root_2 += 30*c1p1*c3p4*c7p1*ss[20];
  root_2 += 30*c1p1*c3p4*c9p1*ss[22];
  root_2 += 30*c1p1*c5p1*c7p4*ss[34];
  root_2 += 30*c1p1*c5p1*c9p4*ss[42];
  root_2 += 60*c1p1*c5p2*c7p3*ss[32];
  root_2 += 60*c1p1*c5p2*c9p3*ss[38];
  root_2 += 60*c1p1*c5p3*c7p2*ss[30];
  root_2 += 60*c1p1*c5p3*c9p2*ss[34];
  root_2 += 30*c1p1*c5p4*c7p1*ss[28];
  root_2 += 30*c1p1*c5p4*c9p1*ss[30];
  root_2 += 30*c1p1*c7p1*c9p4*ss[44];
  root_2 += 60*c1p1*c7p2*c9p3*ss[42];
  root_2 += 60*c1p1*c7p3*c9p2*ss[40];
  root_2 += 30*c1p1*c7p4*c9p1*ss[38];
  root_2 += 60*c1p2*c3p1*c5p3*ss[20];
  root_2 += 60*c1p2*c3p1*c7p3*ss[26];
  root_2 += 60*c1p2*c3p1*c9p3*ss[32];
  root_2 += 90*c1p2*c3p2*c5p2*ss[18];
  root_2 += 90*c1p2*c3p2*c7p2*ss[22];
  root_2 += 90*c1p2*c3p2*c9p2*ss[26];
  root_2 += 60*c1p2*c3p3*c5p1*ss[16];
  root_2 += 60*c1p2*c3p3*c7p1*ss[18];
  root_2 += 60*c1p2*c3p3*c9p1*ss[20];
  root_2 += 60*c1p2*c5p1*c7p3*ss[28];
  root_2 += 60*c1p2*c5p1*c9p3*ss[34];
  root_2 += 90*c1p2*c5p2*c7p2*ss[26];
  root_2 += 90*c1p2*c5p2*c9p2*ss[30];
  root_2 += 60*c1p2*c5p3*c7p1*ss[24];
  root_2 += 60*c1p2*c5p3*c9p1*ss[26];
  root_2 += 60*c1p2*c7p1*c9p3*ss[36];
  root_2 += 90*c1p2*c7p2*c9p2*ss[34];
  root_2 += 60*c1p2*c7p3*c9p1*ss[32];
  root_2 += 60*c1p3*c3p1*c5p2*ss[16];
  root_2 += 60*c1p3*c3p1*c7p2*ss[20];
  root_2 += 60*c1p3*c3p1*c9p2*ss[24];
  root_2 += 60*c1p3*c3p2*c5p1*ss[14];
  root_2 += 60*c1p3*c3p2*c7p1*ss[16];
  root_2 += 60*c1p3*c3p2*c9p1*ss[18];
  root_2 += 60*c1p3*c5p1*c7p2*ss[22];
  root_2 += 60*c1p3*c5p1*c9p2*ss[26];
  root_2 += 60*c1p3*c5p2*c7p1*ss[20];
  root_2 += 60*c1p3*c5p2*c9p1*ss[22];
  root_2 += 60*c1p3*c7p1*c9p2*ss[28];
  root_2 += 60*c1p3*c7p2*c9p1*ss[26];
  root_2 += 30*c1p4*c3p1*c5p1*ss[12];
  root_2 += 30*c1p4*c3p1*c7p1*ss[14];
  root_2 += 30*c1p4*c3p1*c9p1*ss[16];
  root_2 += 30*c1p4*c5p1*c7p1*ss[16];
  root_2 += 30*c1p4*c5p1*c9p1*ss[18];
  root_2 += 30*c1p4*c7p1*c9p1*ss[20];
  root_2 += 30*c3p1*c5p1*c7p4*ss[36];
  root_2 += 30*c3p1*c5p1*c9p4*ss[44];
  root_2 += 60*c3p1*c5p2*c7p3*ss[34];
  root_2 += 60*c3p1*c5p2*c9p3*ss[40];
  root_2 += 60*c3p1*c5p3*c7p2*ss[32];
  root_2 += 60*c3p1*c5p3*c9p2*ss[36];
  root_2 += 30*c3p1*c5p4*c7p1*ss[30];
  root_2 += 30*c3p1*c5p4*c9p1*ss[32];
  root_2 += 30*c3p1*c7p1*c9p4*ss[46];
  root_2 += 60*c3p1*c7p2*c9p3*ss[44];
  root_2 += 60*c3p1*c7p3*c9p2*ss[42];
  root_2 += 30*c3p1*c7p4*c9p1*ss[40];
  root_2 += 60*c3p2*c5p1*c7p3*ss[32];
  root_2 += 60*c3p2*c5p1*c9p3*ss[38];
  root_2 += 90*c3p2*c5p2*c7p2*ss[30];
  root_2 += 90*c3p2*c5p2*c9p2*ss[34];
  root_2 += 60*c3p2*c5p3*c7p1*ss[28];
  root_2 += 60*c3p2*c5p3*c9p1*ss[30];
  root_2 += 60*c3p2*c7p1*c9p3*ss[40];
  root_2 += 90*c3p2*c7p2*c9p2*ss[38];
  root_2 += 60*c3p2*c7p3*c9p1*ss[36];
  root_2 += 60*c3p3*c5p1*c7p2*ss[28];
  root_2 += 60*c3p3*c5p1*c9p2*ss[32];
  root_2 += 60*c3p3*c5p2*c7p1*ss[26];
  root_2 += 60*c3p3*c5p2*c9p1*ss[28];
  root_2 += 60*c3p3*c7p1*c9p2*ss[34];
  root_2 += 60*c3p3*c7p2*c9p1*ss[32];
  root_2 += 30*c3p4*c5p1*c7p1*ss[24];
  root_2 += 30*c3p4*c5p1*c9p1*ss[26];
  root_2 += 30*c3p4*c7p1*c9p1*ss[28];
  root_2 += 30*c5p1*c7p1*c9p4*ss[48];
  root_2 += 60*c5p1*c7p2*c9p3*ss[46];
  root_2 += 60*c5p1*c7p3*c9p2*ss[44];
  root_2 += 30*c5p1*c7p4*c9p1*ss[42];
  root_2 += 60*c5p2*c7p1*c9p3*ss[44];
  root_2 += 90*c5p2*c7p2*c9p2*ss[42];
  root_2 += 60*c5p2*c7p3*c9p1*ss[40];
  root_2 += 60*c5p3*c7p1*c9p2*ss[40];
  root_2 += 60*c5p3*c7p2*c9p1*ss[38];
  root_2 += 30*c5p4*c7p1*c9p1*ss[36];
  root_2 += 6*c1p1*c3p5*ss[16];
  root_2 += 6*c1p1*c5p5*ss[26];
  root_2 += 6*c1p1*c7p5*ss[36];
  root_2 += 6*c1p1*c9p5*ss[46];
  root_2 += 15*c1p2*c3p4*ss[14];
  root_2 += 15*c1p2*c5p4*ss[22];
  root_2 += 15*c1p2*c7p4*ss[30];
  root_2 += 15*c1p2*c9p4*ss[38];
  root_2 += 20*c1p3*c3p3*ss[12];
  root_2 += 20*c1p3*c5p3*ss[18];
  root_2 += 20*c1p3*c7p3*ss[24];
  root_2 += 20*c1p3*c9p3*ss[30];
  root_2 += 15*c1p4*c3p2*ss[10];
  root_2 += 15*c1p4*c5p2*ss[14];
  root_2 += 15*c1p4*c7p2*ss[18];
  root_2 += 15*c1p4*c9p2*ss[22];
  root_2 += 6*c1p5*c3p1*ss[8];
  root_2 += 6*c1p5*c5p1*ss[10];
  root_2 += 6*c1p5*c7p1*ss[12];
  root_2 += 6*c1p5*c9p1*ss[14];
  root_2 += 6*c3p1*c5p5*ss[28];
  root_2 += 6*c3p1*c7p5*ss[38];
  root_2 += 6*c3p1*c9p5*ss[48];
  root_2 += 15*c3p2*c5p4*ss[26];
  root_2 += 15*c3p2*c7p4*ss[34];
  root_2 += 15*c3p2*c9p4*ss[42];
  root_2 += 20*c3p3*c5p3*ss[24];
  root_2 += 20*c3p3*c7p3*ss[30];
  root_2 += 20*c3p3*c9p3*ss[36];
  root_2 += 15*c3p4*c5p2*ss[22];
  root_2 += 15*c3p4*c7p2*ss[26];
  root_2 += 15*c3p4*c9p2*ss[30];
  root_2 += 6*c3p5*c5p1*ss[20];
  root_2 += 6*c3p5*c7p1*ss[22];
  root_2 += 6*c3p5*c9p1*ss[24];
  root_2 += 6*c5p1*c7p5*ss[40];
  root_2 += 6*c5p1*c9p5*ss[50];
  root_2 += 15*c5p2*c7p4*ss[38];
  root_2 += 15*c5p2*c9p4*ss[46];
  root_2 += 20*c5p3*c7p3*ss[36];
  root_2 += 20*c5p3*c9p3*ss[42];
  root_2 += 15*c5p4*c7p2*ss[34];
  root_2 += 15*c5p4*c9p2*ss[38];
  root_2 += 6*c5p5*c7p1*ss[32];
  root_2 += 6*c5p5*c9p1*ss[34];
  root_2 += 6*c7p1*c9p5*ss[52];
  root_2 += 15*c7p2*c9p4*ss[50];
  root_2 += 20*c7p3*c9p3*ss[48];
  root_2 += 15*c7p4*c9p2*ss[46];
  root_2 += 6*c7p5*c9p1*ss[44];
  root_2 += c1p6*ss[6];
  root_2 += c3p6*ss[18];
  root_2 += c5p6*ss[30];
  root_2 += c7p6*ss[42];
  root_2 += c9p6*ss[54];
  root_2 += -tmom[6];
  T c1p7 = c1p6*c1p1;
  T c1p8 = c1p7*c1p1;
  T c3p7 = c3p6*c3p1;
  T c3p8 = c3p7*c3p1;
  T c5p7 = c5p6*c5p1;
  T c5p8 = c5p7*c5p1;
  T c7p7 = c7p6*c7p1;
  T c7p8 = c7p7*c7p1;
  T c9p7 = c9p6*c9p1;
  T c9p8 = c9p7*c9p1;
  TACC root_3 = convert<TACC>(1680*c1p1*c3p1*c5p1*c7p1*c9p4*ss[52]);
  root_3 += 3360*c1p1*c3p1*c5p1*c7p2*c9p3*ss[50];
  root_3 += 3360*c1p1*c3p1*c5p1*c7p3*c9p2*ss[48];
  root_3 += 1680*c1p1*c3p1*c5p1*c7p4*c9p1*ss[46];
  root_3 += 3360*c1p1*c3p1*c5p2*c7p1*c9p3*ss[48];
  root_3 += 5040*c1p1*c3p1*c5p2*c7p2*c9p2*ss[46];
  root_3 += 3360*c1p1*c3p1*c5p2*c7p3*c9p1*ss[44];
  root_3 += 3360*c1p1*c3p1*c5p3*c7p1*c9p2*ss[44];
  root_3 += 3360*c1p1*c3p1*c5p3*c7p2*c9p1*ss[42];
  root_3 += 1680*c1p1*c3p1*c5p4*c7p1*c9p1*ss[40];
  root_3 += 3360*c1p1*c3p2*c5p1*c7p1*c9p3*ss[46];
  root_3 += 5040*c1p1*c3p2*c5p1*c7p2*c9p2*ss[44];
  root_3 += 3360*c1p1*c3p2*c5p1*c7p3*c9p1*ss[42];
  root_3 += 5040*c1p1*c3p2*c5p2*c7p1*c9p2*ss[42];
  root_3 += 5040*c1p1*c3p2*c5p2*c7p2*c9p1*ss[40];
  root_3 += 3360*c1p1*c3p2*c5p3*c7p1*c9p1*ss[38];
  root_3 += 3360*c1p1*c3p3*c5p1*c7p1*c9p2*ss[40];
  root_3 += 3360*c1p1*c3p3*c5p1*c7p2*c9p1*ss[38];
  root_3 += 3360*c1p1*c3p3*c5p2*c7p1*c9p1*ss[36];
  root_3 += 1680*c1p1*c3p4*c5p1*c7p1*c9p1*ss[34];
  root_3 += 3360*c1p2*c3p1*c5p1*c7p1*c9p3*ss[44];
  root_3 += 5040*c1p2*c3p1*c5p1*c7p2*c9p2*ss[42];
  root_3 += 3360*c1p2*c3p1*c5p1*c7p3*c9p1*ss[40];
  root_3 += 5040*c1p2*c3p1*c5p2*c7p1*c9p2*ss[40];
  root_3 += 5040*c1p2*c3p1*c5p2*c7p2*c9p1*ss[38];
  root_3 += 3360*c1p2*c3p1*c5p3*c7p1*c9p1*ss[36];
  root_3 += 5040*c1p2*c3p2*c5p1*c7p1*c9p2*ss[38];
  root_3 += 5040*c1p2*c3p2*c5p1*c7p2*c9p1*ss[36];
  root_3 += 5040*c1p2*c3p2*c5p2*c7p1*c9p1*ss[34];
  root_3 += 3360*c1p2*c3p3*c5p1*c7p1*c9p1*ss[32];
  root_3 += 3360*c1p3*c3p1*c5p1*c7p1*c9p2*ss[36];
  root_3 += 3360*c1p3*c3p1*c5p1*c7p2*c9p1*ss[34];
  root_3 += 3360*c1p3*c3p1*c5p2*c7p1*c9p1*ss[32];
  root_3 += 3360*c1p3*c3p2*c5p1*c7p1*c9p1*ss[30];
  root_3 += 1680*c1p4*c3p1*c5p1*c7p1*c9p1*ss[28];
  root_3 += 336*c1p1*c3p1*c5p1*c7p5*ss[44];
  root_3 += 336*c1p1*c3p1*c5p1*c9p5*ss[54];
  root_3 += 840*c1p1*c3p1*c5p2*c7p4*ss[42];
  root_3 += 840*c1p1*c3p1*c5p2*c9p4*ss[50];
  root_3 += 1120*c1p1*c3p1*c5p3*c7p3*ss[40];
  root_3 += 1120*c1p1*c3p1*c5p3*c9p3*ss[46];
  root_3 += 840*c1p1*c3p1*c5p4*c7p2*ss[38];
  root_3 += 840*c1p1*c3p1*c5p4*c9p2*ss[42];
  root_3 += 336*c1p1*c3p1*c5p5*c7p1*ss[36];
  root_3 += 336*c1p1*c3p1*c5p5*c9p1*ss[38];
  root_3 += 336*c1p1*c3p1*c7p1*c9p5*ss[56];
  root_3 += 840*c1p1*c3p1*c7p2*c9p4*ss[54];
  root_3 += 1120*c1p1*c3p1*c7p3*c9p3*ss[52];
  root_3 += 840*c1p1*c3p1*c7p4*c9p2*ss[50];
  root_3 += 336*c1p1*c3p1*c7p5*c9p1*ss[48];
  root_3 += 840*c1p1*c3p2*c5p1*c7p4*ss[40];
  root_3 += 840*c1p1*c3p2*c5p1*c9p4*ss[48];
  root_3 += 1680*c1p1*c3p2*c5p2*c7p3*ss[38];
  root_3 += 1680*c1p1*c3p2*c5p2*c9p3*ss[44];
  root_3 += 1680*c1p1*c3p2*c5p3*c7p2*ss[36];
  root_3 += 1680*c1p1*c3p2*c5p3*c9p2*ss[40];
  root_3 += 840*c1p1*c3p2*c5p4*c7p1*ss[34];
  root_3 += 840*c1p1*c3p2*c5p4*c9p1*ss[36];
  root_3 += 840*c1p1*c3p2*c7p1*c9p4*ss[50];
  root_3 += 1680*c1p1*c3p2*c7p2*c9p3*ss[48];
  root_3 += 1680*c1p1*c3p2*c7p3*c9p2*ss[46];
  root_3 += 840*c1p1*c3p2*c7p4*c9p1*ss[44];
  root_3 += 1120*c1p1*c3p3*c5p1*c7p3*ss[36];
  root_3 += 1120*c1p1*c3p3*c5p1*c9p3*ss[42];
  root_3 += 1680*c1p1*c3p3*c5p2*c7p2*ss[34];
  root_3 += 1680*c1p1*c3p3*c5p2*c9p2*ss[38];
  root_3 += 1120*c1p1*c3p3*c5p3*c7p1*ss[32];
  root_3 += 1120*c1p1*c3p3*c5p3*c9p1*ss[34];
  root_3 += 1120*c1p1*c3p3*c7p1*c9p3*ss[44];
  root_3 += 1680*c1p1*c3p3*c7p2*c9p2*ss[42];
  root_3 += 1120*c1p1*c3p3*c7p3*c9p1*ss[40];
  root_3 += 840*c1p1*c3p4*c5p1*c7p2*ss[32];
  root_3 += 840*c1p1*c3p4*c5p1*c9p2*ss[36];
  root_3 += 840*c1p1*c3p4*c5p2*c7p1*ss[30];
  root_3 += 840*c1p1*c3p4*c5p2*c9p1*ss[32];
  root_3 += 840*c1p1*c3p4*c7p1*c9p2*ss[38];
  root_3 += 840*c1p1*c3p4*c7p2*c9p1*ss[36];
  root_3 += 336*c1p1*c3p5*c5p1*c7p1*ss[28];
  root_3 += 336*c1p1*c3p5*c5p1*c9p1*ss[30];
  root_3 += 336*c1p1*c3p5*c7p1*c9p1*ss[32];
  root_3 += 336*c1p1*c5p1*c7p1*c9p5*ss[58];
  root_3 += 840*c1p1*c5p1*c7p2*c9p4*ss[56];
  root_3 += 1120*c1p1*c5p1*c7p3*c9p3*ss[54];
  root_3 += 840*c1p1*c5p1*c7p4*c9p2*ss[52];
  root_3 += 336*c1p1*c5p1*c7p5*c9p1*ss[50];
  root_3 += 840*c1p1*c5p2*c7p1*c9p4*ss[54];
  root_3 += 1680*c1p1*c5p2*c7p2*c9p3*ss[52];
  root_3 += 1680*c1p1*c5p2*c7p3*c9p2*ss[50];
  root_3 += 840*c1p1*c5p2*c7p4*c9p1*ss[48];
  root_3 += 1120*c1p1*c5p3*c7p1*c9p3*ss[50];
  root_3 += 1680*c1p1*c5p3*c7p2*c9p2*ss[48];
  root_3 += 1120*c1p1*c5p3*c7p3*c9p1*ss[46];
  root_3 += 840*c1p1*c5p4*c7p1*c9p2*ss[46];
  root_3 += 840*c1p1*c5p4*c7p2*c9p1*ss[44];
  root_3 += 336*c1p1*c5p5*c7p1*c9p1*ss[42];
  root_3 += 840*c1p2*c3p1*c5p1*c7p4*ss[38];
  root_3 += 840*c1p2*c3p1*c5p1*c9p4*ss[46];
  root_3 += 1680*c1p2*c3p1*c5p2*c7p3*ss[36];
  root_3 += 1680*c1p2*c3p1*c5p2*c9p3*ss[42];
  root_3 += 1680*c1p2*c3p1*c5p3*c7p2*ss[34];
  root_3 += 1680*c1p2*c3p1*c5p3*c9p2*ss[38];
  root_3 += 840*c1p2*c3p1*c5p4*c7p1*ss[32];
  root_3 += 840*c1p2*c3p1*c5p4*c9p1*ss[34];
  root_3 += 840*c1p2*c3p1*c7p1*c9p4*ss[48];
  root_3 += 1680*c1p2*c3p1*c7p2*c9p3*ss[46];
  root_3 += 1680*c1p2*c3p1*c7p3*c9p2*ss[44];
  root_3 += 840*c1p2*c3p1*c7p4*c9p1*ss[42];
  root_3 += 1680*c1p2*c3p2*c5p1*c7p3*ss[34];
  root_3 += 1680*c1p2*c3p2*c5p1*c9p3*ss[40];
  root_3 += 2520*c1p2*c3p2*c5p2*c7p2*ss[32];
  root_3 += 2520*c1p2*c3p2*c5p2*c9p2*ss[36];
  root_3 += 1680*c1p2*c3p2*c5p3*c7p1*ss[30];
  root_3 += 1680*c1p2*c3p2*c5p3*c9p1*ss[32];
  root_3 += 1680*c1p2*c3p2*c7p1*c9p3*ss[42];
  root_3 += 2520*c1p2*c3p2*c7p2*c9p2*ss[40];
  root_3 += 1680*c1p2*c3p2*c7p3*c9p1*ss[38];
  root_3 += 1680*c1p2*c3p3*c5p1*c7p2*ss[30];
  root_3 += 1680*c1p2*c3p3*c5p1*c9p2*ss[34];
  root_3 += 1680*c1p2*c3p3*c5p2*c7p1*ss[28];
  root_3 += 1680*c1p2*c3p3*c5p2*c9p1*ss[30];
  root_3 += 1680*c1p2*c3p3*c7p1*c9p2*ss[36];
  root_3 += 1680*c1p2*c3p3*c7p2*c9p1*ss[34];
  root_3 += 840*c1p2*c3p4*c5p1*c7p1*ss[26];
  root_3 += 840*c1p2*c3p4*c5p1*c9p1*ss[28];
  root_3 += 840*c1p2*c3p4*c7p1*c9p1*ss[30];
  root_3 += 840*c1p2*c5p1*c7p1*c9p4*ss[50];
  root_3 += 1680*c1p2*c5p1*c7p2*c9p3*ss[48];
  root_3 += 1680*c1p2*c5p1*c7p3*c9p2*ss[46];
  root_3 += 840*c1p2*c5p1*c7p4*c9p1*ss[44];
  root_3 += 1680*c1p2*c5p2*c7p1*c9p3*ss[46];
  root_3 += 2520*c1p2*c5p2*c7p2*c9p2*ss[44];
  root_3 += 1680*c1p2*c5p2*c7p3*c9p1*ss[42];
  root_3 += 1680*c1p2*c5p3*c7p1*c9p2*ss[42];
  root_3 += 1680*c1p2*c5p3*c7p2*c9p1*ss[40];
  root_3 += 840*c1p2*c5p4*c7p1*c9p1*ss[38];
  root_3 += 1120*c1p3*c3p1*c5p1*c7p3*ss[32];
  root_3 += 1120*c1p3*c3p1*c5p1*c9p3*ss[38];
  root_3 += 1680*c1p3*c3p1*c5p2*c7p2*ss[30];
  root_3 += 1680*c1p3*c3p1*c5p2*c9p2*ss[34];
  root_3 += 1120*c1p3*c3p1*c5p3*c7p1*ss[28];
  root_3 += 1120*c1p3*c3p1*c5p3*c9p1*ss[30];
  root_3 += 1120*c1p3*c3p1*c7p1*c9p3*ss[40];
  root_3 += 1680*c1p3*c3p1*c7p2*c9p2*ss[38];
  root_3 += 1120*c1p3*c3p1*c7p3*c9p1*ss[36];
  root_3 += 1680*c1p3*c3p2*c5p1*c7p2*ss[28];
  root_3 += 1680*c1p3*c3p2*c5p1*c9p2*ss[32];
  root_3 += 1680*c1p3*c3p2*c5p2*c7p1*ss[26];
  root_3 += 1680*c1p3*c3p2*c5p2*c9p1*ss[28];
  root_3 += 1680*c1p3*c3p2*c7p1*c9p2*ss[34];
  root_3 += 1680*c1p3*c3p2*c7p2*c9p1*ss[32];
  root_3 += 1120*c1p3*c3p3*c5p1*c7p1*ss[24];
  root_3 += 1120*c1p3*c3p3*c5p1*c9p1*ss[26];
  root_3 += 1120*c1p3*c3p3*c7p1*c9p1*ss[28];
  root_3 += 1120*c1p3*c5p1*c7p1*c9p3*ss[42];
  root_3 += 1680*c1p3*c5p1*c7p2*c9p2*ss[40];
  root_3 += 1120*c1p3*c5p1*c7p3*c9p1*ss[38];
  root_3 += 1680*c1p3*c5p2*c7p1*c9p2*ss[38];
  root_3 += 1680*c1p3*c5p2*c7p2*c9p1*ss[36];
  root_3 += 1120*c1p3*c5p3*c7p1*c9p1*ss[34];
  root_3 += 840*c1p4*c3p1*c5p1*c7p2*ss[26];
  root_3 += 840*c1p4*c3p1*c5p1*c9p2*ss[30];
  root_3 += 840*c1p4*c3p1*c5p2*c7p1*ss[24];
  root_3 += 840*c1p4*c3p1*c5p2*c9p1*ss[26];
  root_3 += 840*c1p4*c3p1*c7p1*c9p2*ss[32];
  root_3 += 840*c1p4*c3p1*c7p2*c9p1*ss[30];
  root_3 += 840*c1p4*c3p2*c5p1*c7p1*ss[22];
  root_3 += 840*c1p4*c3p2*c5p1*c9p1*ss[24];
  root_3 += 840*c1p4*c3p2*c7p1*c9p1*ss[26];
  root_3 += 840*c1p4*c5p1*c7p1*c9p2*ss[34];
  root_3 += 840*c1p4*c5p1*c7p2*c9p1*ss[32];
  root_3 += 840*c1p4*c5p2*c7p1*c9p1*ss[30];
  root_3 += 336*c1p5*c3p1*c5p1*c7p1*ss[20];
  root_3 += 336*c1p5*c3p1*c5p1*c9p1*ss[22];
  root_3 += 336*c1p5*c3p1*c7p1*c9p1*ss[24];
  root_3 += 336*c1p5*c5p1*c7p1*c9p1*ss[26];
  root_3 += 336*c3p1*c5p1*c7p1*c9p5*ss[60];
  root_3 += 840*c3p1*c5p1*c7p2*c9p4*ss[58];
  root_3 += 1120*c3p1*c5p1*c7p3*c9p3*ss[56];
  root_3 += 840*c3p1*c5p1*c7p4*c9p2*ss[54];
  root_3 += 336*c3p1*c5p1*c7p5*c9p1*ss[52];
  root_3 += 840*c3p1*c5p2*c7p1*c9p4*ss[56];
  root_3 += 1680*c3p1*c5p2*c7p2*c9p3*ss[54];
  root_3 += 1680*c3p1*c5p2*c7p3*c9p2*ss[52];
  root_3 += 840*c3p1*c5p2*c7p4*c9p1*ss[50];
  root_3 += 1120*c3p1*c5p3*c7p1*c9p3*ss[52];
  root_3 += 1680*c3p1*c5p3*c7p2*c9p2*ss[50];
  root_3 += 1120*c3p1*c5p3*c7p3*c9p1*ss[48];
  root_3 += 840*c3p1*c5p4*c7p1*c9p2*ss[48];
  root_3 += 840*c3p1*c5p4*c7p2*c9p1*ss[46];
  root_3 += 336*c3p1*c5p5*c7p1*c9p1*ss[44];
  root_3 += 840*c3p2*c5p1*c7p1*c9p4*ss[54];
  root_3 += 1680*c3p2*c5p1*c7p2*c9p3*ss[52];
  root_3 += 1680*c3p2*c5p1*c7p3*c9p2*ss[50];
  root_3 += 840*c3p2*c5p1*c7p4*c9p1*ss[48];
  root_3 += 1680*c3p2*c5p2*c7p1*c9p3*ss[50];
  root_3 += 2520*c3p2*c5p2*c7p2*c9p2*ss[48];
  root_3 += 1680*c3p2*c5p2*c7p3*c9p1*ss[46];
  root_3 += 1680*c3p2*c5p3*c7p1*c9p2*ss[46];
  root_3 += 1680*c3p2*c5p3*c7p2*c9p1*ss[44];
  root_3 += 840*c3p2*c5p4*c7p1*c9p1*ss[42];
  root_3 += 1120*c3p3*c5p1*c7p1*c9p3*ss[48];
  root_3 += 1680*c3p3*c5p1*c7p2*c9p2*ss[46];
  root_3 += 1120*c3p3*c5p1*c7p3*c9p1*ss[44];
  root_3 += 1680*c3p3*c5p2*c7p1*c9p2*ss[44];
  root_3 += 1680*c3p3*c5p2*c7p2*c9p1*ss[42];
  root_3 += 1120*c3p3*c5p3*c7p1*c9p1*ss[40];
  root_3 += 840*c3p4*c5p1*c7p1*c9p2*ss[42];
  root_3 += 840*c3p4*c5p1*c7p2*c9p1*ss[40];
  root_3 += 840*c3p4*c5p2*c7p1*c9p1*ss[38];
  root_3 += 336*c3p5*c5p1*c7p1*c9p1*ss[36];
  root_3 += 56*c1p1*c3p1*c5p6*ss[34];
  root_3 += 56*c1p1*c3p1*c7p6*ss[46];
  root_3 += 56*c1p1*c3p1*c9p6*ss[58];
  root_3 += 168*c1p1*c3p2*c5p5*ss[32];
  root_3 += 168*c1p1*c3p2*c7p5*ss[42];
  root_3 += 168*c1p1*c3p2*c9p5*ss[52];
  root_3 += 280*c1p1*c3p3*c5p4*ss[30];
  root_3 += 280*c1p1*c3p3*c7p4*ss[38];
  root_3 += 280*c1p1*c3p3*c9p4*ss[46];
  root_3 += 280*c1p1*c3p4*c5p3*ss[28];
  root_3 += 280*c1p1*c3p4*c7p3*ss[34];
  root_3 += 280*c1p1*c3p4*c9p3*ss[40];
  root_3 += 168*c1p1*c3p5*c5p2*ss[26];
  root_3 += 168*c1p1*c3p5*c7p2*ss[30];
  root_3 += 168*c1p1*c3p5*c9p2*ss[34];
  root_3 += 56*c1p1*c3p6*c5p1*ss[24];
  root_3 += 56*c1p1*c3p6*c7p1*ss[26];
  root_3 += 56*c1p1*c3p6*c9p1*ss[28];
  root_3 += 56*c1p1*c5p1*c7p6*ss[48];
  root_3 += 56*c1p1*c5p1*c9p6*ss[60];
  root_3 += 168*c1p1*c5p2*c7p5*ss[46];
  root_3 += 168*c1p1*c5p2*c9p5*ss[56];
  root_3 += 280*c1p1*c5p3*c7p4*ss[44];
  root_3 += 280*c1p1*c5p3*c9p4*ss[52];
  root_3 += 280*c1p1*c5p4*c7p3*ss[42];
  root_3 += 280*c1p1*c5p4*c9p3*ss[48];
  root_3 += 168*c1p1*c5p5*c7p2*ss[40];
  root_3 += 168*c1p1*c5p5*c9p2*ss[44];
  root_3 += 56*c1p1*c5p6*c7p1*ss[38];
  root_3 += 56*c1p1*c5p6*c9p1*ss[40];
  root_3 += 56*c1p1*c7p1*c9p6*ss[62];
  root_3 += 168*c1p1*c7p2*c9p5*ss[60];
  root_3 += 280*c1p1*c7p3*c9p4*ss[58];
  root_3 += 280*c1p1*c7p4*c9p3*ss[56];
  root_3 += 168*c1p1*c7p5*c9p2*ss[54];
  root_3 += 56*c1p1*c7p6*c9p1*ss[52];
  root_3 += 168*c1p2*c3p1*c5p5*ss[30];
  root_3 += 168*c1p2*c3p1*c7p5*ss[40];
  root_3 += 168*c1p2*c3p1*c9p5*ss[50];
  root_3 += 420*c1p2*c3p2*c5p4*ss[28];
  root_3 += 420*c1p2*c3p2*c7p4*ss[36];
  root_3 += 420*c1p2*c3p2*c9p4*ss[44];
  root_3 += 560*c1p2*c3p3*c5p3*ss[26];
  root_3 += 560*c1p2*c3p3*c7p3*ss[32];
  root_3 += 560*c1p2*c3p3*c9p3*ss[38];
  root_3 += 420*c1p2*c3p4*c5p2*ss[24];
  root_3 += 420*c1p2*c3p4*c7p2*ss[28];
  root_3 += 420*c1p2*c3p4*c9p2*ss[32];
  root_3 += 168*c1p2*c3p5*c5p1*ss[22];
  root_3 += 168*c1p2*c3p5*c7p1*ss[24];
  root_3 += 168*c1p2*c3p5*c9p1*ss[26];
  root_3 += 168*c1p2*c5p1*c7p5*ss[42];
  root_3 += 168*c1p2*c5p1*c9p5*ss[52];
  root_3 += 420*c1p2*c5p2*c7p4*ss[40];
  root_3 += 420*c1p2*c5p2*c9p4*ss[48];
  root_3 += 560*c1p2*c5p3*c7p3*ss[38];
  root_3 += 560*c1p2*c5p3*c9p3*ss[44];
  root_3 += 420*c1p2*c5p4*c7p2*ss[36];
  root_3 += 420*c1p2*c5p4*c9p2*ss[40];
  root_3 += 168*c1p2*c5p5*c7p1*ss[34];
  root_3 += 168*c1p2*c5p5*c9p1*ss[36];
  root_3 += 168*c1p2*c7p1*c9p5*ss[54];
  root_3 += 420*c1p2*c7p2*c9p4*ss[52];
  root_3 += 560*c1p2*c7p3*c9p3*ss[50];
  root_3 += 420*c1p2*c7p4*c9p2*ss[48];
  root_3 += 168*c1p2*c7p5*c9p1*ss[46];
  root_3 += 280*c1p3*c3p1*c5p4*ss[26];
  root_3 += 280*c1p3*c3p1*c7p4*ss[34];
  root_3 += 280*c1p3*c3p1*c9p4*ss[42];
  root_3 += 560*c1p3*c3p2*c5p3*ss[24];
  root_3 += 560*c1p3*c3p2*c7p3*ss[30];
  root_3 += 560*c1p3*c3p2*c9p3*ss[36];
  root_3 += 560*c1p3*c3p3*c5p2*ss[22];
  root_3 += 560*c1p3*c3p3*c7p2*ss[26];
  root_3 += 560*c1p3*c3p3*c9p2*ss[30];
  root_3 += 280*c1p3*c3p4*c5p1*ss[20];
  root_3 += 280*c1p3*c3p4*c7p1*ss[22];
  root_3 += 280*c1p3*c3p4*c9p1*ss[24];
  root_3 += 280*c1p3*c5p1*c7p4*ss[36];
  root_3 += 280*c1p3*c5p1*c9p4*ss[44];
  root_3 += 560*c1p3*c5p2*c7p3*ss[34];
  root_3 += 560*c1p3*c5p2*c9p3*ss[40];
  root_3 += 560*c1p3*c5p3*c7p2*ss[32];
  root_3 += 560*c1p3*c5p3*c9p2*ss[36];
  root_3 += 280*c1p3*c5p4*c7p1*ss[30];
  root_3 += 280*c1p3*c5p4*c9p1*ss[32];
  root_3 += 280*c1p3*c7p1*c9p4*ss[46];
  root_3 += 560*c1p3*c7p2*c9p3*ss[44];
  root_3 += 560*c1p3*c7p3*c9p2*ss[42];
  root_3 += 280*c1p3*c7p4*c9p1*ss[40];
  root_3 += 280*c1p4*c3p1*c5p3*ss[22];
  root_3 += 280*c1p4*c3p1*c7p3*ss[28];
  root_3 += 280*c1p4*c3p1*c9p3*ss[34];
  root_3 += 420*c1p4*c3p2*c5p2*ss[20];
  root_3 += 420*c1p4*c3p2*c7p2*ss[24];
  root_3 += 420*c1p4*c3p2*c9p2*ss[28];
  root_3 += 280*c1p4*c3p3*c5p1*ss[18];
  root_3 += 280*c1p4*c3p3*c7p1*ss[20];
  root_3 += 280*c1p4*c3p3*c9p1*ss[22];
  root_3 += 280*c1p4*c5p1*c7p3*ss[30];
  root_3 += 280*c1p4*c5p1*c9p3*ss[36];
  root_3 += 420*c1p4*c5p2*c7p2*ss[28];
  root_3 += 420*c1p4*c5p2*c9p2*ss[32];
  root_3 += 280*c1p4*c5p3*c7p1*ss[26];
  root_3 += 280*c1p4*c5p3*c9p1*ss[28];
  root_3 += 280*c1p4*c7p1*c9p3*ss[38];
  root_3 += 420*c1p4*c7p2*c9p2*ss[36];
  root_3 += 280*c1p4*c7p3*c9p1*ss[34];
  root_3 += 168*c1p5*c3p1*c5p2*ss[18];
  root_3 += 168*c1p5*c3p1*c7p2*ss[22];
  root_3 += 168*c1p5*c3p1*c9p2*ss[26];
  root_3 += 168*c1p5*c3p2*c5p1*ss[16];
  root_3 += 168*c1p5*c3p2*c7p1*ss[18];
  root_3 += 168*c1p5*c3p2*c9p1*ss[20];
  root_3 += 168*c1p5*c5p1*c7p2*ss[24];
  root_3 += 168*c1p5*c5p1*c9p2*ss[28];
  root_3 += 168*c1p5*c5p2*c7p1*ss[22];
  root_3 += 168*c1p5*c5p2*c9p1*ss[24];
  root_3 += 168*c1p5*c7p1*c9p2*ss[30];
  root_3 += 168*c1p5*c7p2*c9p1*ss[28];
  root_3 += 56*c1p6*c3p1*c5p1*ss[14];
  root_3 += 56*c1p6*c3p1*c7p1*ss[16];
  root_3 += 56*c1p6*c3p1*c9p1*ss[18];
  root_3 += 56*c1p6*c5p1*c7p1*ss[18];
  root_3 += 56*c1p6*c5p1*c9p1*ss[20];
  root_3 += 56*c1p6*c7p1*c9p1*ss[22];
  root_3 += 56*c3p1*c5p1*c7p6*ss[50];
  root_3 += 56*c3p1*c5p1*c9p6*ss[62];
  root_3 += 168*c3p1*c5p2*c7p5*ss[48];
  root_3 += 168*c3p1*c5p2*c9p5*ss[58];
  root_3 += 280*c3p1*c5p3*c7p4*ss[46];
  root_3 += 280*c3p1*c5p3*c9p4*ss[54];
  root_3 += 280*c3p1*c5p4*c7p3*ss[44];
  root_3 += 280*c3p1*c5p4*c9p3*ss[50];
  root_3 += 168*c3p1*c5p5*c7p2*ss[42];
  root_3 += 168*c3p1*c5p5*c9p2*ss[46];
  root_3 += 56*c3p1*c5p6*c7p1*ss[40];
  root_3 += 56*c3p1*c5p6*c9p1*ss[42];
  root_3 += 56*c3p1*c7p1*c9p6*ss[64];
  root_3 += 168*c3p1*c7p2*c9p5*ss[62];
  root_3 += 280*c3p1*c7p3*c9p4*ss[60];
  root_3 += 280*c3p1*c7p4*c9p3*ss[58];
  root_3 += 168*c3p1*c7p5*c9p2*ss[56];
  root_3 += 56*c3p1*c7p6*c9p1*ss[54];
  root_3 += 168*c3p2*c5p1*c7p5*ss[46];
  root_3 += 168*c3p2*c5p1*c9p5*ss[56];
  root_3 += 420*c3p2*c5p2*c7p4*ss[44];
  root_3 += 420*c3p2*c5p2*c9p4*ss[52];
  root_3 += 560*c3p2*c5p3*c7p3*ss[42];
  root_3 += 560*c3p2*c5p3*c9p3*ss[48];
  root_3 += 420*c3p2*c5p4*c7p2*ss[40];
  root_3 += 420*c3p2*c5p4*c9p2*ss[44];
  root_3 += 168*c3p2*c5p5*c7p1*ss[38];
  root_3 += 168*c3p2*c5p5*c9p1*ss[40];
  root_3 += 168*c3p2*c7p1*c9p5*ss[58];
  root_3 += 420*c3p2*c7p2*c9p4*ss[56];
  root_3 += 560*c3p2*c7p3*c9p3*ss[54];
  root_3 += 420*c3p2*c7p4*c9p2*ss[52];
  root_3 += 168*c3p2*c7p5*c9p1*ss[50];
  root_3 += 280*c3p3*c5p1*c7p4*ss[42];
  root_3 += 280*c3p3*c5p1*c9p4*ss[50];
  root_3 += 560*c3p3*c5p2*c7p3*ss[40];
  root_3 += 560*c3p3*c5p2*c9p3*ss[46];
  root_3 += 560*c3p3*c5p3*c7p2*ss[38];
  root_3 += 560*c3p3*c5p3*c9p2*ss[42];
  root_3 += 280*c3p3*c5p4*c7p1*ss[36];
  root_3 += 280*c3p3*c5p4*c9p1*ss[38];
  root_3 += 280*c3p3*c7p1*c9p4*ss[52];
  root_3 += 560*c3p3*c7p2*c9p3*ss[50];
  root_3 += 560*c3p3*c7p3*c9p2*ss[48];
  root_3 += 280*c3p3*c7p4*c9p1*ss[46];
  root_3 += 280*c3p4*c5p1*c7p3*ss[38];
  root_3 += 280*c3p4*c5p1*c9p3*ss[44];
  root_3 += 420*c3p4*c5p2*c7p2*ss[36];
  root_3 += 420*c3p4*c5p2*c9p2*ss[40];
  root_3 += 280*c3p4*c5p3*c7p1*ss[34];
  root_3 += 280*c3p4*c5p3*c9p1*ss[36];
  root_3 += 280*c3p4*c7p1*c9p3*ss[46];
  root_3 += 420*c3p4*c7p2*c9p2*ss[44];
  root_3 += 280*c3p4*c7p3*c9p1*ss[42];
  root_3 += 168*c3p5*c5p1*c7p2*ss[34];
  root_3 += 168*c3p5*c5p1*c9p2*ss[38];
  root_3 += 168*c3p5*c5p2*c7p1*ss[32];
  root_3 += 168*c3p5*c5p2*c9p1*ss[34];
  root_3 += 168*c3p5*c7p1*c9p2*ss[40];
  root_3 += 168*c3p5*c7p2*c9p1*ss[38];
  root_3 += 56*c3p6*c5p1*c7p1*ss[30];
  root_3 += 56*c3p6*c5p1*c9p1*ss[32];
  root_3 += 56*c3p6*c7p1*c9p1*ss[34];
  root_3 += 56*c5p1*c7p1*c9p6*ss[66];
  root_3 += 168*c5p1*c7p2*c9p5*ss[64];
  root_3 += 280*c5p1*c7p3*c9p4*ss[62];
  root_3 += 280*c5p1*c7p4*c9p3*ss[60];
  root_3 += 168*c5p1*c7p5*c9p2*ss[58];
  root_3 += 56*c5p1*c7p6*c9p1*ss[56];
  root_3 += 168*c5p2*c7p1*c9p5*ss[62];
  root_3 += 420*c5p2*c7p2*c9p4*ss[60];
  root_3 += 560*c5p2*c7p3*c9p3*ss[58];
  root_3 += 420*c5p2*c7p4*c9p2*ss[56];
  root_3 += 168*c5p2*c7p5*c9p1*ss[54];
  root_3 += 280*c5p3*c7p1*c9p4*ss[58];
  root_3 += 560*c5p3*c7p2*c9p3*ss[56];
  root_3 += 560*c5p3*c7p3*c9p2*ss[54];
  root_3 += 280*c5p3*c7p4*c9p1*ss[52];
  root_3 += 280*c5p4*c7p1*c9p3*ss[54];
  root_3 += 420*c5p4*c7p2*c9p2*ss[52];
  root_3 += 280*c5p4*c7p3*c9p1*ss[50];
  root_3 += 168*c5p5*c7p1*c9p2*ss[50];
  root_3 += 168*c5p5*c7p2*c9p1*ss[48];
  root_3 += 56*c5p6*c7p1*c9p1*ss[46];
  root_3 += 8*c1p1*c3p7*ss[22];
  root_3 += 8*c1p1*c5p7*ss[36];
  root_3 += 8*c1p1*c7p7*ss[50];
  root_3 += 8*c1p1*c9p7*ss[64];
  root_3 += 28*c1p2*c3p6*ss[20];
  root_3 += 28*c1p2*c5p6*ss[32];
  root_3 += 28*c1p2*c7p6*ss[44];
  root_3 += 28*c1p2*c9p6*ss[56];
  root_3 += 56*c1p3*c3p5*ss[18];
  root_3 += 56*c1p3*c5p5*ss[28];
  root_3 += 56*c1p3*c7p5*ss[38];
  root_3 += 56*c1p3*c9p5*ss[48];
  root_3 += 70*c1p4*c3p4*ss[16];
  root_3 += 70*c1p4*c5p4*ss[24];
  root_3 += 70*c1p4*c7p4*ss[32];
  root_3 += 70*c1p4*c9p4*ss[40];
  root_3 += 56*c1p5*c3p3*ss[14];
  root_3 += 56*c1p5*c5p3*ss[20];
  root_3 += 56*c1p5*c7p3*ss[26];
  root_3 += 56*c1p5*c9p3*ss[32];
  root_3 += 28*c1p6*c3p2*ss[12];
  root_3 += 28*c1p6*c5p2*ss[16];
  root_3 += 28*c1p6*c7p2*ss[20];
  root_3 += 28*c1p6*c9p2*ss[24];
  root_3 += 8*c1p7*c3p1*ss[10];
  root_3 += 8*c1p7*c5p1*ss[12];
  root_3 += 8*c1p7*c7p1*ss[14];
  root_3 += 8*c1p7*c9p1*ss[16];
  root_3 += 8*c3p1*c5p7*ss[38];
  root_3 += 8*c3p1*c7p7*ss[52];
  root_3 += 8*c3p1*c9p7*ss[66];
  root_3 += 28*c3p2*c5p6*ss[36];
  root_3 += 28*c3p2*c7p6*ss[48];
  root_3 += 28*c3p2*c9p6*ss[60];
  root_3 += 56*c3p3*c5p5*ss[34];
  root_3 += 56*c3p3*c7p5*ss[44];
  root_3 += 56*c3p3*c9p5*ss[54];
  root_3 += 70*c3p4*c5p4*ss[32];
  root_3 += 70*c3p4*c7p4*ss[40];
  root_3 += 70*c3p4*c9p4*ss[48];
  root_3 += 56*c3p5*c5p3*ss[30];
  root_3 += 56*c3p5*c7p3*ss[36];
  root_3 += 56*c3p5*c9p3*ss[42];
  root_3 += 28*c3p6*c5p2*ss[28];
  root_3 += 28*c3p6*c7p2*ss[32];
  root_3 += 28*c3p6*c9p2*ss[36];
  root_3 += 8*c3p7*c5p1*ss[26];
  root_3 += 8*c3p7*c7p1*ss[28];
  root_3 += 8*c3p7*c9p1*ss[30];
  root_3 += 8*c5p1*c7p7*ss[54];
  root_3 += 8*c5p1*c9p7*ss[68];
  root_3 += 28*c5p2*c7p6*ss[52];
  root_3 += 28*c5p2*c9p6*ss[64];
  root_3 += 56*c5p3*c7p5*ss[50];
  root_3 += 56*c5p3*c9p5*ss[60];
  root_3 += 70*c5p4*c7p4*ss[48];
  root_3 += 70*c5p4*c9p4*ss[56];
  root_3 += 56*c5p5*c7p3*ss[46];
  root_3 += 56*c5p5*c9p3*ss[52];
  root_3 += 28*c5p6*c7p2*ss[44];
  root_3 += 28*c5p6*c9p2*ss[48];
  root_3 += 8*c5p7*c7p1*ss[42];
  root_3 += 8*c5p7*c9p1*ss[44];
  root_3 += 8*c7p1*c9p7*ss[70];
  root_3 += 28*c7p2*c9p6*ss[68];
  root_3 += 56*c7p3*c9p5*ss[66];
  root_3 += 70*c7p4*c9p4*ss[64];
  root_3 += 56*c7p5*c9p3*ss[62];
  root_3 += 28*c7p6*c9p2*ss[60];
  root_3 += 8*c7p7*c9p1*ss[58];
  root_3 += c1p8*ss[8];
  root_3 += c3p8*ss[24];
  root_3 += c5p8*ss[40];
  root_3 += c7p8*ss[56];
  root_3 += c9p8*ss[72];
  root_3 += -tmom[8];
  T c1p9 = c1p8*c1p1;
  T c1p10 = c1p9*c1p1;
  T c3p9 = c3p8*c3p1;
  T c3p10 = c3p9*c3p1;
  T c5p9 = c5p8*c5p1;
  T c5p10 = c5p9*c5p1;
  T c7p9 = c7p8*c7p1;
  T c7p10 = c7p9*c7p1;
  T c9p9 = c9p8*c9p1;
  T c9p10 = c9p9*c9p1;
  TACC root_4 = convert<TACC>(5040*c1p1*c3p1*c5p1*c7p1*c9p6*ss[70]);
  root_4 += 15120*c1p1*c3p1*c5p1*c7p2*c9p5*ss[68];
  root_4 += 25200*c1p1*c3p1*c5p1*c7p3*c9p4*ss[66];
  root_4 += 25200*c1p1*c3p1*c5p1*c7p4*c9p3*ss[64];
  root_4 += 15120*c1p1*c3p1*c5p1*c7p5*c9p2*ss[62];
  root_4 += 5040*c1p1*c3p1*c5p1*c7p6*c9p1*ss[60];
  root_4 += 15120*c1p1*c3p1*c5p2*c7p1*c9p5*ss[66];
  root_4 += 37800*c1p1*c3p1*c5p2*c7p2*c9p4*ss[64];
  root_4 += 50400*c1p1*c3p1*c5p2*c7p3*c9p3*ss[62];
  root_4 += 37800*c1p1*c3p1*c5p2*c7p4*c9p2*ss[60];
  root_4 += 15120*c1p1*c3p1*c5p2*c7p5*c9p1*ss[58];
  root_4 += 25200*c1p1*c3p1*c5p3*c7p1*c9p4*ss[62];
  root_4 += 50400*c1p1*c3p1*c5p3*c7p2*c9p3*ss[60];
  root_4 += 50400*c1p1*c3p1*c5p3*c7p3*c9p2*ss[58];
  root_4 += 25200*c1p1*c3p1*c5p3*c7p4*c9p1*ss[56];
  root_4 += 25200*c1p1*c3p1*c5p4*c7p1*c9p3*ss[58];
  root_4 += 37800*c1p1*c3p1*c5p4*c7p2*c9p2*ss[56];
  root_4 += 25200*c1p1*c3p1*c5p4*c7p3*c9p1*ss[54];
  root_4 += 15120*c1p1*c3p1*c5p5*c7p1*c9p2*ss[54];
  root_4 += 15120*c1p1*c3p1*c5p5*c7p2*c9p1*ss[52];
  root_4 += 5040*c1p1*c3p1*c5p6*c7p1*c9p1*ss[50];
  root_4 += 15120*c1p1*c3p2*c5p1*c7p1*c9p5*ss[64];
  root_4 += 37800*c1p1*c3p2*c5p1*c7p2*c9p4*ss[62];
  root_4 += 50400*c1p1*c3p2*c5p1*c7p3*c9p3*ss[60];
  root_4 += 37800*c1p1*c3p2*c5p1*c7p4*c9p2*ss[58];
  root_4 += 15120*c1p1*c3p2*c5p1*c7p5*c9p1*ss[56];
  root_4 += 37800*c1p1*c3p2*c5p2*c7p1*c9p4*ss[60];
  root_4 += 75600*c1p1*c3p2*c5p2*c7p2*c9p3*ss[58];
  root_4 += 75600*c1p1*c3p2*c5p2*c7p3*c9p2*ss[56];
  root_4 += 37800*c1p1*c3p2*c5p2*c7p4*c9p1*ss[54];
  root_4 += 50400*c1p1*c3p2*c5p3*c7p1*c9p3*ss[56];
  root_4 += 75600*c1p1*c3p2*c5p3*c7p2*c9p2*ss[54];
  root_4 += 50400*c1p1*c3p2*c5p3*c7p3*c9p1*ss[52];
  root_4 += 37800*c1p1*c3p2*c5p4*c7p1*c9p2*ss[52];
  root_4 += 37800*c1p1*c3p2*c5p4*c7p2*c9p1*ss[50];
  root_4 += 15120*c1p1*c3p2*c5p5*c7p1*c9p1*ss[48];
  root_4 += 25200*c1p1*c3p3*c5p1*c7p1*c9p4*ss[58];
  root_4 += 50400*c1p1*c3p3*c5p1*c7p2*c9p3*ss[56];
  root_4 += 50400*c1p1*c3p3*c5p1*c7p3*c9p2*ss[54];
  root_4 += 25200*c1p1*c3p3*c5p1*c7p4*c9p1*ss[52];
  root_4 += 50400*c1p1*c3p3*c5p2*c7p1*c9p3*ss[54];
  root_4 += 75600*c1p1*c3p3*c5p2*c7p2*c9p2*ss[52];
  root_4 += 50400*c1p1*c3p3*c5p2*c7p3*c9p1*ss[50];
  root_4 += 50400*c1p1*c3p3*c5p3*c7p1*c9p2*ss[50];
  root_4 += 50400*c1p1*c3p3*c5p3*c7p2*c9p1*ss[48];
  root_4 += 25200*c1p1*c3p3*c5p4*c7p1*c9p1*ss[46];
  root_4 += 25200*c1p1*c3p4*c5p1*c7p1*c9p3*ss[52];
  root_4 += 37800*c1p1*c3p4*c5p1*c7p2*c9p2*ss[50];
  root_4 += 25200*c1p1*c3p4*c5p1*c7p3*c9p1*ss[48];
  root_4 += 37800*c1p1*c3p4*c5p2*c7p1*c9p2*ss[48];
  root_4 += 37800*c1p1*c3p4*c5p2*c7p2*c9p1*ss[46];
  root_4 += 25200*c1p1*c3p4*c5p3*c7p1*c9p1*ss[44];
  root_4 += 15120*c1p1*c3p5*c5p1*c7p1*c9p2*ss[46];
  root_4 += 15120*c1p1*c3p5*c5p1*c7p2*c9p1*ss[44];
  root_4 += 15120*c1p1*c3p5*c5p2*c7p1*c9p1*ss[42];
  root_4 += 5040*c1p1*c3p6*c5p1*c7p1*c9p1*ss[40];
  root_4 += 15120*c1p2*c3p1*c5p1*c7p1*c9p5*ss[62];
  root_4 += 37800*c1p2*c3p1*c5p1*c7p2*c9p4*ss[60];
  root_4 += 50400*c1p2*c3p1*c5p1*c7p3*c9p3*ss[58];
  root_4 += 37800*c1p2*c3p1*c5p1*c7p4*c9p2*ss[56];
  root_4 += 15120*c1p2*c3p1*c5p1*c7p5*c9p1*ss[54];
  root_4 += 37800*c1p2*c3p1*c5p2*c7p1*c9p4*ss[58];
  root_4 += 75600*c1p2*c3p1*c5p2*c7p2*c9p3*ss[56];
  root_4 += 75600*c1p2*c3p1*c5p2*c7p3*c9p2*ss[54];
  root_4 += 37800*c1p2*c3p1*c5p2*c7p4*c9p1*ss[52];
  root_4 += 50400*c1p2*c3p1*c5p3*c7p1*c9p3*ss[54];
  root_4 += 75600*c1p2*c3p1*c5p3*c7p2*c9p2*ss[52];
  root_4 += 50400*c1p2*c3p1*c5p3*c7p3*c9p1*ss[50];
  root_4 += 37800*c1p2*c3p1*c5p4*c7p1*c9p2*ss[50];
  root_4 += 37800*c1p2*c3p1*c5p4*c7p2*c9p1*ss[48];
  root_4 += 15120*c1p2*c3p1*c5p5*c7p1*c9p1*ss[46];
  root_4 += 37800*c1p2*c3p2*c5p1*c7p1*c9p4*ss[56];
  root_4 += 75600*c1p2*c3p2*c5p1*c7p2*c9p3*ss[54];
  root_4 += 75600*c1p2*c3p2*c5p1*c7p3*c9p2*ss[52];
  root_4 += 37800*c1p2*c3p2*c5p1*c7p4*c9p1*ss[50];
  root_4 += 75600*c1p2*c3p2*c5p2*c7p1*c9p3*ss[52];
  root_4 += 113400*c1p2*c3p2*c5p2*c7p2*c9p2*ss[50];
  root_4 += 75600*c1p2*c3p2*c5p2*c7p3*c9p1*ss[48];
  root_4 += 75600*c1p2*c3p2*c5p3*c7p1*c9p2*ss[48];
  root_4 += 75600*c1p2*c3p2*c5p3*c7p2*c9p1*ss[46];
  root_4 += 37800*c1p2*c3p2*c5p4*c7p1*c9p1*ss[44];
  root_4 += 50400*c1p2*c3p3*c5p1*c7p1*c9p3*ss[50];
  root_4 += 75600*c1p2*c3p3*c5p1*c7p2*c9p2*ss[48];
  root_4 += 50400*c1p2*c3p3*c5p1*c7p3*c9p1*ss[46];
  root_4 += 75600*c1p2*c3p3*c5p2*c7p1*c9p2*ss[46];
  root_4 += 75600*c1p2*c3p3*c5p2*c7p2*c9p1*ss[44];
  root_4 += 50400*c1p2*c3p3*c5p3*c7p1*c9p1*ss[42];
  root_4 += 37800*c1p2*c3p4*c5p1*c7p1*c9p2*ss[44];
  root_4 += 37800*c1p2*c3p4*c5p1*c7p2*c9p1*ss[42];
  root_4 += 37800*c1p2*c3p4*c5p2*c7p1*c9p1*ss[40];
  root_4 += 15120*c1p2*c3p5*c5p1*c7p1*c9p1*ss[38];
  root_4 += 25200*c1p3*c3p1*c5p1*c7p1*c9p4*ss[54];
  root_4 += 50400*c1p3*c3p1*c5p1*c7p2*c9p3*ss[52];
  root_4 += 50400*c1p3*c3p1*c5p1*c7p3*c9p2*ss[50];
  root_4 += 25200*c1p3*c3p1*c5p1*c7p4*c9p1*ss[48];
  root_4 += 50400*c1p3*c3p1*c5p2*c7p1*c9p3*ss[50];
  root_4 += 75600*c1p3*c3p1*c5p2*c7p2*c9p2*ss[48];
  root_4 += 50400*c1p3*c3p1*c5p2*c7p3*c9p1*ss[46];
  root_4 += 50400*c1p3*c3p1*c5p3*c7p1*c9p2*ss[46];
  root_4 += 50400*c1p3*c3p1*c5p3*c7p2*c9p1*ss[44];
  root_4 += 25200*c1p3*c3p1*c5p4*c7p1*c9p1*ss[42];
  root_4 += 50400*c1p3*c3p2*c5p1*c7p1*c9p3*ss[48];
  root_4 += 75600*c1p3*c3p2*c5p1*c7p2*c9p2*ss[46];
  root_4 += 50400*c1p3*c3p2*c5p1*c7p3*c9p1*ss[44];
  root_4 += 75600*c1p3*c3p2*c5p2*c7p1*c9p2*ss[44];
  root_4 += 75600*c1p3*c3p2*c5p2*c7p2*c9p1*ss[42];
  root_4 += 50400*c1p3*c3p2*c5p3*c7p1*c9p1*ss[40];
  root_4 += 50400*c1p3*c3p3*c5p1*c7p1*c9p2*ss[42];
  root_4 += 50400*c1p3*c3p3*c5p1*c7p2*c9p1*ss[40];
  root_4 += 50400*c1p3*c3p3*c5p2*c7p1*c9p1*ss[38];
  root_4 += 25200*c1p3*c3p4*c5p1*c7p1*c9p1*ss[36];
  root_4 += 25200*c1p4*c3p1*c5p1*c7p1*c9p3*ss[46];
  root_4 += 37800*c1p4*c3p1*c5p1*c7p2*c9p2*ss[44];
  root_4 += 25200*c1p4*c3p1*c5p1*c7p3*c9p1*ss[42];
  root_4 += 37800*c1p4*c3p1*c5p2*c7p1*c9p2*ss[42];
  root_4 += 37800*c1p4*c3p1*c5p2*c7p2*c9p1*ss[40];
  root_4 += 25200*c1p4*c3p1*c5p3*c7p1*c9p1*ss[38];
  root_4 += 37800*c1p4*c3p2*c5p1*c7p1*c9p2*ss[40];
  root_4 += 37800*c1p4*c3p2*c5p1*c7p2*c9p1*ss[38];
  root_4 += 37800*c1p4*c3p2*c5p2*c7p1*c9p1*ss[36];
  root_4 += 25200*c1p4*c3p3*c5p1*c7p1*c9p1*ss[34];
  root_4 += 15120*c1p5*c3p1*c5p1*c7p1*c9p2*ss[38];
  root_4 += 15120*c1p5*c3p1*c5p1*c7p2*c9p1*ss[36];
  root_4 += 15120*c1p5*c3p1*c5p2*c7p1*c9p1*ss[34];
  root_4 += 15120*c1p5*c3p2*c5p1*c7p1*c9p1*ss[32];
  root_4 += 5040*c1p6*c3p1*c5p1*c7p1*c9p1*ss[30];
  root_4 += 720*c1p1*c3p1*c5p1*c7p7*ss[58];
  root_4 += 720*c1p1*c3p1*c5p1*c9p7*ss[72];
  root_4 += 2520*c1p1*c3p1*c5p2*c7p6*ss[56];
  root_4 += 2520*c1p1*c3p1*c5p2*c9p6*ss[68];
  root_4 += 5040*c1p1*c3p1*c5p3*c7p5*ss[54];
  root_4 += 5040*c1p1*c3p1*c5p3*c9p5*ss[64];
  root_4 += 6300*c1p1*c3p1*c5p4*c7p4*ss[52];
  root_4 += 6300*c1p1*c3p1*c5p4*c9p4*ss[60];
  root_4 += 5040*c1p1*c3p1*c5p5*c7p3*ss[50];
  root_4 += 5040*c1p1*c3p1*c5p5*c9p3*ss[56];
  root_4 += 2520*c1p1*c3p1*c5p6*c7p2*ss[48];
  root_4 += 2520*c1p1*c3p1*c5p6*c9p2*ss[52];
  root_4 += 720*c1p1*c3p1*c5p7*c7p1*ss[46];
  root_4 += 720*c1p1*c3p1*c5p7*c9p1*ss[48];
  root_4 += 720*c1p1*c3p1*c7p1*c9p7*ss[74];
  root_4 += 2520*c1p1*c3p1*c7p2*c9p6*ss[72];
  root_4 += 5040*c1p1*c3p1*c7p3*c9p5*ss[70];
  root_4 += 6300*c1p1*c3p1*c7p4*c9p4*ss[68];
  root_4 += 5040*c1p1*c3p1*c7p5*c9p3*ss[66];
  root_4 += 2520*c1p1*c3p1*c7p6*c9p2*ss[64];
  root_4 += 720*c1p1*c3p1*c7p7*c9p1*ss[62];
  root_4 += 2520*c1p1*c3p2*c5p1*c7p6*ss[54];
  root_4 += 2520*c1p1*c3p2*c5p1*c9p6*ss[66];
  root_4 += 7560*c1p1*c3p2*c5p2*c7p5*ss[52];
  root_4 += 7560*c1p1*c3p2*c5p2*c9p5*ss[62];
  root_4 += 12600*c1p1*c3p2*c5p3*c7p4*ss[50];
  root_4 += 12600*c1p1*c3p2*c5p3*c9p4*ss[58];
  root_4 += 12600*c1p1*c3p2*c5p4*c7p3*ss[48];
  root_4 += 12600*c1p1*c3p2*c5p4*c9p3*ss[54];
  root_4 += 7560*c1p1*c3p2*c5p5*c7p2*ss[46];
  root_4 += 7560*c1p1*c3p2*c5p5*c9p2*ss[50];
  root_4 += 2520*c1p1*c3p2*c5p6*c7p1*ss[44];
  root_4 += 2520*c1p1*c3p2*c5p6*c9p1*ss[46];
  root_4 += 2520*c1p1*c3p2*c7p1*c9p6*ss[68];
  root_4 += 7560*c1p1*c3p2*c7p2*c9p5*ss[66];
  root_4 += 12600*c1p1*c3p2*c7p3*c9p4*ss[64];
  root_4 += 12600*c1p1*c3p2*c7p4*c9p3*ss[62];
  root_4 += 7560*c1p1*c3p2*c7p5*c9p2*ss[60];
  root_4 += 2520*c1p1*c3p2*c7p6*c9p1*ss[58];
  root_4 += 5040*c1p1*c3p3*c5p1*c7p5*ss[50];
  root_4 += 5040*c1p1*c3p3*c5p1*c9p5*ss[60];
  root_4 += 12600*c1p1*c3p3*c5p2*c7p4*ss[48];
  root_4 += 12600*c1p1*c3p3*c5p2*c9p4*ss[56];
  root_4 += 16800*c1p1*c3p3*c5p3*c7p3*ss[46];
  root_4 += 16800*c1p1*c3p3*c5p3*c9p3*ss[52];
  root_4 += 12600*c1p1*c3p3*c5p4*c7p2*ss[44];
  root_4 += 12600*c1p1*c3p3*c5p4*c9p2*ss[48];
  root_4 += 5040*c1p1*c3p3*c5p5*c7p1*ss[42];
  root_4 += 5040*c1p1*c3p3*c5p5*c9p1*ss[44];
  root_4 += 5040*c1p1*c3p3*c7p1*c9p5*ss[62];
  root_4 += 12600*c1p1*c3p3*c7p2*c9p4*ss[60];
  root_4 += 16800*c1p1*c3p3*c7p3*c9p3*ss[58];
  root_4 += 12600*c1p1*c3p3*c7p4*c9p2*ss[56];
  root_4 += 5040*c1p1*c3p3*c7p5*c9p1*ss[54];
  root_4 += 6300*c1p1*c3p4*c5p1*c7p4*ss[46];
  root_4 += 6300*c1p1*c3p4*c5p1*c9p4*ss[54];
  root_4 += 12600*c1p1*c3p4*c5p2*c7p3*ss[44];
  root_4 += 12600*c1p1*c3p4*c5p2*c9p3*ss[50];
  root_4 += 12600*c1p1*c3p4*c5p3*c7p2*ss[42];
  root_4 += 12600*c1p1*c3p4*c5p3*c9p2*ss[46];
  root_4 += 6300*c1p1*c3p4*c5p4*c7p1*ss[40];
  root_4 += 6300*c1p1*c3p4*c5p4*c9p1*ss[42];
  root_4 += 6300*c1p1*c3p4*c7p1*c9p4*ss[56];
  root_4 += 12600*c1p1*c3p4*c7p2*c9p3*ss[54];
  root_4 += 12600*c1p1*c3p4*c7p3*c9p2*ss[52];
  root_4 += 6300*c1p1*c3p4*c7p4*c9p1*ss[50];
  root_4 += 5040*c1p1*c3p5*c5p1*c7p3*ss[42];
  root_4 += 5040*c1p1*c3p5*c5p1*c9p3*ss[48];
  root_4 += 7560*c1p1*c3p5*c5p2*c7p2*ss[40];
  root_4 += 7560*c1p1*c3p5*c5p2*c9p2*ss[44];
  root_4 += 5040*c1p1*c3p5*c5p3*c7p1*ss[38];
  root_4 += 5040*c1p1*c3p5*c5p3*c9p1*ss[40];
  root_4 += 5040*c1p1*c3p5*c7p1*c9p3*ss[50];
  root_4 += 7560*c1p1*c3p5*c7p2*c9p2*ss[48];
  root_4 += 5040*c1p1*c3p5*c7p3*c9p1*ss[46];
  root_4 += 2520*c1p1*c3p6*c5p1*c7p2*ss[38];
  root_4 += 2520*c1p1*c3p6*c5p1*c9p2*ss[42];
  root_4 += 2520*c1p1*c3p6*c5p2*c7p1*ss[36];
  root_4 += 2520*c1p1*c3p6*c5p2*c9p1*ss[38];
  root_4 += 2520*c1p1*c3p6*c7p1*c9p2*ss[44];
  root_4 += 2520*c1p1*c3p6*c7p2*c9p1*ss[42];
  root_4 += 720*c1p1*c3p7*c5p1*c7p1*ss[34];
  root_4 += 720*c1p1*c3p7*c5p1*c9p1*ss[36];
  root_4 += 720*c1p1*c3p7*c7p1*c9p1*ss[38];
  root_4 += 720*c1p1*c5p1*c7p1*c9p7*ss[76];
  root_4 += 2520*c1p1*c5p1*c7p2*c9p6*ss[74];
  root_4 += 5040*c1p1*c5p1*c7p3*c9p5*ss[72];
  root_4 += 6300*c1p1*c5p1*c7p4*c9p4*ss[70];
  root_4 += 5040*c1p1*c5p1*c7p5*c9p3*ss[68];
  root_4 += 2520*c1p1*c5p1*c7p6*c9p2*ss[66];
  root_4 += 720*c1p1*c5p1*c7p7*c9p1*ss[64];
  root_4 += 2520*c1p1*c5p2*c7p1*c9p6*ss[72];
  root_4 += 7560*c1p1*c5p2*c7p2*c9p5*ss[70];
  root_4 += 12600*c1p1*c5p2*c7p3*c9p4*ss[68];
  root_4 += 12600*c1p1*c5p2*c7p4*c9p3*ss[66];
  root_4 += 7560*c1p1*c5p2*c7p5*c9p2*ss[64];
  root_4 += 2520*c1p1*c5p2*c7p6*c9p1*ss[62];
  root_4 += 5040*c1p1*c5p3*c7p1*c9p5*ss[68];
  root_4 += 12600*c1p1*c5p3*c7p2*c9p4*ss[66];
  root_4 += 16800*c1p1*c5p3*c7p3*c9p3*ss[64];
  root_4 += 12600*c1p1*c5p3*c7p4*c9p2*ss[62];
  root_4 += 5040*c1p1*c5p3*c7p5*c9p1*ss[60];
  root_4 += 6300*c1p1*c5p4*c7p1*c9p4*ss[64];
  root_4 += 12600*c1p1*c5p4*c7p2*c9p3*ss[62];
  root_4 += 12600*c1p1*c5p4*c7p3*c9p2*ss[60];
  root_4 += 6300*c1p1*c5p4*c7p4*c9p1*ss[58];
  root_4 += 5040*c1p1*c5p5*c7p1*c9p3*ss[60];
  root_4 += 7560*c1p1*c5p5*c7p2*c9p2*ss[58];
  root_4 += 5040*c1p1*c5p5*c7p3*c9p1*ss[56];
  root_4 += 2520*c1p1*c5p6*c7p1*c9p2*ss[56];
  root_4 += 2520*c1p1*c5p6*c7p2*c9p1*ss[54];
  root_4 += 720*c1p1*c5p7*c7p1*c9p1*ss[52];
  root_4 += 2520*c1p2*c3p1*c5p1*c7p6*ss[52];
  root_4 += 2520*c1p2*c3p1*c5p1*c9p6*ss[64];
  root_4 += 7560*c1p2*c3p1*c5p2*c7p5*ss[50];
  root_4 += 7560*c1p2*c3p1*c5p2*c9p5*ss[60];
  root_4 += 12600*c1p2*c3p1*c5p3*c7p4*ss[48];
  root_4 += 12600*c1p2*c3p1*c5p3*c9p4*ss[56];
  root_4 += 12600*c1p2*c3p1*c5p4*c7p3*ss[46];
  root_4 += 12600*c1p2*c3p1*c5p4*c9p3*ss[52];
  root_4 += 7560*c1p2*c3p1*c5p5*c7p2*ss[44];
  root_4 += 7560*c1p2*c3p1*c5p5*c9p2*ss[48];
  root_4 += 2520*c1p2*c3p1*c5p6*c7p1*ss[42];
  root_4 += 2520*c1p2*c3p1*c5p6*c9p1*ss[44];
  root_4 += 2520*c1p2*c3p1*c7p1*c9p6*ss[66];
  root_4 += 7560*c1p2*c3p1*c7p2*c9p5*ss[64];
  root_4 += 12600*c1p2*c3p1*c7p3*c9p4*ss[62];
  root_4 += 12600*c1p2*c3p1*c7p4*c9p3*ss[60];
  root_4 += 7560*c1p2*c3p1*c7p5*c9p2*ss[58];
  root_4 += 2520*c1p2*c3p1*c7p6*c9p1*ss[56];
  root_4 += 7560*c1p2*c3p2*c5p1*c7p5*ss[48];
  root_4 += 7560*c1p2*c3p2*c5p1*c9p5*ss[58];
  root_4 += 18900*c1p2*c3p2*c5p2*c7p4*ss[46];
  root_4 += 18900*c1p2*c3p2*c5p2*c9p4*ss[54];
  root_4 += 25200*c1p2*c3p2*c5p3*c7p3*ss[44];
  root_4 += 25200*c1p2*c3p2*c5p3*c9p3*ss[50];
  root_4 += 18900*c1p2*c3p2*c5p4*c7p2*ss[42];
  root_4 += 18900*c1p2*c3p2*c5p4*c9p2*ss[46];
  root_4 += 7560*c1p2*c3p2*c5p5*c7p1*ss[40];
  root_4 += 7560*c1p2*c3p2*c5p5*c9p1*ss[42];
  root_4 += 7560*c1p2*c3p2*c7p1*c9p5*ss[60];
  root_4 += 18900*c1p2*c3p2*c7p2*c9p4*ss[58];
  root_4 += 25200*c1p2*c3p2*c7p3*c9p3*ss[56];
  root_4 += 18900*c1p2*c3p2*c7p4*c9p2*ss[54];
  root_4 += 7560*c1p2*c3p2*c7p5*c9p1*ss[52];
  root_4 += 12600*c1p2*c3p3*c5p1*c7p4*ss[44];
  root_4 += 12600*c1p2*c3p3*c5p1*c9p4*ss[52];
  root_4 += 25200*c1p2*c3p3*c5p2*c7p3*ss[42];
  root_4 += 25200*c1p2*c3p3*c5p2*c9p3*ss[48];
  root_4 += 25200*c1p2*c3p3*c5p3*c7p2*ss[40];
  root_4 += 25200*c1p2*c3p3*c5p3*c9p2*ss[44];
  root_4 += 12600*c1p2*c3p3*c5p4*c7p1*ss[38];
  root_4 += 12600*c1p2*c3p3*c5p4*c9p1*ss[40];
  root_4 += 12600*c1p2*c3p3*c7p1*c9p4*ss[54];
  root_4 += 25200*c1p2*c3p3*c7p2*c9p3*ss[52];
  root_4 += 25200*c1p2*c3p3*c7p3*c9p2*ss[50];
  root_4 += 12600*c1p2*c3p3*c7p4*c9p1*ss[48];
  root_4 += 12600*c1p2*c3p4*c5p1*c7p3*ss[40];
  root_4 += 12600*c1p2*c3p4*c5p1*c9p3*ss[46];
  root_4 += 18900*c1p2*c3p4*c5p2*c7p2*ss[38];
  root_4 += 18900*c1p2*c3p4*c5p2*c9p2*ss[42];
  root_4 += 12600*c1p2*c3p4*c5p3*c7p1*ss[36];
  root_4 += 12600*c1p2*c3p4*c5p3*c9p1*ss[38];
  root_4 += 12600*c1p2*c3p4*c7p1*c9p3*ss[48];
  root_4 += 18900*c1p2*c3p4*c7p2*c9p2*ss[46];
  root_4 += 12600*c1p2*c3p4*c7p3*c9p1*ss[44];
  root_4 += 7560*c1p2*c3p5*c5p1*c7p2*ss[36];
  root_4 += 7560*c1p2*c3p5*c5p1*c9p2*ss[40];
  root_4 += 7560*c1p2*c3p5*c5p2*c7p1*ss[34];
  root_4 += 7560*c1p2*c3p5*c5p2*c9p1*ss[36];
  root_4 += 7560*c1p2*c3p5*c7p1*c9p2*ss[42];
  root_4 += 7560*c1p2*c3p5*c7p2*c9p1*ss[40];
  root_4 += 2520*c1p2*c3p6*c5p1*c7p1*ss[32];
  root_4 += 2520*c1p2*c3p6*c5p1*c9p1*ss[34];
  root_4 += 2520*c1p2*c3p6*c7p1*c9p1*ss[36];
  root_4 += 2520*c1p2*c5p1*c7p1*c9p6*ss[68];
  root_4 += 7560*c1p2*c5p1*c7p2*c9p5*ss[66];
  root_4 += 12600*c1p2*c5p1*c7p3*c9p4*ss[64];
  root_4 += 12600*c1p2*c5p1*c7p4*c9p3*ss[62];
  root_4 += 7560*c1p2*c5p1*c7p5*c9p2*ss[60];
  root_4 += 2520*c1p2*c5p1*c7p6*c9p1*ss[58];
  root_4 += 7560*c1p2*c5p2*c7p1*c9p5*ss[64];
  root_4 += 18900*c1p2*c5p2*c7p2*c9p4*ss[62];
  root_4 += 25200*c1p2*c5p2*c7p3*c9p3*ss[60];
  root_4 += 18900*c1p2*c5p2*c7p4*c9p2*ss[58];
  root_4 += 7560*c1p2*c5p2*c7p5*c9p1*ss[56];
  root_4 += 12600*c1p2*c5p3*c7p1*c9p4*ss[60];
  root_4 += 25200*c1p2*c5p3*c7p2*c9p3*ss[58];
  root_4 += 25200*c1p2*c5p3*c7p3*c9p2*ss[56];
  root_4 += 12600*c1p2*c5p3*c7p4*c9p1*ss[54];
  root_4 += 12600*c1p2*c5p4*c7p1*c9p3*ss[56];
  root_4 += 18900*c1p2*c5p4*c7p2*c9p2*ss[54];
  root_4 += 12600*c1p2*c5p4*c7p3*c9p1*ss[52];
  root_4 += 7560*c1p2*c5p5*c7p1*c9p2*ss[52];
  root_4 += 7560*c1p2*c5p5*c7p2*c9p1*ss[50];
  root_4 += 2520*c1p2*c5p6*c7p1*c9p1*ss[48];
  root_4 += 5040*c1p3*c3p1*c5p1*c7p5*ss[46];
  root_4 += 5040*c1p3*c3p1*c5p1*c9p5*ss[56];
  root_4 += 12600*c1p3*c3p1*c5p2*c7p4*ss[44];
  root_4 += 12600*c1p3*c3p1*c5p2*c9p4*ss[52];
  root_4 += 16800*c1p3*c3p1*c5p3*c7p3*ss[42];
  root_4 += 16800*c1p3*c3p1*c5p3*c9p3*ss[48];
  root_4 += 12600*c1p3*c3p1*c5p4*c7p2*ss[40];
  root_4 += 12600*c1p3*c3p1*c5p4*c9p2*ss[44];
  root_4 += 5040*c1p3*c3p1*c5p5*c7p1*ss[38];
  root_4 += 5040*c1p3*c3p1*c5p5*c9p1*ss[40];
  root_4 += 5040*c1p3*c3p1*c7p1*c9p5*ss[58];
  root_4 += 12600*c1p3*c3p1*c7p2*c9p4*ss[56];
  root_4 += 16800*c1p3*c3p1*c7p3*c9p3*ss[54];
  root_4 += 12600*c1p3*c3p1*c7p4*c9p2*ss[52];
  root_4 += 5040*c1p3*c3p1*c7p5*c9p1*ss[50];
  root_4 += 12600*c1p3*c3p2*c5p1*c7p4*ss[42];
  root_4 += 12600*c1p3*c3p2*c5p1*c9p4*ss[50];
  root_4 += 25200*c1p3*c3p2*c5p2*c7p3*ss[40];
  root_4 += 25200*c1p3*c3p2*c5p2*c9p3*ss[46];
  root_4 += 25200*c1p3*c3p2*c5p3*c7p2*ss[38];
  root_4 += 25200*c1p3*c3p2*c5p3*c9p2*ss[42];
  root_4 += 12600*c1p3*c3p2*c5p4*c7p1*ss[36];
  root_4 += 12600*c1p3*c3p2*c5p4*c9p1*ss[38];
  root_4 += 12600*c1p3*c3p2*c7p1*c9p4*ss[52];
  root_4 += 25200*c1p3*c3p2*c7p2*c9p3*ss[50];
  root_4 += 25200*c1p3*c3p2*c7p3*c9p2*ss[48];
  root_4 += 12600*c1p3*c3p2*c7p4*c9p1*ss[46];
  root_4 += 16800*c1p3*c3p3*c5p1*c7p3*ss[38];
  root_4 += 16800*c1p3*c3p3*c5p1*c9p3*ss[44];
  root_4 += 25200*c1p3*c3p3*c5p2*c7p2*ss[36];
  root_4 += 25200*c1p3*c3p3*c5p2*c9p2*ss[40];
  root_4 += 16800*c1p3*c3p3*c5p3*c7p1*ss[34];
  root_4 += 16800*c1p3*c3p3*c5p3*c9p1*ss[36];
  root_4 += 16800*c1p3*c3p3*c7p1*c9p3*ss[46];
  root_4 += 25200*c1p3*c3p3*c7p2*c9p2*ss[44];
  root_4 += 16800*c1p3*c3p3*c7p3*c9p1*ss[42];
  root_4 += 12600*c1p3*c3p4*c5p1*c7p2*ss[34];
  root_4 += 12600*c1p3*c3p4*c5p1*c9p2*ss[38];
  root_4 += 12600*c1p3*c3p4*c5p2*c7p1*ss[32];
  root_4 += 12600*c1p3*c3p4*c5p2*c9p1*ss[34];
  root_4 += 12600*c1p3*c3p4*c7p1*c9p2*ss[40];
  root_4 += 12600*c1p3*c3p4*c7p2*c9p1*ss[38];
  root_4 += 5040*c1p3*c3p5*c5p1*c7p1*ss[30];
  root_4 += 5040*c1p3*c3p5*c5p1*c9p1*ss[32];
  root_4 += 5040*c1p3*c3p5*c7p1*c9p1*ss[34];
  root_4 += 5040*c1p3*c5p1*c7p1*c9p5*ss[60];
  root_4 += 12600*c1p3*c5p1*c7p2*c9p4*ss[58];
  root_4 += 16800*c1p3*c5p1*c7p3*c9p3*ss[56];
  root_4 += 12600*c1p3*c5p1*c7p4*c9p2*ss[54];
  root_4 += 5040*c1p3*c5p1*c7p5*c9p1*ss[52];
  root_4 += 12600*c1p3*c5p2*c7p1*c9p4*ss[56];
  root_4 += 25200*c1p3*c5p2*c7p2*c9p3*ss[54];
  root_4 += 25200*c1p3*c5p2*c7p3*c9p2*ss[52];
  root_4 += 12600*c1p3*c5p2*c7p4*c9p1*ss[50];
  root_4 += 16800*c1p3*c5p3*c7p1*c9p3*ss[52];
  root_4 += 25200*c1p3*c5p3*c7p2*c9p2*ss[50];
  root_4 += 16800*c1p3*c5p3*c7p3*c9p1*ss[48];
  root_4 += 12600*c1p3*c5p4*c7p1*c9p2*ss[48];
  root_4 += 12600*c1p3*c5p4*c7p2*c9p1*ss[46];
  root_4 += 5040*c1p3*c5p5*c7p1*c9p1*ss[44];
  root_4 += 6300*c1p4*c3p1*c5p1*c7p4*ss[40];
  root_4 += 6300*c1p4*c3p1*c5p1*c9p4*ss[48];
  root_4 += 12600*c1p4*c3p1*c5p2*c7p3*ss[38];
  root_4 += 12600*c1p4*c3p1*c5p2*c9p3*ss[44];
  root_4 += 12600*c1p4*c3p1*c5p3*c7p2*ss[36];
  root_4 += 12600*c1p4*c3p1*c5p3*c9p2*ss[40];
  root_4 += 6300*c1p4*c3p1*c5p4*c7p1*ss[34];
  root_4 += 6300*c1p4*c3p1*c5p4*c9p1*ss[36];
  root_4 += 6300*c1p4*c3p1*c7p1*c9p4*ss[50];
  root_4 += 12600*c1p4*c3p1*c7p2*c9p3*ss[48];
  root_4 += 12600*c1p4*c3p1*c7p3*c9p2*ss[46];
  root_4 += 6300*c1p4*c3p1*c7p4*c9p1*ss[44];
  root_4 += 12600*c1p4*c3p2*c5p1*c7p3*ss[36];
  root_4 += 12600*c1p4*c3p2*c5p1*c9p3*ss[42];
  root_4 += 18900*c1p4*c3p2*c5p2*c7p2*ss[34];
  root_4 += 18900*c1p4*c3p2*c5p2*c9p2*ss[38];
  root_4 += 12600*c1p4*c3p2*c5p3*c7p1*ss[32];
  root_4 += 12600*c1p4*c3p2*c5p3*c9p1*ss[34];
  root_4 += 12600*c1p4*c3p2*c7p1*c9p3*ss[44];
  root_4 += 18900*c1p4*c3p2*c7p2*c9p2*ss[42];
  root_4 += 12600*c1p4*c3p2*c7p3*c9p1*ss[40];
  root_4 += 12600*c1p4*c3p3*c5p1*c7p2*ss[32];
  root_4 += 12600*c1p4*c3p3*c5p1*c9p2*ss[36];
  root_4 += 12600*c1p4*c3p3*c5p2*c7p1*ss[30];
  root_4 += 12600*c1p4*c3p3*c5p2*c9p1*ss[32];
  root_4 += 12600*c1p4*c3p3*c7p1*c9p2*ss[38];
  root_4 += 12600*c1p4*c3p3*c7p2*c9p1*ss[36];
  root_4 += 6300*c1p4*c3p4*c5p1*c7p1*ss[28];
  root_4 += 6300*c1p4*c3p4*c5p1*c9p1*ss[30];
  root_4 += 6300*c1p4*c3p4*c7p1*c9p1*ss[32];
  root_4 += 6300*c1p4*c5p1*c7p1*c9p4*ss[52];
  root_4 += 12600*c1p4*c5p1*c7p2*c9p3*ss[50];
  root_4 += 12600*c1p4*c5p1*c7p3*c9p2*ss[48];
  root_4 += 6300*c1p4*c5p1*c7p4*c9p1*ss[46];
  root_4 += 12600*c1p4*c5p2*c7p1*c9p3*ss[48];
  root_4 += 18900*c1p4*c5p2*c7p2*c9p2*ss[46];
  root_4 += 12600*c1p4*c5p2*c7p3*c9p1*ss[44];
  root_4 += 12600*c1p4*c5p3*c7p1*c9p2*ss[44];
  root_4 += 12600*c1p4*c5p3*c7p2*c9p1*ss[42];
  root_4 += 6300*c1p4*c5p4*c7p1*c9p1*ss[40];
  root_4 += 5040*c1p5*c3p1*c5p1*c7p3*ss[34];
  root_4 += 5040*c1p5*c3p1*c5p1*c9p3*ss[40];
  root_4 += 7560*c1p5*c3p1*c5p2*c7p2*ss[32];
  root_4 += 7560*c1p5*c3p1*c5p2*c9p2*ss[36];
  root_4 += 5040*c1p5*c3p1*c5p3*c7p1*ss[30];
  root_4 += 5040*c1p5*c3p1*c5p3*c9p1*ss[32];
  root_4 += 5040*c1p5*c3p1*c7p1*c9p3*ss[42];
  root_4 += 7560*c1p5*c3p1*c7p2*c9p2*ss[40];
  root_4 += 5040*c1p5*c3p1*c7p3*c9p1*ss[38];
  root_4 += 7560*c1p5*c3p2*c5p1*c7p2*ss[30];
  root_4 += 7560*c1p5*c3p2*c5p1*c9p2*ss[34];
  root_4 += 7560*c1p5*c3p2*c5p2*c7p1*ss[28];
  root_4 += 7560*c1p5*c3p2*c5p2*c9p1*ss[30];
  root_4 += 7560*c1p5*c3p2*c7p1*c9p2*ss[36];
  root_4 += 7560*c1p5*c3p2*c7p2*c9p1*ss[34];
  root_4 += 5040*c1p5*c3p3*c5p1*c7p1*ss[26];
  root_4 += 5040*c1p5*c3p3*c5p1*c9p1*ss[28];
  root_4 += 5040*c1p5*c3p3*c7p1*c9p1*ss[30];
  root_4 += 5040*c1p5*c5p1*c7p1*c9p3*ss[44];
  root_4 += 7560*c1p5*c5p1*c7p2*c9p2*ss[42];
  root_4 += 5040*c1p5*c5p1*c7p3*c9p1*ss[40];
  root_4 += 7560*c1p5*c5p2*c7p1*c9p2*ss[40];
  root_4 += 7560*c1p5*c5p2*c7p2*c9p1*ss[38];
  root_4 += 5040*c1p5*c5p3*c7p1*c9p1*ss[36];
  root_4 += 2520*c1p6*c3p1*c5p1*c7p2*ss[28];
  root_4 += 2520*c1p6*c3p1*c5p1*c9p2*ss[32];
  root_4 += 2520*c1p6*c3p1*c5p2*c7p1*ss[26];
  root_4 += 2520*c1p6*c3p1*c5p2*c9p1*ss[28];
  root_4 += 2520*c1p6*c3p1*c7p1*c9p2*ss[34];
  root_4 += 2520*c1p6*c3p1*c7p2*c9p1*ss[32];
  root_4 += 2520*c1p6*c3p2*c5p1*c7p1*ss[24];
  root_4 += 2520*c1p6*c3p2*c5p1*c9p1*ss[26];
  root_4 += 2520*c1p6*c3p2*c7p1*c9p1*ss[28];
  root_4 += 2520*c1p6*c5p1*c7p1*c9p2*ss[36];
  root_4 += 2520*c1p6*c5p1*c7p2*c9p1*ss[34];
  root_4 += 2520*c1p6*c5p2*c7p1*c9p1*ss[32];
  root_4 += 720*c1p7*c3p1*c5p1*c7p1*ss[22];
  root_4 += 720*c1p7*c3p1*c5p1*c9p1*ss[24];
  root_4 += 720*c1p7*c3p1*c7p1*c9p1*ss[26];
  root_4 += 720*c1p7*c5p1*c7p1*c9p1*ss[28];
  root_4 += 720*c3p1*c5p1*c7p1*c9p7*ss[78];
  root_4 += 2520*c3p1*c5p1*c7p2*c9p6*ss[76];
  root_4 += 5040*c3p1*c5p1*c7p3*c9p5*ss[74];
  root_4 += 6300*c3p1*c5p1*c7p4*c9p4*ss[72];
  root_4 += 5040*c3p1*c5p1*c7p5*c9p3*ss[70];
  root_4 += 2520*c3p1*c5p1*c7p6*c9p2*ss[68];
  root_4 += 720*c3p1*c5p1*c7p7*c9p1*ss[66];
  root_4 += 2520*c3p1*c5p2*c7p1*c9p6*ss[74];
  root_4 += 7560*c3p1*c5p2*c7p2*c9p5*ss[72];
  root_4 += 12600*c3p1*c5p2*c7p3*c9p4*ss[70];
  root_4 += 12600*c3p1*c5p2*c7p4*c9p3*ss[68];
  root_4 += 7560*c3p1*c5p2*c7p5*c9p2*ss[66];
  root_4 += 2520*c3p1*c5p2*c7p6*c9p1*ss[64];
  root_4 += 5040*c3p1*c5p3*c7p1*c9p5*ss[70];
  root_4 += 12600*c3p1*c5p3*c7p2*c9p4*ss[68];
  root_4 += 16800*c3p1*c5p3*c7p3*c9p3*ss[66];
  root_4 += 12600*c3p1*c5p3*c7p4*c9p2*ss[64];
  root_4 += 5040*c3p1*c5p3*c7p5*c9p1*ss[62];
  root_4 += 6300*c3p1*c5p4*c7p1*c9p4*ss[66];
  root_4 += 12600*c3p1*c5p4*c7p2*c9p3*ss[64];
  root_4 += 12600*c3p1*c5p4*c7p3*c9p2*ss[62];
  root_4 += 6300*c3p1*c5p4*c7p4*c9p1*ss[60];
  root_4 += 5040*c3p1*c5p5*c7p1*c9p3*ss[62];
  root_4 += 7560*c3p1*c5p5*c7p2*c9p2*ss[60];
  root_4 += 5040*c3p1*c5p5*c7p3*c9p1*ss[58];
  root_4 += 2520*c3p1*c5p6*c7p1*c9p2*ss[58];
  root_4 += 2520*c3p1*c5p6*c7p2*c9p1*ss[56];
  root_4 += 720*c3p1*c5p7*c7p1*c9p1*ss[54];
  root_4 += 2520*c3p2*c5p1*c7p1*c9p6*ss[72];
  root_4 += 7560*c3p2*c5p1*c7p2*c9p5*ss[70];
  root_4 += 12600*c3p2*c5p1*c7p3*c9p4*ss[68];
  root_4 += 12600*c3p2*c5p1*c7p4*c9p3*ss[66];
  root_4 += 7560*c3p2*c5p1*c7p5*c9p2*ss[64];
  root_4 += 2520*c3p2*c5p1*c7p6*c9p1*ss[62];
  root_4 += 7560*c3p2*c5p2*c7p1*c9p5*ss[68];
  root_4 += 18900*c3p2*c5p2*c7p2*c9p4*ss[66];
  root_4 += 25200*c3p2*c5p2*c7p3*c9p3*ss[64];
  root_4 += 18900*c3p2*c5p2*c7p4*c9p2*ss[62];
  root_4 += 7560*c3p2*c5p2*c7p5*c9p1*ss[60];
  root_4 += 12600*c3p2*c5p3*c7p1*c9p4*ss[64];
  root_4 += 25200*c3p2*c5p3*c7p2*c9p3*ss[62];
  root_4 += 25200*c3p2*c5p3*c7p3*c9p2*ss[60];
  root_4 += 12600*c3p2*c5p3*c7p4*c9p1*ss[58];
  root_4 += 12600*c3p2*c5p4*c7p1*c9p3*ss[60];
  root_4 += 18900*c3p2*c5p4*c7p2*c9p2*ss[58];
  root_4 += 12600*c3p2*c5p4*c7p3*c9p1*ss[56];
  root_4 += 7560*c3p2*c5p5*c7p1*c9p2*ss[56];
  root_4 += 7560*c3p2*c5p5*c7p2*c9p1*ss[54];
  root_4 += 2520*c3p2*c5p6*c7p1*c9p1*ss[52];
  root_4 += 5040*c3p3*c5p1*c7p1*c9p5*ss[66];
  root_4 += 12600*c3p3*c5p1*c7p2*c9p4*ss[64];
  root_4 += 16800*c3p3*c5p1*c7p3*c9p3*ss[62];
  root_4 += 12600*c3p3*c5p1*c7p4*c9p2*ss[60];
  root_4 += 5040*c3p3*c5p1*c7p5*c9p1*ss[58];
  root_4 += 12600*c3p3*c5p2*c7p1*c9p4*ss[62];
  root_4 += 25200*c3p3*c5p2*c7p2*c9p3*ss[60];
  root_4 += 25200*c3p3*c5p2*c7p3*c9p2*ss[58];
  root_4 += 12600*c3p3*c5p2*c7p4*c9p1*ss[56];
  root_4 += 16800*c3p3*c5p3*c7p1*c9p3*ss[58];
  root_4 += 25200*c3p3*c5p3*c7p2*c9p2*ss[56];
  root_4 += 16800*c3p3*c5p3*c7p3*c9p1*ss[54];
  root_4 += 12600*c3p3*c5p4*c7p1*c9p2*ss[54];
  root_4 += 12600*c3p3*c5p4*c7p2*c9p1*ss[52];
  root_4 += 5040*c3p3*c5p5*c7p1*c9p1*ss[50];
  root_4 += 6300*c3p4*c5p1*c7p1*c9p4*ss[60];
  root_4 += 12600*c3p4*c5p1*c7p2*c9p3*ss[58];
  root_4 += 12600*c3p4*c5p1*c7p3*c9p2*ss[56];
  root_4 += 6300*c3p4*c5p1*c7p4*c9p1*ss[54];
  root_4 += 12600*c3p4*c5p2*c7p1*c9p3*ss[56];
  root_4 += 18900*c3p4*c5p2*c7p2*c9p2*ss[54];
  root_4 += 12600*c3p4*c5p2*c7p3*c9p1*ss[52];
  root_4 += 12600*c3p4*c5p3*c7p1*c9p2*ss[52];
  root_4 += 12600*c3p4*c5p3*c7p2*c9p1*ss[50];
  root_4 += 6300*c3p4*c5p4*c7p1*c9p1*ss[48];
  root_4 += 5040*c3p5*c5p1*c7p1*c9p3*ss[54];
  root_4 += 7560*c3p5*c5p1*c7p2*c9p2*ss[52];
  root_4 += 5040*c3p5*c5p1*c7p3*c9p1*ss[50];
  root_4 += 7560*c3p5*c5p2*c7p1*c9p2*ss[50];
  root_4 += 7560*c3p5*c5p2*c7p2*c9p1*ss[48];
  root_4 += 5040*c3p5*c5p3*c7p1*c9p1*ss[46];
  root_4 += 2520*c3p6*c5p1*c7p1*c9p2*ss[48];
  root_4 += 2520*c3p6*c5p1*c7p2*c9p1*ss[46];
  root_4 += 2520*c3p6*c5p2*c7p1*c9p1*ss[44];
  root_4 += 720*c3p7*c5p1*c7p1*c9p1*ss[42];
  root_4 += 90*c1p1*c3p1*c5p8*ss[44];
  root_4 += 90*c1p1*c3p1*c7p8*ss[60];
  root_4 += 90*c1p1*c3p1*c9p8*ss[76];
  root_4 += 360*c1p1*c3p2*c5p7*ss[42];
  root_4 += 360*c1p1*c3p2*c7p7*ss[56];
  root_4 += 360*c1p1*c3p2*c9p7*ss[70];
  root_4 += 840*c1p1*c3p3*c5p6*ss[40];
  root_4 += 840*c1p1*c3p3*c7p6*ss[52];
  root_4 += 840*c1p1*c3p3*c9p6*ss[64];
  root_4 += 1260*c1p1*c3p4*c5p5*ss[38];
  root_4 += 1260*c1p1*c3p4*c7p5*ss[48];
  root_4 += 1260*c1p1*c3p4*c9p5*ss[58];
  root_4 += 1260*c1p1*c3p5*c5p4*ss[36];
  root_4 += 1260*c1p1*c3p5*c7p4*ss[44];
  root_4 += 1260*c1p1*c3p5*c9p4*ss[52];
  root_4 += 840*c1p1*c3p6*c5p3*ss[34];
  root_4 += 840*c1p1*c3p6*c7p3*ss[40];
  root_4 += 840*c1p1*c3p6*c9p3*ss[46];
  root_4 += 360*c1p1*c3p7*c5p2*ss[32];
  root_4 += 360*c1p1*c3p7*c7p2*ss[36];
  root_4 += 360*c1p1*c3p7*c9p2*ss[40];
  root_4 += 90*c1p1*c3p8*c5p1*ss[30];
  root_4 += 90*c1p1*c3p8*c7p1*ss[32];
  root_4 += 90*c1p1*c3p8*c9p1*ss[34];
  root_4 += 90*c1p1*c5p1*c7p8*ss[62];
  root_4 += 90*c1p1*c5p1*c9p8*ss[78];
  root_4 += 360*c1p1*c5p2*c7p7*ss[60];
  root_4 += 360*c1p1*c5p2*c9p7*ss[74];
  root_4 += 840*c1p1*c5p3*c7p6*ss[58];
  root_4 += 840*c1p1*c5p3*c9p6*ss[70];
  root_4 += 1260*c1p1*c5p4*c7p5*ss[56];
  root_4 += 1260*c1p1*c5p4*c9p5*ss[66];
  root_4 += 1260*c1p1*c5p5*c7p4*ss[54];
  root_4 += 1260*c1p1*c5p5*c9p4*ss[62];
  root_4 += 840*c1p1*c5p6*c7p3*ss[52];
  root_4 += 840*c1p1*c5p6*c9p3*ss[58];
  root_4 += 360*c1p1*c5p7*c7p2*ss[50];
  root_4 += 360*c1p1*c5p7*c9p2*ss[54];
  root_4 += 90*c1p1*c5p8*c7p1*ss[48];
  root_4 += 90*c1p1*c5p8*c9p1*ss[50];
  root_4 += 90*c1p1*c7p1*c9p8*ss[80];
  root_4 += 360*c1p1*c7p2*c9p7*ss[78];
  root_4 += 840*c1p1*c7p3*c9p6*ss[76];
  root_4 += 1260*c1p1*c7p4*c9p5*ss[74];
  root_4 += 1260*c1p1*c7p5*c9p4*ss[72];
  root_4 += 840*c1p1*c7p6*c9p3*ss[70];
  root_4 += 360*c1p1*c7p7*c9p2*ss[68];
  root_4 += 90*c1p1*c7p8*c9p1*ss[66];
  root_4 += 360*c1p2*c3p1*c5p7*ss[40];
  root_4 += 360*c1p2*c3p1*c7p7*ss[54];
  root_4 += 360*c1p2*c3p1*c9p7*ss[68];
  root_4 += 1260*c1p2*c3p2*c5p6*ss[38];
  root_4 += 1260*c1p2*c3p2*c7p6*ss[50];
  root_4 += 1260*c1p2*c3p2*c9p6*ss[62];
  root_4 += 2520*c1p2*c3p3*c5p5*ss[36];
  root_4 += 2520*c1p2*c3p3*c7p5*ss[46];
  root_4 += 2520*c1p2*c3p3*c9p5*ss[56];
  root_4 += 3150*c1p2*c3p4*c5p4*ss[34];
  root_4 += 3150*c1p2*c3p4*c7p4*ss[42];
  root_4 += 3150*c1p2*c3p4*c9p4*ss[50];
  root_4 += 2520*c1p2*c3p5*c5p3*ss[32];
  root_4 += 2520*c1p2*c3p5*c7p3*ss[38];
  root_4 += 2520*c1p2*c3p5*c9p3*ss[44];
  root_4 += 1260*c1p2*c3p6*c5p2*ss[30];
  root_4 += 1260*c1p2*c3p6*c7p2*ss[34];
  root_4 += 1260*c1p2*c3p6*c9p2*ss[38];
  root_4 += 360*c1p2*c3p7*c5p1*ss[28];
  root_4 += 360*c1p2*c3p7*c7p1*ss[30];
  root_4 += 360*c1p2*c3p7*c9p1*ss[32];
  root_4 += 360*c1p2*c5p1*c7p7*ss[56];
  root_4 += 360*c1p2*c5p1*c9p7*ss[70];
  root_4 += 1260*c1p2*c5p2*c7p6*ss[54];
  root_4 += 1260*c1p2*c5p2*c9p6*ss[66];
  root_4 += 2520*c1p2*c5p3*c7p5*ss[52];
  root_4 += 2520*c1p2*c5p3*c9p5*ss[62];
  root_4 += 3150*c1p2*c5p4*c7p4*ss[50];
  root_4 += 3150*c1p2*c5p4*c9p4*ss[58];
  root_4 += 2520*c1p2*c5p5*c7p3*ss[48];
  root_4 += 2520*c1p2*c5p5*c9p3*ss[54];
  root_4 += 1260*c1p2*c5p6*c7p2*ss[46];
  root_4 += 1260*c1p2*c5p6*c9p2*ss[50];
  root_4 += 360*c1p2*c5p7*c7p1*ss[44];
  root_4 += 360*c1p2*c5p7*c9p1*ss[46];
  root_4 += 360*c1p2*c7p1*c9p7*ss[72];
  root_4 += 1260*c1p2*c7p2*c9p6*ss[70];
  root_4 += 2520*c1p2*c7p3*c9p5*ss[68];
  root_4 += 3150*c1p2*c7p4*c9p4*ss[66];
  root_4 += 2520*c1p2*c7p5*c9p3*ss[64];
  root_4 += 1260*c1p2*c7p6*c9p2*ss[62];
  root_4 += 360*c1p2*c7p7*c9p1*ss[60];
  root_4 += 840*c1p3*c3p1*c5p6*ss[36];
  root_4 += 840*c1p3*c3p1*c7p6*ss[48];
  root_4 += 840*c1p3*c3p1*c9p6*ss[60];
  root_4 += 2520*c1p3*c3p2*c5p5*ss[34];
  root_4 += 2520*c1p3*c3p2*c7p5*ss[44];
  root_4 += 2520*c1p3*c3p2*c9p5*ss[54];
  root_4 += 4200*c1p3*c3p3*c5p4*ss[32];
  root_4 += 4200*c1p3*c3p3*c7p4*ss[40];
  root_4 += 4200*c1p3*c3p3*c9p4*ss[48];
  root_4 += 4200*c1p3*c3p4*c5p3*ss[30];
  root_4 += 4200*c1p3*c3p4*c7p3*ss[36];
  root_4 += 4200*c1p3*c3p4*c9p3*ss[42];
  root_4 += 2520*c1p3*c3p5*c5p2*ss[28];
  root_4 += 2520*c1p3*c3p5*c7p2*ss[32];
  root_4 += 2520*c1p3*c3p5*c9p2*ss[36];
  root_4 += 840*c1p3*c3p6*c5p1*ss[26];
  root_4 += 840*c1p3*c3p6*c7p1*ss[28];
  root_4 += 840*c1p3*c3p6*c9p1*ss[30];
  root_4 += 840*c1p3*c5p1*c7p6*ss[50];
  root_4 += 840*c1p3*c5p1*c9p6*ss[62];
  root_4 += 2520*c1p3*c5p2*c7p5*ss[48];
  root_4 += 2520*c1p3*c5p2*c9p5*ss[58];
  root_4 += 4200*c1p3*c5p3*c7p4*ss[46];
  root_4 += 4200*c1p3*c5p3*c9p4*ss[54];
  root_4 += 4200*c1p3*c5p4*c7p3*ss[44];
  root_4 += 4200*c1p3*c5p4*c9p3*ss[50];
  root_4 += 2520*c1p3*c5p5*c7p2*ss[42];
  root_4 += 2520*c1p3*c5p5*c9p2*ss[46];
  root_4 += 840*c1p3*c5p6*c7p1*ss[40];
  root_4 += 840*c1p3*c5p6*c9p1*ss[42];
  root_4 += 840*c1p3*c7p1*c9p6*ss[64];
  root_4 += 2520*c1p3*c7p2*c9p5*ss[62];
  root_4 += 4200*c1p3*c7p3*c9p4*ss[60];
  root_4 += 4200*c1p3*c7p4*c9p3*ss[58];
  root_4 += 2520*c1p3*c7p5*c9p2*ss[56];
  root_4 += 840*c1p3*c7p6*c9p1*ss[54];
  root_4 += 1260*c1p4*c3p1*c5p5*ss[32];
  root_4 += 1260*c1p4*c3p1*c7p5*ss[42];
  root_4 += 1260*c1p4*c3p1*c9p5*ss[52];
  root_4 += 3150*c1p4*c3p2*c5p4*ss[30];
  root_4 += 3150*c1p4*c3p2*c7p4*ss[38];
  root_4 += 3150*c1p4*c3p2*c9p4*ss[46];
  root_4 += 4200*c1p4*c3p3*c5p3*ss[28];
  root_4 += 4200*c1p4*c3p3*c7p3*ss[34];
  root_4 += 4200*c1p4*c3p3*c9p3*ss[40];
  root_4 += 3150*c1p4*c3p4*c5p2*ss[26];
  root_4 += 3150*c1p4*c3p4*c7p2*ss[30];
  root_4 += 3150*c1p4*c3p4*c9p2*ss[34];
  root_4 += 1260*c1p4*c3p5*c5p1*ss[24];
  root_4 += 1260*c1p4*c3p5*c7p1*ss[26];
  root_4 += 1260*c1p4*c3p5*c9p1*ss[28];
  root_4 += 1260*c1p4*c5p1*c7p5*ss[44];
  root_4 += 1260*c1p4*c5p1*c9p5*ss[54];
  root_4 += 3150*c1p4*c5p2*c7p4*ss[42];
  root_4 += 3150*c1p4*c5p2*c9p4*ss[50];
  root_4 += 4200*c1p4*c5p3*c7p3*ss[40];
  root_4 += 4200*c1p4*c5p3*c9p3*ss[46];
  root_4 += 3150*c1p4*c5p4*c7p2*ss[38];
  root_4 += 3150*c1p4*c5p4*c9p2*ss[42];
  root_4 += 1260*c1p4*c5p5*c7p1*ss[36];
  root_4 += 1260*c1p4*c5p5*c9p1*ss[38];
  root_4 += 1260*c1p4*c7p1*c9p5*ss[56];
  root_4 += 3150*c1p4*c7p2*c9p4*ss[54];
  root_4 += 4200*c1p4*c7p3*c9p3*ss[52];
  root_4 += 3150*c1p4*c7p4*c9p2*ss[50];
  root_4 += 1260*c1p4*c7p5*c9p1*ss[48];
  root_4 += 1260*c1p5*c3p1*c5p4*ss[28];
  root_4 += 1260*c1p5*c3p1*c7p4*ss[36];
  root_4 += 1260*c1p5*c3p1*c9p4*ss[44];
  root_4 += 2520*c1p5*c3p2*c5p3*ss[26];
  root_4 += 2520*c1p5*c3p2*c7p3*ss[32];
  root_4 += 2520*c1p5*c3p2*c9p3*ss[38];
  root_4 += 2520*c1p5*c3p3*c5p2*ss[24];
  root_4 += 2520*c1p5*c3p3*c7p2*ss[28];
  root_4 += 2520*c1p5*c3p3*c9p2*ss[32];
  root_4 += 1260*c1p5*c3p4*c5p1*ss[22];
  root_4 += 1260*c1p5*c3p4*c7p1*ss[24];
  root_4 += 1260*c1p5*c3p4*c9p1*ss[26];
  root_4 += 1260*c1p5*c5p1*c7p4*ss[38];
  root_4 += 1260*c1p5*c5p1*c9p4*ss[46];
  root_4 += 2520*c1p5*c5p2*c7p3*ss[36];
  root_4 += 2520*c1p5*c5p2*c9p3*ss[42];
  root_4 += 2520*c1p5*c5p3*c7p2*ss[34];
  root_4 += 2520*c1p5*c5p3*c9p2*ss[38];
  root_4 += 1260*c1p5*c5p4*c7p1*ss[32];
  root_4 += 1260*c1p5*c5p4*c9p1*ss[34];
  root_4 += 1260*c1p5*c7p1*c9p4*ss[48];
  root_4 += 2520*c1p5*c7p2*c9p3*ss[46];
  root_4 += 2520*c1p5*c7p3*c9p2*ss[44];
  root_4 += 1260*c1p5*c7p4*c9p1*ss[42];
  root_4 += 840*c1p6*c3p1*c5p3*ss[24];
  root_4 += 840*c1p6*c3p1*c7p3*ss[30];
  root_4 += 840*c1p6*c3p1*c9p3*ss[36];
  root_4 += 1260*c1p6*c3p2*c5p2*ss[22];
  root_4 += 1260*c1p6*c3p2*c7p2*ss[26];
  root_4 += 1260*c1p6*c3p2*c9p2*ss[30];
  root_4 += 840*c1p6*c3p3*c5p1*ss[20];
  root_4 += 840*c1p6*c3p3*c7p1*ss[22];
  root_4 += 840*c1p6*c3p3*c9p1*ss[24];
  root_4 += 840*c1p6*c5p1*c7p3*ss[32];
  root_4 += 840*c1p6*c5p1*c9p3*ss[38];
  root_4 += 1260*c1p6*c5p2*c7p2*ss[30];
  root_4 += 1260*c1p6*c5p2*c9p2*ss[34];
  root_4 += 840*c1p6*c5p3*c7p1*ss[28];
  root_4 += 840*c1p6*c5p3*c9p1*ss[30];
  root_4 += 840*c1p6*c7p1*c9p3*ss[40];
  root_4 += 1260*c1p6*c7p2*c9p2*ss[38];
  root_4 += 840*c1p6*c7p3*c9p1*ss[36];
  root_4 += 360*c1p7*c3p1*c5p2*ss[20];
  root_4 += 360*c1p7*c3p1*c7p2*ss[24];
  root_4 += 360*c1p7*c3p1*c9p2*ss[28];
  root_4 += 360*c1p7*c3p2*c5p1*ss[18];
  root_4 += 360*c1p7*c3p2*c7p1*ss[20];
  root_4 += 360*c1p7*c3p2*c9p1*ss[22];
  root_4 += 360*c1p7*c5p1*c7p2*ss[26];
  root_4 += 360*c1p7*c5p1*c9p2*ss[30];
  root_4 += 360*c1p7*c5p2*c7p1*ss[24];
  root_4 += 360*c1p7*c5p2*c9p1*ss[26];
  root_4 += 360*c1p7*c7p1*c9p2*ss[32];
  root_4 += 360*c1p7*c7p2*c9p1*ss[30];
  root_4 += 90*c1p8*c3p1*c5p1*ss[16];
  root_4 += 90*c1p8*c3p1*c7p1*ss[18];
  root_4 += 90*c1p8*c3p1*c9p1*ss[20];
  root_4 += 90*c1p8*c5p1*c7p1*ss[20];
  root_4 += 90*c1p8*c5p1*c9p1*ss[22];
  root_4 += 90*c1p8*c7p1*c9p1*ss[24];
  root_4 += 90*c3p1*c5p1*c7p8*ss[64];
  root_4 += 90*c3p1*c5p1*c9p8*ss[80];
  root_4 += 360*c3p1*c5p2*c7p7*ss[62];
  root_4 += 360*c3p1*c5p2*c9p7*ss[76];
  root_4 += 840*c3p1*c5p3*c7p6*ss[60];
  root_4 += 840*c3p1*c5p3*c9p6*ss[72];
  root_4 += 1260*c3p1*c5p4*c7p5*ss[58];
  root_4 += 1260*c3p1*c5p4*c9p5*ss[68];
  root_4 += 1260*c3p1*c5p5*c7p4*ss[56];
  root_4 += 1260*c3p1*c5p5*c9p4*ss[64];
  root_4 += 840*c3p1*c5p6*c7p3*ss[54];
  root_4 += 840*c3p1*c5p6*c9p3*ss[60];
  root_4 += 360*c3p1*c5p7*c7p2*ss[52];
  root_4 += 360*c3p1*c5p7*c9p2*ss[56];
  root_4 += 90*c3p1*c5p8*c7p1*ss[50];
  root_4 += 90*c3p1*c5p8*c9p1*ss[52];
  root_4 += 90*c3p1*c7p1*c9p8*ss[82];
  root_4 += 360*c3p1*c7p2*c9p7*ss[80];
  root_4 += 840*c3p1*c7p3*c9p6*ss[78];
  root_4 += 1260*c3p1*c7p4*c9p5*ss[76];
  root_4 += 1260*c3p1*c7p5*c9p4*ss[74];
  root_4 += 840*c3p1*c7p6*c9p3*ss[72];
  root_4 += 360*c3p1*c7p7*c9p2*ss[70];
  root_4 += 90*c3p1*c7p8*c9p1*ss[68];
  root_4 += 360*c3p2*c5p1*c7p7*ss[60];
  root_4 += 360*c3p2*c5p1*c9p7*ss[74];
  root_4 += 1260*c3p2*c5p2*c7p6*ss[58];
  root_4 += 1260*c3p2*c5p2*c9p6*ss[70];
  root_4 += 2520*c3p2*c5p3*c7p5*ss[56];
  root_4 += 2520*c3p2*c5p3*c9p5*ss[66];
  root_4 += 3150*c3p2*c5p4*c7p4*ss[54];
  root_4 += 3150*c3p2*c5p4*c9p4*ss[62];
  root_4 += 2520*c3p2*c5p5*c7p3*ss[52];
  root_4 += 2520*c3p2*c5p5*c9p3*ss[58];
  root_4 += 1260*c3p2*c5p6*c7p2*ss[50];
  root_4 += 1260*c3p2*c5p6*c9p2*ss[54];
  root_4 += 360*c3p2*c5p7*c7p1*ss[48];
  root_4 += 360*c3p2*c5p7*c9p1*ss[50];
  root_4 += 360*c3p2*c7p1*c9p7*ss[76];
  root_4 += 1260*c3p2*c7p2*c9p6*ss[74];
  root_4 += 2520*c3p2*c7p3*c9p5*ss[72];
  root_4 += 3150*c3p2*c7p4*c9p4*ss[70];
  root_4 += 2520*c3p2*c7p5*c9p3*ss[68];
  root_4 += 1260*c3p2*c7p6*c9p2*ss[66];
  root_4 += 360*c3p2*c7p7*c9p1*ss[64];
  root_4 += 840*c3p3*c5p1*c7p6*ss[56];
  root_4 += 840*c3p3*c5p1*c9p6*ss[68];
  root_4 += 2520*c3p3*c5p2*c7p5*ss[54];
  root_4 += 2520*c3p3*c5p2*c9p5*ss[64];
  root_4 += 4200*c3p3*c5p3*c7p4*ss[52];
  root_4 += 4200*c3p3*c5p3*c9p4*ss[60];
  root_4 += 4200*c3p3*c5p4*c7p3*ss[50];
  root_4 += 4200*c3p3*c5p4*c9p3*ss[56];
  root_4 += 2520*c3p3*c5p5*c7p2*ss[48];
  root_4 += 2520*c3p3*c5p5*c9p2*ss[52];
  root_4 += 840*c3p3*c5p6*c7p1*ss[46];
  root_4 += 840*c3p3*c5p6*c9p1*ss[48];
  root_4 += 840*c3p3*c7p1*c9p6*ss[70];
  root_4 += 2520*c3p3*c7p2*c9p5*ss[68];
  root_4 += 4200*c3p3*c7p3*c9p4*ss[66];
  root_4 += 4200*c3p3*c7p4*c9p3*ss[64];
  root_4 += 2520*c3p3*c7p5*c9p2*ss[62];
  root_4 += 840*c3p3*c7p6*c9p1*ss[60];
  root_4 += 1260*c3p4*c5p1*c7p5*ss[52];
  root_4 += 1260*c3p4*c5p1*c9p5*ss[62];
  root_4 += 3150*c3p4*c5p2*c7p4*ss[50];
  root_4 += 3150*c3p4*c5p2*c9p4*ss[58];
  root_4 += 4200*c3p4*c5p3*c7p3*ss[48];
  root_4 += 4200*c3p4*c5p3*c9p3*ss[54];
  root_4 += 3150*c3p4*c5p4*c7p2*ss[46];
  root_4 += 3150*c3p4*c5p4*c9p2*ss[50];
  root_4 += 1260*c3p4*c5p5*c7p1*ss[44];
  root_4 += 1260*c3p4*c5p5*c9p1*ss[46];
  root_4 += 1260*c3p4*c7p1*c9p5*ss[64];
  root_4 += 3150*c3p4*c7p2*c9p4*ss[62];
  root_4 += 4200*c3p4*c7p3*c9p3*ss[60];
  root_4 += 3150*c3p4*c7p4*c9p2*ss[58];
  root_4 += 1260*c3p4*c7p5*c9p1*ss[56];
  root_4 += 1260*c3p5*c5p1*c7p4*ss[48];
  root_4 += 1260*c3p5*c5p1*c9p4*ss[56];
  root_4 += 2520*c3p5*c5p2*c7p3*ss[46];
  root_4 += 2520*c3p5*c5p2*c9p3*ss[52];
  root_4 += 2520*c3p5*c5p3*c7p2*ss[44];
  root_4 += 2520*c3p5*c5p3*c9p2*ss[48];
  root_4 += 1260*c3p5*c5p4*c7p1*ss[42];
  root_4 += 1260*c3p5*c5p4*c9p1*ss[44];
  root_4 += 1260*c3p5*c7p1*c9p4*ss[58];
  root_4 += 2520*c3p5*c7p2*c9p3*ss[56];
  root_4 += 2520*c3p5*c7p3*c9p2*ss[54];
  root_4 += 1260*c3p5*c7p4*c9p1*ss[52];
  root_4 += 840*c3p6*c5p1*c7p3*ss[44];
  root_4 += 840*c3p6*c5p1*c9p3*ss[50];
  root_4 += 1260*c3p6*c5p2*c7p2*ss[42];
  root_4 += 1260*c3p6*c5p2*c9p2*ss[46];
  root_4 += 840*c3p6*c5p3*c7p1*ss[40];
  root_4 += 840*c3p6*c5p3*c9p1*ss[42];
  root_4 += 840*c3p6*c7p1*c9p3*ss[52];
  root_4 += 1260*c3p6*c7p2*c9p2*ss[50];
  root_4 += 840*c3p6*c7p3*c9p1*ss[48];
  root_4 += 360*c3p7*c5p1*c7p2*ss[40];
  root_4 += 360*c3p7*c5p1*c9p2*ss[44];
  root_4 += 360*c3p7*c5p2*c7p1*ss[38];
  root_4 += 360*c3p7*c5p2*c9p1*ss[40];
  root_4 += 360*c3p7*c7p1*c9p2*ss[46];
  root_4 += 360*c3p7*c7p2*c9p1*ss[44];
  root_4 += 90*c3p8*c5p1*c7p1*ss[36];
  root_4 += 90*c3p8*c5p1*c9p1*ss[38];
  root_4 += 90*c3p8*c7p1*c9p1*ss[40];
  root_4 += 90*c5p1*c7p1*c9p8*ss[84];
  root_4 += 360*c5p1*c7p2*c9p7*ss[82];
  root_4 += 840*c5p1*c7p3*c9p6*ss[80];
  root_4 += 1260*c5p1*c7p4*c9p5*ss[78];
  root_4 += 1260*c5p1*c7p5*c9p4*ss[76];
  root_4 += 840*c5p1*c7p6*c9p3*ss[74];
  root_4 += 360*c5p1*c7p7*c9p2*ss[72];
  root_4 += 90*c5p1*c7p8*c9p1*ss[70];
  root_4 += 360*c5p2*c7p1*c9p7*ss[80];
  root_4 += 1260*c5p2*c7p2*c9p6*ss[78];
  root_4 += 2520*c5p2*c7p3*c9p5*ss[76];
  root_4 += 3150*c5p2*c7p4*c9p4*ss[74];
  root_4 += 2520*c5p2*c7p5*c9p3*ss[72];
  root_4 += 1260*c5p2*c7p6*c9p2*ss[70];
  root_4 += 360*c5p2*c7p7*c9p1*ss[68];
  root_4 += 840*c5p3*c7p1*c9p6*ss[76];
  root_4 += 2520*c5p3*c7p2*c9p5*ss[74];
  root_4 += 4200*c5p3*c7p3*c9p4*ss[72];
  root_4 += 4200*c5p3*c7p4*c9p3*ss[70];
  root_4 += 2520*c5p3*c7p5*c9p2*ss[68];
  root_4 += 840*c5p3*c7p6*c9p1*ss[66];
  root_4 += 1260*c5p4*c7p1*c9p5*ss[72];
  root_4 += 3150*c5p4*c7p2*c9p4*ss[70];
  root_4 += 4200*c5p4*c7p3*c9p3*ss[68];
  root_4 += 3150*c5p4*c7p4*c9p2*ss[66];
  root_4 += 1260*c5p4*c7p5*c9p1*ss[64];
  root_4 += 1260*c5p5*c7p1*c9p4*ss[68];
  root_4 += 2520*c5p5*c7p2*c9p3*ss[66];
  root_4 += 2520*c5p5*c7p3*c9p2*ss[64];
  root_4 += 1260*c5p5*c7p4*c9p1*ss[62];
  root_4 += 840*c5p6*c7p1*c9p3*ss[64];
  root_4 += 1260*c5p6*c7p2*c9p2*ss[62];
  root_4 += 840*c5p6*c7p3*c9p1*ss[60];
  root_4 += 360*c5p7*c7p1*c9p2*ss[60];
  root_4 += 360*c5p7*c7p2*c9p1*ss[58];
  root_4 += 90*c5p8*c7p1*c9p1*ss[56];
  root_4 += 10*c1p1*c3p9*ss[28];
  root_4 += 10*c1p1*c5p9*ss[46];
  root_4 += 10*c1p1*c7p9*ss[64];
  root_4 += 10*c1p1*c9p9*ss[82];
  root_4 += 45*c1p2*c3p8*ss[26];
  root_4 += 45*c1p2*c5p8*ss[42];
  root_4 += 45*c1p2*c7p8*ss[58];
  root_4 += 45*c1p2*c9p8*ss[74];
  root_4 += 120*c1p3*c3p7*ss[24];
  root_4 += 120*c1p3*c5p7*ss[38];
  root_4 += 120*c1p3*c7p7*ss[52];
  root_4 += 120*c1p3*c9p7*ss[66];
  root_4 += 210*c1p4*c3p6*ss[22];
  root_4 += 210*c1p4*c5p6*ss[34];
  root_4 += 210*c1p4*c7p6*ss[46];
  root_4 += 210*c1p4*c9p6*ss[58];
  root_4 += 252*c1p5*c3p5*ss[20];
  root_4 += 252*c1p5*c5p5*ss[30];
  root_4 += 252*c1p5*c7p5*ss[40];
  root_4 += 252*c1p5*c9p5*ss[50];
  root_4 += 210*c1p6*c3p4*ss[18];
  root_4 += 210*c1p6*c5p4*ss[26];
  root_4 += 210*c1p6*c7p4*ss[34];
  root_4 += 210*c1p6*c9p4*ss[42];
  root_4 += 120*c1p7*c3p3*ss[16];
  root_4 += 120*c1p7*c5p3*ss[22];
  root_4 += 120*c1p7*c7p3*ss[28];
  root_4 += 120*c1p7*c9p3*ss[34];
  root_4 += 45*c1p8*c3p2*ss[14];
  root_4 += 45*c1p8*c5p2*ss[18];
  root_4 += 45*c1p8*c7p2*ss[22];
  root_4 += 45*c1p8*c9p2*ss[26];
  root_4 += 10*c1p9*c3p1*ss[12];
  root_4 += 10*c1p9*c5p1*ss[14];
  root_4 += 10*c1p9*c7p1*ss[16];
  root_4 += 10*c1p9*c9p1*ss[18];
  root_4 += 10*c3p1*c5p9*ss[48];
  root_4 += 10*c3p1*c7p9*ss[66];
  root_4 += 10*c3p1*c9p9*ss[84];
  root_4 += 45*c3p2*c5p8*ss[46];
  root_4 += 45*c3p2*c7p8*ss[62];
  root_4 += 45*c3p2*c9p8*ss[78];
  root_4 += 120*c3p3*c5p7*ss[44];
  root_4 += 120*c3p3*c7p7*ss[58];
  root_4 += 120*c3p3*c9p7*ss[72];
  root_4 += 210*c3p4*c5p6*ss[42];
  root_4 += 210*c3p4*c7p6*ss[54];
  root_4 += 210*c3p4*c9p6*ss[66];
  root_4 += 252*c3p5*c5p5*ss[40];
  root_4 += 252*c3p5*c7p5*ss[50];
  root_4 += 252*c3p5*c9p5*ss[60];
  root_4 += 210*c3p6*c5p4*ss[38];
  root_4 += 210*c3p6*c7p4*ss[46];
  root_4 += 210*c3p6*c9p4*ss[54];
  root_4 += 120*c3p7*c5p3*ss[36];
  root_4 += 120*c3p7*c7p3*ss[42];
  root_4 += 120*c3p7*c9p3*ss[48];
  root_4 += 45*c3p8*c5p2*ss[34];
  root_4 += 45*c3p8*c7p2*ss[38];
  root_4 += 45*c3p8*c9p2*ss[42];
  root_4 += 10*c3p9*c5p1*ss[32];
  root_4 += 10*c3p9*c7p1*ss[34];
  root_4 += 10*c3p9*c9p1*ss[36];
  root_4 += 10*c5p1*c7p9*ss[68];
  root_4 += 10*c5p1*c9p9*ss[86];
  root_4 += 45*c5p2*c7p8*ss[66];
  root_4 += 45*c5p2*c9p8*ss[82];
  root_4 += 120*c5p3*c7p7*ss[64];
  root_4 += 120*c5p3*c9p7*ss[78];
  root_4 += 210*c5p4*c7p6*ss[62];
  root_4 += 210*c5p4*c9p6*ss[74];
  root_4 += 252*c5p5*c7p5*ss[60];
  root_4 += 252*c5p5*c9p5*ss[70];
  root_4 += 210*c5p6*c7p4*ss[58];
  root_4 += 210*c5p6*c9p4*ss[66];
  root_4 += 120*c5p7*c7p3*ss[56];
  root_4 += 120*c5p7*c9p3*ss[62];
  root_4 += 45*c5p8*c7p2*ss[54];
  root_4 += 45*c5p8*c9p2*ss[58];
  root_4 += 10*c5p9*c7p1*ss[52];
  root_4 += 10*c5p9*c9p1*ss[54];
  root_4 += 10*c7p1*c9p9*ss[88];
  root_4 += 45*c7p2*c9p8*ss[86];
  root_4 += 120*c7p3*c9p7*ss[84];
  root_4 += 210*c7p4*c9p6*ss[82];
  root_4 += 252*c7p5*c9p5*ss[80];
  root_4 += 210*c7p6*c9p4*ss[78];
  root_4 += 120*c7p7*c9p3*ss[76];
  root_4 += 45*c7p8*c9p2*ss[74];
  root_4 += 10*c7p9*c9p1*ss[72];
  root_4 += c1p10*ss[10];
  root_4 += c3p10*ss[30];
  root_4 += c5p10*ss[50];
  root_4 += c7p10*ss[70];
  root_4 += c9p10*ss[90];
  root_4 += -tmom[10];
  TACC jac_0_0 = convert<TACC>(2*c1p1*ss[2]);
  jac_0_0 += 2*c3p1*ss[4];
  jac_0_0 += 2*c5p1*ss[6];
  jac_0_0 += 2*c7p1*ss[8];
  jac_0_0 += 2*c9p1*ss[10];
  TACC jac_0_1 = convert<TACC>(2*c1p1*ss[4]);
  jac_0_1 += 2*c3p1*ss[6];
  jac_0_1 += 2*c5p1*ss[8];
  jac_0_1 += 2*c7p1*ss[10];
  jac_0_1 += 2*c9p1*ss[12];
  TACC jac_0_2 = convert<TACC>(2*c1p1*ss[6]);
  jac_0_2 += 2*c3p1*ss[8];
  jac_0_2 += 2*c5p1*ss[10];
  jac_0_2 += 2*c7p1*ss[12];
  jac_0_2 += 2*c9p1*ss[14];
  TACC jac_0_3 = convert<TACC>(2*c1p1*ss[8]);
  jac_0_3 += 2*c3p1*ss[10];
  jac_0_3 += 2*c5p1*ss[12];
  jac_0_3 += 2*c7p1*ss[14];
  jac_0_3 += 2*c9p1*ss[16];
  TACC jac_0_4 = convert<TACC>(2*c1p1*ss[10]);
  jac_0_4 += 2*c3p1*ss[12];
  jac_0_4 += 2*c5p1*ss[14];
  jac_0_4 += 2*c7p1*ss[16];
  jac_0_4 += 2*c9p1*ss[18];
  TACC jac_1_0 = convert<TACC>(24*c1p1*c3p1*c5p1*ss[10]);
  jac_1_0 += 24*c1p1*c3p1*c7p1*ss[12];
  jac_1_0 += 24*c1p1*c3p1*c9p1*ss[14];
  jac_1_0 += 24*c1p1*c5p1*c7p1*ss[14];
  jac_1_0 += 24*c1p1*c5p1*c9p1*ss[16];
  jac_1_0 += 24*c1p1*c7p1*c9p1*ss[18];
  jac_1_0 += 24*c3p1*c5p1*c7p1*ss[16];
  jac_1_0 += 24*c3p1*c5p1*c9p1*ss[18];
  jac_1_0 += 24*c3p1*c7p1*c9p1*ss[20];
  jac_1_0 += 24*c5p1*c7p1*c9p1*ss[22];
  jac_1_0 += 12*c1p1*c3p2*ss[8];
  jac_1_0 += 12*c1p1*c5p2*ss[12];
  jac_1_0 += 12*c1p1*c7p2*ss[16];
  jac_1_0 += 12*c1p1*c9p2*ss[20];
  jac_1_0 += 12*c1p2*c3p1*ss[6];
  jac_1_0 += 12*c1p2*c5p1*ss[8];
  jac_1_0 += 12*c1p2*c7p1*ss[10];
  jac_1_0 += 12*c1p2*c9p1*ss[12];
  jac_1_0 += 12*c3p1*c5p2*ss[14];
  jac_1_0 += 12*c3p1*c7p2*ss[18];
  jac_1_0 += 12*c3p1*c9p2*ss[22];
  jac_1_0 += 12*c3p2*c5p1*ss[12];
  jac_1_0 += 12*c3p2*c7p1*ss[14];
  jac_1_0 += 12*c3p2*c9p1*ss[16];
  jac_1_0 += 12*c5p1*c7p2*ss[20];
  jac_1_0 += 12*c5p1*c9p2*ss[24];
  jac_1_0 += 12*c5p2*c7p1*ss[18];
  jac_1_0 += 12*c5p2*c9p1*ss[20];
  jac_1_0 += 12*c7p1*c9p2*ss[26];
  jac_1_0 += 12*c7p2*c9p1*ss[24];
  jac_1_0 += 4*c1p3*ss[4];
  jac_1_0 += 4*c3p3*ss[10];
  jac_1_0 += 4*c5p3*ss[16];
  jac_1_0 += 4*c7p3*ss[22];
  jac_1_0 += 4*c9p3*ss[28];
  TACC jac_1_1 = convert<TACC>(24*c1p1*c3p1*c5p1*ss[12]);
  jac_1_1 += 24*c1p1*c3p1*c7p1*ss[14];
  jac_1_1 += 24*c1p1*c3p1*c9p1*ss[16];
  jac_1_1 += 24*c1p1*c5p1*c7p1*ss[16];
  jac_1_1 += 24*c1p1*c5p1*c9p1*ss[18];
  jac_1_1 += 24*c1p1*c7p1*c9p1*ss[20];
  jac_1_1 += 24*c3p1*c5p1*c7p1*ss[18];
  jac_1_1 += 24*c3p1*c5p1*c9p1*ss[20];
  jac_1_1 += 24*c3p1*c7p1*c9p1*ss[22];
  jac_1_1 += 24*c5p1*c7p1*c9p1*ss[24];
  jac_1_1 += 12*c1p1*c3p2*ss[10];
  jac_1_1 += 12*c1p1*c5p2*ss[14];
  jac_1_1 += 12*c1p1*c7p2*ss[18];
  jac_1_1 += 12*c1p1*c9p2*ss[22];
  jac_1_1 += 12*c1p2*c3p1*ss[8];
  jac_1_1 += 12*c1p2*c5p1*ss[10];
  jac_1_1 += 12*c1p2*c7p1*ss[12];
  jac_1_1 += 12*c1p2*c9p1*ss[14];
  jac_1_1 += 12*c3p1*c5p2*ss[16];
  jac_1_1 += 12*c3p1*c7p2*ss[20];
  jac_1_1 += 12*c3p1*c9p2*ss[24];
  jac_1_1 += 12*c3p2*c5p1*ss[14];
  jac_1_1 += 12*c3p2*c7p1*ss[16];
  jac_1_1 += 12*c3p2*c9p1*ss[18];
  jac_1_1 += 12*c5p1*c7p2*ss[22];
  jac_1_1 += 12*c5p1*c9p2*ss[26];
  jac_1_1 += 12*c5p2*c7p1*ss[20];
  jac_1_1 += 12*c5p2*c9p1*ss[22];
  jac_1_1 += 12*c7p1*c9p2*ss[28];
  jac_1_1 += 12*c7p2*c9p1*ss[26];
  jac_1_1 += 4*c1p3*ss[6];
  jac_1_1 += 4*c3p3*ss[12];
  jac_1_1 += 4*c5p3*ss[18];
  jac_1_1 += 4*c7p3*ss[24];
  jac_1_1 += 4*c9p3*ss[30];
  TACC jac_1_2 = convert<TACC>(24*c1p1*c3p1*c5p1*ss[14]);
  jac_1_2 += 24*c1p1*c3p1*c7p1*ss[16];
  jac_1_2 += 24*c1p1*c3p1*c9p1*ss[18];
  jac_1_2 += 24*c1p1*c5p1*c7p1*ss[18];
  jac_1_2 += 24*c1p1*c5p1*c9p1*ss[20];
  jac_1_2 += 24*c1p1*c7p1*c9p1*ss[22];
  jac_1_2 += 24*c3p1*c5p1*c7p1*ss[20];
  jac_1_2 += 24*c3p1*c5p1*c9p1*ss[22];
  jac_1_2 += 24*c3p1*c7p1*c9p1*ss[24];
  jac_1_2 += 24*c5p1*c7p1*c9p1*ss[26];
  jac_1_2 += 12*c1p1*c3p2*ss[12];
  jac_1_2 += 12*c1p1*c5p2*ss[16];
  jac_1_2 += 12*c1p1*c7p2*ss[20];
  jac_1_2 += 12*c1p1*c9p2*ss[24];
  jac_1_2 += 12*c1p2*c3p1*ss[10];
  jac_1_2 += 12*c1p2*c5p1*ss[12];
  jac_1_2 += 12*c1p2*c7p1*ss[14];
  jac_1_2 += 12*c1p2*c9p1*ss[16];
  jac_1_2 += 12*c3p1*c5p2*ss[18];
  jac_1_2 += 12*c3p1*c7p2*ss[22];
  jac_1_2 += 12*c3p1*c9p2*ss[26];
  jac_1_2 += 12*c3p2*c5p1*ss[16];
  jac_1_2 += 12*c3p2*c7p1*ss[18];
  jac_1_2 += 12*c3p2*c9p1*ss[20];
  jac_1_2 += 12*c5p1*c7p2*ss[24];
  jac_1_2 += 12*c5p1*c9p2*ss[28];
  jac_1_2 += 12*c5p2*c7p1*ss[22];
  jac_1_2 += 12*c5p2*c9p1*ss[24];
  jac_1_2 += 12*c7p1*c9p2*ss[30];
  jac_1_2 += 12*c7p2*c9p1*ss[28];
  jac_1_2 += 4*c1p3*ss[8];
  jac_1_2 += 4*c3p3*ss[14];
  jac_1_2 += 4*c5p3*ss[20];
  jac_1_2 += 4*c7p3*ss[26];
  jac_1_2 += 4*c9p3*ss[32];
  TACC jac_1_3 = convert<TACC>(24*c1p1*c3p1*c5p1*ss[16]);
  jac_1_3 += 24*c1p1*c3p1*c7p1*ss[18];
  jac_1_3 += 24*c1p1*c3p1*c9p1*ss[20];
  jac_1_3 += 24*c1p1*c5p1*c7p1*ss[20];
  jac_1_3 += 24*c1p1*c5p1*c9p1*ss[22];
  jac_1_3 += 24*c1p1*c7p1*c9p1*ss[24];
  jac_1_3 += 24*c3p1*c5p1*c7p1*ss[22];
  jac_1_3 += 24*c3p1*c5p1*c9p1*ss[24];
  jac_1_3 += 24*c3p1*c7p1*c9p1*ss[26];
  jac_1_3 += 24*c5p1*c7p1*c9p1*ss[28];
  jac_1_3 += 12*c1p1*c3p2*ss[14];
  jac_1_3 += 12*c1p1*c5p2*ss[18];
  jac_1_3 += 12*c1p1*c7p2*ss[22];
  jac_1_3 += 12*c1p1*c9p2*ss[26];
  jac_1_3 += 12*c1p2*c3p1*ss[12];
  jac_1_3 += 12*c1p2*c5p1*ss[14];
  jac_1_3 += 12*c1p2*c7p1*ss[16];
  jac_1_3 += 12*c1p2*c9p1*ss[18];
  jac_1_3 += 12*c3p1*c5p2*ss[20];
  jac_1_3 += 12*c3p1*c7p2*ss[24];
  jac_1_3 += 12*c3p1*c9p2*ss[28];
  jac_1_3 += 12*c3p2*c5p1*ss[18];
  jac_1_3 += 12*c3p2*c7p1*ss[20];
  jac_1_3 += 12*c3p2*c9p1*ss[22];
  jac_1_3 += 12*c5p1*c7p2*ss[26];
  jac_1_3 += 12*c5p1*c9p2*ss[30];
  jac_1_3 += 12*c5p2*c7p1*ss[24];
  jac_1_3 += 12*c5p2*c9p1*ss[26];
  jac_1_3 += 12*c7p1*c9p2*ss[32];
  jac_1_3 += 12*c7p2*c9p1*ss[30];
  jac_1_3 += 4*c1p3*ss[10];
  jac_1_3 += 4*c3p3*ss[16];
  jac_1_3 += 4*c5p3*ss[22];
  jac_1_3 += 4*c7p3*ss[28];
  jac_1_3 += 4*c9p3*ss[34];
  TACC jac_1_4 = convert<TACC>(24*c1p1*c3p1*c5p1*ss[18]);
  jac_1_4 += 24*c1p1*c3p1*c7p1*ss[20];
  jac_1_4 += 24*c1p1*c3p1*c9p1*ss[22];
  jac_1_4 += 24*c1p1*c5p1*c7p1*ss[22];
  jac_1_4 += 24*c1p1*c5p1*c9p1*ss[24];
  jac_1_4 += 24*c1p1*c7p1*c9p1*ss[26];
  jac_1_4 += 24*c3p1*c5p1*c7p1*ss[24];
  jac_1_4 += 24*c3p1*c5p1*c9p1*ss[26];
  jac_1_4 += 24*c3p1*c7p1*c9p1*ss[28];
  jac_1_4 += 24*c5p1*c7p1*c9p1*ss[30];
  jac_1_4 += 12*c1p1*c3p2*ss[16];
  jac_1_4 += 12*c1p1*c5p2*ss[20];
  jac_1_4 += 12*c1p1*c7p2*ss[24];
  jac_1_4 += 12*c1p1*c9p2*ss[28];
  jac_1_4 += 12*c1p2*c3p1*ss[14];
  jac_1_4 += 12*c1p2*c5p1*ss[16];
  jac_1_4 += 12*c1p2*c7p1*ss[18];
  jac_1_4 += 12*c1p2*c9p1*ss[20];
  jac_1_4 += 12*c3p1*c5p2*ss[22];
  jac_1_4 += 12*c3p1*c7p2*ss[26];
  jac_1_4 += 12*c3p1*c9p2*ss[30];
  jac_1_4 += 12*c3p2*c5p1*ss[20];
  jac_1_4 += 12*c3p2*c7p1*ss[22];
  jac_1_4 += 12*c3p2*c9p1*ss[24];
  jac_1_4 += 12*c5p1*c7p2*ss[28];
  jac_1_4 += 12*c5p1*c9p2*ss[32];
  jac_1_4 += 12*c5p2*c7p1*ss[26];
  jac_1_4 += 12*c5p2*c9p1*ss[28];
  jac_1_4 += 12*c7p1*c9p2*ss[34];
  jac_1_4 += 12*c7p2*c9p1*ss[32];
  jac_1_4 += 4*c1p3*ss[12];
  jac_1_4 += 4*c3p3*ss[18];
  jac_1_4 += 4*c5p3*ss[24];
  jac_1_4 += 4*c7p3*ss[30];
  jac_1_4 += 4*c9p3*ss[36];
  TACC jac_2_0 = convert<TACC>(720*c1p1*c3p1*c5p1*c7p1*c9p1*ss[26]);
  jac_2_0 += 360*c1p1*c3p1*c5p1*c7p2*ss[24];
  jac_2_0 += 360*c1p1*c3p1*c5p1*c9p2*ss[28];
  jac_2_0 += 360*c1p1*c3p1*c5p2*c7p1*ss[22];
  jac_2_0 += 360*c1p1*c3p1*c5p2*c9p1*ss[24];
  jac_2_0 += 360*c1p1*c3p1*c7p1*c9p2*ss[30];
  jac_2_0 += 360*c1p1*c3p1*c7p2*c9p1*ss[28];
  jac_2_0 += 360*c1p1*c3p2*c5p1*c7p1*ss[20];
  jac_2_0 += 360*c1p1*c3p2*c5p1*c9p1*ss[22];
  jac_2_0 += 360*c1p1*c3p2*c7p1*c9p1*ss[24];
  jac_2_0 += 360*c1p1*c5p1*c7p1*c9p2*ss[32];
  jac_2_0 += 360*c1p1*c5p1*c7p2*c9p1*ss[30];
  jac_2_0 += 360*c1p1*c5p2*c7p1*c9p1*ss[28];
  jac_2_0 += 360*c1p2*c3p1*c5p1*c7p1*ss[18];
  jac_2_0 += 360*c1p2*c3p1*c5p1*c9p1*ss[20];
  jac_2_0 += 360*c1p2*c3p1*c7p1*c9p1*ss[22];
  jac_2_0 += 360*c1p2*c5p1*c7p1*c9p1*ss[24];
  jac_2_0 += 360*c3p1*c5p1*c7p1*c9p2*ss[34];
  jac_2_0 += 360*c3p1*c5p1*c7p2*c9p1*ss[32];
  jac_2_0 += 360*c3p1*c5p2*c7p1*c9p1*ss[30];
  jac_2_0 += 360*c3p2*c5p1*c7p1*c9p1*ss[28];
  jac_2_0 += 120*c1p1*c3p1*c5p3*ss[20];
  jac_2_0 += 120*c1p1*c3p1*c7p3*ss[26];
  jac_2_0 += 120*c1p1*c3p1*c9p3*ss[32];
  jac_2_0 += 180*c1p1*c3p2*c5p2*ss[18];
  jac_2_0 += 180*c1p1*c3p2*c7p2*ss[22];
  jac_2_0 += 180*c1p1*c3p2*c9p2*ss[26];
  jac_2_0 += 120*c1p1*c3p3*c5p1*ss[16];
  jac_2_0 += 120*c1p1*c3p3*c7p1*ss[18];
  jac_2_0 += 120*c1p1*c3p3*c9p1*ss[20];
  jac_2_0 += 120*c1p1*c5p1*c7p3*ss[28];
  jac_2_0 += 120*c1p1*c5p1*c9p3*ss[34];
  jac_2_0 += 180*c1p1*c5p2*c7p2*ss[26];
  jac_2_0 += 180*c1p1*c5p2*c9p2*ss[30];
  jac_2_0 += 120*c1p1*c5p3*c7p1*ss[24];
  jac_2_0 += 120*c1p1*c5p3*c9p1*ss[26];
  jac_2_0 += 120*c1p1*c7p1*c9p3*ss[36];
  jac_2_0 += 180*c1p1*c7p2*c9p2*ss[34];
  jac_2_0 += 120*c1p1*c7p3*c9p1*ss[32];
  jac_2_0 += 180*c1p2*c3p1*c5p2*ss[16];
  jac_2_0 += 180*c1p2*c3p1*c7p2*ss[20];
  jac_2_0 += 180*c1p2*c3p1*c9p2*ss[24];
  jac_2_0 += 180*c1p2*c3p2*c5p1*ss[14];
  jac_2_0 += 180*c1p2*c3p2*c7p1*ss[16];
  jac_2_0 += 180*c1p2*c3p2*c9p1*ss[18];
  jac_2_0 += 180*c1p2*c5p1*c7p2*ss[22];
  jac_2_0 += 180*c1p2*c5p1*c9p2*ss[26];
  jac_2_0 += 180*c1p2*c5p2*c7p1*ss[20];
  jac_2_0 += 180*c1p2*c5p2*c9p1*ss[22];
  jac_2_0 += 180*c1p2*c7p1*c9p2*ss[28];
  jac_2_0 += 180*c1p2*c7p2*c9p1*ss[26];
  jac_2_0 += 120*c1p3*c3p1*c5p1*ss[12];
  jac_2_0 += 120*c1p3*c3p1*c7p1*ss[14];
  jac_2_0 += 120*c1p3*c3p1*c9p1*ss[16];
  jac_2_0 += 120*c1p3*c5p1*c7p1*ss[16];
  jac_2_0 += 120*c1p3*c5p1*c9p1*ss[18];
  jac_2_0 += 120*c1p3*c7p1*c9p1*ss[20];
  jac_2_0 += 120*c3p1*c5p1*c7p3*ss[30];
  jac_2_0 += 120*c3p1*c5p1*c9p3*ss[36];
  jac_2_0 += 180*c3p1*c5p2*c7p2*ss[28];
  jac_2_0 += 180*c3p1*c5p2*c9p2*ss[32];
  jac_2_0 += 120*c3p1*c5p3*c7p1*ss[26];
  jac_2_0 += 120*c3p1*c5p3*c9p1*ss[28];
  jac_2_0 += 120*c3p1*c7p1*c9p3*ss[38];
  jac_2_0 += 180*c3p1*c7p2*c9p2*ss[36];
  jac_2_0 += 120*c3p1*c7p3*c9p1*ss[34];
  jac_2_0 += 180*c3p2*c5p1*c7p2*ss[26];
  jac_2_0 += 180*c3p2*c5p1*c9p2*ss[30];
  jac_2_0 += 180*c3p2*c5p2*c7p1*ss[24];
  jac_2_0 += 180*c3p2*c5p2*c9p1*ss[26];
  jac_2_0 += 180*c3p2*c7p1*c9p2*ss[32];
  jac_2_0 += 180*c3p2*c7p2*c9p1*ss[30];
  jac_2_0 += 120*c3p3*c5p1*c7p1*ss[22];
  jac_2_0 += 120*c3p3*c5p1*c9p1*ss[24];
  jac_2_0 += 120*c3p3*c7p1*c9p1*ss[26];
  jac_2_0 += 120*c5p1*c7p1*c9p3*ss[40];
  jac_2_0 += 180*c5p1*c7p2*c9p2*ss[38];
  jac_2_0 += 120*c5p1*c7p3*c9p1*ss[36];
  jac_2_0 += 180*c5p2*c7p1*c9p2*ss[36];
  jac_2_0 += 180*c5p2*c7p2*c9p1*ss[34];
  jac_2_0 += 120*c5p3*c7p1*c9p1*ss[32];
  jac_2_0 += 30*c1p1*c3p4*ss[14];
  jac_2_0 += 30*c1p1*c5p4*ss[22];
  jac_2_0 += 30*c1p1*c7p4*ss[30];
  jac_2_0 += 30*c1p1*c9p4*ss[38];
  jac_2_0 += 60*c1p2*c3p3*ss[12];
  jac_2_0 += 60*c1p2*c5p3*ss[18];
  jac_2_0 += 60*c1p2*c7p3*ss[24];
  jac_2_0 += 60*c1p2*c9p3*ss[30];
  jac_2_0 += 60*c1p3*c3p2*ss[10];
  jac_2_0 += 60*c1p3*c5p2*ss[14];
  jac_2_0 += 60*c1p3*c7p2*ss[18];
  jac_2_0 += 60*c1p3*c9p2*ss[22];
  jac_2_0 += 30*c1p4*c3p1*ss[8];
  jac_2_0 += 30*c1p4*c5p1*ss[10];
  jac_2_0 += 30*c1p4*c7p1*ss[12];
  jac_2_0 += 30*c1p4*c9p1*ss[14];
  jac_2_0 += 30*c3p1*c5p4*ss[24];
  jac_2_0 += 30*c3p1*c7p4*ss[32];
  jac_2_0 += 30*c3p1*c9p4*ss[40];
  jac_2_0 += 60*c3p2*c5p3*ss[22];
  jac_2_0 += 60*c3p2*c7p3*ss[28];
  jac_2_0 += 60*c3p2*c9p3*ss[34];
  jac_2_0 += 60*c3p3*c5p2*ss[20];
  jac_2_0 += 60*c3p3*c7p2*ss[24];
  jac_2_0 += 60*c3p3*c9p2*ss[28];
  jac_2_0 += 30*c3p4*c5p1*ss[18];
  jac_2_0 += 30*c3p4*c7p1*ss[20];
  jac_2_0 += 30*c3p4*c9p1*ss[22];
  jac_2_0 += 30*c5p1*c7p4*ss[34];
  jac_2_0 += 30*c5p1*c9p4*ss[42];
  jac_2_0 += 60*c5p2*c7p3*ss[32];
  jac_2_0 += 60*c5p2*c9p3*ss[38];
  jac_2_0 += 60*c5p3*c7p2*ss[30];
  jac_2_0 += 60*c5p3*c9p2*ss[34];
  jac_2_0 += 30*c5p4*c7p1*ss[28];
  jac_2_0 += 30*c5p4*c9p1*ss[30];
  jac_2_0 += 30*c7p1*c9p4*ss[44];
  jac_2_0 += 60*c7p2*c9p3*ss[42];
  jac_2_0 += 60*c7p3*c9p2*ss[40];
  jac_2_0 += 30*c7p4*c9p1*ss[38];
  jac_2_0 += 6*c1p5*ss[6];
  jac_2_0 += 6*c3p5*ss[16];
  jac_2_0 += 6*c5p5*ss[26];
  jac_2_0 += 6*c7p5*ss[36];
  jac_2_0 += 6*c9p5*ss[46];
  TACC jac_2_1 = convert<TACC>(720*c1p1*c3p1*c5p1*c7p1*c9p1*ss[28]);
  jac_2_1 += 360*c1p1*c3p1*c5p1*c7p2*ss[26];
  jac_2_1 += 360*c1p1*c3p1*c5p1*c9p2*ss[30];
  jac_2_1 += 360*c1p1*c3p1*c5p2*c7p1*ss[24];
  jac_2_1 += 360*c1p1*c3p1*c5p2*c9p1*ss[26];
  jac_2_1 += 360*c1p1*c3p1*c7p1*c9p2*ss[32];
  jac_2_1 += 360*c1p1*c3p1*c7p2*c9p1*ss[30];
  jac_2_1 += 360*c1p1*c3p2*c5p1*c7p1*ss[22];
  jac_2_1 += 360*c1p1*c3p2*c5p1*c9p1*ss[24];
  jac_2_1 += 360*c1p1*c3p2*c7p1*c9p1*ss[26];
  jac_2_1 += 360*c1p1*c5p1*c7p1*c9p2*ss[34];
  jac_2_1 += 360*c1p1*c5p1*c7p2*c9p1*ss[32];
  jac_2_1 += 360*c1p1*c5p2*c7p1*c9p1*ss[30];
  jac_2_1 += 360*c1p2*c3p1*c5p1*c7p1*ss[20];
  jac_2_1 += 360*c1p2*c3p1*c5p1*c9p1*ss[22];
  jac_2_1 += 360*c1p2*c3p1*c7p1*c9p1*ss[24];
  jac_2_1 += 360*c1p2*c5p1*c7p1*c9p1*ss[26];
  jac_2_1 += 360*c3p1*c5p1*c7p1*c9p2*ss[36];
  jac_2_1 += 360*c3p1*c5p1*c7p2*c9p1*ss[34];
  jac_2_1 += 360*c3p1*c5p2*c7p1*c9p1*ss[32];
  jac_2_1 += 360*c3p2*c5p1*c7p1*c9p1*ss[30];
  jac_2_1 += 120*c1p1*c3p1*c5p3*ss[22];
  jac_2_1 += 120*c1p1*c3p1*c7p3*ss[28];
  jac_2_1 += 120*c1p1*c3p1*c9p3*ss[34];
  jac_2_1 += 180*c1p1*c3p2*c5p2*ss[20];
  jac_2_1 += 180*c1p1*c3p2*c7p2*ss[24];
  jac_2_1 += 180*c1p1*c3p2*c9p2*ss[28];
  jac_2_1 += 120*c1p1*c3p3*c5p1*ss[18];
  jac_2_1 += 120*c1p1*c3p3*c7p1*ss[20];
  jac_2_1 += 120*c1p1*c3p3*c9p1*ss[22];
  jac_2_1 += 120*c1p1*c5p1*c7p3*ss[30];
  jac_2_1 += 120*c1p1*c5p1*c9p3*ss[36];
  jac_2_1 += 180*c1p1*c5p2*c7p2*ss[28];
  jac_2_1 += 180*c1p1*c5p2*c9p2*ss[32];
  jac_2_1 += 120*c1p1*c5p3*c7p1*ss[26];
  jac_2_1 += 120*c1p1*c5p3*c9p1*ss[28];
  jac_2_1 += 120*c1p1*c7p1*c9p3*ss[38];
  jac_2_1 += 180*c1p1*c7p2*c9p2*ss[36];
  jac_2_1 += 120*c1p1*c7p3*c9p1*ss[34];
  jac_2_1 += 180*c1p2*c3p1*c5p2*ss[18];
  jac_2_1 += 180*c1p2*c3p1*c7p2*ss[22];
  jac_2_1 += 180*c1p2*c3p1*c9p2*ss[26];
  jac_2_1 += 180*c1p2*c3p2*c5p1*ss[16];
  jac_2_1 += 180*c1p2*c3p2*c7p1*ss[18];
  jac_2_1 += 180*c1p2*c3p2*c9p1*ss[20];
  jac_2_1 += 180*c1p2*c5p1*c7p2*ss[24];
  jac_2_1 += 180*c1p2*c5p1*c9p2*ss[28];
  jac_2_1 += 180*c1p2*c5p2*c7p1*ss[22];
  jac_2_1 += 180*c1p2*c5p2*c9p1*ss[24];
  jac_2_1 += 180*c1p2*c7p1*c9p2*ss[30];
  jac_2_1 += 180*c1p2*c7p2*c9p1*ss[28];
  jac_2_1 += 120*c1p3*c3p1*c5p1*ss[14];
  jac_2_1 += 120*c1p3*c3p1*c7p1*ss[16];
  jac_2_1 += 120*c1p3*c3p1*c9p1*ss[18];
  jac_2_1 += 120*c1p3*c5p1*c7p1*ss[18];
  jac_2_1 += 120*c1p3*c5p1*c9p1*ss[20];
  jac_2_1 += 120*c1p3*c7p1*c9p1*ss[22];
  jac_2_1 += 120*c3p1*c5p1*c7p3*ss[32];
  jac_2_1 += 120*c3p1*c5p1*c9p3*ss[38];
  jac_2_1 += 180*c3p1*c5p2*c7p2*ss[30];
  jac_2_1 += 180*c3p1*c5p2*c9p2*ss[34];
  jac_2_1 += 120*c3p1*c5p3*c7p1*ss[28];
  jac_2_1 += 120*c3p1*c5p3*c9p1*ss[30];
  jac_2_1 += 120*c3p1*c7p1*c9p3*ss[40];
  jac_2_1 += 180*c3p1*c7p2*c9p2*ss[38];
  jac_2_1 += 120*c3p1*c7p3*c9p1*ss[36];
  jac_2_1 += 180*c3p2*c5p1*c7p2*ss[28];
  jac_2_1 += 180*c3p2*c5p1*c9p2*ss[32];
  jac_2_1 += 180*c3p2*c5p2*c7p1*ss[26];
  jac_2_1 += 180*c3p2*c5p2*c9p1*ss[28];
  jac_2_1 += 180*c3p2*c7p1*c9p2*ss[34];
  jac_2_1 += 180*c3p2*c7p2*c9p1*ss[32];
  jac_2_1 += 120*c3p3*c5p1*c7p1*ss[24];
  jac_2_1 += 120*c3p3*c5p1*c9p1*ss[26];
  jac_2_1 += 120*c3p3*c7p1*c9p1*ss[28];
  jac_2_1 += 120*c5p1*c7p1*c9p3*ss[42];
  jac_2_1 += 180*c5p1*c7p2*c9p2*ss[40];
  jac_2_1 += 120*c5p1*c7p3*c9p1*ss[38];
  jac_2_1 += 180*c5p2*c7p1*c9p2*ss[38];
  jac_2_1 += 180*c5p2*c7p2*c9p1*ss[36];
  jac_2_1 += 120*c5p3*c7p1*c9p1*ss[34];
  jac_2_1 += 30*c1p1*c3p4*ss[16];
  jac_2_1 += 30*c1p1*c5p4*ss[24];
  jac_2_1 += 30*c1p1*c7p4*ss[32];
  jac_2_1 += 30*c1p1*c9p4*ss[40];
  jac_2_1 += 60*c1p2*c3p3*ss[14];
  jac_2_1 += 60*c1p2*c5p3*ss[20];
  jac_2_1 += 60*c1p2*c7p3*ss[26];
  jac_2_1 += 60*c1p2*c9p3*ss[32];
  jac_2_1 += 60*c1p3*c3p2*ss[12];
  jac_2_1 += 60*c1p3*c5p2*ss[16];
  jac_2_1 += 60*c1p3*c7p2*ss[20];
  jac_2_1 += 60*c1p3*c9p2*ss[24];
  jac_2_1 += 30*c1p4*c3p1*ss[10];
  jac_2_1 += 30*c1p4*c5p1*ss[12];
  jac_2_1 += 30*c1p4*c7p1*ss[14];
  jac_2_1 += 30*c1p4*c9p1*ss[16];
  jac_2_1 += 30*c3p1*c5p4*ss[26];
  jac_2_1 += 30*c3p1*c7p4*ss[34];
  jac_2_1 += 30*c3p1*c9p4*ss[42];
  jac_2_1 += 60*c3p2*c5p3*ss[24];
  jac_2_1 += 60*c3p2*c7p3*ss[30];
  jac_2_1 += 60*c3p2*c9p3*ss[36];
  jac_2_1 += 60*c3p3*c5p2*ss[22];
  jac_2_1 += 60*c3p3*c7p2*ss[26];
  jac_2_1 += 60*c3p3*c9p2*ss[30];
  jac_2_1 += 30*c3p4*c5p1*ss[20];
  jac_2_1 += 30*c3p4*c7p1*ss[22];
  jac_2_1 += 30*c3p4*c9p1*ss[24];
  jac_2_1 += 30*c5p1*c7p4*ss[36];
  jac_2_1 += 30*c5p1*c9p4*ss[44];
  jac_2_1 += 60*c5p2*c7p3*ss[34];
  jac_2_1 += 60*c5p2*c9p3*ss[40];
  jac_2_1 += 60*c5p3*c7p2*ss[32];
  jac_2_1 += 60*c5p3*c9p2*ss[36];
  jac_2_1 += 30*c5p4*c7p1*ss[30];
  jac_2_1 += 30*c5p4*c9p1*ss[32];
  jac_2_1 += 30*c7p1*c9p4*ss[46];
  jac_2_1 += 60*c7p2*c9p3*ss[44];
  jac_2_1 += 60*c7p3*c9p2*ss[42];
  jac_2_1 += 30*c7p4*c9p1*ss[40];
  jac_2_1 += 6*c1p5*ss[8];
  jac_2_1 += 6*c3p5*ss[18];
  jac_2_1 += 6*c5p5*ss[28];
  jac_2_1 += 6*c7p5*ss[38];
  jac_2_1 += 6*c9p5*ss[48];
  TACC jac_2_2 = convert<TACC>(720*c1p1*c3p1*c5p1*c7p1*c9p1*ss[30]);
  jac_2_2 += 360*c1p1*c3p1*c5p1*c7p2*ss[28];
  jac_2_2 += 360*c1p1*c3p1*c5p1*c9p2*ss[32];
  jac_2_2 += 360*c1p1*c3p1*c5p2*c7p1*ss[26];
  jac_2_2 += 360*c1p1*c3p1*c5p2*c9p1*ss[28];
  jac_2_2 += 360*c1p1*c3p1*c7p1*c9p2*ss[34];
  jac_2_2 += 360*c1p1*c3p1*c7p2*c9p1*ss[32];
  jac_2_2 += 360*c1p1*c3p2*c5p1*c7p1*ss[24];
  jac_2_2 += 360*c1p1*c3p2*c5p1*c9p1*ss[26];
  jac_2_2 += 360*c1p1*c3p2*c7p1*c9p1*ss[28];
  jac_2_2 += 360*c1p1*c5p1*c7p1*c9p2*ss[36];
  jac_2_2 += 360*c1p1*c5p1*c7p2*c9p1*ss[34];
  jac_2_2 += 360*c1p1*c5p2*c7p1*c9p1*ss[32];
  jac_2_2 += 360*c1p2*c3p1*c5p1*c7p1*ss[22];
  jac_2_2 += 360*c1p2*c3p1*c5p1*c9p1*ss[24];
  jac_2_2 += 360*c1p2*c3p1*c7p1*c9p1*ss[26];
  jac_2_2 += 360*c1p2*c5p1*c7p1*c9p1*ss[28];
  jac_2_2 += 360*c3p1*c5p1*c7p1*c9p2*ss[38];
  jac_2_2 += 360*c3p1*c5p1*c7p2*c9p1*ss[36];
  jac_2_2 += 360*c3p1*c5p2*c7p1*c9p1*ss[34];
  jac_2_2 += 360*c3p2*c5p1*c7p1*c9p1*ss[32];
  jac_2_2 += 120*c1p1*c3p1*c5p3*ss[24];
  jac_2_2 += 120*c1p1*c3p1*c7p3*ss[30];
  jac_2_2 += 120*c1p1*c3p1*c9p3*ss[36];
  jac_2_2 += 180*c1p1*c3p2*c5p2*ss[22];
  jac_2_2 += 180*c1p1*c3p2*c7p2*ss[26];
  jac_2_2 += 180*c1p1*c3p2*c9p2*ss[30];
  jac_2_2 += 120*c1p1*c3p3*c5p1*ss[20];
  jac_2_2 += 120*c1p1*c3p3*c7p1*ss[22];
  jac_2_2 += 120*c1p1*c3p3*c9p1*ss[24];
  jac_2_2 += 120*c1p1*c5p1*c7p3*ss[32];
  jac_2_2 += 120*c1p1*c5p1*c9p3*ss[38];
  jac_2_2 += 180*c1p1*c5p2*c7p2*ss[30];
  jac_2_2 += 180*c1p1*c5p2*c9p2*ss[34];
  jac_2_2 += 120*c1p1*c5p3*c7p1*ss[28];
  jac_2_2 += 120*c1p1*c5p3*c9p1*ss[30];
  jac_2_2 += 120*c1p1*c7p1*c9p3*ss[40];
  jac_2_2 += 180*c1p1*c7p2*c9p2*ss[38];
  jac_2_2 += 120*c1p1*c7p3*c9p1*ss[36];
  jac_2_2 += 180*c1p2*c3p1*c5p2*ss[20];
  jac_2_2 += 180*c1p2*c3p1*c7p2*ss[24];
  jac_2_2 += 180*c1p2*c3p1*c9p2*ss[28];
  jac_2_2 += 180*c1p2*c3p2*c5p1*ss[18];
  jac_2_2 += 180*c1p2*c3p2*c7p1*ss[20];
  jac_2_2 += 180*c1p2*c3p2*c9p1*ss[22];
  jac_2_2 += 180*c1p2*c5p1*c7p2*ss[26];
  jac_2_2 += 180*c1p2*c5p1*c9p2*ss[30];
  jac_2_2 += 180*c1p2*c5p2*c7p1*ss[24];
  jac_2_2 += 180*c1p2*c5p2*c9p1*ss[26];
  jac_2_2 += 180*c1p2*c7p1*c9p2*ss[32];
  jac_2_2 += 180*c1p2*c7p2*c9p1*ss[30];
  jac_2_2 += 120*c1p3*c3p1*c5p1*ss[16];
  jac_2_2 += 120*c1p3*c3p1*c7p1*ss[18];
  jac_2_2 += 120*c1p3*c3p1*c9p1*ss[20];
  jac_2_2 += 120*c1p3*c5p1*c7p1*ss[20];
  jac_2_2 += 120*c1p3*c5p1*c9p1*ss[22];
  jac_2_2 += 120*c1p3*c7p1*c9p1*ss[24];
  jac_2_2 += 120*c3p1*c5p1*c7p3*ss[34];
  jac_2_2 += 120*c3p1*c5p1*c9p3*ss[40];
  jac_2_2 += 180*c3p1*c5p2*c7p2*ss[32];
  jac_2_2 += 180*c3p1*c5p2*c9p2*ss[36];
  jac_2_2 += 120*c3p1*c5p3*c7p1*ss[30];
  jac_2_2 += 120*c3p1*c5p3*c9p1*ss[32];
  jac_2_2 += 120*c3p1*c7p1*c9p3*ss[42];
  jac_2_2 += 180*c3p1*c7p2*c9p2*ss[40];
  jac_2_2 += 120*c3p1*c7p3*c9p1*ss[38];
  jac_2_2 += 180*c3p2*c5p1*c7p2*ss[30];
  jac_2_2 += 180*c3p2*c5p1*c9p2*ss[34];
  jac_2_2 += 180*c3p2*c5p2*c7p1*ss[28];
  jac_2_2 += 180*c3p2*c5p2*c9p1*ss[30];
  jac_2_2 += 180*c3p2*c7p1*c9p2*ss[36];
  jac_2_2 += 180*c3p2*c7p2*c9p1*ss[34];
  jac_2_2 += 120*c3p3*c5p1*c7p1*ss[26];
  jac_2_2 += 120*c3p3*c5p1*c9p1*ss[28];
  jac_2_2 += 120*c3p3*c7p1*c9p1*ss[30];
  jac_2_2 += 120*c5p1*c7p1*c9p3*ss[44];
  jac_2_2 += 180*c5p1*c7p2*c9p2*ss[42];
  jac_2_2 += 120*c5p1*c7p3*c9p1*ss[40];
  jac_2_2 += 180*c5p2*c7p1*c9p2*ss[40];
  jac_2_2 += 180*c5p2*c7p2*c9p1*ss[38];
  jac_2_2 += 120*c5p3*c7p1*c9p1*ss[36];
  jac_2_2 += 30*c1p1*c3p4*ss[18];
  jac_2_2 += 30*c1p1*c5p4*ss[26];
  jac_2_2 += 30*c1p1*c7p4*ss[34];
  jac_2_2 += 30*c1p1*c9p4*ss[42];
  jac_2_2 += 60*c1p2*c3p3*ss[16];
  jac_2_2 += 60*c1p2*c5p3*ss[22];
  jac_2_2 += 60*c1p2*c7p3*ss[28];
  jac_2_2 += 60*c1p2*c9p3*ss[34];
  jac_2_2 += 60*c1p3*c3p2*ss[14];
  jac_2_2 += 60*c1p3*c5p2*ss[18];
  jac_2_2 += 60*c1p3*c7p2*ss[22];
  jac_2_2 += 60*c1p3*c9p2*ss[26];
  jac_2_2 += 30*c1p4*c3p1*ss[12];
  jac_2_2 += 30*c1p4*c5p1*ss[14];
  jac_2_2 += 30*c1p4*c7p1*ss[16];
  jac_2_2 += 30*c1p4*c9p1*ss[18];
  jac_2_2 += 30*c3p1*c5p4*ss[28];
  jac_2_2 += 30*c3p1*c7p4*ss[36];
  jac_2_2 += 30*c3p1*c9p4*ss[44];
  jac_2_2 += 60*c3p2*c5p3*ss[26];
  jac_2_2 += 60*c3p2*c7p3*ss[32];
  jac_2_2 += 60*c3p2*c9p3*ss[38];
  jac_2_2 += 60*c3p3*c5p2*ss[24];
  jac_2_2 += 60*c3p3*c7p2*ss[28];
  jac_2_2 += 60*c3p3*c9p2*ss[32];
  jac_2_2 += 30*c3p4*c5p1*ss[22];
  jac_2_2 += 30*c3p4*c7p1*ss[24];
  jac_2_2 += 30*c3p4*c9p1*ss[26];
  jac_2_2 += 30*c5p1*c7p4*ss[38];
  jac_2_2 += 30*c5p1*c9p4*ss[46];
  jac_2_2 += 60*c5p2*c7p3*ss[36];
  jac_2_2 += 60*c5p2*c9p3*ss[42];
  jac_2_2 += 60*c5p3*c7p2*ss[34];
  jac_2_2 += 60*c5p3*c9p2*ss[38];
  jac_2_2 += 30*c5p4*c7p1*ss[32];
  jac_2_2 += 30*c5p4*c9p1*ss[34];
  jac_2_2 += 30*c7p1*c9p4*ss[48];
  jac_2_2 += 60*c7p2*c9p3*ss[46];
  jac_2_2 += 60*c7p3*c9p2*ss[44];
  jac_2_2 += 30*c7p4*c9p1*ss[42];
  jac_2_2 += 6*c1p5*ss[10];
  jac_2_2 += 6*c3p5*ss[20];
  jac_2_2 += 6*c5p5*ss[30];
  jac_2_2 += 6*c7p5*ss[40];
  jac_2_2 += 6*c9p5*ss[50];
  TACC jac_2_3 = convert<TACC>(720*c1p1*c3p1*c5p1*c7p1*c9p1*ss[32]);
  jac_2_3 += 360*c1p1*c3p1*c5p1*c7p2*ss[30];
  jac_2_3 += 360*c1p1*c3p1*c5p1*c9p2*ss[34];
  jac_2_3 += 360*c1p1*c3p1*c5p2*c7p1*ss[28];
  jac_2_3 += 360*c1p1*c3p1*c5p2*c9p1*ss[30];
  jac_2_3 += 360*c1p1*c3p1*c7p1*c9p2*ss[36];
  jac_2_3 += 360*c1p1*c3p1*c7p2*c9p1*ss[34];
  jac_2_3 += 360*c1p1*c3p2*c5p1*c7p1*ss[26];
  jac_2_3 += 360*c1p1*c3p2*c5p1*c9p1*ss[28];
  jac_2_3 += 360*c1p1*c3p2*c7p1*c9p1*ss[30];
  jac_2_3 += 360*c1p1*c5p1*c7p1*c9p2*ss[38];
  jac_2_3 += 360*c1p1*c5p1*c7p2*c9p1*ss[36];
  jac_2_3 += 360*c1p1*c5p2*c7p1*c9p1*ss[34];
  jac_2_3 += 360*c1p2*c3p1*c5p1*c7p1*ss[24];
  jac_2_3 += 360*c1p2*c3p1*c5p1*c9p1*ss[26];
  jac_2_3 += 360*c1p2*c3p1*c7p1*c9p1*ss[28];
  jac_2_3 += 360*c1p2*c5p1*c7p1*c9p1*ss[30];
  jac_2_3 += 360*c3p1*c5p1*c7p1*c9p2*ss[40];
  jac_2_3 += 360*c3p1*c5p1*c7p2*c9p1*ss[38];
  jac_2_3 += 360*c3p1*c5p2*c7p1*c9p1*ss[36];
  jac_2_3 += 360*c3p2*c5p1*c7p1*c9p1*ss[34];
  jac_2_3 += 120*c1p1*c3p1*c5p3*ss[26];
  jac_2_3 += 120*c1p1*c3p1*c7p3*ss[32];
  jac_2_3 += 120*c1p1*c3p1*c9p3*ss[38];
  jac_2_3 += 180*c1p1*c3p2*c5p2*ss[24];
  jac_2_3 += 180*c1p1*c3p2*c7p2*ss[28];
  jac_2_3 += 180*c1p1*c3p2*c9p2*ss[32];
  jac_2_3 += 120*c1p1*c3p3*c5p1*ss[22];
  jac_2_3 += 120*c1p1*c3p3*c7p1*ss[24];
  jac_2_3 += 120*c1p1*c3p3*c9p1*ss[26];
  jac_2_3 += 120*c1p1*c5p1*c7p3*ss[34];
  jac_2_3 += 120*c1p1*c5p1*c9p3*ss[40];
  jac_2_3 += 180*c1p1*c5p2*c7p2*ss[32];
  jac_2_3 += 180*c1p1*c5p2*c9p2*ss[36];
  jac_2_3 += 120*c1p1*c5p3*c7p1*ss[30];
  jac_2_3 += 120*c1p1*c5p3*c9p1*ss[32];
  jac_2_3 += 120*c1p1*c7p1*c9p3*ss[42];
  jac_2_3 += 180*c1p1*c7p2*c9p2*ss[40];
  jac_2_3 += 120*c1p1*c7p3*c9p1*ss[38];
  jac_2_3 += 180*c1p2*c3p1*c5p2*ss[22];
  jac_2_3 += 180*c1p2*c3p1*c7p2*ss[26];
  jac_2_3 += 180*c1p2*c3p1*c9p2*ss[30];
  jac_2_3 += 180*c1p2*c3p2*c5p1*ss[20];
  jac_2_3 += 180*c1p2*c3p2*c7p1*ss[22];
  jac_2_3 += 180*c1p2*c3p2*c9p1*ss[24];
  jac_2_3 += 180*c1p2*c5p1*c7p2*ss[28];
  jac_2_3 += 180*c1p2*c5p1*c9p2*ss[32];
  jac_2_3 += 180*c1p2*c5p2*c7p1*ss[26];
  jac_2_3 += 180*c1p2*c5p2*c9p1*ss[28];
  jac_2_3 += 180*c1p2*c7p1*c9p2*ss[34];
  jac_2_3 += 180*c1p2*c7p2*c9p1*ss[32];
  jac_2_3 += 120*c1p3*c3p1*c5p1*ss[18];
  jac_2_3 += 120*c1p3*c3p1*c7p1*ss[20];
  jac_2_3 += 120*c1p3*c3p1*c9p1*ss[22];
  jac_2_3 += 120*c1p3*c5p1*c7p1*ss[22];
  jac_2_3 += 120*c1p3*c5p1*c9p1*ss[24];
  jac_2_3 += 120*c1p3*c7p1*c9p1*ss[26];
  jac_2_3 += 120*c3p1*c5p1*c7p3*ss[36];
  jac_2_3 += 120*c3p1*c5p1*c9p3*ss[42];
  jac_2_3 += 180*c3p1*c5p2*c7p2*ss[34];
  jac_2_3 += 180*c3p1*c5p2*c9p2*ss[38];
  jac_2_3 += 120*c3p1*c5p3*c7p1*ss[32];
  jac_2_3 += 120*c3p1*c5p3*c9p1*ss[34];
  jac_2_3 += 120*c3p1*c7p1*c9p3*ss[44];
  jac_2_3 += 180*c3p1*c7p2*c9p2*ss[42];
  jac_2_3 += 120*c3p1*c7p3*c9p1*ss[40];
  jac_2_3 += 180*c3p2*c5p1*c7p2*ss[32];
  jac_2_3 += 180*c3p2*c5p1*c9p2*ss[36];
  jac_2_3 += 180*c3p2*c5p2*c7p1*ss[30];
  jac_2_3 += 180*c3p2*c5p2*c9p1*ss[32];
  jac_2_3 += 180*c3p2*c7p1*c9p2*ss[38];
  jac_2_3 += 180*c3p2*c7p2*c9p1*ss[36];
  jac_2_3 += 120*c3p3*c5p1*c7p1*ss[28];
  jac_2_3 += 120*c3p3*c5p1*c9p1*ss[30];
  jac_2_3 += 120*c3p3*c7p1*c9p1*ss[32];
  jac_2_3 += 120*c5p1*c7p1*c9p3*ss[46];
  jac_2_3 += 180*c5p1*c7p2*c9p2*ss[44];
  jac_2_3 += 120*c5p1*c7p3*c9p1*ss[42];
  jac_2_3 += 180*c5p2*c7p1*c9p2*ss[42];
  jac_2_3 += 180*c5p2*c7p2*c9p1*ss[40];
  jac_2_3 += 120*c5p3*c7p1*c9p1*ss[38];
  jac_2_3 += 30*c1p1*c3p4*ss[20];
  jac_2_3 += 30*c1p1*c5p4*ss[28];
  jac_2_3 += 30*c1p1*c7p4*ss[36];
  jac_2_3 += 30*c1p1*c9p4*ss[44];
  jac_2_3 += 60*c1p2*c3p3*ss[18];
  jac_2_3 += 60*c1p2*c5p3*ss[24];
  jac_2_3 += 60*c1p2*c7p3*ss[30];
  jac_2_3 += 60*c1p2*c9p3*ss[36];
  jac_2_3 += 60*c1p3*c3p2*ss[16];
  jac_2_3 += 60*c1p3*c5p2*ss[20];
  jac_2_3 += 60*c1p3*c7p2*ss[24];
  jac_2_3 += 60*c1p3*c9p2*ss[28];
  jac_2_3 += 30*c1p4*c3p1*ss[14];
  jac_2_3 += 30*c1p4*c5p1*ss[16];
  jac_2_3 += 30*c1p4*c7p1*ss[18];
  jac_2_3 += 30*c1p4*c9p1*ss[20];
  jac_2_3 += 30*c3p1*c5p4*ss[30];
  jac_2_3 += 30*c3p1*c7p4*ss[38];
  jac_2_3 += 30*c3p1*c9p4*ss[46];
  jac_2_3 += 60*c3p2*c5p3*ss[28];
  jac_2_3 += 60*c3p2*c7p3*ss[34];
  jac_2_3 += 60*c3p2*c9p3*ss[40];
  jac_2_3 += 60*c3p3*c5p2*ss[26];
  jac_2_3 += 60*c3p3*c7p2*ss[30];
  jac_2_3 += 60*c3p3*c9p2*ss[34];
  jac_2_3 += 30*c3p4*c5p1*ss[24];
  jac_2_3 += 30*c3p4*c7p1*ss[26];
  jac_2_3 += 30*c3p4*c9p1*ss[28];
  jac_2_3 += 30*c5p1*c7p4*ss[40];
  jac_2_3 += 30*c5p1*c9p4*ss[48];
  jac_2_3 += 60*c5p2*c7p3*ss[38];
  jac_2_3 += 60*c5p2*c9p3*ss[44];
  jac_2_3 += 60*c5p3*c7p2*ss[36];
  jac_2_3 += 60*c5p3*c9p2*ss[40];
  jac_2_3 += 30*c5p4*c7p1*ss[34];
  jac_2_3 += 30*c5p4*c9p1*ss[36];
  jac_2_3 += 30*c7p1*c9p4*ss[50];
  jac_2_3 += 60*c7p2*c9p3*ss[48];
  jac_2_3 += 60*c7p3*c9p2*ss[46];
  jac_2_3 += 30*c7p4*c9p1*ss[44];
  jac_2_3 += 6*c1p5*ss[12];
  jac_2_3 += 6*c3p5*ss[22];
  jac_2_3 += 6*c5p5*ss[32];
  jac_2_3 += 6*c7p5*ss[42];
  jac_2_3 += 6*c9p5*ss[52];
  TACC jac_2_4 = convert<TACC>(720*c1p1*c3p1*c5p1*c7p1*c9p1*ss[34]);
  jac_2_4 += 360*c1p1*c3p1*c5p1*c7p2*ss[32];
  jac_2_4 += 360*c1p1*c3p1*c5p1*c9p2*ss[36];
  jac_2_4 += 360*c1p1*c3p1*c5p2*c7p1*ss[30];
  jac_2_4 += 360*c1p1*c3p1*c5p2*c9p1*ss[32];
  jac_2_4 += 360*c1p1*c3p1*c7p1*c9p2*ss[38];
  jac_2_4 += 360*c1p1*c3p1*c7p2*c9p1*ss[36];
  jac_2_4 += 360*c1p1*c3p2*c5p1*c7p1*ss[28];
  jac_2_4 += 360*c1p1*c3p2*c5p1*c9p1*ss[30];
  jac_2_4 += 360*c1p1*c3p2*c7p1*c9p1*ss[32];
  jac_2_4 += 360*c1p1*c5p1*c7p1*c9p2*ss[40];
  jac_2_4 += 360*c1p1*c5p1*c7p2*c9p1*ss[38];
  jac_2_4 += 360*c1p1*c5p2*c7p1*c9p1*ss[36];
  jac_2_4 += 360*c1p2*c3p1*c5p1*c7p1*ss[26];
  jac_2_4 += 360*c1p2*c3p1*c5p1*c9p1*ss[28];
  jac_2_4 += 360*c1p2*c3p1*c7p1*c9p1*ss[30];
  jac_2_4 += 360*c1p2*c5p1*c7p1*c9p1*ss[32];
  jac_2_4 += 360*c3p1*c5p1*c7p1*c9p2*ss[42];
  jac_2_4 += 360*c3p1*c5p1*c7p2*c9p1*ss[40];
  jac_2_4 += 360*c3p1*c5p2*c7p1*c9p1*ss[38];
  jac_2_4 += 360*c3p2*c5p1*c7p1*c9p1*ss[36];
  jac_2_4 += 120*c1p1*c3p1*c5p3*ss[28];
  jac_2_4 += 120*c1p1*c3p1*c7p3*ss[34];
  jac_2_4 += 120*c1p1*c3p1*c9p3*ss[40];
  jac_2_4 += 180*c1p1*c3p2*c5p2*ss[26];
  jac_2_4 += 180*c1p1*c3p2*c7p2*ss[30];
  jac_2_4 += 180*c1p1*c3p2*c9p2*ss[34];
  jac_2_4 += 120*c1p1*c3p3*c5p1*ss[24];
  jac_2_4 += 120*c1p1*c3p3*c7p1*ss[26];
  jac_2_4 += 120*c1p1*c3p3*c9p1*ss[28];
  jac_2_4 += 120*c1p1*c5p1*c7p3*ss[36];
  jac_2_4 += 120*c1p1*c5p1*c9p3*ss[42];
  jac_2_4 += 180*c1p1*c5p2*c7p2*ss[34];
  jac_2_4 += 180*c1p1*c5p2*c9p2*ss[38];
  jac_2_4 += 120*c1p1*c5p3*c7p1*ss[32];
  jac_2_4 += 120*c1p1*c5p3*c9p1*ss[34];
  jac_2_4 += 120*c1p1*c7p1*c9p3*ss[44];
  jac_2_4 += 180*c1p1*c7p2*c9p2*ss[42];
  jac_2_4 += 120*c1p1*c7p3*c9p1*ss[40];
  jac_2_4 += 180*c1p2*c3p1*c5p2*ss[24];
  jac_2_4 += 180*c1p2*c3p1*c7p2*ss[28];
  jac_2_4 += 180*c1p2*c3p1*c9p2*ss[32];
  jac_2_4 += 180*c1p2*c3p2*c5p1*ss[22];
  jac_2_4 += 180*c1p2*c3p2*c7p1*ss[24];
  jac_2_4 += 180*c1p2*c3p2*c9p1*ss[26];
  jac_2_4 += 180*c1p2*c5p1*c7p2*ss[30];
  jac_2_4 += 180*c1p2*c5p1*c9p2*ss[34];
  jac_2_4 += 180*c1p2*c5p2*c7p1*ss[28];
  jac_2_4 += 180*c1p2*c5p2*c9p1*ss[30];
  jac_2_4 += 180*c1p2*c7p1*c9p2*ss[36];
  jac_2_4 += 180*c1p2*c7p2*c9p1*ss[34];
  jac_2_4 += 120*c1p3*c3p1*c5p1*ss[20];
  jac_2_4 += 120*c1p3*c3p1*c7p1*ss[22];
  jac_2_4 += 120*c1p3*c3p1*c9p1*ss[24];
  jac_2_4 += 120*c1p3*c5p1*c7p1*ss[24];
  jac_2_4 += 120*c1p3*c5p1*c9p1*ss[26];
  jac_2_4 += 120*c1p3*c7p1*c9p1*ss[28];
  jac_2_4 += 120*c3p1*c5p1*c7p3*ss[38];
  jac_2_4 += 120*c3p1*c5p1*c9p3*ss[44];
  jac_2_4 += 180*c3p1*c5p2*c7p2*ss[36];
  jac_2_4 += 180*c3p1*c5p2*c9p2*ss[40];
  jac_2_4 += 120*c3p1*c5p3*c7p1*ss[34];
  jac_2_4 += 120*c3p1*c5p3*c9p1*ss[36];
  jac_2_4 += 120*c3p1*c7p1*c9p3*ss[46];
  jac_2_4 += 180*c3p1*c7p2*c9p2*ss[44];
  jac_2_4 += 120*c3p1*c7p3*c9p1*ss[42];
  jac_2_4 += 180*c3p2*c5p1*c7p2*ss[34];
  jac_2_4 += 180*c3p2*c5p1*c9p2*ss[38];
  jac_2_4 += 180*c3p2*c5p2*c7p1*ss[32];
  jac_2_4 += 180*c3p2*c5p2*c9p1*ss[34];
  jac_2_4 += 180*c3p2*c7p1*c9p2*ss[40];
  jac_2_4 += 180*c3p2*c7p2*c9p1*ss[38];
  jac_2_4 += 120*c3p3*c5p1*c7p1*ss[30];
  jac_2_4 += 120*c3p3*c5p1*c9p1*ss[32];
  jac_2_4 += 120*c3p3*c7p1*c9p1*ss[34];
  jac_2_4 += 120*c5p1*c7p1*c9p3*ss[48];
  jac_2_4 += 180*c5p1*c7p2*c9p2*ss[46];
  jac_2_4 += 120*c5p1*c7p3*c9p1*ss[44];
  jac_2_4 += 180*c5p2*c7p1*c9p2*ss[44];
  jac_2_4 += 180*c5p2*c7p2*c9p1*ss[42];
  jac_2_4 += 120*c5p3*c7p1*c9p1*ss[40];
  jac_2_4 += 30*c1p1*c3p4*ss[22];
  jac_2_4 += 30*c1p1*c5p4*ss[30];
  jac_2_4 += 30*c1p1*c7p4*ss[38];
  jac_2_4 += 30*c1p1*c9p4*ss[46];
  jac_2_4 += 60*c1p2*c3p3*ss[20];
  jac_2_4 += 60*c1p2*c5p3*ss[26];
  jac_2_4 += 60*c1p2*c7p3*ss[32];
  jac_2_4 += 60*c1p2*c9p3*ss[38];
  jac_2_4 += 60*c1p3*c3p2*ss[18];
  jac_2_4 += 60*c1p3*c5p2*ss[22];
  jac_2_4 += 60*c1p3*c7p2*ss[26];
  jac_2_4 += 60*c1p3*c9p2*ss[30];
  jac_2_4 += 30*c1p4*c3p1*ss[16];
  jac_2_4 += 30*c1p4*c5p1*ss[18];
  jac_2_4 += 30*c1p4*c7p1*ss[20];
  jac_2_4 += 30*c1p4*c9p1*ss[22];
  jac_2_4 += 30*c3p1*c5p4*ss[32];
  jac_2_4 += 30*c3p1*c7p4*ss[40];
  jac_2_4 += 30*c3p1*c9p4*ss[48];
  jac_2_4 += 60*c3p2*c5p3*ss[30];
  jac_2_4 += 60*c3p2*c7p3*ss[36];
  jac_2_4 += 60*c3p2*c9p3*ss[42];
  jac_2_4 += 60*c3p3*c5p2*ss[28];
  jac_2_4 += 60*c3p3*c7p2*ss[32];
  jac_2_4 += 60*c3p3*c9p2*ss[36];
  jac_2_4 += 30*c3p4*c5p1*ss[26];
  jac_2_4 += 30*c3p4*c7p1*ss[28];
  jac_2_4 += 30*c3p4*c9p1*ss[30];
  jac_2_4 += 30*c5p1*c7p4*ss[42];
  jac_2_4 += 30*c5p1*c9p4*ss[50];
  jac_2_4 += 60*c5p2*c7p3*ss[40];
  jac_2_4 += 60*c5p2*c9p3*ss[46];
  jac_2_4 += 60*c5p3*c7p2*ss[38];
  jac_2_4 += 60*c5p3*c9p2*ss[42];
  jac_2_4 += 30*c5p4*c7p1*ss[36];
  jac_2_4 += 30*c5p4*c9p1*ss[38];
  jac_2_4 += 30*c7p1*c9p4*ss[52];
  jac_2_4 += 60*c7p2*c9p3*ss[50];
  jac_2_4 += 60*c7p3*c9p2*ss[48];
  jac_2_4 += 30*c7p4*c9p1*ss[46];
  jac_2_4 += 6*c1p5*ss[14];
  jac_2_4 += 6*c3p5*ss[24];
  jac_2_4 += 6*c5p5*ss[34];
  jac_2_4 += 6*c7p5*ss[44];
  jac_2_4 += 6*c9p5*ss[54];
  TACC jac_3_0 = convert<TACC>(6720*c1p1*c3p1*c5p1*c7p1*c9p3*ss[44]);
  jac_3_0 += 10080*c1p1*c3p1*c5p1*c7p2*c9p2*ss[42];
  jac_3_0 += 6720*c1p1*c3p1*c5p1*c7p3*c9p1*ss[40];
  jac_3_0 += 10080*c1p1*c3p1*c5p2*c7p1*c9p2*ss[40];
  jac_3_0 += 10080*c1p1*c3p1*c5p2*c7p2*c9p1*ss[38];
  jac_3_0 += 6720*c1p1*c3p1*c5p3*c7p1*c9p1*ss[36];
  jac_3_0 += 10080*c1p1*c3p2*c5p1*c7p1*c9p2*ss[38];
  jac_3_0 += 10080*c1p1*c3p2*c5p1*c7p2*c9p1*ss[36];
  jac_3_0 += 10080*c1p1*c3p2*c5p2*c7p1*c9p1*ss[34];
  jac_3_0 += 6720*c1p1*c3p3*c5p1*c7p1*c9p1*ss[32];
  jac_3_0 += 10080*c1p2*c3p1*c5p1*c7p1*c9p2*ss[36];
  jac_3_0 += 10080*c1p2*c3p1*c5p1*c7p2*c9p1*ss[34];
  jac_3_0 += 10080*c1p2*c3p1*c5p2*c7p1*c9p1*ss[32];
  jac_3_0 += 10080*c1p2*c3p2*c5p1*c7p1*c9p1*ss[30];
  jac_3_0 += 6720*c1p3*c3p1*c5p1*c7p1*c9p1*ss[28];
  jac_3_0 += 1680*c1p1*c3p1*c5p1*c7p4*ss[38];
  jac_3_0 += 1680*c1p1*c3p1*c5p1*c9p4*ss[46];
  jac_3_0 += 3360*c1p1*c3p1*c5p2*c7p3*ss[36];
  jac_3_0 += 3360*c1p1*c3p1*c5p2*c9p3*ss[42];
  jac_3_0 += 3360*c1p1*c3p1*c5p3*c7p2*ss[34];
  jac_3_0 += 3360*c1p1*c3p1*c5p3*c9p2*ss[38];
  jac_3_0 += 1680*c1p1*c3p1*c5p4*c7p1*ss[32];
  jac_3_0 += 1680*c1p1*c3p1*c5p4*c9p1*ss[34];
  jac_3_0 += 1680*c1p1*c3p1*c7p1*c9p4*ss[48];
  jac_3_0 += 3360*c1p1*c3p1*c7p2*c9p3*ss[46];
  jac_3_0 += 3360*c1p1*c3p1*c7p3*c9p2*ss[44];
  jac_3_0 += 1680*c1p1*c3p1*c7p4*c9p1*ss[42];
  jac_3_0 += 3360*c1p1*c3p2*c5p1*c7p3*ss[34];
  jac_3_0 += 3360*c1p1*c3p2*c5p1*c9p3*ss[40];
  jac_3_0 += 5040*c1p1*c3p2*c5p2*c7p2*ss[32];
  jac_3_0 += 5040*c1p1*c3p2*c5p2*c9p2*ss[36];
  jac_3_0 += 3360*c1p1*c3p2*c5p3*c7p1*ss[30];
  jac_3_0 += 3360*c1p1*c3p2*c5p3*c9p1*ss[32];
  jac_3_0 += 3360*c1p1*c3p2*c7p1*c9p3*ss[42];
  jac_3_0 += 5040*c1p1*c3p2*c7p2*c9p2*ss[40];
  jac_3_0 += 3360*c1p1*c3p2*c7p3*c9p1*ss[38];
  jac_3_0 += 3360*c1p1*c3p3*c5p1*c7p2*ss[30];
  jac_3_0 += 3360*c1p1*c3p3*c5p1*c9p2*ss[34];
  jac_3_0 += 3360*c1p1*c3p3*c5p2*c7p1*ss[28];
  jac_3_0 += 3360*c1p1*c3p3*c5p2*c9p1*ss[30];
  jac_3_0 += 3360*c1p1*c3p3*c7p1*c9p2*ss[36];
  jac_3_0 += 3360*c1p1*c3p3*c7p2*c9p1*ss[34];
  jac_3_0 += 1680*c1p1*c3p4*c5p1*c7p1*ss[26];
  jac_3_0 += 1680*c1p1*c3p4*c5p1*c9p1*ss[28];
  jac_3_0 += 1680*c1p1*c3p4*c7p1*c9p1*ss[30];
  jac_3_0 += 1680*c1p1*c5p1*c7p1*c9p4*ss[50];
  jac_3_0 += 3360*c1p1*c5p1*c7p2*c9p3*ss[48];
  jac_3_0 += 3360*c1p1*c5p1*c7p3*c9p2*ss[46];
  jac_3_0 += 1680*c1p1*c5p1*c7p4*c9p1*ss[44];
  jac_3_0 += 3360*c1p1*c5p2*c7p1*c9p3*ss[46];
  jac_3_0 += 5040*c1p1*c5p2*c7p2*c9p2*ss[44];
  jac_3_0 += 3360*c1p1*c5p2*c7p3*c9p1*ss[42];
  jac_3_0 += 3360*c1p1*c5p3*c7p1*c9p2*ss[42];
  jac_3_0 += 3360*c1p1*c5p3*c7p2*c9p1*ss[40];
  jac_3_0 += 1680*c1p1*c5p4*c7p1*c9p1*ss[38];
  jac_3_0 += 3360*c1p2*c3p1*c5p1*c7p3*ss[32];
  jac_3_0 += 3360*c1p2*c3p1*c5p1*c9p3*ss[38];
  jac_3_0 += 5040*c1p2*c3p1*c5p2*c7p2*ss[30];
  jac_3_0 += 5040*c1p2*c3p1*c5p2*c9p2*ss[34];
  jac_3_0 += 3360*c1p2*c3p1*c5p3*c7p1*ss[28];
  jac_3_0 += 3360*c1p2*c3p1*c5p3*c9p1*ss[30];
  jac_3_0 += 3360*c1p2*c3p1*c7p1*c9p3*ss[40];
  jac_3_0 += 5040*c1p2*c3p1*c7p2*c9p2*ss[38];
  jac_3_0 += 3360*c1p2*c3p1*c7p3*c9p1*ss[36];
  jac_3_0 += 5040*c1p2*c3p2*c5p1*c7p2*ss[28];
  jac_3_0 += 5040*c1p2*c3p2*c5p1*c9p2*ss[32];
  jac_3_0 += 5040*c1p2*c3p2*c5p2*c7p1*ss[26];
  jac_3_0 += 5040*c1p2*c3p2*c5p2*c9p1*ss[28];
  jac_3_0 += 5040*c1p2*c3p2*c7p1*c9p2*ss[34];
  jac_3_0 += 5040*c1p2*c3p2*c7p2*c9p1*ss[32];
  jac_3_0 += 3360*c1p2*c3p3*c5p1*c7p1*ss[24];
  jac_3_0 += 3360*c1p2*c3p3*c5p1*c9p1*ss[26];
  jac_3_0 += 3360*c1p2*c3p3*c7p1*c9p1*ss[28];
  jac_3_0 += 3360*c1p2*c5p1*c7p1*c9p3*ss[42];
  jac_3_0 += 5040*c1p2*c5p1*c7p2*c9p2*ss[40];
  jac_3_0 += 3360*c1p2*c5p1*c7p3*c9p1*ss[38];
  jac_3_0 += 5040*c1p2*c5p2*c7p1*c9p2*ss[38];
  jac_3_0 += 5040*c1p2*c5p2*c7p2*c9p1*ss[36];
  jac_3_0 += 3360*c1p2*c5p3*c7p1*c9p1*ss[34];
  jac_3_0 += 3360*c1p3*c3p1*c5p1*c7p2*ss[26];
  jac_3_0 += 3360*c1p3*c3p1*c5p1*c9p2*ss[30];
  jac_3_0 += 3360*c1p3*c3p1*c5p2*c7p1*ss[24];
  jac_3_0 += 3360*c1p3*c3p1*c5p2*c9p1*ss[26];
  jac_3_0 += 3360*c1p3*c3p1*c7p1*c9p2*ss[32];
  jac_3_0 += 3360*c1p3*c3p1*c7p2*c9p1*ss[30];
  jac_3_0 += 3360*c1p3*c3p2*c5p1*c7p1*ss[22];
  jac_3_0 += 3360*c1p3*c3p2*c5p1*c9p1*ss[24];
  jac_3_0 += 3360*c1p3*c3p2*c7p1*c9p1*ss[26];
  jac_3_0 += 3360*c1p3*c5p1*c7p1*c9p2*ss[34];
  jac_3_0 += 3360*c1p3*c5p1*c7p2*c9p1*ss[32];
  jac_3_0 += 3360*c1p3*c5p2*c7p1*c9p1*ss[30];
  jac_3_0 += 1680*c1p4*c3p1*c5p1*c7p1*ss[20];
  jac_3_0 += 1680*c1p4*c3p1*c5p1*c9p1*ss[22];
  jac_3_0 += 1680*c1p4*c3p1*c7p1*c9p1*ss[24];
  jac_3_0 += 1680*c1p4*c5p1*c7p1*c9p1*ss[26];
  jac_3_0 += 1680*c3p1*c5p1*c7p1*c9p4*ss[52];
  jac_3_0 += 3360*c3p1*c5p1*c7p2*c9p3*ss[50];
  jac_3_0 += 3360*c3p1*c5p1*c7p3*c9p2*ss[48];
  jac_3_0 += 1680*c3p1*c5p1*c7p4*c9p1*ss[46];
  jac_3_0 += 3360*c3p1*c5p2*c7p1*c9p3*ss[48];
  jac_3_0 += 5040*c3p1*c5p2*c7p2*c9p2*ss[46];
  jac_3_0 += 3360*c3p1*c5p2*c7p3*c9p1*ss[44];
  jac_3_0 += 3360*c3p1*c5p3*c7p1*c9p2*ss[44];
  jac_3_0 += 3360*c3p1*c5p3*c7p2*c9p1*ss[42];
  jac_3_0 += 1680*c3p1*c5p4*c7p1*c9p1*ss[40];
  jac_3_0 += 3360*c3p2*c5p1*c7p1*c9p3*ss[46];
  jac_3_0 += 5040*c3p2*c5p1*c7p2*c9p2*ss[44];
  jac_3_0 += 3360*c3p2*c5p1*c7p3*c9p1*ss[42];
  jac_3_0 += 5040*c3p2*c5p2*c7p1*c9p2*ss[42];
  jac_3_0 += 5040*c3p2*c5p2*c7p2*c9p1*ss[40];
  jac_3_0 += 3360*c3p2*c5p3*c7p1*c9p1*ss[38];
  jac_3_0 += 3360*c3p3*c5p1*c7p1*c9p2*ss[40];
  jac_3_0 += 3360*c3p3*c5p1*c7p2*c9p1*ss[38];
  jac_3_0 += 3360*c3p3*c5p2*c7p1*c9p1*ss[36];
  jac_3_0 += 1680*c3p4*c5p1*c7p1*c9p1*ss[34];
  jac_3_0 += 336*c1p1*c3p1*c5p5*ss[30];
  jac_3_0 += 336*c1p1*c3p1*c7p5*ss[40];
  jac_3_0 += 336*c1p1*c3p1*c9p5*ss[50];
  jac_3_0 += 840*c1p1*c3p2*c5p4*ss[28];
  jac_3_0 += 840*c1p1*c3p2*c7p4*ss[36];
  jac_3_0 += 840*c1p1*c3p2*c9p4*ss[44];
  jac_3_0 += 1120*c1p1*c3p3*c5p3*ss[26];
  jac_3_0 += 1120*c1p1*c3p3*c7p3*ss[32];
  jac_3_0 += 1120*c1p1*c3p3*c9p3*ss[38];
  jac_3_0 += 840*c1p1*c3p4*c5p2*ss[24];
  jac_3_0 += 840*c1p1*c3p4*c7p2*ss[28];
  jac_3_0 += 840*c1p1*c3p4*c9p2*ss[32];
  jac_3_0 += 336*c1p1*c3p5*c5p1*ss[22];
  jac_3_0 += 336*c1p1*c3p5*c7p1*ss[24];
  jac_3_0 += 336*c1p1*c3p5*c9p1*ss[26];
  jac_3_0 += 336*c1p1*c5p1*c7p5*ss[42];
  jac_3_0 += 336*c1p1*c5p1*c9p5*ss[52];
  jac_3_0 += 840*c1p1*c5p2*c7p4*ss[40];
  jac_3_0 += 840*c1p1*c5p2*c9p4*ss[48];
  jac_3_0 += 1120*c1p1*c5p3*c7p3*ss[38];
  jac_3_0 += 1120*c1p1*c5p3*c9p3*ss[44];
  jac_3_0 += 840*c1p1*c5p4*c7p2*ss[36];
  jac_3_0 += 840*c1p1*c5p4*c9p2*ss[40];
  jac_3_0 += 336*c1p1*c5p5*c7p1*ss[34];
  jac_3_0 += 336*c1p1*c5p5*c9p1*ss[36];
  jac_3_0 += 336*c1p1*c7p1*c9p5*ss[54];
  jac_3_0 += 840*c1p1*c7p2*c9p4*ss[52];
  jac_3_0 += 1120*c1p1*c7p3*c9p3*ss[50];
  jac_3_0 += 840*c1p1*c7p4*c9p2*ss[48];
  jac_3_0 += 336*c1p1*c7p5*c9p1*ss[46];
  jac_3_0 += 840*c1p2*c3p1*c5p4*ss[26];
  jac_3_0 += 840*c1p2*c3p1*c7p4*ss[34];
  jac_3_0 += 840*c1p2*c3p1*c9p4*ss[42];
  jac_3_0 += 1680*c1p2*c3p2*c5p3*ss[24];
  jac_3_0 += 1680*c1p2*c3p2*c7p3*ss[30];
  jac_3_0 += 1680*c1p2*c3p2*c9p3*ss[36];
  jac_3_0 += 1680*c1p2*c3p3*c5p2*ss[22];
  jac_3_0 += 1680*c1p2*c3p3*c7p2*ss[26];
  jac_3_0 += 1680*c1p2*c3p3*c9p2*ss[30];
  jac_3_0 += 840*c1p2*c3p4*c5p1*ss[20];
  jac_3_0 += 840*c1p2*c3p4*c7p1*ss[22];
  jac_3_0 += 840*c1p2*c3p4*c9p1*ss[24];
  jac_3_0 += 840*c1p2*c5p1*c7p4*ss[36];
  jac_3_0 += 840*c1p2*c5p1*c9p4*ss[44];
  jac_3_0 += 1680*c1p2*c5p2*c7p3*ss[34];
  jac_3_0 += 1680*c1p2*c5p2*c9p3*ss[40];
  jac_3_0 += 1680*c1p2*c5p3*c7p2*ss[32];
  jac_3_0 += 1680*c1p2*c5p3*c9p2*ss[36];
  jac_3_0 += 840*c1p2*c5p4*c7p1*ss[30];
  jac_3_0 += 840*c1p2*c5p4*c9p1*ss[32];
  jac_3_0 += 840*c1p2*c7p1*c9p4*ss[46];
  jac_3_0 += 1680*c1p2*c7p2*c9p3*ss[44];
  jac_3_0 += 1680*c1p2*c7p3*c9p2*ss[42];
  jac_3_0 += 840*c1p2*c7p4*c9p1*ss[40];
  jac_3_0 += 1120*c1p3*c3p1*c5p3*ss[22];
  jac_3_0 += 1120*c1p3*c3p1*c7p3*ss[28];
  jac_3_0 += 1120*c1p3*c3p1*c9p3*ss[34];
  jac_3_0 += 1680*c1p3*c3p2*c5p2*ss[20];
  jac_3_0 += 1680*c1p3*c3p2*c7p2*ss[24];
  jac_3_0 += 1680*c1p3*c3p2*c9p2*ss[28];
  jac_3_0 += 1120*c1p3*c3p3*c5p1*ss[18];
  jac_3_0 += 1120*c1p3*c3p3*c7p1*ss[20];
  jac_3_0 += 1120*c1p3*c3p3*c9p1*ss[22];
  jac_3_0 += 1120*c1p3*c5p1*c7p3*ss[30];
  jac_3_0 += 1120*c1p3*c5p1*c9p3*ss[36];
  jac_3_0 += 1680*c1p3*c5p2*c7p2*ss[28];
  jac_3_0 += 1680*c1p3*c5p2*c9p2*ss[32];
  jac_3_0 += 1120*c1p3*c5p3*c7p1*ss[26];
  jac_3_0 += 1120*c1p3*c5p3*c9p1*ss[28];
  jac_3_0 += 1120*c1p3*c7p1*c9p3*ss[38];
  jac_3_0 += 1680*c1p3*c7p2*c9p2*ss[36];
  jac_3_0 += 1120*c1p3*c7p3*c9p1*ss[34];
  jac_3_0 += 840*c1p4*c3p1*c5p2*ss[18];
  jac_3_0 += 840*c1p4*c3p1*c7p2*ss[22];
  jac_3_0 += 840*c1p4*c3p1*c9p2*ss[26];
  jac_3_0 += 840*c1p4*c3p2*c5p1*ss[16];
  jac_3_0 += 840*c1p4*c3p2*c7p1*ss[18];
  jac_3_0 += 840*c1p4*c3p2*c9p1*ss[20];
  jac_3_0 += 840*c1p4*c5p1*c7p2*ss[24];
  jac_3_0 += 840*c1p4*c5p1*c9p2*ss[28];
  jac_3_0 += 840*c1p4*c5p2*c7p1*ss[22];
  jac_3_0 += 840*c1p4*c5p2*c9p1*ss[24];
  jac_3_0 += 840*c1p4*c7p1*c9p2*ss[30];
  jac_3_0 += 840*c1p4*c7p2*c9p1*ss[28];
  jac_3_0 += 336*c1p5*c3p1*c5p1*ss[14];
  jac_3_0 += 336*c1p5*c3p1*c7p1*ss[16];
  jac_3_0 += 336*c1p5*c3p1*c9p1*ss[18];
  jac_3_0 += 336*c1p5*c5p1*c7p1*ss[18];
  jac_3_0 += 336*c1p5*c5p1*c9p1*ss[20];
  jac_3_0 += 336*c1p5*c7p1*c9p1*ss[22];
  jac_3_0 += 336*c3p1*c5p1*c7p5*ss[44];
  jac_3_0 += 336*c3p1*c5p1*c9p5*ss[54];
  jac_3_0 += 840*c3p1*c5p2*c7p4*ss[42];
  jac_3_0 += 840*c3p1*c5p2*c9p4*ss[50];
  jac_3_0 += 1120*c3p1*c5p3*c7p3*ss[40];
  jac_3_0 += 1120*c3p1*c5p3*c9p3*ss[46];
  jac_3_0 += 840*c3p1*c5p4*c7p2*ss[38];
  jac_3_0 += 840*c3p1*c5p4*c9p2*ss[42];
  jac_3_0 += 336*c3p1*c5p5*c7p1*ss[36];
  jac_3_0 += 336*c3p1*c5p5*c9p1*ss[38];
  jac_3_0 += 336*c3p1*c7p1*c9p5*ss[56];
  jac_3_0 += 840*c3p1*c7p2*c9p4*ss[54];
  jac_3_0 += 1120*c3p1*c7p3*c9p3*ss[52];
  jac_3_0 += 840*c3p1*c7p4*c9p2*ss[50];
  jac_3_0 += 336*c3p1*c7p5*c9p1*ss[48];
  jac_3_0 += 840*c3p2*c5p1*c7p4*ss[40];
  jac_3_0 += 840*c3p2*c5p1*c9p4*ss[48];
  jac_3_0 += 1680*c3p2*c5p2*c7p3*ss[38];
  jac_3_0 += 1680*c3p2*c5p2*c9p3*ss[44];
  jac_3_0 += 1680*c3p2*c5p3*c7p2*ss[36];
  jac_3_0 += 1680*c3p2*c5p3*c9p2*ss[40];
  jac_3_0 += 840*c3p2*c5p4*c7p1*ss[34];
  jac_3_0 += 840*c3p2*c5p4*c9p1*ss[36];
  jac_3_0 += 840*c3p2*c7p1*c9p4*ss[50];
  jac_3_0 += 1680*c3p2*c7p2*c9p3*ss[48];
  jac_3_0 += 1680*c3p2*c7p3*c9p2*ss[46];
  jac_3_0 += 840*c3p2*c7p4*c9p1*ss[44];
  jac_3_0 += 1120*c3p3*c5p1*c7p3*ss[36];
  jac_3_0 += 1120*c3p3*c5p1*c9p3*ss[42];
  jac_3_0 += 1680*c3p3*c5p2*c7p2*ss[34];
  jac_3_0 += 1680*c3p3*c5p2*c9p2*ss[38];
  jac_3_0 += 1120*c3p3*c5p3*c7p1*ss[32];
  jac_3_0 += 1120*c3p3*c5p3*c9p1*ss[34];
  jac_3_0 += 1120*c3p3*c7p1*c9p3*ss[44];
  jac_3_0 += 1680*c3p3*c7p2*c9p2*ss[42];
  jac_3_0 += 1120*c3p3*c7p3*c9p1*ss[40];
  jac_3_0 += 840*c3p4*c5p1*c7p2*ss[32];
  jac_3_0 += 840*c3p4*c5p1*c9p2*ss[36];
  jac_3_0 += 840*c3p4*c5p2*c7p1*ss[30];
  jac_3_0 += 840*c3p4*c5p2*c9p1*ss[32];
  jac_3_0 += 840*c3p4*c7p1*c9p2*ss[38];
  jac_3_0 += 840*c3p4*c7p2*c9p1*ss[36];
  jac_3_0 += 336*c3p5*c5p1*c7p1*ss[28];
  jac_3_0 += 336*c3p5*c5p1*c9p1*ss[30];
  jac_3_0 += 336*c3p5*c7p1*c9p1*ss[32];
  jac_3_0 += 336*c5p1*c7p1*c9p5*ss[58];
  jac_3_0 += 840*c5p1*c7p2*c9p4*ss[56];
  jac_3_0 += 1120*c5p1*c7p3*c9p3*ss[54];
  jac_3_0 += 840*c5p1*c7p4*c9p2*ss[52];
  jac_3_0 += 336*c5p1*c7p5*c9p1*ss[50];
  jac_3_0 += 840*c5p2*c7p1*c9p4*ss[54];
  jac_3_0 += 1680*c5p2*c7p2*c9p3*ss[52];
  jac_3_0 += 1680*c5p2*c7p3*c9p2*ss[50];
  jac_3_0 += 840*c5p2*c7p4*c9p1*ss[48];
  jac_3_0 += 1120*c5p3*c7p1*c9p3*ss[50];
  jac_3_0 += 1680*c5p3*c7p2*c9p2*ss[48];
  jac_3_0 += 1120*c5p3*c7p3*c9p1*ss[46];
  jac_3_0 += 840*c5p4*c7p1*c9p2*ss[46];
  jac_3_0 += 840*c5p4*c7p2*c9p1*ss[44];
  jac_3_0 += 336*c5p5*c7p1*c9p1*ss[42];
  jac_3_0 += 56*c1p1*c3p6*ss[20];
  jac_3_0 += 56*c1p1*c5p6*ss[32];
  jac_3_0 += 56*c1p1*c7p6*ss[44];
  jac_3_0 += 56*c1p1*c9p6*ss[56];
  jac_3_0 += 168*c1p2*c3p5*ss[18];
  jac_3_0 += 168*c1p2*c5p5*ss[28];
  jac_3_0 += 168*c1p2*c7p5*ss[38];
  jac_3_0 += 168*c1p2*c9p5*ss[48];
  jac_3_0 += 280*c1p3*c3p4*ss[16];
  jac_3_0 += 280*c1p3*c5p4*ss[24];
  jac_3_0 += 280*c1p3*c7p4*ss[32];
  jac_3_0 += 280*c1p3*c9p4*ss[40];
  jac_3_0 += 280*c1p4*c3p3*ss[14];
  jac_3_0 += 280*c1p4*c5p3*ss[20];
  jac_3_0 += 280*c1p4*c7p3*ss[26];
  jac_3_0 += 280*c1p4*c9p3*ss[32];
  jac_3_0 += 168*c1p5*c3p2*ss[12];
  jac_3_0 += 168*c1p5*c5p2*ss[16];
  jac_3_0 += 168*c1p5*c7p2*ss[20];
  jac_3_0 += 168*c1p5*c9p2*ss[24];
  jac_3_0 += 56*c1p6*c3p1*ss[10];
  jac_3_0 += 56*c1p6*c5p1*ss[12];
  jac_3_0 += 56*c1p6*c7p1*ss[14];
  jac_3_0 += 56*c1p6*c9p1*ss[16];
  jac_3_0 += 56*c3p1*c5p6*ss[34];
  jac_3_0 += 56*c3p1*c7p6*ss[46];
  jac_3_0 += 56*c3p1*c9p6*ss[58];
  jac_3_0 += 168*c3p2*c5p5*ss[32];
  jac_3_0 += 168*c3p2*c7p5*ss[42];
  jac_3_0 += 168*c3p2*c9p5*ss[52];
  jac_3_0 += 280*c3p3*c5p4*ss[30];
  jac_3_0 += 280*c3p3*c7p4*ss[38];
  jac_3_0 += 280*c3p3*c9p4*ss[46];
  jac_3_0 += 280*c3p4*c5p3*ss[28];
  jac_3_0 += 280*c3p4*c7p3*ss[34];
  jac_3_0 += 280*c3p4*c9p3*ss[40];
  jac_3_0 += 168*c3p5*c5p2*ss[26];
  jac_3_0 += 168*c3p5*c7p2*ss[30];
  jac_3_0 += 168*c3p5*c9p2*ss[34];
  jac_3_0 += 56*c3p6*c5p1*ss[24];
  jac_3_0 += 56*c3p6*c7p1*ss[26];
  jac_3_0 += 56*c3p6*c9p1*ss[28];
  jac_3_0 += 56*c5p1*c7p6*ss[48];
  jac_3_0 += 56*c5p1*c9p6*ss[60];
  jac_3_0 += 168*c5p2*c7p5*ss[46];
  jac_3_0 += 168*c5p2*c9p5*ss[56];
  jac_3_0 += 280*c5p3*c7p4*ss[44];
  jac_3_0 += 280*c5p3*c9p4*ss[52];
  jac_3_0 += 280*c5p4*c7p3*ss[42];
  jac_3_0 += 280*c5p4*c9p3*ss[48];
  jac_3_0 += 168*c5p5*c7p2*ss[40];
  jac_3_0 += 168*c5p5*c9p2*ss[44];
  jac_3_0 += 56*c5p6*c7p1*ss[38];
  jac_3_0 += 56*c5p6*c9p1*ss[40];
  jac_3_0 += 56*c7p1*c9p6*ss[62];
  jac_3_0 += 168*c7p2*c9p5*ss[60];
  jac_3_0 += 280*c7p3*c9p4*ss[58];
  jac_3_0 += 280*c7p4*c9p3*ss[56];
  jac_3_0 += 168*c7p5*c9p2*ss[54];
  jac_3_0 += 56*c7p6*c9p1*ss[52];
  jac_3_0 += 8*c1p7*ss[8];
  jac_3_0 += 8*c3p7*ss[22];
  jac_3_0 += 8*c5p7*ss[36];
  jac_3_0 += 8*c7p7*ss[50];
  jac_3_0 += 8*c9p7*ss[64];
  TACC jac_3_1 = convert<TACC>(6720*c1p1*c3p1*c5p1*c7p1*c9p3*ss[46]);
  jac_3_1 += 10080*c1p1*c3p1*c5p1*c7p2*c9p2*ss[44];
  jac_3_1 += 6720*c1p1*c3p1*c5p1*c7p3*c9p1*ss[42];
  jac_3_1 += 10080*c1p1*c3p1*c5p2*c7p1*c9p2*ss[42];
  jac_3_1 += 10080*c1p1*c3p1*c5p2*c7p2*c9p1*ss[40];
  jac_3_1 += 6720*c1p1*c3p1*c5p3*c7p1*c9p1*ss[38];
  jac_3_1 += 10080*c1p1*c3p2*c5p1*c7p1*c9p2*ss[40];
  jac_3_1 += 10080*c1p1*c3p2*c5p1*c7p2*c9p1*ss[38];
  jac_3_1 += 10080*c1p1*c3p2*c5p2*c7p1*c9p1*ss[36];
  jac_3_1 += 6720*c1p1*c3p3*c5p1*c7p1*c9p1*ss[34];
  jac_3_1 += 10080*c1p2*c3p1*c5p1*c7p1*c9p2*ss[38];
  jac_3_1 += 10080*c1p2*c3p1*c5p1*c7p2*c9p1*ss[36];
  jac_3_1 += 10080*c1p2*c3p1*c5p2*c7p1*c9p1*ss[34];
  jac_3_1 += 10080*c1p2*c3p2*c5p1*c7p1*c9p1*ss[32];
  jac_3_1 += 6720*c1p3*c3p1*c5p1*c7p1*c9p1*ss[30];
  jac_3_1 += 1680*c1p1*c3p1*c5p1*c7p4*ss[40];
  jac_3_1 += 1680*c1p1*c3p1*c5p1*c9p4*ss[48];
  jac_3_1 += 3360*c1p1*c3p1*c5p2*c7p3*ss[38];
  jac_3_1 += 3360*c1p1*c3p1*c5p2*c9p3*ss[44];
  jac_3_1 += 3360*c1p1*c3p1*c5p3*c7p2*ss[36];
  jac_3_1 += 3360*c1p1*c3p1*c5p3*c9p2*ss[40];
  jac_3_1 += 1680*c1p1*c3p1*c5p4*c7p1*ss[34];
  jac_3_1 += 1680*c1p1*c3p1*c5p4*c9p1*ss[36];
  jac_3_1 += 1680*c1p1*c3p1*c7p1*c9p4*ss[50];
  jac_3_1 += 3360*c1p1*c3p1*c7p2*c9p3*ss[48];
  jac_3_1 += 3360*c1p1*c3p1*c7p3*c9p2*ss[46];
  jac_3_1 += 1680*c1p1*c3p1*c7p4*c9p1*ss[44];
  jac_3_1 += 3360*c1p1*c3p2*c5p1*c7p3*ss[36];
  jac_3_1 += 3360*c1p1*c3p2*c5p1*c9p3*ss[42];
  jac_3_1 += 5040*c1p1*c3p2*c5p2*c7p2*ss[34];
  jac_3_1 += 5040*c1p1*c3p2*c5p2*c9p2*ss[38];
  jac_3_1 += 3360*c1p1*c3p2*c5p3*c7p1*ss[32];
  jac_3_1 += 3360*c1p1*c3p2*c5p3*c9p1*ss[34];
  jac_3_1 += 3360*c1p1*c3p2*c7p1*c9p3*ss[44];
  jac_3_1 += 5040*c1p1*c3p2*c7p2*c9p2*ss[42];
  jac_3_1 += 3360*c1p1*c3p2*c7p3*c9p1*ss[40];
  jac_3_1 += 3360*c1p1*c3p3*c5p1*c7p2*ss[32];
  jac_3_1 += 3360*c1p1*c3p3*c5p1*c9p2*ss[36];
  jac_3_1 += 3360*c1p1*c3p3*c5p2*c7p1*ss[30];
  jac_3_1 += 3360*c1p1*c3p3*c5p2*c9p1*ss[32];
  jac_3_1 += 3360*c1p1*c3p3*c7p1*c9p2*ss[38];
  jac_3_1 += 3360*c1p1*c3p3*c7p2*c9p1*ss[36];
  jac_3_1 += 1680*c1p1*c3p4*c5p1*c7p1*ss[28];
  jac_3_1 += 1680*c1p1*c3p4*c5p1*c9p1*ss[30];
  jac_3_1 += 1680*c1p1*c3p4*c7p1*c9p1*ss[32];
  jac_3_1 += 1680*c1p1*c5p1*c7p1*c9p4*ss[52];
  jac_3_1 += 3360*c1p1*c5p1*c7p2*c9p3*ss[50];
  jac_3_1 += 3360*c1p1*c5p1*c7p3*c9p2*ss[48];
  jac_3_1 += 1680*c1p1*c5p1*c7p4*c9p1*ss[46];
  jac_3_1 += 3360*c1p1*c5p2*c7p1*c9p3*ss[48];
  jac_3_1 += 5040*c1p1*c5p2*c7p2*c9p2*ss[46];
  jac_3_1 += 3360*c1p1*c5p2*c7p3*c9p1*ss[44];
  jac_3_1 += 3360*c1p1*c5p3*c7p1*c9p2*ss[44];
  jac_3_1 += 3360*c1p1*c5p3*c7p2*c9p1*ss[42];
  jac_3_1 += 1680*c1p1*c5p4*c7p1*c9p1*ss[40];
  jac_3_1 += 3360*c1p2*c3p1*c5p1*c7p3*ss[34];
  jac_3_1 += 3360*c1p2*c3p1*c5p1*c9p3*ss[40];
  jac_3_1 += 5040*c1p2*c3p1*c5p2*c7p2*ss[32];
  jac_3_1 += 5040*c1p2*c3p1*c5p2*c9p2*ss[36];
  jac_3_1 += 3360*c1p2*c3p1*c5p3*c7p1*ss[30];
  jac_3_1 += 3360*c1p2*c3p1*c5p3*c9p1*ss[32];
  jac_3_1 += 3360*c1p2*c3p1*c7p1*c9p3*ss[42];
  jac_3_1 += 5040*c1p2*c3p1*c7p2*c9p2*ss[40];
  jac_3_1 += 3360*c1p2*c3p1*c7p3*c9p1*ss[38];
  jac_3_1 += 5040*c1p2*c3p2*c5p1*c7p2*ss[30];
  jac_3_1 += 5040*c1p2*c3p2*c5p1*c9p2*ss[34];
  jac_3_1 += 5040*c1p2*c3p2*c5p2*c7p1*ss[28];
  jac_3_1 += 5040*c1p2*c3p2*c5p2*c9p1*ss[30];
  jac_3_1 += 5040*c1p2*c3p2*c7p1*c9p2*ss[36];
  jac_3_1 += 5040*c1p2*c3p2*c7p2*c9p1*ss[34];
  jac_3_1 += 3360*c1p2*c3p3*c5p1*c7p1*ss[26];
  jac_3_1 += 3360*c1p2*c3p3*c5p1*c9p1*ss[28];
  jac_3_1 += 3360*c1p2*c3p3*c7p1*c9p1*ss[30];
  jac_3_1 += 3360*c1p2*c5p1*c7p1*c9p3*ss[44];
  jac_3_1 += 5040*c1p2*c5p1*c7p2*c9p2*ss[42];
  jac_3_1 += 3360*c1p2*c5p1*c7p3*c9p1*ss[40];
  jac_3_1 += 5040*c1p2*c5p2*c7p1*c9p2*ss[40];
  jac_3_1 += 5040*c1p2*c5p2*c7p2*c9p1*ss[38];
  jac_3_1 += 3360*c1p2*c5p3*c7p1*c9p1*ss[36];
  jac_3_1 += 3360*c1p3*c3p1*c5p1*c7p2*ss[28];
  jac_3_1 += 3360*c1p3*c3p1*c5p1*c9p2*ss[32];
  jac_3_1 += 3360*c1p3*c3p1*c5p2*c7p1*ss[26];
  jac_3_1 += 3360*c1p3*c3p1*c5p2*c9p1*ss[28];
  jac_3_1 += 3360*c1p3*c3p1*c7p1*c9p2*ss[34];
  jac_3_1 += 3360*c1p3*c3p1*c7p2*c9p1*ss[32];
  jac_3_1 += 3360*c1p3*c3p2*c5p1*c7p1*ss[24];
  jac_3_1 += 3360*c1p3*c3p2*c5p1*c9p1*ss[26];
  jac_3_1 += 3360*c1p3*c3p2*c7p1*c9p1*ss[28];
  jac_3_1 += 3360*c1p3*c5p1*c7p1*c9p2*ss[36];
  jac_3_1 += 3360*c1p3*c5p1*c7p2*c9p1*ss[34];
  jac_3_1 += 3360*c1p3*c5p2*c7p1*c9p1*ss[32];
  jac_3_1 += 1680*c1p4*c3p1*c5p1*c7p1*ss[22];
  jac_3_1 += 1680*c1p4*c3p1*c5p1*c9p1*ss[24];
  jac_3_1 += 1680*c1p4*c3p1*c7p1*c9p1*ss[26];
  jac_3_1 += 1680*c1p4*c5p1*c7p1*c9p1*ss[28];
  jac_3_1 += 1680*c3p1*c5p1*c7p1*c9p4*ss[54];
  jac_3_1 += 3360*c3p1*c5p1*c7p2*c9p3*ss[52];
  jac_3_1 += 3360*c3p1*c5p1*c7p3*c9p2*ss[50];
  jac_3_1 += 1680*c3p1*c5p1*c7p4*c9p1*ss[48];
  jac_3_1 += 3360*c3p1*c5p2*c7p1*c9p3*ss[50];
  jac_3_1 += 5040*c3p1*c5p2*c7p2*c9p2*ss[48];
  jac_3_1 += 3360*c3p1*c5p2*c7p3*c9p1*ss[46];
  jac_3_1 += 3360*c3p1*c5p3*c7p1*c9p2*ss[46];
  jac_3_1 += 3360*c3p1*c5p3*c7p2*c9p1*ss[44];
  jac_3_1 += 1680*c3p1*c5p4*c7p1*c9p1*ss[42];
  jac_3_1 += 3360*c3p2*c5p1*c7p1*c9p3*ss[48];
  jac_3_1 += 5040*c3p2*c5p1*c7p2*c9p2*ss[46];
  jac_3_1 += 3360*c3p2*c5p1*c7p3*c9p1*ss[44];
  jac_3_1 += 5040*c3p2*c5p2*c7p1*c9p2*ss[44];
  jac_3_1 += 5040*c3p2*c5p2*c7p2*c9p1*ss[42];
  jac_3_1 += 3360*c3p2*c5p3*c7p1*c9p1*ss[40];
  jac_3_1 += 3360*c3p3*c5p1*c7p1*c9p2*ss[42];
  jac_3_1 += 3360*c3p3*c5p1*c7p2*c9p1*ss[40];
  jac_3_1 += 3360*c3p3*c5p2*c7p1*c9p1*ss[38];
  jac_3_1 += 1680*c3p4*c5p1*c7p1*c9p1*ss[36];
  jac_3_1 += 336*c1p1*c3p1*c5p5*ss[32];
  jac_3_1 += 336*c1p1*c3p1*c7p5*ss[42];
  jac_3_1 += 336*c1p1*c3p1*c9p5*ss[52];
  jac_3_1 += 840*c1p1*c3p2*c5p4*ss[30];
  jac_3_1 += 840*c1p1*c3p2*c7p4*ss[38];
  jac_3_1 += 840*c1p1*c3p2*c9p4*ss[46];
  jac_3_1 += 1120*c1p1*c3p3*c5p3*ss[28];
  jac_3_1 += 1120*c1p1*c3p3*c7p3*ss[34];
  jac_3_1 += 1120*c1p1*c3p3*c9p3*ss[40];
  jac_3_1 += 840*c1p1*c3p4*c5p2*ss[26];
  jac_3_1 += 840*c1p1*c3p4*c7p2*ss[30];
  jac_3_1 += 840*c1p1*c3p4*c9p2*ss[34];
  jac_3_1 += 336*c1p1*c3p5*c5p1*ss[24];
  jac_3_1 += 336*c1p1*c3p5*c7p1*ss[26];
  jac_3_1 += 336*c1p1*c3p5*c9p1*ss[28];
  jac_3_1 += 336*c1p1*c5p1*c7p5*ss[44];
  jac_3_1 += 336*c1p1*c5p1*c9p5*ss[54];
  jac_3_1 += 840*c1p1*c5p2*c7p4*ss[42];
  jac_3_1 += 840*c1p1*c5p2*c9p4*ss[50];
  jac_3_1 += 1120*c1p1*c5p3*c7p3*ss[40];
  jac_3_1 += 1120*c1p1*c5p3*c9p3*ss[46];
  jac_3_1 += 840*c1p1*c5p4*c7p2*ss[38];
  jac_3_1 += 840*c1p1*c5p4*c9p2*ss[42];
  jac_3_1 += 336*c1p1*c5p5*c7p1*ss[36];
  jac_3_1 += 336*c1p1*c5p5*c9p1*ss[38];
  jac_3_1 += 336*c1p1*c7p1*c9p5*ss[56];
  jac_3_1 += 840*c1p1*c7p2*c9p4*ss[54];
  jac_3_1 += 1120*c1p1*c7p3*c9p3*ss[52];
  jac_3_1 += 840*c1p1*c7p4*c9p2*ss[50];
  jac_3_1 += 336*c1p1*c7p5*c9p1*ss[48];
  jac_3_1 += 840*c1p2*c3p1*c5p4*ss[28];
  jac_3_1 += 840*c1p2*c3p1*c7p4*ss[36];
  jac_3_1 += 840*c1p2*c3p1*c9p4*ss[44];
  jac_3_1 += 1680*c1p2*c3p2*c5p3*ss[26];
  jac_3_1 += 1680*c1p2*c3p2*c7p3*ss[32];
  jac_3_1 += 1680*c1p2*c3p2*c9p3*ss[38];
  jac_3_1 += 1680*c1p2*c3p3*c5p2*ss[24];
  jac_3_1 += 1680*c1p2*c3p3*c7p2*ss[28];
  jac_3_1 += 1680*c1p2*c3p3*c9p2*ss[32];
  jac_3_1 += 840*c1p2*c3p4*c5p1*ss[22];
  jac_3_1 += 840*c1p2*c3p4*c7p1*ss[24];
  jac_3_1 += 840*c1p2*c3p4*c9p1*ss[26];
  jac_3_1 += 840*c1p2*c5p1*c7p4*ss[38];
  jac_3_1 += 840*c1p2*c5p1*c9p4*ss[46];
  jac_3_1 += 1680*c1p2*c5p2*c7p3*ss[36];
  jac_3_1 += 1680*c1p2*c5p2*c9p3*ss[42];
  jac_3_1 += 1680*c1p2*c5p3*c7p2*ss[34];
  jac_3_1 += 1680*c1p2*c5p3*c9p2*ss[38];
  jac_3_1 += 840*c1p2*c5p4*c7p1*ss[32];
  jac_3_1 += 840*c1p2*c5p4*c9p1*ss[34];
  jac_3_1 += 840*c1p2*c7p1*c9p4*ss[48];
  jac_3_1 += 1680*c1p2*c7p2*c9p3*ss[46];
  jac_3_1 += 1680*c1p2*c7p3*c9p2*ss[44];
  jac_3_1 += 840*c1p2*c7p4*c9p1*ss[42];
  jac_3_1 += 1120*c1p3*c3p1*c5p3*ss[24];
  jac_3_1 += 1120*c1p3*c3p1*c7p3*ss[30];
  jac_3_1 += 1120*c1p3*c3p1*c9p3*ss[36];
  jac_3_1 += 1680*c1p3*c3p2*c5p2*ss[22];
  jac_3_1 += 1680*c1p3*c3p2*c7p2*ss[26];
  jac_3_1 += 1680*c1p3*c3p2*c9p2*ss[30];
  jac_3_1 += 1120*c1p3*c3p3*c5p1*ss[20];
  jac_3_1 += 1120*c1p3*c3p3*c7p1*ss[22];
  jac_3_1 += 1120*c1p3*c3p3*c9p1*ss[24];
  jac_3_1 += 1120*c1p3*c5p1*c7p3*ss[32];
  jac_3_1 += 1120*c1p3*c5p1*c9p3*ss[38];
  jac_3_1 += 1680*c1p3*c5p2*c7p2*ss[30];
  jac_3_1 += 1680*c1p3*c5p2*c9p2*ss[34];
  jac_3_1 += 1120*c1p3*c5p3*c7p1*ss[28];
  jac_3_1 += 1120*c1p3*c5p3*c9p1*ss[30];
  jac_3_1 += 1120*c1p3*c7p1*c9p3*ss[40];
  jac_3_1 += 1680*c1p3*c7p2*c9p2*ss[38];
  jac_3_1 += 1120*c1p3*c7p3*c9p1*ss[36];
  jac_3_1 += 840*c1p4*c3p1*c5p2*ss[20];
  jac_3_1 += 840*c1p4*c3p1*c7p2*ss[24];
  jac_3_1 += 840*c1p4*c3p1*c9p2*ss[28];
  jac_3_1 += 840*c1p4*c3p2*c5p1*ss[18];
  jac_3_1 += 840*c1p4*c3p2*c7p1*ss[20];
  jac_3_1 += 840*c1p4*c3p2*c9p1*ss[22];
  jac_3_1 += 840*c1p4*c5p1*c7p2*ss[26];
  jac_3_1 += 840*c1p4*c5p1*c9p2*ss[30];
  jac_3_1 += 840*c1p4*c5p2*c7p1*ss[24];
  jac_3_1 += 840*c1p4*c5p2*c9p1*ss[26];
  jac_3_1 += 840*c1p4*c7p1*c9p2*ss[32];
  jac_3_1 += 840*c1p4*c7p2*c9p1*ss[30];
  jac_3_1 += 336*c1p5*c3p1*c5p1*ss[16];
  jac_3_1 += 336*c1p5*c3p1*c7p1*ss[18];
  jac_3_1 += 336*c1p5*c3p1*c9p1*ss[20];
  jac_3_1 += 336*c1p5*c5p1*c7p1*ss[20];
  jac_3_1 += 336*c1p5*c5p1*c9p1*ss[22];
  jac_3_1 += 336*c1p5*c7p1*c9p1*ss[24];
  jac_3_1 += 336*c3p1*c5p1*c7p5*ss[46];
  jac_3_1 += 336*c3p1*c5p1*c9p5*ss[56];
  jac_3_1 += 840*c3p1*c5p2*c7p4*ss[44];
  jac_3_1 += 840*c3p1*c5p2*c9p4*ss[52];
  jac_3_1 += 1120*c3p1*c5p3*c7p3*ss[42];
  jac_3_1 += 1120*c3p1*c5p3*c9p3*ss[48];
  jac_3_1 += 840*c3p1*c5p4*c7p2*ss[40];
  jac_3_1 += 840*c3p1*c5p4*c9p2*ss[44];
  jac_3_1 += 336*c3p1*c5p5*c7p1*ss[38];
  jac_3_1 += 336*c3p1*c5p5*c9p1*ss[40];
  jac_3_1 += 336*c3p1*c7p1*c9p5*ss[58];
  jac_3_1 += 840*c3p1*c7p2*c9p4*ss[56];
  jac_3_1 += 1120*c3p1*c7p3*c9p3*ss[54];
  jac_3_1 += 840*c3p1*c7p4*c9p2*ss[52];
  jac_3_1 += 336*c3p1*c7p5*c9p1*ss[50];
  jac_3_1 += 840*c3p2*c5p1*c7p4*ss[42];
  jac_3_1 += 840*c3p2*c5p1*c9p4*ss[50];
  jac_3_1 += 1680*c3p2*c5p2*c7p3*ss[40];
  jac_3_1 += 1680*c3p2*c5p2*c9p3*ss[46];
  jac_3_1 += 1680*c3p2*c5p3*c7p2*ss[38];
  jac_3_1 += 1680*c3p2*c5p3*c9p2*ss[42];
  jac_3_1 += 840*c3p2*c5p4*c7p1*ss[36];
  jac_3_1 += 840*c3p2*c5p4*c9p1*ss[38];
  jac_3_1 += 840*c3p2*c7p1*c9p4*ss[52];
  jac_3_1 += 1680*c3p2*c7p2*c9p3*ss[50];
  jac_3_1 += 1680*c3p2*c7p3*c9p2*ss[48];
  jac_3_1 += 840*c3p2*c7p4*c9p1*ss[46];
  jac_3_1 += 1120*c3p3*c5p1*c7p3*ss[38];
  jac_3_1 += 1120*c3p3*c5p1*c9p3*ss[44];
  jac_3_1 += 1680*c3p3*c5p2*c7p2*ss[36];
  jac_3_1 += 1680*c3p3*c5p2*c9p2*ss[40];
  jac_3_1 += 1120*c3p3*c5p3*c7p1*ss[34];
  jac_3_1 += 1120*c3p3*c5p3*c9p1*ss[36];
  jac_3_1 += 1120*c3p3*c7p1*c9p3*ss[46];
  jac_3_1 += 1680*c3p3*c7p2*c9p2*ss[44];
  jac_3_1 += 1120*c3p3*c7p3*c9p1*ss[42];
  jac_3_1 += 840*c3p4*c5p1*c7p2*ss[34];
  jac_3_1 += 840*c3p4*c5p1*c9p2*ss[38];
  jac_3_1 += 840*c3p4*c5p2*c7p1*ss[32];
  jac_3_1 += 840*c3p4*c5p2*c9p1*ss[34];
  jac_3_1 += 840*c3p4*c7p1*c9p2*ss[40];
  jac_3_1 += 840*c3p4*c7p2*c9p1*ss[38];
  jac_3_1 += 336*c3p5*c5p1*c7p1*ss[30];
  jac_3_1 += 336*c3p5*c5p1*c9p1*ss[32];
  jac_3_1 += 336*c3p5*c7p1*c9p1*ss[34];
  jac_3_1 += 336*c5p1*c7p1*c9p5*ss[60];
  jac_3_1 += 840*c5p1*c7p2*c9p4*ss[58];
  jac_3_1 += 1120*c5p1*c7p3*c9p3*ss[56];
  jac_3_1 += 840*c5p1*c7p4*c9p2*ss[54];
  jac_3_1 += 336*c5p1*c7p5*c9p1*ss[52];
  jac_3_1 += 840*c5p2*c7p1*c9p4*ss[56];
  jac_3_1 += 1680*c5p2*c7p2*c9p3*ss[54];
  jac_3_1 += 1680*c5p2*c7p3*c9p2*ss[52];
  jac_3_1 += 840*c5p2*c7p4*c9p1*ss[50];
  jac_3_1 += 1120*c5p3*c7p1*c9p3*ss[52];
  jac_3_1 += 1680*c5p3*c7p2*c9p2*ss[50];
  jac_3_1 += 1120*c5p3*c7p3*c9p1*ss[48];
  jac_3_1 += 840*c5p4*c7p1*c9p2*ss[48];
  jac_3_1 += 840*c5p4*c7p2*c9p1*ss[46];
  jac_3_1 += 336*c5p5*c7p1*c9p1*ss[44];
  jac_3_1 += 56*c1p1*c3p6*ss[22];
  jac_3_1 += 56*c1p1*c5p6*ss[34];
  jac_3_1 += 56*c1p1*c7p6*ss[46];
  jac_3_1 += 56*c1p1*c9p6*ss[58];
  jac_3_1 += 168*c1p2*c3p5*ss[20];
  jac_3_1 += 168*c1p2*c5p5*ss[30];
  jac_3_1 += 168*c1p2*c7p5*ss[40];
  jac_3_1 += 168*c1p2*c9p5*ss[50];
  jac_3_1 += 280*c1p3*c3p4*ss[18];
  jac_3_1 += 280*c1p3*c5p4*ss[26];
  jac_3_1 += 280*c1p3*c7p4*ss[34];
  jac_3_1 += 280*c1p3*c9p4*ss[42];
  jac_3_1 += 280*c1p4*c3p3*ss[16];
  jac_3_1 += 280*c1p4*c5p3*ss[22];
  jac_3_1 += 280*c1p4*c7p3*ss[28];
  jac_3_1 += 280*c1p4*c9p3*ss[34];
  jac_3_1 += 168*c1p5*c3p2*ss[14];
  jac_3_1 += 168*c1p5*c5p2*ss[18];
  jac_3_1 += 168*c1p5*c7p2*ss[22];
  jac_3_1 += 168*c1p5*c9p2*ss[26];
  jac_3_1 += 56*c1p6*c3p1*ss[12];
  jac_3_1 += 56*c1p6*c5p1*ss[14];
  jac_3_1 += 56*c1p6*c7p1*ss[16];
  jac_3_1 += 56*c1p6*c9p1*ss[18];
  jac_3_1 += 56*c3p1*c5p6*ss[36];
  jac_3_1 += 56*c3p1*c7p6*ss[48];
  jac_3_1 += 56*c3p1*c9p6*ss[60];
  jac_3_1 += 168*c3p2*c5p5*ss[34];
  jac_3_1 += 168*c3p2*c7p5*ss[44];
  jac_3_1 += 168*c3p2*c9p5*ss[54];
  jac_3_1 += 280*c3p3*c5p4*ss[32];
  jac_3_1 += 280*c3p3*c7p4*ss[40];
  jac_3_1 += 280*c3p3*c9p4*ss[48];
  jac_3_1 += 280*c3p4*c5p3*ss[30];
  jac_3_1 += 280*c3p4*c7p3*ss[36];
  jac_3_1 += 280*c3p4*c9p3*ss[42];
  jac_3_1 += 168*c3p5*c5p2*ss[28];
  jac_3_1 += 168*c3p5*c7p2*ss[32];
  jac_3_1 += 168*c3p5*c9p2*ss[36];
  jac_3_1 += 56*c3p6*c5p1*ss[26];
  jac_3_1 += 56*c3p6*c7p1*ss[28];
  jac_3_1 += 56*c3p6*c9p1*ss[30];
  jac_3_1 += 56*c5p1*c7p6*ss[50];
  jac_3_1 += 56*c5p1*c9p6*ss[62];
  jac_3_1 += 168*c5p2*c7p5*ss[48];
  jac_3_1 += 168*c5p2*c9p5*ss[58];
  jac_3_1 += 280*c5p3*c7p4*ss[46];
  jac_3_1 += 280*c5p3*c9p4*ss[54];
  jac_3_1 += 280*c5p4*c7p3*ss[44];
  jac_3_1 += 280*c5p4*c9p3*ss[50];
  jac_3_1 += 168*c5p5*c7p2*ss[42];
  jac_3_1 += 168*c5p5*c9p2*ss[46];
  jac_3_1 += 56*c5p6*c7p1*ss[40];
  jac_3_1 += 56*c5p6*c9p1*ss[42];
  jac_3_1 += 56*c7p1*c9p6*ss[64];
  jac_3_1 += 168*c7p2*c9p5*ss[62];
  jac_3_1 += 280*c7p3*c9p4*ss[60];
  jac_3_1 += 280*c7p4*c9p3*ss[58];
  jac_3_1 += 168*c7p5*c9p2*ss[56];
  jac_3_1 += 56*c7p6*c9p1*ss[54];
  jac_3_1 += 8*c1p7*ss[10];
  jac_3_1 += 8*c3p7*ss[24];
  jac_3_1 += 8*c5p7*ss[38];
  jac_3_1 += 8*c7p7*ss[52];
  jac_3_1 += 8*c9p7*ss[66];
  TACC jac_3_2 = convert<TACC>(6720*c1p1*c3p1*c5p1*c7p1*c9p3*ss[48]);
  jac_3_2 += 10080*c1p1*c3p1*c5p1*c7p2*c9p2*ss[46];
  jac_3_2 += 6720*c1p1*c3p1*c5p1*c7p3*c9p1*ss[44];
  jac_3_2 += 10080*c1p1*c3p1*c5p2*c7p1*c9p2*ss[44];
  jac_3_2 += 10080*c1p1*c3p1*c5p2*c7p2*c9p1*ss[42];
  jac_3_2 += 6720*c1p1*c3p1*c5p3*c7p1*c9p1*ss[40];
  jac_3_2 += 10080*c1p1*c3p2*c5p1*c7p1*c9p2*ss[42];
  jac_3_2 += 10080*c1p1*c3p2*c5p1*c7p2*c9p1*ss[40];
  jac_3_2 += 10080*c1p1*c3p2*c5p2*c7p1*c9p1*ss[38];
  jac_3_2 += 6720*c1p1*c3p3*c5p1*c7p1*c9p1*ss[36];
  jac_3_2 += 10080*c1p2*c3p1*c5p1*c7p1*c9p2*ss[40];
  jac_3_2 += 10080*c1p2*c3p1*c5p1*c7p2*c9p1*ss[38];
  jac_3_2 += 10080*c1p2*c3p1*c5p2*c7p1*c9p1*ss[36];
  jac_3_2 += 10080*c1p2*c3p2*c5p1*c7p1*c9p1*ss[34];
  jac_3_2 += 6720*c1p3*c3p1*c5p1*c7p1*c9p1*ss[32];
  jac_3_2 += 1680*c1p1*c3p1*c5p1*c7p4*ss[42];
  jac_3_2 += 1680*c1p1*c3p1*c5p1*c9p4*ss[50];
  jac_3_2 += 3360*c1p1*c3p1*c5p2*c7p3*ss[40];
  jac_3_2 += 3360*c1p1*c3p1*c5p2*c9p3*ss[46];
  jac_3_2 += 3360*c1p1*c3p1*c5p3*c7p2*ss[38];
  jac_3_2 += 3360*c1p1*c3p1*c5p3*c9p2*ss[42];
  jac_3_2 += 1680*c1p1*c3p1*c5p4*c7p1*ss[36];
  jac_3_2 += 1680*c1p1*c3p1*c5p4*c9p1*ss[38];
  jac_3_2 += 1680*c1p1*c3p1*c7p1*c9p4*ss[52];
  jac_3_2 += 3360*c1p1*c3p1*c7p2*c9p3*ss[50];
  jac_3_2 += 3360*c1p1*c3p1*c7p3*c9p2*ss[48];
  jac_3_2 += 1680*c1p1*c3p1*c7p4*c9p1*ss[46];
  jac_3_2 += 3360*c1p1*c3p2*c5p1*c7p3*ss[38];
  jac_3_2 += 3360*c1p1*c3p2*c5p1*c9p3*ss[44];
  jac_3_2 += 5040*c1p1*c3p2*c5p2*c7p2*ss[36];
  jac_3_2 += 5040*c1p1*c3p2*c5p2*c9p2*ss[40];
  jac_3_2 += 3360*c1p1*c3p2*c5p3*c7p1*ss[34];
  jac_3_2 += 3360*c1p1*c3p2*c5p3*c9p1*ss[36];
  jac_3_2 += 3360*c1p1*c3p2*c7p1*c9p3*ss[46];
  jac_3_2 += 5040*c1p1*c3p2*c7p2*c9p2*ss[44];
  jac_3_2 += 3360*c1p1*c3p2*c7p3*c9p1*ss[42];
  jac_3_2 += 3360*c1p1*c3p3*c5p1*c7p2*ss[34];
  jac_3_2 += 3360*c1p1*c3p3*c5p1*c9p2*ss[38];
  jac_3_2 += 3360*c1p1*c3p3*c5p2*c7p1*ss[32];
  jac_3_2 += 3360*c1p1*c3p3*c5p2*c9p1*ss[34];
  jac_3_2 += 3360*c1p1*c3p3*c7p1*c9p2*ss[40];
  jac_3_2 += 3360*c1p1*c3p3*c7p2*c9p1*ss[38];
  jac_3_2 += 1680*c1p1*c3p4*c5p1*c7p1*ss[30];
  jac_3_2 += 1680*c1p1*c3p4*c5p1*c9p1*ss[32];
  jac_3_2 += 1680*c1p1*c3p4*c7p1*c9p1*ss[34];
  jac_3_2 += 1680*c1p1*c5p1*c7p1*c9p4*ss[54];
  jac_3_2 += 3360*c1p1*c5p1*c7p2*c9p3*ss[52];
  jac_3_2 += 3360*c1p1*c5p1*c7p3*c9p2*ss[50];
  jac_3_2 += 1680*c1p1*c5p1*c7p4*c9p1*ss[48];
  jac_3_2 += 3360*c1p1*c5p2*c7p1*c9p3*ss[50];
  jac_3_2 += 5040*c1p1*c5p2*c7p2*c9p2*ss[48];
  jac_3_2 += 3360*c1p1*c5p2*c7p3*c9p1*ss[46];
  jac_3_2 += 3360*c1p1*c5p3*c7p1*c9p2*ss[46];
  jac_3_2 += 3360*c1p1*c5p3*c7p2*c9p1*ss[44];
  jac_3_2 += 1680*c1p1*c5p4*c7p1*c9p1*ss[42];
  jac_3_2 += 3360*c1p2*c3p1*c5p1*c7p3*ss[36];
  jac_3_2 += 3360*c1p2*c3p1*c5p1*c9p3*ss[42];
  jac_3_2 += 5040*c1p2*c3p1*c5p2*c7p2*ss[34];
  jac_3_2 += 5040*c1p2*c3p1*c5p2*c9p2*ss[38];
  jac_3_2 += 3360*c1p2*c3p1*c5p3*c7p1*ss[32];
  jac_3_2 += 3360*c1p2*c3p1*c5p3*c9p1*ss[34];
  jac_3_2 += 3360*c1p2*c3p1*c7p1*c9p3*ss[44];
  jac_3_2 += 5040*c1p2*c3p1*c7p2*c9p2*ss[42];
  jac_3_2 += 3360*c1p2*c3p1*c7p3*c9p1*ss[40];
  jac_3_2 += 5040*c1p2*c3p2*c5p1*c7p2*ss[32];
  jac_3_2 += 5040*c1p2*c3p2*c5p1*c9p2*ss[36];
  jac_3_2 += 5040*c1p2*c3p2*c5p2*c7p1*ss[30];
  jac_3_2 += 5040*c1p2*c3p2*c5p2*c9p1*ss[32];
  jac_3_2 += 5040*c1p2*c3p2*c7p1*c9p2*ss[38];
  jac_3_2 += 5040*c1p2*c3p2*c7p2*c9p1*ss[36];
  jac_3_2 += 3360*c1p2*c3p3*c5p1*c7p1*ss[28];
  jac_3_2 += 3360*c1p2*c3p3*c5p1*c9p1*ss[30];
  jac_3_2 += 3360*c1p2*c3p3*c7p1*c9p1*ss[32];
  jac_3_2 += 3360*c1p2*c5p1*c7p1*c9p3*ss[46];
  jac_3_2 += 5040*c1p2*c5p1*c7p2*c9p2*ss[44];
  jac_3_2 += 3360*c1p2*c5p1*c7p3*c9p1*ss[42];
  jac_3_2 += 5040*c1p2*c5p2*c7p1*c9p2*ss[42];
  jac_3_2 += 5040*c1p2*c5p2*c7p2*c9p1*ss[40];
  jac_3_2 += 3360*c1p2*c5p3*c7p1*c9p1*ss[38];
  jac_3_2 += 3360*c1p3*c3p1*c5p1*c7p2*ss[30];
  jac_3_2 += 3360*c1p3*c3p1*c5p1*c9p2*ss[34];
  jac_3_2 += 3360*c1p3*c3p1*c5p2*c7p1*ss[28];
  jac_3_2 += 3360*c1p3*c3p1*c5p2*c9p1*ss[30];
  jac_3_2 += 3360*c1p3*c3p1*c7p1*c9p2*ss[36];
  jac_3_2 += 3360*c1p3*c3p1*c7p2*c9p1*ss[34];
  jac_3_2 += 3360*c1p3*c3p2*c5p1*c7p1*ss[26];
  jac_3_2 += 3360*c1p3*c3p2*c5p1*c9p1*ss[28];
  jac_3_2 += 3360*c1p3*c3p2*c7p1*c9p1*ss[30];
  jac_3_2 += 3360*c1p3*c5p1*c7p1*c9p2*ss[38];
  jac_3_2 += 3360*c1p3*c5p1*c7p2*c9p1*ss[36];
  jac_3_2 += 3360*c1p3*c5p2*c7p1*c9p1*ss[34];
  jac_3_2 += 1680*c1p4*c3p1*c5p1*c7p1*ss[24];
  jac_3_2 += 1680*c1p4*c3p1*c5p1*c9p1*ss[26];
  jac_3_2 += 1680*c1p4*c3p1*c7p1*c9p1*ss[28];
  jac_3_2 += 1680*c1p4*c5p1*c7p1*c9p1*ss[30];
  jac_3_2 += 1680*c3p1*c5p1*c7p1*c9p4*ss[56];
  jac_3_2 += 3360*c3p1*c5p1*c7p2*c9p3*ss[54];
  jac_3_2 += 3360*c3p1*c5p1*c7p3*c9p2*ss[52];
  jac_3_2 += 1680*c3p1*c5p1*c7p4*c9p1*ss[50];
  jac_3_2 += 3360*c3p1*c5p2*c7p1*c9p3*ss[52];
  jac_3_2 += 5040*c3p1*c5p2*c7p2*c9p2*ss[50];
  jac_3_2 += 3360*c3p1*c5p2*c7p3*c9p1*ss[48];
  jac_3_2 += 3360*c3p1*c5p3*c7p1*c9p2*ss[48];
  jac_3_2 += 3360*c3p1*c5p3*c7p2*c9p1*ss[46];
  jac_3_2 += 1680*c3p1*c5p4*c7p1*c9p1*ss[44];
  jac_3_2 += 3360*c3p2*c5p1*c7p1*c9p3*ss[50];
  jac_3_2 += 5040*c3p2*c5p1*c7p2*c9p2*ss[48];
  jac_3_2 += 3360*c3p2*c5p1*c7p3*c9p1*ss[46];
  jac_3_2 += 5040*c3p2*c5p2*c7p1*c9p2*ss[46];
  jac_3_2 += 5040*c3p2*c5p2*c7p2*c9p1*ss[44];
  jac_3_2 += 3360*c3p2*c5p3*c7p1*c9p1*ss[42];
  jac_3_2 += 3360*c3p3*c5p1*c7p1*c9p2*ss[44];
  jac_3_2 += 3360*c3p3*c5p1*c7p2*c9p1*ss[42];
  jac_3_2 += 3360*c3p3*c5p2*c7p1*c9p1*ss[40];
  jac_3_2 += 1680*c3p4*c5p1*c7p1*c9p1*ss[38];
  jac_3_2 += 336*c1p1*c3p1*c5p5*ss[34];
  jac_3_2 += 336*c1p1*c3p1*c7p5*ss[44];
  jac_3_2 += 336*c1p1*c3p1*c9p5*ss[54];
  jac_3_2 += 840*c1p1*c3p2*c5p4*ss[32];
  jac_3_2 += 840*c1p1*c3p2*c7p4*ss[40];
  jac_3_2 += 840*c1p1*c3p2*c9p4*ss[48];
  jac_3_2 += 1120*c1p1*c3p3*c5p3*ss[30];
  jac_3_2 += 1120*c1p1*c3p3*c7p3*ss[36];
  jac_3_2 += 1120*c1p1*c3p3*c9p3*ss[42];
  jac_3_2 += 840*c1p1*c3p4*c5p2*ss[28];
  jac_3_2 += 840*c1p1*c3p4*c7p2*ss[32];
  jac_3_2 += 840*c1p1*c3p4*c9p2*ss[36];
  jac_3_2 += 336*c1p1*c3p5*c5p1*ss[26];
  jac_3_2 += 336*c1p1*c3p5*c7p1*ss[28];
  jac_3_2 += 336*c1p1*c3p5*c9p1*ss[30];
  jac_3_2 += 336*c1p1*c5p1*c7p5*ss[46];
  jac_3_2 += 336*c1p1*c5p1*c9p5*ss[56];
  jac_3_2 += 840*c1p1*c5p2*c7p4*ss[44];
  jac_3_2 += 840*c1p1*c5p2*c9p4*ss[52];
  jac_3_2 += 1120*c1p1*c5p3*c7p3*ss[42];
  jac_3_2 += 1120*c1p1*c5p3*c9p3*ss[48];
  jac_3_2 += 840*c1p1*c5p4*c7p2*ss[40];
  jac_3_2 += 840*c1p1*c5p4*c9p2*ss[44];
  jac_3_2 += 336*c1p1*c5p5*c7p1*ss[38];
  jac_3_2 += 336*c1p1*c5p5*c9p1*ss[40];
  jac_3_2 += 336*c1p1*c7p1*c9p5*ss[58];
  jac_3_2 += 840*c1p1*c7p2*c9p4*ss[56];
  jac_3_2 += 1120*c1p1*c7p3*c9p3*ss[54];
  jac_3_2 += 840*c1p1*c7p4*c9p2*ss[52];
  jac_3_2 += 336*c1p1*c7p5*c9p1*ss[50];
  jac_3_2 += 840*c1p2*c3p1*c5p4*ss[30];
  jac_3_2 += 840*c1p2*c3p1*c7p4*ss[38];
  jac_3_2 += 840*c1p2*c3p1*c9p4*ss[46];
  jac_3_2 += 1680*c1p2*c3p2*c5p3*ss[28];
  jac_3_2 += 1680*c1p2*c3p2*c7p3*ss[34];
  jac_3_2 += 1680*c1p2*c3p2*c9p3*ss[40];
  jac_3_2 += 1680*c1p2*c3p3*c5p2*ss[26];
  jac_3_2 += 1680*c1p2*c3p3*c7p2*ss[30];
  jac_3_2 += 1680*c1p2*c3p3*c9p2*ss[34];
  jac_3_2 += 840*c1p2*c3p4*c5p1*ss[24];
  jac_3_2 += 840*c1p2*c3p4*c7p1*ss[26];
  jac_3_2 += 840*c1p2*c3p4*c9p1*ss[28];
  jac_3_2 += 840*c1p2*c5p1*c7p4*ss[40];
  jac_3_2 += 840*c1p2*c5p1*c9p4*ss[48];
  jac_3_2 += 1680*c1p2*c5p2*c7p3*ss[38];
  jac_3_2 += 1680*c1p2*c5p2*c9p3*ss[44];
  jac_3_2 += 1680*c1p2*c5p3*c7p2*ss[36];
  jac_3_2 += 1680*c1p2*c5p3*c9p2*ss[40];
  jac_3_2 += 840*c1p2*c5p4*c7p1*ss[34];
  jac_3_2 += 840*c1p2*c5p4*c9p1*ss[36];
  jac_3_2 += 840*c1p2*c7p1*c9p4*ss[50];
  jac_3_2 += 1680*c1p2*c7p2*c9p3*ss[48];
  jac_3_2 += 1680*c1p2*c7p3*c9p2*ss[46];
  jac_3_2 += 840*c1p2*c7p4*c9p1*ss[44];
  jac_3_2 += 1120*c1p3*c3p1*c5p3*ss[26];
  jac_3_2 += 1120*c1p3*c3p1*c7p3*ss[32];
  jac_3_2 += 1120*c1p3*c3p1*c9p3*ss[38];
  jac_3_2 += 1680*c1p3*c3p2*c5p2*ss[24];
  jac_3_2 += 1680*c1p3*c3p2*c7p2*ss[28];
  jac_3_2 += 1680*c1p3*c3p2*c9p2*ss[32];
  jac_3_2 += 1120*c1p3*c3p3*c5p1*ss[22];
  jac_3_2 += 1120*c1p3*c3p3*c7p1*ss[24];
  jac_3_2 += 1120*c1p3*c3p3*c9p1*ss[26];
  jac_3_2 += 1120*c1p3*c5p1*c7p3*ss[34];
  jac_3_2 += 1120*c1p3*c5p1*c9p3*ss[40];
  jac_3_2 += 1680*c1p3*c5p2*c7p2*ss[32];
  jac_3_2 += 1680*c1p3*c5p2*c9p2*ss[36];
  jac_3_2 += 1120*c1p3*c5p3*c7p1*ss[30];
  jac_3_2 += 1120*c1p3*c5p3*c9p1*ss[32];
  jac_3_2 += 1120*c1p3*c7p1*c9p3*ss[42];
  jac_3_2 += 1680*c1p3*c7p2*c9p2*ss[40];
  jac_3_2 += 1120*c1p3*c7p3*c9p1*ss[38];
  jac_3_2 += 840*c1p4*c3p1*c5p2*ss[22];
  jac_3_2 += 840*c1p4*c3p1*c7p2*ss[26];
  jac_3_2 += 840*c1p4*c3p1*c9p2*ss[30];
  jac_3_2 += 840*c1p4*c3p2*c5p1*ss[20];
  jac_3_2 += 840*c1p4*c3p2*c7p1*ss[22];
  jac_3_2 += 840*c1p4*c3p2*c9p1*ss[24];
  jac_3_2 += 840*c1p4*c5p1*c7p2*ss[28];
  jac_3_2 += 840*c1p4*c5p1*c9p2*ss[32];
  jac_3_2 += 840*c1p4*c5p2*c7p1*ss[26];
  jac_3_2 += 840*c1p4*c5p2*c9p1*ss[28];
  jac_3_2 += 840*c1p4*c7p1*c9p2*ss[34];
  jac_3_2 += 840*c1p4*c7p2*c9p1*ss[32];
  jac_3_2 += 336*c1p5*c3p1*c5p1*ss[18];
  jac_3_2 += 336*c1p5*c3p1*c7p1*ss[20];
  jac_3_2 += 336*c1p5*c3p1*c9p1*ss[22];
  jac_3_2 += 336*c1p5*c5p1*c7p1*ss[22];
  jac_3_2 += 336*c1p5*c5p1*c9p1*ss[24];
  jac_3_2 += 336*c1p5*c7p1*c9p1*ss[26];
  jac_3_2 += 336*c3p1*c5p1*c7p5*ss[48];
  jac_3_2 += 336*c3p1*c5p1*c9p5*ss[58];
  jac_3_2 += 840*c3p1*c5p2*c7p4*ss[46];
  jac_3_2 += 840*c3p1*c5p2*c9p4*ss[54];
  jac_3_2 += 1120*c3p1*c5p3*c7p3*ss[44];
  jac_3_2 += 1120*c3p1*c5p3*c9p3*ss[50];
  jac_3_2 += 840*c3p1*c5p4*c7p2*ss[42];
  jac_3_2 += 840*c3p1*c5p4*c9p2*ss[46];
  jac_3_2 += 336*c3p1*c5p5*c7p1*ss[40];
  jac_3_2 += 336*c3p1*c5p5*c9p1*ss[42];
  jac_3_2 += 336*c3p1*c7p1*c9p5*ss[60];
  jac_3_2 += 840*c3p1*c7p2*c9p4*ss[58];
  jac_3_2 += 1120*c3p1*c7p3*c9p3*ss[56];
  jac_3_2 += 840*c3p1*c7p4*c9p2*ss[54];
  jac_3_2 += 336*c3p1*c7p5*c9p1*ss[52];
  jac_3_2 += 840*c3p2*c5p1*c7p4*ss[44];
  jac_3_2 += 840*c3p2*c5p1*c9p4*ss[52];
  jac_3_2 += 1680*c3p2*c5p2*c7p3*ss[42];
  jac_3_2 += 1680*c3p2*c5p2*c9p3*ss[48];
  jac_3_2 += 1680*c3p2*c5p3*c7p2*ss[40];
  jac_3_2 += 1680*c3p2*c5p3*c9p2*ss[44];
  jac_3_2 += 840*c3p2*c5p4*c7p1*ss[38];
  jac_3_2 += 840*c3p2*c5p4*c9p1*ss[40];
  jac_3_2 += 840*c3p2*c7p1*c9p4*ss[54];
  jac_3_2 += 1680*c3p2*c7p2*c9p3*ss[52];
  jac_3_2 += 1680*c3p2*c7p3*c9p2*ss[50];
  jac_3_2 += 840*c3p2*c7p4*c9p1*ss[48];
  jac_3_2 += 1120*c3p3*c5p1*c7p3*ss[40];
  jac_3_2 += 1120*c3p3*c5p1*c9p3*ss[46];
  jac_3_2 += 1680*c3p3*c5p2*c7p2*ss[38];
  jac_3_2 += 1680*c3p3*c5p2*c9p2*ss[42];
  jac_3_2 += 1120*c3p3*c5p3*c7p1*ss[36];
  jac_3_2 += 1120*c3p3*c5p3*c9p1*ss[38];
  jac_3_2 += 1120*c3p3*c7p1*c9p3*ss[48];
  jac_3_2 += 1680*c3p3*c7p2*c9p2*ss[46];
  jac_3_2 += 1120*c3p3*c7p3*c9p1*ss[44];
  jac_3_2 += 840*c3p4*c5p1*c7p2*ss[36];
  jac_3_2 += 840*c3p4*c5p1*c9p2*ss[40];
  jac_3_2 += 840*c3p4*c5p2*c7p1*ss[34];
  jac_3_2 += 840*c3p4*c5p2*c9p1*ss[36];
  jac_3_2 += 840*c3p4*c7p1*c9p2*ss[42];
  jac_3_2 += 840*c3p4*c7p2*c9p1*ss[40];
  jac_3_2 += 336*c3p5*c5p1*c7p1*ss[32];
  jac_3_2 += 336*c3p5*c5p1*c9p1*ss[34];
  jac_3_2 += 336*c3p5*c7p1*c9p1*ss[36];
  jac_3_2 += 336*c5p1*c7p1*c9p5*ss[62];
  jac_3_2 += 840*c5p1*c7p2*c9p4*ss[60];
  jac_3_2 += 1120*c5p1*c7p3*c9p3*ss[58];
  jac_3_2 += 840*c5p1*c7p4*c9p2*ss[56];
  jac_3_2 += 336*c5p1*c7p5*c9p1*ss[54];
  jac_3_2 += 840*c5p2*c7p1*c9p4*ss[58];
  jac_3_2 += 1680*c5p2*c7p2*c9p3*ss[56];
  jac_3_2 += 1680*c5p2*c7p3*c9p2*ss[54];
  jac_3_2 += 840*c5p2*c7p4*c9p1*ss[52];
  jac_3_2 += 1120*c5p3*c7p1*c9p3*ss[54];
  jac_3_2 += 1680*c5p3*c7p2*c9p2*ss[52];
  jac_3_2 += 1120*c5p3*c7p3*c9p1*ss[50];
  jac_3_2 += 840*c5p4*c7p1*c9p2*ss[50];
  jac_3_2 += 840*c5p4*c7p2*c9p1*ss[48];
  jac_3_2 += 336*c5p5*c7p1*c9p1*ss[46];
  jac_3_2 += 56*c1p1*c3p6*ss[24];
  jac_3_2 += 56*c1p1*c5p6*ss[36];
  jac_3_2 += 56*c1p1*c7p6*ss[48];
  jac_3_2 += 56*c1p1*c9p6*ss[60];
  jac_3_2 += 168*c1p2*c3p5*ss[22];
  jac_3_2 += 168*c1p2*c5p5*ss[32];
  jac_3_2 += 168*c1p2*c7p5*ss[42];
  jac_3_2 += 168*c1p2*c9p5*ss[52];
  jac_3_2 += 280*c1p3*c3p4*ss[20];
  jac_3_2 += 280*c1p3*c5p4*ss[28];
  jac_3_2 += 280*c1p3*c7p4*ss[36];
  jac_3_2 += 280*c1p3*c9p4*ss[44];
  jac_3_2 += 280*c1p4*c3p3*ss[18];
  jac_3_2 += 280*c1p4*c5p3*ss[24];
  jac_3_2 += 280*c1p4*c7p3*ss[30];
  jac_3_2 += 280*c1p4*c9p3*ss[36];
  jac_3_2 += 168*c1p5*c3p2*ss[16];
  jac_3_2 += 168*c1p5*c5p2*ss[20];
  jac_3_2 += 168*c1p5*c7p2*ss[24];
  jac_3_2 += 168*c1p5*c9p2*ss[28];
  jac_3_2 += 56*c1p6*c3p1*ss[14];
  jac_3_2 += 56*c1p6*c5p1*ss[16];
  jac_3_2 += 56*c1p6*c7p1*ss[18];
  jac_3_2 += 56*c1p6*c9p1*ss[20];
  jac_3_2 += 56*c3p1*c5p6*ss[38];
  jac_3_2 += 56*c3p1*c7p6*ss[50];
  jac_3_2 += 56*c3p1*c9p6*ss[62];
  jac_3_2 += 168*c3p2*c5p5*ss[36];
  jac_3_2 += 168*c3p2*c7p5*ss[46];
  jac_3_2 += 168*c3p2*c9p5*ss[56];
  jac_3_2 += 280*c3p3*c5p4*ss[34];
  jac_3_2 += 280*c3p3*c7p4*ss[42];
  jac_3_2 += 280*c3p3*c9p4*ss[50];
  jac_3_2 += 280*c3p4*c5p3*ss[32];
  jac_3_2 += 280*c3p4*c7p3*ss[38];
  jac_3_2 += 280*c3p4*c9p3*ss[44];
  jac_3_2 += 168*c3p5*c5p2*ss[30];
  jac_3_2 += 168*c3p5*c7p2*ss[34];
  jac_3_2 += 168*c3p5*c9p2*ss[38];
  jac_3_2 += 56*c3p6*c5p1*ss[28];
  jac_3_2 += 56*c3p6*c7p1*ss[30];
  jac_3_2 += 56*c3p6*c9p1*ss[32];
  jac_3_2 += 56*c5p1*c7p6*ss[52];
  jac_3_2 += 56*c5p1*c9p6*ss[64];
  jac_3_2 += 168*c5p2*c7p5*ss[50];
  jac_3_2 += 168*c5p2*c9p5*ss[60];
  jac_3_2 += 280*c5p3*c7p4*ss[48];
  jac_3_2 += 280*c5p3*c9p4*ss[56];
  jac_3_2 += 280*c5p4*c7p3*ss[46];
  jac_3_2 += 280*c5p4*c9p3*ss[52];
  jac_3_2 += 168*c5p5*c7p2*ss[44];
  jac_3_2 += 168*c5p5*c9p2*ss[48];
  jac_3_2 += 56*c5p6*c7p1*ss[42];
  jac_3_2 += 56*c5p6*c9p1*ss[44];
  jac_3_2 += 56*c7p1*c9p6*ss[66];
  jac_3_2 += 168*c7p2*c9p5*ss[64];
  jac_3_2 += 280*c7p3*c9p4*ss[62];
  jac_3_2 += 280*c7p4*c9p3*ss[60];
  jac_3_2 += 168*c7p5*c9p2*ss[58];
  jac_3_2 += 56*c7p6*c9p1*ss[56];
  jac_3_2 += 8*c1p7*ss[12];
  jac_3_2 += 8*c3p7*ss[26];
  jac_3_2 += 8*c5p7*ss[40];
  jac_3_2 += 8*c7p7*ss[54];
  jac_3_2 += 8*c9p7*ss[68];
  TACC jac_3_3 = convert<TACC>(6720*c1p1*c3p1*c5p1*c7p1*c9p3*ss[50]);
  jac_3_3 += 10080*c1p1*c3p1*c5p1*c7p2*c9p2*ss[48];
  jac_3_3 += 6720*c1p1*c3p1*c5p1*c7p3*c9p1*ss[46];
  jac_3_3 += 10080*c1p1*c3p1*c5p2*c7p1*c9p2*ss[46];
  jac_3_3 += 10080*c1p1*c3p1*c5p2*c7p2*c9p1*ss[44];
  jac_3_3 += 6720*c1p1*c3p1*c5p3*c7p1*c9p1*ss[42];
  jac_3_3 += 10080*c1p1*c3p2*c5p1*c7p1*c9p2*ss[44];
  jac_3_3 += 10080*c1p1*c3p2*c5p1*c7p2*c9p1*ss[42];
  jac_3_3 += 10080*c1p1*c3p2*c5p2*c7p1*c9p1*ss[40];
  jac_3_3 += 6720*c1p1*c3p3*c5p1*c7p1*c9p1*ss[38];
  jac_3_3 += 10080*c1p2*c3p1*c5p1*c7p1*c9p2*ss[42];
  jac_3_3 += 10080*c1p2*c3p1*c5p1*c7p2*c9p1*ss[40];
  jac_3_3 += 10080*c1p2*c3p1*c5p2*c7p1*c9p1*ss[38];
  jac_3_3 += 10080*c1p2*c3p2*c5p1*c7p1*c9p1*ss[36];
  jac_3_3 += 6720*c1p3*c3p1*c5p1*c7p1*c9p1*ss[34];
  jac_3_3 += 1680*c1p1*c3p1*c5p1*c7p4*ss[44];
  jac_3_3 += 1680*c1p1*c3p1*c5p1*c9p4*ss[52];
  jac_3_3 += 3360*c1p1*c3p1*c5p2*c7p3*ss[42];
  jac_3_3 += 3360*c1p1*c3p1*c5p2*c9p3*ss[48];
  jac_3_3 += 3360*c1p1*c3p1*c5p3*c7p2*ss[40];
  jac_3_3 += 3360*c1p1*c3p1*c5p3*c9p2*ss[44];
  jac_3_3 += 1680*c1p1*c3p1*c5p4*c7p1*ss[38];
  jac_3_3 += 1680*c1p1*c3p1*c5p4*c9p1*ss[40];
  jac_3_3 += 1680*c1p1*c3p1*c7p1*c9p4*ss[54];
  jac_3_3 += 3360*c1p1*c3p1*c7p2*c9p3*ss[52];
  jac_3_3 += 3360*c1p1*c3p1*c7p3*c9p2*ss[50];
  jac_3_3 += 1680*c1p1*c3p1*c7p4*c9p1*ss[48];
  jac_3_3 += 3360*c1p1*c3p2*c5p1*c7p3*ss[40];
  jac_3_3 += 3360*c1p1*c3p2*c5p1*c9p3*ss[46];
  jac_3_3 += 5040*c1p1*c3p2*c5p2*c7p2*ss[38];
  jac_3_3 += 5040*c1p1*c3p2*c5p2*c9p2*ss[42];
  jac_3_3 += 3360*c1p1*c3p2*c5p3*c7p1*ss[36];
  jac_3_3 += 3360*c1p1*c3p2*c5p3*c9p1*ss[38];
  jac_3_3 += 3360*c1p1*c3p2*c7p1*c9p3*ss[48];
  jac_3_3 += 5040*c1p1*c3p2*c7p2*c9p2*ss[46];
  jac_3_3 += 3360*c1p1*c3p2*c7p3*c9p1*ss[44];
  jac_3_3 += 3360*c1p1*c3p3*c5p1*c7p2*ss[36];
  jac_3_3 += 3360*c1p1*c3p3*c5p1*c9p2*ss[40];
  jac_3_3 += 3360*c1p1*c3p3*c5p2*c7p1*ss[34];
  jac_3_3 += 3360*c1p1*c3p3*c5p2*c9p1*ss[36];
  jac_3_3 += 3360*c1p1*c3p3*c7p1*c9p2*ss[42];
  jac_3_3 += 3360*c1p1*c3p3*c7p2*c9p1*ss[40];
  jac_3_3 += 1680*c1p1*c3p4*c5p1*c7p1*ss[32];
  jac_3_3 += 1680*c1p1*c3p4*c5p1*c9p1*ss[34];
  jac_3_3 += 1680*c1p1*c3p4*c7p1*c9p1*ss[36];
  jac_3_3 += 1680*c1p1*c5p1*c7p1*c9p4*ss[56];
  jac_3_3 += 3360*c1p1*c5p1*c7p2*c9p3*ss[54];
  jac_3_3 += 3360*c1p1*c5p1*c7p3*c9p2*ss[52];
  jac_3_3 += 1680*c1p1*c5p1*c7p4*c9p1*ss[50];
  jac_3_3 += 3360*c1p1*c5p2*c7p1*c9p3*ss[52];
  jac_3_3 += 5040*c1p1*c5p2*c7p2*c9p2*ss[50];
  jac_3_3 += 3360*c1p1*c5p2*c7p3*c9p1*ss[48];
  jac_3_3 += 3360*c1p1*c5p3*c7p1*c9p2*ss[48];
  jac_3_3 += 3360*c1p1*c5p3*c7p2*c9p1*ss[46];
  jac_3_3 += 1680*c1p1*c5p4*c7p1*c9p1*ss[44];
  jac_3_3 += 3360*c1p2*c3p1*c5p1*c7p3*ss[38];
  jac_3_3 += 3360*c1p2*c3p1*c5p1*c9p3*ss[44];
  jac_3_3 += 5040*c1p2*c3p1*c5p2*c7p2*ss[36];
  jac_3_3 += 5040*c1p2*c3p1*c5p2*c9p2*ss[40];
  jac_3_3 += 3360*c1p2*c3p1*c5p3*c7p1*ss[34];
  jac_3_3 += 3360*c1p2*c3p1*c5p3*c9p1*ss[36];
  jac_3_3 += 3360*c1p2*c3p1*c7p1*c9p3*ss[46];
  jac_3_3 += 5040*c1p2*c3p1*c7p2*c9p2*ss[44];
  jac_3_3 += 3360*c1p2*c3p1*c7p3*c9p1*ss[42];
  jac_3_3 += 5040*c1p2*c3p2*c5p1*c7p2*ss[34];
  jac_3_3 += 5040*c1p2*c3p2*c5p1*c9p2*ss[38];
  jac_3_3 += 5040*c1p2*c3p2*c5p2*c7p1*ss[32];
  jac_3_3 += 5040*c1p2*c3p2*c5p2*c9p1*ss[34];
  jac_3_3 += 5040*c1p2*c3p2*c7p1*c9p2*ss[40];
  jac_3_3 += 5040*c1p2*c3p2*c7p2*c9p1*ss[38];
  jac_3_3 += 3360*c1p2*c3p3*c5p1*c7p1*ss[30];
  jac_3_3 += 3360*c1p2*c3p3*c5p1*c9p1*ss[32];
  jac_3_3 += 3360*c1p2*c3p3*c7p1*c9p1*ss[34];
  jac_3_3 += 3360*c1p2*c5p1*c7p1*c9p3*ss[48];
  jac_3_3 += 5040*c1p2*c5p1*c7p2*c9p2*ss[46];
  jac_3_3 += 3360*c1p2*c5p1*c7p3*c9p1*ss[44];
  jac_3_3 += 5040*c1p2*c5p2*c7p1*c9p2*ss[44];
  jac_3_3 += 5040*c1p2*c5p2*c7p2*c9p1*ss[42];
  jac_3_3 += 3360*c1p2*c5p3*c7p1*c9p1*ss[40];
  jac_3_3 += 3360*c1p3*c3p1*c5p1*c7p2*ss[32];
  jac_3_3 += 3360*c1p3*c3p1*c5p1*c9p2*ss[36];
  jac_3_3 += 3360*c1p3*c3p1*c5p2*c7p1*ss[30];
  jac_3_3 += 3360*c1p3*c3p1*c5p2*c9p1*ss[32];
  jac_3_3 += 3360*c1p3*c3p1*c7p1*c9p2*ss[38];
  jac_3_3 += 3360*c1p3*c3p1*c7p2*c9p1*ss[36];
  jac_3_3 += 3360*c1p3*c3p2*c5p1*c7p1*ss[28];
  jac_3_3 += 3360*c1p3*c3p2*c5p1*c9p1*ss[30];
  jac_3_3 += 3360*c1p3*c3p2*c7p1*c9p1*ss[32];
  jac_3_3 += 3360*c1p3*c5p1*c7p1*c9p2*ss[40];
  jac_3_3 += 3360*c1p3*c5p1*c7p2*c9p1*ss[38];
  jac_3_3 += 3360*c1p3*c5p2*c7p1*c9p1*ss[36];
  jac_3_3 += 1680*c1p4*c3p1*c5p1*c7p1*ss[26];
  jac_3_3 += 1680*c1p4*c3p1*c5p1*c9p1*ss[28];
  jac_3_3 += 1680*c1p4*c3p1*c7p1*c9p1*ss[30];
  jac_3_3 += 1680*c1p4*c5p1*c7p1*c9p1*ss[32];
  jac_3_3 += 1680*c3p1*c5p1*c7p1*c9p4*ss[58];
  jac_3_3 += 3360*c3p1*c5p1*c7p2*c9p3*ss[56];
  jac_3_3 += 3360*c3p1*c5p1*c7p3*c9p2*ss[54];
  jac_3_3 += 1680*c3p1*c5p1*c7p4*c9p1*ss[52];
  jac_3_3 += 3360*c3p1*c5p2*c7p1*c9p3*ss[54];
  jac_3_3 += 5040*c3p1*c5p2*c7p2*c9p2*ss[52];
  jac_3_3 += 3360*c3p1*c5p2*c7p3*c9p1*ss[50];
  jac_3_3 += 3360*c3p1*c5p3*c7p1*c9p2*ss[50];
  jac_3_3 += 3360*c3p1*c5p3*c7p2*c9p1*ss[48];
  jac_3_3 += 1680*c3p1*c5p4*c7p1*c9p1*ss[46];
  jac_3_3 += 3360*c3p2*c5p1*c7p1*c9p3*ss[52];
  jac_3_3 += 5040*c3p2*c5p1*c7p2*c9p2*ss[50];
  jac_3_3 += 3360*c3p2*c5p1*c7p3*c9p1*ss[48];
  jac_3_3 += 5040*c3p2*c5p2*c7p1*c9p2*ss[48];
  jac_3_3 += 5040*c3p2*c5p2*c7p2*c9p1*ss[46];
  jac_3_3 += 3360*c3p2*c5p3*c7p1*c9p1*ss[44];
  jac_3_3 += 3360*c3p3*c5p1*c7p1*c9p2*ss[46];
  jac_3_3 += 3360*c3p3*c5p1*c7p2*c9p1*ss[44];
  jac_3_3 += 3360*c3p3*c5p2*c7p1*c9p1*ss[42];
  jac_3_3 += 1680*c3p4*c5p1*c7p1*c9p1*ss[40];
  jac_3_3 += 336*c1p1*c3p1*c5p5*ss[36];
  jac_3_3 += 336*c1p1*c3p1*c7p5*ss[46];
  jac_3_3 += 336*c1p1*c3p1*c9p5*ss[56];
  jac_3_3 += 840*c1p1*c3p2*c5p4*ss[34];
  jac_3_3 += 840*c1p1*c3p2*c7p4*ss[42];
  jac_3_3 += 840*c1p1*c3p2*c9p4*ss[50];
  jac_3_3 += 1120*c1p1*c3p3*c5p3*ss[32];
  jac_3_3 += 1120*c1p1*c3p3*c7p3*ss[38];
  jac_3_3 += 1120*c1p1*c3p3*c9p3*ss[44];
  jac_3_3 += 840*c1p1*c3p4*c5p2*ss[30];
  jac_3_3 += 840*c1p1*c3p4*c7p2*ss[34];
  jac_3_3 += 840*c1p1*c3p4*c9p2*ss[38];
  jac_3_3 += 336*c1p1*c3p5*c5p1*ss[28];
  jac_3_3 += 336*c1p1*c3p5*c7p1*ss[30];
  jac_3_3 += 336*c1p1*c3p5*c9p1*ss[32];
  jac_3_3 += 336*c1p1*c5p1*c7p5*ss[48];
  jac_3_3 += 336*c1p1*c5p1*c9p5*ss[58];
  jac_3_3 += 840*c1p1*c5p2*c7p4*ss[46];
  jac_3_3 += 840*c1p1*c5p2*c9p4*ss[54];
  jac_3_3 += 1120*c1p1*c5p3*c7p3*ss[44];
  jac_3_3 += 1120*c1p1*c5p3*c9p3*ss[50];
  jac_3_3 += 840*c1p1*c5p4*c7p2*ss[42];
  jac_3_3 += 840*c1p1*c5p4*c9p2*ss[46];
  jac_3_3 += 336*c1p1*c5p5*c7p1*ss[40];
  jac_3_3 += 336*c1p1*c5p5*c9p1*ss[42];
  jac_3_3 += 336*c1p1*c7p1*c9p5*ss[60];
  jac_3_3 += 840*c1p1*c7p2*c9p4*ss[58];
  jac_3_3 += 1120*c1p1*c7p3*c9p3*ss[56];
  jac_3_3 += 840*c1p1*c7p4*c9p2*ss[54];
  jac_3_3 += 336*c1p1*c7p5*c9p1*ss[52];
  jac_3_3 += 840*c1p2*c3p1*c5p4*ss[32];
  jac_3_3 += 840*c1p2*c3p1*c7p4*ss[40];
  jac_3_3 += 840*c1p2*c3p1*c9p4*ss[48];
  jac_3_3 += 1680*c1p2*c3p2*c5p3*ss[30];
  jac_3_3 += 1680*c1p2*c3p2*c7p3*ss[36];
  jac_3_3 += 1680*c1p2*c3p2*c9p3*ss[42];
  jac_3_3 += 1680*c1p2*c3p3*c5p2*ss[28];
  jac_3_3 += 1680*c1p2*c3p3*c7p2*ss[32];
  jac_3_3 += 1680*c1p2*c3p3*c9p2*ss[36];
  jac_3_3 += 840*c1p2*c3p4*c5p1*ss[26];
  jac_3_3 += 840*c1p2*c3p4*c7p1*ss[28];
  jac_3_3 += 840*c1p2*c3p4*c9p1*ss[30];
  jac_3_3 += 840*c1p2*c5p1*c7p4*ss[42];
  jac_3_3 += 840*c1p2*c5p1*c9p4*ss[50];
  jac_3_3 += 1680*c1p2*c5p2*c7p3*ss[40];
  jac_3_3 += 1680*c1p2*c5p2*c9p3*ss[46];
  jac_3_3 += 1680*c1p2*c5p3*c7p2*ss[38];
  jac_3_3 += 1680*c1p2*c5p3*c9p2*ss[42];
  jac_3_3 += 840*c1p2*c5p4*c7p1*ss[36];
  jac_3_3 += 840*c1p2*c5p4*c9p1*ss[38];
  jac_3_3 += 840*c1p2*c7p1*c9p4*ss[52];
  jac_3_3 += 1680*c1p2*c7p2*c9p3*ss[50];
  jac_3_3 += 1680*c1p2*c7p3*c9p2*ss[48];
  jac_3_3 += 840*c1p2*c7p4*c9p1*ss[46];
  jac_3_3 += 1120*c1p3*c3p1*c5p3*ss[28];
  jac_3_3 += 1120*c1p3*c3p1*c7p3*ss[34];
  jac_3_3 += 1120*c1p3*c3p1*c9p3*ss[40];
  jac_3_3 += 1680*c1p3*c3p2*c5p2*ss[26];
  jac_3_3 += 1680*c1p3*c3p2*c7p2*ss[30];
  jac_3_3 += 1680*c1p3*c3p2*c9p2*ss[34];
  jac_3_3 += 1120*c1p3*c3p3*c5p1*ss[24];
  jac_3_3 += 1120*c1p3*c3p3*c7p1*ss[26];
  jac_3_3 += 1120*c1p3*c3p3*c9p1*ss[28];
  jac_3_3 += 1120*c1p3*c5p1*c7p3*ss[36];
  jac_3_3 += 1120*c1p3*c5p1*c9p3*ss[42];
  jac_3_3 += 1680*c1p3*c5p2*c7p2*ss[34];
  jac_3_3 += 1680*c1p3*c5p2*c9p2*ss[38];
  jac_3_3 += 1120*c1p3*c5p3*c7p1*ss[32];
  jac_3_3 += 1120*c1p3*c5p3*c9p1*ss[34];
  jac_3_3 += 1120*c1p3*c7p1*c9p3*ss[44];
  jac_3_3 += 1680*c1p3*c7p2*c9p2*ss[42];
  jac_3_3 += 1120*c1p3*c7p3*c9p1*ss[40];
  jac_3_3 += 840*c1p4*c3p1*c5p2*ss[24];
  jac_3_3 += 840*c1p4*c3p1*c7p2*ss[28];
  jac_3_3 += 840*c1p4*c3p1*c9p2*ss[32];
  jac_3_3 += 840*c1p4*c3p2*c5p1*ss[22];
  jac_3_3 += 840*c1p4*c3p2*c7p1*ss[24];
  jac_3_3 += 840*c1p4*c3p2*c9p1*ss[26];
  jac_3_3 += 840*c1p4*c5p1*c7p2*ss[30];
  jac_3_3 += 840*c1p4*c5p1*c9p2*ss[34];
  jac_3_3 += 840*c1p4*c5p2*c7p1*ss[28];
  jac_3_3 += 840*c1p4*c5p2*c9p1*ss[30];
  jac_3_3 += 840*c1p4*c7p1*c9p2*ss[36];
  jac_3_3 += 840*c1p4*c7p2*c9p1*ss[34];
  jac_3_3 += 336*c1p5*c3p1*c5p1*ss[20];
  jac_3_3 += 336*c1p5*c3p1*c7p1*ss[22];
  jac_3_3 += 336*c1p5*c3p1*c9p1*ss[24];
  jac_3_3 += 336*c1p5*c5p1*c7p1*ss[24];
  jac_3_3 += 336*c1p5*c5p1*c9p1*ss[26];
  jac_3_3 += 336*c1p5*c7p1*c9p1*ss[28];
  jac_3_3 += 336*c3p1*c5p1*c7p5*ss[50];
  jac_3_3 += 336*c3p1*c5p1*c9p5*ss[60];
  jac_3_3 += 840*c3p1*c5p2*c7p4*ss[48];
  jac_3_3 += 840*c3p1*c5p2*c9p4*ss[56];
  jac_3_3 += 1120*c3p1*c5p3*c7p3*ss[46];
  jac_3_3 += 1120*c3p1*c5p3*c9p3*ss[52];
  jac_3_3 += 840*c3p1*c5p4*c7p2*ss[44];
  jac_3_3 += 840*c3p1*c5p4*c9p2*ss[48];
  jac_3_3 += 336*c3p1*c5p5*c7p1*ss[42];
  jac_3_3 += 336*c3p1*c5p5*c9p1*ss[44];
  jac_3_3 += 336*c3p1*c7p1*c9p5*ss[62];
  jac_3_3 += 840*c3p1*c7p2*c9p4*ss[60];
  jac_3_3 += 1120*c3p1*c7p3*c9p3*ss[58];
  jac_3_3 += 840*c3p1*c7p4*c9p2*ss[56];
  jac_3_3 += 336*c3p1*c7p5*c9p1*ss[54];
  jac_3_3 += 840*c3p2*c5p1*c7p4*ss[46];
  jac_3_3 += 840*c3p2*c5p1*c9p4*ss[54];
  jac_3_3 += 1680*c3p2*c5p2*c7p3*ss[44];
  jac_3_3 += 1680*c3p2*c5p2*c9p3*ss[50];
  jac_3_3 += 1680*c3p2*c5p3*c7p2*ss[42];
  jac_3_3 += 1680*c3p2*c5p3*c9p2*ss[46];
  jac_3_3 += 840*c3p2*c5p4*c7p1*ss[40];
  jac_3_3 += 840*c3p2*c5p4*c9p1*ss[42];
  jac_3_3 += 840*c3p2*c7p1*c9p4*ss[56];
  jac_3_3 += 1680*c3p2*c7p2*c9p3*ss[54];
  jac_3_3 += 1680*c3p2*c7p3*c9p2*ss[52];
  jac_3_3 += 840*c3p2*c7p4*c9p1*ss[50];
  jac_3_3 += 1120*c3p3*c5p1*c7p3*ss[42];
  jac_3_3 += 1120*c3p3*c5p1*c9p3*ss[48];
  jac_3_3 += 1680*c3p3*c5p2*c7p2*ss[40];
  jac_3_3 += 1680*c3p3*c5p2*c9p2*ss[44];
  jac_3_3 += 1120*c3p3*c5p3*c7p1*ss[38];
  jac_3_3 += 1120*c3p3*c5p3*c9p1*ss[40];
  jac_3_3 += 1120*c3p3*c7p1*c9p3*ss[50];
  jac_3_3 += 1680*c3p3*c7p2*c9p2*ss[48];
  jac_3_3 += 1120*c3p3*c7p3*c9p1*ss[46];
  jac_3_3 += 840*c3p4*c5p1*c7p2*ss[38];
  jac_3_3 += 840*c3p4*c5p1*c9p2*ss[42];
  jac_3_3 += 840*c3p4*c5p2*c7p1*ss[36];
  jac_3_3 += 840*c3p4*c5p2*c9p1*ss[38];
  jac_3_3 += 840*c3p4*c7p1*c9p2*ss[44];
  jac_3_3 += 840*c3p4*c7p2*c9p1*ss[42];
  jac_3_3 += 336*c3p5*c5p1*c7p1*ss[34];
  jac_3_3 += 336*c3p5*c5p1*c9p1*ss[36];
  jac_3_3 += 336*c3p5*c7p1*c9p1*ss[38];
  jac_3_3 += 336*c5p1*c7p1*c9p5*ss[64];
  jac_3_3 += 840*c5p1*c7p2*c9p4*ss[62];
  jac_3_3 += 1120*c5p1*c7p3*c9p3*ss[60];
  jac_3_3 += 840*c5p1*c7p4*c9p2*ss[58];
  jac_3_3 += 336*c5p1*c7p5*c9p1*ss[56];
  jac_3_3 += 840*c5p2*c7p1*c9p4*ss[60];
  jac_3_3 += 1680*c5p2*c7p2*c9p3*ss[58];
  jac_3_3 += 1680*c5p2*c7p3*c9p2*ss[56];
  jac_3_3 += 840*c5p2*c7p4*c9p1*ss[54];
  jac_3_3 += 1120*c5p3*c7p1*c9p3*ss[56];
  jac_3_3 += 1680*c5p3*c7p2*c9p2*ss[54];
  jac_3_3 += 1120*c5p3*c7p3*c9p1*ss[52];
  jac_3_3 += 840*c5p4*c7p1*c9p2*ss[52];
  jac_3_3 += 840*c5p4*c7p2*c9p1*ss[50];
  jac_3_3 += 336*c5p5*c7p1*c9p1*ss[48];
  jac_3_3 += 56*c1p1*c3p6*ss[26];
  jac_3_3 += 56*c1p1*c5p6*ss[38];
  jac_3_3 += 56*c1p1*c7p6*ss[50];
  jac_3_3 += 56*c1p1*c9p6*ss[62];
  jac_3_3 += 168*c1p2*c3p5*ss[24];
  jac_3_3 += 168*c1p2*c5p5*ss[34];
  jac_3_3 += 168*c1p2*c7p5*ss[44];
  jac_3_3 += 168*c1p2*c9p5*ss[54];
  jac_3_3 += 280*c1p3*c3p4*ss[22];
  jac_3_3 += 280*c1p3*c5p4*ss[30];
  jac_3_3 += 280*c1p3*c7p4*ss[38];
  jac_3_3 += 280*c1p3*c9p4*ss[46];
  jac_3_3 += 280*c1p4*c3p3*ss[20];
  jac_3_3 += 280*c1p4*c5p3*ss[26];
  jac_3_3 += 280*c1p4*c7p3*ss[32];
  jac_3_3 += 280*c1p4*c9p3*ss[38];
  jac_3_3 += 168*c1p5*c3p2*ss[18];
  jac_3_3 += 168*c1p5*c5p2*ss[22];
  jac_3_3 += 168*c1p5*c7p2*ss[26];
  jac_3_3 += 168*c1p5*c9p2*ss[30];
  jac_3_3 += 56*c1p6*c3p1*ss[16];
  jac_3_3 += 56*c1p6*c5p1*ss[18];
  jac_3_3 += 56*c1p6*c7p1*ss[20];
  jac_3_3 += 56*c1p6*c9p1*ss[22];
  jac_3_3 += 56*c3p1*c5p6*ss[40];
  jac_3_3 += 56*c3p1*c7p6*ss[52];
  jac_3_3 += 56*c3p1*c9p6*ss[64];
  jac_3_3 += 168*c3p2*c5p5*ss[38];
  jac_3_3 += 168*c3p2*c7p5*ss[48];
  jac_3_3 += 168*c3p2*c9p5*ss[58];
  jac_3_3 += 280*c3p3*c5p4*ss[36];
  jac_3_3 += 280*c3p3*c7p4*ss[44];
  jac_3_3 += 280*c3p3*c9p4*ss[52];
  jac_3_3 += 280*c3p4*c5p3*ss[34];
  jac_3_3 += 280*c3p4*c7p3*ss[40];
  jac_3_3 += 280*c3p4*c9p3*ss[46];
  jac_3_3 += 168*c3p5*c5p2*ss[32];
  jac_3_3 += 168*c3p5*c7p2*ss[36];
  jac_3_3 += 168*c3p5*c9p2*ss[40];
  jac_3_3 += 56*c3p6*c5p1*ss[30];
  jac_3_3 += 56*c3p6*c7p1*ss[32];
  jac_3_3 += 56*c3p6*c9p1*ss[34];
  jac_3_3 += 56*c5p1*c7p6*ss[54];
  jac_3_3 += 56*c5p1*c9p6*ss[66];
  jac_3_3 += 168*c5p2*c7p5*ss[52];
  jac_3_3 += 168*c5p2*c9p5*ss[62];
  jac_3_3 += 280*c5p3*c7p4*ss[50];
  jac_3_3 += 280*c5p3*c9p4*ss[58];
  jac_3_3 += 280*c5p4*c7p3*ss[48];
  jac_3_3 += 280*c5p4*c9p3*ss[54];
  jac_3_3 += 168*c5p5*c7p2*ss[46];
  jac_3_3 += 168*c5p5*c9p2*ss[50];
  jac_3_3 += 56*c5p6*c7p1*ss[44];
  jac_3_3 += 56*c5p6*c9p1*ss[46];
  jac_3_3 += 56*c7p1*c9p6*ss[68];
  jac_3_3 += 168*c7p2*c9p5*ss[66];
  jac_3_3 += 280*c7p3*c9p4*ss[64];
  jac_3_3 += 280*c7p4*c9p3*ss[62];
  jac_3_3 += 168*c7p5*c9p2*ss[60];
  jac_3_3 += 56*c7p6*c9p1*ss[58];
  jac_3_3 += 8*c1p7*ss[14];
  jac_3_3 += 8*c3p7*ss[28];
  jac_3_3 += 8*c5p7*ss[42];
  jac_3_3 += 8*c7p7*ss[56];
  jac_3_3 += 8*c9p7*ss[70];
  TACC jac_3_4 = convert<TACC>(6720*c1p1*c3p1*c5p1*c7p1*c9p3*ss[52]);
  jac_3_4 += 10080*c1p1*c3p1*c5p1*c7p2*c9p2*ss[50];
  jac_3_4 += 6720*c1p1*c3p1*c5p1*c7p3*c9p1*ss[48];
  jac_3_4 += 10080*c1p1*c3p1*c5p2*c7p1*c9p2*ss[48];
  jac_3_4 += 10080*c1p1*c3p1*c5p2*c7p2*c9p1*ss[46];
  jac_3_4 += 6720*c1p1*c3p1*c5p3*c7p1*c9p1*ss[44];
  jac_3_4 += 10080*c1p1*c3p2*c5p1*c7p1*c9p2*ss[46];
  jac_3_4 += 10080*c1p1*c3p2*c5p1*c7p2*c9p1*ss[44];
  jac_3_4 += 10080*c1p1*c3p2*c5p2*c7p1*c9p1*ss[42];
  jac_3_4 += 6720*c1p1*c3p3*c5p1*c7p1*c9p1*ss[40];
  jac_3_4 += 10080*c1p2*c3p1*c5p1*c7p1*c9p2*ss[44];
  jac_3_4 += 10080*c1p2*c3p1*c5p1*c7p2*c9p1*ss[42];
  jac_3_4 += 10080*c1p2*c3p1*c5p2*c7p1*c9p1*ss[40];
  jac_3_4 += 10080*c1p2*c3p2*c5p1*c7p1*c9p1*ss[38];
  jac_3_4 += 6720*c1p3*c3p1*c5p1*c7p1*c9p1*ss[36];
  jac_3_4 += 1680*c1p1*c3p1*c5p1*c7p4*ss[46];
  jac_3_4 += 1680*c1p1*c3p1*c5p1*c9p4*ss[54];
  jac_3_4 += 3360*c1p1*c3p1*c5p2*c7p3*ss[44];
  jac_3_4 += 3360*c1p1*c3p1*c5p2*c9p3*ss[50];
  jac_3_4 += 3360*c1p1*c3p1*c5p3*c7p2*ss[42];
  jac_3_4 += 3360*c1p1*c3p1*c5p3*c9p2*ss[46];
  jac_3_4 += 1680*c1p1*c3p1*c5p4*c7p1*ss[40];
  jac_3_4 += 1680*c1p1*c3p1*c5p4*c9p1*ss[42];
  jac_3_4 += 1680*c1p1*c3p1*c7p1*c9p4*ss[56];
  jac_3_4 += 3360*c1p1*c3p1*c7p2*c9p3*ss[54];
  jac_3_4 += 3360*c1p1*c3p1*c7p3*c9p2*ss[52];
  jac_3_4 += 1680*c1p1*c3p1*c7p4*c9p1*ss[50];
  jac_3_4 += 3360*c1p1*c3p2*c5p1*c7p3*ss[42];
  jac_3_4 += 3360*c1p1*c3p2*c5p1*c9p3*ss[48];
  jac_3_4 += 5040*c1p1*c3p2*c5p2*c7p2*ss[40];
  jac_3_4 += 5040*c1p1*c3p2*c5p2*c9p2*ss[44];
  jac_3_4 += 3360*c1p1*c3p2*c5p3*c7p1*ss[38];
  jac_3_4 += 3360*c1p1*c3p2*c5p3*c9p1*ss[40];
  jac_3_4 += 3360*c1p1*c3p2*c7p1*c9p3*ss[50];
  jac_3_4 += 5040*c1p1*c3p2*c7p2*c9p2*ss[48];
  jac_3_4 += 3360*c1p1*c3p2*c7p3*c9p1*ss[46];
  jac_3_4 += 3360*c1p1*c3p3*c5p1*c7p2*ss[38];
  jac_3_4 += 3360*c1p1*c3p3*c5p1*c9p2*ss[42];
  jac_3_4 += 3360*c1p1*c3p3*c5p2*c7p1*ss[36];
  jac_3_4 += 3360*c1p1*c3p3*c5p2*c9p1*ss[38];
  jac_3_4 += 3360*c1p1*c3p3*c7p1*c9p2*ss[44];
  jac_3_4 += 3360*c1p1*c3p3*c7p2*c9p1*ss[42];
  jac_3_4 += 1680*c1p1*c3p4*c5p1*c7p1*ss[34];
  jac_3_4 += 1680*c1p1*c3p4*c5p1*c9p1*ss[36];
  jac_3_4 += 1680*c1p1*c3p4*c7p1*c9p1*ss[38];
  jac_3_4 += 1680*c1p1*c5p1*c7p1*c9p4*ss[58];
  jac_3_4 += 3360*c1p1*c5p1*c7p2*c9p3*ss[56];
  jac_3_4 += 3360*c1p1*c5p1*c7p3*c9p2*ss[54];
  jac_3_4 += 1680*c1p1*c5p1*c7p4*c9p1*ss[52];
  jac_3_4 += 3360*c1p1*c5p2*c7p1*c9p3*ss[54];
  jac_3_4 += 5040*c1p1*c5p2*c7p2*c9p2*ss[52];
  jac_3_4 += 3360*c1p1*c5p2*c7p3*c9p1*ss[50];
  jac_3_4 += 3360*c1p1*c5p3*c7p1*c9p2*ss[50];
  jac_3_4 += 3360*c1p1*c5p3*c7p2*c9p1*ss[48];
  jac_3_4 += 1680*c1p1*c5p4*c7p1*c9p1*ss[46];
  jac_3_4 += 3360*c1p2*c3p1*c5p1*c7p3*ss[40];
  jac_3_4 += 3360*c1p2*c3p1*c5p1*c9p3*ss[46];
  jac_3_4 += 5040*c1p2*c3p1*c5p2*c7p2*ss[38];
  jac_3_4 += 5040*c1p2*c3p1*c5p2*c9p2*ss[42];
  jac_3_4 += 3360*c1p2*c3p1*c5p3*c7p1*ss[36];
  jac_3_4 += 3360*c1p2*c3p1*c5p3*c9p1*ss[38];
  jac_3_4 += 3360*c1p2*c3p1*c7p1*c9p3*ss[48];
  jac_3_4 += 5040*c1p2*c3p1*c7p2*c9p2*ss[46];
  jac_3_4 += 3360*c1p2*c3p1*c7p3*c9p1*ss[44];
  jac_3_4 += 5040*c1p2*c3p2*c5p1*c7p2*ss[36];
  jac_3_4 += 5040*c1p2*c3p2*c5p1*c9p2*ss[40];
  jac_3_4 += 5040*c1p2*c3p2*c5p2*c7p1*ss[34];
  jac_3_4 += 5040*c1p2*c3p2*c5p2*c9p1*ss[36];
  jac_3_4 += 5040*c1p2*c3p2*c7p1*c9p2*ss[42];
  jac_3_4 += 5040*c1p2*c3p2*c7p2*c9p1*ss[40];
  jac_3_4 += 3360*c1p2*c3p3*c5p1*c7p1*ss[32];
  jac_3_4 += 3360*c1p2*c3p3*c5p1*c9p1*ss[34];
  jac_3_4 += 3360*c1p2*c3p3*c7p1*c9p1*ss[36];
  jac_3_4 += 3360*c1p2*c5p1*c7p1*c9p3*ss[50];
  jac_3_4 += 5040*c1p2*c5p1*c7p2*c9p2*ss[48];
  jac_3_4 += 3360*c1p2*c5p1*c7p3*c9p1*ss[46];
  jac_3_4 += 5040*c1p2*c5p2*c7p1*c9p2*ss[46];
  jac_3_4 += 5040*c1p2*c5p2*c7p2*c9p1*ss[44];
  jac_3_4 += 3360*c1p2*c5p3*c7p1*c9p1*ss[42];
  jac_3_4 += 3360*c1p3*c3p1*c5p1*c7p2*ss[34];
  jac_3_4 += 3360*c1p3*c3p1*c5p1*c9p2*ss[38];
  jac_3_4 += 3360*c1p3*c3p1*c5p2*c7p1*ss[32];
  jac_3_4 += 3360*c1p3*c3p1*c5p2*c9p1*ss[34];
  jac_3_4 += 3360*c1p3*c3p1*c7p1*c9p2*ss[40];
  jac_3_4 += 3360*c1p3*c3p1*c7p2*c9p1*ss[38];
  jac_3_4 += 3360*c1p3*c3p2*c5p1*c7p1*ss[30];
  jac_3_4 += 3360*c1p3*c3p2*c5p1*c9p1*ss[32];
  jac_3_4 += 3360*c1p3*c3p2*c7p1*c9p1*ss[34];
  jac_3_4 += 3360*c1p3*c5p1*c7p1*c9p2*ss[42];
  jac_3_4 += 3360*c1p3*c5p1*c7p2*c9p1*ss[40];
  jac_3_4 += 3360*c1p3*c5p2*c7p1*c9p1*ss[38];
  jac_3_4 += 1680*c1p4*c3p1*c5p1*c7p1*ss[28];
  jac_3_4 += 1680*c1p4*c3p1*c5p1*c9p1*ss[30];
  jac_3_4 += 1680*c1p4*c3p1*c7p1*c9p1*ss[32];
  jac_3_4 += 1680*c1p4*c5p1*c7p1*c9p1*ss[34];
  jac_3_4 += 1680*c3p1*c5p1*c7p1*c9p4*ss[60];
  jac_3_4 += 3360*c3p1*c5p1*c7p2*c9p3*ss[58];
  jac_3_4 += 3360*c3p1*c5p1*c7p3*c9p2*ss[56];
  jac_3_4 += 1680*c3p1*c5p1*c7p4*c9p1*ss[54];
  jac_3_4 += 3360*c3p1*c5p2*c7p1*c9p3*ss[56];
  jac_3_4 += 5040*c3p1*c5p2*c7p2*c9p2*ss[54];
  jac_3_4 += 3360*c3p1*c5p2*c7p3*c9p1*ss[52];
  jac_3_4 += 3360*c3p1*c5p3*c7p1*c9p2*ss[52];
  jac_3_4 += 3360*c3p1*c5p3*c7p2*c9p1*ss[50];
  jac_3_4 += 1680*c3p1*c5p4*c7p1*c9p1*ss[48];
  jac_3_4 += 3360*c3p2*c5p1*c7p1*c9p3*ss[54];
  jac_3_4 += 5040*c3p2*c5p1*c7p2*c9p2*ss[52];
  jac_3_4 += 3360*c3p2*c5p1*c7p3*c9p1*ss[50];
  jac_3_4 += 5040*c3p2*c5p2*c7p1*c9p2*ss[50];
  jac_3_4 += 5040*c3p2*c5p2*c7p2*c9p1*ss[48];
  jac_3_4 += 3360*c3p2*c5p3*c7p1*c9p1*ss[46];
  jac_3_4 += 3360*c3p3*c5p1*c7p1*c9p2*ss[48];
  jac_3_4 += 3360*c3p3*c5p1*c7p2*c9p1*ss[46];
  jac_3_4 += 3360*c3p3*c5p2*c7p1*c9p1*ss[44];
  jac_3_4 += 1680*c3p4*c5p1*c7p1*c9p1*ss[42];
  jac_3_4 += 336*c1p1*c3p1*c5p5*ss[38];
  jac_3_4 += 336*c1p1*c3p1*c7p5*ss[48];
  jac_3_4 += 336*c1p1*c3p1*c9p5*ss[58];
  jac_3_4 += 840*c1p1*c3p2*c5p4*ss[36];
  jac_3_4 += 840*c1p1*c3p2*c7p4*ss[44];
  jac_3_4 += 840*c1p1*c3p2*c9p4*ss[52];
  jac_3_4 += 1120*c1p1*c3p3*c5p3*ss[34];
  jac_3_4 += 1120*c1p1*c3p3*c7p3*ss[40];
  jac_3_4 += 1120*c1p1*c3p3*c9p3*ss[46];
  jac_3_4 += 840*c1p1*c3p4*c5p2*ss[32];
  jac_3_4 += 840*c1p1*c3p4*c7p2*ss[36];
  jac_3_4 += 840*c1p1*c3p4*c9p2*ss[40];
  jac_3_4 += 336*c1p1*c3p5*c5p1*ss[30];
  jac_3_4 += 336*c1p1*c3p5*c7p1*ss[32];
  jac_3_4 += 336*c1p1*c3p5*c9p1*ss[34];
  jac_3_4 += 336*c1p1*c5p1*c7p5*ss[50];
  jac_3_4 += 336*c1p1*c5p1*c9p5*ss[60];
  jac_3_4 += 840*c1p1*c5p2*c7p4*ss[48];
  jac_3_4 += 840*c1p1*c5p2*c9p4*ss[56];
  jac_3_4 += 1120*c1p1*c5p3*c7p3*ss[46];
  jac_3_4 += 1120*c1p1*c5p3*c9p3*ss[52];
  jac_3_4 += 840*c1p1*c5p4*c7p2*ss[44];
  jac_3_4 += 840*c1p1*c5p4*c9p2*ss[48];
  jac_3_4 += 336*c1p1*c5p5*c7p1*ss[42];
  jac_3_4 += 336*c1p1*c5p5*c9p1*ss[44];
  jac_3_4 += 336*c1p1*c7p1*c9p5*ss[62];
  jac_3_4 += 840*c1p1*c7p2*c9p4*ss[60];
  jac_3_4 += 1120*c1p1*c7p3*c9p3*ss[58];
  jac_3_4 += 840*c1p1*c7p4*c9p2*ss[56];
  jac_3_4 += 336*c1p1*c7p5*c9p1*ss[54];
  jac_3_4 += 840*c1p2*c3p1*c5p4*ss[34];
  jac_3_4 += 840*c1p2*c3p1*c7p4*ss[42];
  jac_3_4 += 840*c1p2*c3p1*c9p4*ss[50];
  jac_3_4 += 1680*c1p2*c3p2*c5p3*ss[32];
  jac_3_4 += 1680*c1p2*c3p2*c7p3*ss[38];
  jac_3_4 += 1680*c1p2*c3p2*c9p3*ss[44];
  jac_3_4 += 1680*c1p2*c3p3*c5p2*ss[30];
  jac_3_4 += 1680*c1p2*c3p3*c7p2*ss[34];
  jac_3_4 += 1680*c1p2*c3p3*c9p2*ss[38];
  jac_3_4 += 840*c1p2*c3p4*c5p1*ss[28];
  jac_3_4 += 840*c1p2*c3p4*c7p1*ss[30];
  jac_3_4 += 840*c1p2*c3p4*c9p1*ss[32];
  jac_3_4 += 840*c1p2*c5p1*c7p4*ss[44];
  jac_3_4 += 840*c1p2*c5p1*c9p4*ss[52];
  jac_3_4 += 1680*c1p2*c5p2*c7p3*ss[42];
  jac_3_4 += 1680*c1p2*c5p2*c9p3*ss[48];
  jac_3_4 += 1680*c1p2*c5p3*c7p2*ss[40];
  jac_3_4 += 1680*c1p2*c5p3*c9p2*ss[44];
  jac_3_4 += 840*c1p2*c5p4*c7p1*ss[38];
  jac_3_4 += 840*c1p2*c5p4*c9p1*ss[40];
  jac_3_4 += 840*c1p2*c7p1*c9p4*ss[54];
  jac_3_4 += 1680*c1p2*c7p2*c9p3*ss[52];
  jac_3_4 += 1680*c1p2*c7p3*c9p2*ss[50];
  jac_3_4 += 840*c1p2*c7p4*c9p1*ss[48];
  jac_3_4 += 1120*c1p3*c3p1*c5p3*ss[30];
  jac_3_4 += 1120*c1p3*c3p1*c7p3*ss[36];
  jac_3_4 += 1120*c1p3*c3p1*c9p3*ss[42];
  jac_3_4 += 1680*c1p3*c3p2*c5p2*ss[28];
  jac_3_4 += 1680*c1p3*c3p2*c7p2*ss[32];
  jac_3_4 += 1680*c1p3*c3p2*c9p2*ss[36];
  jac_3_4 += 1120*c1p3*c3p3*c5p1*ss[26];
  jac_3_4 += 1120*c1p3*c3p3*c7p1*ss[28];
  jac_3_4 += 1120*c1p3*c3p3*c9p1*ss[30];
  jac_3_4 += 1120*c1p3*c5p1*c7p3*ss[38];
  jac_3_4 += 1120*c1p3*c5p1*c9p3*ss[44];
  jac_3_4 += 1680*c1p3*c5p2*c7p2*ss[36];
  jac_3_4 += 1680*c1p3*c5p2*c9p2*ss[40];
  jac_3_4 += 1120*c1p3*c5p3*c7p1*ss[34];
  jac_3_4 += 1120*c1p3*c5p3*c9p1*ss[36];
  jac_3_4 += 1120*c1p3*c7p1*c9p3*ss[46];
  jac_3_4 += 1680*c1p3*c7p2*c9p2*ss[44];
  jac_3_4 += 1120*c1p3*c7p3*c9p1*ss[42];
  jac_3_4 += 840*c1p4*c3p1*c5p2*ss[26];
  jac_3_4 += 840*c1p4*c3p1*c7p2*ss[30];
  jac_3_4 += 840*c1p4*c3p1*c9p2*ss[34];
  jac_3_4 += 840*c1p4*c3p2*c5p1*ss[24];
  jac_3_4 += 840*c1p4*c3p2*c7p1*ss[26];
  jac_3_4 += 840*c1p4*c3p2*c9p1*ss[28];
  jac_3_4 += 840*c1p4*c5p1*c7p2*ss[32];
  jac_3_4 += 840*c1p4*c5p1*c9p2*ss[36];
  jac_3_4 += 840*c1p4*c5p2*c7p1*ss[30];
  jac_3_4 += 840*c1p4*c5p2*c9p1*ss[32];
  jac_3_4 += 840*c1p4*c7p1*c9p2*ss[38];
  jac_3_4 += 840*c1p4*c7p2*c9p1*ss[36];
  jac_3_4 += 336*c1p5*c3p1*c5p1*ss[22];
  jac_3_4 += 336*c1p5*c3p1*c7p1*ss[24];
  jac_3_4 += 336*c1p5*c3p1*c9p1*ss[26];
  jac_3_4 += 336*c1p5*c5p1*c7p1*ss[26];
  jac_3_4 += 336*c1p5*c5p1*c9p1*ss[28];
  jac_3_4 += 336*c1p5*c7p1*c9p1*ss[30];
  jac_3_4 += 336*c3p1*c5p1*c7p5*ss[52];
  jac_3_4 += 336*c3p1*c5p1*c9p5*ss[62];
  jac_3_4 += 840*c3p1*c5p2*c7p4*ss[50];
  jac_3_4 += 840*c3p1*c5p2*c9p4*ss[58];
  jac_3_4 += 1120*c3p1*c5p3*c7p3*ss[48];
  jac_3_4 += 1120*c3p1*c5p3*c9p3*ss[54];
  jac_3_4 += 840*c3p1*c5p4*c7p2*ss[46];
  jac_3_4 += 840*c3p1*c5p4*c9p2*ss[50];
  jac_3_4 += 336*c3p1*c5p5*c7p1*ss[44];
  jac_3_4 += 336*c3p1*c5p5*c9p1*ss[46];
  jac_3_4 += 336*c3p1*c7p1*c9p5*ss[64];
  jac_3_4 += 840*c3p1*c7p2*c9p4*ss[62];
  jac_3_4 += 1120*c3p1*c7p3*c9p3*ss[60];
  jac_3_4 += 840*c3p1*c7p4*c9p2*ss[58];
  jac_3_4 += 336*c3p1*c7p5*c9p1*ss[56];
  jac_3_4 += 840*c3p2*c5p1*c7p4*ss[48];
  jac_3_4 += 840*c3p2*c5p1*c9p4*ss[56];
  jac_3_4 += 1680*c3p2*c5p2*c7p3*ss[46];
  jac_3_4 += 1680*c3p2*c5p2*c9p3*ss[52];
  jac_3_4 += 1680*c3p2*c5p3*c7p2*ss[44];
  jac_3_4 += 1680*c3p2*c5p3*c9p2*ss[48];
  jac_3_4 += 840*c3p2*c5p4*c7p1*ss[42];
  jac_3_4 += 840*c3p2*c5p4*c9p1*ss[44];
  jac_3_4 += 840*c3p2*c7p1*c9p4*ss[58];
  jac_3_4 += 1680*c3p2*c7p2*c9p3*ss[56];
  jac_3_4 += 1680*c3p2*c7p3*c9p2*ss[54];
  jac_3_4 += 840*c3p2*c7p4*c9p1*ss[52];
  jac_3_4 += 1120*c3p3*c5p1*c7p3*ss[44];
  jac_3_4 += 1120*c3p3*c5p1*c9p3*ss[50];
  jac_3_4 += 1680*c3p3*c5p2*c7p2*ss[42];
  jac_3_4 += 1680*c3p3*c5p2*c9p2*ss[46];
  jac_3_4 += 1120*c3p3*c5p3*c7p1*ss[40];
  jac_3_4 += 1120*c3p3*c5p3*c9p1*ss[42];
  jac_3_4 += 1120*c3p3*c7p1*c9p3*ss[52];
  jac_3_4 += 1680*c3p3*c7p2*c9p2*ss[50];
  jac_3_4 += 1120*c3p3*c7p3*c9p1*ss[48];
  jac_3_4 += 840*c3p4*c5p1*c7p2*ss[40];
  jac_3_4 += 840*c3p4*c5p1*c9p2*ss[44];
  jac_3_4 += 840*c3p4*c5p2*c7p1*ss[38];
  jac_3_4 += 840*c3p4*c5p2*c9p1*ss[40];
  jac_3_4 += 840*c3p4*c7p1*c9p2*ss[46];
  jac_3_4 += 840*c3p4*c7p2*c9p1*ss[44];
  jac_3_4 += 336*c3p5*c5p1*c7p1*ss[36];
  jac_3_4 += 336*c3p5*c5p1*c9p1*ss[38];
  jac_3_4 += 336*c3p5*c7p1*c9p1*ss[40];
  jac_3_4 += 336*c5p1*c7p1*c9p5*ss[66];
  jac_3_4 += 840*c5p1*c7p2*c9p4*ss[64];
  jac_3_4 += 1120*c5p1*c7p3*c9p3*ss[62];
  jac_3_4 += 840*c5p1*c7p4*c9p2*ss[60];
  jac_3_4 += 336*c5p1*c7p5*c9p1*ss[58];
  jac_3_4 += 840*c5p2*c7p1*c9p4*ss[62];
  jac_3_4 += 1680*c5p2*c7p2*c9p3*ss[60];
  jac_3_4 += 1680*c5p2*c7p3*c9p2*ss[58];
  jac_3_4 += 840*c5p2*c7p4*c9p1*ss[56];
  jac_3_4 += 1120*c5p3*c7p1*c9p3*ss[58];
  jac_3_4 += 1680*c5p3*c7p2*c9p2*ss[56];
  jac_3_4 += 1120*c5p3*c7p3*c9p1*ss[54];
  jac_3_4 += 840*c5p4*c7p1*c9p2*ss[54];
  jac_3_4 += 840*c5p4*c7p2*c9p1*ss[52];
  jac_3_4 += 336*c5p5*c7p1*c9p1*ss[50];
  jac_3_4 += 56*c1p1*c3p6*ss[28];
  jac_3_4 += 56*c1p1*c5p6*ss[40];
  jac_3_4 += 56*c1p1*c7p6*ss[52];
  jac_3_4 += 56*c1p1*c9p6*ss[64];
  jac_3_4 += 168*c1p2*c3p5*ss[26];
  jac_3_4 += 168*c1p2*c5p5*ss[36];
  jac_3_4 += 168*c1p2*c7p5*ss[46];
  jac_3_4 += 168*c1p2*c9p5*ss[56];
  jac_3_4 += 280*c1p3*c3p4*ss[24];
  jac_3_4 += 280*c1p3*c5p4*ss[32];
  jac_3_4 += 280*c1p3*c7p4*ss[40];
  jac_3_4 += 280*c1p3*c9p4*ss[48];
  jac_3_4 += 280*c1p4*c3p3*ss[22];
  jac_3_4 += 280*c1p4*c5p3*ss[28];
  jac_3_4 += 280*c1p4*c7p3*ss[34];
  jac_3_4 += 280*c1p4*c9p3*ss[40];
  jac_3_4 += 168*c1p5*c3p2*ss[20];
  jac_3_4 += 168*c1p5*c5p2*ss[24];
  jac_3_4 += 168*c1p5*c7p2*ss[28];
  jac_3_4 += 168*c1p5*c9p2*ss[32];
  jac_3_4 += 56*c1p6*c3p1*ss[18];
  jac_3_4 += 56*c1p6*c5p1*ss[20];
  jac_3_4 += 56*c1p6*c7p1*ss[22];
  jac_3_4 += 56*c1p6*c9p1*ss[24];
  jac_3_4 += 56*c3p1*c5p6*ss[42];
  jac_3_4 += 56*c3p1*c7p6*ss[54];
  jac_3_4 += 56*c3p1*c9p6*ss[66];
  jac_3_4 += 168*c3p2*c5p5*ss[40];
  jac_3_4 += 168*c3p2*c7p5*ss[50];
  jac_3_4 += 168*c3p2*c9p5*ss[60];
  jac_3_4 += 280*c3p3*c5p4*ss[38];
  jac_3_4 += 280*c3p3*c7p4*ss[46];
  jac_3_4 += 280*c3p3*c9p4*ss[54];
  jac_3_4 += 280*c3p4*c5p3*ss[36];
  jac_3_4 += 280*c3p4*c7p3*ss[42];
  jac_3_4 += 280*c3p4*c9p3*ss[48];
  jac_3_4 += 168*c3p5*c5p2*ss[34];
  jac_3_4 += 168*c3p5*c7p2*ss[38];
  jac_3_4 += 168*c3p5*c9p2*ss[42];
  jac_3_4 += 56*c3p6*c5p1*ss[32];
  jac_3_4 += 56*c3p6*c7p1*ss[34];
  jac_3_4 += 56*c3p6*c9p1*ss[36];
  jac_3_4 += 56*c5p1*c7p6*ss[56];
  jac_3_4 += 56*c5p1*c9p6*ss[68];
  jac_3_4 += 168*c5p2*c7p5*ss[54];
  jac_3_4 += 168*c5p2*c9p5*ss[64];
  jac_3_4 += 280*c5p3*c7p4*ss[52];
  jac_3_4 += 280*c5p3*c9p4*ss[60];
  jac_3_4 += 280*c5p4*c7p3*ss[50];
  jac_3_4 += 280*c5p4*c9p3*ss[56];
  jac_3_4 += 168*c5p5*c7p2*ss[48];
  jac_3_4 += 168*c5p5*c9p2*ss[52];
  jac_3_4 += 56*c5p6*c7p1*ss[46];
  jac_3_4 += 56*c5p6*c9p1*ss[48];
  jac_3_4 += 56*c7p1*c9p6*ss[70];
  jac_3_4 += 168*c7p2*c9p5*ss[68];
  jac_3_4 += 280*c7p3*c9p4*ss[66];
  jac_3_4 += 280*c7p4*c9p3*ss[64];
  jac_3_4 += 168*c7p5*c9p2*ss[62];
  jac_3_4 += 56*c7p6*c9p1*ss[60];
  jac_3_4 += 8*c1p7*ss[16];
  jac_3_4 += 8*c3p7*ss[30];
  jac_3_4 += 8*c5p7*ss[44];
  jac_3_4 += 8*c7p7*ss[58];
  jac_3_4 += 8*c9p7*ss[72];
  TACC jac_4_0 = convert<TACC>(30240*c1p1*c3p1*c5p1*c7p1*c9p5*ss[62]);
  jac_4_0 += 75600*c1p1*c3p1*c5p1*c7p2*c9p4*ss[60];
  jac_4_0 += 100800*c1p1*c3p1*c5p1*c7p3*c9p3*ss[58];
  jac_4_0 += 75600*c1p1*c3p1*c5p1*c7p4*c9p2*ss[56];
  jac_4_0 += 30240*c1p1*c3p1*c5p1*c7p5*c9p1*ss[54];
  jac_4_0 += 75600*c1p1*c3p1*c5p2*c7p1*c9p4*ss[58];
  jac_4_0 += 151200*c1p1*c3p1*c5p2*c7p2*c9p3*ss[56];
  jac_4_0 += 151200*c1p1*c3p1*c5p2*c7p3*c9p2*ss[54];
  jac_4_0 += 75600*c1p1*c3p1*c5p2*c7p4*c9p1*ss[52];
  jac_4_0 += 100800*c1p1*c3p1*c5p3*c7p1*c9p3*ss[54];
  jac_4_0 += 151200*c1p1*c3p1*c5p3*c7p2*c9p2*ss[52];
  jac_4_0 += 100800*c1p1*c3p1*c5p3*c7p3*c9p1*ss[50];
  jac_4_0 += 75600*c1p1*c3p1*c5p4*c7p1*c9p2*ss[50];
  jac_4_0 += 75600*c1p1*c3p1*c5p4*c7p2*c9p1*ss[48];
  jac_4_0 += 30240*c1p1*c3p1*c5p5*c7p1*c9p1*ss[46];
  jac_4_0 += 75600*c1p1*c3p2*c5p1*c7p1*c9p4*ss[56];
  jac_4_0 += 151200*c1p1*c3p2*c5p1*c7p2*c9p3*ss[54];
  jac_4_0 += 151200*c1p1*c3p2*c5p1*c7p3*c9p2*ss[52];
  jac_4_0 += 75600*c1p1*c3p2*c5p1*c7p4*c9p1*ss[50];
  jac_4_0 += 151200*c1p1*c3p2*c5p2*c7p1*c9p3*ss[52];
  jac_4_0 += 226800*c1p1*c3p2*c5p2*c7p2*c9p2*ss[50];
  jac_4_0 += 151200*c1p1*c3p2*c5p2*c7p3*c9p1*ss[48];
  jac_4_0 += 151200*c1p1*c3p2*c5p3*c7p1*c9p2*ss[48];
  jac_4_0 += 151200*c1p1*c3p2*c5p3*c7p2*c9p1*ss[46];
  jac_4_0 += 75600*c1p1*c3p2*c5p4*c7p1*c9p1*ss[44];
  jac_4_0 += 100800*c1p1*c3p3*c5p1*c7p1*c9p3*ss[50];
  jac_4_0 += 151200*c1p1*c3p3*c5p1*c7p2*c9p2*ss[48];
  jac_4_0 += 100800*c1p1*c3p3*c5p1*c7p3*c9p1*ss[46];
  jac_4_0 += 151200*c1p1*c3p3*c5p2*c7p1*c9p2*ss[46];
  jac_4_0 += 151200*c1p1*c3p3*c5p2*c7p2*c9p1*ss[44];
  jac_4_0 += 100800*c1p1*c3p3*c5p3*c7p1*c9p1*ss[42];
  jac_4_0 += 75600*c1p1*c3p4*c5p1*c7p1*c9p2*ss[44];
  jac_4_0 += 75600*c1p1*c3p4*c5p1*c7p2*c9p1*ss[42];
  jac_4_0 += 75600*c1p1*c3p4*c5p2*c7p1*c9p1*ss[40];
  jac_4_0 += 30240*c1p1*c3p5*c5p1*c7p1*c9p1*ss[38];
  jac_4_0 += 75600*c1p2*c3p1*c5p1*c7p1*c9p4*ss[54];
  jac_4_0 += 151200*c1p2*c3p1*c5p1*c7p2*c9p3*ss[52];
  jac_4_0 += 151200*c1p2*c3p1*c5p1*c7p3*c9p2*ss[50];
  jac_4_0 += 75600*c1p2*c3p1*c5p1*c7p4*c9p1*ss[48];
  jac_4_0 += 151200*c1p2*c3p1*c5p2*c7p1*c9p3*ss[50];
  jac_4_0 += 226800*c1p2*c3p1*c5p2*c7p2*c9p2*ss[48];
  jac_4_0 += 151200*c1p2*c3p1*c5p2*c7p3*c9p1*ss[46];
  jac_4_0 += 151200*c1p2*c3p1*c5p3*c7p1*c9p2*ss[46];
  jac_4_0 += 151200*c1p2*c3p1*c5p3*c7p2*c9p1*ss[44];
  jac_4_0 += 75600*c1p2*c3p1*c5p4*c7p1*c9p1*ss[42];
  jac_4_0 += 151200*c1p2*c3p2*c5p1*c7p1*c9p3*ss[48];
  jac_4_0 += 226800*c1p2*c3p2*c5p1*c7p2*c9p2*ss[46];
  jac_4_0 += 151200*c1p2*c3p2*c5p1*c7p3*c9p1*ss[44];
  jac_4_0 += 226800*c1p2*c3p2*c5p2*c7p1*c9p2*ss[44];
  jac_4_0 += 226800*c1p2*c3p2*c5p2*c7p2*c9p1*ss[42];
  jac_4_0 += 151200*c1p2*c3p2*c5p3*c7p1*c9p1*ss[40];
  jac_4_0 += 151200*c1p2*c3p3*c5p1*c7p1*c9p2*ss[42];
  jac_4_0 += 151200*c1p2*c3p3*c5p1*c7p2*c9p1*ss[40];
  jac_4_0 += 151200*c1p2*c3p3*c5p2*c7p1*c9p1*ss[38];
  jac_4_0 += 75600*c1p2*c3p4*c5p1*c7p1*c9p1*ss[36];
  jac_4_0 += 100800*c1p3*c3p1*c5p1*c7p1*c9p3*ss[46];
  jac_4_0 += 151200*c1p3*c3p1*c5p1*c7p2*c9p2*ss[44];
  jac_4_0 += 100800*c1p3*c3p1*c5p1*c7p3*c9p1*ss[42];
  jac_4_0 += 151200*c1p3*c3p1*c5p2*c7p1*c9p2*ss[42];
  jac_4_0 += 151200*c1p3*c3p1*c5p2*c7p2*c9p1*ss[40];
  jac_4_0 += 100800*c1p3*c3p1*c5p3*c7p1*c9p1*ss[38];
  jac_4_0 += 151200*c1p3*c3p2*c5p1*c7p1*c9p2*ss[40];
  jac_4_0 += 151200*c1p3*c3p2*c5p1*c7p2*c9p1*ss[38];
  jac_4_0 += 151200*c1p3*c3p2*c5p2*c7p1*c9p1*ss[36];
  jac_4_0 += 100800*c1p3*c3p3*c5p1*c7p1*c9p1*ss[34];
  jac_4_0 += 75600*c1p4*c3p1*c5p1*c7p1*c9p2*ss[38];
  jac_4_0 += 75600*c1p4*c3p1*c5p1*c7p2*c9p1*ss[36];
  jac_4_0 += 75600*c1p4*c3p1*c5p2*c7p1*c9p1*ss[34];
  jac_4_0 += 75600*c1p4*c3p2*c5p1*c7p1*c9p1*ss[32];
  jac_4_0 += 30240*c1p5*c3p1*c5p1*c7p1*c9p1*ss[30];
  jac_4_0 += 5040*c1p1*c3p1*c5p1*c7p6*ss[52];
  jac_4_0 += 5040*c1p1*c3p1*c5p1*c9p6*ss[64];
  jac_4_0 += 15120*c1p1*c3p1*c5p2*c7p5*ss[50];
  jac_4_0 += 15120*c1p1*c3p1*c5p2*c9p5*ss[60];
  jac_4_0 += 25200*c1p1*c3p1*c5p3*c7p4*ss[48];
  jac_4_0 += 25200*c1p1*c3p1*c5p3*c9p4*ss[56];
  jac_4_0 += 25200*c1p1*c3p1*c5p4*c7p3*ss[46];
  jac_4_0 += 25200*c1p1*c3p1*c5p4*c9p3*ss[52];
  jac_4_0 += 15120*c1p1*c3p1*c5p5*c7p2*ss[44];
  jac_4_0 += 15120*c1p1*c3p1*c5p5*c9p2*ss[48];
  jac_4_0 += 5040*c1p1*c3p1*c5p6*c7p1*ss[42];
  jac_4_0 += 5040*c1p1*c3p1*c5p6*c9p1*ss[44];
  jac_4_0 += 5040*c1p1*c3p1*c7p1*c9p6*ss[66];
  jac_4_0 += 15120*c1p1*c3p1*c7p2*c9p5*ss[64];
  jac_4_0 += 25200*c1p1*c3p1*c7p3*c9p4*ss[62];
  jac_4_0 += 25200*c1p1*c3p1*c7p4*c9p3*ss[60];
  jac_4_0 += 15120*c1p1*c3p1*c7p5*c9p2*ss[58];
  jac_4_0 += 5040*c1p1*c3p1*c7p6*c9p1*ss[56];
  jac_4_0 += 15120*c1p1*c3p2*c5p1*c7p5*ss[48];
  jac_4_0 += 15120*c1p1*c3p2*c5p1*c9p5*ss[58];
  jac_4_0 += 37800*c1p1*c3p2*c5p2*c7p4*ss[46];
  jac_4_0 += 37800*c1p1*c3p2*c5p2*c9p4*ss[54];
  jac_4_0 += 50400*c1p1*c3p2*c5p3*c7p3*ss[44];
  jac_4_0 += 50400*c1p1*c3p2*c5p3*c9p3*ss[50];
  jac_4_0 += 37800*c1p1*c3p2*c5p4*c7p2*ss[42];
  jac_4_0 += 37800*c1p1*c3p2*c5p4*c9p2*ss[46];
  jac_4_0 += 15120*c1p1*c3p2*c5p5*c7p1*ss[40];
  jac_4_0 += 15120*c1p1*c3p2*c5p5*c9p1*ss[42];
  jac_4_0 += 15120*c1p1*c3p2*c7p1*c9p5*ss[60];
  jac_4_0 += 37800*c1p1*c3p2*c7p2*c9p4*ss[58];
  jac_4_0 += 50400*c1p1*c3p2*c7p3*c9p3*ss[56];
  jac_4_0 += 37800*c1p1*c3p2*c7p4*c9p2*ss[54];
  jac_4_0 += 15120*c1p1*c3p2*c7p5*c9p1*ss[52];
  jac_4_0 += 25200*c1p1*c3p3*c5p1*c7p4*ss[44];
  jac_4_0 += 25200*c1p1*c3p3*c5p1*c9p4*ss[52];
  jac_4_0 += 50400*c1p1*c3p3*c5p2*c7p3*ss[42];
  jac_4_0 += 50400*c1p1*c3p3*c5p2*c9p3*ss[48];
  jac_4_0 += 50400*c1p1*c3p3*c5p3*c7p2*ss[40];
  jac_4_0 += 50400*c1p1*c3p3*c5p3*c9p2*ss[44];
  jac_4_0 += 25200*c1p1*c3p3*c5p4*c7p1*ss[38];
  jac_4_0 += 25200*c1p1*c3p3*c5p4*c9p1*ss[40];
  jac_4_0 += 25200*c1p1*c3p3*c7p1*c9p4*ss[54];
  jac_4_0 += 50400*c1p1*c3p3*c7p2*c9p3*ss[52];
  jac_4_0 += 50400*c1p1*c3p3*c7p3*c9p2*ss[50];
  jac_4_0 += 25200*c1p1*c3p3*c7p4*c9p1*ss[48];
  jac_4_0 += 25200*c1p1*c3p4*c5p1*c7p3*ss[40];
  jac_4_0 += 25200*c1p1*c3p4*c5p1*c9p3*ss[46];
  jac_4_0 += 37800*c1p1*c3p4*c5p2*c7p2*ss[38];
  jac_4_0 += 37800*c1p1*c3p4*c5p2*c9p2*ss[42];
  jac_4_0 += 25200*c1p1*c3p4*c5p3*c7p1*ss[36];
  jac_4_0 += 25200*c1p1*c3p4*c5p3*c9p1*ss[38];
  jac_4_0 += 25200*c1p1*c3p4*c7p1*c9p3*ss[48];
  jac_4_0 += 37800*c1p1*c3p4*c7p2*c9p2*ss[46];
  jac_4_0 += 25200*c1p1*c3p4*c7p3*c9p1*ss[44];
  jac_4_0 += 15120*c1p1*c3p5*c5p1*c7p2*ss[36];
  jac_4_0 += 15120*c1p1*c3p5*c5p1*c9p2*ss[40];
  jac_4_0 += 15120*c1p1*c3p5*c5p2*c7p1*ss[34];
  jac_4_0 += 15120*c1p1*c3p5*c5p2*c9p1*ss[36];
  jac_4_0 += 15120*c1p1*c3p5*c7p1*c9p2*ss[42];
  jac_4_0 += 15120*c1p1*c3p5*c7p2*c9p1*ss[40];
  jac_4_0 += 5040*c1p1*c3p6*c5p1*c7p1*ss[32];
  jac_4_0 += 5040*c1p1*c3p6*c5p1*c9p1*ss[34];
  jac_4_0 += 5040*c1p1*c3p6*c7p1*c9p1*ss[36];
  jac_4_0 += 5040*c1p1*c5p1*c7p1*c9p6*ss[68];
  jac_4_0 += 15120*c1p1*c5p1*c7p2*c9p5*ss[66];
  jac_4_0 += 25200*c1p1*c5p1*c7p3*c9p4*ss[64];
  jac_4_0 += 25200*c1p1*c5p1*c7p4*c9p3*ss[62];
  jac_4_0 += 15120*c1p1*c5p1*c7p5*c9p2*ss[60];
  jac_4_0 += 5040*c1p1*c5p1*c7p6*c9p1*ss[58];
  jac_4_0 += 15120*c1p1*c5p2*c7p1*c9p5*ss[64];
  jac_4_0 += 37800*c1p1*c5p2*c7p2*c9p4*ss[62];
  jac_4_0 += 50400*c1p1*c5p2*c7p3*c9p3*ss[60];
  jac_4_0 += 37800*c1p1*c5p2*c7p4*c9p2*ss[58];
  jac_4_0 += 15120*c1p1*c5p2*c7p5*c9p1*ss[56];
  jac_4_0 += 25200*c1p1*c5p3*c7p1*c9p4*ss[60];
  jac_4_0 += 50400*c1p1*c5p3*c7p2*c9p3*ss[58];
  jac_4_0 += 50400*c1p1*c5p3*c7p3*c9p2*ss[56];
  jac_4_0 += 25200*c1p1*c5p3*c7p4*c9p1*ss[54];
  jac_4_0 += 25200*c1p1*c5p4*c7p1*c9p3*ss[56];
  jac_4_0 += 37800*c1p1*c5p4*c7p2*c9p2*ss[54];
  jac_4_0 += 25200*c1p1*c5p4*c7p3*c9p1*ss[52];
  jac_4_0 += 15120*c1p1*c5p5*c7p1*c9p2*ss[52];
  jac_4_0 += 15120*c1p1*c5p5*c7p2*c9p1*ss[50];
  jac_4_0 += 5040*c1p1*c5p6*c7p1*c9p1*ss[48];
  jac_4_0 += 15120*c1p2*c3p1*c5p1*c7p5*ss[46];
  jac_4_0 += 15120*c1p2*c3p1*c5p1*c9p5*ss[56];
  jac_4_0 += 37800*c1p2*c3p1*c5p2*c7p4*ss[44];
  jac_4_0 += 37800*c1p2*c3p1*c5p2*c9p4*ss[52];
  jac_4_0 += 50400*c1p2*c3p1*c5p3*c7p3*ss[42];
  jac_4_0 += 50400*c1p2*c3p1*c5p3*c9p3*ss[48];
  jac_4_0 += 37800*c1p2*c3p1*c5p4*c7p2*ss[40];
  jac_4_0 += 37800*c1p2*c3p1*c5p4*c9p2*ss[44];
  jac_4_0 += 15120*c1p2*c3p1*c5p5*c7p1*ss[38];
  jac_4_0 += 15120*c1p2*c3p1*c5p5*c9p1*ss[40];
  jac_4_0 += 15120*c1p2*c3p1*c7p1*c9p5*ss[58];
  jac_4_0 += 37800*c1p2*c3p1*c7p2*c9p4*ss[56];
  jac_4_0 += 50400*c1p2*c3p1*c7p3*c9p3*ss[54];
  jac_4_0 += 37800*c1p2*c3p1*c7p4*c9p2*ss[52];
  jac_4_0 += 15120*c1p2*c3p1*c7p5*c9p1*ss[50];
  jac_4_0 += 37800*c1p2*c3p2*c5p1*c7p4*ss[42];
  jac_4_0 += 37800*c1p2*c3p2*c5p1*c9p4*ss[50];
  jac_4_0 += 75600*c1p2*c3p2*c5p2*c7p3*ss[40];
  jac_4_0 += 75600*c1p2*c3p2*c5p2*c9p3*ss[46];
  jac_4_0 += 75600*c1p2*c3p2*c5p3*c7p2*ss[38];
  jac_4_0 += 75600*c1p2*c3p2*c5p3*c9p2*ss[42];
  jac_4_0 += 37800*c1p2*c3p2*c5p4*c7p1*ss[36];
  jac_4_0 += 37800*c1p2*c3p2*c5p4*c9p1*ss[38];
  jac_4_0 += 37800*c1p2*c3p2*c7p1*c9p4*ss[52];
  jac_4_0 += 75600*c1p2*c3p2*c7p2*c9p3*ss[50];
  jac_4_0 += 75600*c1p2*c3p2*c7p3*c9p2*ss[48];
  jac_4_0 += 37800*c1p2*c3p2*c7p4*c9p1*ss[46];
  jac_4_0 += 50400*c1p2*c3p3*c5p1*c7p3*ss[38];
  jac_4_0 += 50400*c1p2*c3p3*c5p1*c9p3*ss[44];
  jac_4_0 += 75600*c1p2*c3p3*c5p2*c7p2*ss[36];
  jac_4_0 += 75600*c1p2*c3p3*c5p2*c9p2*ss[40];
  jac_4_0 += 50400*c1p2*c3p3*c5p3*c7p1*ss[34];
  jac_4_0 += 50400*c1p2*c3p3*c5p3*c9p1*ss[36];
  jac_4_0 += 50400*c1p2*c3p3*c7p1*c9p3*ss[46];
  jac_4_0 += 75600*c1p2*c3p3*c7p2*c9p2*ss[44];
  jac_4_0 += 50400*c1p2*c3p3*c7p3*c9p1*ss[42];
  jac_4_0 += 37800*c1p2*c3p4*c5p1*c7p2*ss[34];
  jac_4_0 += 37800*c1p2*c3p4*c5p1*c9p2*ss[38];
  jac_4_0 += 37800*c1p2*c3p4*c5p2*c7p1*ss[32];
  jac_4_0 += 37800*c1p2*c3p4*c5p2*c9p1*ss[34];
  jac_4_0 += 37800*c1p2*c3p4*c7p1*c9p2*ss[40];
  jac_4_0 += 37800*c1p2*c3p4*c7p2*c9p1*ss[38];
  jac_4_0 += 15120*c1p2*c3p5*c5p1*c7p1*ss[30];
  jac_4_0 += 15120*c1p2*c3p5*c5p1*c9p1*ss[32];
  jac_4_0 += 15120*c1p2*c3p5*c7p1*c9p1*ss[34];
  jac_4_0 += 15120*c1p2*c5p1*c7p1*c9p5*ss[60];
  jac_4_0 += 37800*c1p2*c5p1*c7p2*c9p4*ss[58];
  jac_4_0 += 50400*c1p2*c5p1*c7p3*c9p3*ss[56];
  jac_4_0 += 37800*c1p2*c5p1*c7p4*c9p2*ss[54];
  jac_4_0 += 15120*c1p2*c5p1*c7p5*c9p1*ss[52];
  jac_4_0 += 37800*c1p2*c5p2*c7p1*c9p4*ss[56];
  jac_4_0 += 75600*c1p2*c5p2*c7p2*c9p3*ss[54];
  jac_4_0 += 75600*c1p2*c5p2*c7p3*c9p2*ss[52];
  jac_4_0 += 37800*c1p2*c5p2*c7p4*c9p1*ss[50];
  jac_4_0 += 50400*c1p2*c5p3*c7p1*c9p3*ss[52];
  jac_4_0 += 75600*c1p2*c5p3*c7p2*c9p2*ss[50];
  jac_4_0 += 50400*c1p2*c5p3*c7p3*c9p1*ss[48];
  jac_4_0 += 37800*c1p2*c5p4*c7p1*c9p2*ss[48];
  jac_4_0 += 37800*c1p2*c5p4*c7p2*c9p1*ss[46];
  jac_4_0 += 15120*c1p2*c5p5*c7p1*c9p1*ss[44];
  jac_4_0 += 25200*c1p3*c3p1*c5p1*c7p4*ss[40];
  jac_4_0 += 25200*c1p3*c3p1*c5p1*c9p4*ss[48];
  jac_4_0 += 50400*c1p3*c3p1*c5p2*c7p3*ss[38];
  jac_4_0 += 50400*c1p3*c3p1*c5p2*c9p3*ss[44];
  jac_4_0 += 50400*c1p3*c3p1*c5p3*c7p2*ss[36];
  jac_4_0 += 50400*c1p3*c3p1*c5p3*c9p2*ss[40];
  jac_4_0 += 25200*c1p3*c3p1*c5p4*c7p1*ss[34];
  jac_4_0 += 25200*c1p3*c3p1*c5p4*c9p1*ss[36];
  jac_4_0 += 25200*c1p3*c3p1*c7p1*c9p4*ss[50];
  jac_4_0 += 50400*c1p3*c3p1*c7p2*c9p3*ss[48];
  jac_4_0 += 50400*c1p3*c3p1*c7p3*c9p2*ss[46];
  jac_4_0 += 25200*c1p3*c3p1*c7p4*c9p1*ss[44];
  jac_4_0 += 50400*c1p3*c3p2*c5p1*c7p3*ss[36];
  jac_4_0 += 50400*c1p3*c3p2*c5p1*c9p3*ss[42];
  jac_4_0 += 75600*c1p3*c3p2*c5p2*c7p2*ss[34];
  jac_4_0 += 75600*c1p3*c3p2*c5p2*c9p2*ss[38];
  jac_4_0 += 50400*c1p3*c3p2*c5p3*c7p1*ss[32];
  jac_4_0 += 50400*c1p3*c3p2*c5p3*c9p1*ss[34];
  jac_4_0 += 50400*c1p3*c3p2*c7p1*c9p3*ss[44];
  jac_4_0 += 75600*c1p3*c3p2*c7p2*c9p2*ss[42];
  jac_4_0 += 50400*c1p3*c3p2*c7p3*c9p1*ss[40];
  jac_4_0 += 50400*c1p3*c3p3*c5p1*c7p2*ss[32];
  jac_4_0 += 50400*c1p3*c3p3*c5p1*c9p2*ss[36];
  jac_4_0 += 50400*c1p3*c3p3*c5p2*c7p1*ss[30];
  jac_4_0 += 50400*c1p3*c3p3*c5p2*c9p1*ss[32];
  jac_4_0 += 50400*c1p3*c3p3*c7p1*c9p2*ss[38];
  jac_4_0 += 50400*c1p3*c3p3*c7p2*c9p1*ss[36];
  jac_4_0 += 25200*c1p3*c3p4*c5p1*c7p1*ss[28];
  jac_4_0 += 25200*c1p3*c3p4*c5p1*c9p1*ss[30];
  jac_4_0 += 25200*c1p3*c3p4*c7p1*c9p1*ss[32];
  jac_4_0 += 25200*c1p3*c5p1*c7p1*c9p4*ss[52];
  jac_4_0 += 50400*c1p3*c5p1*c7p2*c9p3*ss[50];
  jac_4_0 += 50400*c1p3*c5p1*c7p3*c9p2*ss[48];
  jac_4_0 += 25200*c1p3*c5p1*c7p4*c9p1*ss[46];
  jac_4_0 += 50400*c1p3*c5p2*c7p1*c9p3*ss[48];
  jac_4_0 += 75600*c1p3*c5p2*c7p2*c9p2*ss[46];
  jac_4_0 += 50400*c1p3*c5p2*c7p3*c9p1*ss[44];
  jac_4_0 += 50400*c1p3*c5p3*c7p1*c9p2*ss[44];
  jac_4_0 += 50400*c1p3*c5p3*c7p2*c9p1*ss[42];
  jac_4_0 += 25200*c1p3*c5p4*c7p1*c9p1*ss[40];
  jac_4_0 += 25200*c1p4*c3p1*c5p1*c7p3*ss[34];
  jac_4_0 += 25200*c1p4*c3p1*c5p1*c9p3*ss[40];
  jac_4_0 += 37800*c1p4*c3p1*c5p2*c7p2*ss[32];
  jac_4_0 += 37800*c1p4*c3p1*c5p2*c9p2*ss[36];
  jac_4_0 += 25200*c1p4*c3p1*c5p3*c7p1*ss[30];
  jac_4_0 += 25200*c1p4*c3p1*c5p3*c9p1*ss[32];
  jac_4_0 += 25200*c1p4*c3p1*c7p1*c9p3*ss[42];
  jac_4_0 += 37800*c1p4*c3p1*c7p2*c9p2*ss[40];
  jac_4_0 += 25200*c1p4*c3p1*c7p3*c9p1*ss[38];
  jac_4_0 += 37800*c1p4*c3p2*c5p1*c7p2*ss[30];
  jac_4_0 += 37800*c1p4*c3p2*c5p1*c9p2*ss[34];
  jac_4_0 += 37800*c1p4*c3p2*c5p2*c7p1*ss[28];
  jac_4_0 += 37800*c1p4*c3p2*c5p2*c9p1*ss[30];
  jac_4_0 += 37800*c1p4*c3p2*c7p1*c9p2*ss[36];
  jac_4_0 += 37800*c1p4*c3p2*c7p2*c9p1*ss[34];
  jac_4_0 += 25200*c1p4*c3p3*c5p1*c7p1*ss[26];
  jac_4_0 += 25200*c1p4*c3p3*c5p1*c9p1*ss[28];
  jac_4_0 += 25200*c1p4*c3p3*c7p1*c9p1*ss[30];
  jac_4_0 += 25200*c1p4*c5p1*c7p1*c9p3*ss[44];
  jac_4_0 += 37800*c1p4*c5p1*c7p2*c9p2*ss[42];
  jac_4_0 += 25200*c1p4*c5p1*c7p3*c9p1*ss[40];
  jac_4_0 += 37800*c1p4*c5p2*c7p1*c9p2*ss[40];
  jac_4_0 += 37800*c1p4*c5p2*c7p2*c9p1*ss[38];
  jac_4_0 += 25200*c1p4*c5p3*c7p1*c9p1*ss[36];
  jac_4_0 += 15120*c1p5*c3p1*c5p1*c7p2*ss[28];
  jac_4_0 += 15120*c1p5*c3p1*c5p1*c9p2*ss[32];
  jac_4_0 += 15120*c1p5*c3p1*c5p2*c7p1*ss[26];
  jac_4_0 += 15120*c1p5*c3p1*c5p2*c9p1*ss[28];
  jac_4_0 += 15120*c1p5*c3p1*c7p1*c9p2*ss[34];
  jac_4_0 += 15120*c1p5*c3p1*c7p2*c9p1*ss[32];
  jac_4_0 += 15120*c1p5*c3p2*c5p1*c7p1*ss[24];
  jac_4_0 += 15120*c1p5*c3p2*c5p1*c9p1*ss[26];
  jac_4_0 += 15120*c1p5*c3p2*c7p1*c9p1*ss[28];
  jac_4_0 += 15120*c1p5*c5p1*c7p1*c9p2*ss[36];
  jac_4_0 += 15120*c1p5*c5p1*c7p2*c9p1*ss[34];
  jac_4_0 += 15120*c1p5*c5p2*c7p1*c9p1*ss[32];
  jac_4_0 += 5040*c1p6*c3p1*c5p1*c7p1*ss[22];
  jac_4_0 += 5040*c1p6*c3p1*c5p1*c9p1*ss[24];
  jac_4_0 += 5040*c1p6*c3p1*c7p1*c9p1*ss[26];
  jac_4_0 += 5040*c1p6*c5p1*c7p1*c9p1*ss[28];
  jac_4_0 += 5040*c3p1*c5p1*c7p1*c9p6*ss[70];
  jac_4_0 += 15120*c3p1*c5p1*c7p2*c9p5*ss[68];
  jac_4_0 += 25200*c3p1*c5p1*c7p3*c9p4*ss[66];
  jac_4_0 += 25200*c3p1*c5p1*c7p4*c9p3*ss[64];
  jac_4_0 += 15120*c3p1*c5p1*c7p5*c9p2*ss[62];
  jac_4_0 += 5040*c3p1*c5p1*c7p6*c9p1*ss[60];
  jac_4_0 += 15120*c3p1*c5p2*c7p1*c9p5*ss[66];
  jac_4_0 += 37800*c3p1*c5p2*c7p2*c9p4*ss[64];
  jac_4_0 += 50400*c3p1*c5p2*c7p3*c9p3*ss[62];
  jac_4_0 += 37800*c3p1*c5p2*c7p4*c9p2*ss[60];
  jac_4_0 += 15120*c3p1*c5p2*c7p5*c9p1*ss[58];
  jac_4_0 += 25200*c3p1*c5p3*c7p1*c9p4*ss[62];
  jac_4_0 += 50400*c3p1*c5p3*c7p2*c9p3*ss[60];
  jac_4_0 += 50400*c3p1*c5p3*c7p3*c9p2*ss[58];
  jac_4_0 += 25200*c3p1*c5p3*c7p4*c9p1*ss[56];
  jac_4_0 += 25200*c3p1*c5p4*c7p1*c9p3*ss[58];
  jac_4_0 += 37800*c3p1*c5p4*c7p2*c9p2*ss[56];
  jac_4_0 += 25200*c3p1*c5p4*c7p3*c9p1*ss[54];
  jac_4_0 += 15120*c3p1*c5p5*c7p1*c9p2*ss[54];
  jac_4_0 += 15120*c3p1*c5p5*c7p2*c9p1*ss[52];
  jac_4_0 += 5040*c3p1*c5p6*c7p1*c9p1*ss[50];
  jac_4_0 += 15120*c3p2*c5p1*c7p1*c9p5*ss[64];
  jac_4_0 += 37800*c3p2*c5p1*c7p2*c9p4*ss[62];
  jac_4_0 += 50400*c3p2*c5p1*c7p3*c9p3*ss[60];
  jac_4_0 += 37800*c3p2*c5p1*c7p4*c9p2*ss[58];
  jac_4_0 += 15120*c3p2*c5p1*c7p5*c9p1*ss[56];
  jac_4_0 += 37800*c3p2*c5p2*c7p1*c9p4*ss[60];
  jac_4_0 += 75600*c3p2*c5p2*c7p2*c9p3*ss[58];
  jac_4_0 += 75600*c3p2*c5p2*c7p3*c9p2*ss[56];
  jac_4_0 += 37800*c3p2*c5p2*c7p4*c9p1*ss[54];
  jac_4_0 += 50400*c3p2*c5p3*c7p1*c9p3*ss[56];
  jac_4_0 += 75600*c3p2*c5p3*c7p2*c9p2*ss[54];
  jac_4_0 += 50400*c3p2*c5p3*c7p3*c9p1*ss[52];
  jac_4_0 += 37800*c3p2*c5p4*c7p1*c9p2*ss[52];
  jac_4_0 += 37800*c3p2*c5p4*c7p2*c9p1*ss[50];
  jac_4_0 += 15120*c3p2*c5p5*c7p1*c9p1*ss[48];
  jac_4_0 += 25200*c3p3*c5p1*c7p1*c9p4*ss[58];
  jac_4_0 += 50400*c3p3*c5p1*c7p2*c9p3*ss[56];
  jac_4_0 += 50400*c3p3*c5p1*c7p3*c9p2*ss[54];
  jac_4_0 += 25200*c3p3*c5p1*c7p4*c9p1*ss[52];
  jac_4_0 += 50400*c3p3*c5p2*c7p1*c9p3*ss[54];
  jac_4_0 += 75600*c3p3*c5p2*c7p2*c9p2*ss[52];
  jac_4_0 += 50400*c3p3*c5p2*c7p3*c9p1*ss[50];
  jac_4_0 += 50400*c3p3*c5p3*c7p1*c9p2*ss[50];
  jac_4_0 += 50400*c3p3*c5p3*c7p2*c9p1*ss[48];
  jac_4_0 += 25200*c3p3*c5p4*c7p1*c9p1*ss[46];
  jac_4_0 += 25200*c3p4*c5p1*c7p1*c9p3*ss[52];
  jac_4_0 += 37800*c3p4*c5p1*c7p2*c9p2*ss[50];
  jac_4_0 += 25200*c3p4*c5p1*c7p3*c9p1*ss[48];
  jac_4_0 += 37800*c3p4*c5p2*c7p1*c9p2*ss[48];
  jac_4_0 += 37800*c3p4*c5p2*c7p2*c9p1*ss[46];
  jac_4_0 += 25200*c3p4*c5p3*c7p1*c9p1*ss[44];
  jac_4_0 += 15120*c3p5*c5p1*c7p1*c9p2*ss[46];
  jac_4_0 += 15120*c3p5*c5p1*c7p2*c9p1*ss[44];
  jac_4_0 += 15120*c3p5*c5p2*c7p1*c9p1*ss[42];
  jac_4_0 += 5040*c3p6*c5p1*c7p1*c9p1*ss[40];
  jac_4_0 += 720*c1p1*c3p1*c5p7*ss[40];
  jac_4_0 += 720*c1p1*c3p1*c7p7*ss[54];
  jac_4_0 += 720*c1p1*c3p1*c9p7*ss[68];
  jac_4_0 += 2520*c1p1*c3p2*c5p6*ss[38];
  jac_4_0 += 2520*c1p1*c3p2*c7p6*ss[50];
  jac_4_0 += 2520*c1p1*c3p2*c9p6*ss[62];
  jac_4_0 += 5040*c1p1*c3p3*c5p5*ss[36];
  jac_4_0 += 5040*c1p1*c3p3*c7p5*ss[46];
  jac_4_0 += 5040*c1p1*c3p3*c9p5*ss[56];
  jac_4_0 += 6300*c1p1*c3p4*c5p4*ss[34];
  jac_4_0 += 6300*c1p1*c3p4*c7p4*ss[42];
  jac_4_0 += 6300*c1p1*c3p4*c9p4*ss[50];
  jac_4_0 += 5040*c1p1*c3p5*c5p3*ss[32];
  jac_4_0 += 5040*c1p1*c3p5*c7p3*ss[38];
  jac_4_0 += 5040*c1p1*c3p5*c9p3*ss[44];
  jac_4_0 += 2520*c1p1*c3p6*c5p2*ss[30];
  jac_4_0 += 2520*c1p1*c3p6*c7p2*ss[34];
  jac_4_0 += 2520*c1p1*c3p6*c9p2*ss[38];
  jac_4_0 += 720*c1p1*c3p7*c5p1*ss[28];
  jac_4_0 += 720*c1p1*c3p7*c7p1*ss[30];
  jac_4_0 += 720*c1p1*c3p7*c9p1*ss[32];
  jac_4_0 += 720*c1p1*c5p1*c7p7*ss[56];
  jac_4_0 += 720*c1p1*c5p1*c9p7*ss[70];
  jac_4_0 += 2520*c1p1*c5p2*c7p6*ss[54];
  jac_4_0 += 2520*c1p1*c5p2*c9p6*ss[66];
  jac_4_0 += 5040*c1p1*c5p3*c7p5*ss[52];
  jac_4_0 += 5040*c1p1*c5p3*c9p5*ss[62];
  jac_4_0 += 6300*c1p1*c5p4*c7p4*ss[50];
  jac_4_0 += 6300*c1p1*c5p4*c9p4*ss[58];
  jac_4_0 += 5040*c1p1*c5p5*c7p3*ss[48];
  jac_4_0 += 5040*c1p1*c5p5*c9p3*ss[54];
  jac_4_0 += 2520*c1p1*c5p6*c7p2*ss[46];
  jac_4_0 += 2520*c1p1*c5p6*c9p2*ss[50];
  jac_4_0 += 720*c1p1*c5p7*c7p1*ss[44];
  jac_4_0 += 720*c1p1*c5p7*c9p1*ss[46];
  jac_4_0 += 720*c1p1*c7p1*c9p7*ss[72];
  jac_4_0 += 2520*c1p1*c7p2*c9p6*ss[70];
  jac_4_0 += 5040*c1p1*c7p3*c9p5*ss[68];
  jac_4_0 += 6300*c1p1*c7p4*c9p4*ss[66];
  jac_4_0 += 5040*c1p1*c7p5*c9p3*ss[64];
  jac_4_0 += 2520*c1p1*c7p6*c9p2*ss[62];
  jac_4_0 += 720*c1p1*c7p7*c9p1*ss[60];
  jac_4_0 += 2520*c1p2*c3p1*c5p6*ss[36];
  jac_4_0 += 2520*c1p2*c3p1*c7p6*ss[48];
  jac_4_0 += 2520*c1p2*c3p1*c9p6*ss[60];
  jac_4_0 += 7560*c1p2*c3p2*c5p5*ss[34];
  jac_4_0 += 7560*c1p2*c3p2*c7p5*ss[44];
  jac_4_0 += 7560*c1p2*c3p2*c9p5*ss[54];
  jac_4_0 += 12600*c1p2*c3p3*c5p4*ss[32];
  jac_4_0 += 12600*c1p2*c3p3*c7p4*ss[40];
  jac_4_0 += 12600*c1p2*c3p3*c9p4*ss[48];
  jac_4_0 += 12600*c1p2*c3p4*c5p3*ss[30];
  jac_4_0 += 12600*c1p2*c3p4*c7p3*ss[36];
  jac_4_0 += 12600*c1p2*c3p4*c9p3*ss[42];
  jac_4_0 += 7560*c1p2*c3p5*c5p2*ss[28];
  jac_4_0 += 7560*c1p2*c3p5*c7p2*ss[32];
  jac_4_0 += 7560*c1p2*c3p5*c9p2*ss[36];
  jac_4_0 += 2520*c1p2*c3p6*c5p1*ss[26];
  jac_4_0 += 2520*c1p2*c3p6*c7p1*ss[28];
  jac_4_0 += 2520*c1p2*c3p6*c9p1*ss[30];
  jac_4_0 += 2520*c1p2*c5p1*c7p6*ss[50];
  jac_4_0 += 2520*c1p2*c5p1*c9p6*ss[62];
  jac_4_0 += 7560*c1p2*c5p2*c7p5*ss[48];
  jac_4_0 += 7560*c1p2*c5p2*c9p5*ss[58];
  jac_4_0 += 12600*c1p2*c5p3*c7p4*ss[46];
  jac_4_0 += 12600*c1p2*c5p3*c9p4*ss[54];
  jac_4_0 += 12600*c1p2*c5p4*c7p3*ss[44];
  jac_4_0 += 12600*c1p2*c5p4*c9p3*ss[50];
  jac_4_0 += 7560*c1p2*c5p5*c7p2*ss[42];
  jac_4_0 += 7560*c1p2*c5p5*c9p2*ss[46];
  jac_4_0 += 2520*c1p2*c5p6*c7p1*ss[40];
  jac_4_0 += 2520*c1p2*c5p6*c9p1*ss[42];
  jac_4_0 += 2520*c1p2*c7p1*c9p6*ss[64];
  jac_4_0 += 7560*c1p2*c7p2*c9p5*ss[62];
  jac_4_0 += 12600*c1p2*c7p3*c9p4*ss[60];
  jac_4_0 += 12600*c1p2*c7p4*c9p3*ss[58];
  jac_4_0 += 7560*c1p2*c7p5*c9p2*ss[56];
  jac_4_0 += 2520*c1p2*c7p6*c9p1*ss[54];
  jac_4_0 += 5040*c1p3*c3p1*c5p5*ss[32];
  jac_4_0 += 5040*c1p3*c3p1*c7p5*ss[42];
  jac_4_0 += 5040*c1p3*c3p1*c9p5*ss[52];
  jac_4_0 += 12600*c1p3*c3p2*c5p4*ss[30];
  jac_4_0 += 12600*c1p3*c3p2*c7p4*ss[38];
  jac_4_0 += 12600*c1p3*c3p2*c9p4*ss[46];
  jac_4_0 += 16800*c1p3*c3p3*c5p3*ss[28];
  jac_4_0 += 16800*c1p3*c3p3*c7p3*ss[34];
  jac_4_0 += 16800*c1p3*c3p3*c9p3*ss[40];
  jac_4_0 += 12600*c1p3*c3p4*c5p2*ss[26];
  jac_4_0 += 12600*c1p3*c3p4*c7p2*ss[30];
  jac_4_0 += 12600*c1p3*c3p4*c9p2*ss[34];
  jac_4_0 += 5040*c1p3*c3p5*c5p1*ss[24];
  jac_4_0 += 5040*c1p3*c3p5*c7p1*ss[26];
  jac_4_0 += 5040*c1p3*c3p5*c9p1*ss[28];
  jac_4_0 += 5040*c1p3*c5p1*c7p5*ss[44];
  jac_4_0 += 5040*c1p3*c5p1*c9p5*ss[54];
  jac_4_0 += 12600*c1p3*c5p2*c7p4*ss[42];
  jac_4_0 += 12600*c1p3*c5p2*c9p4*ss[50];
  jac_4_0 += 16800*c1p3*c5p3*c7p3*ss[40];
  jac_4_0 += 16800*c1p3*c5p3*c9p3*ss[46];
  jac_4_0 += 12600*c1p3*c5p4*c7p2*ss[38];
  jac_4_0 += 12600*c1p3*c5p4*c9p2*ss[42];
  jac_4_0 += 5040*c1p3*c5p5*c7p1*ss[36];
  jac_4_0 += 5040*c1p3*c5p5*c9p1*ss[38];
  jac_4_0 += 5040*c1p3*c7p1*c9p5*ss[56];
  jac_4_0 += 12600*c1p3*c7p2*c9p4*ss[54];
  jac_4_0 += 16800*c1p3*c7p3*c9p3*ss[52];
  jac_4_0 += 12600*c1p3*c7p4*c9p2*ss[50];
  jac_4_0 += 5040*c1p3*c7p5*c9p1*ss[48];
  jac_4_0 += 6300*c1p4*c3p1*c5p4*ss[28];
  jac_4_0 += 6300*c1p4*c3p1*c7p4*ss[36];
  jac_4_0 += 6300*c1p4*c3p1*c9p4*ss[44];
  jac_4_0 += 12600*c1p4*c3p2*c5p3*ss[26];
  jac_4_0 += 12600*c1p4*c3p2*c7p3*ss[32];
  jac_4_0 += 12600*c1p4*c3p2*c9p3*ss[38];
  jac_4_0 += 12600*c1p4*c3p3*c5p2*ss[24];
  jac_4_0 += 12600*c1p4*c3p3*c7p2*ss[28];
  jac_4_0 += 12600*c1p4*c3p3*c9p2*ss[32];
  jac_4_0 += 6300*c1p4*c3p4*c5p1*ss[22];
  jac_4_0 += 6300*c1p4*c3p4*c7p1*ss[24];
  jac_4_0 += 6300*c1p4*c3p4*c9p1*ss[26];
  jac_4_0 += 6300*c1p4*c5p1*c7p4*ss[38];
  jac_4_0 += 6300*c1p4*c5p1*c9p4*ss[46];
  jac_4_0 += 12600*c1p4*c5p2*c7p3*ss[36];
  jac_4_0 += 12600*c1p4*c5p2*c9p3*ss[42];
  jac_4_0 += 12600*c1p4*c5p3*c7p2*ss[34];
  jac_4_0 += 12600*c1p4*c5p3*c9p2*ss[38];
  jac_4_0 += 6300*c1p4*c5p4*c7p1*ss[32];
  jac_4_0 += 6300*c1p4*c5p4*c9p1*ss[34];
  jac_4_0 += 6300*c1p4*c7p1*c9p4*ss[48];
  jac_4_0 += 12600*c1p4*c7p2*c9p3*ss[46];
  jac_4_0 += 12600*c1p4*c7p3*c9p2*ss[44];
  jac_4_0 += 6300*c1p4*c7p4*c9p1*ss[42];
  jac_4_0 += 5040*c1p5*c3p1*c5p3*ss[24];
  jac_4_0 += 5040*c1p5*c3p1*c7p3*ss[30];
  jac_4_0 += 5040*c1p5*c3p1*c9p3*ss[36];
  jac_4_0 += 7560*c1p5*c3p2*c5p2*ss[22];
  jac_4_0 += 7560*c1p5*c3p2*c7p2*ss[26];
  jac_4_0 += 7560*c1p5*c3p2*c9p2*ss[30];
  jac_4_0 += 5040*c1p5*c3p3*c5p1*ss[20];
  jac_4_0 += 5040*c1p5*c3p3*c7p1*ss[22];
  jac_4_0 += 5040*c1p5*c3p3*c9p1*ss[24];
  jac_4_0 += 5040*c1p5*c5p1*c7p3*ss[32];
  jac_4_0 += 5040*c1p5*c5p1*c9p3*ss[38];
  jac_4_0 += 7560*c1p5*c5p2*c7p2*ss[30];
  jac_4_0 += 7560*c1p5*c5p2*c9p2*ss[34];
  jac_4_0 += 5040*c1p5*c5p3*c7p1*ss[28];
  jac_4_0 += 5040*c1p5*c5p3*c9p1*ss[30];
  jac_4_0 += 5040*c1p5*c7p1*c9p3*ss[40];
  jac_4_0 += 7560*c1p5*c7p2*c9p2*ss[38];
  jac_4_0 += 5040*c1p5*c7p3*c9p1*ss[36];
  jac_4_0 += 2520*c1p6*c3p1*c5p2*ss[20];
  jac_4_0 += 2520*c1p6*c3p1*c7p2*ss[24];
  jac_4_0 += 2520*c1p6*c3p1*c9p2*ss[28];
  jac_4_0 += 2520*c1p6*c3p2*c5p1*ss[18];
  jac_4_0 += 2520*c1p6*c3p2*c7p1*ss[20];
  jac_4_0 += 2520*c1p6*c3p2*c9p1*ss[22];
  jac_4_0 += 2520*c1p6*c5p1*c7p2*ss[26];
  jac_4_0 += 2520*c1p6*c5p1*c9p2*ss[30];
  jac_4_0 += 2520*c1p6*c5p2*c7p1*ss[24];
  jac_4_0 += 2520*c1p6*c5p2*c9p1*ss[26];
  jac_4_0 += 2520*c1p6*c7p1*c9p2*ss[32];
  jac_4_0 += 2520*c1p6*c7p2*c9p1*ss[30];
  jac_4_0 += 720*c1p7*c3p1*c5p1*ss[16];
  jac_4_0 += 720*c1p7*c3p1*c7p1*ss[18];
  jac_4_0 += 720*c1p7*c3p1*c9p1*ss[20];
  jac_4_0 += 720*c1p7*c5p1*c7p1*ss[20];
  jac_4_0 += 720*c1p7*c5p1*c9p1*ss[22];
  jac_4_0 += 720*c1p7*c7p1*c9p1*ss[24];
  jac_4_0 += 720*c3p1*c5p1*c7p7*ss[58];
  jac_4_0 += 720*c3p1*c5p1*c9p7*ss[72];
  jac_4_0 += 2520*c3p1*c5p2*c7p6*ss[56];
  jac_4_0 += 2520*c3p1*c5p2*c9p6*ss[68];
  jac_4_0 += 5040*c3p1*c5p3*c7p5*ss[54];
  jac_4_0 += 5040*c3p1*c5p3*c9p5*ss[64];
  jac_4_0 += 6300*c3p1*c5p4*c7p4*ss[52];
  jac_4_0 += 6300*c3p1*c5p4*c9p4*ss[60];
  jac_4_0 += 5040*c3p1*c5p5*c7p3*ss[50];
  jac_4_0 += 5040*c3p1*c5p5*c9p3*ss[56];
  jac_4_0 += 2520*c3p1*c5p6*c7p2*ss[48];
  jac_4_0 += 2520*c3p1*c5p6*c9p2*ss[52];
  jac_4_0 += 720*c3p1*c5p7*c7p1*ss[46];
  jac_4_0 += 720*c3p1*c5p7*c9p1*ss[48];
  jac_4_0 += 720*c3p1*c7p1*c9p7*ss[74];
  jac_4_0 += 2520*c3p1*c7p2*c9p6*ss[72];
  jac_4_0 += 5040*c3p1*c7p3*c9p5*ss[70];
  jac_4_0 += 6300*c3p1*c7p4*c9p4*ss[68];
  jac_4_0 += 5040*c3p1*c7p5*c9p3*ss[66];
  jac_4_0 += 2520*c3p1*c7p6*c9p2*ss[64];
  jac_4_0 += 720*c3p1*c7p7*c9p1*ss[62];
  jac_4_0 += 2520*c3p2*c5p1*c7p6*ss[54];
  jac_4_0 += 2520*c3p2*c5p1*c9p6*ss[66];
  jac_4_0 += 7560*c3p2*c5p2*c7p5*ss[52];
  jac_4_0 += 7560*c3p2*c5p2*c9p5*ss[62];
  jac_4_0 += 12600*c3p2*c5p3*c7p4*ss[50];
  jac_4_0 += 12600*c3p2*c5p3*c9p4*ss[58];
  jac_4_0 += 12600*c3p2*c5p4*c7p3*ss[48];
  jac_4_0 += 12600*c3p2*c5p4*c9p3*ss[54];
  jac_4_0 += 7560*c3p2*c5p5*c7p2*ss[46];
  jac_4_0 += 7560*c3p2*c5p5*c9p2*ss[50];
  jac_4_0 += 2520*c3p2*c5p6*c7p1*ss[44];
  jac_4_0 += 2520*c3p2*c5p6*c9p1*ss[46];
  jac_4_0 += 2520*c3p2*c7p1*c9p6*ss[68];
  jac_4_0 += 7560*c3p2*c7p2*c9p5*ss[66];
  jac_4_0 += 12600*c3p2*c7p3*c9p4*ss[64];
  jac_4_0 += 12600*c3p2*c7p4*c9p3*ss[62];
  jac_4_0 += 7560*c3p2*c7p5*c9p2*ss[60];
  jac_4_0 += 2520*c3p2*c7p6*c9p1*ss[58];
  jac_4_0 += 5040*c3p3*c5p1*c7p5*ss[50];
  jac_4_0 += 5040*c3p3*c5p1*c9p5*ss[60];
  jac_4_0 += 12600*c3p3*c5p2*c7p4*ss[48];
  jac_4_0 += 12600*c3p3*c5p2*c9p4*ss[56];
  jac_4_0 += 16800*c3p3*c5p3*c7p3*ss[46];
  jac_4_0 += 16800*c3p3*c5p3*c9p3*ss[52];
  jac_4_0 += 12600*c3p3*c5p4*c7p2*ss[44];
  jac_4_0 += 12600*c3p3*c5p4*c9p2*ss[48];
  jac_4_0 += 5040*c3p3*c5p5*c7p1*ss[42];
  jac_4_0 += 5040*c3p3*c5p5*c9p1*ss[44];
  jac_4_0 += 5040*c3p3*c7p1*c9p5*ss[62];
  jac_4_0 += 12600*c3p3*c7p2*c9p4*ss[60];
  jac_4_0 += 16800*c3p3*c7p3*c9p3*ss[58];
  jac_4_0 += 12600*c3p3*c7p4*c9p2*ss[56];
  jac_4_0 += 5040*c3p3*c7p5*c9p1*ss[54];
  jac_4_0 += 6300*c3p4*c5p1*c7p4*ss[46];
  jac_4_0 += 6300*c3p4*c5p1*c9p4*ss[54];
  jac_4_0 += 12600*c3p4*c5p2*c7p3*ss[44];
  jac_4_0 += 12600*c3p4*c5p2*c9p3*ss[50];
  jac_4_0 += 12600*c3p4*c5p3*c7p2*ss[42];
  jac_4_0 += 12600*c3p4*c5p3*c9p2*ss[46];
  jac_4_0 += 6300*c3p4*c5p4*c7p1*ss[40];
  jac_4_0 += 6300*c3p4*c5p4*c9p1*ss[42];
  jac_4_0 += 6300*c3p4*c7p1*c9p4*ss[56];
  jac_4_0 += 12600*c3p4*c7p2*c9p3*ss[54];
  jac_4_0 += 12600*c3p4*c7p3*c9p2*ss[52];
  jac_4_0 += 6300*c3p4*c7p4*c9p1*ss[50];
  jac_4_0 += 5040*c3p5*c5p1*c7p3*ss[42];
  jac_4_0 += 5040*c3p5*c5p1*c9p3*ss[48];
  jac_4_0 += 7560*c3p5*c5p2*c7p2*ss[40];
  jac_4_0 += 7560*c3p5*c5p2*c9p2*ss[44];
  jac_4_0 += 5040*c3p5*c5p3*c7p1*ss[38];
  jac_4_0 += 5040*c3p5*c5p3*c9p1*ss[40];
  jac_4_0 += 5040*c3p5*c7p1*c9p3*ss[50];
  jac_4_0 += 7560*c3p5*c7p2*c9p2*ss[48];
  jac_4_0 += 5040*c3p5*c7p3*c9p1*ss[46];
  jac_4_0 += 2520*c3p6*c5p1*c7p2*ss[38];
  jac_4_0 += 2520*c3p6*c5p1*c9p2*ss[42];
  jac_4_0 += 2520*c3p6*c5p2*c7p1*ss[36];
  jac_4_0 += 2520*c3p6*c5p2*c9p1*ss[38];
  jac_4_0 += 2520*c3p6*c7p1*c9p2*ss[44];
  jac_4_0 += 2520*c3p6*c7p2*c9p1*ss[42];
  jac_4_0 += 720*c3p7*c5p1*c7p1*ss[34];
  jac_4_0 += 720*c3p7*c5p1*c9p1*ss[36];
  jac_4_0 += 720*c3p7*c7p1*c9p1*ss[38];
  jac_4_0 += 720*c5p1*c7p1*c9p7*ss[76];
  jac_4_0 += 2520*c5p1*c7p2*c9p6*ss[74];
  jac_4_0 += 5040*c5p1*c7p3*c9p5*ss[72];
  jac_4_0 += 6300*c5p1*c7p4*c9p4*ss[70];
  jac_4_0 += 5040*c5p1*c7p5*c9p3*ss[68];
  jac_4_0 += 2520*c5p1*c7p6*c9p2*ss[66];
  jac_4_0 += 720*c5p1*c7p7*c9p1*ss[64];
  jac_4_0 += 2520*c5p2*c7p1*c9p6*ss[72];
  jac_4_0 += 7560*c5p2*c7p2*c9p5*ss[70];
  jac_4_0 += 12600*c5p2*c7p3*c9p4*ss[68];
  jac_4_0 += 12600*c5p2*c7p4*c9p3*ss[66];
  jac_4_0 += 7560*c5p2*c7p5*c9p2*ss[64];
  jac_4_0 += 2520*c5p2*c7p6*c9p1*ss[62];
  jac_4_0 += 5040*c5p3*c7p1*c9p5*ss[68];
  jac_4_0 += 12600*c5p3*c7p2*c9p4*ss[66];
  jac_4_0 += 16800*c5p3*c7p3*c9p3*ss[64];
  jac_4_0 += 12600*c5p3*c7p4*c9p2*ss[62];
  jac_4_0 += 5040*c5p3*c7p5*c9p1*ss[60];
  jac_4_0 += 6300*c5p4*c7p1*c9p4*ss[64];
  jac_4_0 += 12600*c5p4*c7p2*c9p3*ss[62];
  jac_4_0 += 12600*c5p4*c7p3*c9p2*ss[60];
  jac_4_0 += 6300*c5p4*c7p4*c9p1*ss[58];
  jac_4_0 += 5040*c5p5*c7p1*c9p3*ss[60];
  jac_4_0 += 7560*c5p5*c7p2*c9p2*ss[58];
  jac_4_0 += 5040*c5p5*c7p3*c9p1*ss[56];
  jac_4_0 += 2520*c5p6*c7p1*c9p2*ss[56];
  jac_4_0 += 2520*c5p6*c7p2*c9p1*ss[54];
  jac_4_0 += 720*c5p7*c7p1*c9p1*ss[52];
  jac_4_0 += 90*c1p1*c3p8*ss[26];
  jac_4_0 += 90*c1p1*c5p8*ss[42];
  jac_4_0 += 90*c1p1*c7p8*ss[58];
  jac_4_0 += 90*c1p1*c9p8*ss[74];
  jac_4_0 += 360*c1p2*c3p7*ss[24];
  jac_4_0 += 360*c1p2*c5p7*ss[38];
  jac_4_0 += 360*c1p2*c7p7*ss[52];
  jac_4_0 += 360*c1p2*c9p7*ss[66];
  jac_4_0 += 840*c1p3*c3p6*ss[22];
  jac_4_0 += 840*c1p3*c5p6*ss[34];
  jac_4_0 += 840*c1p3*c7p6*ss[46];
  jac_4_0 += 840*c1p3*c9p6*ss[58];
  jac_4_0 += 1260*c1p4*c3p5*ss[20];
  jac_4_0 += 1260*c1p4*c5p5*ss[30];
  jac_4_0 += 1260*c1p4*c7p5*ss[40];
  jac_4_0 += 1260*c1p4*c9p5*ss[50];
  jac_4_0 += 1260*c1p5*c3p4*ss[18];
  jac_4_0 += 1260*c1p5*c5p4*ss[26];
  jac_4_0 += 1260*c1p5*c7p4*ss[34];
  jac_4_0 += 1260*c1p5*c9p4*ss[42];
  jac_4_0 += 840*c1p6*c3p3*ss[16];
  jac_4_0 += 840*c1p6*c5p3*ss[22];
  jac_4_0 += 840*c1p6*c7p3*ss[28];
  jac_4_0 += 840*c1p6*c9p3*ss[34];
  jac_4_0 += 360*c1p7*c3p2*ss[14];
  jac_4_0 += 360*c1p7*c5p2*ss[18];
  jac_4_0 += 360*c1p7*c7p2*ss[22];
  jac_4_0 += 360*c1p7*c9p2*ss[26];
  jac_4_0 += 90*c1p8*c3p1*ss[12];
  jac_4_0 += 90*c1p8*c5p1*ss[14];
  jac_4_0 += 90*c1p8*c7p1*ss[16];
  jac_4_0 += 90*c1p8*c9p1*ss[18];
  jac_4_0 += 90*c3p1*c5p8*ss[44];
  jac_4_0 += 90*c3p1*c7p8*ss[60];
  jac_4_0 += 90*c3p1*c9p8*ss[76];
  jac_4_0 += 360*c3p2*c5p7*ss[42];
  jac_4_0 += 360*c3p2*c7p7*ss[56];
  jac_4_0 += 360*c3p2*c9p7*ss[70];
  jac_4_0 += 840*c3p3*c5p6*ss[40];
  jac_4_0 += 840*c3p3*c7p6*ss[52];
  jac_4_0 += 840*c3p3*c9p6*ss[64];
  jac_4_0 += 1260*c3p4*c5p5*ss[38];
  jac_4_0 += 1260*c3p4*c7p5*ss[48];
  jac_4_0 += 1260*c3p4*c9p5*ss[58];
  jac_4_0 += 1260*c3p5*c5p4*ss[36];
  jac_4_0 += 1260*c3p5*c7p4*ss[44];
  jac_4_0 += 1260*c3p5*c9p4*ss[52];
  jac_4_0 += 840*c3p6*c5p3*ss[34];
  jac_4_0 += 840*c3p6*c7p3*ss[40];
  jac_4_0 += 840*c3p6*c9p3*ss[46];
  jac_4_0 += 360*c3p7*c5p2*ss[32];
  jac_4_0 += 360*c3p7*c7p2*ss[36];
  jac_4_0 += 360*c3p7*c9p2*ss[40];
  jac_4_0 += 90*c3p8*c5p1*ss[30];
  jac_4_0 += 90*c3p8*c7p1*ss[32];
  jac_4_0 += 90*c3p8*c9p1*ss[34];
  jac_4_0 += 90*c5p1*c7p8*ss[62];
  jac_4_0 += 90*c5p1*c9p8*ss[78];
  jac_4_0 += 360*c5p2*c7p7*ss[60];
  jac_4_0 += 360*c5p2*c9p7*ss[74];
  jac_4_0 += 840*c5p3*c7p6*ss[58];
  jac_4_0 += 840*c5p3*c9p6*ss[70];
  jac_4_0 += 1260*c5p4*c7p5*ss[56];
  jac_4_0 += 1260*c5p4*c9p5*ss[66];
  jac_4_0 += 1260*c5p5*c7p4*ss[54];
  jac_4_0 += 1260*c5p5*c9p4*ss[62];
  jac_4_0 += 840*c5p6*c7p3*ss[52];
  jac_4_0 += 840*c5p6*c9p3*ss[58];
  jac_4_0 += 360*c5p7*c7p2*ss[50];
  jac_4_0 += 360*c5p7*c9p2*ss[54];
  jac_4_0 += 90*c5p8*c7p1*ss[48];
  jac_4_0 += 90*c5p8*c9p1*ss[50];
  jac_4_0 += 90*c7p1*c9p8*ss[80];
  jac_4_0 += 360*c7p2*c9p7*ss[78];
  jac_4_0 += 840*c7p3*c9p6*ss[76];
  jac_4_0 += 1260*c7p4*c9p5*ss[74];
  jac_4_0 += 1260*c7p5*c9p4*ss[72];
  jac_4_0 += 840*c7p6*c9p3*ss[70];
  jac_4_0 += 360*c7p7*c9p2*ss[68];
  jac_4_0 += 90*c7p8*c9p1*ss[66];
  jac_4_0 += 10*c1p9*ss[10];
  jac_4_0 += 10*c3p9*ss[28];
  jac_4_0 += 10*c5p9*ss[46];
  jac_4_0 += 10*c7p9*ss[64];
  jac_4_0 += 10*c9p9*ss[82];
  TACC jac_4_1 = convert<TACC>(30240*c1p1*c3p1*c5p1*c7p1*c9p5*ss[64]);
  jac_4_1 += 75600*c1p1*c3p1*c5p1*c7p2*c9p4*ss[62];
  jac_4_1 += 100800*c1p1*c3p1*c5p1*c7p3*c9p3*ss[60];
  jac_4_1 += 75600*c1p1*c3p1*c5p1*c7p4*c9p2*ss[58];
  jac_4_1 += 30240*c1p1*c3p1*c5p1*c7p5*c9p1*ss[56];
  jac_4_1 += 75600*c1p1*c3p1*c5p2*c7p1*c9p4*ss[60];
  jac_4_1 += 151200*c1p1*c3p1*c5p2*c7p2*c9p3*ss[58];
  jac_4_1 += 151200*c1p1*c3p1*c5p2*c7p3*c9p2*ss[56];
  jac_4_1 += 75600*c1p1*c3p1*c5p2*c7p4*c9p1*ss[54];
  jac_4_1 += 100800*c1p1*c3p1*c5p3*c7p1*c9p3*ss[56];
  jac_4_1 += 151200*c1p1*c3p1*c5p3*c7p2*c9p2*ss[54];
  jac_4_1 += 100800*c1p1*c3p1*c5p3*c7p3*c9p1*ss[52];
  jac_4_1 += 75600*c1p1*c3p1*c5p4*c7p1*c9p2*ss[52];
  jac_4_1 += 75600*c1p1*c3p1*c5p4*c7p2*c9p1*ss[50];
  jac_4_1 += 30240*c1p1*c3p1*c5p5*c7p1*c9p1*ss[48];
  jac_4_1 += 75600*c1p1*c3p2*c5p1*c7p1*c9p4*ss[58];
  jac_4_1 += 151200*c1p1*c3p2*c5p1*c7p2*c9p3*ss[56];
  jac_4_1 += 151200*c1p1*c3p2*c5p1*c7p3*c9p2*ss[54];
  jac_4_1 += 75600*c1p1*c3p2*c5p1*c7p4*c9p1*ss[52];
  jac_4_1 += 151200*c1p1*c3p2*c5p2*c7p1*c9p3*ss[54];
  jac_4_1 += 226800*c1p1*c3p2*c5p2*c7p2*c9p2*ss[52];
  jac_4_1 += 151200*c1p1*c3p2*c5p2*c7p3*c9p1*ss[50];
  jac_4_1 += 151200*c1p1*c3p2*c5p3*c7p1*c9p2*ss[50];
  jac_4_1 += 151200*c1p1*c3p2*c5p3*c7p2*c9p1*ss[48];
  jac_4_1 += 75600*c1p1*c3p2*c5p4*c7p1*c9p1*ss[46];
  jac_4_1 += 100800*c1p1*c3p3*c5p1*c7p1*c9p3*ss[52];
  jac_4_1 += 151200*c1p1*c3p3*c5p1*c7p2*c9p2*ss[50];
  jac_4_1 += 100800*c1p1*c3p3*c5p1*c7p3*c9p1*ss[48];
  jac_4_1 += 151200*c1p1*c3p3*c5p2*c7p1*c9p2*ss[48];
  jac_4_1 += 151200*c1p1*c3p3*c5p2*c7p2*c9p1*ss[46];
  jac_4_1 += 100800*c1p1*c3p3*c5p3*c7p1*c9p1*ss[44];
  jac_4_1 += 75600*c1p1*c3p4*c5p1*c7p1*c9p2*ss[46];
  jac_4_1 += 75600*c1p1*c3p4*c5p1*c7p2*c9p1*ss[44];
  jac_4_1 += 75600*c1p1*c3p4*c5p2*c7p1*c9p1*ss[42];
  jac_4_1 += 30240*c1p1*c3p5*c5p1*c7p1*c9p1*ss[40];
  jac_4_1 += 75600*c1p2*c3p1*c5p1*c7p1*c9p4*ss[56];
  jac_4_1 += 151200*c1p2*c3p1*c5p1*c7p2*c9p3*ss[54];
  jac_4_1 += 151200*c1p2*c3p1*c5p1*c7p3*c9p2*ss[52];
  jac_4_1 += 75600*c1p2*c3p1*c5p1*c7p4*c9p1*ss[50];
  jac_4_1 += 151200*c1p2*c3p1*c5p2*c7p1*c9p3*ss[52];
  jac_4_1 += 226800*c1p2*c3p1*c5p2*c7p2*c9p2*ss[50];
  jac_4_1 += 151200*c1p2*c3p1*c5p2*c7p3*c9p1*ss[48];
  jac_4_1 += 151200*c1p2*c3p1*c5p3*c7p1*c9p2*ss[48];
  jac_4_1 += 151200*c1p2*c3p1*c5p3*c7p2*c9p1*ss[46];
  jac_4_1 += 75600*c1p2*c3p1*c5p4*c7p1*c9p1*ss[44];
  jac_4_1 += 151200*c1p2*c3p2*c5p1*c7p1*c9p3*ss[50];
  jac_4_1 += 226800*c1p2*c3p2*c5p1*c7p2*c9p2*ss[48];
  jac_4_1 += 151200*c1p2*c3p2*c5p1*c7p3*c9p1*ss[46];
  jac_4_1 += 226800*c1p2*c3p2*c5p2*c7p1*c9p2*ss[46];
  jac_4_1 += 226800*c1p2*c3p2*c5p2*c7p2*c9p1*ss[44];
  jac_4_1 += 151200*c1p2*c3p2*c5p3*c7p1*c9p1*ss[42];
  jac_4_1 += 151200*c1p2*c3p3*c5p1*c7p1*c9p2*ss[44];
  jac_4_1 += 151200*c1p2*c3p3*c5p1*c7p2*c9p1*ss[42];
  jac_4_1 += 151200*c1p2*c3p3*c5p2*c7p1*c9p1*ss[40];
  jac_4_1 += 75600*c1p2*c3p4*c5p1*c7p1*c9p1*ss[38];
  jac_4_1 += 100800*c1p3*c3p1*c5p1*c7p1*c9p3*ss[48];
  jac_4_1 += 151200*c1p3*c3p1*c5p1*c7p2*c9p2*ss[46];
  jac_4_1 += 100800*c1p3*c3p1*c5p1*c7p3*c9p1*ss[44];
  jac_4_1 += 151200*c1p3*c3p1*c5p2*c7p1*c9p2*ss[44];
  jac_4_1 += 151200*c1p3*c3p1*c5p2*c7p2*c9p1*ss[42];
  jac_4_1 += 100800*c1p3*c3p1*c5p3*c7p1*c9p1*ss[40];
  jac_4_1 += 151200*c1p3*c3p2*c5p1*c7p1*c9p2*ss[42];
  jac_4_1 += 151200*c1p3*c3p2*c5p1*c7p2*c9p1*ss[40];
  jac_4_1 += 151200*c1p3*c3p2*c5p2*c7p1*c9p1*ss[38];
  jac_4_1 += 100800*c1p3*c3p3*c5p1*c7p1*c9p1*ss[36];
  jac_4_1 += 75600*c1p4*c3p1*c5p1*c7p1*c9p2*ss[40];
  jac_4_1 += 75600*c1p4*c3p1*c5p1*c7p2*c9p1*ss[38];
  jac_4_1 += 75600*c1p4*c3p1*c5p2*c7p1*c9p1*ss[36];
  jac_4_1 += 75600*c1p4*c3p2*c5p1*c7p1*c9p1*ss[34];
  jac_4_1 += 30240*c1p5*c3p1*c5p1*c7p1*c9p1*ss[32];
  jac_4_1 += 5040*c1p1*c3p1*c5p1*c7p6*ss[54];
  jac_4_1 += 5040*c1p1*c3p1*c5p1*c9p6*ss[66];
  jac_4_1 += 15120*c1p1*c3p1*c5p2*c7p5*ss[52];
  jac_4_1 += 15120*c1p1*c3p1*c5p2*c9p5*ss[62];
  jac_4_1 += 25200*c1p1*c3p1*c5p3*c7p4*ss[50];
  jac_4_1 += 25200*c1p1*c3p1*c5p3*c9p4*ss[58];
  jac_4_1 += 25200*c1p1*c3p1*c5p4*c7p3*ss[48];
  jac_4_1 += 25200*c1p1*c3p1*c5p4*c9p3*ss[54];
  jac_4_1 += 15120*c1p1*c3p1*c5p5*c7p2*ss[46];
  jac_4_1 += 15120*c1p1*c3p1*c5p5*c9p2*ss[50];
  jac_4_1 += 5040*c1p1*c3p1*c5p6*c7p1*ss[44];
  jac_4_1 += 5040*c1p1*c3p1*c5p6*c9p1*ss[46];
  jac_4_1 += 5040*c1p1*c3p1*c7p1*c9p6*ss[68];
  jac_4_1 += 15120*c1p1*c3p1*c7p2*c9p5*ss[66];
  jac_4_1 += 25200*c1p1*c3p1*c7p3*c9p4*ss[64];
  jac_4_1 += 25200*c1p1*c3p1*c7p4*c9p3*ss[62];
  jac_4_1 += 15120*c1p1*c3p1*c7p5*c9p2*ss[60];
  jac_4_1 += 5040*c1p1*c3p1*c7p6*c9p1*ss[58];
  jac_4_1 += 15120*c1p1*c3p2*c5p1*c7p5*ss[50];
  jac_4_1 += 15120*c1p1*c3p2*c5p1*c9p5*ss[60];
  jac_4_1 += 37800*c1p1*c3p2*c5p2*c7p4*ss[48];
  jac_4_1 += 37800*c1p1*c3p2*c5p2*c9p4*ss[56];
  jac_4_1 += 50400*c1p1*c3p2*c5p3*c7p3*ss[46];
  jac_4_1 += 50400*c1p1*c3p2*c5p3*c9p3*ss[52];
  jac_4_1 += 37800*c1p1*c3p2*c5p4*c7p2*ss[44];
  jac_4_1 += 37800*c1p1*c3p2*c5p4*c9p2*ss[48];
  jac_4_1 += 15120*c1p1*c3p2*c5p5*c7p1*ss[42];
  jac_4_1 += 15120*c1p1*c3p2*c5p5*c9p1*ss[44];
  jac_4_1 += 15120*c1p1*c3p2*c7p1*c9p5*ss[62];
  jac_4_1 += 37800*c1p1*c3p2*c7p2*c9p4*ss[60];
  jac_4_1 += 50400*c1p1*c3p2*c7p3*c9p3*ss[58];
  jac_4_1 += 37800*c1p1*c3p2*c7p4*c9p2*ss[56];
  jac_4_1 += 15120*c1p1*c3p2*c7p5*c9p1*ss[54];
  jac_4_1 += 25200*c1p1*c3p3*c5p1*c7p4*ss[46];
  jac_4_1 += 25200*c1p1*c3p3*c5p1*c9p4*ss[54];
  jac_4_1 += 50400*c1p1*c3p3*c5p2*c7p3*ss[44];
  jac_4_1 += 50400*c1p1*c3p3*c5p2*c9p3*ss[50];
  jac_4_1 += 50400*c1p1*c3p3*c5p3*c7p2*ss[42];
  jac_4_1 += 50400*c1p1*c3p3*c5p3*c9p2*ss[46];
  jac_4_1 += 25200*c1p1*c3p3*c5p4*c7p1*ss[40];
  jac_4_1 += 25200*c1p1*c3p3*c5p4*c9p1*ss[42];
  jac_4_1 += 25200*c1p1*c3p3*c7p1*c9p4*ss[56];
  jac_4_1 += 50400*c1p1*c3p3*c7p2*c9p3*ss[54];
  jac_4_1 += 50400*c1p1*c3p3*c7p3*c9p2*ss[52];
  jac_4_1 += 25200*c1p1*c3p3*c7p4*c9p1*ss[50];
  jac_4_1 += 25200*c1p1*c3p4*c5p1*c7p3*ss[42];
  jac_4_1 += 25200*c1p1*c3p4*c5p1*c9p3*ss[48];
  jac_4_1 += 37800*c1p1*c3p4*c5p2*c7p2*ss[40];
  jac_4_1 += 37800*c1p1*c3p4*c5p2*c9p2*ss[44];
  jac_4_1 += 25200*c1p1*c3p4*c5p3*c7p1*ss[38];
  jac_4_1 += 25200*c1p1*c3p4*c5p3*c9p1*ss[40];
  jac_4_1 += 25200*c1p1*c3p4*c7p1*c9p3*ss[50];
  jac_4_1 += 37800*c1p1*c3p4*c7p2*c9p2*ss[48];
  jac_4_1 += 25200*c1p1*c3p4*c7p3*c9p1*ss[46];
  jac_4_1 += 15120*c1p1*c3p5*c5p1*c7p2*ss[38];
  jac_4_1 += 15120*c1p1*c3p5*c5p1*c9p2*ss[42];
  jac_4_1 += 15120*c1p1*c3p5*c5p2*c7p1*ss[36];
  jac_4_1 += 15120*c1p1*c3p5*c5p2*c9p1*ss[38];
  jac_4_1 += 15120*c1p1*c3p5*c7p1*c9p2*ss[44];
  jac_4_1 += 15120*c1p1*c3p5*c7p2*c9p1*ss[42];
  jac_4_1 += 5040*c1p1*c3p6*c5p1*c7p1*ss[34];
  jac_4_1 += 5040*c1p1*c3p6*c5p1*c9p1*ss[36];
  jac_4_1 += 5040*c1p1*c3p6*c7p1*c9p1*ss[38];
  jac_4_1 += 5040*c1p1*c5p1*c7p1*c9p6*ss[70];
  jac_4_1 += 15120*c1p1*c5p1*c7p2*c9p5*ss[68];
  jac_4_1 += 25200*c1p1*c5p1*c7p3*c9p4*ss[66];
  jac_4_1 += 25200*c1p1*c5p1*c7p4*c9p3*ss[64];
  jac_4_1 += 15120*c1p1*c5p1*c7p5*c9p2*ss[62];
  jac_4_1 += 5040*c1p1*c5p1*c7p6*c9p1*ss[60];
  jac_4_1 += 15120*c1p1*c5p2*c7p1*c9p5*ss[66];
  jac_4_1 += 37800*c1p1*c5p2*c7p2*c9p4*ss[64];
  jac_4_1 += 50400*c1p1*c5p2*c7p3*c9p3*ss[62];
  jac_4_1 += 37800*c1p1*c5p2*c7p4*c9p2*ss[60];
  jac_4_1 += 15120*c1p1*c5p2*c7p5*c9p1*ss[58];
  jac_4_1 += 25200*c1p1*c5p3*c7p1*c9p4*ss[62];
  jac_4_1 += 50400*c1p1*c5p3*c7p2*c9p3*ss[60];
  jac_4_1 += 50400*c1p1*c5p3*c7p3*c9p2*ss[58];
  jac_4_1 += 25200*c1p1*c5p3*c7p4*c9p1*ss[56];
  jac_4_1 += 25200*c1p1*c5p4*c7p1*c9p3*ss[58];
  jac_4_1 += 37800*c1p1*c5p4*c7p2*c9p2*ss[56];
  jac_4_1 += 25200*c1p1*c5p4*c7p3*c9p1*ss[54];
  jac_4_1 += 15120*c1p1*c5p5*c7p1*c9p2*ss[54];
  jac_4_1 += 15120*c1p1*c5p5*c7p2*c9p1*ss[52];
  jac_4_1 += 5040*c1p1*c5p6*c7p1*c9p1*ss[50];
  jac_4_1 += 15120*c1p2*c3p1*c5p1*c7p5*ss[48];
  jac_4_1 += 15120*c1p2*c3p1*c5p1*c9p5*ss[58];
  jac_4_1 += 37800*c1p2*c3p1*c5p2*c7p4*ss[46];
  jac_4_1 += 37800*c1p2*c3p1*c5p2*c9p4*ss[54];
  jac_4_1 += 50400*c1p2*c3p1*c5p3*c7p3*ss[44];
  jac_4_1 += 50400*c1p2*c3p1*c5p3*c9p3*ss[50];
  jac_4_1 += 37800*c1p2*c3p1*c5p4*c7p2*ss[42];
  jac_4_1 += 37800*c1p2*c3p1*c5p4*c9p2*ss[46];
  jac_4_1 += 15120*c1p2*c3p1*c5p5*c7p1*ss[40];
  jac_4_1 += 15120*c1p2*c3p1*c5p5*c9p1*ss[42];
  jac_4_1 += 15120*c1p2*c3p1*c7p1*c9p5*ss[60];
  jac_4_1 += 37800*c1p2*c3p1*c7p2*c9p4*ss[58];
  jac_4_1 += 50400*c1p2*c3p1*c7p3*c9p3*ss[56];
  jac_4_1 += 37800*c1p2*c3p1*c7p4*c9p2*ss[54];
  jac_4_1 += 15120*c1p2*c3p1*c7p5*c9p1*ss[52];
  jac_4_1 += 37800*c1p2*c3p2*c5p1*c7p4*ss[44];
  jac_4_1 += 37800*c1p2*c3p2*c5p1*c9p4*ss[52];
  jac_4_1 += 75600*c1p2*c3p2*c5p2*c7p3*ss[42];
  jac_4_1 += 75600*c1p2*c3p2*c5p2*c9p3*ss[48];
  jac_4_1 += 75600*c1p2*c3p2*c5p3*c7p2*ss[40];
  jac_4_1 += 75600*c1p2*c3p2*c5p3*c9p2*ss[44];
  jac_4_1 += 37800*c1p2*c3p2*c5p4*c7p1*ss[38];
  jac_4_1 += 37800*c1p2*c3p2*c5p4*c9p1*ss[40];
  jac_4_1 += 37800*c1p2*c3p2*c7p1*c9p4*ss[54];
  jac_4_1 += 75600*c1p2*c3p2*c7p2*c9p3*ss[52];
  jac_4_1 += 75600*c1p2*c3p2*c7p3*c9p2*ss[50];
  jac_4_1 += 37800*c1p2*c3p2*c7p4*c9p1*ss[48];
  jac_4_1 += 50400*c1p2*c3p3*c5p1*c7p3*ss[40];
  jac_4_1 += 50400*c1p2*c3p3*c5p1*c9p3*ss[46];
  jac_4_1 += 75600*c1p2*c3p3*c5p2*c7p2*ss[38];
  jac_4_1 += 75600*c1p2*c3p3*c5p2*c9p2*ss[42];
  jac_4_1 += 50400*c1p2*c3p3*c5p3*c7p1*ss[36];
  jac_4_1 += 50400*c1p2*c3p3*c5p3*c9p1*ss[38];
  jac_4_1 += 50400*c1p2*c3p3*c7p1*c9p3*ss[48];
  jac_4_1 += 75600*c1p2*c3p3*c7p2*c9p2*ss[46];
  jac_4_1 += 50400*c1p2*c3p3*c7p3*c9p1*ss[44];
  jac_4_1 += 37800*c1p2*c3p4*c5p1*c7p2*ss[36];
  jac_4_1 += 37800*c1p2*c3p4*c5p1*c9p2*ss[40];
  jac_4_1 += 37800*c1p2*c3p4*c5p2*c7p1*ss[34];
  jac_4_1 += 37800*c1p2*c3p4*c5p2*c9p1*ss[36];
  jac_4_1 += 37800*c1p2*c3p4*c7p1*c9p2*ss[42];
  jac_4_1 += 37800*c1p2*c3p4*c7p2*c9p1*ss[40];
  jac_4_1 += 15120*c1p2*c3p5*c5p1*c7p1*ss[32];
  jac_4_1 += 15120*c1p2*c3p5*c5p1*c9p1*ss[34];
  jac_4_1 += 15120*c1p2*c3p5*c7p1*c9p1*ss[36];
  jac_4_1 += 15120*c1p2*c5p1*c7p1*c9p5*ss[62];
  jac_4_1 += 37800*c1p2*c5p1*c7p2*c9p4*ss[60];
  jac_4_1 += 50400*c1p2*c5p1*c7p3*c9p3*ss[58];
  jac_4_1 += 37800*c1p2*c5p1*c7p4*c9p2*ss[56];
  jac_4_1 += 15120*c1p2*c5p1*c7p5*c9p1*ss[54];
  jac_4_1 += 37800*c1p2*c5p2*c7p1*c9p4*ss[58];
  jac_4_1 += 75600*c1p2*c5p2*c7p2*c9p3*ss[56];
  jac_4_1 += 75600*c1p2*c5p2*c7p3*c9p2*ss[54];
  jac_4_1 += 37800*c1p2*c5p2*c7p4*c9p1*ss[52];
  jac_4_1 += 50400*c1p2*c5p3*c7p1*c9p3*ss[54];
  jac_4_1 += 75600*c1p2*c5p3*c7p2*c9p2*ss[52];
  jac_4_1 += 50400*c1p2*c5p3*c7p3*c9p1*ss[50];
  jac_4_1 += 37800*c1p2*c5p4*c7p1*c9p2*ss[50];
  jac_4_1 += 37800*c1p2*c5p4*c7p2*c9p1*ss[48];
  jac_4_1 += 15120*c1p2*c5p5*c7p1*c9p1*ss[46];
  jac_4_1 += 25200*c1p3*c3p1*c5p1*c7p4*ss[42];
  jac_4_1 += 25200*c1p3*c3p1*c5p1*c9p4*ss[50];
  jac_4_1 += 50400*c1p3*c3p1*c5p2*c7p3*ss[40];
  jac_4_1 += 50400*c1p3*c3p1*c5p2*c9p3*ss[46];
  jac_4_1 += 50400*c1p3*c3p1*c5p3*c7p2*ss[38];
  jac_4_1 += 50400*c1p3*c3p1*c5p3*c9p2*ss[42];
  jac_4_1 += 25200*c1p3*c3p1*c5p4*c7p1*ss[36];
  jac_4_1 += 25200*c1p3*c3p1*c5p4*c9p1*ss[38];
  jac_4_1 += 25200*c1p3*c3p1*c7p1*c9p4*ss[52];
  jac_4_1 += 50400*c1p3*c3p1*c7p2*c9p3*ss[50];
  jac_4_1 += 50400*c1p3*c3p1*c7p3*c9p2*ss[48];
  jac_4_1 += 25200*c1p3*c3p1*c7p4*c9p1*ss[46];
  jac_4_1 += 50400*c1p3*c3p2*c5p1*c7p3*ss[38];
  jac_4_1 += 50400*c1p3*c3p2*c5p1*c9p3*ss[44];
  jac_4_1 += 75600*c1p3*c3p2*c5p2*c7p2*ss[36];
  jac_4_1 += 75600*c1p3*c3p2*c5p2*c9p2*ss[40];
  jac_4_1 += 50400*c1p3*c3p2*c5p3*c7p1*ss[34];
  jac_4_1 += 50400*c1p3*c3p2*c5p3*c9p1*ss[36];
  jac_4_1 += 50400*c1p3*c3p2*c7p1*c9p3*ss[46];
  jac_4_1 += 75600*c1p3*c3p2*c7p2*c9p2*ss[44];
  jac_4_1 += 50400*c1p3*c3p2*c7p3*c9p1*ss[42];
  jac_4_1 += 50400*c1p3*c3p3*c5p1*c7p2*ss[34];
  jac_4_1 += 50400*c1p3*c3p3*c5p1*c9p2*ss[38];
  jac_4_1 += 50400*c1p3*c3p3*c5p2*c7p1*ss[32];
  jac_4_1 += 50400*c1p3*c3p3*c5p2*c9p1*ss[34];
  jac_4_1 += 50400*c1p3*c3p3*c7p1*c9p2*ss[40];
  jac_4_1 += 50400*c1p3*c3p3*c7p2*c9p1*ss[38];
  jac_4_1 += 25200*c1p3*c3p4*c5p1*c7p1*ss[30];
  jac_4_1 += 25200*c1p3*c3p4*c5p1*c9p1*ss[32];
  jac_4_1 += 25200*c1p3*c3p4*c7p1*c9p1*ss[34];
  jac_4_1 += 25200*c1p3*c5p1*c7p1*c9p4*ss[54];
  jac_4_1 += 50400*c1p3*c5p1*c7p2*c9p3*ss[52];
  jac_4_1 += 50400*c1p3*c5p1*c7p3*c9p2*ss[50];
  jac_4_1 += 25200*c1p3*c5p1*c7p4*c9p1*ss[48];
  jac_4_1 += 50400*c1p3*c5p2*c7p1*c9p3*ss[50];
  jac_4_1 += 75600*c1p3*c5p2*c7p2*c9p2*ss[48];
  jac_4_1 += 50400*c1p3*c5p2*c7p3*c9p1*ss[46];
  jac_4_1 += 50400*c1p3*c5p3*c7p1*c9p2*ss[46];
  jac_4_1 += 50400*c1p3*c5p3*c7p2*c9p1*ss[44];
  jac_4_1 += 25200*c1p3*c5p4*c7p1*c9p1*ss[42];
  jac_4_1 += 25200*c1p4*c3p1*c5p1*c7p3*ss[36];
  jac_4_1 += 25200*c1p4*c3p1*c5p1*c9p3*ss[42];
  jac_4_1 += 37800*c1p4*c3p1*c5p2*c7p2*ss[34];
  jac_4_1 += 37800*c1p4*c3p1*c5p2*c9p2*ss[38];
  jac_4_1 += 25200*c1p4*c3p1*c5p3*c7p1*ss[32];
  jac_4_1 += 25200*c1p4*c3p1*c5p3*c9p1*ss[34];
  jac_4_1 += 25200*c1p4*c3p1*c7p1*c9p3*ss[44];
  jac_4_1 += 37800*c1p4*c3p1*c7p2*c9p2*ss[42];
  jac_4_1 += 25200*c1p4*c3p1*c7p3*c9p1*ss[40];
  jac_4_1 += 37800*c1p4*c3p2*c5p1*c7p2*ss[32];
  jac_4_1 += 37800*c1p4*c3p2*c5p1*c9p2*ss[36];
  jac_4_1 += 37800*c1p4*c3p2*c5p2*c7p1*ss[30];
  jac_4_1 += 37800*c1p4*c3p2*c5p2*c9p1*ss[32];
  jac_4_1 += 37800*c1p4*c3p2*c7p1*c9p2*ss[38];
  jac_4_1 += 37800*c1p4*c3p2*c7p2*c9p1*ss[36];
  jac_4_1 += 25200*c1p4*c3p3*c5p1*c7p1*ss[28];
  jac_4_1 += 25200*c1p4*c3p3*c5p1*c9p1*ss[30];
  jac_4_1 += 25200*c1p4*c3p3*c7p1*c9p1*ss[32];
  jac_4_1 += 25200*c1p4*c5p1*c7p1*c9p3*ss[46];
  jac_4_1 += 37800*c1p4*c5p1*c7p2*c9p2*ss[44];
  jac_4_1 += 25200*c1p4*c5p1*c7p3*c9p1*ss[42];
  jac_4_1 += 37800*c1p4*c5p2*c7p1*c9p2*ss[42];
  jac_4_1 += 37800*c1p4*c5p2*c7p2*c9p1*ss[40];
  jac_4_1 += 25200*c1p4*c5p3*c7p1*c9p1*ss[38];
  jac_4_1 += 15120*c1p5*c3p1*c5p1*c7p2*ss[30];
  jac_4_1 += 15120*c1p5*c3p1*c5p1*c9p2*ss[34];
  jac_4_1 += 15120*c1p5*c3p1*c5p2*c7p1*ss[28];
  jac_4_1 += 15120*c1p5*c3p1*c5p2*c9p1*ss[30];
  jac_4_1 += 15120*c1p5*c3p1*c7p1*c9p2*ss[36];
  jac_4_1 += 15120*c1p5*c3p1*c7p2*c9p1*ss[34];
  jac_4_1 += 15120*c1p5*c3p2*c5p1*c7p1*ss[26];
  jac_4_1 += 15120*c1p5*c3p2*c5p1*c9p1*ss[28];
  jac_4_1 += 15120*c1p5*c3p2*c7p1*c9p1*ss[30];
  jac_4_1 += 15120*c1p5*c5p1*c7p1*c9p2*ss[38];
  jac_4_1 += 15120*c1p5*c5p1*c7p2*c9p1*ss[36];
  jac_4_1 += 15120*c1p5*c5p2*c7p1*c9p1*ss[34];
  jac_4_1 += 5040*c1p6*c3p1*c5p1*c7p1*ss[24];
  jac_4_1 += 5040*c1p6*c3p1*c5p1*c9p1*ss[26];
  jac_4_1 += 5040*c1p6*c3p1*c7p1*c9p1*ss[28];
  jac_4_1 += 5040*c1p6*c5p1*c7p1*c9p1*ss[30];
  jac_4_1 += 5040*c3p1*c5p1*c7p1*c9p6*ss[72];
  jac_4_1 += 15120*c3p1*c5p1*c7p2*c9p5*ss[70];
  jac_4_1 += 25200*c3p1*c5p1*c7p3*c9p4*ss[68];
  jac_4_1 += 25200*c3p1*c5p1*c7p4*c9p3*ss[66];
  jac_4_1 += 15120*c3p1*c5p1*c7p5*c9p2*ss[64];
  jac_4_1 += 5040*c3p1*c5p1*c7p6*c9p1*ss[62];
  jac_4_1 += 15120*c3p1*c5p2*c7p1*c9p5*ss[68];
  jac_4_1 += 37800*c3p1*c5p2*c7p2*c9p4*ss[66];
  jac_4_1 += 50400*c3p1*c5p2*c7p3*c9p3*ss[64];
  jac_4_1 += 37800*c3p1*c5p2*c7p4*c9p2*ss[62];
  jac_4_1 += 15120*c3p1*c5p2*c7p5*c9p1*ss[60];
  jac_4_1 += 25200*c3p1*c5p3*c7p1*c9p4*ss[64];
  jac_4_1 += 50400*c3p1*c5p3*c7p2*c9p3*ss[62];
  jac_4_1 += 50400*c3p1*c5p3*c7p3*c9p2*ss[60];
  jac_4_1 += 25200*c3p1*c5p3*c7p4*c9p1*ss[58];
  jac_4_1 += 25200*c3p1*c5p4*c7p1*c9p3*ss[60];
  jac_4_1 += 37800*c3p1*c5p4*c7p2*c9p2*ss[58];
  jac_4_1 += 25200*c3p1*c5p4*c7p3*c9p1*ss[56];
  jac_4_1 += 15120*c3p1*c5p5*c7p1*c9p2*ss[56];
  jac_4_1 += 15120*c3p1*c5p5*c7p2*c9p1*ss[54];
  jac_4_1 += 5040*c3p1*c5p6*c7p1*c9p1*ss[52];
  jac_4_1 += 15120*c3p2*c5p1*c7p1*c9p5*ss[66];
  jac_4_1 += 37800*c3p2*c5p1*c7p2*c9p4*ss[64];
  jac_4_1 += 50400*c3p2*c5p1*c7p3*c9p3*ss[62];
  jac_4_1 += 37800*c3p2*c5p1*c7p4*c9p2*ss[60];
  jac_4_1 += 15120*c3p2*c5p1*c7p5*c9p1*ss[58];
  jac_4_1 += 37800*c3p2*c5p2*c7p1*c9p4*ss[62];
  jac_4_1 += 75600*c3p2*c5p2*c7p2*c9p3*ss[60];
  jac_4_1 += 75600*c3p2*c5p2*c7p3*c9p2*ss[58];
  jac_4_1 += 37800*c3p2*c5p2*c7p4*c9p1*ss[56];
  jac_4_1 += 50400*c3p2*c5p3*c7p1*c9p3*ss[58];
  jac_4_1 += 75600*c3p2*c5p3*c7p2*c9p2*ss[56];
  jac_4_1 += 50400*c3p2*c5p3*c7p3*c9p1*ss[54];
  jac_4_1 += 37800*c3p2*c5p4*c7p1*c9p2*ss[54];
  jac_4_1 += 37800*c3p2*c5p4*c7p2*c9p1*ss[52];
  jac_4_1 += 15120*c3p2*c5p5*c7p1*c9p1*ss[50];
  jac_4_1 += 25200*c3p3*c5p1*c7p1*c9p4*ss[60];
  jac_4_1 += 50400*c3p3*c5p1*c7p2*c9p3*ss[58];
  jac_4_1 += 50400*c3p3*c5p1*c7p3*c9p2*ss[56];
  jac_4_1 += 25200*c3p3*c5p1*c7p4*c9p1*ss[54];
  jac_4_1 += 50400*c3p3*c5p2*c7p1*c9p3*ss[56];
  jac_4_1 += 75600*c3p3*c5p2*c7p2*c9p2*ss[54];
  jac_4_1 += 50400*c3p3*c5p2*c7p3*c9p1*ss[52];
  jac_4_1 += 50400*c3p3*c5p3*c7p1*c9p2*ss[52];
  jac_4_1 += 50400*c3p3*c5p3*c7p2*c9p1*ss[50];
  jac_4_1 += 25200*c3p3*c5p4*c7p1*c9p1*ss[48];
  jac_4_1 += 25200*c3p4*c5p1*c7p1*c9p3*ss[54];
  jac_4_1 += 37800*c3p4*c5p1*c7p2*c9p2*ss[52];
  jac_4_1 += 25200*c3p4*c5p1*c7p3*c9p1*ss[50];
  jac_4_1 += 37800*c3p4*c5p2*c7p1*c9p2*ss[50];
  jac_4_1 += 37800*c3p4*c5p2*c7p2*c9p1*ss[48];
  jac_4_1 += 25200*c3p4*c5p3*c7p1*c9p1*ss[46];
  jac_4_1 += 15120*c3p5*c5p1*c7p1*c9p2*ss[48];
  jac_4_1 += 15120*c3p5*c5p1*c7p2*c9p1*ss[46];
  jac_4_1 += 15120*c3p5*c5p2*c7p1*c9p1*ss[44];
  jac_4_1 += 5040*c3p6*c5p1*c7p1*c9p1*ss[42];
  jac_4_1 += 720*c1p1*c3p1*c5p7*ss[42];
  jac_4_1 += 720*c1p1*c3p1*c7p7*ss[56];
  jac_4_1 += 720*c1p1*c3p1*c9p7*ss[70];
  jac_4_1 += 2520*c1p1*c3p2*c5p6*ss[40];
  jac_4_1 += 2520*c1p1*c3p2*c7p6*ss[52];
  jac_4_1 += 2520*c1p1*c3p2*c9p6*ss[64];
  jac_4_1 += 5040*c1p1*c3p3*c5p5*ss[38];
  jac_4_1 += 5040*c1p1*c3p3*c7p5*ss[48];
  jac_4_1 += 5040*c1p1*c3p3*c9p5*ss[58];
  jac_4_1 += 6300*c1p1*c3p4*c5p4*ss[36];
  jac_4_1 += 6300*c1p1*c3p4*c7p4*ss[44];
  jac_4_1 += 6300*c1p1*c3p4*c9p4*ss[52];
  jac_4_1 += 5040*c1p1*c3p5*c5p3*ss[34];
  jac_4_1 += 5040*c1p1*c3p5*c7p3*ss[40];
  jac_4_1 += 5040*c1p1*c3p5*c9p3*ss[46];
  jac_4_1 += 2520*c1p1*c3p6*c5p2*ss[32];
  jac_4_1 += 2520*c1p1*c3p6*c7p2*ss[36];
  jac_4_1 += 2520*c1p1*c3p6*c9p2*ss[40];
  jac_4_1 += 720*c1p1*c3p7*c5p1*ss[30];
  jac_4_1 += 720*c1p1*c3p7*c7p1*ss[32];
  jac_4_1 += 720*c1p1*c3p7*c9p1*ss[34];
  jac_4_1 += 720*c1p1*c5p1*c7p7*ss[58];
  jac_4_1 += 720*c1p1*c5p1*c9p7*ss[72];
  jac_4_1 += 2520*c1p1*c5p2*c7p6*ss[56];
  jac_4_1 += 2520*c1p1*c5p2*c9p6*ss[68];
  jac_4_1 += 5040*c1p1*c5p3*c7p5*ss[54];
  jac_4_1 += 5040*c1p1*c5p3*c9p5*ss[64];
  jac_4_1 += 6300*c1p1*c5p4*c7p4*ss[52];
  jac_4_1 += 6300*c1p1*c5p4*c9p4*ss[60];
  jac_4_1 += 5040*c1p1*c5p5*c7p3*ss[50];
  jac_4_1 += 5040*c1p1*c5p5*c9p3*ss[56];
  jac_4_1 += 2520*c1p1*c5p6*c7p2*ss[48];
  jac_4_1 += 2520*c1p1*c5p6*c9p2*ss[52];
  jac_4_1 += 720*c1p1*c5p7*c7p1*ss[46];
  jac_4_1 += 720*c1p1*c5p7*c9p1*ss[48];
  jac_4_1 += 720*c1p1*c7p1*c9p7*ss[74];
  jac_4_1 += 2520*c1p1*c7p2*c9p6*ss[72];
  jac_4_1 += 5040*c1p1*c7p3*c9p5*ss[70];
  jac_4_1 += 6300*c1p1*c7p4*c9p4*ss[68];
  jac_4_1 += 5040*c1p1*c7p5*c9p3*ss[66];
  jac_4_1 += 2520*c1p1*c7p6*c9p2*ss[64];
  jac_4_1 += 720*c1p1*c7p7*c9p1*ss[62];
  jac_4_1 += 2520*c1p2*c3p1*c5p6*ss[38];
  jac_4_1 += 2520*c1p2*c3p1*c7p6*ss[50];
  jac_4_1 += 2520*c1p2*c3p1*c9p6*ss[62];
  jac_4_1 += 7560*c1p2*c3p2*c5p5*ss[36];
  jac_4_1 += 7560*c1p2*c3p2*c7p5*ss[46];
  jac_4_1 += 7560*c1p2*c3p2*c9p5*ss[56];
  jac_4_1 += 12600*c1p2*c3p3*c5p4*ss[34];
  jac_4_1 += 12600*c1p2*c3p3*c7p4*ss[42];
  jac_4_1 += 12600*c1p2*c3p3*c9p4*ss[50];
  jac_4_1 += 12600*c1p2*c3p4*c5p3*ss[32];
  jac_4_1 += 12600*c1p2*c3p4*c7p3*ss[38];
  jac_4_1 += 12600*c1p2*c3p4*c9p3*ss[44];
  jac_4_1 += 7560*c1p2*c3p5*c5p2*ss[30];
  jac_4_1 += 7560*c1p2*c3p5*c7p2*ss[34];
  jac_4_1 += 7560*c1p2*c3p5*c9p2*ss[38];
  jac_4_1 += 2520*c1p2*c3p6*c5p1*ss[28];
  jac_4_1 += 2520*c1p2*c3p6*c7p1*ss[30];
  jac_4_1 += 2520*c1p2*c3p6*c9p1*ss[32];
  jac_4_1 += 2520*c1p2*c5p1*c7p6*ss[52];
  jac_4_1 += 2520*c1p2*c5p1*c9p6*ss[64];
  jac_4_1 += 7560*c1p2*c5p2*c7p5*ss[50];
  jac_4_1 += 7560*c1p2*c5p2*c9p5*ss[60];
  jac_4_1 += 12600*c1p2*c5p3*c7p4*ss[48];
  jac_4_1 += 12600*c1p2*c5p3*c9p4*ss[56];
  jac_4_1 += 12600*c1p2*c5p4*c7p3*ss[46];
  jac_4_1 += 12600*c1p2*c5p4*c9p3*ss[52];
  jac_4_1 += 7560*c1p2*c5p5*c7p2*ss[44];
  jac_4_1 += 7560*c1p2*c5p5*c9p2*ss[48];
  jac_4_1 += 2520*c1p2*c5p6*c7p1*ss[42];
  jac_4_1 += 2520*c1p2*c5p6*c9p1*ss[44];
  jac_4_1 += 2520*c1p2*c7p1*c9p6*ss[66];
  jac_4_1 += 7560*c1p2*c7p2*c9p5*ss[64];
  jac_4_1 += 12600*c1p2*c7p3*c9p4*ss[62];
  jac_4_1 += 12600*c1p2*c7p4*c9p3*ss[60];
  jac_4_1 += 7560*c1p2*c7p5*c9p2*ss[58];
  jac_4_1 += 2520*c1p2*c7p6*c9p1*ss[56];
  jac_4_1 += 5040*c1p3*c3p1*c5p5*ss[34];
  jac_4_1 += 5040*c1p3*c3p1*c7p5*ss[44];
  jac_4_1 += 5040*c1p3*c3p1*c9p5*ss[54];
  jac_4_1 += 12600*c1p3*c3p2*c5p4*ss[32];
  jac_4_1 += 12600*c1p3*c3p2*c7p4*ss[40];
  jac_4_1 += 12600*c1p3*c3p2*c9p4*ss[48];
  jac_4_1 += 16800*c1p3*c3p3*c5p3*ss[30];
  jac_4_1 += 16800*c1p3*c3p3*c7p3*ss[36];
  jac_4_1 += 16800*c1p3*c3p3*c9p3*ss[42];
  jac_4_1 += 12600*c1p3*c3p4*c5p2*ss[28];
  jac_4_1 += 12600*c1p3*c3p4*c7p2*ss[32];
  jac_4_1 += 12600*c1p3*c3p4*c9p2*ss[36];
  jac_4_1 += 5040*c1p3*c3p5*c5p1*ss[26];
  jac_4_1 += 5040*c1p3*c3p5*c7p1*ss[28];
  jac_4_1 += 5040*c1p3*c3p5*c9p1*ss[30];
  jac_4_1 += 5040*c1p3*c5p1*c7p5*ss[46];
  jac_4_1 += 5040*c1p3*c5p1*c9p5*ss[56];
  jac_4_1 += 12600*c1p3*c5p2*c7p4*ss[44];
  jac_4_1 += 12600*c1p3*c5p2*c9p4*ss[52];
  jac_4_1 += 16800*c1p3*c5p3*c7p3*ss[42];
  jac_4_1 += 16800*c1p3*c5p3*c9p3*ss[48];
  jac_4_1 += 12600*c1p3*c5p4*c7p2*ss[40];
  jac_4_1 += 12600*c1p3*c5p4*c9p2*ss[44];
  jac_4_1 += 5040*c1p3*c5p5*c7p1*ss[38];
  jac_4_1 += 5040*c1p3*c5p5*c9p1*ss[40];
  jac_4_1 += 5040*c1p3*c7p1*c9p5*ss[58];
  jac_4_1 += 12600*c1p3*c7p2*c9p4*ss[56];
  jac_4_1 += 16800*c1p3*c7p3*c9p3*ss[54];
  jac_4_1 += 12600*c1p3*c7p4*c9p2*ss[52];
  jac_4_1 += 5040*c1p3*c7p5*c9p1*ss[50];
  jac_4_1 += 6300*c1p4*c3p1*c5p4*ss[30];
  jac_4_1 += 6300*c1p4*c3p1*c7p4*ss[38];
  jac_4_1 += 6300*c1p4*c3p1*c9p4*ss[46];
  jac_4_1 += 12600*c1p4*c3p2*c5p3*ss[28];
  jac_4_1 += 12600*c1p4*c3p2*c7p3*ss[34];
  jac_4_1 += 12600*c1p4*c3p2*c9p3*ss[40];
  jac_4_1 += 12600*c1p4*c3p3*c5p2*ss[26];
  jac_4_1 += 12600*c1p4*c3p3*c7p2*ss[30];
  jac_4_1 += 12600*c1p4*c3p3*c9p2*ss[34];
  jac_4_1 += 6300*c1p4*c3p4*c5p1*ss[24];
  jac_4_1 += 6300*c1p4*c3p4*c7p1*ss[26];
  jac_4_1 += 6300*c1p4*c3p4*c9p1*ss[28];
  jac_4_1 += 6300*c1p4*c5p1*c7p4*ss[40];
  jac_4_1 += 6300*c1p4*c5p1*c9p4*ss[48];
  jac_4_1 += 12600*c1p4*c5p2*c7p3*ss[38];
  jac_4_1 += 12600*c1p4*c5p2*c9p3*ss[44];
  jac_4_1 += 12600*c1p4*c5p3*c7p2*ss[36];
  jac_4_1 += 12600*c1p4*c5p3*c9p2*ss[40];
  jac_4_1 += 6300*c1p4*c5p4*c7p1*ss[34];
  jac_4_1 += 6300*c1p4*c5p4*c9p1*ss[36];
  jac_4_1 += 6300*c1p4*c7p1*c9p4*ss[50];
  jac_4_1 += 12600*c1p4*c7p2*c9p3*ss[48];
  jac_4_1 += 12600*c1p4*c7p3*c9p2*ss[46];
  jac_4_1 += 6300*c1p4*c7p4*c9p1*ss[44];
  jac_4_1 += 5040*c1p5*c3p1*c5p3*ss[26];
  jac_4_1 += 5040*c1p5*c3p1*c7p3*ss[32];
  jac_4_1 += 5040*c1p5*c3p1*c9p3*ss[38];
  jac_4_1 += 7560*c1p5*c3p2*c5p2*ss[24];
  jac_4_1 += 7560*c1p5*c3p2*c7p2*ss[28];
  jac_4_1 += 7560*c1p5*c3p2*c9p2*ss[32];
  jac_4_1 += 5040*c1p5*c3p3*c5p1*ss[22];
  jac_4_1 += 5040*c1p5*c3p3*c7p1*ss[24];
  jac_4_1 += 5040*c1p5*c3p3*c9p1*ss[26];
  jac_4_1 += 5040*c1p5*c5p1*c7p3*ss[34];
  jac_4_1 += 5040*c1p5*c5p1*c9p3*ss[40];
  jac_4_1 += 7560*c1p5*c5p2*c7p2*ss[32];
  jac_4_1 += 7560*c1p5*c5p2*c9p2*ss[36];
  jac_4_1 += 5040*c1p5*c5p3*c7p1*ss[30];
  jac_4_1 += 5040*c1p5*c5p3*c9p1*ss[32];
  jac_4_1 += 5040*c1p5*c7p1*c9p3*ss[42];
  jac_4_1 += 7560*c1p5*c7p2*c9p2*ss[40];
  jac_4_1 += 5040*c1p5*c7p3*c9p1*ss[38];
  jac_4_1 += 2520*c1p6*c3p1*c5p2*ss[22];
  jac_4_1 += 2520*c1p6*c3p1*c7p2*ss[26];
  jac_4_1 += 2520*c1p6*c3p1*c9p2*ss[30];
  jac_4_1 += 2520*c1p6*c3p2*c5p1*ss[20];
  jac_4_1 += 2520*c1p6*c3p2*c7p1*ss[22];
  jac_4_1 += 2520*c1p6*c3p2*c9p1*ss[24];
  jac_4_1 += 2520*c1p6*c5p1*c7p2*ss[28];
  jac_4_1 += 2520*c1p6*c5p1*c9p2*ss[32];
  jac_4_1 += 2520*c1p6*c5p2*c7p1*ss[26];
  jac_4_1 += 2520*c1p6*c5p2*c9p1*ss[28];
  jac_4_1 += 2520*c1p6*c7p1*c9p2*ss[34];
  jac_4_1 += 2520*c1p6*c7p2*c9p1*ss[32];
  jac_4_1 += 720*c1p7*c3p1*c5p1*ss[18];
  jac_4_1 += 720*c1p7*c3p1*c7p1*ss[20];
  jac_4_1 += 720*c1p7*c3p1*c9p1*ss[22];
  jac_4_1 += 720*c1p7*c5p1*c7p1*ss[22];
  jac_4_1 += 720*c1p7*c5p1*c9p1*ss[24];
  jac_4_1 += 720*c1p7*c7p1*c9p1*ss[26];
  jac_4_1 += 720*c3p1*c5p1*c7p7*ss[60];
  jac_4_1 += 720*c3p1*c5p1*c9p7*ss[74];
  jac_4_1 += 2520*c3p1*c5p2*c7p6*ss[58];
  jac_4_1 += 2520*c3p1*c5p2*c9p6*ss[70];
  jac_4_1 += 5040*c3p1*c5p3*c7p5*ss[56];
  jac_4_1 += 5040*c3p1*c5p3*c9p5*ss[66];
  jac_4_1 += 6300*c3p1*c5p4*c7p4*ss[54];
  jac_4_1 += 6300*c3p1*c5p4*c9p4*ss[62];
  jac_4_1 += 5040*c3p1*c5p5*c7p3*ss[52];
  jac_4_1 += 5040*c3p1*c5p5*c9p3*ss[58];
  jac_4_1 += 2520*c3p1*c5p6*c7p2*ss[50];
  jac_4_1 += 2520*c3p1*c5p6*c9p2*ss[54];
  jac_4_1 += 720*c3p1*c5p7*c7p1*ss[48];
  jac_4_1 += 720*c3p1*c5p7*c9p1*ss[50];
  jac_4_1 += 720*c3p1*c7p1*c9p7*ss[76];
  jac_4_1 += 2520*c3p1*c7p2*c9p6*ss[74];
  jac_4_1 += 5040*c3p1*c7p3*c9p5*ss[72];
  jac_4_1 += 6300*c3p1*c7p4*c9p4*ss[70];
  jac_4_1 += 5040*c3p1*c7p5*c9p3*ss[68];
  jac_4_1 += 2520*c3p1*c7p6*c9p2*ss[66];
  jac_4_1 += 720*c3p1*c7p7*c9p1*ss[64];
  jac_4_1 += 2520*c3p2*c5p1*c7p6*ss[56];
  jac_4_1 += 2520*c3p2*c5p1*c9p6*ss[68];
  jac_4_1 += 7560*c3p2*c5p2*c7p5*ss[54];
  jac_4_1 += 7560*c3p2*c5p2*c9p5*ss[64];
  jac_4_1 += 12600*c3p2*c5p3*c7p4*ss[52];
  jac_4_1 += 12600*c3p2*c5p3*c9p4*ss[60];
  jac_4_1 += 12600*c3p2*c5p4*c7p3*ss[50];
  jac_4_1 += 12600*c3p2*c5p4*c9p3*ss[56];
  jac_4_1 += 7560*c3p2*c5p5*c7p2*ss[48];
  jac_4_1 += 7560*c3p2*c5p5*c9p2*ss[52];
  jac_4_1 += 2520*c3p2*c5p6*c7p1*ss[46];
  jac_4_1 += 2520*c3p2*c5p6*c9p1*ss[48];
  jac_4_1 += 2520*c3p2*c7p1*c9p6*ss[70];
  jac_4_1 += 7560*c3p2*c7p2*c9p5*ss[68];
  jac_4_1 += 12600*c3p2*c7p3*c9p4*ss[66];
  jac_4_1 += 12600*c3p2*c7p4*c9p3*ss[64];
  jac_4_1 += 7560*c3p2*c7p5*c9p2*ss[62];
  jac_4_1 += 2520*c3p2*c7p6*c9p1*ss[60];
  jac_4_1 += 5040*c3p3*c5p1*c7p5*ss[52];
  jac_4_1 += 5040*c3p3*c5p1*c9p5*ss[62];
  jac_4_1 += 12600*c3p3*c5p2*c7p4*ss[50];
  jac_4_1 += 12600*c3p3*c5p2*c9p4*ss[58];
  jac_4_1 += 16800*c3p3*c5p3*c7p3*ss[48];
  jac_4_1 += 16800*c3p3*c5p3*c9p3*ss[54];
  jac_4_1 += 12600*c3p3*c5p4*c7p2*ss[46];
  jac_4_1 += 12600*c3p3*c5p4*c9p2*ss[50];
  jac_4_1 += 5040*c3p3*c5p5*c7p1*ss[44];
  jac_4_1 += 5040*c3p3*c5p5*c9p1*ss[46];
  jac_4_1 += 5040*c3p3*c7p1*c9p5*ss[64];
  jac_4_1 += 12600*c3p3*c7p2*c9p4*ss[62];
  jac_4_1 += 16800*c3p3*c7p3*c9p3*ss[60];
  jac_4_1 += 12600*c3p3*c7p4*c9p2*ss[58];
  jac_4_1 += 5040*c3p3*c7p5*c9p1*ss[56];
  jac_4_1 += 6300*c3p4*c5p1*c7p4*ss[48];
  jac_4_1 += 6300*c3p4*c5p1*c9p4*ss[56];
  jac_4_1 += 12600*c3p4*c5p2*c7p3*ss[46];
  jac_4_1 += 12600*c3p4*c5p2*c9p3*ss[52];
  jac_4_1 += 12600*c3p4*c5p3*c7p2*ss[44];
  jac_4_1 += 12600*c3p4*c5p3*c9p2*ss[48];
  jac_4_1 += 6300*c3p4*c5p4*c7p1*ss[42];
  jac_4_1 += 6300*c3p4*c5p4*c9p1*ss[44];
  jac_4_1 += 6300*c3p4*c7p1*c9p4*ss[58];
  jac_4_1 += 12600*c3p4*c7p2*c9p3*ss[56];
  jac_4_1 += 12600*c3p4*c7p3*c9p2*ss[54];
  jac_4_1 += 6300*c3p4*c7p4*c9p1*ss[52];
  jac_4_1 += 5040*c3p5*c5p1*c7p3*ss[44];
  jac_4_1 += 5040*c3p5*c5p1*c9p3*ss[50];
  jac_4_1 += 7560*c3p5*c5p2*c7p2*ss[42];
  jac_4_1 += 7560*c3p5*c5p2*c9p2*ss[46];
  jac_4_1 += 5040*c3p5*c5p3*c7p1*ss[40];
  jac_4_1 += 5040*c3p5*c5p3*c9p1*ss[42];
  jac_4_1 += 5040*c3p5*c7p1*c9p3*ss[52];
  jac_4_1 += 7560*c3p5*c7p2*c9p2*ss[50];
  jac_4_1 += 5040*c3p5*c7p3*c9p1*ss[48];
  jac_4_1 += 2520*c3p6*c5p1*c7p2*ss[40];
  jac_4_1 += 2520*c3p6*c5p1*c9p2*ss[44];
  jac_4_1 += 2520*c3p6*c5p2*c7p1*ss[38];
  jac_4_1 += 2520*c3p6*c5p2*c9p1*ss[40];
  jac_4_1 += 2520*c3p6*c7p1*c9p2*ss[46];
  jac_4_1 += 2520*c3p6*c7p2*c9p1*ss[44];
  jac_4_1 += 720*c3p7*c5p1*c7p1*ss[36];
  jac_4_1 += 720*c3p7*c5p1*c9p1*ss[38];
  jac_4_1 += 720*c3p7*c7p1*c9p1*ss[40];
  jac_4_1 += 720*c5p1*c7p1*c9p7*ss[78];
  jac_4_1 += 2520*c5p1*c7p2*c9p6*ss[76];
  jac_4_1 += 5040*c5p1*c7p3*c9p5*ss[74];
  jac_4_1 += 6300*c5p1*c7p4*c9p4*ss[72];
  jac_4_1 += 5040*c5p1*c7p5*c9p3*ss[70];
  jac_4_1 += 2520*c5p1*c7p6*c9p2*ss[68];
  jac_4_1 += 720*c5p1*c7p7*c9p1*ss[66];
  jac_4_1 += 2520*c5p2*c7p1*c9p6*ss[74];
  jac_4_1 += 7560*c5p2*c7p2*c9p5*ss[72];
  jac_4_1 += 12600*c5p2*c7p3*c9p4*ss[70];
  jac_4_1 += 12600*c5p2*c7p4*c9p3*ss[68];
  jac_4_1 += 7560*c5p2*c7p5*c9p2*ss[66];
  jac_4_1 += 2520*c5p2*c7p6*c9p1*ss[64];
  jac_4_1 += 5040*c5p3*c7p1*c9p5*ss[70];
  jac_4_1 += 12600*c5p3*c7p2*c9p4*ss[68];
  jac_4_1 += 16800*c5p3*c7p3*c9p3*ss[66];
  jac_4_1 += 12600*c5p3*c7p4*c9p2*ss[64];
  jac_4_1 += 5040*c5p3*c7p5*c9p1*ss[62];
  jac_4_1 += 6300*c5p4*c7p1*c9p4*ss[66];
  jac_4_1 += 12600*c5p4*c7p2*c9p3*ss[64];
  jac_4_1 += 12600*c5p4*c7p3*c9p2*ss[62];
  jac_4_1 += 6300*c5p4*c7p4*c9p1*ss[60];
  jac_4_1 += 5040*c5p5*c7p1*c9p3*ss[62];
  jac_4_1 += 7560*c5p5*c7p2*c9p2*ss[60];
  jac_4_1 += 5040*c5p5*c7p3*c9p1*ss[58];
  jac_4_1 += 2520*c5p6*c7p1*c9p2*ss[58];
  jac_4_1 += 2520*c5p6*c7p2*c9p1*ss[56];
  jac_4_1 += 720*c5p7*c7p1*c9p1*ss[54];
  jac_4_1 += 90*c1p1*c3p8*ss[28];
  jac_4_1 += 90*c1p1*c5p8*ss[44];
  jac_4_1 += 90*c1p1*c7p8*ss[60];
  jac_4_1 += 90*c1p1*c9p8*ss[76];
  jac_4_1 += 360*c1p2*c3p7*ss[26];
  jac_4_1 += 360*c1p2*c5p7*ss[40];
  jac_4_1 += 360*c1p2*c7p7*ss[54];
  jac_4_1 += 360*c1p2*c9p7*ss[68];
  jac_4_1 += 840*c1p3*c3p6*ss[24];
  jac_4_1 += 840*c1p3*c5p6*ss[36];
  jac_4_1 += 840*c1p3*c7p6*ss[48];
  jac_4_1 += 840*c1p3*c9p6*ss[60];
  jac_4_1 += 1260*c1p4*c3p5*ss[22];
  jac_4_1 += 1260*c1p4*c5p5*ss[32];
  jac_4_1 += 1260*c1p4*c7p5*ss[42];
  jac_4_1 += 1260*c1p4*c9p5*ss[52];
  jac_4_1 += 1260*c1p5*c3p4*ss[20];
  jac_4_1 += 1260*c1p5*c5p4*ss[28];
  jac_4_1 += 1260*c1p5*c7p4*ss[36];
  jac_4_1 += 1260*c1p5*c9p4*ss[44];
  jac_4_1 += 840*c1p6*c3p3*ss[18];
  jac_4_1 += 840*c1p6*c5p3*ss[24];
  jac_4_1 += 840*c1p6*c7p3*ss[30];
  jac_4_1 += 840*c1p6*c9p3*ss[36];
  jac_4_1 += 360*c1p7*c3p2*ss[16];
  jac_4_1 += 360*c1p7*c5p2*ss[20];
  jac_4_1 += 360*c1p7*c7p2*ss[24];
  jac_4_1 += 360*c1p7*c9p2*ss[28];
  jac_4_1 += 90*c1p8*c3p1*ss[14];
  jac_4_1 += 90*c1p8*c5p1*ss[16];
  jac_4_1 += 90*c1p8*c7p1*ss[18];
  jac_4_1 += 90*c1p8*c9p1*ss[20];
  jac_4_1 += 90*c3p1*c5p8*ss[46];
  jac_4_1 += 90*c3p1*c7p8*ss[62];
  jac_4_1 += 90*c3p1*c9p8*ss[78];
  jac_4_1 += 360*c3p2*c5p7*ss[44];
  jac_4_1 += 360*c3p2*c7p7*ss[58];
  jac_4_1 += 360*c3p2*c9p7*ss[72];
  jac_4_1 += 840*c3p3*c5p6*ss[42];
  jac_4_1 += 840*c3p3*c7p6*ss[54];
  jac_4_1 += 840*c3p3*c9p6*ss[66];
  jac_4_1 += 1260*c3p4*c5p5*ss[40];
  jac_4_1 += 1260*c3p4*c7p5*ss[50];
  jac_4_1 += 1260*c3p4*c9p5*ss[60];
  jac_4_1 += 1260*c3p5*c5p4*ss[38];
  jac_4_1 += 1260*c3p5*c7p4*ss[46];
  jac_4_1 += 1260*c3p5*c9p4*ss[54];
  jac_4_1 += 840*c3p6*c5p3*ss[36];
  jac_4_1 += 840*c3p6*c7p3*ss[42];
  jac_4_1 += 840*c3p6*c9p3*ss[48];
  jac_4_1 += 360*c3p7*c5p2*ss[34];
  jac_4_1 += 360*c3p7*c7p2*ss[38];
  jac_4_1 += 360*c3p7*c9p2*ss[42];
  jac_4_1 += 90*c3p8*c5p1*ss[32];
  jac_4_1 += 90*c3p8*c7p1*ss[34];
  jac_4_1 += 90*c3p8*c9p1*ss[36];
  jac_4_1 += 90*c5p1*c7p8*ss[64];
  jac_4_1 += 90*c5p1*c9p8*ss[80];
  jac_4_1 += 360*c5p2*c7p7*ss[62];
  jac_4_1 += 360*c5p2*c9p7*ss[76];
  jac_4_1 += 840*c5p3*c7p6*ss[60];
  jac_4_1 += 840*c5p3*c9p6*ss[72];
  jac_4_1 += 1260*c5p4*c7p5*ss[58];
  jac_4_1 += 1260*c5p4*c9p5*ss[68];
  jac_4_1 += 1260*c5p5*c7p4*ss[56];
  jac_4_1 += 1260*c5p5*c9p4*ss[64];
  jac_4_1 += 840*c5p6*c7p3*ss[54];
  jac_4_1 += 840*c5p6*c9p3*ss[60];
  jac_4_1 += 360*c5p7*c7p2*ss[52];
  jac_4_1 += 360*c5p7*c9p2*ss[56];
  jac_4_1 += 90*c5p8*c7p1*ss[50];
  jac_4_1 += 90*c5p8*c9p1*ss[52];
  jac_4_1 += 90*c7p1*c9p8*ss[82];
  jac_4_1 += 360*c7p2*c9p7*ss[80];
  jac_4_1 += 840*c7p3*c9p6*ss[78];
  jac_4_1 += 1260*c7p4*c9p5*ss[76];
  jac_4_1 += 1260*c7p5*c9p4*ss[74];
  jac_4_1 += 840*c7p6*c9p3*ss[72];
  jac_4_1 += 360*c7p7*c9p2*ss[70];
  jac_4_1 += 90*c7p8*c9p1*ss[68];
  jac_4_1 += 10*c1p9*ss[12];
  jac_4_1 += 10*c3p9*ss[30];
  jac_4_1 += 10*c5p9*ss[48];
  jac_4_1 += 10*c7p9*ss[66];
  jac_4_1 += 10*c9p9*ss[84];
  TACC jac_4_2 = convert<TACC>(30240*c1p1*c3p1*c5p1*c7p1*c9p5*ss[66]);
  jac_4_2 += 75600*c1p1*c3p1*c5p1*c7p2*c9p4*ss[64];
  jac_4_2 += 100800*c1p1*c3p1*c5p1*c7p3*c9p3*ss[62];
  jac_4_2 += 75600*c1p1*c3p1*c5p1*c7p4*c9p2*ss[60];
  jac_4_2 += 30240*c1p1*c3p1*c5p1*c7p5*c9p1*ss[58];
  jac_4_2 += 75600*c1p1*c3p1*c5p2*c7p1*c9p4*ss[62];
  jac_4_2 += 151200*c1p1*c3p1*c5p2*c7p2*c9p3*ss[60];
  jac_4_2 += 151200*c1p1*c3p1*c5p2*c7p3*c9p2*ss[58];
  jac_4_2 += 75600*c1p1*c3p1*c5p2*c7p4*c9p1*ss[56];
  jac_4_2 += 100800*c1p1*c3p1*c5p3*c7p1*c9p3*ss[58];
  jac_4_2 += 151200*c1p1*c3p1*c5p3*c7p2*c9p2*ss[56];
  jac_4_2 += 100800*c1p1*c3p1*c5p3*c7p3*c9p1*ss[54];
  jac_4_2 += 75600*c1p1*c3p1*c5p4*c7p1*c9p2*ss[54];
  jac_4_2 += 75600*c1p1*c3p1*c5p4*c7p2*c9p1*ss[52];
  jac_4_2 += 30240*c1p1*c3p1*c5p5*c7p1*c9p1*ss[50];
  jac_4_2 += 75600*c1p1*c3p2*c5p1*c7p1*c9p4*ss[60];
  jac_4_2 += 151200*c1p1*c3p2*c5p1*c7p2*c9p3*ss[58];
  jac_4_2 += 151200*c1p1*c3p2*c5p1*c7p3*c9p2*ss[56];
  jac_4_2 += 75600*c1p1*c3p2*c5p1*c7p4*c9p1*ss[54];
  jac_4_2 += 151200*c1p1*c3p2*c5p2*c7p1*c9p3*ss[56];
  jac_4_2 += 226800*c1p1*c3p2*c5p2*c7p2*c9p2*ss[54];
  jac_4_2 += 151200*c1p1*c3p2*c5p2*c7p3*c9p1*ss[52];
  jac_4_2 += 151200*c1p1*c3p2*c5p3*c7p1*c9p2*ss[52];
  jac_4_2 += 151200*c1p1*c3p2*c5p3*c7p2*c9p1*ss[50];
  jac_4_2 += 75600*c1p1*c3p2*c5p4*c7p1*c9p1*ss[48];
  jac_4_2 += 100800*c1p1*c3p3*c5p1*c7p1*c9p3*ss[54];
  jac_4_2 += 151200*c1p1*c3p3*c5p1*c7p2*c9p2*ss[52];
  jac_4_2 += 100800*c1p1*c3p3*c5p1*c7p3*c9p1*ss[50];
  jac_4_2 += 151200*c1p1*c3p3*c5p2*c7p1*c9p2*ss[50];
  jac_4_2 += 151200*c1p1*c3p3*c5p2*c7p2*c9p1*ss[48];
  jac_4_2 += 100800*c1p1*c3p3*c5p3*c7p1*c9p1*ss[46];
  jac_4_2 += 75600*c1p1*c3p4*c5p1*c7p1*c9p2*ss[48];
  jac_4_2 += 75600*c1p1*c3p4*c5p1*c7p2*c9p1*ss[46];
  jac_4_2 += 75600*c1p1*c3p4*c5p2*c7p1*c9p1*ss[44];
  jac_4_2 += 30240*c1p1*c3p5*c5p1*c7p1*c9p1*ss[42];
  jac_4_2 += 75600*c1p2*c3p1*c5p1*c7p1*c9p4*ss[58];
  jac_4_2 += 151200*c1p2*c3p1*c5p1*c7p2*c9p3*ss[56];
  jac_4_2 += 151200*c1p2*c3p1*c5p1*c7p3*c9p2*ss[54];
  jac_4_2 += 75600*c1p2*c3p1*c5p1*c7p4*c9p1*ss[52];
  jac_4_2 += 151200*c1p2*c3p1*c5p2*c7p1*c9p3*ss[54];
  jac_4_2 += 226800*c1p2*c3p1*c5p2*c7p2*c9p2*ss[52];
  jac_4_2 += 151200*c1p2*c3p1*c5p2*c7p3*c9p1*ss[50];
  jac_4_2 += 151200*c1p2*c3p1*c5p3*c7p1*c9p2*ss[50];
  jac_4_2 += 151200*c1p2*c3p1*c5p3*c7p2*c9p1*ss[48];
  jac_4_2 += 75600*c1p2*c3p1*c5p4*c7p1*c9p1*ss[46];
  jac_4_2 += 151200*c1p2*c3p2*c5p1*c7p1*c9p3*ss[52];
  jac_4_2 += 226800*c1p2*c3p2*c5p1*c7p2*c9p2*ss[50];
  jac_4_2 += 151200*c1p2*c3p2*c5p1*c7p3*c9p1*ss[48];
  jac_4_2 += 226800*c1p2*c3p2*c5p2*c7p1*c9p2*ss[48];
  jac_4_2 += 226800*c1p2*c3p2*c5p2*c7p2*c9p1*ss[46];
  jac_4_2 += 151200*c1p2*c3p2*c5p3*c7p1*c9p1*ss[44];
  jac_4_2 += 151200*c1p2*c3p3*c5p1*c7p1*c9p2*ss[46];
  jac_4_2 += 151200*c1p2*c3p3*c5p1*c7p2*c9p1*ss[44];
  jac_4_2 += 151200*c1p2*c3p3*c5p2*c7p1*c9p1*ss[42];
  jac_4_2 += 75600*c1p2*c3p4*c5p1*c7p1*c9p1*ss[40];
  jac_4_2 += 100800*c1p3*c3p1*c5p1*c7p1*c9p3*ss[50];
  jac_4_2 += 151200*c1p3*c3p1*c5p1*c7p2*c9p2*ss[48];
  jac_4_2 += 100800*c1p3*c3p1*c5p1*c7p3*c9p1*ss[46];
  jac_4_2 += 151200*c1p3*c3p1*c5p2*c7p1*c9p2*ss[46];
  jac_4_2 += 151200*c1p3*c3p1*c5p2*c7p2*c9p1*ss[44];
  jac_4_2 += 100800*c1p3*c3p1*c5p3*c7p1*c9p1*ss[42];
  jac_4_2 += 151200*c1p3*c3p2*c5p1*c7p1*c9p2*ss[44];
  jac_4_2 += 151200*c1p3*c3p2*c5p1*c7p2*c9p1*ss[42];
  jac_4_2 += 151200*c1p3*c3p2*c5p2*c7p1*c9p1*ss[40];
  jac_4_2 += 100800*c1p3*c3p3*c5p1*c7p1*c9p1*ss[38];
  jac_4_2 += 75600*c1p4*c3p1*c5p1*c7p1*c9p2*ss[42];
  jac_4_2 += 75600*c1p4*c3p1*c5p1*c7p2*c9p1*ss[40];
  jac_4_2 += 75600*c1p4*c3p1*c5p2*c7p1*c9p1*ss[38];
  jac_4_2 += 75600*c1p4*c3p2*c5p1*c7p1*c9p1*ss[36];
  jac_4_2 += 30240*c1p5*c3p1*c5p1*c7p1*c9p1*ss[34];
  jac_4_2 += 5040*c1p1*c3p1*c5p1*c7p6*ss[56];
  jac_4_2 += 5040*c1p1*c3p1*c5p1*c9p6*ss[68];
  jac_4_2 += 15120*c1p1*c3p1*c5p2*c7p5*ss[54];
  jac_4_2 += 15120*c1p1*c3p1*c5p2*c9p5*ss[64];
  jac_4_2 += 25200*c1p1*c3p1*c5p3*c7p4*ss[52];
  jac_4_2 += 25200*c1p1*c3p1*c5p3*c9p4*ss[60];
  jac_4_2 += 25200*c1p1*c3p1*c5p4*c7p3*ss[50];
  jac_4_2 += 25200*c1p1*c3p1*c5p4*c9p3*ss[56];
  jac_4_2 += 15120*c1p1*c3p1*c5p5*c7p2*ss[48];
  jac_4_2 += 15120*c1p1*c3p1*c5p5*c9p2*ss[52];
  jac_4_2 += 5040*c1p1*c3p1*c5p6*c7p1*ss[46];
  jac_4_2 += 5040*c1p1*c3p1*c5p6*c9p1*ss[48];
  jac_4_2 += 5040*c1p1*c3p1*c7p1*c9p6*ss[70];
  jac_4_2 += 15120*c1p1*c3p1*c7p2*c9p5*ss[68];
  jac_4_2 += 25200*c1p1*c3p1*c7p3*c9p4*ss[66];
  jac_4_2 += 25200*c1p1*c3p1*c7p4*c9p3*ss[64];
  jac_4_2 += 15120*c1p1*c3p1*c7p5*c9p2*ss[62];
  jac_4_2 += 5040*c1p1*c3p1*c7p6*c9p1*ss[60];
  jac_4_2 += 15120*c1p1*c3p2*c5p1*c7p5*ss[52];
  jac_4_2 += 15120*c1p1*c3p2*c5p1*c9p5*ss[62];
  jac_4_2 += 37800*c1p1*c3p2*c5p2*c7p4*ss[50];
  jac_4_2 += 37800*c1p1*c3p2*c5p2*c9p4*ss[58];
  jac_4_2 += 50400*c1p1*c3p2*c5p3*c7p3*ss[48];
  jac_4_2 += 50400*c1p1*c3p2*c5p3*c9p3*ss[54];
  jac_4_2 += 37800*c1p1*c3p2*c5p4*c7p2*ss[46];
  jac_4_2 += 37800*c1p1*c3p2*c5p4*c9p2*ss[50];
  jac_4_2 += 15120*c1p1*c3p2*c5p5*c7p1*ss[44];
  jac_4_2 += 15120*c1p1*c3p2*c5p5*c9p1*ss[46];
  jac_4_2 += 15120*c1p1*c3p2*c7p1*c9p5*ss[64];
  jac_4_2 += 37800*c1p1*c3p2*c7p2*c9p4*ss[62];
  jac_4_2 += 50400*c1p1*c3p2*c7p3*c9p3*ss[60];
  jac_4_2 += 37800*c1p1*c3p2*c7p4*c9p2*ss[58];
  jac_4_2 += 15120*c1p1*c3p2*c7p5*c9p1*ss[56];
  jac_4_2 += 25200*c1p1*c3p3*c5p1*c7p4*ss[48];
  jac_4_2 += 25200*c1p1*c3p3*c5p1*c9p4*ss[56];
  jac_4_2 += 50400*c1p1*c3p3*c5p2*c7p3*ss[46];
  jac_4_2 += 50400*c1p1*c3p3*c5p2*c9p3*ss[52];
  jac_4_2 += 50400*c1p1*c3p3*c5p3*c7p2*ss[44];
  jac_4_2 += 50400*c1p1*c3p3*c5p3*c9p2*ss[48];
  jac_4_2 += 25200*c1p1*c3p3*c5p4*c7p1*ss[42];
  jac_4_2 += 25200*c1p1*c3p3*c5p4*c9p1*ss[44];
  jac_4_2 += 25200*c1p1*c3p3*c7p1*c9p4*ss[58];
  jac_4_2 += 50400*c1p1*c3p3*c7p2*c9p3*ss[56];
  jac_4_2 += 50400*c1p1*c3p3*c7p3*c9p2*ss[54];
  jac_4_2 += 25200*c1p1*c3p3*c7p4*c9p1*ss[52];
  jac_4_2 += 25200*c1p1*c3p4*c5p1*c7p3*ss[44];
  jac_4_2 += 25200*c1p1*c3p4*c5p1*c9p3*ss[50];
  jac_4_2 += 37800*c1p1*c3p4*c5p2*c7p2*ss[42];
  jac_4_2 += 37800*c1p1*c3p4*c5p2*c9p2*ss[46];
  jac_4_2 += 25200*c1p1*c3p4*c5p3*c7p1*ss[40];
  jac_4_2 += 25200*c1p1*c3p4*c5p3*c9p1*ss[42];
  jac_4_2 += 25200*c1p1*c3p4*c7p1*c9p3*ss[52];
  jac_4_2 += 37800*c1p1*c3p4*c7p2*c9p2*ss[50];
  jac_4_2 += 25200*c1p1*c3p4*c7p3*c9p1*ss[48];
  jac_4_2 += 15120*c1p1*c3p5*c5p1*c7p2*ss[40];
  jac_4_2 += 15120*c1p1*c3p5*c5p1*c9p2*ss[44];
  jac_4_2 += 15120*c1p1*c3p5*c5p2*c7p1*ss[38];
  jac_4_2 += 15120*c1p1*c3p5*c5p2*c9p1*ss[40];
  jac_4_2 += 15120*c1p1*c3p5*c7p1*c9p2*ss[46];
  jac_4_2 += 15120*c1p1*c3p5*c7p2*c9p1*ss[44];
  jac_4_2 += 5040*c1p1*c3p6*c5p1*c7p1*ss[36];
  jac_4_2 += 5040*c1p1*c3p6*c5p1*c9p1*ss[38];
  jac_4_2 += 5040*c1p1*c3p6*c7p1*c9p1*ss[40];
  jac_4_2 += 5040*c1p1*c5p1*c7p1*c9p6*ss[72];
  jac_4_2 += 15120*c1p1*c5p1*c7p2*c9p5*ss[70];
  jac_4_2 += 25200*c1p1*c5p1*c7p3*c9p4*ss[68];
  jac_4_2 += 25200*c1p1*c5p1*c7p4*c9p3*ss[66];
  jac_4_2 += 15120*c1p1*c5p1*c7p5*c9p2*ss[64];
  jac_4_2 += 5040*c1p1*c5p1*c7p6*c9p1*ss[62];
  jac_4_2 += 15120*c1p1*c5p2*c7p1*c9p5*ss[68];
  jac_4_2 += 37800*c1p1*c5p2*c7p2*c9p4*ss[66];
  jac_4_2 += 50400*c1p1*c5p2*c7p3*c9p3*ss[64];
  jac_4_2 += 37800*c1p1*c5p2*c7p4*c9p2*ss[62];
  jac_4_2 += 15120*c1p1*c5p2*c7p5*c9p1*ss[60];
  jac_4_2 += 25200*c1p1*c5p3*c7p1*c9p4*ss[64];
  jac_4_2 += 50400*c1p1*c5p3*c7p2*c9p3*ss[62];
  jac_4_2 += 50400*c1p1*c5p3*c7p3*c9p2*ss[60];
  jac_4_2 += 25200*c1p1*c5p3*c7p4*c9p1*ss[58];
  jac_4_2 += 25200*c1p1*c5p4*c7p1*c9p3*ss[60];
  jac_4_2 += 37800*c1p1*c5p4*c7p2*c9p2*ss[58];
  jac_4_2 += 25200*c1p1*c5p4*c7p3*c9p1*ss[56];
  jac_4_2 += 15120*c1p1*c5p5*c7p1*c9p2*ss[56];
  jac_4_2 += 15120*c1p1*c5p5*c7p2*c9p1*ss[54];
  jac_4_2 += 5040*c1p1*c5p6*c7p1*c9p1*ss[52];
  jac_4_2 += 15120*c1p2*c3p1*c5p1*c7p5*ss[50];
  jac_4_2 += 15120*c1p2*c3p1*c5p1*c9p5*ss[60];
  jac_4_2 += 37800*c1p2*c3p1*c5p2*c7p4*ss[48];
  jac_4_2 += 37800*c1p2*c3p1*c5p2*c9p4*ss[56];
  jac_4_2 += 50400*c1p2*c3p1*c5p3*c7p3*ss[46];
  jac_4_2 += 50400*c1p2*c3p1*c5p3*c9p3*ss[52];
  jac_4_2 += 37800*c1p2*c3p1*c5p4*c7p2*ss[44];
  jac_4_2 += 37800*c1p2*c3p1*c5p4*c9p2*ss[48];
  jac_4_2 += 15120*c1p2*c3p1*c5p5*c7p1*ss[42];
  jac_4_2 += 15120*c1p2*c3p1*c5p5*c9p1*ss[44];
  jac_4_2 += 15120*c1p2*c3p1*c7p1*c9p5*ss[62];
  jac_4_2 += 37800*c1p2*c3p1*c7p2*c9p4*ss[60];
  jac_4_2 += 50400*c1p2*c3p1*c7p3*c9p3*ss[58];
  jac_4_2 += 37800*c1p2*c3p1*c7p4*c9p2*ss[56];
  jac_4_2 += 15120*c1p2*c3p1*c7p5*c9p1*ss[54];
  jac_4_2 += 37800*c1p2*c3p2*c5p1*c7p4*ss[46];
  jac_4_2 += 37800*c1p2*c3p2*c5p1*c9p4*ss[54];
  jac_4_2 += 75600*c1p2*c3p2*c5p2*c7p3*ss[44];
  jac_4_2 += 75600*c1p2*c3p2*c5p2*c9p3*ss[50];
  jac_4_2 += 75600*c1p2*c3p2*c5p3*c7p2*ss[42];
  jac_4_2 += 75600*c1p2*c3p2*c5p3*c9p2*ss[46];
  jac_4_2 += 37800*c1p2*c3p2*c5p4*c7p1*ss[40];
  jac_4_2 += 37800*c1p2*c3p2*c5p4*c9p1*ss[42];
  jac_4_2 += 37800*c1p2*c3p2*c7p1*c9p4*ss[56];
  jac_4_2 += 75600*c1p2*c3p2*c7p2*c9p3*ss[54];
  jac_4_2 += 75600*c1p2*c3p2*c7p3*c9p2*ss[52];
  jac_4_2 += 37800*c1p2*c3p2*c7p4*c9p1*ss[50];
  jac_4_2 += 50400*c1p2*c3p3*c5p1*c7p3*ss[42];
  jac_4_2 += 50400*c1p2*c3p3*c5p1*c9p3*ss[48];
  jac_4_2 += 75600*c1p2*c3p3*c5p2*c7p2*ss[40];
  jac_4_2 += 75600*c1p2*c3p3*c5p2*c9p2*ss[44];
  jac_4_2 += 50400*c1p2*c3p3*c5p3*c7p1*ss[38];
  jac_4_2 += 50400*c1p2*c3p3*c5p3*c9p1*ss[40];
  jac_4_2 += 50400*c1p2*c3p3*c7p1*c9p3*ss[50];
  jac_4_2 += 75600*c1p2*c3p3*c7p2*c9p2*ss[48];
  jac_4_2 += 50400*c1p2*c3p3*c7p3*c9p1*ss[46];
  jac_4_2 += 37800*c1p2*c3p4*c5p1*c7p2*ss[38];
  jac_4_2 += 37800*c1p2*c3p4*c5p1*c9p2*ss[42];
  jac_4_2 += 37800*c1p2*c3p4*c5p2*c7p1*ss[36];
  jac_4_2 += 37800*c1p2*c3p4*c5p2*c9p1*ss[38];
  jac_4_2 += 37800*c1p2*c3p4*c7p1*c9p2*ss[44];
  jac_4_2 += 37800*c1p2*c3p4*c7p2*c9p1*ss[42];
  jac_4_2 += 15120*c1p2*c3p5*c5p1*c7p1*ss[34];
  jac_4_2 += 15120*c1p2*c3p5*c5p1*c9p1*ss[36];
  jac_4_2 += 15120*c1p2*c3p5*c7p1*c9p1*ss[38];
  jac_4_2 += 15120*c1p2*c5p1*c7p1*c9p5*ss[64];
  jac_4_2 += 37800*c1p2*c5p1*c7p2*c9p4*ss[62];
  jac_4_2 += 50400*c1p2*c5p1*c7p3*c9p3*ss[60];
  jac_4_2 += 37800*c1p2*c5p1*c7p4*c9p2*ss[58];
  jac_4_2 += 15120*c1p2*c5p1*c7p5*c9p1*ss[56];
  jac_4_2 += 37800*c1p2*c5p2*c7p1*c9p4*ss[60];
  jac_4_2 += 75600*c1p2*c5p2*c7p2*c9p3*ss[58];
  jac_4_2 += 75600*c1p2*c5p2*c7p3*c9p2*ss[56];
  jac_4_2 += 37800*c1p2*c5p2*c7p4*c9p1*ss[54];
  jac_4_2 += 50400*c1p2*c5p3*c7p1*c9p3*ss[56];
  jac_4_2 += 75600*c1p2*c5p3*c7p2*c9p2*ss[54];
  jac_4_2 += 50400*c1p2*c5p3*c7p3*c9p1*ss[52];
  jac_4_2 += 37800*c1p2*c5p4*c7p1*c9p2*ss[52];
  jac_4_2 += 37800*c1p2*c5p4*c7p2*c9p1*ss[50];
  jac_4_2 += 15120*c1p2*c5p5*c7p1*c9p1*ss[48];
  jac_4_2 += 25200*c1p3*c3p1*c5p1*c7p4*ss[44];
  jac_4_2 += 25200*c1p3*c3p1*c5p1*c9p4*ss[52];
  jac_4_2 += 50400*c1p3*c3p1*c5p2*c7p3*ss[42];
  jac_4_2 += 50400*c1p3*c3p1*c5p2*c9p3*ss[48];
  jac_4_2 += 50400*c1p3*c3p1*c5p3*c7p2*ss[40];
  jac_4_2 += 50400*c1p3*c3p1*c5p3*c9p2*ss[44];
  jac_4_2 += 25200*c1p3*c3p1*c5p4*c7p1*ss[38];
  jac_4_2 += 25200*c1p3*c3p1*c5p4*c9p1*ss[40];
  jac_4_2 += 25200*c1p3*c3p1*c7p1*c9p4*ss[54];
  jac_4_2 += 50400*c1p3*c3p1*c7p2*c9p3*ss[52];
  jac_4_2 += 50400*c1p3*c3p1*c7p3*c9p2*ss[50];
  jac_4_2 += 25200*c1p3*c3p1*c7p4*c9p1*ss[48];
  jac_4_2 += 50400*c1p3*c3p2*c5p1*c7p3*ss[40];
  jac_4_2 += 50400*c1p3*c3p2*c5p1*c9p3*ss[46];
  jac_4_2 += 75600*c1p3*c3p2*c5p2*c7p2*ss[38];
  jac_4_2 += 75600*c1p3*c3p2*c5p2*c9p2*ss[42];
  jac_4_2 += 50400*c1p3*c3p2*c5p3*c7p1*ss[36];
  jac_4_2 += 50400*c1p3*c3p2*c5p3*c9p1*ss[38];
  jac_4_2 += 50400*c1p3*c3p2*c7p1*c9p3*ss[48];
  jac_4_2 += 75600*c1p3*c3p2*c7p2*c9p2*ss[46];
  jac_4_2 += 50400*c1p3*c3p2*c7p3*c9p1*ss[44];
  jac_4_2 += 50400*c1p3*c3p3*c5p1*c7p2*ss[36];
  jac_4_2 += 50400*c1p3*c3p3*c5p1*c9p2*ss[40];
  jac_4_2 += 50400*c1p3*c3p3*c5p2*c7p1*ss[34];
  jac_4_2 += 50400*c1p3*c3p3*c5p2*c9p1*ss[36];
  jac_4_2 += 50400*c1p3*c3p3*c7p1*c9p2*ss[42];
  jac_4_2 += 50400*c1p3*c3p3*c7p2*c9p1*ss[40];
  jac_4_2 += 25200*c1p3*c3p4*c5p1*c7p1*ss[32];
  jac_4_2 += 25200*c1p3*c3p4*c5p1*c9p1*ss[34];
  jac_4_2 += 25200*c1p3*c3p4*c7p1*c9p1*ss[36];
  jac_4_2 += 25200*c1p3*c5p1*c7p1*c9p4*ss[56];
  jac_4_2 += 50400*c1p3*c5p1*c7p2*c9p3*ss[54];
  jac_4_2 += 50400*c1p3*c5p1*c7p3*c9p2*ss[52];
  jac_4_2 += 25200*c1p3*c5p1*c7p4*c9p1*ss[50];
  jac_4_2 += 50400*c1p3*c5p2*c7p1*c9p3*ss[52];
  jac_4_2 += 75600*c1p3*c5p2*c7p2*c9p2*ss[50];
  jac_4_2 += 50400*c1p3*c5p2*c7p3*c9p1*ss[48];
  jac_4_2 += 50400*c1p3*c5p3*c7p1*c9p2*ss[48];
  jac_4_2 += 50400*c1p3*c5p3*c7p2*c9p1*ss[46];
  jac_4_2 += 25200*c1p3*c5p4*c7p1*c9p1*ss[44];
  jac_4_2 += 25200*c1p4*c3p1*c5p1*c7p3*ss[38];
  jac_4_2 += 25200*c1p4*c3p1*c5p1*c9p3*ss[44];
  jac_4_2 += 37800*c1p4*c3p1*c5p2*c7p2*ss[36];
  jac_4_2 += 37800*c1p4*c3p1*c5p2*c9p2*ss[40];
  jac_4_2 += 25200*c1p4*c3p1*c5p3*c7p1*ss[34];
  jac_4_2 += 25200*c1p4*c3p1*c5p3*c9p1*ss[36];
  jac_4_2 += 25200*c1p4*c3p1*c7p1*c9p3*ss[46];
  jac_4_2 += 37800*c1p4*c3p1*c7p2*c9p2*ss[44];
  jac_4_2 += 25200*c1p4*c3p1*c7p3*c9p1*ss[42];
  jac_4_2 += 37800*c1p4*c3p2*c5p1*c7p2*ss[34];
  jac_4_2 += 37800*c1p4*c3p2*c5p1*c9p2*ss[38];
  jac_4_2 += 37800*c1p4*c3p2*c5p2*c7p1*ss[32];
  jac_4_2 += 37800*c1p4*c3p2*c5p2*c9p1*ss[34];
  jac_4_2 += 37800*c1p4*c3p2*c7p1*c9p2*ss[40];
  jac_4_2 += 37800*c1p4*c3p2*c7p2*c9p1*ss[38];
  jac_4_2 += 25200*c1p4*c3p3*c5p1*c7p1*ss[30];
  jac_4_2 += 25200*c1p4*c3p3*c5p1*c9p1*ss[32];
  jac_4_2 += 25200*c1p4*c3p3*c7p1*c9p1*ss[34];
  jac_4_2 += 25200*c1p4*c5p1*c7p1*c9p3*ss[48];
  jac_4_2 += 37800*c1p4*c5p1*c7p2*c9p2*ss[46];
  jac_4_2 += 25200*c1p4*c5p1*c7p3*c9p1*ss[44];
  jac_4_2 += 37800*c1p4*c5p2*c7p1*c9p2*ss[44];
  jac_4_2 += 37800*c1p4*c5p2*c7p2*c9p1*ss[42];
  jac_4_2 += 25200*c1p4*c5p3*c7p1*c9p1*ss[40];
  jac_4_2 += 15120*c1p5*c3p1*c5p1*c7p2*ss[32];
  jac_4_2 += 15120*c1p5*c3p1*c5p1*c9p2*ss[36];
  jac_4_2 += 15120*c1p5*c3p1*c5p2*c7p1*ss[30];
  jac_4_2 += 15120*c1p5*c3p1*c5p2*c9p1*ss[32];
  jac_4_2 += 15120*c1p5*c3p1*c7p1*c9p2*ss[38];
  jac_4_2 += 15120*c1p5*c3p1*c7p2*c9p1*ss[36];
  jac_4_2 += 15120*c1p5*c3p2*c5p1*c7p1*ss[28];
  jac_4_2 += 15120*c1p5*c3p2*c5p1*c9p1*ss[30];
  jac_4_2 += 15120*c1p5*c3p2*c7p1*c9p1*ss[32];
  jac_4_2 += 15120*c1p5*c5p1*c7p1*c9p2*ss[40];
  jac_4_2 += 15120*c1p5*c5p1*c7p2*c9p1*ss[38];
  jac_4_2 += 15120*c1p5*c5p2*c7p1*c9p1*ss[36];
  jac_4_2 += 5040*c1p6*c3p1*c5p1*c7p1*ss[26];
  jac_4_2 += 5040*c1p6*c3p1*c5p1*c9p1*ss[28];
  jac_4_2 += 5040*c1p6*c3p1*c7p1*c9p1*ss[30];
  jac_4_2 += 5040*c1p6*c5p1*c7p1*c9p1*ss[32];
  jac_4_2 += 5040*c3p1*c5p1*c7p1*c9p6*ss[74];
  jac_4_2 += 15120*c3p1*c5p1*c7p2*c9p5*ss[72];
  jac_4_2 += 25200*c3p1*c5p1*c7p3*c9p4*ss[70];
  jac_4_2 += 25200*c3p1*c5p1*c7p4*c9p3*ss[68];
  jac_4_2 += 15120*c3p1*c5p1*c7p5*c9p2*ss[66];
  jac_4_2 += 5040*c3p1*c5p1*c7p6*c9p1*ss[64];
  jac_4_2 += 15120*c3p1*c5p2*c7p1*c9p5*ss[70];
  jac_4_2 += 37800*c3p1*c5p2*c7p2*c9p4*ss[68];
  jac_4_2 += 50400*c3p1*c5p2*c7p3*c9p3*ss[66];
  jac_4_2 += 37800*c3p1*c5p2*c7p4*c9p2*ss[64];
  jac_4_2 += 15120*c3p1*c5p2*c7p5*c9p1*ss[62];
  jac_4_2 += 25200*c3p1*c5p3*c7p1*c9p4*ss[66];
  jac_4_2 += 50400*c3p1*c5p3*c7p2*c9p3*ss[64];
  jac_4_2 += 50400*c3p1*c5p3*c7p3*c9p2*ss[62];
  jac_4_2 += 25200*c3p1*c5p3*c7p4*c9p1*ss[60];
  jac_4_2 += 25200*c3p1*c5p4*c7p1*c9p3*ss[62];
  jac_4_2 += 37800*c3p1*c5p4*c7p2*c9p2*ss[60];
  jac_4_2 += 25200*c3p1*c5p4*c7p3*c9p1*ss[58];
  jac_4_2 += 15120*c3p1*c5p5*c7p1*c9p2*ss[58];
  jac_4_2 += 15120*c3p1*c5p5*c7p2*c9p1*ss[56];
  jac_4_2 += 5040*c3p1*c5p6*c7p1*c9p1*ss[54];
  jac_4_2 += 15120*c3p2*c5p1*c7p1*c9p5*ss[68];
  jac_4_2 += 37800*c3p2*c5p1*c7p2*c9p4*ss[66];
  jac_4_2 += 50400*c3p2*c5p1*c7p3*c9p3*ss[64];
  jac_4_2 += 37800*c3p2*c5p1*c7p4*c9p2*ss[62];
  jac_4_2 += 15120*c3p2*c5p1*c7p5*c9p1*ss[60];
  jac_4_2 += 37800*c3p2*c5p2*c7p1*c9p4*ss[64];
  jac_4_2 += 75600*c3p2*c5p2*c7p2*c9p3*ss[62];
  jac_4_2 += 75600*c3p2*c5p2*c7p3*c9p2*ss[60];
  jac_4_2 += 37800*c3p2*c5p2*c7p4*c9p1*ss[58];
  jac_4_2 += 50400*c3p2*c5p3*c7p1*c9p3*ss[60];
  jac_4_2 += 75600*c3p2*c5p3*c7p2*c9p2*ss[58];
  jac_4_2 += 50400*c3p2*c5p3*c7p3*c9p1*ss[56];
  jac_4_2 += 37800*c3p2*c5p4*c7p1*c9p2*ss[56];
  jac_4_2 += 37800*c3p2*c5p4*c7p2*c9p1*ss[54];
  jac_4_2 += 15120*c3p2*c5p5*c7p1*c9p1*ss[52];
  jac_4_2 += 25200*c3p3*c5p1*c7p1*c9p4*ss[62];
  jac_4_2 += 50400*c3p3*c5p1*c7p2*c9p3*ss[60];
  jac_4_2 += 50400*c3p3*c5p1*c7p3*c9p2*ss[58];
  jac_4_2 += 25200*c3p3*c5p1*c7p4*c9p1*ss[56];
  jac_4_2 += 50400*c3p3*c5p2*c7p1*c9p3*ss[58];
  jac_4_2 += 75600*c3p3*c5p2*c7p2*c9p2*ss[56];
  jac_4_2 += 50400*c3p3*c5p2*c7p3*c9p1*ss[54];
  jac_4_2 += 50400*c3p3*c5p3*c7p1*c9p2*ss[54];
  jac_4_2 += 50400*c3p3*c5p3*c7p2*c9p1*ss[52];
  jac_4_2 += 25200*c3p3*c5p4*c7p1*c9p1*ss[50];
  jac_4_2 += 25200*c3p4*c5p1*c7p1*c9p3*ss[56];
  jac_4_2 += 37800*c3p4*c5p1*c7p2*c9p2*ss[54];
  jac_4_2 += 25200*c3p4*c5p1*c7p3*c9p1*ss[52];
  jac_4_2 += 37800*c3p4*c5p2*c7p1*c9p2*ss[52];
  jac_4_2 += 37800*c3p4*c5p2*c7p2*c9p1*ss[50];
  jac_4_2 += 25200*c3p4*c5p3*c7p1*c9p1*ss[48];
  jac_4_2 += 15120*c3p5*c5p1*c7p1*c9p2*ss[50];
  jac_4_2 += 15120*c3p5*c5p1*c7p2*c9p1*ss[48];
  jac_4_2 += 15120*c3p5*c5p2*c7p1*c9p1*ss[46];
  jac_4_2 += 5040*c3p6*c5p1*c7p1*c9p1*ss[44];
  jac_4_2 += 720*c1p1*c3p1*c5p7*ss[44];
  jac_4_2 += 720*c1p1*c3p1*c7p7*ss[58];
  jac_4_2 += 720*c1p1*c3p1*c9p7*ss[72];
  jac_4_2 += 2520*c1p1*c3p2*c5p6*ss[42];
  jac_4_2 += 2520*c1p1*c3p2*c7p6*ss[54];
  jac_4_2 += 2520*c1p1*c3p2*c9p6*ss[66];
  jac_4_2 += 5040*c1p1*c3p3*c5p5*ss[40];
  jac_4_2 += 5040*c1p1*c3p3*c7p5*ss[50];
  jac_4_2 += 5040*c1p1*c3p3*c9p5*ss[60];
  jac_4_2 += 6300*c1p1*c3p4*c5p4*ss[38];
  jac_4_2 += 6300*c1p1*c3p4*c7p4*ss[46];
  jac_4_2 += 6300*c1p1*c3p4*c9p4*ss[54];
  jac_4_2 += 5040*c1p1*c3p5*c5p3*ss[36];
  jac_4_2 += 5040*c1p1*c3p5*c7p3*ss[42];
  jac_4_2 += 5040*c1p1*c3p5*c9p3*ss[48];
  jac_4_2 += 2520*c1p1*c3p6*c5p2*ss[34];
  jac_4_2 += 2520*c1p1*c3p6*c7p2*ss[38];
  jac_4_2 += 2520*c1p1*c3p6*c9p2*ss[42];
  jac_4_2 += 720*c1p1*c3p7*c5p1*ss[32];
  jac_4_2 += 720*c1p1*c3p7*c7p1*ss[34];
  jac_4_2 += 720*c1p1*c3p7*c9p1*ss[36];
  jac_4_2 += 720*c1p1*c5p1*c7p7*ss[60];
  jac_4_2 += 720*c1p1*c5p1*c9p7*ss[74];
  jac_4_2 += 2520*c1p1*c5p2*c7p6*ss[58];
  jac_4_2 += 2520*c1p1*c5p2*c9p6*ss[70];
  jac_4_2 += 5040*c1p1*c5p3*c7p5*ss[56];
  jac_4_2 += 5040*c1p1*c5p3*c9p5*ss[66];
  jac_4_2 += 6300*c1p1*c5p4*c7p4*ss[54];
  jac_4_2 += 6300*c1p1*c5p4*c9p4*ss[62];
  jac_4_2 += 5040*c1p1*c5p5*c7p3*ss[52];
  jac_4_2 += 5040*c1p1*c5p5*c9p3*ss[58];
  jac_4_2 += 2520*c1p1*c5p6*c7p2*ss[50];
  jac_4_2 += 2520*c1p1*c5p6*c9p2*ss[54];
  jac_4_2 += 720*c1p1*c5p7*c7p1*ss[48];
  jac_4_2 += 720*c1p1*c5p7*c9p1*ss[50];
  jac_4_2 += 720*c1p1*c7p1*c9p7*ss[76];
  jac_4_2 += 2520*c1p1*c7p2*c9p6*ss[74];
  jac_4_2 += 5040*c1p1*c7p3*c9p5*ss[72];
  jac_4_2 += 6300*c1p1*c7p4*c9p4*ss[70];
  jac_4_2 += 5040*c1p1*c7p5*c9p3*ss[68];
  jac_4_2 += 2520*c1p1*c7p6*c9p2*ss[66];
  jac_4_2 += 720*c1p1*c7p7*c9p1*ss[64];
  jac_4_2 += 2520*c1p2*c3p1*c5p6*ss[40];
  jac_4_2 += 2520*c1p2*c3p1*c7p6*ss[52];
  jac_4_2 += 2520*c1p2*c3p1*c9p6*ss[64];
  jac_4_2 += 7560*c1p2*c3p2*c5p5*ss[38];
  jac_4_2 += 7560*c1p2*c3p2*c7p5*ss[48];
  jac_4_2 += 7560*c1p2*c3p2*c9p5*ss[58];
  jac_4_2 += 12600*c1p2*c3p3*c5p4*ss[36];
  jac_4_2 += 12600*c1p2*c3p3*c7p4*ss[44];
  jac_4_2 += 12600*c1p2*c3p3*c9p4*ss[52];
  jac_4_2 += 12600*c1p2*c3p4*c5p3*ss[34];
  jac_4_2 += 12600*c1p2*c3p4*c7p3*ss[40];
  jac_4_2 += 12600*c1p2*c3p4*c9p3*ss[46];
  jac_4_2 += 7560*c1p2*c3p5*c5p2*ss[32];
  jac_4_2 += 7560*c1p2*c3p5*c7p2*ss[36];
  jac_4_2 += 7560*c1p2*c3p5*c9p2*ss[40];
  jac_4_2 += 2520*c1p2*c3p6*c5p1*ss[30];
  jac_4_2 += 2520*c1p2*c3p6*c7p1*ss[32];
  jac_4_2 += 2520*c1p2*c3p6*c9p1*ss[34];
  jac_4_2 += 2520*c1p2*c5p1*c7p6*ss[54];
  jac_4_2 += 2520*c1p2*c5p1*c9p6*ss[66];
  jac_4_2 += 7560*c1p2*c5p2*c7p5*ss[52];
  jac_4_2 += 7560*c1p2*c5p2*c9p5*ss[62];
  jac_4_2 += 12600*c1p2*c5p3*c7p4*ss[50];
  jac_4_2 += 12600*c1p2*c5p3*c9p4*ss[58];
  jac_4_2 += 12600*c1p2*c5p4*c7p3*ss[48];
  jac_4_2 += 12600*c1p2*c5p4*c9p3*ss[54];
  jac_4_2 += 7560*c1p2*c5p5*c7p2*ss[46];
  jac_4_2 += 7560*c1p2*c5p5*c9p2*ss[50];
  jac_4_2 += 2520*c1p2*c5p6*c7p1*ss[44];
  jac_4_2 += 2520*c1p2*c5p6*c9p1*ss[46];
  jac_4_2 += 2520*c1p2*c7p1*c9p6*ss[68];
  jac_4_2 += 7560*c1p2*c7p2*c9p5*ss[66];
  jac_4_2 += 12600*c1p2*c7p3*c9p4*ss[64];
  jac_4_2 += 12600*c1p2*c7p4*c9p3*ss[62];
  jac_4_2 += 7560*c1p2*c7p5*c9p2*ss[60];
  jac_4_2 += 2520*c1p2*c7p6*c9p1*ss[58];
  jac_4_2 += 5040*c1p3*c3p1*c5p5*ss[36];
  jac_4_2 += 5040*c1p3*c3p1*c7p5*ss[46];
  jac_4_2 += 5040*c1p3*c3p1*c9p5*ss[56];
  jac_4_2 += 12600*c1p3*c3p2*c5p4*ss[34];
  jac_4_2 += 12600*c1p3*c3p2*c7p4*ss[42];
  jac_4_2 += 12600*c1p3*c3p2*c9p4*ss[50];
  jac_4_2 += 16800*c1p3*c3p3*c5p3*ss[32];
  jac_4_2 += 16800*c1p3*c3p3*c7p3*ss[38];
  jac_4_2 += 16800*c1p3*c3p3*c9p3*ss[44];
  jac_4_2 += 12600*c1p3*c3p4*c5p2*ss[30];
  jac_4_2 += 12600*c1p3*c3p4*c7p2*ss[34];
  jac_4_2 += 12600*c1p3*c3p4*c9p2*ss[38];
  jac_4_2 += 5040*c1p3*c3p5*c5p1*ss[28];
  jac_4_2 += 5040*c1p3*c3p5*c7p1*ss[30];
  jac_4_2 += 5040*c1p3*c3p5*c9p1*ss[32];
  jac_4_2 += 5040*c1p3*c5p1*c7p5*ss[48];
  jac_4_2 += 5040*c1p3*c5p1*c9p5*ss[58];
  jac_4_2 += 12600*c1p3*c5p2*c7p4*ss[46];
  jac_4_2 += 12600*c1p3*c5p2*c9p4*ss[54];
  jac_4_2 += 16800*c1p3*c5p3*c7p3*ss[44];
  jac_4_2 += 16800*c1p3*c5p3*c9p3*ss[50];
  jac_4_2 += 12600*c1p3*c5p4*c7p2*ss[42];
  jac_4_2 += 12600*c1p3*c5p4*c9p2*ss[46];
  jac_4_2 += 5040*c1p3*c5p5*c7p1*ss[40];
  jac_4_2 += 5040*c1p3*c5p5*c9p1*ss[42];
  jac_4_2 += 5040*c1p3*c7p1*c9p5*ss[60];
  jac_4_2 += 12600*c1p3*c7p2*c9p4*ss[58];
  jac_4_2 += 16800*c1p3*c7p3*c9p3*ss[56];
  jac_4_2 += 12600*c1p3*c7p4*c9p2*ss[54];
  jac_4_2 += 5040*c1p3*c7p5*c9p1*ss[52];
  jac_4_2 += 6300*c1p4*c3p1*c5p4*ss[32];
  jac_4_2 += 6300*c1p4*c3p1*c7p4*ss[40];
  jac_4_2 += 6300*c1p4*c3p1*c9p4*ss[48];
  jac_4_2 += 12600*c1p4*c3p2*c5p3*ss[30];
  jac_4_2 += 12600*c1p4*c3p2*c7p3*ss[36];
  jac_4_2 += 12600*c1p4*c3p2*c9p3*ss[42];
  jac_4_2 += 12600*c1p4*c3p3*c5p2*ss[28];
  jac_4_2 += 12600*c1p4*c3p3*c7p2*ss[32];
  jac_4_2 += 12600*c1p4*c3p3*c9p2*ss[36];
  jac_4_2 += 6300*c1p4*c3p4*c5p1*ss[26];
  jac_4_2 += 6300*c1p4*c3p4*c7p1*ss[28];
  jac_4_2 += 6300*c1p4*c3p4*c9p1*ss[30];
  jac_4_2 += 6300*c1p4*c5p1*c7p4*ss[42];
  jac_4_2 += 6300*c1p4*c5p1*c9p4*ss[50];
  jac_4_2 += 12600*c1p4*c5p2*c7p3*ss[40];
  jac_4_2 += 12600*c1p4*c5p2*c9p3*ss[46];
  jac_4_2 += 12600*c1p4*c5p3*c7p2*ss[38];
  jac_4_2 += 12600*c1p4*c5p3*c9p2*ss[42];
  jac_4_2 += 6300*c1p4*c5p4*c7p1*ss[36];
  jac_4_2 += 6300*c1p4*c5p4*c9p1*ss[38];
  jac_4_2 += 6300*c1p4*c7p1*c9p4*ss[52];
  jac_4_2 += 12600*c1p4*c7p2*c9p3*ss[50];
  jac_4_2 += 12600*c1p4*c7p3*c9p2*ss[48];
  jac_4_2 += 6300*c1p4*c7p4*c9p1*ss[46];
  jac_4_2 += 5040*c1p5*c3p1*c5p3*ss[28];
  jac_4_2 += 5040*c1p5*c3p1*c7p3*ss[34];
  jac_4_2 += 5040*c1p5*c3p1*c9p3*ss[40];
  jac_4_2 += 7560*c1p5*c3p2*c5p2*ss[26];
  jac_4_2 += 7560*c1p5*c3p2*c7p2*ss[30];
  jac_4_2 += 7560*c1p5*c3p2*c9p2*ss[34];
  jac_4_2 += 5040*c1p5*c3p3*c5p1*ss[24];
  jac_4_2 += 5040*c1p5*c3p3*c7p1*ss[26];
  jac_4_2 += 5040*c1p5*c3p3*c9p1*ss[28];
  jac_4_2 += 5040*c1p5*c5p1*c7p3*ss[36];
  jac_4_2 += 5040*c1p5*c5p1*c9p3*ss[42];
  jac_4_2 += 7560*c1p5*c5p2*c7p2*ss[34];
  jac_4_2 += 7560*c1p5*c5p2*c9p2*ss[38];
  jac_4_2 += 5040*c1p5*c5p3*c7p1*ss[32];
  jac_4_2 += 5040*c1p5*c5p3*c9p1*ss[34];
  jac_4_2 += 5040*c1p5*c7p1*c9p3*ss[44];
  jac_4_2 += 7560*c1p5*c7p2*c9p2*ss[42];
  jac_4_2 += 5040*c1p5*c7p3*c9p1*ss[40];
  jac_4_2 += 2520*c1p6*c3p1*c5p2*ss[24];
  jac_4_2 += 2520*c1p6*c3p1*c7p2*ss[28];
  jac_4_2 += 2520*c1p6*c3p1*c9p2*ss[32];
  jac_4_2 += 2520*c1p6*c3p2*c5p1*ss[22];
  jac_4_2 += 2520*c1p6*c3p2*c7p1*ss[24];
  jac_4_2 += 2520*c1p6*c3p2*c9p1*ss[26];
  jac_4_2 += 2520*c1p6*c5p1*c7p2*ss[30];
  jac_4_2 += 2520*c1p6*c5p1*c9p2*ss[34];
  jac_4_2 += 2520*c1p6*c5p2*c7p1*ss[28];
  jac_4_2 += 2520*c1p6*c5p2*c9p1*ss[30];
  jac_4_2 += 2520*c1p6*c7p1*c9p2*ss[36];
  jac_4_2 += 2520*c1p6*c7p2*c9p1*ss[34];
  jac_4_2 += 720*c1p7*c3p1*c5p1*ss[20];
  jac_4_2 += 720*c1p7*c3p1*c7p1*ss[22];
  jac_4_2 += 720*c1p7*c3p1*c9p1*ss[24];
  jac_4_2 += 720*c1p7*c5p1*c7p1*ss[24];
  jac_4_2 += 720*c1p7*c5p1*c9p1*ss[26];
  jac_4_2 += 720*c1p7*c7p1*c9p1*ss[28];
  jac_4_2 += 720*c3p1*c5p1*c7p7*ss[62];
  jac_4_2 += 720*c3p1*c5p1*c9p7*ss[76];
  jac_4_2 += 2520*c3p1*c5p2*c7p6*ss[60];
  jac_4_2 += 2520*c3p1*c5p2*c9p6*ss[72];
  jac_4_2 += 5040*c3p1*c5p3*c7p5*ss[58];
  jac_4_2 += 5040*c3p1*c5p3*c9p5*ss[68];
  jac_4_2 += 6300*c3p1*c5p4*c7p4*ss[56];
  jac_4_2 += 6300*c3p1*c5p4*c9p4*ss[64];
  jac_4_2 += 5040*c3p1*c5p5*c7p3*ss[54];
  jac_4_2 += 5040*c3p1*c5p5*c9p3*ss[60];
  jac_4_2 += 2520*c3p1*c5p6*c7p2*ss[52];
  jac_4_2 += 2520*c3p1*c5p6*c9p2*ss[56];
  jac_4_2 += 720*c3p1*c5p7*c7p1*ss[50];
  jac_4_2 += 720*c3p1*c5p7*c9p1*ss[52];
  jac_4_2 += 720*c3p1*c7p1*c9p7*ss[78];
  jac_4_2 += 2520*c3p1*c7p2*c9p6*ss[76];
  jac_4_2 += 5040*c3p1*c7p3*c9p5*ss[74];
  jac_4_2 += 6300*c3p1*c7p4*c9p4*ss[72];
  jac_4_2 += 5040*c3p1*c7p5*c9p3*ss[70];
  jac_4_2 += 2520*c3p1*c7p6*c9p2*ss[68];
  jac_4_2 += 720*c3p1*c7p7*c9p1*ss[66];
  jac_4_2 += 2520*c3p2*c5p1*c7p6*ss[58];
  jac_4_2 += 2520*c3p2*c5p1*c9p6*ss[70];
  jac_4_2 += 7560*c3p2*c5p2*c7p5*ss[56];
  jac_4_2 += 7560*c3p2*c5p2*c9p5*ss[66];
  jac_4_2 += 12600*c3p2*c5p3*c7p4*ss[54];
  jac_4_2 += 12600*c3p2*c5p3*c9p4*ss[62];
  jac_4_2 += 12600*c3p2*c5p4*c7p3*ss[52];
  jac_4_2 += 12600*c3p2*c5p4*c9p3*ss[58];
  jac_4_2 += 7560*c3p2*c5p5*c7p2*ss[50];
  jac_4_2 += 7560*c3p2*c5p5*c9p2*ss[54];
  jac_4_2 += 2520*c3p2*c5p6*c7p1*ss[48];
  jac_4_2 += 2520*c3p2*c5p6*c9p1*ss[50];
  jac_4_2 += 2520*c3p2*c7p1*c9p6*ss[72];
  jac_4_2 += 7560*c3p2*c7p2*c9p5*ss[70];
  jac_4_2 += 12600*c3p2*c7p3*c9p4*ss[68];
  jac_4_2 += 12600*c3p2*c7p4*c9p3*ss[66];
  jac_4_2 += 7560*c3p2*c7p5*c9p2*ss[64];
  jac_4_2 += 2520*c3p2*c7p6*c9p1*ss[62];
  jac_4_2 += 5040*c3p3*c5p1*c7p5*ss[54];
  jac_4_2 += 5040*c3p3*c5p1*c9p5*ss[64];
  jac_4_2 += 12600*c3p3*c5p2*c7p4*ss[52];
  jac_4_2 += 12600*c3p3*c5p2*c9p4*ss[60];
  jac_4_2 += 16800*c3p3*c5p3*c7p3*ss[50];
  jac_4_2 += 16800*c3p3*c5p3*c9p3*ss[56];
  jac_4_2 += 12600*c3p3*c5p4*c7p2*ss[48];
  jac_4_2 += 12600*c3p3*c5p4*c9p2*ss[52];
  jac_4_2 += 5040*c3p3*c5p5*c7p1*ss[46];
  jac_4_2 += 5040*c3p3*c5p5*c9p1*ss[48];
  jac_4_2 += 5040*c3p3*c7p1*c9p5*ss[66];
  jac_4_2 += 12600*c3p3*c7p2*c9p4*ss[64];
  jac_4_2 += 16800*c3p3*c7p3*c9p3*ss[62];
  jac_4_2 += 12600*c3p3*c7p4*c9p2*ss[60];
  jac_4_2 += 5040*c3p3*c7p5*c9p1*ss[58];
  jac_4_2 += 6300*c3p4*c5p1*c7p4*ss[50];
  jac_4_2 += 6300*c3p4*c5p1*c9p4*ss[58];
  jac_4_2 += 12600*c3p4*c5p2*c7p3*ss[48];
  jac_4_2 += 12600*c3p4*c5p2*c9p3*ss[54];
  jac_4_2 += 12600*c3p4*c5p3*c7p2*ss[46];
  jac_4_2 += 12600*c3p4*c5p3*c9p2*ss[50];
  jac_4_2 += 6300*c3p4*c5p4*c7p1*ss[44];
  jac_4_2 += 6300*c3p4*c5p4*c9p1*ss[46];
  jac_4_2 += 6300*c3p4*c7p1*c9p4*ss[60];
  jac_4_2 += 12600*c3p4*c7p2*c9p3*ss[58];
  jac_4_2 += 12600*c3p4*c7p3*c9p2*ss[56];
  jac_4_2 += 6300*c3p4*c7p4*c9p1*ss[54];
  jac_4_2 += 5040*c3p5*c5p1*c7p3*ss[46];
  jac_4_2 += 5040*c3p5*c5p1*c9p3*ss[52];
  jac_4_2 += 7560*c3p5*c5p2*c7p2*ss[44];
  jac_4_2 += 7560*c3p5*c5p2*c9p2*ss[48];
  jac_4_2 += 5040*c3p5*c5p3*c7p1*ss[42];
  jac_4_2 += 5040*c3p5*c5p3*c9p1*ss[44];
  jac_4_2 += 5040*c3p5*c7p1*c9p3*ss[54];
  jac_4_2 += 7560*c3p5*c7p2*c9p2*ss[52];
  jac_4_2 += 5040*c3p5*c7p3*c9p1*ss[50];
  jac_4_2 += 2520*c3p6*c5p1*c7p2*ss[42];
  jac_4_2 += 2520*c3p6*c5p1*c9p2*ss[46];
  jac_4_2 += 2520*c3p6*c5p2*c7p1*ss[40];
  jac_4_2 += 2520*c3p6*c5p2*c9p1*ss[42];
  jac_4_2 += 2520*c3p6*c7p1*c9p2*ss[48];
  jac_4_2 += 2520*c3p6*c7p2*c9p1*ss[46];
  jac_4_2 += 720*c3p7*c5p1*c7p1*ss[38];
  jac_4_2 += 720*c3p7*c5p1*c9p1*ss[40];
  jac_4_2 += 720*c3p7*c7p1*c9p1*ss[42];
  jac_4_2 += 720*c5p1*c7p1*c9p7*ss[80];
  jac_4_2 += 2520*c5p1*c7p2*c9p6*ss[78];
  jac_4_2 += 5040*c5p1*c7p3*c9p5*ss[76];
  jac_4_2 += 6300*c5p1*c7p4*c9p4*ss[74];
  jac_4_2 += 5040*c5p1*c7p5*c9p3*ss[72];
  jac_4_2 += 2520*c5p1*c7p6*c9p2*ss[70];
  jac_4_2 += 720*c5p1*c7p7*c9p1*ss[68];
  jac_4_2 += 2520*c5p2*c7p1*c9p6*ss[76];
  jac_4_2 += 7560*c5p2*c7p2*c9p5*ss[74];
  jac_4_2 += 12600*c5p2*c7p3*c9p4*ss[72];
  jac_4_2 += 12600*c5p2*c7p4*c9p3*ss[70];
  jac_4_2 += 7560*c5p2*c7p5*c9p2*ss[68];
  jac_4_2 += 2520*c5p2*c7p6*c9p1*ss[66];
  jac_4_2 += 5040*c5p3*c7p1*c9p5*ss[72];
  jac_4_2 += 12600*c5p3*c7p2*c9p4*ss[70];
  jac_4_2 += 16800*c5p3*c7p3*c9p3*ss[68];
  jac_4_2 += 12600*c5p3*c7p4*c9p2*ss[66];
  jac_4_2 += 5040*c5p3*c7p5*c9p1*ss[64];
  jac_4_2 += 6300*c5p4*c7p1*c9p4*ss[68];
  jac_4_2 += 12600*c5p4*c7p2*c9p3*ss[66];
  jac_4_2 += 12600*c5p4*c7p3*c9p2*ss[64];
  jac_4_2 += 6300*c5p4*c7p4*c9p1*ss[62];
  jac_4_2 += 5040*c5p5*c7p1*c9p3*ss[64];
  jac_4_2 += 7560*c5p5*c7p2*c9p2*ss[62];
  jac_4_2 += 5040*c5p5*c7p3*c9p1*ss[60];
  jac_4_2 += 2520*c5p6*c7p1*c9p2*ss[60];
  jac_4_2 += 2520*c5p6*c7p2*c9p1*ss[58];
  jac_4_2 += 720*c5p7*c7p1*c9p1*ss[56];
  jac_4_2 += 90*c1p1*c3p8*ss[30];
  jac_4_2 += 90*c1p1*c5p8*ss[46];
  jac_4_2 += 90*c1p1*c7p8*ss[62];
  jac_4_2 += 90*c1p1*c9p8*ss[78];
  jac_4_2 += 360*c1p2*c3p7*ss[28];
  jac_4_2 += 360*c1p2*c5p7*ss[42];
  jac_4_2 += 360*c1p2*c7p7*ss[56];
  jac_4_2 += 360*c1p2*c9p7*ss[70];
  jac_4_2 += 840*c1p3*c3p6*ss[26];
  jac_4_2 += 840*c1p3*c5p6*ss[38];
  jac_4_2 += 840*c1p3*c7p6*ss[50];
  jac_4_2 += 840*c1p3*c9p6*ss[62];
  jac_4_2 += 1260*c1p4*c3p5*ss[24];
  jac_4_2 += 1260*c1p4*c5p5*ss[34];
  jac_4_2 += 1260*c1p4*c7p5*ss[44];
  jac_4_2 += 1260*c1p4*c9p5*ss[54];
  jac_4_2 += 1260*c1p5*c3p4*ss[22];
  jac_4_2 += 1260*c1p5*c5p4*ss[30];
  jac_4_2 += 1260*c1p5*c7p4*ss[38];
  jac_4_2 += 1260*c1p5*c9p4*ss[46];
  jac_4_2 += 840*c1p6*c3p3*ss[20];
  jac_4_2 += 840*c1p6*c5p3*ss[26];
  jac_4_2 += 840*c1p6*c7p3*ss[32];
  jac_4_2 += 840*c1p6*c9p3*ss[38];
  jac_4_2 += 360*c1p7*c3p2*ss[18];
  jac_4_2 += 360*c1p7*c5p2*ss[22];
  jac_4_2 += 360*c1p7*c7p2*ss[26];
  jac_4_2 += 360*c1p7*c9p2*ss[30];
  jac_4_2 += 90*c1p8*c3p1*ss[16];
  jac_4_2 += 90*c1p8*c5p1*ss[18];
  jac_4_2 += 90*c1p8*c7p1*ss[20];
  jac_4_2 += 90*c1p8*c9p1*ss[22];
  jac_4_2 += 90*c3p1*c5p8*ss[48];
  jac_4_2 += 90*c3p1*c7p8*ss[64];
  jac_4_2 += 90*c3p1*c9p8*ss[80];
  jac_4_2 += 360*c3p2*c5p7*ss[46];
  jac_4_2 += 360*c3p2*c7p7*ss[60];
  jac_4_2 += 360*c3p2*c9p7*ss[74];
  jac_4_2 += 840*c3p3*c5p6*ss[44];
  jac_4_2 += 840*c3p3*c7p6*ss[56];
  jac_4_2 += 840*c3p3*c9p6*ss[68];
  jac_4_2 += 1260*c3p4*c5p5*ss[42];
  jac_4_2 += 1260*c3p4*c7p5*ss[52];
  jac_4_2 += 1260*c3p4*c9p5*ss[62];
  jac_4_2 += 1260*c3p5*c5p4*ss[40];
  jac_4_2 += 1260*c3p5*c7p4*ss[48];
  jac_4_2 += 1260*c3p5*c9p4*ss[56];
  jac_4_2 += 840*c3p6*c5p3*ss[38];
  jac_4_2 += 840*c3p6*c7p3*ss[44];
  jac_4_2 += 840*c3p6*c9p3*ss[50];
  jac_4_2 += 360*c3p7*c5p2*ss[36];
  jac_4_2 += 360*c3p7*c7p2*ss[40];
  jac_4_2 += 360*c3p7*c9p2*ss[44];
  jac_4_2 += 90*c3p8*c5p1*ss[34];
  jac_4_2 += 90*c3p8*c7p1*ss[36];
  jac_4_2 += 90*c3p8*c9p1*ss[38];
  jac_4_2 += 90*c5p1*c7p8*ss[66];
  jac_4_2 += 90*c5p1*c9p8*ss[82];
  jac_4_2 += 360*c5p2*c7p7*ss[64];
  jac_4_2 += 360*c5p2*c9p7*ss[78];
  jac_4_2 += 840*c5p3*c7p6*ss[62];
  jac_4_2 += 840*c5p3*c9p6*ss[74];
  jac_4_2 += 1260*c5p4*c7p5*ss[60];
  jac_4_2 += 1260*c5p4*c9p5*ss[70];
  jac_4_2 += 1260*c5p5*c7p4*ss[58];
  jac_4_2 += 1260*c5p5*c9p4*ss[66];
  jac_4_2 += 840*c5p6*c7p3*ss[56];
  jac_4_2 += 840*c5p6*c9p3*ss[62];
  jac_4_2 += 360*c5p7*c7p2*ss[54];
  jac_4_2 += 360*c5p7*c9p2*ss[58];
  jac_4_2 += 90*c5p8*c7p1*ss[52];
  jac_4_2 += 90*c5p8*c9p1*ss[54];
  jac_4_2 += 90*c7p1*c9p8*ss[84];
  jac_4_2 += 360*c7p2*c9p7*ss[82];
  jac_4_2 += 840*c7p3*c9p6*ss[80];
  jac_4_2 += 1260*c7p4*c9p5*ss[78];
  jac_4_2 += 1260*c7p5*c9p4*ss[76];
  jac_4_2 += 840*c7p6*c9p3*ss[74];
  jac_4_2 += 360*c7p7*c9p2*ss[72];
  jac_4_2 += 90*c7p8*c9p1*ss[70];
  jac_4_2 += 10*c1p9*ss[14];
  jac_4_2 += 10*c3p9*ss[32];
  jac_4_2 += 10*c5p9*ss[50];
  jac_4_2 += 10*c7p9*ss[68];
  jac_4_2 += 10*c9p9*ss[86];
  TACC jac_4_3 = convert<TACC>(30240*c1p1*c3p1*c5p1*c7p1*c9p5*ss[68]);
  jac_4_3 += 75600*c1p1*c3p1*c5p1*c7p2*c9p4*ss[66];
  jac_4_3 += 100800*c1p1*c3p1*c5p1*c7p3*c9p3*ss[64];
  jac_4_3 += 75600*c1p1*c3p1*c5p1*c7p4*c9p2*ss[62];
  jac_4_3 += 30240*c1p1*c3p1*c5p1*c7p5*c9p1*ss[60];
  jac_4_3 += 75600*c1p1*c3p1*c5p2*c7p1*c9p4*ss[64];
  jac_4_3 += 151200*c1p1*c3p1*c5p2*c7p2*c9p3*ss[62];
  jac_4_3 += 151200*c1p1*c3p1*c5p2*c7p3*c9p2*ss[60];
  jac_4_3 += 75600*c1p1*c3p1*c5p2*c7p4*c9p1*ss[58];
  jac_4_3 += 100800*c1p1*c3p1*c5p3*c7p1*c9p3*ss[60];
  jac_4_3 += 151200*c1p1*c3p1*c5p3*c7p2*c9p2*ss[58];
  jac_4_3 += 100800*c1p1*c3p1*c5p3*c7p3*c9p1*ss[56];
  jac_4_3 += 75600*c1p1*c3p1*c5p4*c7p1*c9p2*ss[56];
  jac_4_3 += 75600*c1p1*c3p1*c5p4*c7p2*c9p1*ss[54];
  jac_4_3 += 30240*c1p1*c3p1*c5p5*c7p1*c9p1*ss[52];
  jac_4_3 += 75600*c1p1*c3p2*c5p1*c7p1*c9p4*ss[62];
  jac_4_3 += 151200*c1p1*c3p2*c5p1*c7p2*c9p3*ss[60];
  jac_4_3 += 151200*c1p1*c3p2*c5p1*c7p3*c9p2*ss[58];
  jac_4_3 += 75600*c1p1*c3p2*c5p1*c7p4*c9p1*ss[56];
  jac_4_3 += 151200*c1p1*c3p2*c5p2*c7p1*c9p3*ss[58];
  jac_4_3 += 226800*c1p1*c3p2*c5p2*c7p2*c9p2*ss[56];
  jac_4_3 += 151200*c1p1*c3p2*c5p2*c7p3*c9p1*ss[54];
  jac_4_3 += 151200*c1p1*c3p2*c5p3*c7p1*c9p2*ss[54];
  jac_4_3 += 151200*c1p1*c3p2*c5p3*c7p2*c9p1*ss[52];
  jac_4_3 += 75600*c1p1*c3p2*c5p4*c7p1*c9p1*ss[50];
  jac_4_3 += 100800*c1p1*c3p3*c5p1*c7p1*c9p3*ss[56];
  jac_4_3 += 151200*c1p1*c3p3*c5p1*c7p2*c9p2*ss[54];
  jac_4_3 += 100800*c1p1*c3p3*c5p1*c7p3*c9p1*ss[52];
  jac_4_3 += 151200*c1p1*c3p3*c5p2*c7p1*c9p2*ss[52];
  jac_4_3 += 151200*c1p1*c3p3*c5p2*c7p2*c9p1*ss[50];
  jac_4_3 += 100800*c1p1*c3p3*c5p3*c7p1*c9p1*ss[48];
  jac_4_3 += 75600*c1p1*c3p4*c5p1*c7p1*c9p2*ss[50];
  jac_4_3 += 75600*c1p1*c3p4*c5p1*c7p2*c9p1*ss[48];
  jac_4_3 += 75600*c1p1*c3p4*c5p2*c7p1*c9p1*ss[46];
  jac_4_3 += 30240*c1p1*c3p5*c5p1*c7p1*c9p1*ss[44];
  jac_4_3 += 75600*c1p2*c3p1*c5p1*c7p1*c9p4*ss[60];
  jac_4_3 += 151200*c1p2*c3p1*c5p1*c7p2*c9p3*ss[58];
  jac_4_3 += 151200*c1p2*c3p1*c5p1*c7p3*c9p2*ss[56];
  jac_4_3 += 75600*c1p2*c3p1*c5p1*c7p4*c9p1*ss[54];
  jac_4_3 += 151200*c1p2*c3p1*c5p2*c7p1*c9p3*ss[56];
  jac_4_3 += 226800*c1p2*c3p1*c5p2*c7p2*c9p2*ss[54];
  jac_4_3 += 151200*c1p2*c3p1*c5p2*c7p3*c9p1*ss[52];
  jac_4_3 += 151200*c1p2*c3p1*c5p3*c7p1*c9p2*ss[52];
  jac_4_3 += 151200*c1p2*c3p1*c5p3*c7p2*c9p1*ss[50];
  jac_4_3 += 75600*c1p2*c3p1*c5p4*c7p1*c9p1*ss[48];
  jac_4_3 += 151200*c1p2*c3p2*c5p1*c7p1*c9p3*ss[54];
  jac_4_3 += 226800*c1p2*c3p2*c5p1*c7p2*c9p2*ss[52];
  jac_4_3 += 151200*c1p2*c3p2*c5p1*c7p3*c9p1*ss[50];
  jac_4_3 += 226800*c1p2*c3p2*c5p2*c7p1*c9p2*ss[50];
  jac_4_3 += 226800*c1p2*c3p2*c5p2*c7p2*c9p1*ss[48];
  jac_4_3 += 151200*c1p2*c3p2*c5p3*c7p1*c9p1*ss[46];
  jac_4_3 += 151200*c1p2*c3p3*c5p1*c7p1*c9p2*ss[48];
  jac_4_3 += 151200*c1p2*c3p3*c5p1*c7p2*c9p1*ss[46];
  jac_4_3 += 151200*c1p2*c3p3*c5p2*c7p1*c9p1*ss[44];
  jac_4_3 += 75600*c1p2*c3p4*c5p1*c7p1*c9p1*ss[42];
  jac_4_3 += 100800*c1p3*c3p1*c5p1*c7p1*c9p3*ss[52];
  jac_4_3 += 151200*c1p3*c3p1*c5p1*c7p2*c9p2*ss[50];
  jac_4_3 += 100800*c1p3*c3p1*c5p1*c7p3*c9p1*ss[48];
  jac_4_3 += 151200*c1p3*c3p1*c5p2*c7p1*c9p2*ss[48];
  jac_4_3 += 151200*c1p3*c3p1*c5p2*c7p2*c9p1*ss[46];
  jac_4_3 += 100800*c1p3*c3p1*c5p3*c7p1*c9p1*ss[44];
  jac_4_3 += 151200*c1p3*c3p2*c5p1*c7p1*c9p2*ss[46];
  jac_4_3 += 151200*c1p3*c3p2*c5p1*c7p2*c9p1*ss[44];
  jac_4_3 += 151200*c1p3*c3p2*c5p2*c7p1*c9p1*ss[42];
  jac_4_3 += 100800*c1p3*c3p3*c5p1*c7p1*c9p1*ss[40];
  jac_4_3 += 75600*c1p4*c3p1*c5p1*c7p1*c9p2*ss[44];
  jac_4_3 += 75600*c1p4*c3p1*c5p1*c7p2*c9p1*ss[42];
  jac_4_3 += 75600*c1p4*c3p1*c5p2*c7p1*c9p1*ss[40];
  jac_4_3 += 75600*c1p4*c3p2*c5p1*c7p1*c9p1*ss[38];
  jac_4_3 += 30240*c1p5*c3p1*c5p1*c7p1*c9p1*ss[36];
  jac_4_3 += 5040*c1p1*c3p1*c5p1*c7p6*ss[58];
  jac_4_3 += 5040*c1p1*c3p1*c5p1*c9p6*ss[70];
  jac_4_3 += 15120*c1p1*c3p1*c5p2*c7p5*ss[56];
  jac_4_3 += 15120*c1p1*c3p1*c5p2*c9p5*ss[66];
  jac_4_3 += 25200*c1p1*c3p1*c5p3*c7p4*ss[54];
  jac_4_3 += 25200*c1p1*c3p1*c5p3*c9p4*ss[62];
  jac_4_3 += 25200*c1p1*c3p1*c5p4*c7p3*ss[52];
  jac_4_3 += 25200*c1p1*c3p1*c5p4*c9p3*ss[58];
  jac_4_3 += 15120*c1p1*c3p1*c5p5*c7p2*ss[50];
  jac_4_3 += 15120*c1p1*c3p1*c5p5*c9p2*ss[54];
  jac_4_3 += 5040*c1p1*c3p1*c5p6*c7p1*ss[48];
  jac_4_3 += 5040*c1p1*c3p1*c5p6*c9p1*ss[50];
  jac_4_3 += 5040*c1p1*c3p1*c7p1*c9p6*ss[72];
  jac_4_3 += 15120*c1p1*c3p1*c7p2*c9p5*ss[70];
  jac_4_3 += 25200*c1p1*c3p1*c7p3*c9p4*ss[68];
  jac_4_3 += 25200*c1p1*c3p1*c7p4*c9p3*ss[66];
  jac_4_3 += 15120*c1p1*c3p1*c7p5*c9p2*ss[64];
  jac_4_3 += 5040*c1p1*c3p1*c7p6*c9p1*ss[62];
  jac_4_3 += 15120*c1p1*c3p2*c5p1*c7p5*ss[54];
  jac_4_3 += 15120*c1p1*c3p2*c5p1*c9p5*ss[64];
  jac_4_3 += 37800*c1p1*c3p2*c5p2*c7p4*ss[52];
  jac_4_3 += 37800*c1p1*c3p2*c5p2*c9p4*ss[60];
  jac_4_3 += 50400*c1p1*c3p2*c5p3*c7p3*ss[50];
  jac_4_3 += 50400*c1p1*c3p2*c5p3*c9p3*ss[56];
  jac_4_3 += 37800*c1p1*c3p2*c5p4*c7p2*ss[48];
  jac_4_3 += 37800*c1p1*c3p2*c5p4*c9p2*ss[52];
  jac_4_3 += 15120*c1p1*c3p2*c5p5*c7p1*ss[46];
  jac_4_3 += 15120*c1p1*c3p2*c5p5*c9p1*ss[48];
  jac_4_3 += 15120*c1p1*c3p2*c7p1*c9p5*ss[66];
  jac_4_3 += 37800*c1p1*c3p2*c7p2*c9p4*ss[64];
  jac_4_3 += 50400*c1p1*c3p2*c7p3*c9p3*ss[62];
  jac_4_3 += 37800*c1p1*c3p2*c7p4*c9p2*ss[60];
  jac_4_3 += 15120*c1p1*c3p2*c7p5*c9p1*ss[58];
  jac_4_3 += 25200*c1p1*c3p3*c5p1*c7p4*ss[50];
  jac_4_3 += 25200*c1p1*c3p3*c5p1*c9p4*ss[58];
  jac_4_3 += 50400*c1p1*c3p3*c5p2*c7p3*ss[48];
  jac_4_3 += 50400*c1p1*c3p3*c5p2*c9p3*ss[54];
  jac_4_3 += 50400*c1p1*c3p3*c5p3*c7p2*ss[46];
  jac_4_3 += 50400*c1p1*c3p3*c5p3*c9p2*ss[50];
  jac_4_3 += 25200*c1p1*c3p3*c5p4*c7p1*ss[44];
  jac_4_3 += 25200*c1p1*c3p3*c5p4*c9p1*ss[46];
  jac_4_3 += 25200*c1p1*c3p3*c7p1*c9p4*ss[60];
  jac_4_3 += 50400*c1p1*c3p3*c7p2*c9p3*ss[58];
  jac_4_3 += 50400*c1p1*c3p3*c7p3*c9p2*ss[56];
  jac_4_3 += 25200*c1p1*c3p3*c7p4*c9p1*ss[54];
  jac_4_3 += 25200*c1p1*c3p4*c5p1*c7p3*ss[46];
  jac_4_3 += 25200*c1p1*c3p4*c5p1*c9p3*ss[52];
  jac_4_3 += 37800*c1p1*c3p4*c5p2*c7p2*ss[44];
  jac_4_3 += 37800*c1p1*c3p4*c5p2*c9p2*ss[48];
  jac_4_3 += 25200*c1p1*c3p4*c5p3*c7p1*ss[42];
  jac_4_3 += 25200*c1p1*c3p4*c5p3*c9p1*ss[44];
  jac_4_3 += 25200*c1p1*c3p4*c7p1*c9p3*ss[54];
  jac_4_3 += 37800*c1p1*c3p4*c7p2*c9p2*ss[52];
  jac_4_3 += 25200*c1p1*c3p4*c7p3*c9p1*ss[50];
  jac_4_3 += 15120*c1p1*c3p5*c5p1*c7p2*ss[42];
  jac_4_3 += 15120*c1p1*c3p5*c5p1*c9p2*ss[46];
  jac_4_3 += 15120*c1p1*c3p5*c5p2*c7p1*ss[40];
  jac_4_3 += 15120*c1p1*c3p5*c5p2*c9p1*ss[42];
  jac_4_3 += 15120*c1p1*c3p5*c7p1*c9p2*ss[48];
  jac_4_3 += 15120*c1p1*c3p5*c7p2*c9p1*ss[46];
  jac_4_3 += 5040*c1p1*c3p6*c5p1*c7p1*ss[38];
  jac_4_3 += 5040*c1p1*c3p6*c5p1*c9p1*ss[40];
  jac_4_3 += 5040*c1p1*c3p6*c7p1*c9p1*ss[42];
  jac_4_3 += 5040*c1p1*c5p1*c7p1*c9p6*ss[74];
  jac_4_3 += 15120*c1p1*c5p1*c7p2*c9p5*ss[72];
  jac_4_3 += 25200*c1p1*c5p1*c7p3*c9p4*ss[70];
  jac_4_3 += 25200*c1p1*c5p1*c7p4*c9p3*ss[68];
  jac_4_3 += 15120*c1p1*c5p1*c7p5*c9p2*ss[66];
  jac_4_3 += 5040*c1p1*c5p1*c7p6*c9p1*ss[64];
  jac_4_3 += 15120*c1p1*c5p2*c7p1*c9p5*ss[70];
  jac_4_3 += 37800*c1p1*c5p2*c7p2*c9p4*ss[68];
  jac_4_3 += 50400*c1p1*c5p2*c7p3*c9p3*ss[66];
  jac_4_3 += 37800*c1p1*c5p2*c7p4*c9p2*ss[64];
  jac_4_3 += 15120*c1p1*c5p2*c7p5*c9p1*ss[62];
  jac_4_3 += 25200*c1p1*c5p3*c7p1*c9p4*ss[66];
  jac_4_3 += 50400*c1p1*c5p3*c7p2*c9p3*ss[64];
  jac_4_3 += 50400*c1p1*c5p3*c7p3*c9p2*ss[62];
  jac_4_3 += 25200*c1p1*c5p3*c7p4*c9p1*ss[60];
  jac_4_3 += 25200*c1p1*c5p4*c7p1*c9p3*ss[62];
  jac_4_3 += 37800*c1p1*c5p4*c7p2*c9p2*ss[60];
  jac_4_3 += 25200*c1p1*c5p4*c7p3*c9p1*ss[58];
  jac_4_3 += 15120*c1p1*c5p5*c7p1*c9p2*ss[58];
  jac_4_3 += 15120*c1p1*c5p5*c7p2*c9p1*ss[56];
  jac_4_3 += 5040*c1p1*c5p6*c7p1*c9p1*ss[54];
  jac_4_3 += 15120*c1p2*c3p1*c5p1*c7p5*ss[52];
  jac_4_3 += 15120*c1p2*c3p1*c5p1*c9p5*ss[62];
  jac_4_3 += 37800*c1p2*c3p1*c5p2*c7p4*ss[50];
  jac_4_3 += 37800*c1p2*c3p1*c5p2*c9p4*ss[58];
  jac_4_3 += 50400*c1p2*c3p1*c5p3*c7p3*ss[48];
  jac_4_3 += 50400*c1p2*c3p1*c5p3*c9p3*ss[54];
  jac_4_3 += 37800*c1p2*c3p1*c5p4*c7p2*ss[46];
  jac_4_3 += 37800*c1p2*c3p1*c5p4*c9p2*ss[50];
  jac_4_3 += 15120*c1p2*c3p1*c5p5*c7p1*ss[44];
  jac_4_3 += 15120*c1p2*c3p1*c5p5*c9p1*ss[46];
  jac_4_3 += 15120*c1p2*c3p1*c7p1*c9p5*ss[64];
  jac_4_3 += 37800*c1p2*c3p1*c7p2*c9p4*ss[62];
  jac_4_3 += 50400*c1p2*c3p1*c7p3*c9p3*ss[60];
  jac_4_3 += 37800*c1p2*c3p1*c7p4*c9p2*ss[58];
  jac_4_3 += 15120*c1p2*c3p1*c7p5*c9p1*ss[56];
  jac_4_3 += 37800*c1p2*c3p2*c5p1*c7p4*ss[48];
  jac_4_3 += 37800*c1p2*c3p2*c5p1*c9p4*ss[56];
  jac_4_3 += 75600*c1p2*c3p2*c5p2*c7p3*ss[46];
  jac_4_3 += 75600*c1p2*c3p2*c5p2*c9p3*ss[52];
  jac_4_3 += 75600*c1p2*c3p2*c5p3*c7p2*ss[44];
  jac_4_3 += 75600*c1p2*c3p2*c5p3*c9p2*ss[48];
  jac_4_3 += 37800*c1p2*c3p2*c5p4*c7p1*ss[42];
  jac_4_3 += 37800*c1p2*c3p2*c5p4*c9p1*ss[44];
  jac_4_3 += 37800*c1p2*c3p2*c7p1*c9p4*ss[58];
  jac_4_3 += 75600*c1p2*c3p2*c7p2*c9p3*ss[56];
  jac_4_3 += 75600*c1p2*c3p2*c7p3*c9p2*ss[54];
  jac_4_3 += 37800*c1p2*c3p2*c7p4*c9p1*ss[52];
  jac_4_3 += 50400*c1p2*c3p3*c5p1*c7p3*ss[44];
  jac_4_3 += 50400*c1p2*c3p3*c5p1*c9p3*ss[50];
  jac_4_3 += 75600*c1p2*c3p3*c5p2*c7p2*ss[42];
  jac_4_3 += 75600*c1p2*c3p3*c5p2*c9p2*ss[46];
  jac_4_3 += 50400*c1p2*c3p3*c5p3*c7p1*ss[40];
  jac_4_3 += 50400*c1p2*c3p3*c5p3*c9p1*ss[42];
  jac_4_3 += 50400*c1p2*c3p3*c7p1*c9p3*ss[52];
  jac_4_3 += 75600*c1p2*c3p3*c7p2*c9p2*ss[50];
  jac_4_3 += 50400*c1p2*c3p3*c7p3*c9p1*ss[48];
  jac_4_3 += 37800*c1p2*c3p4*c5p1*c7p2*ss[40];
  jac_4_3 += 37800*c1p2*c3p4*c5p1*c9p2*ss[44];
  jac_4_3 += 37800*c1p2*c3p4*c5p2*c7p1*ss[38];
  jac_4_3 += 37800*c1p2*c3p4*c5p2*c9p1*ss[40];
  jac_4_3 += 37800*c1p2*c3p4*c7p1*c9p2*ss[46];
  jac_4_3 += 37800*c1p2*c3p4*c7p2*c9p1*ss[44];
  jac_4_3 += 15120*c1p2*c3p5*c5p1*c7p1*ss[36];
  jac_4_3 += 15120*c1p2*c3p5*c5p1*c9p1*ss[38];
  jac_4_3 += 15120*c1p2*c3p5*c7p1*c9p1*ss[40];
  jac_4_3 += 15120*c1p2*c5p1*c7p1*c9p5*ss[66];
  jac_4_3 += 37800*c1p2*c5p1*c7p2*c9p4*ss[64];
  jac_4_3 += 50400*c1p2*c5p1*c7p3*c9p3*ss[62];
  jac_4_3 += 37800*c1p2*c5p1*c7p4*c9p2*ss[60];
  jac_4_3 += 15120*c1p2*c5p1*c7p5*c9p1*ss[58];
  jac_4_3 += 37800*c1p2*c5p2*c7p1*c9p4*ss[62];
  jac_4_3 += 75600*c1p2*c5p2*c7p2*c9p3*ss[60];
  jac_4_3 += 75600*c1p2*c5p2*c7p3*c9p2*ss[58];
  jac_4_3 += 37800*c1p2*c5p2*c7p4*c9p1*ss[56];
  jac_4_3 += 50400*c1p2*c5p3*c7p1*c9p3*ss[58];
  jac_4_3 += 75600*c1p2*c5p3*c7p2*c9p2*ss[56];
  jac_4_3 += 50400*c1p2*c5p3*c7p3*c9p1*ss[54];
  jac_4_3 += 37800*c1p2*c5p4*c7p1*c9p2*ss[54];
  jac_4_3 += 37800*c1p2*c5p4*c7p2*c9p1*ss[52];
  jac_4_3 += 15120*c1p2*c5p5*c7p1*c9p1*ss[50];
  jac_4_3 += 25200*c1p3*c3p1*c5p1*c7p4*ss[46];
  jac_4_3 += 25200*c1p3*c3p1*c5p1*c9p4*ss[54];
  jac_4_3 += 50400*c1p3*c3p1*c5p2*c7p3*ss[44];
  jac_4_3 += 50400*c1p3*c3p1*c5p2*c9p3*ss[50];
  jac_4_3 += 50400*c1p3*c3p1*c5p3*c7p2*ss[42];
  jac_4_3 += 50400*c1p3*c3p1*c5p3*c9p2*ss[46];
  jac_4_3 += 25200*c1p3*c3p1*c5p4*c7p1*ss[40];
  jac_4_3 += 25200*c1p3*c3p1*c5p4*c9p1*ss[42];
  jac_4_3 += 25200*c1p3*c3p1*c7p1*c9p4*ss[56];
  jac_4_3 += 50400*c1p3*c3p1*c7p2*c9p3*ss[54];
  jac_4_3 += 50400*c1p3*c3p1*c7p3*c9p2*ss[52];
  jac_4_3 += 25200*c1p3*c3p1*c7p4*c9p1*ss[50];
  jac_4_3 += 50400*c1p3*c3p2*c5p1*c7p3*ss[42];
  jac_4_3 += 50400*c1p3*c3p2*c5p1*c9p3*ss[48];
  jac_4_3 += 75600*c1p3*c3p2*c5p2*c7p2*ss[40];
  jac_4_3 += 75600*c1p3*c3p2*c5p2*c9p2*ss[44];
  jac_4_3 += 50400*c1p3*c3p2*c5p3*c7p1*ss[38];
  jac_4_3 += 50400*c1p3*c3p2*c5p3*c9p1*ss[40];
  jac_4_3 += 50400*c1p3*c3p2*c7p1*c9p3*ss[50];
  jac_4_3 += 75600*c1p3*c3p2*c7p2*c9p2*ss[48];
  jac_4_3 += 50400*c1p3*c3p2*c7p3*c9p1*ss[46];
  jac_4_3 += 50400*c1p3*c3p3*c5p1*c7p2*ss[38];
  jac_4_3 += 50400*c1p3*c3p3*c5p1*c9p2*ss[42];
  jac_4_3 += 50400*c1p3*c3p3*c5p2*c7p1*ss[36];
  jac_4_3 += 50400*c1p3*c3p3*c5p2*c9p1*ss[38];
  jac_4_3 += 50400*c1p3*c3p3*c7p1*c9p2*ss[44];
  jac_4_3 += 50400*c1p3*c3p3*c7p2*c9p1*ss[42];
  jac_4_3 += 25200*c1p3*c3p4*c5p1*c7p1*ss[34];
  jac_4_3 += 25200*c1p3*c3p4*c5p1*c9p1*ss[36];
  jac_4_3 += 25200*c1p3*c3p4*c7p1*c9p1*ss[38];
  jac_4_3 += 25200*c1p3*c5p1*c7p1*c9p4*ss[58];
  jac_4_3 += 50400*c1p3*c5p1*c7p2*c9p3*ss[56];
  jac_4_3 += 50400*c1p3*c5p1*c7p3*c9p2*ss[54];
  jac_4_3 += 25200*c1p3*c5p1*c7p4*c9p1*ss[52];
  jac_4_3 += 50400*c1p3*c5p2*c7p1*c9p3*ss[54];
  jac_4_3 += 75600*c1p3*c5p2*c7p2*c9p2*ss[52];
  jac_4_3 += 50400*c1p3*c5p2*c7p3*c9p1*ss[50];
  jac_4_3 += 50400*c1p3*c5p3*c7p1*c9p2*ss[50];
  jac_4_3 += 50400*c1p3*c5p3*c7p2*c9p1*ss[48];
  jac_4_3 += 25200*c1p3*c5p4*c7p1*c9p1*ss[46];
  jac_4_3 += 25200*c1p4*c3p1*c5p1*c7p3*ss[40];
  jac_4_3 += 25200*c1p4*c3p1*c5p1*c9p3*ss[46];
  jac_4_3 += 37800*c1p4*c3p1*c5p2*c7p2*ss[38];
  jac_4_3 += 37800*c1p4*c3p1*c5p2*c9p2*ss[42];
  jac_4_3 += 25200*c1p4*c3p1*c5p3*c7p1*ss[36];
  jac_4_3 += 25200*c1p4*c3p1*c5p3*c9p1*ss[38];
  jac_4_3 += 25200*c1p4*c3p1*c7p1*c9p3*ss[48];
  jac_4_3 += 37800*c1p4*c3p1*c7p2*c9p2*ss[46];
  jac_4_3 += 25200*c1p4*c3p1*c7p3*c9p1*ss[44];
  jac_4_3 += 37800*c1p4*c3p2*c5p1*c7p2*ss[36];
  jac_4_3 += 37800*c1p4*c3p2*c5p1*c9p2*ss[40];
  jac_4_3 += 37800*c1p4*c3p2*c5p2*c7p1*ss[34];
  jac_4_3 += 37800*c1p4*c3p2*c5p2*c9p1*ss[36];
  jac_4_3 += 37800*c1p4*c3p2*c7p1*c9p2*ss[42];
  jac_4_3 += 37800*c1p4*c3p2*c7p2*c9p1*ss[40];
  jac_4_3 += 25200*c1p4*c3p3*c5p1*c7p1*ss[32];
  jac_4_3 += 25200*c1p4*c3p3*c5p1*c9p1*ss[34];
  jac_4_3 += 25200*c1p4*c3p3*c7p1*c9p1*ss[36];
  jac_4_3 += 25200*c1p4*c5p1*c7p1*c9p3*ss[50];
  jac_4_3 += 37800*c1p4*c5p1*c7p2*c9p2*ss[48];
  jac_4_3 += 25200*c1p4*c5p1*c7p3*c9p1*ss[46];
  jac_4_3 += 37800*c1p4*c5p2*c7p1*c9p2*ss[46];
  jac_4_3 += 37800*c1p4*c5p2*c7p2*c9p1*ss[44];
  jac_4_3 += 25200*c1p4*c5p3*c7p1*c9p1*ss[42];
  jac_4_3 += 15120*c1p5*c3p1*c5p1*c7p2*ss[34];
  jac_4_3 += 15120*c1p5*c3p1*c5p1*c9p2*ss[38];
  jac_4_3 += 15120*c1p5*c3p1*c5p2*c7p1*ss[32];
  jac_4_3 += 15120*c1p5*c3p1*c5p2*c9p1*ss[34];
  jac_4_3 += 15120*c1p5*c3p1*c7p1*c9p2*ss[40];
  jac_4_3 += 15120*c1p5*c3p1*c7p2*c9p1*ss[38];
  jac_4_3 += 15120*c1p5*c3p2*c5p1*c7p1*ss[30];
  jac_4_3 += 15120*c1p5*c3p2*c5p1*c9p1*ss[32];
  jac_4_3 += 15120*c1p5*c3p2*c7p1*c9p1*ss[34];
  jac_4_3 += 15120*c1p5*c5p1*c7p1*c9p2*ss[42];
  jac_4_3 += 15120*c1p5*c5p1*c7p2*c9p1*ss[40];
  jac_4_3 += 15120*c1p5*c5p2*c7p1*c9p1*ss[38];
  jac_4_3 += 5040*c1p6*c3p1*c5p1*c7p1*ss[28];
  jac_4_3 += 5040*c1p6*c3p1*c5p1*c9p1*ss[30];
  jac_4_3 += 5040*c1p6*c3p1*c7p1*c9p1*ss[32];
  jac_4_3 += 5040*c1p6*c5p1*c7p1*c9p1*ss[34];
  jac_4_3 += 5040*c3p1*c5p1*c7p1*c9p6*ss[76];
  jac_4_3 += 15120*c3p1*c5p1*c7p2*c9p5*ss[74];
  jac_4_3 += 25200*c3p1*c5p1*c7p3*c9p4*ss[72];
  jac_4_3 += 25200*c3p1*c5p1*c7p4*c9p3*ss[70];
  jac_4_3 += 15120*c3p1*c5p1*c7p5*c9p2*ss[68];
  jac_4_3 += 5040*c3p1*c5p1*c7p6*c9p1*ss[66];
  jac_4_3 += 15120*c3p1*c5p2*c7p1*c9p5*ss[72];
  jac_4_3 += 37800*c3p1*c5p2*c7p2*c9p4*ss[70];
  jac_4_3 += 50400*c3p1*c5p2*c7p3*c9p3*ss[68];
  jac_4_3 += 37800*c3p1*c5p2*c7p4*c9p2*ss[66];
  jac_4_3 += 15120*c3p1*c5p2*c7p5*c9p1*ss[64];
  jac_4_3 += 25200*c3p1*c5p3*c7p1*c9p4*ss[68];
  jac_4_3 += 50400*c3p1*c5p3*c7p2*c9p3*ss[66];
  jac_4_3 += 50400*c3p1*c5p3*c7p3*c9p2*ss[64];
  jac_4_3 += 25200*c3p1*c5p3*c7p4*c9p1*ss[62];
  jac_4_3 += 25200*c3p1*c5p4*c7p1*c9p3*ss[64];
  jac_4_3 += 37800*c3p1*c5p4*c7p2*c9p2*ss[62];
  jac_4_3 += 25200*c3p1*c5p4*c7p3*c9p1*ss[60];
  jac_4_3 += 15120*c3p1*c5p5*c7p1*c9p2*ss[60];
  jac_4_3 += 15120*c3p1*c5p5*c7p2*c9p1*ss[58];
  jac_4_3 += 5040*c3p1*c5p6*c7p1*c9p1*ss[56];
  jac_4_3 += 15120*c3p2*c5p1*c7p1*c9p5*ss[70];
  jac_4_3 += 37800*c3p2*c5p1*c7p2*c9p4*ss[68];
  jac_4_3 += 50400*c3p2*c5p1*c7p3*c9p3*ss[66];
  jac_4_3 += 37800*c3p2*c5p1*c7p4*c9p2*ss[64];
  jac_4_3 += 15120*c3p2*c5p1*c7p5*c9p1*ss[62];
  jac_4_3 += 37800*c3p2*c5p2*c7p1*c9p4*ss[66];
  jac_4_3 += 75600*c3p2*c5p2*c7p2*c9p3*ss[64];
  jac_4_3 += 75600*c3p2*c5p2*c7p3*c9p2*ss[62];
  jac_4_3 += 37800*c3p2*c5p2*c7p4*c9p1*ss[60];
  jac_4_3 += 50400*c3p2*c5p3*c7p1*c9p3*ss[62];
  jac_4_3 += 75600*c3p2*c5p3*c7p2*c9p2*ss[60];
  jac_4_3 += 50400*c3p2*c5p3*c7p3*c9p1*ss[58];
  jac_4_3 += 37800*c3p2*c5p4*c7p1*c9p2*ss[58];
  jac_4_3 += 37800*c3p2*c5p4*c7p2*c9p1*ss[56];
  jac_4_3 += 15120*c3p2*c5p5*c7p1*c9p1*ss[54];
  jac_4_3 += 25200*c3p3*c5p1*c7p1*c9p4*ss[64];
  jac_4_3 += 50400*c3p3*c5p1*c7p2*c9p3*ss[62];
  jac_4_3 += 50400*c3p3*c5p1*c7p3*c9p2*ss[60];
  jac_4_3 += 25200*c3p3*c5p1*c7p4*c9p1*ss[58];
  jac_4_3 += 50400*c3p3*c5p2*c7p1*c9p3*ss[60];
  jac_4_3 += 75600*c3p3*c5p2*c7p2*c9p2*ss[58];
  jac_4_3 += 50400*c3p3*c5p2*c7p3*c9p1*ss[56];
  jac_4_3 += 50400*c3p3*c5p3*c7p1*c9p2*ss[56];
  jac_4_3 += 50400*c3p3*c5p3*c7p2*c9p1*ss[54];
  jac_4_3 += 25200*c3p3*c5p4*c7p1*c9p1*ss[52];
  jac_4_3 += 25200*c3p4*c5p1*c7p1*c9p3*ss[58];
  jac_4_3 += 37800*c3p4*c5p1*c7p2*c9p2*ss[56];
  jac_4_3 += 25200*c3p4*c5p1*c7p3*c9p1*ss[54];
  jac_4_3 += 37800*c3p4*c5p2*c7p1*c9p2*ss[54];
  jac_4_3 += 37800*c3p4*c5p2*c7p2*c9p1*ss[52];
  jac_4_3 += 25200*c3p4*c5p3*c7p1*c9p1*ss[50];
  jac_4_3 += 15120*c3p5*c5p1*c7p1*c9p2*ss[52];
  jac_4_3 += 15120*c3p5*c5p1*c7p2*c9p1*ss[50];
  jac_4_3 += 15120*c3p5*c5p2*c7p1*c9p1*ss[48];
  jac_4_3 += 5040*c3p6*c5p1*c7p1*c9p1*ss[46];
  jac_4_3 += 720*c1p1*c3p1*c5p7*ss[46];
  jac_4_3 += 720*c1p1*c3p1*c7p7*ss[60];
  jac_4_3 += 720*c1p1*c3p1*c9p7*ss[74];
  jac_4_3 += 2520*c1p1*c3p2*c5p6*ss[44];
  jac_4_3 += 2520*c1p1*c3p2*c7p6*ss[56];
  jac_4_3 += 2520*c1p1*c3p2*c9p6*ss[68];
  jac_4_3 += 5040*c1p1*c3p3*c5p5*ss[42];
  jac_4_3 += 5040*c1p1*c3p3*c7p5*ss[52];
  jac_4_3 += 5040*c1p1*c3p3*c9p5*ss[62];
  jac_4_3 += 6300*c1p1*c3p4*c5p4*ss[40];
  jac_4_3 += 6300*c1p1*c3p4*c7p4*ss[48];
  jac_4_3 += 6300*c1p1*c3p4*c9p4*ss[56];
  jac_4_3 += 5040*c1p1*c3p5*c5p3*ss[38];
  jac_4_3 += 5040*c1p1*c3p5*c7p3*ss[44];
  jac_4_3 += 5040*c1p1*c3p5*c9p3*ss[50];
  jac_4_3 += 2520*c1p1*c3p6*c5p2*ss[36];
  jac_4_3 += 2520*c1p1*c3p6*c7p2*ss[40];
  jac_4_3 += 2520*c1p1*c3p6*c9p2*ss[44];
  jac_4_3 += 720*c1p1*c3p7*c5p1*ss[34];
  jac_4_3 += 720*c1p1*c3p7*c7p1*ss[36];
  jac_4_3 += 720*c1p1*c3p7*c9p1*ss[38];
  jac_4_3 += 720*c1p1*c5p1*c7p7*ss[62];
  jac_4_3 += 720*c1p1*c5p1*c9p7*ss[76];
  jac_4_3 += 2520*c1p1*c5p2*c7p6*ss[60];
  jac_4_3 += 2520*c1p1*c5p2*c9p6*ss[72];
  jac_4_3 += 5040*c1p1*c5p3*c7p5*ss[58];
  jac_4_3 += 5040*c1p1*c5p3*c9p5*ss[68];
  jac_4_3 += 6300*c1p1*c5p4*c7p4*ss[56];
  jac_4_3 += 6300*c1p1*c5p4*c9p4*ss[64];
  jac_4_3 += 5040*c1p1*c5p5*c7p3*ss[54];
  jac_4_3 += 5040*c1p1*c5p5*c9p3*ss[60];
  jac_4_3 += 2520*c1p1*c5p6*c7p2*ss[52];
  jac_4_3 += 2520*c1p1*c5p6*c9p2*ss[56];
  jac_4_3 += 720*c1p1*c5p7*c7p1*ss[50];
  jac_4_3 += 720*c1p1*c5p7*c9p1*ss[52];
  jac_4_3 += 720*c1p1*c7p1*c9p7*ss[78];
  jac_4_3 += 2520*c1p1*c7p2*c9p6*ss[76];
  jac_4_3 += 5040*c1p1*c7p3*c9p5*ss[74];
  jac_4_3 += 6300*c1p1*c7p4*c9p4*ss[72];
  jac_4_3 += 5040*c1p1*c7p5*c9p3*ss[70];
  jac_4_3 += 2520*c1p1*c7p6*c9p2*ss[68];
  jac_4_3 += 720*c1p1*c7p7*c9p1*ss[66];
  jac_4_3 += 2520*c1p2*c3p1*c5p6*ss[42];
  jac_4_3 += 2520*c1p2*c3p1*c7p6*ss[54];
  jac_4_3 += 2520*c1p2*c3p1*c9p6*ss[66];
  jac_4_3 += 7560*c1p2*c3p2*c5p5*ss[40];
  jac_4_3 += 7560*c1p2*c3p2*c7p5*ss[50];
  jac_4_3 += 7560*c1p2*c3p2*c9p5*ss[60];
  jac_4_3 += 12600*c1p2*c3p3*c5p4*ss[38];
  jac_4_3 += 12600*c1p2*c3p3*c7p4*ss[46];
  jac_4_3 += 12600*c1p2*c3p3*c9p4*ss[54];
  jac_4_3 += 12600*c1p2*c3p4*c5p3*ss[36];
  jac_4_3 += 12600*c1p2*c3p4*c7p3*ss[42];
  jac_4_3 += 12600*c1p2*c3p4*c9p3*ss[48];
  jac_4_3 += 7560*c1p2*c3p5*c5p2*ss[34];
  jac_4_3 += 7560*c1p2*c3p5*c7p2*ss[38];
  jac_4_3 += 7560*c1p2*c3p5*c9p2*ss[42];
  jac_4_3 += 2520*c1p2*c3p6*c5p1*ss[32];
  jac_4_3 += 2520*c1p2*c3p6*c7p1*ss[34];
  jac_4_3 += 2520*c1p2*c3p6*c9p1*ss[36];
  jac_4_3 += 2520*c1p2*c5p1*c7p6*ss[56];
  jac_4_3 += 2520*c1p2*c5p1*c9p6*ss[68];
  jac_4_3 += 7560*c1p2*c5p2*c7p5*ss[54];
  jac_4_3 += 7560*c1p2*c5p2*c9p5*ss[64];
  jac_4_3 += 12600*c1p2*c5p3*c7p4*ss[52];
  jac_4_3 += 12600*c1p2*c5p3*c9p4*ss[60];
  jac_4_3 += 12600*c1p2*c5p4*c7p3*ss[50];
  jac_4_3 += 12600*c1p2*c5p4*c9p3*ss[56];
  jac_4_3 += 7560*c1p2*c5p5*c7p2*ss[48];
  jac_4_3 += 7560*c1p2*c5p5*c9p2*ss[52];
  jac_4_3 += 2520*c1p2*c5p6*c7p1*ss[46];
  jac_4_3 += 2520*c1p2*c5p6*c9p1*ss[48];
  jac_4_3 += 2520*c1p2*c7p1*c9p6*ss[70];
  jac_4_3 += 7560*c1p2*c7p2*c9p5*ss[68];
  jac_4_3 += 12600*c1p2*c7p3*c9p4*ss[66];
  jac_4_3 += 12600*c1p2*c7p4*c9p3*ss[64];
  jac_4_3 += 7560*c1p2*c7p5*c9p2*ss[62];
  jac_4_3 += 2520*c1p2*c7p6*c9p1*ss[60];
  jac_4_3 += 5040*c1p3*c3p1*c5p5*ss[38];
  jac_4_3 += 5040*c1p3*c3p1*c7p5*ss[48];
  jac_4_3 += 5040*c1p3*c3p1*c9p5*ss[58];
  jac_4_3 += 12600*c1p3*c3p2*c5p4*ss[36];
  jac_4_3 += 12600*c1p3*c3p2*c7p4*ss[44];
  jac_4_3 += 12600*c1p3*c3p2*c9p4*ss[52];
  jac_4_3 += 16800*c1p3*c3p3*c5p3*ss[34];
  jac_4_3 += 16800*c1p3*c3p3*c7p3*ss[40];
  jac_4_3 += 16800*c1p3*c3p3*c9p3*ss[46];
  jac_4_3 += 12600*c1p3*c3p4*c5p2*ss[32];
  jac_4_3 += 12600*c1p3*c3p4*c7p2*ss[36];
  jac_4_3 += 12600*c1p3*c3p4*c9p2*ss[40];
  jac_4_3 += 5040*c1p3*c3p5*c5p1*ss[30];
  jac_4_3 += 5040*c1p3*c3p5*c7p1*ss[32];
  jac_4_3 += 5040*c1p3*c3p5*c9p1*ss[34];
  jac_4_3 += 5040*c1p3*c5p1*c7p5*ss[50];
  jac_4_3 += 5040*c1p3*c5p1*c9p5*ss[60];
  jac_4_3 += 12600*c1p3*c5p2*c7p4*ss[48];
  jac_4_3 += 12600*c1p3*c5p2*c9p4*ss[56];
  jac_4_3 += 16800*c1p3*c5p3*c7p3*ss[46];
  jac_4_3 += 16800*c1p3*c5p3*c9p3*ss[52];
  jac_4_3 += 12600*c1p3*c5p4*c7p2*ss[44];
  jac_4_3 += 12600*c1p3*c5p4*c9p2*ss[48];
  jac_4_3 += 5040*c1p3*c5p5*c7p1*ss[42];
  jac_4_3 += 5040*c1p3*c5p5*c9p1*ss[44];
  jac_4_3 += 5040*c1p3*c7p1*c9p5*ss[62];
  jac_4_3 += 12600*c1p3*c7p2*c9p4*ss[60];
  jac_4_3 += 16800*c1p3*c7p3*c9p3*ss[58];
  jac_4_3 += 12600*c1p3*c7p4*c9p2*ss[56];
  jac_4_3 += 5040*c1p3*c7p5*c9p1*ss[54];
  jac_4_3 += 6300*c1p4*c3p1*c5p4*ss[34];
  jac_4_3 += 6300*c1p4*c3p1*c7p4*ss[42];
  jac_4_3 += 6300*c1p4*c3p1*c9p4*ss[50];
  jac_4_3 += 12600*c1p4*c3p2*c5p3*ss[32];
  jac_4_3 += 12600*c1p4*c3p2*c7p3*ss[38];
  jac_4_3 += 12600*c1p4*c3p2*c9p3*ss[44];
  jac_4_3 += 12600*c1p4*c3p3*c5p2*ss[30];
  jac_4_3 += 12600*c1p4*c3p3*c7p2*ss[34];
  jac_4_3 += 12600*c1p4*c3p3*c9p2*ss[38];
  jac_4_3 += 6300*c1p4*c3p4*c5p1*ss[28];
  jac_4_3 += 6300*c1p4*c3p4*c7p1*ss[30];
  jac_4_3 += 6300*c1p4*c3p4*c9p1*ss[32];
  jac_4_3 += 6300*c1p4*c5p1*c7p4*ss[44];
  jac_4_3 += 6300*c1p4*c5p1*c9p4*ss[52];
  jac_4_3 += 12600*c1p4*c5p2*c7p3*ss[42];
  jac_4_3 += 12600*c1p4*c5p2*c9p3*ss[48];
  jac_4_3 += 12600*c1p4*c5p3*c7p2*ss[40];
  jac_4_3 += 12600*c1p4*c5p3*c9p2*ss[44];
  jac_4_3 += 6300*c1p4*c5p4*c7p1*ss[38];
  jac_4_3 += 6300*c1p4*c5p4*c9p1*ss[40];
  jac_4_3 += 6300*c1p4*c7p1*c9p4*ss[54];
  jac_4_3 += 12600*c1p4*c7p2*c9p3*ss[52];
  jac_4_3 += 12600*c1p4*c7p3*c9p2*ss[50];
  jac_4_3 += 6300*c1p4*c7p4*c9p1*ss[48];
  jac_4_3 += 5040*c1p5*c3p1*c5p3*ss[30];
  jac_4_3 += 5040*c1p5*c3p1*c7p3*ss[36];
  jac_4_3 += 5040*c1p5*c3p1*c9p3*ss[42];
  jac_4_3 += 7560*c1p5*c3p2*c5p2*ss[28];
  jac_4_3 += 7560*c1p5*c3p2*c7p2*ss[32];
  jac_4_3 += 7560*c1p5*c3p2*c9p2*ss[36];
  jac_4_3 += 5040*c1p5*c3p3*c5p1*ss[26];
  jac_4_3 += 5040*c1p5*c3p3*c7p1*ss[28];
  jac_4_3 += 5040*c1p5*c3p3*c9p1*ss[30];
  jac_4_3 += 5040*c1p5*c5p1*c7p3*ss[38];
  jac_4_3 += 5040*c1p5*c5p1*c9p3*ss[44];
  jac_4_3 += 7560*c1p5*c5p2*c7p2*ss[36];
  jac_4_3 += 7560*c1p5*c5p2*c9p2*ss[40];
  jac_4_3 += 5040*c1p5*c5p3*c7p1*ss[34];
  jac_4_3 += 5040*c1p5*c5p3*c9p1*ss[36];
  jac_4_3 += 5040*c1p5*c7p1*c9p3*ss[46];
  jac_4_3 += 7560*c1p5*c7p2*c9p2*ss[44];
  jac_4_3 += 5040*c1p5*c7p3*c9p1*ss[42];
  jac_4_3 += 2520*c1p6*c3p1*c5p2*ss[26];
  jac_4_3 += 2520*c1p6*c3p1*c7p2*ss[30];
  jac_4_3 += 2520*c1p6*c3p1*c9p2*ss[34];
  jac_4_3 += 2520*c1p6*c3p2*c5p1*ss[24];
  jac_4_3 += 2520*c1p6*c3p2*c7p1*ss[26];
  jac_4_3 += 2520*c1p6*c3p2*c9p1*ss[28];
  jac_4_3 += 2520*c1p6*c5p1*c7p2*ss[32];
  jac_4_3 += 2520*c1p6*c5p1*c9p2*ss[36];
  jac_4_3 += 2520*c1p6*c5p2*c7p1*ss[30];
  jac_4_3 += 2520*c1p6*c5p2*c9p1*ss[32];
  jac_4_3 += 2520*c1p6*c7p1*c9p2*ss[38];
  jac_4_3 += 2520*c1p6*c7p2*c9p1*ss[36];
  jac_4_3 += 720*c1p7*c3p1*c5p1*ss[22];
  jac_4_3 += 720*c1p7*c3p1*c7p1*ss[24];
  jac_4_3 += 720*c1p7*c3p1*c9p1*ss[26];
  jac_4_3 += 720*c1p7*c5p1*c7p1*ss[26];
  jac_4_3 += 720*c1p7*c5p1*c9p1*ss[28];
  jac_4_3 += 720*c1p7*c7p1*c9p1*ss[30];
  jac_4_3 += 720*c3p1*c5p1*c7p7*ss[64];
  jac_4_3 += 720*c3p1*c5p1*c9p7*ss[78];
  jac_4_3 += 2520*c3p1*c5p2*c7p6*ss[62];
  jac_4_3 += 2520*c3p1*c5p2*c9p6*ss[74];
  jac_4_3 += 5040*c3p1*c5p3*c7p5*ss[60];
  jac_4_3 += 5040*c3p1*c5p3*c9p5*ss[70];
  jac_4_3 += 6300*c3p1*c5p4*c7p4*ss[58];
  jac_4_3 += 6300*c3p1*c5p4*c9p4*ss[66];
  jac_4_3 += 5040*c3p1*c5p5*c7p3*ss[56];
  jac_4_3 += 5040*c3p1*c5p5*c9p3*ss[62];
  jac_4_3 += 2520*c3p1*c5p6*c7p2*ss[54];
  jac_4_3 += 2520*c3p1*c5p6*c9p2*ss[58];
  jac_4_3 += 720*c3p1*c5p7*c7p1*ss[52];
  jac_4_3 += 720*c3p1*c5p7*c9p1*ss[54];
  jac_4_3 += 720*c3p1*c7p1*c9p7*ss[80];
  jac_4_3 += 2520*c3p1*c7p2*c9p6*ss[78];
  jac_4_3 += 5040*c3p1*c7p3*c9p5*ss[76];
  jac_4_3 += 6300*c3p1*c7p4*c9p4*ss[74];
  jac_4_3 += 5040*c3p1*c7p5*c9p3*ss[72];
  jac_4_3 += 2520*c3p1*c7p6*c9p2*ss[70];
  jac_4_3 += 720*c3p1*c7p7*c9p1*ss[68];
  jac_4_3 += 2520*c3p2*c5p1*c7p6*ss[60];
  jac_4_3 += 2520*c3p2*c5p1*c9p6*ss[72];
  jac_4_3 += 7560*c3p2*c5p2*c7p5*ss[58];
  jac_4_3 += 7560*c3p2*c5p2*c9p5*ss[68];
  jac_4_3 += 12600*c3p2*c5p3*c7p4*ss[56];
  jac_4_3 += 12600*c3p2*c5p3*c9p4*ss[64];
  jac_4_3 += 12600*c3p2*c5p4*c7p3*ss[54];
  jac_4_3 += 12600*c3p2*c5p4*c9p3*ss[60];
  jac_4_3 += 7560*c3p2*c5p5*c7p2*ss[52];
  jac_4_3 += 7560*c3p2*c5p5*c9p2*ss[56];
  jac_4_3 += 2520*c3p2*c5p6*c7p1*ss[50];
  jac_4_3 += 2520*c3p2*c5p6*c9p1*ss[52];
  jac_4_3 += 2520*c3p2*c7p1*c9p6*ss[74];
  jac_4_3 += 7560*c3p2*c7p2*c9p5*ss[72];
  jac_4_3 += 12600*c3p2*c7p3*c9p4*ss[70];
  jac_4_3 += 12600*c3p2*c7p4*c9p3*ss[68];
  jac_4_3 += 7560*c3p2*c7p5*c9p2*ss[66];
  jac_4_3 += 2520*c3p2*c7p6*c9p1*ss[64];
  jac_4_3 += 5040*c3p3*c5p1*c7p5*ss[56];
  jac_4_3 += 5040*c3p3*c5p1*c9p5*ss[66];
  jac_4_3 += 12600*c3p3*c5p2*c7p4*ss[54];
  jac_4_3 += 12600*c3p3*c5p2*c9p4*ss[62];
  jac_4_3 += 16800*c3p3*c5p3*c7p3*ss[52];
  jac_4_3 += 16800*c3p3*c5p3*c9p3*ss[58];
  jac_4_3 += 12600*c3p3*c5p4*c7p2*ss[50];
  jac_4_3 += 12600*c3p3*c5p4*c9p2*ss[54];
  jac_4_3 += 5040*c3p3*c5p5*c7p1*ss[48];
  jac_4_3 += 5040*c3p3*c5p5*c9p1*ss[50];
  jac_4_3 += 5040*c3p3*c7p1*c9p5*ss[68];
  jac_4_3 += 12600*c3p3*c7p2*c9p4*ss[66];
  jac_4_3 += 16800*c3p3*c7p3*c9p3*ss[64];
  jac_4_3 += 12600*c3p3*c7p4*c9p2*ss[62];
  jac_4_3 += 5040*c3p3*c7p5*c9p1*ss[60];
  jac_4_3 += 6300*c3p4*c5p1*c7p4*ss[52];
  jac_4_3 += 6300*c3p4*c5p1*c9p4*ss[60];
  jac_4_3 += 12600*c3p4*c5p2*c7p3*ss[50];
  jac_4_3 += 12600*c3p4*c5p2*c9p3*ss[56];
  jac_4_3 += 12600*c3p4*c5p3*c7p2*ss[48];
  jac_4_3 += 12600*c3p4*c5p3*c9p2*ss[52];
  jac_4_3 += 6300*c3p4*c5p4*c7p1*ss[46];
  jac_4_3 += 6300*c3p4*c5p4*c9p1*ss[48];
  jac_4_3 += 6300*c3p4*c7p1*c9p4*ss[62];
  jac_4_3 += 12600*c3p4*c7p2*c9p3*ss[60];
  jac_4_3 += 12600*c3p4*c7p3*c9p2*ss[58];
  jac_4_3 += 6300*c3p4*c7p4*c9p1*ss[56];
  jac_4_3 += 5040*c3p5*c5p1*c7p3*ss[48];
  jac_4_3 += 5040*c3p5*c5p1*c9p3*ss[54];
  jac_4_3 += 7560*c3p5*c5p2*c7p2*ss[46];
  jac_4_3 += 7560*c3p5*c5p2*c9p2*ss[50];
  jac_4_3 += 5040*c3p5*c5p3*c7p1*ss[44];
  jac_4_3 += 5040*c3p5*c5p3*c9p1*ss[46];
  jac_4_3 += 5040*c3p5*c7p1*c9p3*ss[56];
  jac_4_3 += 7560*c3p5*c7p2*c9p2*ss[54];
  jac_4_3 += 5040*c3p5*c7p3*c9p1*ss[52];
  jac_4_3 += 2520*c3p6*c5p1*c7p2*ss[44];
  jac_4_3 += 2520*c3p6*c5p1*c9p2*ss[48];
  jac_4_3 += 2520*c3p6*c5p2*c7p1*ss[42];
  jac_4_3 += 2520*c3p6*c5p2*c9p1*ss[44];
  jac_4_3 += 2520*c3p6*c7p1*c9p2*ss[50];
  jac_4_3 += 2520*c3p6*c7p2*c9p1*ss[48];
  jac_4_3 += 720*c3p7*c5p1*c7p1*ss[40];
  jac_4_3 += 720*c3p7*c5p1*c9p1*ss[42];
  jac_4_3 += 720*c3p7*c7p1*c9p1*ss[44];
  jac_4_3 += 720*c5p1*c7p1*c9p7*ss[82];
  jac_4_3 += 2520*c5p1*c7p2*c9p6*ss[80];
  jac_4_3 += 5040*c5p1*c7p3*c9p5*ss[78];
  jac_4_3 += 6300*c5p1*c7p4*c9p4*ss[76];
  jac_4_3 += 5040*c5p1*c7p5*c9p3*ss[74];
  jac_4_3 += 2520*c5p1*c7p6*c9p2*ss[72];
  jac_4_3 += 720*c5p1*c7p7*c9p1*ss[70];
  jac_4_3 += 2520*c5p2*c7p1*c9p6*ss[78];
  jac_4_3 += 7560*c5p2*c7p2*c9p5*ss[76];
  jac_4_3 += 12600*c5p2*c7p3*c9p4*ss[74];
  jac_4_3 += 12600*c5p2*c7p4*c9p3*ss[72];
  jac_4_3 += 7560*c5p2*c7p5*c9p2*ss[70];
  jac_4_3 += 2520*c5p2*c7p6*c9p1*ss[68];
  jac_4_3 += 5040*c5p3*c7p1*c9p5*ss[74];
  jac_4_3 += 12600*c5p3*c7p2*c9p4*ss[72];
  jac_4_3 += 16800*c5p3*c7p3*c9p3*ss[70];
  jac_4_3 += 12600*c5p3*c7p4*c9p2*ss[68];
  jac_4_3 += 5040*c5p3*c7p5*c9p1*ss[66];
  jac_4_3 += 6300*c5p4*c7p1*c9p4*ss[70];
  jac_4_3 += 12600*c5p4*c7p2*c9p3*ss[68];
  jac_4_3 += 12600*c5p4*c7p3*c9p2*ss[66];
  jac_4_3 += 6300*c5p4*c7p4*c9p1*ss[64];
  jac_4_3 += 5040*c5p5*c7p1*c9p3*ss[66];
  jac_4_3 += 7560*c5p5*c7p2*c9p2*ss[64];
  jac_4_3 += 5040*c5p5*c7p3*c9p1*ss[62];
  jac_4_3 += 2520*c5p6*c7p1*c9p2*ss[62];
  jac_4_3 += 2520*c5p6*c7p2*c9p1*ss[60];
  jac_4_3 += 720*c5p7*c7p1*c9p1*ss[58];
  jac_4_3 += 90*c1p1*c3p8*ss[32];
  jac_4_3 += 90*c1p1*c5p8*ss[48];
  jac_4_3 += 90*c1p1*c7p8*ss[64];
  jac_4_3 += 90*c1p1*c9p8*ss[80];
  jac_4_3 += 360*c1p2*c3p7*ss[30];
  jac_4_3 += 360*c1p2*c5p7*ss[44];
  jac_4_3 += 360*c1p2*c7p7*ss[58];
  jac_4_3 += 360*c1p2*c9p7*ss[72];
  jac_4_3 += 840*c1p3*c3p6*ss[28];
  jac_4_3 += 840*c1p3*c5p6*ss[40];
  jac_4_3 += 840*c1p3*c7p6*ss[52];
  jac_4_3 += 840*c1p3*c9p6*ss[64];
  jac_4_3 += 1260*c1p4*c3p5*ss[26];
  jac_4_3 += 1260*c1p4*c5p5*ss[36];
  jac_4_3 += 1260*c1p4*c7p5*ss[46];
  jac_4_3 += 1260*c1p4*c9p5*ss[56];
  jac_4_3 += 1260*c1p5*c3p4*ss[24];
  jac_4_3 += 1260*c1p5*c5p4*ss[32];
  jac_4_3 += 1260*c1p5*c7p4*ss[40];
  jac_4_3 += 1260*c1p5*c9p4*ss[48];
  jac_4_3 += 840*c1p6*c3p3*ss[22];
  jac_4_3 += 840*c1p6*c5p3*ss[28];
  jac_4_3 += 840*c1p6*c7p3*ss[34];
  jac_4_3 += 840*c1p6*c9p3*ss[40];
  jac_4_3 += 360*c1p7*c3p2*ss[20];
  jac_4_3 += 360*c1p7*c5p2*ss[24];
  jac_4_3 += 360*c1p7*c7p2*ss[28];
  jac_4_3 += 360*c1p7*c9p2*ss[32];
  jac_4_3 += 90*c1p8*c3p1*ss[18];
  jac_4_3 += 90*c1p8*c5p1*ss[20];
  jac_4_3 += 90*c1p8*c7p1*ss[22];
  jac_4_3 += 90*c1p8*c9p1*ss[24];
  jac_4_3 += 90*c3p1*c5p8*ss[50];
  jac_4_3 += 90*c3p1*c7p8*ss[66];
  jac_4_3 += 90*c3p1*c9p8*ss[82];
  jac_4_3 += 360*c3p2*c5p7*ss[48];
  jac_4_3 += 360*c3p2*c7p7*ss[62];
  jac_4_3 += 360*c3p2*c9p7*ss[76];
  jac_4_3 += 840*c3p3*c5p6*ss[46];
  jac_4_3 += 840*c3p3*c7p6*ss[58];
  jac_4_3 += 840*c3p3*c9p6*ss[70];
  jac_4_3 += 1260*c3p4*c5p5*ss[44];
  jac_4_3 += 1260*c3p4*c7p5*ss[54];
  jac_4_3 += 1260*c3p4*c9p5*ss[64];
  jac_4_3 += 1260*c3p5*c5p4*ss[42];
  jac_4_3 += 1260*c3p5*c7p4*ss[50];
  jac_4_3 += 1260*c3p5*c9p4*ss[58];
  jac_4_3 += 840*c3p6*c5p3*ss[40];
  jac_4_3 += 840*c3p6*c7p3*ss[46];
  jac_4_3 += 840*c3p6*c9p3*ss[52];
  jac_4_3 += 360*c3p7*c5p2*ss[38];
  jac_4_3 += 360*c3p7*c7p2*ss[42];
  jac_4_3 += 360*c3p7*c9p2*ss[46];
  jac_4_3 += 90*c3p8*c5p1*ss[36];
  jac_4_3 += 90*c3p8*c7p1*ss[38];
  jac_4_3 += 90*c3p8*c9p1*ss[40];
  jac_4_3 += 90*c5p1*c7p8*ss[68];
  jac_4_3 += 90*c5p1*c9p8*ss[84];
  jac_4_3 += 360*c5p2*c7p7*ss[66];
  jac_4_3 += 360*c5p2*c9p7*ss[80];
  jac_4_3 += 840*c5p3*c7p6*ss[64];
  jac_4_3 += 840*c5p3*c9p6*ss[76];
  jac_4_3 += 1260*c5p4*c7p5*ss[62];
  jac_4_3 += 1260*c5p4*c9p5*ss[72];
  jac_4_3 += 1260*c5p5*c7p4*ss[60];
  jac_4_3 += 1260*c5p5*c9p4*ss[68];
  jac_4_3 += 840*c5p6*c7p3*ss[58];
  jac_4_3 += 840*c5p6*c9p3*ss[64];
  jac_4_3 += 360*c5p7*c7p2*ss[56];
  jac_4_3 += 360*c5p7*c9p2*ss[60];
  jac_4_3 += 90*c5p8*c7p1*ss[54];
  jac_4_3 += 90*c5p8*c9p1*ss[56];
  jac_4_3 += 90*c7p1*c9p8*ss[86];
  jac_4_3 += 360*c7p2*c9p7*ss[84];
  jac_4_3 += 840*c7p3*c9p6*ss[82];
  jac_4_3 += 1260*c7p4*c9p5*ss[80];
  jac_4_3 += 1260*c7p5*c9p4*ss[78];
  jac_4_3 += 840*c7p6*c9p3*ss[76];
  jac_4_3 += 360*c7p7*c9p2*ss[74];
  jac_4_3 += 90*c7p8*c9p1*ss[72];
  jac_4_3 += 10*c1p9*ss[16];
  jac_4_3 += 10*c3p9*ss[34];
  jac_4_3 += 10*c5p9*ss[52];
  jac_4_3 += 10*c7p9*ss[70];
  jac_4_3 += 10*c9p9*ss[88];
  TACC jac_4_4 = convert<TACC>(30240*c1p1*c3p1*c5p1*c7p1*c9p5*ss[70]);
  jac_4_4 += 75600*c1p1*c3p1*c5p1*c7p2*c9p4*ss[68];
  jac_4_4 += 100800*c1p1*c3p1*c5p1*c7p3*c9p3*ss[66];
  jac_4_4 += 75600*c1p1*c3p1*c5p1*c7p4*c9p2*ss[64];
  jac_4_4 += 30240*c1p1*c3p1*c5p1*c7p5*c9p1*ss[62];
  jac_4_4 += 75600*c1p1*c3p1*c5p2*c7p1*c9p4*ss[66];
  jac_4_4 += 151200*c1p1*c3p1*c5p2*c7p2*c9p3*ss[64];
  jac_4_4 += 151200*c1p1*c3p1*c5p2*c7p3*c9p2*ss[62];
  jac_4_4 += 75600*c1p1*c3p1*c5p2*c7p4*c9p1*ss[60];
  jac_4_4 += 100800*c1p1*c3p1*c5p3*c7p1*c9p3*ss[62];
  jac_4_4 += 151200*c1p1*c3p1*c5p3*c7p2*c9p2*ss[60];
  jac_4_4 += 100800*c1p1*c3p1*c5p3*c7p3*c9p1*ss[58];
  jac_4_4 += 75600*c1p1*c3p1*c5p4*c7p1*c9p2*ss[58];
  jac_4_4 += 75600*c1p1*c3p1*c5p4*c7p2*c9p1*ss[56];
  jac_4_4 += 30240*c1p1*c3p1*c5p5*c7p1*c9p1*ss[54];
  jac_4_4 += 75600*c1p1*c3p2*c5p1*c7p1*c9p4*ss[64];
  jac_4_4 += 151200*c1p1*c3p2*c5p1*c7p2*c9p3*ss[62];
  jac_4_4 += 151200*c1p1*c3p2*c5p1*c7p3*c9p2*ss[60];
  jac_4_4 += 75600*c1p1*c3p2*c5p1*c7p4*c9p1*ss[58];
  jac_4_4 += 151200*c1p1*c3p2*c5p2*c7p1*c9p3*ss[60];
  jac_4_4 += 226800*c1p1*c3p2*c5p2*c7p2*c9p2*ss[58];
  jac_4_4 += 151200*c1p1*c3p2*c5p2*c7p3*c9p1*ss[56];
  jac_4_4 += 151200*c1p1*c3p2*c5p3*c7p1*c9p2*ss[56];
  jac_4_4 += 151200*c1p1*c3p2*c5p3*c7p2*c9p1*ss[54];
  jac_4_4 += 75600*c1p1*c3p2*c5p4*c7p1*c9p1*ss[52];
  jac_4_4 += 100800*c1p1*c3p3*c5p1*c7p1*c9p3*ss[58];
  jac_4_4 += 151200*c1p1*c3p3*c5p1*c7p2*c9p2*ss[56];
  jac_4_4 += 100800*c1p1*c3p3*c5p1*c7p3*c9p1*ss[54];
  jac_4_4 += 151200*c1p1*c3p3*c5p2*c7p1*c9p2*ss[54];
  jac_4_4 += 151200*c1p1*c3p3*c5p2*c7p2*c9p1*ss[52];
  jac_4_4 += 100800*c1p1*c3p3*c5p3*c7p1*c9p1*ss[50];
  jac_4_4 += 75600*c1p1*c3p4*c5p1*c7p1*c9p2*ss[52];
  jac_4_4 += 75600*c1p1*c3p4*c5p1*c7p2*c9p1*ss[50];
  jac_4_4 += 75600*c1p1*c3p4*c5p2*c7p1*c9p1*ss[48];
  jac_4_4 += 30240*c1p1*c3p5*c5p1*c7p1*c9p1*ss[46];
  jac_4_4 += 75600*c1p2*c3p1*c5p1*c7p1*c9p4*ss[62];
  jac_4_4 += 151200*c1p2*c3p1*c5p1*c7p2*c9p3*ss[60];
  jac_4_4 += 151200*c1p2*c3p1*c5p1*c7p3*c9p2*ss[58];
  jac_4_4 += 75600*c1p2*c3p1*c5p1*c7p4*c9p1*ss[56];
  jac_4_4 += 151200*c1p2*c3p1*c5p2*c7p1*c9p3*ss[58];
  jac_4_4 += 226800*c1p2*c3p1*c5p2*c7p2*c9p2*ss[56];
  jac_4_4 += 151200*c1p2*c3p1*c5p2*c7p3*c9p1*ss[54];
  jac_4_4 += 151200*c1p2*c3p1*c5p3*c7p1*c9p2*ss[54];
  jac_4_4 += 151200*c1p2*c3p1*c5p3*c7p2*c9p1*ss[52];
  jac_4_4 += 75600*c1p2*c3p1*c5p4*c7p1*c9p1*ss[50];
  jac_4_4 += 151200*c1p2*c3p2*c5p1*c7p1*c9p3*ss[56];
  jac_4_4 += 226800*c1p2*c3p2*c5p1*c7p2*c9p2*ss[54];
  jac_4_4 += 151200*c1p2*c3p2*c5p1*c7p3*c9p1*ss[52];
  jac_4_4 += 226800*c1p2*c3p2*c5p2*c7p1*c9p2*ss[52];
  jac_4_4 += 226800*c1p2*c3p2*c5p2*c7p2*c9p1*ss[50];
  jac_4_4 += 151200*c1p2*c3p2*c5p3*c7p1*c9p1*ss[48];
  jac_4_4 += 151200*c1p2*c3p3*c5p1*c7p1*c9p2*ss[50];
  jac_4_4 += 151200*c1p2*c3p3*c5p1*c7p2*c9p1*ss[48];
  jac_4_4 += 151200*c1p2*c3p3*c5p2*c7p1*c9p1*ss[46];
  jac_4_4 += 75600*c1p2*c3p4*c5p1*c7p1*c9p1*ss[44];
  jac_4_4 += 100800*c1p3*c3p1*c5p1*c7p1*c9p3*ss[54];
  jac_4_4 += 151200*c1p3*c3p1*c5p1*c7p2*c9p2*ss[52];
  jac_4_4 += 100800*c1p3*c3p1*c5p1*c7p3*c9p1*ss[50];
  jac_4_4 += 151200*c1p3*c3p1*c5p2*c7p1*c9p2*ss[50];
  jac_4_4 += 151200*c1p3*c3p1*c5p2*c7p2*c9p1*ss[48];
  jac_4_4 += 100800*c1p3*c3p1*c5p3*c7p1*c9p1*ss[46];
  jac_4_4 += 151200*c1p3*c3p2*c5p1*c7p1*c9p2*ss[48];
  jac_4_4 += 151200*c1p3*c3p2*c5p1*c7p2*c9p1*ss[46];
  jac_4_4 += 151200*c1p3*c3p2*c5p2*c7p1*c9p1*ss[44];
  jac_4_4 += 100800*c1p3*c3p3*c5p1*c7p1*c9p1*ss[42];
  jac_4_4 += 75600*c1p4*c3p1*c5p1*c7p1*c9p2*ss[46];
  jac_4_4 += 75600*c1p4*c3p1*c5p1*c7p2*c9p1*ss[44];
  jac_4_4 += 75600*c1p4*c3p1*c5p2*c7p1*c9p1*ss[42];
  jac_4_4 += 75600*c1p4*c3p2*c5p1*c7p1*c9p1*ss[40];
  jac_4_4 += 30240*c1p5*c3p1*c5p1*c7p1*c9p1*ss[38];
  jac_4_4 += 5040*c1p1*c3p1*c5p1*c7p6*ss[60];
  jac_4_4 += 5040*c1p1*c3p1*c5p1*c9p6*ss[72];
  jac_4_4 += 15120*c1p1*c3p1*c5p2*c7p5*ss[58];
  jac_4_4 += 15120*c1p1*c3p1*c5p2*c9p5*ss[68];
  jac_4_4 += 25200*c1p1*c3p1*c5p3*c7p4*ss[56];
  jac_4_4 += 25200*c1p1*c3p1*c5p3*c9p4*ss[64];
  jac_4_4 += 25200*c1p1*c3p1*c5p4*c7p3*ss[54];
  jac_4_4 += 25200*c1p1*c3p1*c5p4*c9p3*ss[60];
  jac_4_4 += 15120*c1p1*c3p1*c5p5*c7p2*ss[52];
  jac_4_4 += 15120*c1p1*c3p1*c5p5*c9p2*ss[56];
  jac_4_4 += 5040*c1p1*c3p1*c5p6*c7p1*ss[50];
  jac_4_4 += 5040*c1p1*c3p1*c5p6*c9p1*ss[52];
  jac_4_4 += 5040*c1p1*c3p1*c7p1*c9p6*ss[74];
  jac_4_4 += 15120*c1p1*c3p1*c7p2*c9p5*ss[72];
  jac_4_4 += 25200*c1p1*c3p1*c7p3*c9p4*ss[70];
  jac_4_4 += 25200*c1p1*c3p1*c7p4*c9p3*ss[68];
  jac_4_4 += 15120*c1p1*c3p1*c7p5*c9p2*ss[66];
  jac_4_4 += 5040*c1p1*c3p1*c7p6*c9p1*ss[64];
  jac_4_4 += 15120*c1p1*c3p2*c5p1*c7p5*ss[56];
  jac_4_4 += 15120*c1p1*c3p2*c5p1*c9p5*ss[66];
  jac_4_4 += 37800*c1p1*c3p2*c5p2*c7p4*ss[54];
  jac_4_4 += 37800*c1p1*c3p2*c5p2*c9p4*ss[62];
  jac_4_4 += 50400*c1p1*c3p2*c5p3*c7p3*ss[52];
  jac_4_4 += 50400*c1p1*c3p2*c5p3*c9p3*ss[58];
  jac_4_4 += 37800*c1p1*c3p2*c5p4*c7p2*ss[50];
  jac_4_4 += 37800*c1p1*c3p2*c5p4*c9p2*ss[54];
  jac_4_4 += 15120*c1p1*c3p2*c5p5*c7p1*ss[48];
  jac_4_4 += 15120*c1p1*c3p2*c5p5*c9p1*ss[50];
  jac_4_4 += 15120*c1p1*c3p2*c7p1*c9p5*ss[68];
  jac_4_4 += 37800*c1p1*c3p2*c7p2*c9p4*ss[66];
  jac_4_4 += 50400*c1p1*c3p2*c7p3*c9p3*ss[64];
  jac_4_4 += 37800*c1p1*c3p2*c7p4*c9p2*ss[62];
  jac_4_4 += 15120*c1p1*c3p2*c7p5*c9p1*ss[60];
  jac_4_4 += 25200*c1p1*c3p3*c5p1*c7p4*ss[52];
  jac_4_4 += 25200*c1p1*c3p3*c5p1*c9p4*ss[60];
  jac_4_4 += 50400*c1p1*c3p3*c5p2*c7p3*ss[50];
  jac_4_4 += 50400*c1p1*c3p3*c5p2*c9p3*ss[56];
  jac_4_4 += 50400*c1p1*c3p3*c5p3*c7p2*ss[48];
  jac_4_4 += 50400*c1p1*c3p3*c5p3*c9p2*ss[52];
  jac_4_4 += 25200*c1p1*c3p3*c5p4*c7p1*ss[46];
  jac_4_4 += 25200*c1p1*c3p3*c5p4*c9p1*ss[48];
  jac_4_4 += 25200*c1p1*c3p3*c7p1*c9p4*ss[62];
  jac_4_4 += 50400*c1p1*c3p3*c7p2*c9p3*ss[60];
  jac_4_4 += 50400*c1p1*c3p3*c7p3*c9p2*ss[58];
  jac_4_4 += 25200*c1p1*c3p3*c7p4*c9p1*ss[56];
  jac_4_4 += 25200*c1p1*c3p4*c5p1*c7p3*ss[48];
  jac_4_4 += 25200*c1p1*c3p4*c5p1*c9p3*ss[54];
  jac_4_4 += 37800*c1p1*c3p4*c5p2*c7p2*ss[46];
  jac_4_4 += 37800*c1p1*c3p4*c5p2*c9p2*ss[50];
  jac_4_4 += 25200*c1p1*c3p4*c5p3*c7p1*ss[44];
  jac_4_4 += 25200*c1p1*c3p4*c5p3*c9p1*ss[46];
  jac_4_4 += 25200*c1p1*c3p4*c7p1*c9p3*ss[56];
  jac_4_4 += 37800*c1p1*c3p4*c7p2*c9p2*ss[54];
  jac_4_4 += 25200*c1p1*c3p4*c7p3*c9p1*ss[52];
  jac_4_4 += 15120*c1p1*c3p5*c5p1*c7p2*ss[44];
  jac_4_4 += 15120*c1p1*c3p5*c5p1*c9p2*ss[48];
  jac_4_4 += 15120*c1p1*c3p5*c5p2*c7p1*ss[42];
  jac_4_4 += 15120*c1p1*c3p5*c5p2*c9p1*ss[44];
  jac_4_4 += 15120*c1p1*c3p5*c7p1*c9p2*ss[50];
  jac_4_4 += 15120*c1p1*c3p5*c7p2*c9p1*ss[48];
  jac_4_4 += 5040*c1p1*c3p6*c5p1*c7p1*ss[40];
  jac_4_4 += 5040*c1p1*c3p6*c5p1*c9p1*ss[42];
  jac_4_4 += 5040*c1p1*c3p6*c7p1*c9p1*ss[44];
  jac_4_4 += 5040*c1p1*c5p1*c7p1*c9p6*ss[76];
  jac_4_4 += 15120*c1p1*c5p1*c7p2*c9p5*ss[74];
  jac_4_4 += 25200*c1p1*c5p1*c7p3*c9p4*ss[72];
  jac_4_4 += 25200*c1p1*c5p1*c7p4*c9p3*ss[70];
  jac_4_4 += 15120*c1p1*c5p1*c7p5*c9p2*ss[68];
  jac_4_4 += 5040*c1p1*c5p1*c7p6*c9p1*ss[66];
  jac_4_4 += 15120*c1p1*c5p2*c7p1*c9p5*ss[72];
  jac_4_4 += 37800*c1p1*c5p2*c7p2*c9p4*ss[70];
  jac_4_4 += 50400*c1p1*c5p2*c7p3*c9p3*ss[68];
  jac_4_4 += 37800*c1p1*c5p2*c7p4*c9p2*ss[66];
  jac_4_4 += 15120*c1p1*c5p2*c7p5*c9p1*ss[64];
  jac_4_4 += 25200*c1p1*c5p3*c7p1*c9p4*ss[68];
  jac_4_4 += 50400*c1p1*c5p3*c7p2*c9p3*ss[66];
  jac_4_4 += 50400*c1p1*c5p3*c7p3*c9p2*ss[64];
  jac_4_4 += 25200*c1p1*c5p3*c7p4*c9p1*ss[62];
  jac_4_4 += 25200*c1p1*c5p4*c7p1*c9p3*ss[64];
  jac_4_4 += 37800*c1p1*c5p4*c7p2*c9p2*ss[62];
  jac_4_4 += 25200*c1p1*c5p4*c7p3*c9p1*ss[60];
  jac_4_4 += 15120*c1p1*c5p5*c7p1*c9p2*ss[60];
  jac_4_4 += 15120*c1p1*c5p5*c7p2*c9p1*ss[58];
  jac_4_4 += 5040*c1p1*c5p6*c7p1*c9p1*ss[56];
  jac_4_4 += 15120*c1p2*c3p1*c5p1*c7p5*ss[54];
  jac_4_4 += 15120*c1p2*c3p1*c5p1*c9p5*ss[64];
  jac_4_4 += 37800*c1p2*c3p1*c5p2*c7p4*ss[52];
  jac_4_4 += 37800*c1p2*c3p1*c5p2*c9p4*ss[60];
  jac_4_4 += 50400*c1p2*c3p1*c5p3*c7p3*ss[50];
  jac_4_4 += 50400*c1p2*c3p1*c5p3*c9p3*ss[56];
  jac_4_4 += 37800*c1p2*c3p1*c5p4*c7p2*ss[48];
  jac_4_4 += 37800*c1p2*c3p1*c5p4*c9p2*ss[52];
  jac_4_4 += 15120*c1p2*c3p1*c5p5*c7p1*ss[46];
  jac_4_4 += 15120*c1p2*c3p1*c5p5*c9p1*ss[48];
  jac_4_4 += 15120*c1p2*c3p1*c7p1*c9p5*ss[66];
  jac_4_4 += 37800*c1p2*c3p1*c7p2*c9p4*ss[64];
  jac_4_4 += 50400*c1p2*c3p1*c7p3*c9p3*ss[62];
  jac_4_4 += 37800*c1p2*c3p1*c7p4*c9p2*ss[60];
  jac_4_4 += 15120*c1p2*c3p1*c7p5*c9p1*ss[58];
  jac_4_4 += 37800*c1p2*c3p2*c5p1*c7p4*ss[50];
  jac_4_4 += 37800*c1p2*c3p2*c5p1*c9p4*ss[58];
  jac_4_4 += 75600*c1p2*c3p2*c5p2*c7p3*ss[48];
  jac_4_4 += 75600*c1p2*c3p2*c5p2*c9p3*ss[54];
  jac_4_4 += 75600*c1p2*c3p2*c5p3*c7p2*ss[46];
  jac_4_4 += 75600*c1p2*c3p2*c5p3*c9p2*ss[50];
  jac_4_4 += 37800*c1p2*c3p2*c5p4*c7p1*ss[44];
  jac_4_4 += 37800*c1p2*c3p2*c5p4*c9p1*ss[46];
  jac_4_4 += 37800*c1p2*c3p2*c7p1*c9p4*ss[60];
  jac_4_4 += 75600*c1p2*c3p2*c7p2*c9p3*ss[58];
  jac_4_4 += 75600*c1p2*c3p2*c7p3*c9p2*ss[56];
  jac_4_4 += 37800*c1p2*c3p2*c7p4*c9p1*ss[54];
  jac_4_4 += 50400*c1p2*c3p3*c5p1*c7p3*ss[46];
  jac_4_4 += 50400*c1p2*c3p3*c5p1*c9p3*ss[52];
  jac_4_4 += 75600*c1p2*c3p3*c5p2*c7p2*ss[44];
  jac_4_4 += 75600*c1p2*c3p3*c5p2*c9p2*ss[48];
  jac_4_4 += 50400*c1p2*c3p3*c5p3*c7p1*ss[42];
  jac_4_4 += 50400*c1p2*c3p3*c5p3*c9p1*ss[44];
  jac_4_4 += 50400*c1p2*c3p3*c7p1*c9p3*ss[54];
  jac_4_4 += 75600*c1p2*c3p3*c7p2*c9p2*ss[52];
  jac_4_4 += 50400*c1p2*c3p3*c7p3*c9p1*ss[50];
  jac_4_4 += 37800*c1p2*c3p4*c5p1*c7p2*ss[42];
  jac_4_4 += 37800*c1p2*c3p4*c5p1*c9p2*ss[46];
  jac_4_4 += 37800*c1p2*c3p4*c5p2*c7p1*ss[40];
  jac_4_4 += 37800*c1p2*c3p4*c5p2*c9p1*ss[42];
  jac_4_4 += 37800*c1p2*c3p4*c7p1*c9p2*ss[48];
  jac_4_4 += 37800*c1p2*c3p4*c7p2*c9p1*ss[46];
  jac_4_4 += 15120*c1p2*c3p5*c5p1*c7p1*ss[38];
  jac_4_4 += 15120*c1p2*c3p5*c5p1*c9p1*ss[40];
  jac_4_4 += 15120*c1p2*c3p5*c7p1*c9p1*ss[42];
  jac_4_4 += 15120*c1p2*c5p1*c7p1*c9p5*ss[68];
  jac_4_4 += 37800*c1p2*c5p1*c7p2*c9p4*ss[66];
  jac_4_4 += 50400*c1p2*c5p1*c7p3*c9p3*ss[64];
  jac_4_4 += 37800*c1p2*c5p1*c7p4*c9p2*ss[62];
  jac_4_4 += 15120*c1p2*c5p1*c7p5*c9p1*ss[60];
  jac_4_4 += 37800*c1p2*c5p2*c7p1*c9p4*ss[64];
  jac_4_4 += 75600*c1p2*c5p2*c7p2*c9p3*ss[62];
  jac_4_4 += 75600*c1p2*c5p2*c7p3*c9p2*ss[60];
  jac_4_4 += 37800*c1p2*c5p2*c7p4*c9p1*ss[58];
  jac_4_4 += 50400*c1p2*c5p3*c7p1*c9p3*ss[60];
  jac_4_4 += 75600*c1p2*c5p3*c7p2*c9p2*ss[58];
  jac_4_4 += 50400*c1p2*c5p3*c7p3*c9p1*ss[56];
  jac_4_4 += 37800*c1p2*c5p4*c7p1*c9p2*ss[56];
  jac_4_4 += 37800*c1p2*c5p4*c7p2*c9p1*ss[54];
  jac_4_4 += 15120*c1p2*c5p5*c7p1*c9p1*ss[52];
  jac_4_4 += 25200*c1p3*c3p1*c5p1*c7p4*ss[48];
  jac_4_4 += 25200*c1p3*c3p1*c5p1*c9p4*ss[56];
  jac_4_4 += 50400*c1p3*c3p1*c5p2*c7p3*ss[46];
  jac_4_4 += 50400*c1p3*c3p1*c5p2*c9p3*ss[52];
  jac_4_4 += 50400*c1p3*c3p1*c5p3*c7p2*ss[44];
  jac_4_4 += 50400*c1p3*c3p1*c5p3*c9p2*ss[48];
  jac_4_4 += 25200*c1p3*c3p1*c5p4*c7p1*ss[42];
  jac_4_4 += 25200*c1p3*c3p1*c5p4*c9p1*ss[44];
  jac_4_4 += 25200*c1p3*c3p1*c7p1*c9p4*ss[58];
  jac_4_4 += 50400*c1p3*c3p1*c7p2*c9p3*ss[56];
  jac_4_4 += 50400*c1p3*c3p1*c7p3*c9p2*ss[54];
  jac_4_4 += 25200*c1p3*c3p1*c7p4*c9p1*ss[52];
  jac_4_4 += 50400*c1p3*c3p2*c5p1*c7p3*ss[44];
  jac_4_4 += 50400*c1p3*c3p2*c5p1*c9p3*ss[50];
  jac_4_4 += 75600*c1p3*c3p2*c5p2*c7p2*ss[42];
  jac_4_4 += 75600*c1p3*c3p2*c5p2*c9p2*ss[46];
  jac_4_4 += 50400*c1p3*c3p2*c5p3*c7p1*ss[40];
  jac_4_4 += 50400*c1p3*c3p2*c5p3*c9p1*ss[42];
  jac_4_4 += 50400*c1p3*c3p2*c7p1*c9p3*ss[52];
  jac_4_4 += 75600*c1p3*c3p2*c7p2*c9p2*ss[50];
  jac_4_4 += 50400*c1p3*c3p2*c7p3*c9p1*ss[48];
  jac_4_4 += 50400*c1p3*c3p3*c5p1*c7p2*ss[40];
  jac_4_4 += 50400*c1p3*c3p3*c5p1*c9p2*ss[44];
  jac_4_4 += 50400*c1p3*c3p3*c5p2*c7p1*ss[38];
  jac_4_4 += 50400*c1p3*c3p3*c5p2*c9p1*ss[40];
  jac_4_4 += 50400*c1p3*c3p3*c7p1*c9p2*ss[46];
  jac_4_4 += 50400*c1p3*c3p3*c7p2*c9p1*ss[44];
  jac_4_4 += 25200*c1p3*c3p4*c5p1*c7p1*ss[36];
  jac_4_4 += 25200*c1p3*c3p4*c5p1*c9p1*ss[38];
  jac_4_4 += 25200*c1p3*c3p4*c7p1*c9p1*ss[40];
  jac_4_4 += 25200*c1p3*c5p1*c7p1*c9p4*ss[60];
  jac_4_4 += 50400*c1p3*c5p1*c7p2*c9p3*ss[58];
  jac_4_4 += 50400*c1p3*c5p1*c7p3*c9p2*ss[56];
  jac_4_4 += 25200*c1p3*c5p1*c7p4*c9p1*ss[54];
  jac_4_4 += 50400*c1p3*c5p2*c7p1*c9p3*ss[56];
  jac_4_4 += 75600*c1p3*c5p2*c7p2*c9p2*ss[54];
  jac_4_4 += 50400*c1p3*c5p2*c7p3*c9p1*ss[52];
  jac_4_4 += 50400*c1p3*c5p3*c7p1*c9p2*ss[52];
  jac_4_4 += 50400*c1p3*c5p3*c7p2*c9p1*ss[50];
  jac_4_4 += 25200*c1p3*c5p4*c7p1*c9p1*ss[48];
  jac_4_4 += 25200*c1p4*c3p1*c5p1*c7p3*ss[42];
  jac_4_4 += 25200*c1p4*c3p1*c5p1*c9p3*ss[48];
  jac_4_4 += 37800*c1p4*c3p1*c5p2*c7p2*ss[40];
  jac_4_4 += 37800*c1p4*c3p1*c5p2*c9p2*ss[44];
  jac_4_4 += 25200*c1p4*c3p1*c5p3*c7p1*ss[38];
  jac_4_4 += 25200*c1p4*c3p1*c5p3*c9p1*ss[40];
  jac_4_4 += 25200*c1p4*c3p1*c7p1*c9p3*ss[50];
  jac_4_4 += 37800*c1p4*c3p1*c7p2*c9p2*ss[48];
  jac_4_4 += 25200*c1p4*c3p1*c7p3*c9p1*ss[46];
  jac_4_4 += 37800*c1p4*c3p2*c5p1*c7p2*ss[38];
  jac_4_4 += 37800*c1p4*c3p2*c5p1*c9p2*ss[42];
  jac_4_4 += 37800*c1p4*c3p2*c5p2*c7p1*ss[36];
  jac_4_4 += 37800*c1p4*c3p2*c5p2*c9p1*ss[38];
  jac_4_4 += 37800*c1p4*c3p2*c7p1*c9p2*ss[44];
  jac_4_4 += 37800*c1p4*c3p2*c7p2*c9p1*ss[42];
  jac_4_4 += 25200*c1p4*c3p3*c5p1*c7p1*ss[34];
  jac_4_4 += 25200*c1p4*c3p3*c5p1*c9p1*ss[36];
  jac_4_4 += 25200*c1p4*c3p3*c7p1*c9p1*ss[38];
  jac_4_4 += 25200*c1p4*c5p1*c7p1*c9p3*ss[52];
  jac_4_4 += 37800*c1p4*c5p1*c7p2*c9p2*ss[50];
  jac_4_4 += 25200*c1p4*c5p1*c7p3*c9p1*ss[48];
  jac_4_4 += 37800*c1p4*c5p2*c7p1*c9p2*ss[48];
  jac_4_4 += 37800*c1p4*c5p2*c7p2*c9p1*ss[46];
  jac_4_4 += 25200*c1p4*c5p3*c7p1*c9p1*ss[44];
  jac_4_4 += 15120*c1p5*c3p1*c5p1*c7p2*ss[36];
  jac_4_4 += 15120*c1p5*c3p1*c5p1*c9p2*ss[40];
  jac_4_4 += 15120*c1p5*c3p1*c5p2*c7p1*ss[34];
  jac_4_4 += 15120*c1p5*c3p1*c5p2*c9p1*ss[36];
  jac_4_4 += 15120*c1p5*c3p1*c7p1*c9p2*ss[42];
  jac_4_4 += 15120*c1p5*c3p1*c7p2*c9p1*ss[40];
  jac_4_4 += 15120*c1p5*c3p2*c5p1*c7p1*ss[32];
  jac_4_4 += 15120*c1p5*c3p2*c5p1*c9p1*ss[34];
  jac_4_4 += 15120*c1p5*c3p2*c7p1*c9p1*ss[36];
  jac_4_4 += 15120*c1p5*c5p1*c7p1*c9p2*ss[44];
  jac_4_4 += 15120*c1p5*c5p1*c7p2*c9p1*ss[42];
  jac_4_4 += 15120*c1p5*c5p2*c7p1*c9p1*ss[40];
  jac_4_4 += 5040*c1p6*c3p1*c5p1*c7p1*ss[30];
  jac_4_4 += 5040*c1p6*c3p1*c5p1*c9p1*ss[32];
  jac_4_4 += 5040*c1p6*c3p1*c7p1*c9p1*ss[34];
  jac_4_4 += 5040*c1p6*c5p1*c7p1*c9p1*ss[36];
  jac_4_4 += 5040*c3p1*c5p1*c7p1*c9p6*ss[78];
  jac_4_4 += 15120*c3p1*c5p1*c7p2*c9p5*ss[76];
  jac_4_4 += 25200*c3p1*c5p1*c7p3*c9p4*ss[74];
  jac_4_4 += 25200*c3p1*c5p1*c7p4*c9p3*ss[72];
  jac_4_4 += 15120*c3p1*c5p1*c7p5*c9p2*ss[70];
  jac_4_4 += 5040*c3p1*c5p1*c7p6*c9p1*ss[68];
  jac_4_4 += 15120*c3p1*c5p2*c7p1*c9p5*ss[74];
  jac_4_4 += 37800*c3p1*c5p2*c7p2*c9p4*ss[72];
  jac_4_4 += 50400*c3p1*c5p2*c7p3*c9p3*ss[70];
  jac_4_4 += 37800*c3p1*c5p2*c7p4*c9p2*ss[68];
  jac_4_4 += 15120*c3p1*c5p2*c7p5*c9p1*ss[66];
  jac_4_4 += 25200*c3p1*c5p3*c7p1*c9p4*ss[70];
  jac_4_4 += 50400*c3p1*c5p3*c7p2*c9p3*ss[68];
  jac_4_4 += 50400*c3p1*c5p3*c7p3*c9p2*ss[66];
  jac_4_4 += 25200*c3p1*c5p3*c7p4*c9p1*ss[64];
  jac_4_4 += 25200*c3p1*c5p4*c7p1*c9p3*ss[66];
  jac_4_4 += 37800*c3p1*c5p4*c7p2*c9p2*ss[64];
  jac_4_4 += 25200*c3p1*c5p4*c7p3*c9p1*ss[62];
  jac_4_4 += 15120*c3p1*c5p5*c7p1*c9p2*ss[62];
  jac_4_4 += 15120*c3p1*c5p5*c7p2*c9p1*ss[60];
  jac_4_4 += 5040*c3p1*c5p6*c7p1*c9p1*ss[58];
  jac_4_4 += 15120*c3p2*c5p1*c7p1*c9p5*ss[72];
  jac_4_4 += 37800*c3p2*c5p1*c7p2*c9p4*ss[70];
  jac_4_4 += 50400*c3p2*c5p1*c7p3*c9p3*ss[68];
  jac_4_4 += 37800*c3p2*c5p1*c7p4*c9p2*ss[66];
  jac_4_4 += 15120*c3p2*c5p1*c7p5*c9p1*ss[64];
  jac_4_4 += 37800*c3p2*c5p2*c7p1*c9p4*ss[68];
  jac_4_4 += 75600*c3p2*c5p2*c7p2*c9p3*ss[66];
  jac_4_4 += 75600*c3p2*c5p2*c7p3*c9p2*ss[64];
  jac_4_4 += 37800*c3p2*c5p2*c7p4*c9p1*ss[62];
  jac_4_4 += 50400*c3p2*c5p3*c7p1*c9p3*ss[64];
  jac_4_4 += 75600*c3p2*c5p3*c7p2*c9p2*ss[62];
  jac_4_4 += 50400*c3p2*c5p3*c7p3*c9p1*ss[60];
  jac_4_4 += 37800*c3p2*c5p4*c7p1*c9p2*ss[60];
  jac_4_4 += 37800*c3p2*c5p4*c7p2*c9p1*ss[58];
  jac_4_4 += 15120*c3p2*c5p5*c7p1*c9p1*ss[56];
  jac_4_4 += 25200*c3p3*c5p1*c7p1*c9p4*ss[66];
  jac_4_4 += 50400*c3p3*c5p1*c7p2*c9p3*ss[64];
  jac_4_4 += 50400*c3p3*c5p1*c7p3*c9p2*ss[62];
  jac_4_4 += 25200*c3p3*c5p1*c7p4*c9p1*ss[60];
  jac_4_4 += 50400*c3p3*c5p2*c7p1*c9p3*ss[62];
  jac_4_4 += 75600*c3p3*c5p2*c7p2*c9p2*ss[60];
  jac_4_4 += 50400*c3p3*c5p2*c7p3*c9p1*ss[58];
  jac_4_4 += 50400*c3p3*c5p3*c7p1*c9p2*ss[58];
  jac_4_4 += 50400*c3p3*c5p3*c7p2*c9p1*ss[56];
  jac_4_4 += 25200*c3p3*c5p4*c7p1*c9p1*ss[54];
  jac_4_4 += 25200*c3p4*c5p1*c7p1*c9p3*ss[60];
  jac_4_4 += 37800*c3p4*c5p1*c7p2*c9p2*ss[58];
  jac_4_4 += 25200*c3p4*c5p1*c7p3*c9p1*ss[56];
  jac_4_4 += 37800*c3p4*c5p2*c7p1*c9p2*ss[56];
  jac_4_4 += 37800*c3p4*c5p2*c7p2*c9p1*ss[54];
  jac_4_4 += 25200*c3p4*c5p3*c7p1*c9p1*ss[52];
  jac_4_4 += 15120*c3p5*c5p1*c7p1*c9p2*ss[54];
  jac_4_4 += 15120*c3p5*c5p1*c7p2*c9p1*ss[52];
  jac_4_4 += 15120*c3p5*c5p2*c7p1*c9p1*ss[50];
  jac_4_4 += 5040*c3p6*c5p1*c7p1*c9p1*ss[48];
  jac_4_4 += 720*c1p1*c3p1*c5p7*ss[48];
  jac_4_4 += 720*c1p1*c3p1*c7p7*ss[62];
  jac_4_4 += 720*c1p1*c3p1*c9p7*ss[76];
  jac_4_4 += 2520*c1p1*c3p2*c5p6*ss[46];
  jac_4_4 += 2520*c1p1*c3p2*c7p6*ss[58];
  jac_4_4 += 2520*c1p1*c3p2*c9p6*ss[70];
  jac_4_4 += 5040*c1p1*c3p3*c5p5*ss[44];
  jac_4_4 += 5040*c1p1*c3p3*c7p5*ss[54];
  jac_4_4 += 5040*c1p1*c3p3*c9p5*ss[64];
  jac_4_4 += 6300*c1p1*c3p4*c5p4*ss[42];
  jac_4_4 += 6300*c1p1*c3p4*c7p4*ss[50];
  jac_4_4 += 6300*c1p1*c3p4*c9p4*ss[58];
  jac_4_4 += 5040*c1p1*c3p5*c5p3*ss[40];
  jac_4_4 += 5040*c1p1*c3p5*c7p3*ss[46];
  jac_4_4 += 5040*c1p1*c3p5*c9p3*ss[52];
  jac_4_4 += 2520*c1p1*c3p6*c5p2*ss[38];
  jac_4_4 += 2520*c1p1*c3p6*c7p2*ss[42];
  jac_4_4 += 2520*c1p1*c3p6*c9p2*ss[46];
  jac_4_4 += 720*c1p1*c3p7*c5p1*ss[36];
  jac_4_4 += 720*c1p1*c3p7*c7p1*ss[38];
  jac_4_4 += 720*c1p1*c3p7*c9p1*ss[40];
  jac_4_4 += 720*c1p1*c5p1*c7p7*ss[64];
  jac_4_4 += 720*c1p1*c5p1*c9p7*ss[78];
  jac_4_4 += 2520*c1p1*c5p2*c7p6*ss[62];
  jac_4_4 += 2520*c1p1*c5p2*c9p6*ss[74];
  jac_4_4 += 5040*c1p1*c5p3*c7p5*ss[60];
  jac_4_4 += 5040*c1p1*c5p3*c9p5*ss[70];
  jac_4_4 += 6300*c1p1*c5p4*c7p4*ss[58];
  jac_4_4 += 6300*c1p1*c5p4*c9p4*ss[66];
  jac_4_4 += 5040*c1p1*c5p5*c7p3*ss[56];
  jac_4_4 += 5040*c1p1*c5p5*c9p3*ss[62];
  jac_4_4 += 2520*c1p1*c5p6*c7p2*ss[54];
  jac_4_4 += 2520*c1p1*c5p6*c9p2*ss[58];
  jac_4_4 += 720*c1p1*c5p7*c7p1*ss[52];
  jac_4_4 += 720*c1p1*c5p7*c9p1*ss[54];
  jac_4_4 += 720*c1p1*c7p1*c9p7*ss[80];
  jac_4_4 += 2520*c1p1*c7p2*c9p6*ss[78];
  jac_4_4 += 5040*c1p1*c7p3*c9p5*ss[76];
  jac_4_4 += 6300*c1p1*c7p4*c9p4*ss[74];
  jac_4_4 += 5040*c1p1*c7p5*c9p3*ss[72];
  jac_4_4 += 2520*c1p1*c7p6*c9p2*ss[70];
  jac_4_4 += 720*c1p1*c7p7*c9p1*ss[68];
  jac_4_4 += 2520*c1p2*c3p1*c5p6*ss[44];
  jac_4_4 += 2520*c1p2*c3p1*c7p6*ss[56];
  jac_4_4 += 2520*c1p2*c3p1*c9p6*ss[68];
  jac_4_4 += 7560*c1p2*c3p2*c5p5*ss[42];
  jac_4_4 += 7560*c1p2*c3p2*c7p5*ss[52];
  jac_4_4 += 7560*c1p2*c3p2*c9p5*ss[62];
  jac_4_4 += 12600*c1p2*c3p3*c5p4*ss[40];
  jac_4_4 += 12600*c1p2*c3p3*c7p4*ss[48];
  jac_4_4 += 12600*c1p2*c3p3*c9p4*ss[56];
  jac_4_4 += 12600*c1p2*c3p4*c5p3*ss[38];
  jac_4_4 += 12600*c1p2*c3p4*c7p3*ss[44];
  jac_4_4 += 12600*c1p2*c3p4*c9p3*ss[50];
  jac_4_4 += 7560*c1p2*c3p5*c5p2*ss[36];
  jac_4_4 += 7560*c1p2*c3p5*c7p2*ss[40];
  jac_4_4 += 7560*c1p2*c3p5*c9p2*ss[44];
  jac_4_4 += 2520*c1p2*c3p6*c5p1*ss[34];
  jac_4_4 += 2520*c1p2*c3p6*c7p1*ss[36];
  jac_4_4 += 2520*c1p2*c3p6*c9p1*ss[38];
  jac_4_4 += 2520*c1p2*c5p1*c7p6*ss[58];
  jac_4_4 += 2520*c1p2*c5p1*c9p6*ss[70];
  jac_4_4 += 7560*c1p2*c5p2*c7p5*ss[56];
  jac_4_4 += 7560*c1p2*c5p2*c9p5*ss[66];
  jac_4_4 += 12600*c1p2*c5p3*c7p4*ss[54];
  jac_4_4 += 12600*c1p2*c5p3*c9p4*ss[62];
  jac_4_4 += 12600*c1p2*c5p4*c7p3*ss[52];
  jac_4_4 += 12600*c1p2*c5p4*c9p3*ss[58];
  jac_4_4 += 7560*c1p2*c5p5*c7p2*ss[50];
  jac_4_4 += 7560*c1p2*c5p5*c9p2*ss[54];
  jac_4_4 += 2520*c1p2*c5p6*c7p1*ss[48];
  jac_4_4 += 2520*c1p2*c5p6*c9p1*ss[50];
  jac_4_4 += 2520*c1p2*c7p1*c9p6*ss[72];
  jac_4_4 += 7560*c1p2*c7p2*c9p5*ss[70];
  jac_4_4 += 12600*c1p2*c7p3*c9p4*ss[68];
  jac_4_4 += 12600*c1p2*c7p4*c9p3*ss[66];
  jac_4_4 += 7560*c1p2*c7p5*c9p2*ss[64];
  jac_4_4 += 2520*c1p2*c7p6*c9p1*ss[62];
  jac_4_4 += 5040*c1p3*c3p1*c5p5*ss[40];
  jac_4_4 += 5040*c1p3*c3p1*c7p5*ss[50];
  jac_4_4 += 5040*c1p3*c3p1*c9p5*ss[60];
  jac_4_4 += 12600*c1p3*c3p2*c5p4*ss[38];
  jac_4_4 += 12600*c1p3*c3p2*c7p4*ss[46];
  jac_4_4 += 12600*c1p3*c3p2*c9p4*ss[54];
  jac_4_4 += 16800*c1p3*c3p3*c5p3*ss[36];
  jac_4_4 += 16800*c1p3*c3p3*c7p3*ss[42];
  jac_4_4 += 16800*c1p3*c3p3*c9p3*ss[48];
  jac_4_4 += 12600*c1p3*c3p4*c5p2*ss[34];
  jac_4_4 += 12600*c1p3*c3p4*c7p2*ss[38];
  jac_4_4 += 12600*c1p3*c3p4*c9p2*ss[42];
  jac_4_4 += 5040*c1p3*c3p5*c5p1*ss[32];
  jac_4_4 += 5040*c1p3*c3p5*c7p1*ss[34];
  jac_4_4 += 5040*c1p3*c3p5*c9p1*ss[36];
  jac_4_4 += 5040*c1p3*c5p1*c7p5*ss[52];
  jac_4_4 += 5040*c1p3*c5p1*c9p5*ss[62];
  jac_4_4 += 12600*c1p3*c5p2*c7p4*ss[50];
  jac_4_4 += 12600*c1p3*c5p2*c9p4*ss[58];
  jac_4_4 += 16800*c1p3*c5p3*c7p3*ss[48];
  jac_4_4 += 16800*c1p3*c5p3*c9p3*ss[54];
  jac_4_4 += 12600*c1p3*c5p4*c7p2*ss[46];
  jac_4_4 += 12600*c1p3*c5p4*c9p2*ss[50];
  jac_4_4 += 5040*c1p3*c5p5*c7p1*ss[44];
  jac_4_4 += 5040*c1p3*c5p5*c9p1*ss[46];
  jac_4_4 += 5040*c1p3*c7p1*c9p5*ss[64];
  jac_4_4 += 12600*c1p3*c7p2*c9p4*ss[62];
  jac_4_4 += 16800*c1p3*c7p3*c9p3*ss[60];
  jac_4_4 += 12600*c1p3*c7p4*c9p2*ss[58];
  jac_4_4 += 5040*c1p3*c7p5*c9p1*ss[56];
  jac_4_4 += 6300*c1p4*c3p1*c5p4*ss[36];
  jac_4_4 += 6300*c1p4*c3p1*c7p4*ss[44];
  jac_4_4 += 6300*c1p4*c3p1*c9p4*ss[52];
  jac_4_4 += 12600*c1p4*c3p2*c5p3*ss[34];
  jac_4_4 += 12600*c1p4*c3p2*c7p3*ss[40];
  jac_4_4 += 12600*c1p4*c3p2*c9p3*ss[46];
  jac_4_4 += 12600*c1p4*c3p3*c5p2*ss[32];
  jac_4_4 += 12600*c1p4*c3p3*c7p2*ss[36];
  jac_4_4 += 12600*c1p4*c3p3*c9p2*ss[40];
  jac_4_4 += 6300*c1p4*c3p4*c5p1*ss[30];
  jac_4_4 += 6300*c1p4*c3p4*c7p1*ss[32];
  jac_4_4 += 6300*c1p4*c3p4*c9p1*ss[34];
  jac_4_4 += 6300*c1p4*c5p1*c7p4*ss[46];
  jac_4_4 += 6300*c1p4*c5p1*c9p4*ss[54];
  jac_4_4 += 12600*c1p4*c5p2*c7p3*ss[44];
  jac_4_4 += 12600*c1p4*c5p2*c9p3*ss[50];
  jac_4_4 += 12600*c1p4*c5p3*c7p2*ss[42];
  jac_4_4 += 12600*c1p4*c5p3*c9p2*ss[46];
  jac_4_4 += 6300*c1p4*c5p4*c7p1*ss[40];
  jac_4_4 += 6300*c1p4*c5p4*c9p1*ss[42];
  jac_4_4 += 6300*c1p4*c7p1*c9p4*ss[56];
  jac_4_4 += 12600*c1p4*c7p2*c9p3*ss[54];
  jac_4_4 += 12600*c1p4*c7p3*c9p2*ss[52];
  jac_4_4 += 6300*c1p4*c7p4*c9p1*ss[50];
  jac_4_4 += 5040*c1p5*c3p1*c5p3*ss[32];
  jac_4_4 += 5040*c1p5*c3p1*c7p3*ss[38];
  jac_4_4 += 5040*c1p5*c3p1*c9p3*ss[44];
  jac_4_4 += 7560*c1p5*c3p2*c5p2*ss[30];
  jac_4_4 += 7560*c1p5*c3p2*c7p2*ss[34];
  jac_4_4 += 7560*c1p5*c3p2*c9p2*ss[38];
  jac_4_4 += 5040*c1p5*c3p3*c5p1*ss[28];
  jac_4_4 += 5040*c1p5*c3p3*c7p1*ss[30];
  jac_4_4 += 5040*c1p5*c3p3*c9p1*ss[32];
  jac_4_4 += 5040*c1p5*c5p1*c7p3*ss[40];
  jac_4_4 += 5040*c1p5*c5p1*c9p3*ss[46];
  jac_4_4 += 7560*c1p5*c5p2*c7p2*ss[38];
  jac_4_4 += 7560*c1p5*c5p2*c9p2*ss[42];
  jac_4_4 += 5040*c1p5*c5p3*c7p1*ss[36];
  jac_4_4 += 5040*c1p5*c5p3*c9p1*ss[38];
  jac_4_4 += 5040*c1p5*c7p1*c9p3*ss[48];
  jac_4_4 += 7560*c1p5*c7p2*c9p2*ss[46];
  jac_4_4 += 5040*c1p5*c7p3*c9p1*ss[44];
  jac_4_4 += 2520*c1p6*c3p1*c5p2*ss[28];
  jac_4_4 += 2520*c1p6*c3p1*c7p2*ss[32];
  jac_4_4 += 2520*c1p6*c3p1*c9p2*ss[36];
  jac_4_4 += 2520*c1p6*c3p2*c5p1*ss[26];
  jac_4_4 += 2520*c1p6*c3p2*c7p1*ss[28];
  jac_4_4 += 2520*c1p6*c3p2*c9p1*ss[30];
  jac_4_4 += 2520*c1p6*c5p1*c7p2*ss[34];
  jac_4_4 += 2520*c1p6*c5p1*c9p2*ss[38];
  jac_4_4 += 2520*c1p6*c5p2*c7p1*ss[32];
  jac_4_4 += 2520*c1p6*c5p2*c9p1*ss[34];
  jac_4_4 += 2520*c1p6*c7p1*c9p2*ss[40];
  jac_4_4 += 2520*c1p6*c7p2*c9p1*ss[38];
  jac_4_4 += 720*c1p7*c3p1*c5p1*ss[24];
  jac_4_4 += 720*c1p7*c3p1*c7p1*ss[26];
  jac_4_4 += 720*c1p7*c3p1*c9p1*ss[28];
  jac_4_4 += 720*c1p7*c5p1*c7p1*ss[28];
  jac_4_4 += 720*c1p7*c5p1*c9p1*ss[30];
  jac_4_4 += 720*c1p7*c7p1*c9p1*ss[32];
  jac_4_4 += 720*c3p1*c5p1*c7p7*ss[66];
  jac_4_4 += 720*c3p1*c5p1*c9p7*ss[80];
  jac_4_4 += 2520*c3p1*c5p2*c7p6*ss[64];
  jac_4_4 += 2520*c3p1*c5p2*c9p6*ss[76];
  jac_4_4 += 5040*c3p1*c5p3*c7p5*ss[62];
  jac_4_4 += 5040*c3p1*c5p3*c9p5*ss[72];
  jac_4_4 += 6300*c3p1*c5p4*c7p4*ss[60];
  jac_4_4 += 6300*c3p1*c5p4*c9p4*ss[68];
  jac_4_4 += 5040*c3p1*c5p5*c7p3*ss[58];
  jac_4_4 += 5040*c3p1*c5p5*c9p3*ss[64];
  jac_4_4 += 2520*c3p1*c5p6*c7p2*ss[56];
  jac_4_4 += 2520*c3p1*c5p6*c9p2*ss[60];
  jac_4_4 += 720*c3p1*c5p7*c7p1*ss[54];
  jac_4_4 += 720*c3p1*c5p7*c9p1*ss[56];
  jac_4_4 += 720*c3p1*c7p1*c9p7*ss[82];
  jac_4_4 += 2520*c3p1*c7p2*c9p6*ss[80];
  jac_4_4 += 5040*c3p1*c7p3*c9p5*ss[78];
  jac_4_4 += 6300*c3p1*c7p4*c9p4*ss[76];
  jac_4_4 += 5040*c3p1*c7p5*c9p3*ss[74];
  jac_4_4 += 2520*c3p1*c7p6*c9p2*ss[72];
  jac_4_4 += 720*c3p1*c7p7*c9p1*ss[70];
  jac_4_4 += 2520*c3p2*c5p1*c7p6*ss[62];
  jac_4_4 += 2520*c3p2*c5p1*c9p6*ss[74];
  jac_4_4 += 7560*c3p2*c5p2*c7p5*ss[60];
  jac_4_4 += 7560*c3p2*c5p2*c9p5*ss[70];
  jac_4_4 += 12600*c3p2*c5p3*c7p4*ss[58];
  jac_4_4 += 12600*c3p2*c5p3*c9p4*ss[66];
  jac_4_4 += 12600*c3p2*c5p4*c7p3*ss[56];
  jac_4_4 += 12600*c3p2*c5p4*c9p3*ss[62];
  jac_4_4 += 7560*c3p2*c5p5*c7p2*ss[54];
  jac_4_4 += 7560*c3p2*c5p5*c9p2*ss[58];
  jac_4_4 += 2520*c3p2*c5p6*c7p1*ss[52];
  jac_4_4 += 2520*c3p2*c5p6*c9p1*ss[54];
  jac_4_4 += 2520*c3p2*c7p1*c9p6*ss[76];
  jac_4_4 += 7560*c3p2*c7p2*c9p5*ss[74];
  jac_4_4 += 12600*c3p2*c7p3*c9p4*ss[72];
  jac_4_4 += 12600*c3p2*c7p4*c9p3*ss[70];
  jac_4_4 += 7560*c3p2*c7p5*c9p2*ss[68];
  jac_4_4 += 2520*c3p2*c7p6*c9p1*ss[66];
  jac_4_4 += 5040*c3p3*c5p1*c7p5*ss[58];
  jac_4_4 += 5040*c3p3*c5p1*c9p5*ss[68];
  jac_4_4 += 12600*c3p3*c5p2*c7p4*ss[56];
  jac_4_4 += 12600*c3p3*c5p2*c9p4*ss[64];
  jac_4_4 += 16800*c3p3*c5p3*c7p3*ss[54];
  jac_4_4 += 16800*c3p3*c5p3*c9p3*ss[60];
  jac_4_4 += 12600*c3p3*c5p4*c7p2*ss[52];
  jac_4_4 += 12600*c3p3*c5p4*c9p2*ss[56];
  jac_4_4 += 5040*c3p3*c5p5*c7p1*ss[50];
  jac_4_4 += 5040*c3p3*c5p5*c9p1*ss[52];
  jac_4_4 += 5040*c3p3*c7p1*c9p5*ss[70];
  jac_4_4 += 12600*c3p3*c7p2*c9p4*ss[68];
  jac_4_4 += 16800*c3p3*c7p3*c9p3*ss[66];
  jac_4_4 += 12600*c3p3*c7p4*c9p2*ss[64];
  jac_4_4 += 5040*c3p3*c7p5*c9p1*ss[62];
  jac_4_4 += 6300*c3p4*c5p1*c7p4*ss[54];
  jac_4_4 += 6300*c3p4*c5p1*c9p4*ss[62];
  jac_4_4 += 12600*c3p4*c5p2*c7p3*ss[52];
  jac_4_4 += 12600*c3p4*c5p2*c9p3*ss[58];
  jac_4_4 += 12600*c3p4*c5p3*c7p2*ss[50];
  jac_4_4 += 12600*c3p4*c5p3*c9p2*ss[54];
  jac_4_4 += 6300*c3p4*c5p4*c7p1*ss[48];
  jac_4_4 += 6300*c3p4*c5p4*c9p1*ss[50];
  jac_4_4 += 6300*c3p4*c7p1*c9p4*ss[64];
  jac_4_4 += 12600*c3p4*c7p2*c9p3*ss[62];
  jac_4_4 += 12600*c3p4*c7p3*c9p2*ss[60];
  jac_4_4 += 6300*c3p4*c7p4*c9p1*ss[58];
  jac_4_4 += 5040*c3p5*c5p1*c7p3*ss[50];
  jac_4_4 += 5040*c3p5*c5p1*c9p3*ss[56];
  jac_4_4 += 7560*c3p5*c5p2*c7p2*ss[48];
  jac_4_4 += 7560*c3p5*c5p2*c9p2*ss[52];
  jac_4_4 += 5040*c3p5*c5p3*c7p1*ss[46];
  jac_4_4 += 5040*c3p5*c5p3*c9p1*ss[48];
  jac_4_4 += 5040*c3p5*c7p1*c9p3*ss[58];
  jac_4_4 += 7560*c3p5*c7p2*c9p2*ss[56];
  jac_4_4 += 5040*c3p5*c7p3*c9p1*ss[54];
  jac_4_4 += 2520*c3p6*c5p1*c7p2*ss[46];
  jac_4_4 += 2520*c3p6*c5p1*c9p2*ss[50];
  jac_4_4 += 2520*c3p6*c5p2*c7p1*ss[44];
  jac_4_4 += 2520*c3p6*c5p2*c9p1*ss[46];
  jac_4_4 += 2520*c3p6*c7p1*c9p2*ss[52];
  jac_4_4 += 2520*c3p6*c7p2*c9p1*ss[50];
  jac_4_4 += 720*c3p7*c5p1*c7p1*ss[42];
  jac_4_4 += 720*c3p7*c5p1*c9p1*ss[44];
  jac_4_4 += 720*c3p7*c7p1*c9p1*ss[46];
  jac_4_4 += 720*c5p1*c7p1*c9p7*ss[84];
  jac_4_4 += 2520*c5p1*c7p2*c9p6*ss[82];
  jac_4_4 += 5040*c5p1*c7p3*c9p5*ss[80];
  jac_4_4 += 6300*c5p1*c7p4*c9p4*ss[78];
  jac_4_4 += 5040*c5p1*c7p5*c9p3*ss[76];
  jac_4_4 += 2520*c5p1*c7p6*c9p2*ss[74];
  jac_4_4 += 720*c5p1*c7p7*c9p1*ss[72];
  jac_4_4 += 2520*c5p2*c7p1*c9p6*ss[80];
  jac_4_4 += 7560*c5p2*c7p2*c9p5*ss[78];
  jac_4_4 += 12600*c5p2*c7p3*c9p4*ss[76];
  jac_4_4 += 12600*c5p2*c7p4*c9p3*ss[74];
  jac_4_4 += 7560*c5p2*c7p5*c9p2*ss[72];
  jac_4_4 += 2520*c5p2*c7p6*c9p1*ss[70];
  jac_4_4 += 5040*c5p3*c7p1*c9p5*ss[76];
  jac_4_4 += 12600*c5p3*c7p2*c9p4*ss[74];
  jac_4_4 += 16800*c5p3*c7p3*c9p3*ss[72];
  jac_4_4 += 12600*c5p3*c7p4*c9p2*ss[70];
  jac_4_4 += 5040*c5p3*c7p5*c9p1*ss[68];
  jac_4_4 += 6300*c5p4*c7p1*c9p4*ss[72];
  jac_4_4 += 12600*c5p4*c7p2*c9p3*ss[70];
  jac_4_4 += 12600*c5p4*c7p3*c9p2*ss[68];
  jac_4_4 += 6300*c5p4*c7p4*c9p1*ss[66];
  jac_4_4 += 5040*c5p5*c7p1*c9p3*ss[68];
  jac_4_4 += 7560*c5p5*c7p2*c9p2*ss[66];
  jac_4_4 += 5040*c5p5*c7p3*c9p1*ss[64];
  jac_4_4 += 2520*c5p6*c7p1*c9p2*ss[64];
  jac_4_4 += 2520*c5p6*c7p2*c9p1*ss[62];
  jac_4_4 += 720*c5p7*c7p1*c9p1*ss[60];
  jac_4_4 += 90*c1p1*c3p8*ss[34];
  jac_4_4 += 90*c1p1*c5p8*ss[50];
  jac_4_4 += 90*c1p1*c7p8*ss[66];
  jac_4_4 += 90*c1p1*c9p8*ss[82];
  jac_4_4 += 360*c1p2*c3p7*ss[32];
  jac_4_4 += 360*c1p2*c5p7*ss[46];
  jac_4_4 += 360*c1p2*c7p7*ss[60];
  jac_4_4 += 360*c1p2*c9p7*ss[74];
  jac_4_4 += 840*c1p3*c3p6*ss[30];
  jac_4_4 += 840*c1p3*c5p6*ss[42];
  jac_4_4 += 840*c1p3*c7p6*ss[54];
  jac_4_4 += 840*c1p3*c9p6*ss[66];
  jac_4_4 += 1260*c1p4*c3p5*ss[28];
  jac_4_4 += 1260*c1p4*c5p5*ss[38];
  jac_4_4 += 1260*c1p4*c7p5*ss[48];
  jac_4_4 += 1260*c1p4*c9p5*ss[58];
  jac_4_4 += 1260*c1p5*c3p4*ss[26];
  jac_4_4 += 1260*c1p5*c5p4*ss[34];
  jac_4_4 += 1260*c1p5*c7p4*ss[42];
  jac_4_4 += 1260*c1p5*c9p4*ss[50];
  jac_4_4 += 840*c1p6*c3p3*ss[24];
  jac_4_4 += 840*c1p6*c5p3*ss[30];
  jac_4_4 += 840*c1p6*c7p3*ss[36];
  jac_4_4 += 840*c1p6*c9p3*ss[42];
  jac_4_4 += 360*c1p7*c3p2*ss[22];
  jac_4_4 += 360*c1p7*c5p2*ss[26];
  jac_4_4 += 360*c1p7*c7p2*ss[30];
  jac_4_4 += 360*c1p7*c9p2*ss[34];
  jac_4_4 += 90*c1p8*c3p1*ss[20];
  jac_4_4 += 90*c1p8*c5p1*ss[22];
  jac_4_4 += 90*c1p8*c7p1*ss[24];
  jac_4_4 += 90*c1p8*c9p1*ss[26];
  jac_4_4 += 90*c3p1*c5p8*ss[52];
  jac_4_4 += 90*c3p1*c7p8*ss[68];
  jac_4_4 += 90*c3p1*c9p8*ss[84];
  jac_4_4 += 360*c3p2*c5p7*ss[50];
  jac_4_4 += 360*c3p2*c7p7*ss[64];
  jac_4_4 += 360*c3p2*c9p7*ss[78];
  jac_4_4 += 840*c3p3*c5p6*ss[48];
  jac_4_4 += 840*c3p3*c7p6*ss[60];
  jac_4_4 += 840*c3p3*c9p6*ss[72];
  jac_4_4 += 1260*c3p4*c5p5*ss[46];
  jac_4_4 += 1260*c3p4*c7p5*ss[56];
  jac_4_4 += 1260*c3p4*c9p5*ss[66];
  jac_4_4 += 1260*c3p5*c5p4*ss[44];
  jac_4_4 += 1260*c3p5*c7p4*ss[52];
  jac_4_4 += 1260*c3p5*c9p4*ss[60];
  jac_4_4 += 840*c3p6*c5p3*ss[42];
  jac_4_4 += 840*c3p6*c7p3*ss[48];
  jac_4_4 += 840*c3p6*c9p3*ss[54];
  jac_4_4 += 360*c3p7*c5p2*ss[40];
  jac_4_4 += 360*c3p7*c7p2*ss[44];
  jac_4_4 += 360*c3p7*c9p2*ss[48];
  jac_4_4 += 90*c3p8*c5p1*ss[38];
  jac_4_4 += 90*c3p8*c7p1*ss[40];
  jac_4_4 += 90*c3p8*c9p1*ss[42];
  jac_4_4 += 90*c5p1*c7p8*ss[70];
  jac_4_4 += 90*c5p1*c9p8*ss[86];
  jac_4_4 += 360*c5p2*c7p7*ss[68];
  jac_4_4 += 360*c5p2*c9p7*ss[82];
  jac_4_4 += 840*c5p3*c7p6*ss[66];
  jac_4_4 += 840*c5p3*c9p6*ss[78];
  jac_4_4 += 1260*c5p4*c7p5*ss[64];
  jac_4_4 += 1260*c5p4*c9p5*ss[74];
  jac_4_4 += 1260*c5p5*c7p4*ss[62];
  jac_4_4 += 1260*c5p5*c9p4*ss[70];
  jac_4_4 += 840*c5p6*c7p3*ss[60];
  jac_4_4 += 840*c5p6*c9p3*ss[66];
  jac_4_4 += 360*c5p7*c7p2*ss[58];
  jac_4_4 += 360*c5p7*c9p2*ss[62];
  jac_4_4 += 90*c5p8*c7p1*ss[56];
  jac_4_4 += 90*c5p8*c9p1*ss[58];
  jac_4_4 += 90*c7p1*c9p8*ss[88];
  jac_4_4 += 360*c7p2*c9p7*ss[86];
  jac_4_4 += 840*c7p3*c9p6*ss[84];
  jac_4_4 += 1260*c7p4*c9p5*ss[82];
  jac_4_4 += 1260*c7p5*c9p4*ss[80];
  jac_4_4 += 840*c7p6*c9p3*ss[78];
  jac_4_4 += 360*c7p7*c9p2*ss[76];
  jac_4_4 += 90*c7p8*c9p1*ss[74];
  jac_4_4 += 10*c1p9*ss[18];
  jac_4_4 += 10*c3p9*ss[36];
  jac_4_4 += 10*c5p9*ss[54];
  jac_4_4 += 10*c7p9*ss[72];
  jac_4_4 += 10*c9p9*ss[90];
  roots[0]=convert<T>(root_0);
  roots[1]=convert<T>(root_1);
  roots[2]=convert<T>(root_2);
  roots[3]=convert<T>(root_3);
  roots[4]=convert<T>(root_4);
  jacobian[0] = convert<T>(jac_0_0);
  jacobian[1] = convert<T>(jac_0_1);
  jacobian[2] = convert<T>(jac_0_2);
  jacobian[3] = convert<T>(jac_0_3);
  jacobian[4] = convert<T>(jac_0_4);
  jacobian[5] = convert<T>(jac_1_0);
  jacobian[6] = convert<T>(jac_1_1);
  jacobian[7] = convert<T>(jac_1_2);
  jacobian[8] = convert<T>(jac_1_3);
  jacobian[9] = convert<T>(jac_1_4);
  jacobian[10] = convert<T>(jac_2_0);
  jacobian[11] = convert<T>(jac_2_1);
  jacobian[12] = convert<T>(jac_2_2);
  jacobian[13] = convert<T>(jac_2_3);
  jacobian[14] = convert<T>(jac_2_4);
  jacobian[15] = convert<T>(jac_3_0);
  jacobian[16] = convert<T>(jac_3_1);
  jacobian[17] = convert<T>(jac_3_2);
  jacobian[18] = convert<T>(jac_3_3);
  jacobian[19] = convert<T>(jac_3_4);
  jacobian[20] = convert<T>(jac_4_0);
  jacobian[21] = convert<T>(jac_4_1);
  jacobian[22] = convert<T>(jac_4_2);
  jacobian[23] = convert<T>(jac_4_3);
  jacobian[24] = convert<T>(jac_4_4);
}
}; // SymmetricNonicSystem

}; // systems
}; // random
}; // flopoco

#endif
